@@ -10,7 +10,7 @@
 #include "util/arg_auto_set.h"
 #include "Rev.h"
 
-#define version "1.2.8"
+#define version "2.0.0"
 
 #include "plugin_3state.h"
 
@@ -338,6 +338,9 @@ int hook_detect_target()
 	want_cairo  = plug_is_enabled("export_bboard") | plug_is_enabled("hid_gtk3_cairo");
 	want_xml2   = plug_is_enabled("io_eagle");
 
+/****** TODO #21: core depends on this plugin (yes, this is a bug) ******/
+	hook_custom_arg("buildin-lib_compat_help", NULL);
+
 	require("cc/fpic",  0, 0);
 	host_ansi = get("/host/cc/argstd/ansi");
 	host_ped = get("/host/cc/argstd/pedantic");
@@ -556,9 +559,6 @@ int hook_detect_target()
 	if (want_gtk)
 		want_glib = 1;
 
-	if (plug_is_enabled("export_dsn"))
-		want_glib = 1;
-
 	if (plug_is_enabled("puller"))
 		want_glib = 1;
 
@@ -573,10 +573,6 @@ int hook_detect_target()
 				report_repeat("WARNING: Since GLIB is not found, disabling the puller...\n");
 				hook_custom_arg("Disable-puller", NULL);
 			}
-			if (plug_is_enabled("export_dsn")) {
-				report_repeat("WARNING: Since GLIB is not found, disabling the dsn pcb_exporter...\n");
-				hook_custom_arg("Disable-export_dsn", NULL);
-			}
 		}
 	}
 	else {
@@ -679,14 +675,14 @@ int hook_detect_target()
 		int long_bits      = safe_atoi(get("sys/types/size/signed_long_int")) * 8;
 		int long_long_bits = safe_atoi(get("sys/types/size/signed_long_long_int")) * 8;
 		int int64_bits     = safe_atoi(get("sys/types/size/uint64_t")) * 8;
-		const char *chosen, *abs_name, *postfix;
+		const char *chosen, *postfix;
 		char tmp[64];
 		int need_stdint = 0;
 
-		if (want_coord_bits == int_bits)             { postfix="U";   chosen = "int";           abs_name="abs"; }
-		else if (want_coord_bits == long_bits)       { postfix="UL";  chosen = "long int";      abs_name="labs"; }
-		else if (want_coord_bits == int64_bits)      { postfix="ULL"; chosen = "int64_t";       abs_name="llabs"; need_stdint = 1; }
-		else if (want_coord_bits == long_long_bits)  { postfix="ULL"; chosen = "long long int"; abs_name="llabs"; }
+		if (want_coord_bits == int_bits)             { postfix="U";   chosen = "int";           }
+		else if (want_coord_bits == long_bits)       { postfix="UL";  chosen = "long int";      }
+		else if (want_coord_bits == int64_bits)      { postfix="ULL"; chosen = "int64_t";       need_stdint = 1; }
+		else if (want_coord_bits == long_long_bits)  { postfix="ULL"; chosen = "long long int"; }
 		else {
 			report("ERROR: can't find a suitable integer type for coord to be %d bits wide\n", want_coord_bits);
 			exit(1);
@@ -695,7 +691,6 @@ int hook_detect_target()
 		sprintf(tmp, "((1%s<<%d)-1)", postfix, want_coord_bits - 1);
 		put("/local/pcb/coord_type", chosen);
 		put("/local/pcb/coord_max", tmp);
-		put("/local/pcb/coord_abs", abs_name);
 
 		chosen = NULL;
 		if (istrue(get("/local/pcb/debug"))) { /* debug: c89 */
@@ -836,18 +831,31 @@ static void print_sum_cfg_val(const char *node, const char *desc)
 /* Runs after detection hooks, should generate the output (Makefiles, etc.) */
 int hook_generate()
 {
-	char *rev = "non-svn", *tmp;
+	char *next, *rev = "non-svn", *curr, *tmp;
 	int generr = 0;
 	int res = 0;
+	char apiver[32];
+	long r = 0;
+	int v1, v2, v3;
 
 	tmp = svn_info(0, "../src", "Revision:");
 	if (tmp != NULL) {
+		r = strtol(tmp, NULL, 10);
 		rev = str_concat("", "svn r", tmp, NULL);
 		free(tmp);
 	}
+	strcpy(apiver, version);
+	curr = apiver; next = strchr(curr, '.'); *next = '\n';
+	v1 = atoi(curr);
+	curr = next+1; next = strchr(curr, '.'); *next = '\n';
+	v2 = atoi(curr);
+	v3 = atoi(next+1);
+	sprintf(apiver, "%01d%01d%02d%05ld", v1, v2, v3, r);
+
 	logprintf(0, "scconfig generate version info: version='%s' rev='%s'\n", version, rev);
 	put("/local/revision", rev);
 	put("/local/version",  version);
+	put("/local/apiver", apiver);
 	put("/local/pup/sccbox", "../../scconfig/sccbox");
 
 	printf("Generating Makefile.conf (%d)\n", generr |= tmpasm("..", "Makefile.conf.in", "Makefile.conf"));
