@@ -55,7 +55,7 @@ static int try(int logdepth, const char *cc, const char *test_c, const char *cfl
 	return 0;
 }
 
-int find_c99_stdc(int logdepth, int fatal)
+int find_c99_stdc(const char *name, int logdepth, int fatal)
 {
 	const char *test_c =
 		NL "#include <stdio.h>"
@@ -81,7 +81,7 @@ int find_c99_stdc(int logdepth, int fatal)
 	return 1;
 }
 
-int find_c99_emptymacro(int logdepth, int fatal)
+int find_c99_emptymacro(const char *name, int logdepth, int fatal)
 {
 	const char *test_c =
 		NL "#include <stdio.h>"
@@ -106,7 +106,7 @@ int find_c99_emptymacro(int logdepth, int fatal)
 	return 1;
 }
 
-int find_c99_variadicmacro(int logdepth, int fatal)
+int find_c99_variadicmacro(const char *name, int logdepth, int fatal)
 {
 	const char *test_c =
 		NL "#include <stdio.h>"
@@ -131,7 +131,7 @@ int find_c99_variadicmacro(int logdepth, int fatal)
 	return 1;
 }
 
-int find_c99_funcmacro(int logdepth, int fatal)
+int find_c99_funcmacro(const char *name, int logdepth, int fatal)
 {
 	const char *test_c =
 		NL "#include <stdio.h>"
@@ -156,7 +156,7 @@ int find_c99_funcmacro(int logdepth, int fatal)
 	return 1;
 }
 
-int find_c99_stdint(int logdepth, int fatal)
+int find_c99_stdint(const char *name, int logdepth, int fatal)
 {
 	const char *test_c =
 		/* <stdint.h>:  [u]intNN_t, [u]int_fastNN_t, [u]int_least_t, [u]intmax_t, [u]intptr_t, +limits */
@@ -186,7 +186,7 @@ int find_c99_stdint(int logdepth, int fatal)
 	return 1;
 }
 
-int find_c99_wchar(int logdepth, int fatal)
+int find_c99_wchar(const char *name, int logdepth, int fatal)
 {
 	const char *test_c =
 		NL "#include <stddef.h>"
@@ -214,7 +214,7 @@ int find_c99_wchar(int logdepth, int fatal)
 	return 1;
 }
 
-int find_c99_longlong(int logdepth, int fatal)
+int find_c99_longlong(const char *name, int logdepth, int fatal)
 {
 	const char *test_c =
 		NL "#include <stdio.h>"
@@ -239,7 +239,7 @@ int find_c99_longlong(int logdepth, int fatal)
 	return 1;
 }
 
-int find_c99_bool(int logdepth, int fatal)
+int find_c99_bool(const char *name, int logdepth, int fatal)
 {
 	const char *test_c =
 		NL "#include <stdio.h>"
@@ -266,7 +266,7 @@ int find_c99_bool(int logdepth, int fatal)
 	return 1;
 }
 
-int find_c99_inline(int logdepth, int fatal)
+int find_c99_inline(const char *name, int logdepth, int fatal)
 {
 	const char *test_c =
 		NL "#include <stdio.h>"
@@ -291,7 +291,7 @@ int find_c99_inline(int logdepth, int fatal)
 	return 1;
 }
 
-int find_c99_restrict(int logdepth, int fatal)
+int find_c99_restrict(const char *name, int logdepth, int fatal)
 {
 	const char *test_c =
 		NL "#include <stdio.h>"
@@ -316,7 +316,7 @@ int find_c99_restrict(int logdepth, int fatal)
 	return 1;
 }
 
-int find_c99_comment(int logdepth, int fatal)
+int find_c99_comment(const char *name, int logdepth, int fatal)
 {
 	const char *test_c =
 		NL "#include <stdio.h>"
@@ -341,7 +341,7 @@ int find_c99_comment(int logdepth, int fatal)
 	return 1;
 }
 
-int find_c99_endcomma(int logdepth, int fatal)
+int find_c99_endcomma(const char *name, int logdepth, int fatal)
 {
 	const char *test_c =
 		NL "#include <stdio.h>"
@@ -367,7 +367,7 @@ int find_c99_endcomma(int logdepth, int fatal)
 	return 1;
 }
 
-int find_c99_nonconstinit(int logdepth, int fatal)
+int find_c99_nonconstinit(const char *name, int logdepth, int fatal)
 {
 	const char *test_c =
 		NL "#include <stdio.h>"
@@ -393,7 +393,7 @@ int find_c99_nonconstinit(int logdepth, int fatal)
 	return 1;
 }
 
-int find_c99_init(int logdepth, int fatal)
+int find_c99_init(const char *name, int logdepth, int fatal)
 {
 	const char *test_c =
 		NL "#include <stdio.h>"
@@ -420,7 +420,7 @@ int find_c99_init(int logdepth, int fatal)
 	return 1;
 }
 
-int find_c99_compoundlit(int logdepth, int fatal)
+int find_c99_compoundlit(const char *name, int logdepth, int fatal)
 {
 	const char *test_c =
 		NL "#include <stdio.h>"
@@ -445,7 +445,7 @@ int find_c99_compoundlit(int logdepth, int fatal)
 	return 1;
 }
 
-int find_c99_decl(int logdepth, int fatal)
+int find_c99_decl(const char *name, int logdepth, int fatal)
 {
 	const char *test_c =
 		NL "#include <stdio.h>"
@@ -473,7 +473,7 @@ int find_c99_decl(int logdepth, int fatal)
 	return 1;
 }
 
-int find_c99_for(int logdepth, int fatal)
+int find_c99_for(const char *name, int logdepth, int fatal)
 {
 	const char *test_c =
 		NL "#include <stdio.h>"
@@ -498,7 +498,7 @@ int find_c99_for(int logdepth, int fatal)
 	return 1;
 }
 
-int find_c99_divmod(int logdepth, int fatal)
+int find_c99_divmod(const char *name, int logdepth, int fatal)
 {
 	const char *test_c =
 		NL "#include <stdio.h>"
@@ -523,7 +523,7 @@ int find_c99_divmod(int logdepth, int fatal)
 	return 1;
 }
 
-int find_c99_incomparray(int logdepth, int fatal)
+int find_c99_incomparray(const char *name, int logdepth, int fatal)
 {
 	const char *test_c =
 		NL "#include <stddef.h>"
@@ -550,7 +550,7 @@ int find_c99_incomparray(int logdepth, int fatal)
 	return 1;
 }
 
-int find_c99_snprintf(int logdepth, int fatal)
+int find_c99_snprintf(const char *name, int logdepth, int fatal)
 {
 	const char *test_c =
 		NL "#include <stdio.h>"
@@ -576,7 +576,7 @@ int find_c99_snprintf(int logdepth, int fatal)
 	return 1;
 }
 
-int find_c99_floatfmt(int logdepth, int fatal)
+int find_c99_floatfmt(const char *name, int logdepth, int fatal)
 {
 	const char *test_c =
 		NL "#include <stdio.h>"
@@ -604,64 +604,64 @@ int find_c99_floatfmt(int logdepth, int fatal)
 
 /* TODO: the detected include is broken */
 
-int find_c99_intptr_t(int logdepth, int fatal)
+int find_c99_intptr_t(const char *name, int logdepth, int fatal)
 {
-	return find_types_something_t(logdepth, fatal, "std/c99", "intptr_t", NULL, "#include <stdint.h>");
+	return find_types_something_t(name, logdepth, fatal, "std/c99", "intptr_t", NULL, "#include <stdint.h>");
 }
 
-int find_c99_uintptr_t(int logdepth, int fatal)
+int find_c99_uintptr_t(const char *name, int logdepth, int fatal)
 {
-	return find_types_something_t(logdepth, fatal, "std/c99", "uintptr_t", NULL, "#include <stdint.h>");
+	return find_types_something_t(name, logdepth, fatal, "std/c99", "uintptr_t", NULL, "#include <stdint.h>");
 }
 
-int find_c99_intmax_t(int logdepth, int fatal)
+int find_c99_intmax_t(const char *name, int logdepth, int fatal)
 {
-	return find_types_something_t(logdepth, fatal, "std/c99", "intmax_t", NULL, "#include <stdint.h>");
+	return find_types_something_t(name, logdepth, fatal, "std/c99", "intmax_t", NULL, "#include <stdint.h>");
 }
 
-int find_c99_uintmax_t(int logdepth, int fatal)
+int find_c99_uintmax_t(const char *name, int logdepth, int fatal)
 {
-	return find_types_something_t(logdepth, fatal, "std/c99", "uintmax_t", NULL, "#include <stdint.h>");
+	return find_types_something_t(name, logdepth, fatal, "std/c99", "uintmax_t", NULL, "#include <stdint.h>");
 }
 
-int find_c99_int8_t(int logdepth, int fatal)
+int find_c99_int8_t(const char *name, int logdepth, int fatal)
 {
-	return find_types_something_t(logdepth, fatal, "std/c99", "int8_t", NULL, "#include <stdint.h>");
+	return find_types_something_t(name, logdepth, fatal, "std/c99", "int8_t", NULL, "#include <stdint.h>");
 }
 
-int find_c99_uint8_t(int logdepth, int fatal)
+int find_c99_uint8_t(const char *name, int logdepth, int fatal)
 {
-	return find_types_something_t(logdepth, fatal, "std/c99", "uint8_t", NULL, "#include <stdint.h>");
+	return find_types_something_t(name, logdepth, fatal, "std/c99", "uint8_t", NULL, "#include <stdint.h>");
 }
 
-int find_c99_int16_t(int logdepth, int fatal)
+int find_c99_int16_t(const char *name, int logdepth, int fatal)
 {
-	return find_types_something_t(logdepth, fatal, "std/c99", "int16_t", NULL, "#include <stdint.h>");
+	return find_types_something_t(name, logdepth, fatal, "std/c99", "int16_t", NULL, "#include <stdint.h>");
 }
 
-int find_c99_uint16_t(int logdepth, int fatal)
+int find_c99_uint16_t(const char *name, int logdepth, int fatal)
 {
-	return find_types_something_t(logdepth, fatal, "std/c99", "uint16_t", NULL, "#include <stdint.h>");
+	return find_types_something_t(name, logdepth, fatal, "std/c99", "uint16_t", NULL, "#include <stdint.h>");
 }
 
-int find_c99_int32_t(int logdepth, int fatal)
+int find_c99_int32_t(const char *name, int logdepth, int fatal)
 {
-	return find_types_something_t(logdepth, fatal, "std/c99", "int32_t", NULL, "#include <stdint.h>");
+	return find_types_something_t(name, logdepth, fatal, "std/c99", "int32_t", NULL, "#include <stdint.h>");
 }
 
-int find_c99_uint32_t(int logdepth, int fatal)
+int find_c99_uint32_t(const char *name, int logdepth, int fatal)
 {
-	return find_types_something_t(logdepth, fatal, "std/c99", "uint32_t", NULL, "#include <stdint.h>");
+	return find_types_something_t(name, logdepth, fatal, "std/c99", "uint32_t", NULL, "#include <stdint.h>");
 }
 
-int find_c99_int64_t(int logdepth, int fatal)
+int find_c99_int64_t(const char *name, int logdepth, int fatal)
 {
-	return find_types_something_t(logdepth, fatal, "std/c99", "int64_t", NULL, "#include <stdint.h>");
+	return find_types_something_t(name, logdepth, fatal, "std/c99", "int64_t", NULL, "#include <stdint.h>");
 }
 
-int find_c99_uint64_t(int logdepth, int fatal)
+int find_c99_uint64_t(const char *name, int logdepth, int fatal)
 {
-	return find_types_something_t(logdepth, fatal, "std/c99", "uint64_t", NULL, "#include <stdint.h>");
+	return find_types_something_t(name, logdepth, fatal, "std/c99", "uint64_t", NULL, "#include <stdint.h>");
 }
 
 
