@@ -43,6 +43,12 @@ argtbl_t argument_table[] = {
 	{"pkg-config",  "/arg/sys/pkg-config",  NULL},
 	{"pkg-config-zap","/arg/sys/pkg-config-zap",NULL},
 
+/* wildcard rules for icl() control */
+	{"^ldflags/",   NULL,                   import_icl},
+	{"^cflags/",    NULL,                   import_icl},
+	{"^includes/",  NULL,                   import_icl},
+	{"^prefix/",    NULL,                   import_icl},
+
 /* the followings are autoconf compatibility translations */
 	{"CC",          "/arg/cc/cc",           NULL},
 	{"CFLAGS",      "/arg/cc/cflags",       NULL},
@@ -74,10 +80,10 @@ void process_args(int argc, char *argv[])
 			value++;
 			/* Look in the argument translate table */
 			for(a = argument_table; a->arg != NULL; a++) {
-				if (strcmp(a->arg, key) == 0) {
+				if (((a->arg[0] == '^') && (strncmp(a->arg+1, key, strlen(a->arg+1)) == 0)) || (strcmp(a->arg, key) == 0)) {
 					found = 1;
 					if (a->callback != NULL) {
-						if (!(a->callback(key, value))) {
+						if (a->callback(key, value) != 0) {
 							error("Processing argument '%s' failed in the callback\n", argv[n]);
 							abort();
 						}
