@@ -40,6 +40,7 @@ void deps_default_init(void)
 	dep_add("cc/wloutimplib",           find_cc_wloutimplib);
 	dep_add("cc/wloutputdef",           find_cc_wloutputdef);
 	dep_add("cc/fpic",                  find_fpic);
+	dep_add("cc/fpie/*",                find_cc_fpie);
 	dep_add("cc/fnopie/*",              find_cc_fnopie);
 	dep_add("cc/fnopic/*",              find_cc_fnopic);
 	dep_add("cc/alloca/*",              find_alloca);
@@ -91,10 +92,14 @@ void deps_default_init(void)
 	dep_add("libs/fsmount/struct_statfs/*",find_fsmount_statfs_fields);
 	dep_add("libs/fsmount/struct_statvfs/*",find_fsmount_statvfs_fields);
 	dep_add("libs/fs/ustat/*",          find_fs_ustat);
+	dep_add("libs/fs/statfs/*",         find_fs_statfs);
+	dep_add("libs/fs/statvfs/*",        find_fs_statvfs);
+	dep_add("libs/fs/flock/*",          find_fs_flock);
 
 	dep_add("libs/io/pipe/*",           find_io_pipe);
 	dep_add("libs/io/dup2/*",           find_io_dup2);
 	dep_add("libs/io/fileno/*",         find_io_fileno);
+	dep_add("libs/io/lseek/*",          find_io_lseek);
 	dep_add("libs/time/usleep/*",       find_time_usleep);
 	dep_add("libs/types/stdint/*",      find_types_stdint);
 	dep_add("sys/types/size/*",         find_types_sizes);
