@@ -2,6 +2,7 @@
 int find_cc(const char *name, int logdepth, int fatal);
 int find_cc_argstd(const char *name, int logdepth, int fatal);
 int find_cc_argmachine(const char *name, int logdepth, int fatal);
+int find_cc_fpie(const char *name, int logdepth, int fatal);
 int find_cc_fnopie(const char *name, int logdepth, int fatal);
 int find_cc_fnopic(const char *name, int logdepth, int fatal);
 int find_inline(const char *name, int logdepth, int fatal);
@@ -58,6 +59,9 @@ int find_fsmount_fsstat_fields(const char *name, int logdepth, int fatal);
 int find_fsmount_statfs_fields(const char *name, int logdepth, int fatal);
 int find_fsmount_statvfs_fields(const char *name, int logdepth, int fatal);
 int find_fs_ustat(const char *name, int logdepth, int fatal);
+int find_fs_statfs(const char *name, int logdepth, int fatal);
+int find_fs_statvfs(const char *name, int logdepth, int fatal);
+int find_fs_flock(const char *name, int logdepth, int fatal);
 
 /* printf */
 int find_printf_x(const char *name, int logdepth, int fatal);
@@ -112,6 +116,7 @@ int find_shell(const char *name, int logdepth, int fatal);
 int find_io_pipe(const char *name, int logdepth, int fatal);
 int find_io_dup2(const char *name, int logdepth, int fatal);
 int find_io_fileno(const char *name, int logdepth, int fatal);
+int find_io_lseek(const char *name, int logdepth, int fatal);
 
 /* find_time.c */
 int find_time_usleep(const char *name, int logdepth, int fatal);
