@@ -176,16 +176,17 @@ int find_cc(const char *name, int logdepth, int fatal)
 	return 0;
 }
 
-int find_cc_argstd(const char *name, int logdepth, int fatal)
+int find_cc_argstd(const char *det_name, int logdepth, int fatal)
 {
 	char *test_c = "#include <stdio.h>\nint main() { printf(\"OK\\n\");\nreturn 0;}\n";
 	char *out = NULL;
-	char **flg, *flags[] = {"-ansi", "-pedantic", "-Wall", "-std=c99", "-Werror", "-Wextra", "-W", "-pg", "-no-pie", NULL};
+	char **flg, *flags[] = {"-ansi", "-pedantic", "-Wall", "-std=c89", "-std=c99", "-Werror", "-Wextra", "-W", "-pg", "-no-pie", NULL};
+	const char *det_target = det_list_target(det_name);
 
 	require("cc/cc", logdepth, fatal);
 
-	logprintf(logdepth, "find_cc: Detecting CC args\n");
-	report("Checking for cc args for std... ");
+	logprintf(logdepth, "find_cc: Detecting CC args %s\n", det_target);
+	report("Checking for cc args for std %s... ", det_target);
 
 	for(flg = flags; *flg != NULL; flg++) {
 		char name[128], *end;
@@ -194,6 +195,8 @@ int find_cc_argstd(const char *name, int logdepth, int fatal)
 		end = strchr(name, '=');
 		if (end != NULL)
 			*end = '_';
+		if (!asked_for(name, det_name))
+			continue;
 		if (compile_run(logdepth+1, test_c, NULL, *flg, "", &out) == 0) {
 			if (target_emu_fail(out) || (strncmp(out, "OK", 2) == 0)) {
 				found = *flg;
@@ -205,7 +208,8 @@ int find_cc_argstd(const char *name, int logdepth, int fatal)
 		put(name, found);
 	}
 
-	put("cc/argstd/presents", strue); /* to avoid re-detection*/
+	if (is_dep_wild(det_name))
+		put("cc/argstd/presents", strue); /* to avoid re-detection*/
 
 	report("\n");
 	return 0;
@@ -564,6 +568,27 @@ int find_rdynamic(const char *name, int logdepth, int fatal)
 	return 1;
 }
 
+int find_cc_fpie(const char *name, int logdepth, int fatal)
+{
+	const char *test_c = test_hello_world;
+
+	require("cc/cc", logdepth, fatal);
+	/* TODO: what about -fpic? */
+
+	report("Checking for -fpie... ");
+	logprintf(logdepth, "find_cc_fpie: trying to find -fpie...\n");
+	logdepth++;
+
+	/* NOTE: some gcc configuration might not pass the -pie flag to the linker, so */
+	/* try to detect whether we can force it to the linker */
+	if (try_icl(logdepth, "cc/fpie", test_c, NULL, "-fpie", "-pie -Wl,-pie")) return 0;
+	if (try_icl(logdepth, "cc/fpie", test_c, NULL, "-fPIE", "-pie -Wl,-pie")) return 0;
+	if (try_icl(logdepth, "cc/fpie", test_c, NULL, "-fpie", "-pie")) return 0;
+	if (try_icl(logdepth, "cc/fpie", test_c, NULL, "-fPIE", "-pie")) return 0;
+	if (try_icl(logdepth, "cc/fpie", test_c, NULL, NULL, NULL)) return 0;
+	return try_fail(logdepth, "cc/fpie");
+}
+
 int find_cc_fnopie(const char *name, int logdepth, int fatal)
 {
 	const char *test_c = test_hello_world;
