@@ -66,7 +66,7 @@ int find_time_Sleep(const char *name, int logdepth, int fatal)
 	logprintf(logdepth, "find_time_Sleep: trying to find Sleep...\n");
 	logdepth++;
 
-	if (try_icl(logdepth, "libs/time/Sleep", test_c, "#include \"windows.h\"", NULL, NULL))
+	if (try_icl(logdepth, "libs/time/Sleep", test_c, "#include <windows.h>", NULL, NULL))
 		return 0;
 	return try_fail(logdepth, "libs/time/Sleep");
 }
