@@ -54,7 +54,7 @@ int main(int argc, char *argv[])
 		return 1;
 
 	if (!no_autodetect_sys) {
-		find_target(0, 1);
+		find_target("", 0, 1);
 		blind_save = cross_blind;
 		cross_blind = 0;
 		printf("--- Detecting host\n");
