@@ -177,12 +177,14 @@ int find_glut(const char *name, int logdepth, int fatal, const char *call, const
 		NL;
 	const char *node = "libs/gui/glut", *ipr;
 	char freeglut[4096];
-	char **inc, *incs[] = {freeglut, "#include <openglut.h>", NULL};
+	char **inc, *incs[] = {"", "#include <openglut.h>", NULL};
 	char *cflags = NULL;
 	char *ldflags = NULL;
 	(void) call;  /* not used */
 	(void) arg;  /* not used */
 
+	incs[0] = freeglut;
+
 	if (require("cc/cc", logdepth, fatal))
 		return try_fail(logdepth, node);
 
