@@ -28,7 +28,7 @@
 #include "db.h"
 #include "dep.h"
 
-int find_parsgen_flex(int logdepth, int fatal)
+int find_parsgen_flex(const char *name, int logdepth, int fatal)
 {
 	const char *test_flex =
 		NL "%%"
@@ -75,7 +75,7 @@ int find_parsgen_flex(int logdepth, int fatal)
 	return 1;
 }
 
-int find_parsgen_bison(int logdepth, int fatal)
+int find_parsgen_bison(const char *name, int logdepth, int fatal)
 {
 	const char *test_bison =
 		NL "%union { char *str; double num;}"
