@@ -4,6 +4,8 @@
 void deps_posix_init()
 {
 	dep_add("libs/posix/fs/posix_fallocate/*",         find_posix_fallocate);
+	dep_add("libs/posix/fs/pread/*",                   find_posix_pread);
+	dep_add("libs/posix/fs/pwrite/*",                  find_posix_pwrite);
 	dep_add("libs/posix/dev/zero/*",                   find_posix_devzero);
 	dep_add("libs/posix/dev/null/*",                   find_posix_devnull);
 	dep_add("libs/posix/dev/random/*",                 find_posix_devrandom);
