@@ -1,4 +1,6 @@
 int find_posix_fallocate(const char *name, int logdepth, int fatal);
+int find_posix_pread(const char *name, int logdepth, int fatal);
+int find_posix_pwrite(const char *name, int logdepth, int fatal);
 int find_posix_devzero(const char *name, int logdepth, int fatal);
 int find_posix_devnull(const char *name, int logdepth, int fatal);
 int find_posix_devrandom(const char *name, int logdepth, int fatal);
