@@ -36,6 +36,8 @@ void deps_scripts_init()
 	dep_add("libs/script/lua/*",            find_script_lua);
 	dep_add("libs/script/guile/*",          find_script_guile);
 	dep_add("libs/script/stutter/*",        find_script_stutter);
+	dep_add("libs/script/estutter/*",       find_script_estutter);
+	dep_add("libs/script/duktape/*",        find_script_duktape);
 	dep_add("libs/script/m4/bin/*",         find_script_m4);
 }
 
