@@ -20,4 +20,6 @@ int find_script_mawk(const char *name, int logdepth, int fatal);
 int find_script_lua(const char *name, int logdepth, int fatal);
 int find_script_guile(const char *name, int logdepth, int fatal);
 int find_script_stutter(const char *name, int logdepth, int fatal);
+int find_script_estutter(const char *name, int logdepth, int fatal);
+int find_script_duktape(const char *name, int logdepth, int fatal);
 int find_script_m4(const char *name, int logdepth, int fatal);
