@@ -1,6 +1,7 @@
 /*
     scconfig - socket API detection
     Copyright (C) 2010..2012  Tibor Palinkas
+    Copyright (C) 2018  Aron Barath
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -22,7 +23,7 @@
 
 #include "socket.h"
 
-int find_socket_socket(int logdepth, int fatal)
+int find_socket_socket(const char *name, int logdepth, int fatal)
 {
 	char *test_c =
 		NL "int main() {"
@@ -67,26 +68,30 @@ int find_socket_socket(int logdepth, int fatal)
 	logdepth++;
 
 	/* normally init code is empty */
-	put("libs/socket/socket/init_code",   "");
-	put("libs/socket/socket/uninit_code", "");
+	put("libs/socket/socket/init_code",      "");
+	put("libs/socket/socket/uninit_code",    "");
+	put("libs/socket/socket/type",           "int");
+	put("libs/socket/socket/invalid_socket", "-1");
 
 	/* Look at some standard places */
 	/* on most UNIX systems */
-	if (try_socket_pure(logdepth, "libs/socket/socket", inc_linux, def_empty, test_c, "+", "+", 1)) return 0;
+	if (try_socket_pure(logdepth, "libs/socket/socket", inc_linux, def_empty, test_c, NULL, NULL, 1)) return 0;
 
 	/* this combination seems to be present on sysV forks */
-	if (try_socket_pure(logdepth, "libs/socket/socket", inc_linux, def_empty, test_c, "+", "+-lsocket -lnsl", 1)) return 0;
+	if (try_socket_pure(logdepth, "libs/socket/socket", inc_linux, def_empty, test_c, NULL, "-lsocket -lnsl", 1)) return 0;
 
 	/* Windows has its own socket library (try with WSA init) */
-	if (try_socket_pure(logdepth, "libs/socket/socket", inc_windows, def_windows, test_c, "+", "+-lws2_32", 1)) {
-		put("libs/socket/socket/init_code",   DEF_WINDOWS_INIT);
-		put("libs/socket/socket/uninit_code", DEF_WINDOWS_UNINIT);
+	if (try_socket_pure(logdepth, "libs/socket/socket", inc_windows, def_windows, test_c, NULL, "-lws2_32", 1)) {
+		put("libs/socket/socket/init_code",      DEF_WINDOWS_INIT);
+		put("libs/socket/socket/uninit_code",    DEF_WINDOWS_UNINIT);
+		put("libs/socket/socket/type",           "SOCKET");
+		put("libs/socket/socket/invalid_socket", "INVALID_SOCKET");
 		return 0;
 	}
 
 	/* final fallback: it may be that a sysV-like system has either -lsocket or -lnsl but not both */
-	if (try_socket_pure(logdepth, "libs/socket/socket", inc_linux, def_empty, test_c, "+", "+-lsocket", 1)) return 0;
-	if (try_socket_pure(logdepth, "libs/socket/socket", inc_linux, def_empty, test_c, "+", "+-lnsl", 1)) return 0;
+	if (try_socket_pure(logdepth, "libs/socket/socket", inc_linux, def_empty, test_c, NULL, "-lsocket", 1)) return 0;
+	if (try_socket_pure(logdepth, "libs/socket/socket", inc_linux, def_empty, test_c, NULL, "-lnsl", 1)) return 0;
 
 	put("libs/socket/socket/presents", sfalse);
 	report("not found\n");
@@ -94,7 +99,7 @@ int find_socket_socket(int logdepth, int fatal)
 	return 1;
 }
 
-int find_socket_wsasocket(int logdepth, int fatal)
+int find_socket_wsasocket(const char *name, int logdepth, int fatal)
 {
 	char *test_c =
 		NL "int main() {"
@@ -121,7 +126,7 @@ int find_socket_wsasocket(int logdepth, int fatal)
 	logdepth++;
 
 	/* WSASocket should be specific to win32 socket lib */
-	if (try_socket(logdepth, "libs/socket/wsasocket", inc_windows, test_c, "+", "+-lws2_32", 1)) return 0;
+	if (try_socket(logdepth, "libs/socket/wsasocket", inc_windows, test_c, NULL, "-lws2_32", 1)) return 0;
 
 	put("libs/socket/wsasocket/presents", sfalse);
 	report("not found\n");
@@ -152,7 +157,7 @@ char *test_c_socketpair =
 		NL;
 
 
-int find_socket_socketpair(int logdepth, int fatal)
+int find_socket_socketpair(const char *name, int logdepth, int fatal)
 {
 	char *test_c;
 	const char **family;
@@ -179,7 +184,7 @@ int find_socket_socketpair(int logdepth, int fatal)
 		sprintf(test_c, test_c_socketpair,  *family, "SOCK_STREAM");
 
 		/* Look at some standard places */
-		if (try_socket(logdepth, "libs/socket/socketpair", inc_linux, test_c, "+", "+", 1)) {
+		if (try_socket(logdepth, "libs/socket/socketpair", inc_linux, test_c, NULL, NULL, 1)) {
 			put("libs/socket/socketpair/af_local", *family);
 			free(test_c);
 			return 0;
@@ -192,7 +197,7 @@ int find_socket_socketpair(int logdepth, int fatal)
 	return 1;
 }
 
-int find_socket_types(int logdepth, int fatal)
+int find_socket_types(const char *name, int logdepth, int fatal)
 {
 	char node[256];
 	struct {
@@ -233,7 +238,7 @@ int find_socket_types(int logdepth, int fatal)
 	for(t = type_combos; t->domain != NULL; t++) {
 		sprintf(test_c, test_c_socket, t->domain, t->type);
 		sprintf(node, "libs/socket/types/%s/%s", t->domain, t->type);
-		try_socket(logdepth, node, "", test_c, "+", "+", 0);
+		try_socket(logdepth, node, "", test_c, NULL, NULL, 0);
 	}
 	report("done\n");
 
@@ -243,7 +248,7 @@ int find_socket_types(int logdepth, int fatal)
 }
 
 
-int find_socket_ioctlsocket(int logdepth, int fatal)
+int find_socket_ioctlsocket(const char *name, int logdepth, int fatal)
 {
 	char test_c_temp[] =
 		NL "int main() {"
@@ -269,7 +274,7 @@ int find_socket_ioctlsocket(int logdepth, int fatal)
 	logdepth++;
 
 	sprintf(test_c, test_c_temp, "int", "");
-	if (try_socket(logdepth, "libs/socket/ioctl", "", test_c, "+", "+", 0)) {
+	if (try_socket(logdepth, "libs/socket/ioctl", "", test_c, NULL, NULL, 0)) {
 		put("libs/socket/ioctl/ioctl", "ioctl");
 		put("libs/socket/ioctl/presents", strue);
 		put("libs/socket/ioctl/argtype", "int");
@@ -279,7 +284,7 @@ int find_socket_ioctlsocket(int logdepth, int fatal)
 	}
 
 	sprintf(test_c, test_c_temp, "u_long", "socket");
-	if (try_socket(logdepth, "libs/socket/ioctl", "", test_c, "+", "+", 0)) {
+	if (try_socket(logdepth, "libs/socket/ioctl", "", test_c, NULL, NULL, 0)) {
 		put("libs/socket/ioctlsocket/presents", strue);
 		put("libs/socket/ioctl/argtype", "u_long");
 		put("libs/socket/ioctl/presents", sfalse);
@@ -293,7 +298,7 @@ int find_socket_ioctlsocket(int logdepth, int fatal)
 	return 1;
 }
 
-int find_socket_fionbio(int logdepth, int fatal)
+int find_socket_fionbio(const char *name, int logdepth, int fatal)
 {
 	char test_cf[] =
 		NL "int main() {"
@@ -322,21 +327,21 @@ int find_socket_fionbio(int logdepth, int fatal)
 	logdepth++;
 
 	sprintf(test_c, test_cf, get("libs/socket/ioctl/argtype"), get("libs/socket/ioctl/ioctl"));
-	if (try_socket(logdepth, "libs/socket/ioctl/fionbio", "", test_c, "+", "+", 0)) {
+	if (try_socket(logdepth, "libs/socket/ioctl/fionbio", "", test_c, NULL, NULL, 0)) {
 		put("libs/socket/ioctl/fionbio/presents", strue);
 		report("found (win32-style)\n");
 		return 0;
 	}
 
 	sprintf(test_c, test_cf, get("libs/socket/ioctl/argtype"), get("libs/socket/ioctl/ioctl"));
-	if (try_socket(logdepth, "libs/socket/ioctl/fionbio", "#include <sys/ioctl.h>", test_c, "+", "+", 0)) {
+	if (try_socket(logdepth, "libs/socket/ioctl/fionbio", "#include <sys/ioctl.h>", test_c, NULL, NULL, 0)) {
 		put("libs/socket/ioctl/fionbio/presents", strue);
 		report("found (UNIX-style)\n");
 		return 0;
 	}
 
 	sprintf(test_c, test_cf, get("libs/socket/ioctl/argtype"), get("libs/socket/ioctl/ioctl"));
-	if (try_socket(logdepth, "libs/socket/ioctl/fionbio", "#define BSD_COMP\\n#include <sys/ioctl.h>", test_c, "+", "+", 0)) {
+	if (try_socket(logdepth, "libs/socket/ioctl/fionbio", "#define BSD_COMP\\n#include <sys/ioctl.h>", test_c, NULL, NULL, 0)) {
 		put("libs/socket/ioctl/fionbio/presents", strue);
 		report("found (UNIX-style with BSD_COMP)\n");
 		return 0;
@@ -347,7 +352,7 @@ int find_socket_fionbio(int logdepth, int fatal)
 	return 1;
 }
 
-int find_socket_closesocket(int logdepth, int fatal)
+int find_socket_closesocket(const char *name, int logdepth, int fatal)
 {
 	char test_c[] =
 		NL "int main() {"
@@ -370,7 +375,7 @@ int find_socket_closesocket(int logdepth, int fatal)
 	logprintf(logdepth, "find_socket_closesocket: trying to find closesocket()...\n");
 	logdepth++;
 
-	if (try_socket(logdepth, "libs/socket/closesocket", "", test_c, "+", "+", 0)) {
+	if (try_socket(logdepth, "libs/socket/closesocket", "", test_c, NULL, NULL, 0)) {
 		put("libs/socket/closesocket/presents", strue);
 		report("found\n");
 		return 0;
@@ -381,7 +386,7 @@ int find_socket_closesocket(int logdepth, int fatal)
 	return 1;
 }
 
-int find_socket_SHUT(int logdepth, int fatal)
+int find_socket_SHUT(const char *name, int logdepth, int fatal)
 {
 	char test_c[] =
 		NL "int main() {"
@@ -403,7 +408,7 @@ int find_socket_SHUT(int logdepth, int fatal)
 	logprintf(logdepth, "find_socket_SHUT: trying to find SHUT_* constants...\n");
 	logdepth++;
 
-	if (try_socket(logdepth, "libs/socket/SHUT", "", test_c, "+", "+", 0)) {
+	if (try_socket(logdepth, "libs/socket/SHUT", "", test_c, NULL, NULL, 0)) {
 		report("found\n");
 		return 0;
 	}
@@ -413,7 +418,7 @@ int find_socket_SHUT(int logdepth, int fatal)
 	return 1;
 }
 
-int find_socket_SD(int logdepth, int fatal)
+int find_socket_SD(const char *name, int logdepth, int fatal)
 {
 	char test_c[] =
 		NL "int main() {"
@@ -433,7 +438,7 @@ int find_socket_SD(int logdepth, int fatal)
 	logprintf(logdepth, "find_socket_SD: trying to find SD_* constants...\n");
 	logdepth++;
 
-	if (try_socket(logdepth, "libs/socket/SD", "", test_c, "+", "+", 0)) {
+	if (try_socket(logdepth, "libs/socket/SD", "", test_c, NULL, NULL, 0)) {
 		report("found\n");
 		return 0;
 	}
@@ -443,7 +448,7 @@ int find_socket_SD(int logdepth, int fatal)
 	return 1;
 }
 
-int find_socket_shutdown(int logdepth, int fatal)
+int find_socket_shutdown(const char *name, int logdepth, int fatal)
 {
 	char test_c[] =
 		NL "int main() {"
@@ -467,7 +472,7 @@ int find_socket_shutdown(int logdepth, int fatal)
 	logprintf(logdepth, "find_socket_shutdown: trying to find shutdown()...\n");
 	logdepth++;
 
-	if (try_socket(logdepth, "libs/socket/shutdown", "", test_c, "+", "+", 0)) {
+	if (try_socket(logdepth, "libs/socket/shutdown", "", test_c, NULL, NULL, 0)) {
 		report("found\n");
 		return 0;
 	}
@@ -477,7 +482,7 @@ int find_socket_shutdown(int logdepth, int fatal)
 	return 1;
 }
 
-int find_socket_sockaddr_in(int logdepth, int fatal)
+int find_socket_sockaddr_in(const char *name, int logdepth, int fatal)
 {
 	static char *test_c =
 		NL "#include <stdlib.h>"
@@ -516,9 +521,9 @@ int find_socket_sockaddr_in(int logdepth, int fatal)
 	logdepth++;
 
 	/* Look at some standard places */
-	if (try_socket(logdepth, "libs/socket/sockaddr_in", "", test_c, "+", "+", 1)) return 0;
-	if (try_socket(logdepth, "libs/socket/sockaddr_in", inc_db, test_c, "+", "+", 1)) return 0;
-	if (try_socket(logdepth, "libs/socket/sockaddr_in", inc_inet, test_c, "+", "+", 1)) return 0;
+	if (try_socket(logdepth, "libs/socket/sockaddr_in", "", test_c, NULL, NULL, 1)) return 0;
+	if (try_socket(logdepth, "libs/socket/sockaddr_in", inc_db, test_c, NULL, NULL, 1)) return 0;
+	if (try_socket(logdepth, "libs/socket/sockaddr_in", inc_inet, test_c, NULL, NULL, 1)) return 0;
 
 	put("libs/socket/sockaddr_in/presents", sfalse);
 	report("not found\n");
@@ -564,7 +569,7 @@ static const char *localhost(void)
 	return "0x7f000001";
 }
 
-int find_socket_lac(int logdepth, int fatal)
+int find_socket_lac(const char *name, int logdepth, int fatal)
 {
 	char test_c[1024];
 
@@ -596,14 +601,14 @@ int find_socket_lac(int logdepth, int fatal)
 	free(inc_dns);
 
 	/* Look at some standard places */
-	if (try_socket(logdepth, "libs/socket/lac", inc_linux, test_c, "+", "+", 1)) return 0;
+	if (try_socket(logdepth, "libs/socket/lac", inc_linux, test_c, NULL, NULL, 1)) return 0;
 
 	put("libs/socket/lac/presents", sfalse);
 	report("not found\n");
 	return 1;
 }
 
-int find_socket_recvsend(int logdepth, int fatal)
+int find_socket_recvsend(const char *name, int logdepth, int fatal)
 {
 	char test_c[1024];
 
@@ -644,14 +649,14 @@ int find_socket_recvsend(int logdepth, int fatal)
 	free(inc_dns);
 
 	/* Look at some standard places */
-	if (try_socket(logdepth, "libs/socket/recvsend", inc_linux, test_c, "+", "+", 1)) return 0;
+	if (try_socket(logdepth, "libs/socket/recvsend", inc_linux, test_c, NULL, NULL, 1)) return 0;
 
 	put("libs/socket/recvsend/presents", sfalse);
 	report("not found\n");
 	return 1;
 }
 
-int find_socket_readwrite(int logdepth, int fatal)
+int find_socket_readwrite(const char *name, int logdepth, int fatal)
 {
 	char test_c[1024];
 
@@ -689,14 +694,14 @@ int find_socket_readwrite(int logdepth, int fatal)
 	free(inc_dns);
 
 	/* Look at some standard places */
-	if (try_socket(logdepth, "libs/socket/readwrite", inc_linux, test_c, "+", "+", 1)) return 0;
+	if (try_socket(logdepth, "libs/socket/readwrite", inc_linux, test_c, NULL, NULL, 1)) return 0;
 
 	put("libs/socket/readwrite/presents", sfalse);
 	report("not found\n");
 	return 1;
 }
 
-int find_socket_ntoh(int logdepth, int fatal)
+int find_socket_ntoh(const char *name, int logdepth, int fatal)
 {
 	char test_c[] =
 		NL "int main() {"
@@ -723,14 +728,14 @@ int find_socket_ntoh(int logdepth, int fatal)
 	logdepth++;
 
 	for(i = inc; *i != NULL; i++) {
-		if (try_socket_(logdepth, *i, test_c, "+", "+")) {
+		if (try_socket_(logdepth, *i, test_c, NULL, NULL)) {
 			put("libs/socket/ntoh/presents", strue);
 			put("libs/socket/ntoh/includes", *i);
 			report("OK (%s)\n", *i);
 			return 0;
 		}
 
-		if (try_socket_(logdepth, *i, test_c, "+", "+-lws2_32")) {
+		if (try_socket_(logdepth, *i, test_c, NULL, "-lws2_32")) {
 			put("libs/socket/ntoh/presents", strue);
 			put("libs/socket/ntoh/includes", *i);
 			report("OK (%s)\n", *i);
@@ -742,3 +747,167 @@ int find_socket_ntoh(int logdepth, int fatal)
 	report("not found\n");
 	return 1;
 }
+
+int find_socket_inetaddr(const char *name, int logdepth, int fatal)
+{
+	const char *test_c =
+		NL "#include <stdio.h>"
+		NL "int main() {"
+		NL "	SOCK_INIT;"
+		NL "	if (inet_addr(\"1.2.3.4\") != INADDR_NONE)"
+		NL "		puts(\"OK\");"
+		NL "	return 0;"
+		NL "}"
+		NL;
+
+	if (require("cc/cc", logdepth, fatal))
+		return 1;
+
+	if (require("libs/socket/socket/presents", logdepth, fatal))
+		return 1;
+
+	report("Checking for inet_addr()... ");
+	logprintf(logdepth, "find_socket_inetaddr: trying to find inet_addr...\n");
+	logdepth++;
+
+	if (try_socket(logdepth, "libs/socket/inet_addr", "#include <sys/socket.h>\n#include <netinet/in.h>\n#include <arpa/inet.h>\n", test_c, NULL, NULL, 1)) return 0;
+	if (try_socket(logdepth, "libs/socket/inet_addr", "#include <winsock2.h>\n#include <windows.h>\n", test_c, NULL, NULL, 1)) return 0;
+	return try_fail(logdepth, "libs/socket/inet_addr");
+}
+
+int find_socket_gethostbyname(const char *name, int logdepth, int fatal)
+{
+	const char *test_c =
+		NL "#include <stdio.h>"
+		NL "int main() {"
+		NL "	SOCK_INIT;"
+		NL "	if (gethostbyname(\"127.0.0.1\"))"
+		NL "		puts(\"OK\");"
+		NL "	return 0;"
+		NL "}"
+		NL;
+
+	if (require("cc/cc", logdepth, fatal))
+		return 1;
+
+	if (require("libs/socket/socket/presents", logdepth, fatal))
+		return 1;
+
+	report("Checking for gethostbyname()... ");
+	logprintf(logdepth, "find_socket_gethostbyname: trying to find gethostbyname()...\n");
+	logdepth++;
+
+	if (try_socket(logdepth, "libs/socket/gethostbyname", "#include <netdb.h>\n", test_c, NULL, NULL, 1)) return 0;
+	if (try_socket(logdepth, "libs/socket/gethostbyname", "#include <winsock2.h>\n#include <ws2tcpip.h>\n#include <windows.h>\n", test_c, NULL, NULL, 1)) return 0;
+	return try_fail(logdepth, "libs/socket/gethostbyname");
+}
+
+#if 0
+int find_socket_auxtypes(const char *name, int logdepth, int fatal)
+{
+	const char *test_c_template =
+		NL "#include <stdio.h>"
+		NL "int main() {"
+		NL "	%s x = 1;"
+		NL "	if (x)"
+		NL "		puts(\"OK\");"
+		NL "	return 0;"
+		NL "}"
+		NL;
+	char test_c[1024];
+	char node[128];
+	char *nodeend;
+	const char *includes[] = {
+		"#include <sys/socket.h>",
+		"#include <netinet/in.h>",
+		"#include <winsock2.h>\n#include <windows.h>\n",
+		"#include <winsock2.h>\n#include <ws2tcpip.h>\n#include <windows.h>\n",
+		NULL};
+	const char *types[] = {"socklen_t", "in_port_t", "in_addr_t", "sa_family_t", NULL};
+	const char **inc;
+	const char **typ;
+
+	if (require("cc/cc", logdepth, fatal))
+		return 1;
+
+	strcpy(node, "libs/socket/");
+	nodeend = node + strlen(node);
+
+	report("Checking for auxiliary socket-related types... ");
+	logprintf(logdepth, "find_socket_auxtypes: trying to find auxiliary socket-related types...\n");
+	logdepth++;
+
+	for (typ=types; *typ; ++typ) {
+		sprintf(test_c, test_c_template, *typ);
+		strcpy(nodeend, *typ);
+		for (inc=includes; *inc; ++inc) {
+			if (try_icl(logdepth, node, test_c, *inc, NULL, NULL))
+				break;
+		}
+	}
+
+	return 0;
+}
+#endif
+
+static int find_socket_auxtype_impl(const char *name, int logdepth, int fatal, const char *typ)
+{
+	const char *test_c_template =
+		NL "#include <stdio.h>"
+		NL "int main() {"
+		NL "	%s x = 1;"
+		NL "	if (x)"
+		NL "		puts(\"OK\");"
+		NL "	return 0;"
+		NL "}"
+		NL;
+	char test_c[1024];
+	char node[128];
+	const char *includes[] = {
+		"#include <sys/socket.h>",
+		"#include <netinet/in.h>",
+		"#include <winsock2.h>\n#include <windows.h>\n",
+		"#include <winsock2.h>\n#include <ws2tcpip.h>\n#include <windows.h>\n",
+		NULL};
+	const char **inc;
+
+	if (require("cc/cc", logdepth, fatal))
+		return 1;
+
+	sprintf(node, "libs/socket/%s", typ);
+
+	report("Checking for type '%s'... ", typ);
+	logprintf(logdepth, "find_socket_auxtypes: trying to find type '%s'...\n", typ);
+	logdepth++;
+
+	sprintf(test_c, test_c_template, typ);
+	for (inc=includes; *inc; ++inc) {
+		if (try_icl(logdepth, node, test_c, *inc, NULL, NULL)) {
+			report(" found ('%s')\n", *inc);
+			return 0;
+		}
+	}
+
+	report(" not found\n");
+	return 1;
+}
+
+int find_socket_inaddrt(const char *name, int logdepth, int fatal)
+{
+	return find_socket_auxtype_impl(name, logdepth, fatal, "in_addr_t");
+}
+
+int find_socket_inportt(const char *name, int logdepth, int fatal)
+{
+	return find_socket_auxtype_impl(name, logdepth, fatal, "in_port_t");
+}
+
+int find_socket_socklent(const char *name, int logdepth, int fatal)
+{
+	return find_socket_auxtype_impl(name, logdepth, fatal, "socklen_t");
+}
+
+int find_socket_safamilyt(const char *name, int logdepth, int fatal)
+{
+	return find_socket_auxtype_impl(name, logdepth, fatal, "sa_family_t");
+}
