@@ -324,7 +324,7 @@ static void got_kw(tmpasm_t *ctx, tmpasm_kw_t kw, int terminated)
 
 				/* have to restore context keyword after these */
 			if (TOP->kw == KW_FOREACH)
-				TOP->kw = KW_none;
+				TOP->kw = TOP->old_kw;
 
 			break;
 		case KW_IF:
@@ -369,6 +369,7 @@ static void got_kw(tmpasm_t *ctx, tmpasm_kw_t kw, int terminated)
 			TOP->last_code = code_append(ctx, KW_FOREACH);
 			loc_update();
 			TOP->state = ST_PREDATA;
+			TOP->old_kw = TOP->kw;
 			TOP->kw = KW_FOREACH;
 			break;
 		case KW_IN:
@@ -734,7 +735,7 @@ static void execute(tmpasm_t *ctx, tmpasm_exec_t *e)
 	char *cond, *list;
 	const char *i;
 
-	while((e != NULL) && (ctx->runtime_error == 0)) {
+	while((e != NULL) && (ctx->runtime_error == 0) && (ctx->halt == 0)) {
 		ctx->executing = e;
 		switch(e->kw) {
 			case KW_none:
@@ -794,6 +795,7 @@ void tmpasm_execute(tmpasm_t *ctx)
 		tmpasm_runtime_error(ctx, 3, NULL);
 		return;
 	}
+	ctx->halt = 0;
 	ctx->runtime_error = 0;
 	if (ctx->runtime_error_data != NULL) {
 		free(ctx->runtime_error_data);
