@@ -9,6 +9,7 @@
 #include "find_pty.h"
 #include "find_ioctl.h"
 #include "find_term.h"
+#include "find_readline.h"
 
 void deps_tty_init()
 {
@@ -29,4 +30,6 @@ void deps_tty_init()
 
 	dep_add("libs/tty/cfmakeraw/*",        find_tty_cfmakeraw);
 	dep_add("libs/tty/cfsetspeed/*",       find_tty_cfsetspeed);
+
+	dep_add("libs/tty/readline/*",         find_tty_readline);
 }
