@@ -132,6 +132,7 @@ static void help_install(const char *prg)
 	printf("  --quiet     don't print error messages and set exit status to 0\n");
 	printf("  --force     install: attempt to remove destination\n");
 	printf("  --absolute  convert existing paths to absolute paths\n");
+	printf("  -d          force destination to be a directory (even remove old dest)\n");
 	printf("  -i          install (copy) files\n");
 	printf("  -c          install (copy) files\n");
 	printf("  -u          uninstall (remove) files - see --help uninstall\n");
@@ -307,13 +308,16 @@ static int path_concat(char *out, size_t out_len, const char *dir, const char *f
 	return 0;
 }
 
-static int safe_copy(const char *src, const char *dst, int force, int quiet)
+static int safe_mkdir_p(const char *path);
+
+
+static int safe_copy(const char *src, const char *dst, int force, int quiet, int dst_is_dir)
 {
 	char buff[16384], dst2[MY_PATH_MAX];
 	const char *dst_name = NULL;
 	FILE *fd, *fs;
 	struct stat st, dst_st;
-	int err = 0;
+	int err = 0, remove_dst = 1;
 
 	if (stat(src, &st) != 0) {
 		if (!quiet)
@@ -323,7 +327,8 @@ static int safe_copy(const char *src, const char *dst, int force, int quiet)
 
 	/* make sure symlinks/sockets/etc. are not copied into: remove any dst that's not a dir or a regular file */
 	if (stat(dst, &dst_st) == 0) {
-		if (!S_ISDIR(dst_st.st_mode) || !S_ISREG(dst_st.st_mode)) {
+		int try_remove = dst_is_dir ? (!S_ISDIR(dst_st.st_mode)) : (!S_ISDIR(dst_st.st_mode) || !S_ISREG(dst_st.st_mode));
+		if (try_remove) {
 			if (remove(dst) != 0) {
 				if (!quiet)
 					fprintf(stderr, "sccbox: filed to remove non-regular/non-directory target %s\n", dst);
@@ -332,6 +337,15 @@ static int safe_copy(const char *src, const char *dst, int force, int quiet)
 		}
 	}
 
+	if (dst_is_dir) {
+		remove_dst = 0;
+		if (safe_mkdir_p(dst) != 0) {
+			if (!quiet)
+				fprintf(stderr, "sccbox: filed to create target directory %s\n", dst);
+			return 1;
+		}
+	}
+
 	fs = fopen(src, "rb");
 	if (fs == NULL) {
 		if (!quiet)
@@ -339,7 +353,7 @@ static int safe_copy(const char *src, const char *dst, int force, int quiet)
 		return 1;
 	}
 
-	if (force)
+	if ((force) && (remove_dst))
 		remove(dst);
 
 	fd = fopen(dst, "wb");
@@ -445,10 +459,11 @@ static const char *safe_get_pwd(int quiet)
 
 /* create a symlink - needs to work only on UNIX but needs to compile on win32,
    so use system("ln -s") and avoid link-related syscalls */
-static int safe_link(const char *src, const char *dst, int absolute, int force, int quiet)
+static int safe_link(const char *src, const char *dst, int absolute, int force, int quiet, int dst_is_dir)
 {
 	char cmd[MY_PATH_MAX*2+16], full_src_tmp[MY_PATH_MAX];
 	const char *full_src, *supp;
+	int remove_dst = 1;
 
 	if ((absolute) && (*src != '/')) {
 		/* fix up relative source paths */
@@ -468,6 +483,24 @@ static int safe_link(const char *src, const char *dst, int absolute, int force,
 	else
 		full_src = src;
 
+	if (dst_is_dir) {
+		struct stat dst_st;
+		if ((stat(dst, &dst_st) == 0) && (!S_ISDIR(dst_st.st_mode))) {
+			if (remove(dst) != 0) {
+				if (!quiet)
+					fprintf(stderr, "sccbox: filed to remove non-directory target %s\n", dst);
+				return 1;
+			}
+		}
+
+		remove_dst = 0;
+		if (safe_mkdir_p(dst) != 0) {
+			if (!quiet)
+				fprintf(stderr, "sccbox: filed to create target directory %s\n", dst);
+			return 1;
+		}
+	}
+
 	if (force)
 		remove(dst);
 
@@ -558,7 +591,7 @@ int cmd_rm(int argc, char *argv[])
 
 int cmd_install(int argc, char *argv[], scc_mode_t mode, int absolute)
 {
-	int force = 0, err = 0, quiet = 0, dst_is_dir;
+	int force = 0, err = 0, quiet = 0, dst_is_dir, force_dst_dir = 0;
 	const char *last;
 
 	load_flags(
@@ -581,6 +614,10 @@ int cmd_install(int argc, char *argv[], scc_mode_t mode, int absolute)
 				force = 1;
 				kill_flag;
 				break;
+			case 'd': /* --directory */
+				force_dst_dir = 1;
+				kill_flag;
+				break;
 			case 'q': /* --quiet */
 				quiet = 1;
 				kill_flag;
@@ -600,9 +637,10 @@ int cmd_install(int argc, char *argv[], scc_mode_t mode, int absolute)
 	dst_is_dir = safe_isdir(last);
 
 	load_args_minus(1,
+printf("arg=%s last=%s force=%d q=%d d=%d\n", arg, last, force, quiet, force_dst_dir);
 		switch(mode) {
-			case INSTALL:   err |= safe_copy(arg, last, force, quiet); break;
-			case LINSTALL:  err |= safe_link(arg, last, absolute, force, quiet); break;
+			case INSTALL:   err |= safe_copy(arg, last, force, quiet, force_dst_dir); break;
+			case LINSTALL:  err |= safe_link(arg, last, absolute, force, quiet, force_dst_dir); break;
 			case UNINSTALL:
 				if (dst_is_dir) {
 					char path[MY_PATH_MAX];
