@@ -59,11 +59,8 @@
 #include "compat_nls.h"
 #include "event.h"
 
-#include "obj_pinvia_draw.h"
-#include "obj_pad_draw.h"
 #include "obj_line_draw.h"
 #include "obj_arc_draw.h"
-#include "obj_elem_draw.h"
 #include "obj_text_draw.h"
 #include "obj_rat_draw.h"
 #include "obj_poly_draw.h"
@@ -71,9 +68,6 @@
 
 #include "tool.h"
 
-
-/* --------------------------------------------------------------------------- */
-
 /* %start-doc actions 00delta
 
 Many actions take a @code{delta} parameter as the last parameter,
@@ -126,25 +120,6 @@ Affects all objects which are both selected and of the @var{Type} specified.
 
 %end-doc */
 
-/*  %start-doc actions 00macros
-
-@macro pinshapes
-
-Pins, pads, and vias can have various shapes.  All may be round.  Pins
-and pads may be square (obviously "square" pads are usually
-rectangular).  Pins and vias may be octagonal.  When you change a
-shape flag of an element, you actually change all of its pins and
-pads.
-
-Note that the square flag takes precedence over the octagon flag,
-thus, if both the square and octagon flags are set, the object is
-square.  When the square flag is cleared, the pins and pads will be
-either round or, if the octagon flag is set, octagonal.
-
-@end macro
-
-%end-doc */
-
 int defer_updates = 0;
 int defer_needs_update = 0;
 
@@ -158,30 +133,6 @@ void pcb_clear_warnings()
 
 	conf_core.temp.rat_warn = pcb_false;
 
-	for(n = pcb_r_first(PCB->Data->pin_tree, &it); n != NULL; n = pcb_r_next(&it)) {
-		if (PCB_FLAG_TEST(PCB_FLAG_WARN, (pcb_any_obj_t *)n)) {
-			PCB_FLAG_CLEAR(PCB_FLAG_WARN, (pcb_any_obj_t *)n);
-			pcb_pin_invalidate_draw((pcb_pin_t *)n);
-		}
-	}
-	pcb_r_end(&it);
-
-	for(n = pcb_r_first(PCB->Data->via_tree, &it); n != NULL; n = pcb_r_next(&it)) {
-		if (PCB_FLAG_TEST(PCB_FLAG_WARN, (pcb_any_obj_t *)n)) {
-			PCB_FLAG_CLEAR(PCB_FLAG_WARN, (pcb_any_obj_t *)n);
-			pcb_via_invalidate_draw((pcb_pin_t *)n);
-		}
-	}
-	pcb_r_end(&it);
-
-	for(n = pcb_r_first(PCB->Data->pad_tree, &it); n != NULL; n = pcb_r_next(&it)) {
-		if (PCB_FLAG_TEST(PCB_FLAG_WARN, (pcb_any_obj_t *)n)) {
-			PCB_FLAG_CLEAR(PCB_FLAG_WARN, (pcb_any_obj_t *)n);
-			pcb_pad_invalidate_draw((pcb_pad_t *)n);
-		}
-	}
-	pcb_r_end(&it);
-
 	for(n = pcb_r_first(PCB->Data->padstack_tree, &it); n != NULL; n = pcb_r_next(&it)) {
 		if (PCB_FLAG_TEST(PCB_FLAG_WARN, (pcb_any_obj_t *)n)) {
 			PCB_FLAG_CLEAR(PCB_FLAG_WARN, (pcb_any_obj_t *)n);
@@ -232,7 +183,7 @@ void pcb_release_mode(void)
 	pcb_tool_release_mode();
 
 	if (pcb_tool_is_saved)
-		pcb_crosshair_restore_mode();
+		pcb_tool_restore();
 	pcb_tool_is_saved = pcb_false;
 	pcb_draw();
 }
@@ -250,7 +201,7 @@ void pcb_notify_mode(void)
 	pcb_draw();
 }
 
-void pcb_event_move_crosshair(int ev_x, int ev_y)
+void pcb_event_move_crosshair(pcb_coord_t ev_x, pcb_coord_t ev_y)
 {
 	if (pcb_mid_stroke)
 		pcb_stub_stroke_record(ev_x, ev_y);
