@@ -38,6 +38,7 @@
 #include "draw.h"
 #include "event.h"
 #include "safe_fs.h"
+#include "tool.h"
 
 
 pcb_board_t *PCB;
@@ -79,8 +80,8 @@ pcb_board_t *pcb_board_new_(pcb_bool SetDefaultNames)
 	ptr = calloc(1, sizeof(pcb_board_t));
 	ptr->Data = pcb_buffer_new(ptr);
 
-	ptr->ThermStyle = 4;
-	ptr->IsleArea = 2.e8;
+	conf_set(CFR_INTERNAL, "design/poly_isle_area", -1, "200000000", POL_OVERWRITE);
+
 	ptr->RatDraw = pcb_false;
 
 	/* NOTE: we used to set all the pcb flags on ptr here, but we don't need to do that anymore due to the new conf system */
@@ -90,22 +91,13 @@ pcb_board_t *pcb_board_new_(pcb_bool SetDefaultNames)
 	save = PCB;
 	PCB = ptr;
 	pcb_layer_parse_group_string(ptr, conf_core.design.groups, PCB_MAX_LAYER, 0);
-	pcb_event(PCB_EVENT_ROUTE_STYLES_CHANGED, NULL);
 	PCB = save;
 
 	ptr->Zoom = conf_core.editor.zoom;
-	ptr->MaxWidth = conf_core.design.max_width;
-	ptr->MaxHeight = conf_core.design.max_height;
+	ptr->MaxHeight = ptr->MaxWidth = PCB_MM_TO_COORD(20); /* should be overriden by the default design */
 	ptr->ID = pcb_create_ID_get();
 	ptr->ThermScale = 0.5;
 
-	ptr->Bloat = conf_core.design.bloat;
-	ptr->Shrink = conf_core.design.shrink;
-	ptr->minWid = conf_core.design.min_wid;
-	ptr->minSlk = conf_core.design.min_slk;
-	ptr->minDrill = conf_core.design.min_drill;
-	ptr->minRing = conf_core.design.min_ring;
-
 	for (i = 0; i < PCB_MAX_LAYER; i++)
 		ptr->Data->Layer[i].name = pcb_strdup(conf_core.design.default_layer_name[i]);
 
@@ -160,6 +152,8 @@ pcb_board_t *pcb_board_new(int inhibit_events)
 		}
 	}
 
+	pcb_event(PCB_EVENT_ROUTE_STYLES_CHANGED, NULL);
+
 	PCB = old;
 	return nw;
 }
@@ -187,17 +181,6 @@ typedef struct {
 	int nunplated;
 } HoleCountStruct;
 
-static pcb_r_dir_t hole_counting_old_callback(const pcb_box_t * b, void *cl)
-{
-	pcb_pin_t *pin = (pcb_pin_t *) b;
-	HoleCountStruct *hcs = (HoleCountStruct *) cl;
-	if (PCB_FLAG_TEST(PCB_FLAG_HOLE, pin))
-		hcs->nunplated++;
-	else
-		hcs->nplated++;
-	return PCB_R_DIR_FOUND_CONTINUE;
-}
-
 #warning padstack TODO: move this to obj_pstk.c after pinvia removal
 #include "obj_pstk_inlines.h"
 static pcb_r_dir_t hole_counting_callback(const pcb_box_t * b, void *cl)
@@ -219,8 +202,6 @@ void pcb_board_count_holes(pcb_board_t *pcb, int *plated, int *unplated, const p
 {
 	HoleCountStruct hcs = { 0, 0 };
 
-	pcb_r_search(pcb->Data->pin_tree, within_area, NULL, hole_counting_old_callback, &hcs, NULL);
-	pcb_r_search(pcb->Data->via_tree, within_area, NULL, hole_counting_old_callback, &hcs, NULL);
 	pcb_r_search(pcb->Data->padstack_tree, within_area, NULL, hole_counting_callback, &hcs, NULL);
 
 	if (plated != NULL)
@@ -275,12 +256,12 @@ void pcb_board_remove(pcb_board_t *Ptr)
 }
 
 /* sets cursor grid with respect to grid offset values */
-void pcb_board_set_grid(pcb_coord_t Grid, pcb_bool align)
+void pcb_board_set_grid(pcb_coord_t Grid, pcb_bool align, pcb_coord_t ox, pcb_coord_t oy)
 {
 	if (Grid >= 1 && Grid <= PCB_MAX_GRID) {
 		if (align) {
-			PCB->GridOffsetX = pcb_crosshair.X % Grid;
-			PCB->GridOffsetY = pcb_crosshair.Y % Grid;
+			PCB->GridOffsetX = ox % Grid;
+			PCB->GridOffsetY = oy % Grid;
 		}
 		PCB->Grid = Grid;
 		conf_set_design("editor/grid", "%$mS", Grid);
@@ -289,6 +270,14 @@ void pcb_board_set_grid(pcb_coord_t Grid, pcb_bool align)
 	}
 }
 
+void pcb_board_set_unit(pcb_board_t *pcb, const pcb_unit_t *new_unit)
+{
+	if (new_unit != NULL && new_unit->allow != PCB_UNIT_NO_PRINT) {
+		conf_set(CFR_DESIGN, "editor/grid_unit", -1, new_unit->suffix, POL_OVERWRITE);
+		pcb_attrib_put(pcb, "PCB::grid::unit", new_unit->suffix);
+	}
+}
+
 /* sets a new line thickness */
 void pcb_board_set_line_width(pcb_coord_t Size)
 {
