@@ -39,8 +39,6 @@ enum {
 	PCB_NUM_NETLISTS         /* so that we know how many netlists we are dealing with */
 };
 
-#include "const.h"
-#include "macro.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -64,45 +62,40 @@ enum {
 	   |  This struct is also used for the remove list and for buffer handling
 	 */
 struct pcb_board_s {
-	long ID;											/* see macro.h */
-	char *Name,										/* name of board */
-	 *Filename,										/* name of file (from load) */
-	 *PrintFilename,							/* from print dialog */
-	 *Netlistname,								/* name of netlist file */
-	  ThermStyle;									/* type of thermal to place with thermal tool */
-	pcb_bool Changed,									/* layout has been changed */
-	  ViaOn,											/* visibility flags */
-	  RatOn, InvisibleObjectsOn, PinOn,
-	  RatDraw,										/* we're drawing rats */
-	  SubcOn, SubcPartsOn, padstack_mark_on, hole_on;
-	pcb_bool loose_subc;          /* when set, subc parts are not locked into the subc */
-	pcb_coord_t CursorX,									/* cursor position as saved with layout */
-	  CursorY;
-	pcb_coord_t Bloat,									/* drc sizes saved with layout */
-	  Shrink, minWid, minSlk, minDrill, minRing;
-	pcb_coord_t GridOffsetX,						/* as saved with layout */
-	  GridOffsetY, MaxWidth,			/* allowed size */
-	  MaxHeight;
+	long ID;
+	char *Name;                                     /* name of board */
+	char *Filename;                                 /* name of file (from load) */
+	char *PrintFilename;                            /* from print dialog */
+	char *Netlistname;                              /* name of netlist file */
+
+	pcb_bool Changed;                               /* layout has been changed */
+
+	pcb_bool pstk_on, RatOn, InvisibleObjectsOn,
+	         SubcOn, SubcPartsOn,
+	         padstack_mark_on, hole_on;             /* visibility flags */
+	pcb_bool RatDraw;                               /* we're drawing rats */
+
+	pcb_bool loose_subc;                            /* when set, subc parts are not locked into the subc */
+	pcb_coord_t Zoom, CursorX, CursorY;             /* cursor position as saved with layout */
+	pcb_coord_t Grid, GridOffsetX, GridOffsetY;     /* as saved with layout */
+	pcb_coord_t MaxWidth, MaxHeight;                /* board dimensions (drawing area extents) */
+
+	double ThermScale;                              /* scale factor used with thermals */
 
-	pcb_coord_t Grid;										/* used grid with offsets */
-	double Zoom,									/* zoom factor */
-	  IsleArea,										/* minimum poly island to retain */
-	  ThermScale;									/* scale factor used with thermals */
 	pcb_fontkit_t fontkit;
 	pcb_layer_stack_t LayerGroups;
 	vtroutestyle_t RouteStyle;
 	pcb_lib_t NetlistLib[PCB_NUM_NETLISTS];
 	pcb_ratspatch_line_t *NetlistPatches, *NetlistPatchLast;
 	pcb_attribute_list_t Attributes;
-	pcb_data_t *Data;							/* entire database */
 
-	pcb_bool is_footprint;						/* If set, the user has loaded a footprint, not a pcb. */
+	pcb_data_t *Data;                               /* entire database */
+	pcb_bool is_footprint;                          /* If set, the user has loaded a footprint, not a pcb. */
 
 	const pcb_attribute_list_t *pen_attr;
 
-
 /* netlist states */
-	int netlist_frozen;                /* counter */
+	int netlist_frozen;                             /* counter */
 	unsigned netlist_needs_update:1;
 };
 
@@ -137,7 +130,7 @@ void pcb_board_count_holes(pcb_board_t *pcb, int *plated, int *unplated, const p
 #define	PCB_CSWAP_Y(y, h, cond)		((cond) ? (h+PCB_SWAP_SIGN_Y(y)) : (y))
 
 /* Conditionally allow subc parts to be reached directly in search masks */
-#define PCB_LOOSE_SUBC (PCB->loose_subc ? PCB_TYPE_SUBC_PART : 0)
+#define PCB_LOOSE_SUBC (PCB->loose_subc ? PCB_OBJ_SUBC_PART : 0)
 
 
 /* changes the name of a layout; Name is allocated by the caller (no strdup() is made) */
@@ -151,8 +144,10 @@ void pcb_board_resize(pcb_coord_t Width, pcb_coord_t Height);
 /* free the board and remove its undo list */
 void pcb_board_remove(pcb_board_t *Ptr);
 
-/* sets cursor grid with respect to grid offset values */
-void pcb_board_set_grid(pcb_coord_t Grid, pcb_bool align);
+/* sets cursor grid with respect to grid offset values or default unit*/
+void pcb_board_set_grid(pcb_coord_t Grid, pcb_bool align, pcb_coord_t ox, pcb_coord_t oy);
+void pcb_board_set_unit(pcb_board_t *pcb, const pcb_unit_t *new_unit);
+
 
 /* sets a new line thickness */
 void pcb_board_set_line_width(pcb_coord_t Size);
