@@ -46,23 +46,23 @@
 #include "funchash_core.h"
 #include "compat_misc.h"
 #include "compat_nls.h"
-#include "obj_all_op.h"
+#include "obj_arc_op.h"
+#include "obj_line_op.h"
+#include "obj_text_op.h"
 #include "obj_subc_op.h"
+#include "obj_poly_op.h"
 #include "obj_pstk_op.h"
+#include "obj_rat_op.h"
 #include "obj_pstk.h"
 #include "layer_grp.h"
 #include "event.h"
 #include "safe_fs.h"
+#include "hid_actions.h"
 
 static pcb_opfunc_t AddBufferFunctions = {
 	pcb_lineop_add_to_buffer,
 	pcb_textop_add_to_buffer,
 	pcb_polyop_add_to_buffer,
-	pcb_viaop_add_to_buffer,
-	pcb_elemop_add_to_buffer,
-	NULL,
-	NULL,
-	NULL,
 	NULL,
 	NULL,
 	pcb_arcop_add_to_buffer,
@@ -72,16 +72,10 @@ static pcb_opfunc_t AddBufferFunctions = {
 	pcb_pstkop_add_to_buffer,
 };
 
-#warning TODO: rename this; move_to_buffer is also used to move from buffer to pcb
 static pcb_opfunc_t MoveBufferFunctions = {
 	pcb_lineop_move_buffer,
 	pcb_textop_move_buffer,
 	pcb_polyop_move_buffer,
-	pcb_viaop_move_buffer,
-	pcb_elemop_move_buffer,
-	NULL,
-	NULL,
-	NULL,
 	NULL,
 	NULL,
 	pcb_arcop_move_buffer,
@@ -123,7 +117,7 @@ void pcb_buffer_clear(pcb_board_t *pcb, pcb_buffer_t *Buffer)
 	pcb_buffer_clear_(pcb, Buffer, pcb_true);
 }
 
-/* add or move selected */
+/* add (or move) selected */
 static void pcb_buffer_toss_selected(pcb_opfunc_t *fnc, pcb_board_t *pcb, pcb_buffer_t *Buffer, pcb_coord_t X, pcb_coord_t Y, pcb_bool LeaveSelected)
 {
 	pcb_opctx_t ctx;
@@ -141,7 +135,7 @@ static void pcb_buffer_toss_selected(pcb_opfunc_t *fnc, pcb_board_t *pcb, pcb_bu
 	pcb_notify_crosshair_change(pcb_false);
 	ctx.buffer.src = pcb->Data;
 	ctx.buffer.dst = Buffer->Data;
-	pcb_selected_operation(pcb, pcb->Data, fnc, &ctx, pcb_false, PCB_TYPEMASK_ALL);
+	pcb_selected_operation(pcb, pcb->Data, fnc, &ctx, pcb_false, PCB_OBJ_ANY & (~PCB_OBJ_SUBC_PART));
 
 	/* set origin to passed or current position */
 	if (X || Y) {
@@ -161,11 +155,6 @@ void pcb_buffer_add_selected(pcb_board_t *pcb, pcb_buffer_t *Buffer, pcb_coord_t
 	pcb_buffer_toss_selected(&AddBufferFunctions, pcb, Buffer, X, Y, LeaveSelected);
 }
 
-void pcb_buffer_move_selected(pcb_board_t *pcb, pcb_buffer_t *Buffer, pcb_coord_t X, pcb_coord_t Y, pcb_bool LeaveSelected)
-{
-	pcb_buffer_toss_selected(&MoveBufferFunctions, pcb, Buffer, X, Y, LeaveSelected);
-}
-
 /*---------------------------------------------------------------------------*/
 
 static const char pcb_acts_LoadFootprint[] = "pcb_load_footprint(filename[,refdes,value])";
@@ -185,49 +174,30 @@ int pcb_act_LoadFootprint(int argc, const char **argv, pcb_coord_t x, pcb_coord_
 	const char *name = PCB_ACTION_ARG(0);
 	const char *refdes = PCB_ACTION_ARG(1);
 	const char *value = PCB_ACTION_ARG(2);
-	pcb_element_t *e;
 	pcb_subc_t *s;
 	pcb_cardinal_t len;
 
 	if (!name)
 		PCB_ACT_FAIL(LoadFootprint);
 
-	if (pcb_element_load_footprint_by_name(PCB_PASTEBUFFER, name))
+	if (!pcb_buffer_load_footprint(PCB_PASTEBUFFER, name, NULL))
 		return 1;
 
-	len = elementlist_length(&PCB_PASTEBUFFER->Data->Element) + pcb_subclist_length(&PCB_PASTEBUFFER->Data->subc);
+	len = pcb_subclist_length(&PCB_PASTEBUFFER->Data->subc);
 
 	if (len == 0) {
-		pcb_message(PCB_MSG_ERROR, "Footprint %s contains no elements or subcircuits", name);
+		pcb_message(PCB_MSG_ERROR, "Footprint %s contains no subcircuits", name);
 		return 1;
 	}
 	if (len > 1) {
-		pcb_message(PCB_MSG_ERROR, "Footprint %s contains multiple elements and/or subcircuits", name);
+		pcb_message(PCB_MSG_ERROR, "Footprint %s contains multiple subcircuits", name);
 		return 1;
 	}
 
-	e = elementlist_first(&PCB_PASTEBUFFER->Data->Element);
-	if (e != NULL) {
-	if (e->Name[0].TextString)
-		free(e->Name[0].TextString);
-	e->Name[0].TextString = pcb_strdup(name);
-
-	if (e->Name[1].TextString)
-		free(e->Name[1].TextString);
-	e->Name[1].TextString = refdes ? pcb_strdup(refdes) : 0;
-
-	if (e->Name[2].TextString)
-		free(e->Name[2].TextString);
-	e->Name[2].TextString = value ? pcb_strdup(value) : 0;
-	}
-
 	s = pcb_subclist_first(&PCB_PASTEBUFFER->Data->subc);
-#warning element TODO: remove this check when elements are removed: we know len == 1
-	if (s != NULL) {
-		pcb_attribute_put(&s->Attributes, "refdes", refdes);
-		pcb_attribute_put(&s->Attributes, "name", name);
-		pcb_attribute_put(&s->Attributes, "value", value);
-	}
+	pcb_attribute_put(&s->Attributes, "refdes", refdes);
+	pcb_attribute_put(&s->Attributes, "footprint", name);
+	pcb_attribute_put(&s->Attributes, "value", value);
 
 	return 0;
 }
@@ -265,15 +235,8 @@ pcb_bool pcb_buffer_load_layout(pcb_board_t *pcb, pcb_buffer_t *Buffer, const ch
 
 void pcb_buffer_rotate90(pcb_buffer_t *Buffer, pcb_uint8_t Number)
 {
-	/* rotate vias */
-	PCB_VIA_LOOP(Buffer->Data);
-	{
-		pcb_r_delete_entry(Buffer->Data->via_tree, (pcb_box_t *) via);
-		PCB_VIA_ROTATE90(via, Buffer->X, Buffer->Y, Number);
-		pcb_pin_bbox(via);
-		pcb_r_insert_entry(Buffer->Data->via_tree, (pcb_box_t *) via);
-	}
-	PCB_END_LOOP;
+	pcb_undo_freeze_serial();
+	pcb_undo_freeze_add();
 
 	PCB_PADSTACK_LOOP(Buffer->Data);
 	{
@@ -281,81 +244,74 @@ void pcb_buffer_rotate90(pcb_buffer_t *Buffer, pcb_uint8_t Number)
 	}
 	PCB_END_LOOP;
 
-	/* elements */
-	PCB_ELEMENT_LOOP(Buffer->Data);
-	{
-		pcb_element_rotate90(Buffer->Data, element, Buffer->X, Buffer->Y, Number);
-	}
-	PCB_END_LOOP;
-
 	PCB_SUBC_LOOP(Buffer->Data);
 	{
-		pcb_obj_rotate90(PCB_TYPE_SUBC, subc, subc, subc, Buffer->X, Buffer->Y, Number);
+		pcb_obj_rotate90(PCB_OBJ_SUBC, subc, subc, subc, Buffer->X, Buffer->Y, Number);
 	}
 	PCB_END_LOOP;
 
 	/* all layer related objects */
 	PCB_LINE_ALL_LOOP(Buffer->Data);
 	{
-		pcb_r_delete_entry(layer->line_tree, (pcb_box_t *) line);
+		if (layer->line_tree != NULL)
+			pcb_r_delete_entry(layer->line_tree, (pcb_box_t *) line);
 		pcb_line_rotate90(line, Buffer->X, Buffer->Y, Number);
-		pcb_r_insert_entry(layer->line_tree, (pcb_box_t *) line);
+		if (layer->line_tree != NULL)
+			pcb_r_insert_entry(layer->line_tree, (pcb_box_t *) line);
 	}
 	PCB_ENDALL_LOOP;
 	PCB_ARC_ALL_LOOP(Buffer->Data);
 	{
-		pcb_r_delete_entry(layer->arc_tree, (pcb_box_t *) arc);
+		if (layer->arc_tree != NULL)
+			pcb_r_delete_entry(layer->arc_tree, (pcb_box_t *) arc);
 		pcb_arc_rotate90(arc, Buffer->X, Buffer->Y, Number);
-		pcb_r_insert_entry(layer->arc_tree, (pcb_box_t *) arc);
+		if (layer->arc_tree != NULL)
+			pcb_r_insert_entry(layer->arc_tree, (pcb_box_t *) arc);
 	}
 	PCB_ENDALL_LOOP;
 	PCB_TEXT_ALL_LOOP(Buffer->Data);
 	{
-		pcb_r_delete_entry(layer->text_tree, (pcb_box_t *) text);
+		if (layer->text_tree != NULL)
+			pcb_r_delete_entry(layer->text_tree, (pcb_box_t *) text);
 		pcb_text_rotate90(text, Buffer->X, Buffer->Y, Number);
-		pcb_r_insert_entry(layer->text_tree, (pcb_box_t *) text);
+		if (layer->text_tree != NULL)
+			pcb_r_insert_entry(layer->text_tree, (pcb_box_t *) text);
 	}
 	PCB_ENDALL_LOOP;
 	PCB_POLY_ALL_LOOP(Buffer->Data);
 	{
-		pcb_r_delete_entry(layer->polygon_tree, (pcb_box_t *) polygon);
+		if (layer->polygon_tree != NULL)
+			pcb_r_delete_entry(layer->polygon_tree, (pcb_box_t *) polygon);
 		pcb_poly_rotate90(polygon, Buffer->X, Buffer->Y, Number);
-		pcb_r_insert_entry(layer->polygon_tree, (pcb_box_t *) polygon);
+		if (layer->polygon_tree != NULL)
+			pcb_r_insert_entry(layer->polygon_tree, (pcb_box_t *) polygon);
 	}
 	PCB_ENDALL_LOOP;
 
 	/* finally the origin and the bounding box */
 	PCB_COORD_ROTATE90(Buffer->X, Buffer->Y, Buffer->X, Buffer->Y, Number);
 	pcb_box_rotate90(&Buffer->BoundingBox, Buffer->X, Buffer->Y, Number);
+
+	pcb_undo_unfreeze_add();
+	pcb_undo_unfreeze_serial();
 }
 
 void pcb_buffer_rotate(pcb_buffer_t *Buffer, pcb_angle_t angle)
 {
 	double cosa, sina;
 
+	pcb_undo_freeze_serial();
+	pcb_undo_freeze_add();
+
 	cosa = cos(angle * M_PI / 180.0);
 	sina = sin(angle * M_PI / 180.0);
 
-	/* rotate vias */
-	PCB_VIA_LOOP(Buffer->Data);
-	{
-		pcb_via_rotate(Buffer->Data, via, Buffer->X, Buffer->Y, cosa, sina);
-	}
-	PCB_END_LOOP;
-
 	PCB_PADSTACK_LOOP(Buffer->Data);
 	{
 		pcb_pstk_rotate(padstack, Buffer->X, Buffer->Y, cosa, sina, angle);
 	}
 	PCB_END_LOOP;
 
-	/* elements */
-	PCB_ELEMENT_LOOP(Buffer->Data);
-	{
-		pcb_element_rotate(Buffer->Data, element, Buffer->X, Buffer->Y, cosa, sina, angle);
-	}
-	PCB_END_LOOP;
-
 	PCB_SUBC_LOOP(Buffer->Data);
 	{
 		pcb_subc_rotate(subc, Buffer->X, Buffer->Y, cosa, sina, angle);
@@ -381,6 +337,8 @@ void pcb_buffer_rotate(pcb_buffer_t *Buffer, pcb_angle_t angle)
 	PCB_ENDALL_LOOP;
 
 	pcb_set_buffer_bbox(Buffer);
+	pcb_undo_unfreeze_add();
+	pcb_undo_unfreeze_serial();
 }
 
 pcb_data_t *pcb_buffer_new(pcb_board_t *pcb)
@@ -444,11 +402,6 @@ void pcb_buffer_mirror(pcb_board_t *pcb, pcb_buffer_t *Buffer)
 {
 	int i, num_layers;
 
-	if (elementlist_length(&Buffer->Data->Element)) {
-		pcb_message(PCB_MSG_ERROR, _("You can't mirror a buffer that has elements!\n"));
-		return;
-	}
-
 	num_layers = PCB_PASTEBUFFER->Data->LayerN;
 	if (num_layers == 0) /* some buffers don't have layers, just simple objects */
 		num_layers = pcb->Data->LayerN;
@@ -463,7 +416,7 @@ void pcb_buffer_mirror(pcb_board_t *pcb, pcb_buffer_t *Buffer)
 	/* set buffer offset to 'mark' position */
 	Buffer->X = PCB_SWAP_X(Buffer->X);
 	Buffer->Y = PCB_SWAP_Y(Buffer->Y);
-	pcb_data_mirror(Buffer->Data, 0, pcb_false, pcb_false);
+	pcb_data_mirror(Buffer->Data, 0, PCB_TXM_COORD, pcb_false);
 	pcb_set_buffer_bbox(Buffer);
 }
 
@@ -474,21 +427,22 @@ void pcb_buffer_flip_side(pcb_board_t *pcb, pcb_buffer_t *Buffer)
 	pcb_layergrp_id_t sgroup, cgroup;
 	pcb_layer_t swap;
 
-	PCB_ELEMENT_LOOP(Buffer->Data);
+#if 0
+/* this results in saving flipped (bottom-side) footprints whenlooking at the board from the bottom */
+	PCB_SUBC_LOOP(Buffer->Data);
 	{
-		r_delete_element(Buffer->Data, element);
-		pcb_element_mirror(Buffer->Data, element, 0);
+		pcb_subc_change_side(&subc, /*2 * pcb_crosshair.Y - PCB->MaxHeight*/0);
 	}
 	PCB_END_LOOP;
-
-#warning subc TODO
+#endif
 
 	/* set buffer offset to 'mark' position */
 	Buffer->X = PCB_SWAP_X(Buffer->X);
 	Buffer->Y = PCB_SWAP_Y(Buffer->Y);
-	PCB_VIA_LOOP(Buffer->Data);
+
+	PCB_PADSTACK_LOOP(Buffer->Data);
 	{
-		pcb_via_flip_side(Buffer->Data, via);
+		pcb_pstk_mirror(padstack, PCB_PSTK_DONT_MIRROR_COORDS, 1, 0);
 	}
 	PCB_END_LOOP;
 	PCB_LINE_ALL_LOOP(Buffer->Data);
@@ -542,23 +496,6 @@ void pcb_buffer_flip_side(pcb_board_t *pcb, pcb_buffer_t *Buffer)
 			Buffer->Data->Layer[snumber] = Buffer->Data->Layer[cnumber];
 			Buffer->Data->Layer[cnumber] = swap;
 			k++;
-			/* move the thermal flags with the layers */
-			PCB_PIN_ALL_LOOP(Buffer->Data);
-			{
-				t1 = PCB_FLAG_THERM_TEST(snumber, pin);
-				t2 = PCB_FLAG_THERM_TEST(cnumber, pin);
-				PCB_FLAG_THERM_ASSIGN(snumber, t2, pin);
-				PCB_FLAG_THERM_ASSIGN(cnumber, t1, pin);
-			}
-			PCB_ENDALL_LOOP;
-			PCB_VIA_LOOP(Buffer->Data);
-			{
-				t1 = PCB_FLAG_THERM_TEST(snumber, via);
-				t2 = PCB_FLAG_THERM_TEST(cnumber, via);
-				PCB_FLAG_THERM_ASSIGN(snumber, t2, via);
-				PCB_FLAG_THERM_ASSIGN(cnumber, t1, via);
-			}
-			PCB_END_LOOP;
 		}
 	}
 	pcb_set_buffer_bbox(Buffer);
@@ -660,21 +597,6 @@ pcb_bool pcb_buffer_copy_to_layout(pcb_board_t *pcb, pcb_coord_t X, pcb_coord_t
 		}
 	}
 
-	/* paste elements */
-	if (pcb->PinOn && pcb_silk_on(pcb)) {
-		PCB_ELEMENT_LOOP(PCB_PASTEBUFFER->Data);
-		{
-#ifdef DEBUG
-			printf("In CopyPastebufferToLayout, pasting element %s\n", element->Name[1].TextString);
-#endif
-			if (PCB_FRONT(element) || pcb->InvisibleObjectsOn) {
-				pcb_elemop_copy(&ctx, element);
-				changed = pcb_true;
-			}
-		}
-		PCB_END_LOOP;
-	}
-
 	/* paste subcircuits */
 	PCB_SUBC_LOOP(PCB_PASTEBUFFER->Data);
 	{
@@ -687,17 +609,8 @@ pcb_bool pcb_buffer_copy_to_layout(pcb_board_t *pcb, pcb_coord_t X, pcb_coord_t
 	}
 	PCB_END_LOOP;
 
-	/* finally the vias and padstacks */
-	if (pcb->ViaOn) {
-		changed |= (pinlist_length(&(PCB_PASTEBUFFER->Data->Via)) != 0);
-		PCB_VIA_LOOP(PCB_PASTEBUFFER->Data);
-		{
-			pcb_viaop_copy(&ctx, via);
-		}
-		PCB_END_LOOP;
-	}
-
-	if (pcb->ViaOn) {
+	/* finally: padstacks */
+	if (pcb->pstk_on) {
 		pcb_board_t dummy;
 		changed |= (padstacklist_length(&(PCB_PASTEBUFFER->Data->padstack)) != 0);
 
@@ -744,6 +657,33 @@ void pcb_buffer_set_number(int Number)
 	}
 }
 
+/* loads footprint data from file/library into buffer (as subcircuit)
+ * returns pcb_false on error
+ * if successful, update some other stuff and reposition the pastebuffer */
+pcb_bool pcb_buffer_load_footprint(pcb_buffer_t *Buffer, const char *Name, const char *fmt)
+{
+	pcb_buffer_clear(PCB, Buffer);
+	if (!pcb_parse_footprint(Buffer->Data, Name, fmt)) {
+		if (conf_core.editor.show_solder_side)
+			pcb_buffer_flip_side(PCB, Buffer);
+		pcb_set_buffer_bbox(Buffer);
+
+		Buffer->X = 0;
+		Buffer->Y = 0;
+		Buffer->from_outside = 1;
+
+		if (pcb_subclist_length(&Buffer->Data->subc)) {
+			pcb_subc_t *subc = pcb_subclist_first(&Buffer->Data->subc);
+			pcb_subc_get_origin(subc, &Buffer->X, &Buffer->Y);
+		}
+		return pcb_true;
+	}
+
+	/* release memory which might have been acquired */
+	pcb_buffer_clear(PCB, Buffer);
+	return pcb_false;
+}
+
 
 /* ---------------------------------------------------------------------- */
 
@@ -753,7 +693,10 @@ static const char pcb_acts_PasteBuffer[] =
 	"PasteBuffer(Convert|Restore|Mirror)\n"
 	"PasteBuffer(ToLayout, X, Y, units)\n"
 	"PasteBuffer(ToLayout, crosshair)\n"
-	"PasteBuffer(Save, Filename, [format], [force])";
+	"PasteBuffer(Save, Filename, [format], [force])\n"
+	"PasteBuffer(Push)\n"
+	"PasteBuffer(Pop)\n"
+	;
 
 static const char pcb_acth_PasteBuffer[] = "Various operations on the paste buffer.";
 
@@ -773,11 +716,10 @@ Copies the selected objects to the current paste buffer.
 Remove all objects from the current paste buffer.
 
 @item Convert
-Convert the current paste buffer to an element.  Vias are converted to
-pins, lines are converted to pads.
+Convert the current paste buffer to an subcircuit.
 
 @item Restore
-Convert any elements in the paste buffer back to vias and lines.
+Convert any subcircuit in the paste buffer back to plain objects.
 
 @item Mirror
 Flip all objects in the paste buffer vertically (up/down flip).  To mirror
@@ -788,13 +730,18 @@ Rotates the current buffer.  The number to pass is 1..3, where 1 means
 90 degrees counter clockwise, 2 means 180 degrees, and 3 means 90
 degrees clockwise (270 CCW).
 
+@item Normalize
+Set the buffer origin to the center of the paste buffer bounding box.
+This is useful especially if an import plugin loaded objects in the buffer
+with a large offset.
+
 @item Save
 Saves any elements in the current buffer to the indicated file. If
 format is specified, try to use that file format, else use the default.
 If force is specified, overwrite target, don't ask.
 
 @item ToLayout
-Pastes any elements in the current buffer to the indicated X, Y
+Pastes objects in the current buffer to the indicated X, Y
 coordinates in the layout.  The @code{X} and @code{Y} are treated like
 @code{delta} is for many other objects.  For each, if it's prefixed by
 @code{+} or @code{-}, then that amount is relative to the last
@@ -820,6 +767,8 @@ static int pcb_act_PasteBuffer(int argc, const char **argv, pcb_coord_t x, pcb_c
 	static char *default_file = NULL;
 	pcb_bool free_name = pcb_false;
 	int force = (forces != NULL) && ((*forces == '1') || (*forces == 'y') || (*forces == 'Y'));
+	static int stack[32];
+	static int sp = 0;
 
 	pcb_notify_crosshair_change(pcb_false);
 	if (function) {
@@ -829,30 +778,49 @@ static int pcb_act_PasteBuffer(int argc, const char **argv, pcb_coord_t x, pcb_c
 			pcb_buffer_clear(PCB, PCB_PASTEBUFFER);
 			break;
 
+		case F_Push:
+			if (sp < sizeof(stack) / sizeof(stack[0]))
+				stack[sp++] = conf_core.editor.buffer_number;
+			else
+				pcb_message(PCB_MSG_ERROR, "Paste buffer stack overflow on push.\n");
+			break;
+
+		case F_Pop:
+			if (sp > 0)
+				pcb_buffer_set_number(stack[--sp]);
+			else
+				pcb_message(PCB_MSG_ERROR, "Paste buffer stack underflow on pop.\n");
+			break;
+
 			/* copies objects to paste buffer */
 		case F_AddSelected:
 			pcb_buffer_add_selected(PCB, PCB_PASTEBUFFER, 0, 0, pcb_false);
+			if (pcb_data_is_empty(PCB_PASTEBUFFER->Data)) {
+				pcb_message(PCB_MSG_WARNING, "Nothing buffer-movable is selected, nothing copied to the paste buffer\n");
+				goto error;
+			}
 			break;
 
-			/* moves objects to paste buffer */
+			/* moves objects to paste buffer: kept for compatibility with old menu files */
 		case F_MoveSelected:
-			pcb_buffer_move_selected(PCB, PCB_PASTEBUFFER, 0, 0, pcb_false);
+			pcb_buffer_add_selected(PCB, PCB_PASTEBUFFER, 0, 0, pcb_false);
+			if (pcb_data_is_empty(PCB_PASTEBUFFER->Data)) {
+				pcb_message(PCB_MSG_WARNING, "Nothing buffer-movable is selected, nothing moved to the paste buffer\n");
+				goto error;
+			}
+			pcb_hid_actionl("RemoveSelected", NULL);
 			break;
 
-			/* converts buffer contents into an element */
+			/* converts buffer contents into a subcircuit */
 		case F_Convert:
-			pcb_element_convert_from_buffer(PCB_PASTEBUFFER);
-			break;
-
-			/* converts buffer contents into an element */
 		case F_ConvertSubc:
 			pcb_subc_convert_from_buffer(PCB_PASTEBUFFER);
 			break;
 
-			/* break up element for editing */
+			/* break up subcircuit for editing */
 		case F_Restore:
-			if (!pcb_element_smash_buffer(PCB_PASTEBUFFER) && !pcb_subc_smash_buffer(PCB_PASTEBUFFER))
-				pcb_message(PCB_MSG_ERROR, _("Error!  Buffer doesn't contain a single element or subcircuit\n"));
+			if (!pcb_subc_smash_buffer(PCB_PASTEBUFFER))
+				pcb_message(PCB_MSG_ERROR, _("Error!  Buffer doesn't contain a single subcircuit\n"));
 			break;
 
 			/* Mirror buffer */
@@ -923,6 +891,7 @@ static int pcb_act_PasteBuffer(int argc, const char **argv, pcb_coord_t x, pcb_c
 					if (!absolute)
 						y += oldy;
 				}
+
 				else {
 					pcb_notify_crosshair_change(pcb_true);
 					PCB_ACT_FAIL(PasteBuffer);
@@ -935,6 +904,13 @@ static int pcb_act_PasteBuffer(int argc, const char **argv, pcb_coord_t x, pcb_c
 			}
 			break;
 
+		case F_Normalize:
+			pcb_set_buffer_bbox(PCB_PASTEBUFFER);
+			PCB_PASTEBUFFER->X = pcb_round(((double)PCB_PASTEBUFFER->BoundingBox.X1 + (double)PCB_PASTEBUFFER->BoundingBox.X2) / 2.0);
+			PCB_PASTEBUFFER->Y = pcb_round(((double)PCB_PASTEBUFFER->BoundingBox.Y1 + (double)PCB_PASTEBUFFER->BoundingBox.Y2) / 2.0);
+			pcb_crosshair_range_to_buffer();
+			break;
+
 			/* set number */
 		default:
 			{
@@ -949,6 +925,10 @@ static int pcb_act_PasteBuffer(int argc, const char **argv, pcb_coord_t x, pcb_c
 
 	pcb_notify_crosshair_change(pcb_true);
 	return 0;
+
+	error:;
+	pcb_notify_crosshair_change(pcb_true);
+	return 1;
 }
 
 /* --------------------------------------------------------------------------- */
