@@ -43,16 +43,28 @@
 #include "safe_fs.h"
 #include "build_run.h"
 
+#define CONF_USER_DIR "~/" DOT_PCB_RND
+
 /* conf list node's name */
 const char *conf_list_name = "pcb-rnd-conf-v1";
-static const char *conf_user_fn = "~/" DOT_PCB_RND "/pcb-conf.lht";
+static const char *conf_user_fn = CONF_USER_DIR "/pcb-conf.lht";
 static const char *flcat = "conf";
 
 
-lht_doc_t *conf_root[CFR_max_alloc];
-int conf_root_lock[CFR_max_alloc];
+/* The main conf: monolithic config files affecting all parts of the conf tree;
+   By default every operation is done on these trees. */
+lht_doc_t *conf_main_root[CFR_max_alloc];
+int conf_main_root_lock[CFR_max_alloc];
 int conf_lht_dirty[CFR_max_alloc];
 
+/* Plugin config: only plugin configuration is accepted; never edited, only
+   merged in. Merge takes two steps: first all files per role are merged into
+   a single conf_plug_root[R] (lihata level merge), then conf_plug_root[R]
+   is merged in using the normal conf merge mechanism. Plug roots are
+   merged before main roots so main root overwrite are stronger. */
+lht_doc_t *conf_plug_root[CFR_max_alloc];
+
+
 htsp_t *conf_fields = NULL;
 const int conf_default_prio[] = {
 /*	CFR_INTERNAL */   100,
@@ -68,17 +80,6 @@ const int conf_default_prio[] = {
 
 extern const char *conf_internal;
 
-/*static lht_doc_t *conf_plugin;*/
-
-static int conf_ignore_old_paths(const char *path)
-{
-	static const char *ignores[] = { "editor/show_mask", "editor/show_paste", NULL };
-	const char **i;
-	for(i = ignores; *i != NULL; i++)
-		if (strcmp(path, *i) == 0)
-			return 1;
-	return 0;
-}
 
 static lht_node_t *conf_lht_get_confroot(lht_node_t *cwd)
 {
@@ -103,29 +104,63 @@ int conf_insert_tree_as(conf_role_t role, lht_node_t *root)
 	if ((root->type != LHT_LIST) || (strcmp(root->name, "pcb-rnd-conf-v1") != 0))
 		return -1;
 
-	if (conf_root[role] != NULL) {
-		lht_dom_uninit(conf_root[role]);
-		conf_root[role] = NULL;
+	if (conf_main_root[role] != NULL) {
+		lht_dom_uninit(conf_main_root[role]);
+		conf_main_root[role] = NULL;
 	}
 
 	d = lht_dom_init();
 	d->root = lht_dom_node_alloc(LHT_LIST, "pcb-rnd-conf-v1");
 	d->root->doc = d;
 	lht_tree_merge(d->root, root);
-	conf_root[role] = d;
+	conf_main_root[role] = d;
 	return 0;
 }
 
+static lht_doc_t *conf_load_plug_file(const char *fn, int fn_is_text)
+{
+	lht_doc_t *d;
+	const char *ifn;
+
+	if (fn_is_text) {
+		d = pcb_hid_cfg_load_str(fn);
+		ifn = "<string>";
+	}
+	else {
+		d = pcb_hid_cfg_load_lht(fn);
+		ifn = fn;
+	}
+
+	if (d == NULL) {
+		pcb_message(PCB_MSG_ERROR, "error: failed to load lht plugin config: %s (can't open the file or syntax error)\n", ifn);
+		return NULL;
+	}
+
+	if (d->root == NULL) {
+		pcb_message(PCB_MSG_ERROR, "error: failed to load lht plugin config: %s (no root)\n", ifn);
+		lht_dom_uninit(d);
+		return NULL;
+	}
+
+	if ((d->root->type != LHT_LIST) || (strcmp(d->root->name, "pcb-rnd-conf-v1") != 0)) {
+		pcb_message(PCB_MSG_ERROR, "error: failed to load lht plugin config: %s (not a pcb-rnd-conf-v1)\n", ifn);
+		lht_dom_uninit(d);
+		return NULL;
+	}
+
+	return d;
+}
+
 int conf_load_as(conf_role_t role, const char *fn, int fn_is_text)
 {
 	lht_doc_t *d;
 	const char *ifn, *role_name = conf_role_name(role);
 
-	if (conf_root_lock[role])
+	if (conf_main_root_lock[role])
 		return -1;
-	if (conf_root[role] != NULL) {
-		lht_dom_uninit(conf_root[role]);
-		conf_root[role] = NULL;
+	if (conf_main_root[role] != NULL) {
+		lht_dom_uninit(conf_main_root[role]);
+		conf_main_root[role] = NULL;
 		if (role_name != NULL)
 			pcb_file_loaded_del_at(flcat, role_name);
 	}
@@ -159,14 +194,14 @@ int conf_load_as(conf_role_t role, const char *fn, int fn_is_text)
 		prjroot->doc = d;
 		confroot->doc = d;
 		d->root = prjroot;
-		conf_root[role] = d;
+		conf_main_root[role] = d;
 		if (role_name != NULL)
 			pcb_file_loaded_set_at(flcat, role_name, ifn, "project/conf");
 		return 0;
 	}
 
 	if ((d->root->type == LHT_LIST) && (strcmp(d->root->name, "pcb-rnd-conf-v1") == 0)) {
-		conf_root[role] = d;
+		conf_main_root[role] = d;
 		if (role_name != NULL)
 			pcb_file_loaded_set_at(flcat, role_name, ifn, "conf");
 		return 0;
@@ -180,14 +215,14 @@ int conf_load_as(conf_role_t role, const char *fn, int fn_is_text)
 			pcb_file_loaded_set_at(flcat, role_name, ifn, "project/conf");
 
 		if ((confroot != NULL)  && (confroot->type == LHT_LIST) && (strcmp(confroot->name, "li:pcb-rnd-conf-v1") == 0)) {
-			conf_root[role] = d;
+			conf_main_root[role] = d;
 			return 0;
 		}
 
 		/* project file with no config root */
 		confroot = lht_dom_node_alloc(LHT_LIST, "pcb-rnd-conf-v1");
 		lht_dom_hash_put(d->root, confroot);
-		conf_root[role] = d;
+		conf_main_root[role] = d;
 		return 0;
 	}
 
@@ -198,6 +233,86 @@ int conf_load_as(conf_role_t role, const char *fn, int fn_is_text)
 	return -1;
 }
 
+#include "conf_regfile.c"
+
+static int conf_merge_plug(lht_doc_t *d, conf_role_t role, const char *path)
+{
+	lht_err_t err;
+
+	if (conf_plug_root[role] == NULL) {
+		conf_plug_root[role] = lht_dom_init();
+		conf_plug_root[role]->root = lht_dom_node_alloc(LHT_LIST, "pcb-rnd-conf-v1");
+		conf_plug_root[role]->root->doc = conf_plug_root[role];
+	}
+	err = lht_tree_merge(conf_plug_root[role]->root, d->root);
+	lht_dom_uninit(d);
+	if (err != 0) {
+		pcb_message(PCB_MSG_ERROR, "Failed to lihata-merge plugin config %s: %s\n", path, lht_err_str(err));
+		return 0;
+	}
+	return 1;
+}
+
+/* Load plugin config files */
+static int conf_load_plug_files(conf_role_t role, const char *dir)
+{
+	char path[PCB_PATH_MAX], *fn;
+	int dlen, cnt = 0;
+	htsi_entry_t *e;
+
+	if (conf_plug_root[role] != NULL) {
+		lht_dom_uninit(conf_plug_root[role]);
+		conf_plug_root[role] = NULL;
+	}
+
+	if (!conf_files_inited) return 0;
+
+	dlen = strlen(dir);
+	memcpy(path, dir, dlen);
+	path[dlen] = '/';
+	fn = path+dlen+1;
+
+	for (e = htsi_first(&conf_files); e; e = htsi_next(&conf_files, e)) {
+		strcpy(fn, e->key);
+		if (pcb_file_readable(path)) {
+			lht_doc_t *d = conf_load_plug_file(path, 0);
+			if (d != NULL) {
+				int res = conf_merge_plug(d, role, path);
+				if (res) {
+					char id[256];
+					pcb_snprintf(id, sizeof(id), "%s:%s", conf_role_name(role), fn);
+					pcb_file_loaded_set_at("conf/plugin", id, path, NULL);
+				}
+				cnt += res;
+			}
+		}
+	}
+
+	return cnt;
+}
+
+static int conf_load_plug_interns(conf_role_t role)
+{
+	int cnt = 0;
+	htsi_entry_t *e;
+
+	if (conf_plug_root[role] != NULL) {
+		lht_dom_uninit(conf_plug_root[role]);
+		conf_plug_root[role] = NULL;
+	}
+
+	if (!conf_files_inited) return 0;
+	for (e = htsi_first(&conf_interns); e; e = htsi_next(&conf_interns, e)) {
+		lht_doc_t *d = conf_load_plug_file(e->key, 1);
+		if (d != NULL) {
+			int res = conf_merge_plug(d, role, NULL);
+			cnt += res;
+		}
+	}
+
+	return cnt;
+}
+
 conf_policy_t conf_policy_parse(const char *s)
 {
 	if (pcb_strcasecmp(s, "overwrite") == 0)  return  POL_OVERWRITE;
@@ -306,45 +421,6 @@ int conf_get_policy_prio(lht_node_t *node, conf_policy_t *gpolicy, long *gprio)
 	}
 }
 
-static int conf_parse_increments(pcb_increments_t *inc, lht_node_t *node)
-{
-	lht_node_t *val;
-
-	if (node->type != LHT_HASH) {
-		pcb_hid_cfg_error(node, "Increments need to be a hash\n");
-		return -1;
-	}
-
-#define incload(field) \
-	val = lht_dom_hash_get(node, #field); \
-	if (val != NULL) {\
-		if (val->type == LHT_TEXT) {\
-			pcb_bool succ; \
-			inc->field = pcb_get_value(val->data.text.value, NULL, NULL, &succ); \
-			if (!succ) \
-				pcb_hid_cfg_error(node, "invalid numeric value in increment field " #field ": %s\n", val->data.text.value); \
-		} \
-		else\
-			pcb_hid_cfg_error(node, "increment field " #field " needs to be a text node\n"); \
-	}
-
-	incload(grid);
-	incload(grid_min);
-	incload(grid_max);
-	incload(size);
-	incload(size_min);
-	incload(size_max);
-	incload(line);
-	incload(line_min);
-	incload(line_max);
-	incload(clear);
-	incload(clear_min);
-	incload(clear_max);
-
-#undef incload
-	return 0;
-}
-
 const char *conf_get_user_conf_name()
 {
 	return conf_user_fn;
@@ -464,8 +540,6 @@ int conf_parse_text(confitem_t *dst, int idx, conf_native_type_t type, const cha
 			else
 				dst->unit[idx] = u;
 			break;
-		case CFN_INCREMENTS:
-			return conf_parse_increments(&(dst->increments[idx]), err_node);
 		case CFN_COLOR:
 			dst->color[idx] = text; /* let the HID check validity to support flexibility of the format */
 			break;
@@ -511,7 +585,6 @@ static void conf_insert_arr(conf_native_t *dest, int offs)
 			CASE_PCB_MOVE(CFN_UNIT, unit);
 			CASE_PCB_MOVE(CFN_COLOR, color);
 			CASE_PCB_MOVE(CFN_LIST, list);
-			CASE_PCB_MOVE(CFN_INCREMENTS, increments);
 		}
 	}
 #undef CASE_MOVE
@@ -560,7 +633,7 @@ int conf_merge_patch_array(conf_native_t *dest, lht_node_t *src_lst, int prio, c
 				break;
 			}
 
-			if (conf_parse_text(&dest->val, didx, dest->type, s->data.text.value, s) == 0) {
+			if ((s->data.text.value != NULL) && (conf_parse_text(&dest->val, didx, dest->type, s->data.text.value, s) == 0)) {
 				dest->prop[didx].prio = prio;
 				dest->prop[didx].src  = s;
 				if (didx >= dest->used)
@@ -568,7 +641,7 @@ int conf_merge_patch_array(conf_native_t *dest, lht_node_t *src_lst, int prio, c
 			}
 		}
 		else {
-			pcb_hid_cfg_error(s, "List item must be text\n");
+			pcb_hid_cfg_error(s, "List item (in an array) must be text\n");
 			res = -1;
 		}
 	}
@@ -578,65 +651,119 @@ int conf_merge_patch_array(conf_native_t *dest, lht_node_t *src_lst, int prio, c
 
 int conf_merge_patch_list(conf_native_t *dest, lht_node_t *src_lst, int prio, conf_policy_t pol)
 {
-	lht_node_t *s;
+	lht_node_t *s, *prev;
 	int res = 0;
 	conf_listitem_t *i;
 
-	if ((pol == POL_DISABLE) || (pol == POL_invalid))
-		return 0;
+	switch(pol) {
+		case POL_DISABLE:
+		case POL_invalid:
+			return 0;
 
-	if (pol == POL_OVERWRITE) {
-		/* overwrite the whole list: make it empty then append new elements */
-		while((i = conflist_first(dest->val.list)) != NULL)
-			conflist_remove(i);
-	}
-
-	for(s = src_lst->data.list.first; s != NULL; s = s->next) {
-		if (s->type == LHT_TEXT) {
-			i = calloc(sizeof(conf_listitem_t), 1);
-			i->val.string = &i->payload;
-			i->prop.prio = prio;
-			i->prop.src  = s;
-			if (conf_parse_text(&i->val, 0, CFN_STRING, s->data.text.value, s) != 0) {
-				free(i);
-				continue;
-			}
-
-			switch(pol) {
-				case POL_PREPEND:
+		case POL_PREPEND:
+			for(s = src_lst->data.list.last; s != NULL; s = prev) {
+				/* because lihata lists are single linked, need to look up the previous node manually; O(N^2) */
+				if (s != src_lst->data.list.first)
+					for(prev = src_lst->data.list.first; prev->next != s; prev = prev->next);
+				else
+					prev = NULL;
+				if (s->type == LHT_TEXT) {
+					i = calloc(sizeof(conf_listitem_t), 1);
+					i->name = s->name;
+					i->val.string = &i->payload;
+					i->prop.prio = prio;
+					i->prop.src  = s;
+					if (conf_parse_text(&i->val, 0, CFN_STRING, s->data.text.value, s) != 0) {
+						free(i);
+						continue;
+					}
 					conflist_insert(dest->val.list, i);
 					dest->used |= 1;
-					break;
-				case POL_APPEND:
-				case POL_OVERWRITE:
+				}
+				else {
+					pcb_hid_cfg_error(s, "List item (on a list, prepend) must be text\n");
+					res = -1;
+				}
+			}
+			break;
+		case POL_OVERWRITE:
+			/* overwrite the whole list: make it empty then append new elements */
+			while((i = conflist_first(dest->val.list)) != NULL)
+				conflist_remove(i);
+			/* fall through */
+		case POL_APPEND:
+			for(s = src_lst->data.list.first; s != NULL; s = s->next) {
+				if (s->type == LHT_TEXT) {
+					i = calloc(sizeof(conf_listitem_t), 1);
+					i->name = s->name;
+					i->val.string = &i->payload;
+					i->prop.prio = prio;
+					i->prop.src  = s;
+					if (conf_parse_text(&i->val, 0, CFN_STRING, s->data.text.value, s) != 0) {
+						free(i);
+						continue;
+					}
 					conflist_append(dest->val.list, i);
 					dest->used |= 1;
-					break;
-				case POL_DISABLE: case POL_invalid: return 0; /* compiler warning */
+				}
+				else {
+					pcb_hid_cfg_error(s, "List item (on a list) must be text\n");
+					res = -1;
+				}
 			}
-		}
-		else {
-			pcb_hid_cfg_error(s, "List item must be text\n");
-			res = -1;
-		}
+			break;
 	}
 	return res;
 }
 
 int conf_merge_patch_recurse(lht_node_t *sect, int default_prio, conf_policy_t default_policy, const char *path_prefix);
 
+typedef struct conf_ignore_s {
+	const char *name;
+	int len;
+	int warned;
+} conf_ignore_t;
+
+static conf_ignore_t conf_ignores[] = {
+	/* it is normal to have configuration for plugins and utils not loaded - ignore these */
+	{"plugins/", 8, 1},
+	{"utils/", 6, 1},
+
+	/* old config paths - warn once and move on */
+	{"editor/show_mask", 16, 0},
+	{"editor/show_paste", 17, 0},
+	{"editor/increments", 17, 0},
+	{"design/max_width", 16, 0},
+	{"design/max_height", 17, 0},
+
+	{NULL, 0, 0}
+};
+
+static void conf_warn_unknown_paths(const char *path, lht_node_t *n)
+{
+	conf_ignore_t *i;
+
+	for(i = conf_ignores; i->name != NULL; i++) {
+		if (strncmp(path, i->name, i->len) == 0) {
+			if (i->warned)
+				return; /* do not warn again */
+			i->warned = 1;
+			break;
+		}
+	}
+	pcb_hid_cfg_error(n, "conf error: lht->bin conversion: can't find path '%s'\n(it may be an obsolete setting, check your lht)\n", path);
+}
+
 int conf_merge_patch_item(const char *path, lht_node_t *n, int default_prio, conf_policy_t default_policy)
 {
 	conf_native_t *target = conf_get_field(path);
 	int res = 0;
 	switch(n->type) {
 		case LHT_TEXT:
-			if (target == NULL) {
-				if ((strncmp(path, "plugins/", 8) != 0) && (strncmp(path, "utils/", 6) != 0) && !conf_ignore_old_paths(path)) /* it is normal to have configuration for plugins and utils not loaded - ignore these */
-					pcb_hid_cfg_error(n, "conf error: lht->bin conversion: can't find path '%s' - check your lht!\n", path);
-				break;
-			}
-			conf_merge_patch_text(target, n, default_prio, default_policy);
+			if (target == NULL)
+				conf_warn_unknown_paths(path, n);
+			else
+				conf_merge_patch_text(target, n, default_prio, default_policy);
 			break;
 		case LHT_HASH:
 			if (target == NULL) /* no leaf: another level of structs */
@@ -645,10 +772,8 @@ int conf_merge_patch_item(const char *path, lht_node_t *n, int default_prio, con
 				conf_merge_patch_text(target, n, default_prio, default_policy);
 			break;
 		case LHT_LIST:
-			if (target == NULL) {
-				if ((strncmp(path, "plugins/", 8) != 0) && (strncmp(path, "utils/", 6) != 0))/* it is normal to have configuration for plugins and utils not loaded - ignore these */
-					pcb_hid_cfg_error(n, "conf error: lht->bin conversion: can't find path '%s' - check your lht; may it be that it should be a hash instead of a list?\n", path);
-			}
+			if (target == NULL)
+				conf_warn_unknown_paths(path, n);
 			else if (target->type == CFN_LIST)
 				res |= conf_merge_patch_list(target, n, default_prio, default_policy);
 			else if (target->array_size > 1)
@@ -677,12 +802,21 @@ int conf_merge_patch_recurse(lht_node_t *sect, int default_prio, conf_policy_t d
 	char *namee;
 	int nl, ppl = strlen(path_prefix), res = 0;
 
+	if (ppl >= sizeof(path)) {
+		pcb_message(PCB_MSG_ERROR, "Can not merge conf patch: name too long: '%s'\n", path_prefix);
+		return -1;
+	}
+
 	memcpy(path, path_prefix, ppl);
 	path[ppl] = '/';
 	pathe = path + ppl + 1;
 
 	for(n = lht_dom_first(&it, sect); n != NULL; n = lht_dom_next(&it)) {
 		nl = strlen(n->name);
+		if (nl >= (sizeof(path) - (pathe - path))) {
+			pcb_message(PCB_MSG_ERROR, "Can not merge conf patch: name too long: '%s'\n", n->name);
+			return -1;
+		}
 		memcpy(pathe, n->name, nl);
 		namee = pathe+nl;
 		*namee = '\0';
@@ -752,28 +886,36 @@ static int mst_prio_cmp(const void *a, const void *b)
 	return -1;
 }
 
+static void conf_merge_all_top(int role, const char *path, lht_node_t *cr)
+{
+	lht_node_t *r, *r2;
+	for(r = cr->data.list.first; r != NULL; r = r->next) {
+		if (path != NULL) {
+			r2 = lht_tree_path_(r->doc, r, path, 1, 0, NULL);
+			if (r2 != NULL)
+				add_subtree(role, r, r2);
+		}
+		else
+			add_subtree(role, r, r);
+	}
+}
+
 int conf_merge_all(const char *path)
 {
 	int n, ret = 0;
 	vmst_truncate(&merge_subtree, 0);
 
 	for(n = 0; n < CFR_max_real; n++) {
-		lht_node_t *cr, *r, *r2;
-		if (conf_root[n] == NULL)
-			continue;
-
-		cr = conf_lht_get_confroot(conf_root[n]->root);
-		if (cr == NULL)
-			continue;
-
-		for(r = cr->data.list.first; r != NULL; r = r->next) {
-			if (path != NULL) {
-				r2 = lht_tree_path_(r->doc, r, path, 1, 0, NULL);
-				if (r2 != NULL)
-					add_subtree(n, r, r2);
-			}
-			else
-				add_subtree(n, r, r);
+		lht_node_t *cr;
+		if (conf_main_root[n] != NULL) {
+			cr = conf_lht_get_confroot(conf_main_root[n]->root);
+			if (cr != NULL)
+				conf_merge_all_top(n, path, cr);
+		}
+		if (conf_plug_root[n] != NULL) {
+			cr = conf_lht_get_confroot(conf_plug_root[n]->root);
+			if (cr != NULL)
+				conf_merge_all_top(n, path, cr);
 		}
 	}
 
@@ -801,7 +943,6 @@ static void conf_field_clear(conf_native_t *f)
 			case CFN_COORD:       clr(coord); break;
 			case CFN_UNIT:        clr(unit); break;
 			case CFN_COLOR:       clr(color); break;
-			case CFN_INCREMENTS:  clr(increments); break;
 			case CFN_LIST:
 				while(conflist_first(f->val.list)) { /* need to free all items of a list before clearing the main struct */
 					conf_listitem_t *i = conflist_first(f->val.list);
@@ -847,19 +988,6 @@ void conf_update(const char *path, int arr_idx)
 				n = conf_get_field(path_);
 			}
 
-			/* It may be a field of an increment, truncate last portion */
-			if (n == NULL) {
-				field = strrchr(path_, '/');
-				if (field == NULL) {
-					free(path_);
-					return;
-				}
-				*field = '\0';
-				n = conf_get_field(path_);
-				if (n->type != CFN_INCREMENTS)
-					n = NULL;
-			}
-
 			/* if a valid node is found, update it */
 			if (n != NULL)
 				conf_update(path_, arr_idx);
@@ -930,9 +1058,9 @@ lht_node_t *conf_lht_get_first(conf_role_t target, int create)
 	assert(target != CFR_invalid);
 	assert(target >= 0);
 	assert(target < CFR_max_alloc);
-	if (conf_root[target] == NULL)
+	if (conf_main_root[target] == NULL)
 		return NULL;
-	return conf_lht_get_first_(conf_root[target]->root, POL_ANY, create);
+	return conf_lht_get_first_(conf_main_root[target]->root, POL_ANY, create);
 }
 
 lht_node_t *conf_lht_get_first_pol(conf_role_t target, conf_policy_t pol, int create)
@@ -940,9 +1068,9 @@ lht_node_t *conf_lht_get_first_pol(conf_role_t target, conf_policy_t pol, int cr
 	assert(target != CFR_invalid);
 	assert(target >= 0);
 	assert(target < CFR_max_alloc);
-	if (conf_root[target] == NULL)
+	if (conf_main_root[target] == NULL)
 		return NULL;
-	return conf_lht_get_first_(conf_root[target]->root, pol, create);
+	return conf_lht_get_first_(conf_main_root[target]->root, pol, create);
 }
 
 static lht_node_t *conf_lht_get_at_(conf_role_t target, const char *conf_path, const char *lht_path, int create)
@@ -953,7 +1081,7 @@ static lht_node_t *conf_lht_get_at_(conf_role_t target, const char *conf_path, c
 		return NULL;
 	r = lht_tree_path_(n->doc, n, lht_path, 1, 0, NULL);
 	if ((r == NULL) && (create)) {
-		conf_set_dry(target, conf_path, -1, "", POL_OVERWRITE);
+		conf_set_dry(target, conf_path, -1, "", POL_OVERWRITE, 0);
 		r = lht_tree_path_(n->doc, n, lht_path, 1, 0, NULL);
 	}
 	return r;
@@ -963,7 +1091,7 @@ lht_node_t *conf_lht_get_at(conf_role_t target, const char *path, int create)
 {
 	lht_node_t *r;
 	char *pc, *end;
-	if (conf_root[target] == NULL) {
+	if (conf_main_root[target] == NULL) {
 		if (!create)
 			return NULL;
 		conf_reset(target, "<conf_lht_get_at>");
@@ -1004,7 +1132,7 @@ void conf_load_all(const char *project_fn, const char *pcb_fn)
 		char buf[20];
 		if (dln == NULL) {
 			sprintf(buf, "signal%d", i + 1);
-			if (conf_set_dry(CFR_INTERNAL, "design/default_layer_name", i, buf, POL_OVERWRITE) != 0)
+			if (conf_set_dry(CFR_INTERNAL, "design/default_layer_name", i, buf, POL_OVERWRITE, 0) != 0)
 				printf("Can't set layer name\n");
 		}
 		else
@@ -1022,10 +1150,21 @@ void conf_load_all(const char *project_fn, const char *pcb_fn)
 	/* create the user config (in-memory-lht) if it does not exist on disk;
 	   this is needed so if the user makes config changes from the GUI things
 	   get saved. */
-	if (conf_root[CFR_USER] == NULL)
+	if (conf_main_root[CFR_USER] == NULL)
 		conf_reset(CFR_USER, conf_user_fn);
 }
 
+void conf_load_extra(const char *project_fn, const char *pcb_fn)
+{
+	int cnt;
+	cnt = conf_load_plug_files(CFR_SYSTEM, PCBSHAREDIR);
+	cnt += conf_load_plug_files(CFR_USER, CONF_USER_DIR);
+	cnt += conf_load_plug_interns(CFR_INTERNAL);
+	if (cnt > 0)
+		conf_merge_all(NULL);
+}
+
+
 void conf_load_project(const char *project_fn, const char *pcb_fn)
 {
 	const char *pc, *try;
@@ -1101,9 +1240,9 @@ conf_native_t *conf_get_field(const char *path)
 	return htsp_get(conf_fields, path);
 }
 
-int conf_set_dry(conf_role_t target, const char *path_, int arr_idx, const char *new_val, conf_policy_t pol)
+int conf_set_dry(conf_role_t target, const char *path_, int arr_idx, const char *new_val, conf_policy_t pol, int mkdirp)
 {
-	char *path, *basename, *next, *last, *sidx, *increment_field = NULL;
+	char *path, *basename, *next, *last, *sidx;
 	conf_native_t *nat;
 	lht_node_t *cwd, *nn;
 	lht_node_type_t ty;
@@ -1113,6 +1252,9 @@ int conf_set_dry(conf_role_t target, const char *path_, int arr_idx, const char
 	if ((new_val == NULL) && (pol != POL_OVERWRITE))
 		return -1;
 
+	if (mkdirp)
+		conf_lht_get_first_pol(target, pol, 1);
+
 	path = pcb_strdup(path_);
 	sidx = strrchr(path, '[');
 	if (sidx != NULL) {
@@ -1134,24 +1276,12 @@ int conf_set_dry(conf_role_t target, const char *path_, int arr_idx, const char
 
 	nat = conf_get_field(path);
 	if (nat == NULL) {
-		/* might be increments */
-		increment_field = strrchr(path, '/');
-		if (increment_field == NULL) {
-			free(path);
-			return -1;
-		}
-		*increment_field = '\0';
-		nat = conf_get_field(path);
-		if ((nat == NULL) || (nat->type != CFN_INCREMENTS)) {
-			free(path);
-			return -1;
-		}
-		*increment_field = '/';
-		increment_field++;
+		free(path);
+		return -1;
 	}
 
 
-	if (conf_root[target] == NULL) {
+	if (conf_main_root[target] == NULL) {
 		free(path);
 		return -1;
 	}
@@ -1191,13 +1321,13 @@ int conf_set_dry(conf_role_t target, const char *path_, int arr_idx, const char
 		if (next != NULL)
 			*next = '\0';
 
-		nn = lht_tree_path_(conf_root[target], cwd, last, 1, 0, NULL);
+		nn = lht_tree_path_(conf_main_root[target], cwd, last, 1, 0, NULL);
 		if (nn == NULL) {
 			if (new_val == NULL) {
 				free(path);
 				return 0;
 			}
-			if (conf_root_lock[target]) {
+			if (conf_main_root_lock[target]) {
 				pcb_message(PCB_MSG_WARNING, "WARNING: can't set config item %s because target in-memory lihata does not have the node and is tree-locked\n", path_);
 				free(path);
 				return -1;
@@ -1221,13 +1351,19 @@ int conf_set_dry(conf_role_t target, const char *path_, int arr_idx, const char
 	else
 		ty = LHT_TEXT;
 
-	nn = lht_tree_path_(conf_root[target], cwd, basename, 1, 0, NULL);
+	nn = lht_tree_path_(conf_main_root[target], cwd, basename, 1, 0, NULL);
 	if (nn == NULL) {
-		if (conf_root_lock[target]) {
+		if (conf_main_root_lock[target]) {
 			free(path);
 			return -1;
 		}
 		nn = lht_dom_node_alloc(ty, basename);
+		if (cwd->type != LHT_HASH) {
+			pcb_message(PCB_MSG_ERROR, "Expected HASH conf subtree '%s' (in path '%s'); cleaning up broken conf, check your config sources!\n", cwd->name, path);
+			lht_tree_del(cwd);
+			free(path);
+			return -1;
+		}
 		if (lht_dom_hash_put(cwd, nn) != LHTE_SUCCESS) {
 			lht_dom_node_free(nn);
 			free(path);
@@ -1314,7 +1450,7 @@ int conf_set_dry(conf_role_t target, const char *path_, int arr_idx, const char
 			free(cwd->data.text.value);
 
 		cwd->data.text.value = pcb_strdup(new_val);
-		cwd->file_name = conf_root[target]->active_file;
+		cwd->file_name = conf_main_root[target]->active_file;
 	}
 	else
 		lht_tree_del(cwd);
@@ -1328,7 +1464,7 @@ int conf_set_dry(conf_role_t target, const char *path_, int arr_idx, const char
 int conf_set(conf_role_t target, const char *path, int arr_idx, const char *new_val, conf_policy_t pol)
 {
 	int res;
-	res = conf_set_dry(target, path, arr_idx, new_val, pol);
+	res = conf_set_dry(target, path, arr_idx, new_val, pol, 1);
 	if (res < 0)
 		return res;
 	conf_update(path, arr_idx);
@@ -1338,7 +1474,7 @@ int conf_set(conf_role_t target, const char *path, int arr_idx, const char *new_
 int conf_del(conf_role_t target, const char *path, int arr_idx)
 {
 	int res;
-	res = conf_set_dry(target, path, arr_idx, NULL, POL_OVERWRITE);
+	res = conf_set_dry(target, path, arr_idx, NULL, POL_OVERWRITE, 0);
 	if (res < 0)
 		return res;
 	conf_update(path, arr_idx);
@@ -1434,7 +1570,7 @@ int conf_set_from_cli(const char *prefix, const char *arg_, const char *val, con
 	return ret;
 }
 
-void conf_parse_arguments(const char *prefix, int *argc, char ***argv)
+int conf_parse_arguments(const char *prefix, int *argc, char ***argv)
 {
 	int n, dst;
 
@@ -1458,6 +1594,7 @@ void conf_parse_arguments(const char *prefix, int *argc, char ***argv)
 			dst--;
 	}
 	*argc = dst;
+	return 0;
 }
 
 void conf_usage(const char *prefix, void (*print)(const char *name, const char *help))
@@ -1487,10 +1624,7 @@ void conf_usage(const char *prefix, void (*print)(const char *name, const char *
 					case CFN_COORD:   strcpy(s, " coord"); break;
 					case CFN_UNIT:    strcpy(s, " unit"); break;
 					case CFN_COLOR:   strcpy(s, " color"); break;
-					case CFN_LIST:
-					case CFN_INCREMENTS:
-						strcpy(s, " ???");
-						break;
+					case CFN_LIST:    strcpy(s, " <list>"); break;
 				}
 				print(name, n->description);
 				free(name);
@@ -1692,6 +1826,7 @@ conf_listitem_t *conf_list_first_str(conflist_t *list, const char **item_str, in
 conf_listitem_t *conf_list_next_str(conf_listitem_t *item_li, const char **item_str, int *idx)
 {
 	while((item_li = conflist_next(item_li)) != NULL) {
+		(*idx)++;
 		if (item_li->type != CFN_STRING)
 			continue;
 		/* found next string */
@@ -1699,6 +1834,7 @@ conf_listitem_t *conf_list_next_str(conf_listitem_t *item_li, const char **item_
 		return item_li;
 	}
 	/* found end of the list */
+	(*idx)++;
 	*item_str = NULL;
 	return item_li;
 }
@@ -1729,17 +1865,17 @@ const char *conf_concat_strlist(const conflist_t *lst, gds_t *buff, int *inited,
 
 void conf_lock(conf_role_t target)
 {
-	conf_root_lock[target] = 1;
+	conf_main_root_lock[target] = 1;
 }
 
 void conf_unlock(conf_role_t target)
 {
-	conf_root_lock[target] = 0;
+	conf_main_root_lock[target] = 0;
 }
 
 int conf_islocked(conf_role_t target)
 {
-	return conf_root_lock[target];
+	return conf_main_root_lock[target];
 }
 
 int conf_isdirty(conf_role_t target)
@@ -1755,7 +1891,7 @@ conf_role_t conf_lookup_role(const lht_node_t *nd)
 {
 	conf_role_t r;
 	for(r = 0; r < CFR_max_real; r++)
-		if (conf_root[r] == nd->doc)
+		if (conf_main_root[r] == nd->doc)
 			return r;
 
 	return CFR_invalid;
@@ -1765,15 +1901,15 @@ void conf_reset(conf_role_t target, const char *source_fn)
 {
 	lht_node_t *n;
 
-	if (conf_root[target] != NULL)
-		lht_dom_uninit(conf_root[target]);
+	if (conf_main_root[target] != NULL)
+		lht_dom_uninit(conf_main_root[target]);
 
-	conf_root[target] = lht_dom_init();
-	lht_dom_loc_newfile(conf_root[target], source_fn);
-	conf_root[target]->root = lht_dom_node_alloc(LHT_LIST, conf_list_name);
-	conf_root[target]->root->doc = conf_root[target];
+	conf_main_root[target] = lht_dom_init();
+	lht_dom_loc_newfile(conf_main_root[target], source_fn);
+	conf_main_root[target]->root = lht_dom_node_alloc(LHT_LIST, conf_list_name);
+	conf_main_root[target]->root->doc = conf_main_root[target];
 	n = lht_dom_node_alloc(LHT_HASH, "overwrite");
-	lht_dom_list_insert(conf_root[target]->root, n);
+	lht_dom_list_insert(conf_main_root[target]->root, n);
 }
 
 /*****************/
@@ -1812,16 +1948,6 @@ int conf_print_native_field(conf_pfn pfn, void *ctx, int verbose, confitem_t *va
 		case CFN_COORD:   ret += pfn(ctx, "%$mS", val->coord[idx]); break;
 		case CFN_UNIT:    print_str_or_null(pfn, ctx, verbose, val->unit[idx], val->unit[idx]->suffix); break;
 		case CFN_COLOR:   print_str_or_null(pfn, ctx, verbose, val->color[idx], val->color[idx]); break;
-		case CFN_INCREMENTS:
-			{
-				pcb_increments_t *i = &val->increments[idx];
-				ret += pfn(ctx, "{ grid=%$mS/%$mS/%$mS size=%$mS/%$mS/%$mS line=%$mS/%$mS/%$mS clear=%$mS/%$mS/%$mS}",
-				i->grid, i->grid_min, i->grid_max,
-				i->size, i->size_min, i->size_max,
-				i->line, i->line_min, i->line_max,
-				i->clear, i->clear_min, i->clear_max);
-			}
-			break;
 		case CFN_LIST:
 			{
 				conf_listitem_t *n;
@@ -1905,11 +2031,19 @@ void conf_uninit(void)
 	int n;
 	htsp_entry_t *e;
 
+	conf_fields_foreach(e) {
+		if (strncmp(e->key, "plugins/", 8) == 0)
+			fprintf(stderr, "pcb-rnd conf ERROR: conf node '%s' is not unregistered\n", e->key);
+	}
+
 	conf_pcb_hid_uninit();
 
-	for(n = 0; n < CFR_max_alloc; n++)
-		if (conf_root[n] != NULL)
-			lht_dom_uninit(conf_root[n]);
+	for(n = 0; n < CFR_max_alloc; n++) {
+		if (conf_main_root[n] != NULL)
+			lht_dom_uninit(conf_main_root[n]);
+		if (conf_plug_root[n] != NULL)
+			lht_dom_uninit(conf_plug_root[n]);
+	}
 
 	conf_fields_foreach(e) {
 		conf_free_native(e->value);
