@@ -54,15 +54,14 @@ typedef enum { /* bitfield */
 } conf_flag_t;
 
 
-typedef const char * CFT_STRING;
-typedef int          CFT_BOOLEAN;
-typedef long         CFT_INTEGER;
-typedef double       CFT_REAL;
-typedef pcb_coord_t        CFT_COORD;
-typedef pcb_unit_t *       CFT_UNIT;
-typedef char *       CFT_COLOR;
-typedef conflist_t   CFT_LIST;
-typedef pcb_increments_t   CFT_INCREMENTS;
+typedef const char *  CFT_STRING;
+typedef int           CFT_BOOLEAN;
+typedef long          CFT_INTEGER;
+typedef double        CFT_REAL;
+typedef pcb_coord_t   CFT_COORD;
+typedef pcb_unit_t *  CFT_UNIT;
+typedef char *        CFT_COLOR;
+typedef conflist_t    CFT_LIST;
 
 typedef enum {
 	CFN_STRING,
@@ -72,8 +71,7 @@ typedef enum {
 	CFN_COORD,
 	CFN_UNIT,
 	CFN_COLOR,
-	CFN_LIST,
-	CFN_INCREMENTS
+	CFN_LIST
 } conf_native_type_t;
 
 union confitem_u {
@@ -85,7 +83,6 @@ union confitem_u {
 	const pcb_unit_t **unit;
 	const char **color;
 	conflist_t *list;
-	pcb_increments_t *increments;
 	void *any;
 };
 
@@ -122,6 +119,7 @@ struct conf_listitem_s {
 	conf_native_type_t type;
 	confitem_t val;   /* value is always an array (len 1 for the common case)  */
 	confprop_t prop; /* an array of properties allocated as big as val's array */
+	const char *name;
 	const char *payload;
 	gdl_elem_t link;
 };
@@ -166,6 +164,8 @@ int conf_insert_tree_as(conf_role_t role, lht_node_t *root);
    found. Runs conf_update(NULL); */
 void conf_load_project(const char *project_fn, const char *pcb_fn);
 
+void conf_load_extra(const char *project_fn, const char *pcb_fn);
+
 /* Update the binary representation from the memory-lht representation */
 void conf_update(const char *path, int arr_idx);
 
@@ -185,8 +185,9 @@ int conf_set(conf_role_t target, const char *path, int arr_idx, const char *new_
 int conf_del(conf_role_t target, const char *path, int arr_idx);
 
 /* Same as conf_set, but without updating the binary - useful for multiple
-   conf_set_dry calls and a single all-tree conf_udpate(NULL) for transactions. */
-int conf_set_dry(conf_role_t target, const char *path_, int arr_idx, const char *new_val, conf_policy_t pol);
+   conf_set_dry calls and a single all-tree conf_udpate(NULL) for transactions.
+   If mkdirp is non-zero, automatically create the policy subtree if it doesn't exist. */
+int conf_set_dry(conf_role_t target, const char *path_, int arr_idx, const char *new_val, conf_policy_t pol, int mkdirp);
 
 /* Same as conf_set, but doesn't look up where to set things: change the value of
    the lihata node backing the native field */
@@ -201,7 +202,7 @@ int conf_set_native(conf_native_t *field, int arr_idx, const char *new_val);
 int conf_set_from_cli(const char *prefix, const char *arg_, const char *val, const char **why);
 
 /* Attempt to consume argv[] using conf_set_from_cli */
-void conf_parse_arguments(const char *prefix, int *argc, char ***argv);
+int conf_parse_arguments(const char *prefix, int *argc, char ***argv);
 
 #define conf_reg_field_array(globvar, field, type_name, path, desc, flags) \
 	conf_reg_field_((void *)&globvar.field, (sizeof(globvar.field) / sizeof(globvar.field[0])), type_name, path, desc, flags)
@@ -332,7 +333,7 @@ conf_listitem_t *conf_list_next_str(conf_listitem_t *item_li, const char **item_
 
 /*conf_listitem_t *item;*/
 #define conf_loop_list(list, item, idx) \
-	for (idx = 0, item = conflist_first((conflist_t *)cl); item != NULL; item = conflist_next(item), idx++)
+	for (idx = 0, item = conflist_first((conflist_t *)list); item != NULL; item = conflist_next(item), idx++)
 
 /*conf_listitem_t *item; const char *item_str; */
 #define conf_loop_list_str(list, item_li, item_str, idx) \
@@ -364,7 +365,7 @@ int conf_get_policy_prio(lht_node_t *node, conf_policy_t *gpolicy, long *gprio);
    fields depending on type */
 int conf_parse_text(confitem_t *dst, int idx, conf_native_type_t type, const char *text, lht_node_t *err_node);
 
-/** Returns the user configuration file name */
+/* Returns the user configuration file name */
 const char *conf_get_user_conf_name();
 
 /* Determine the file name of the project file - project_fn and pcb_fn can be NULL */
@@ -380,4 +381,11 @@ int pcb_conf_cmd_is_safe_(const char *path, const char *value, const char **val_
    subtree if needed */
 lht_node_t *conf_lht_get_first_pol(conf_role_t target, conf_policy_t pol, int create);
 
+/* (un)register a custom config file name (not path, just file name);
+   if intern is not NULL, it is the internal (executable-embedded)
+   version; it's not strdup'd, the caller needs to keep the string available
+   until conf_unreg_file(). path is strdup'd */
+void conf_reg_file(const char *path, const char *intern);
+void conf_unreg_file(const char *path, const char *intern);
+
 #endif
