@@ -82,8 +82,6 @@ typedef struct {								/* holds cursor information */
 	enum pcb_crosshair_shape_e shape;			/* shape of crosshair */
 	vtop_t onpoint_objs;
 	vtop_t old_onpoint_objs;
-	pcb_pad_t *snapped_pad;
-	pcb_pin_t *snapped_pin;
 	pcb_pstk_t *snapped_pstk;
 
 
@@ -104,8 +102,6 @@ typedef struct {								/* holds cursor information */
 extern pcb_crosshair_t pcb_crosshair;
 extern pcb_mark_t pcb_marked;
 
-
-pcb_coord_t pcb_grid_fit(pcb_coord_t x, pcb_coord_t grid_spacing, pcb_coord_t grid_offset);
 void pcb_notify_crosshair_change(pcb_bool changes_complete);
 void pcb_notify_mark_change(pcb_bool changes_complete);
 void pcb_draw_attached(void);
@@ -118,25 +114,19 @@ void pcb_crosshair_uninit(void);
 void pcb_crosshair_grid_fit(pcb_coord_t, pcb_coord_t);
 void pcb_center_display(pcb_coord_t X, pcb_coord_t Y);
 
-/*** cursor/crosshair mode ***/
-
 /* sets the crosshair range to the current buffer extents */
 void pcb_crosshair_range_to_buffer(void);
-void pcb_crosshair_save_mode(void);
-void pcb_crosshair_restore_mode(void);
-void pcb_crosshair_set_mode(int Mode);
 void pcb_crosshair_set_local_ref(pcb_coord_t X, pcb_coord_t Y, pcb_bool Showing);
 
 
 /*** utility for plugins ***/
-void XORDrawAttachedLine(pcb_coord_t, pcb_coord_t, pcb_coord_t, pcb_coord_t, pcb_coord_t);
-void XORPolygon(pcb_poly_t *polygon, pcb_coord_t dx, pcb_coord_t dy, int dash_last);
-void XORPolygon_subc(pcb_poly_t *polygon, pcb_coord_t dx, pcb_coord_t dy, pcb_coord_t w, pcb_coord_t h, int mirr);
-void XORDrawPinViaDRCOutline(pcb_pin_t * pv,pcb_coord_t clearance);
-void XORDrawAttachedArc(pcb_coord_t thick);
-void XORDrawBuffer(pcb_buffer_t *Buffer);
-void XORDrawMoveOrCopy(void);
-void XORDrawInsertPointObject(void);
+void pcb_xordraw_attached_line(pcb_coord_t, pcb_coord_t, pcb_coord_t, pcb_coord_t, pcb_coord_t);
+void pcb_xordraw_poly(pcb_poly_t *polygon, pcb_coord_t dx, pcb_coord_t dy, int dash_last);
+void pcb_xordraw_poly_subc(pcb_poly_t *polygon, pcb_coord_t dx, pcb_coord_t dy, pcb_coord_t w, pcb_coord_t h, int mirr);
+void pcb_xordraw_attached_arc(pcb_coord_t thick);
+void pcb_xordraw_buffer(pcb_buffer_t *Buffer);
+void pcb_xordraw_movecopy(void);
+void pcb_xordraw_insert_pt_obj(void);
 
 
 
