@@ -30,20 +30,24 @@
  */
 #include "config.h"
 
+#include <time.h>
+
 #include "board.h"
 #include "data.h"
 #include "data_list.h"
 #include "data_it.h"
 #include "rtree.h"
 #include "list_common.h"
-#include "obj_all.h"
 #include "layer_it.h"
 #include "operation.h"
 #include "flag.h"
+#include "obj_arc.h"
+#include "obj_line.h"
+#include "obj_poly.h"
+#include "obj_text.h"
+#include "obj_pstk.h"
+#include "obj_subc.h"
 
-/* ---------------------------------------------------------------------------
- * some shared identifiers
- */
 int pcb_layer_stack[PCB_MAX_LAYER];			/* determines the layer draw order */
 
 pcb_buffer_t pcb_buffers[PCB_MAX_BUFFER]; /* my buffers */
@@ -124,75 +128,6 @@ void pcb_loop_subc(pcb_board_t *pcb, void *ctx, pcb_subc_cb_t scb)
 	}
 }
 
-
-void pcb_loop_elements(pcb_board_t *pcb, void *ctx, pcb_element_cb_t ecb, pcb_eline_cb_t elcb, pcb_earc_cb_t eacb, pcb_etext_cb_t etcb, pcb_epin_cb_t epicb, pcb_epad_cb_t epacb)
-{
-	if ((ecb != NULL) || (elcb != NULL) || (eacb != NULL)  || (etcb != NULL) || (epicb != NULL) || (epacb != NULL)) {
-		PCB_ELEMENT_LOOP(PCB_REAL_DATA(pcb));
-		{
-			if (ecb != NULL)
-				if (ecb(ctx, pcb, element, 1))
-					continue;
-
-			if (elcb != NULL) {
-				PCB_ELEMENT_PCB_LINE_LOOP(element);
-				{
-					elcb(ctx, pcb, element, line);
-				}
-				PCB_END_LOOP;
-			}
-
-			if (eacb != NULL) {
-				PCB_ELEMENT_ARC_LOOP(element);
-				{
-					eacb(ctx, pcb, element, arc);
-				}
-				PCB_END_LOOP;
-			}
-
-			if (etcb != NULL) {
-				PCB_ELEMENT_PCB_TEXT_LOOP(element);
-				{
-					etcb(ctx, pcb, element, text);
-				}
-				PCB_END_LOOP;
-			}
-
-			if (epicb != NULL) {
-				PCB_PIN_LOOP(element);
-				{
-					epicb(ctx, pcb, element, pin);
-				}
-				PCB_END_LOOP;
-			}
-
-
-			if (epacb != NULL) {
-				PCB_PAD_LOOP(element);
-				{
-					epacb(ctx, pcb, element, pad);
-				}
-				PCB_END_LOOP;
-			}
-
-			if (ecb != NULL)
-				ecb(ctx, pcb, element, 0);
-		}
-		PCB_END_LOOP;
-	}
-}
-
-void pcb_loop_vias(pcb_board_t *pcb, void *ctx, pcb_via_cb_t vcb)
-{
-	if (vcb != NULL) {
-		PCB_VIA_LOOP(PCB_REAL_DATA(pcb));
-		{
-			vcb(ctx, pcb, via);
-		}
-		PCB_END_LOOP;
-	}
-}
-
 void pcb_loop_pstk(pcb_board_t *pcb, void *ctx, pcb_pstk_cb_t pscb)
 {
 	if (pscb != NULL) {
@@ -206,21 +141,15 @@ void pcb_loop_pstk(pcb_board_t *pcb, void *ctx, pcb_pstk_cb_t pscb)
 
 void pcb_loop_all(pcb_board_t *pcb, void *ctx,
 	pcb_layer_cb_t lacb, pcb_line_cb_t lcb, pcb_arc_cb_t acb, pcb_text_cb_t tcb, pcb_poly_cb_t pocb,
-	pcb_element_cb_t ecb, pcb_eline_cb_t elcb, pcb_earc_cb_t eacb, pcb_etext_cb_t etcb, pcb_epin_cb_t epicb, pcb_epad_cb_t epacb,
 	pcb_subc_cb_t scb,
-	pcb_via_cb_t vcb, pcb_pstk_cb_t pscb
+	pcb_pstk_cb_t pscb
 	)
 {
 	pcb_loop_layers(pcb, ctx, lacb, lcb, acb, tcb, pocb);
-	pcb_loop_elements(pcb, ctx, ecb, elcb, eacb, etcb, epicb, epacb);
 	pcb_loop_subc(pcb, ctx, scb);
-	pcb_loop_vias(pcb, ctx, vcb);
 	pcb_loop_pstk(pcb, ctx, pscb);
 }
 
-/* ---------------------------------------------------------------------------
- * free memory used by data struct
- */
 void pcb_data_free(pcb_data_t * data)
 {
 	pcb_layer_t *layer;
@@ -231,45 +160,20 @@ void pcb_data_free(pcb_data_t * data)
 
 	subc = (data->parent_type == PCB_PARENT_SUBC);
 
-
-	PCB_VIA_LOOP(data);
-	{
-		free(via->Name);
-	}
-	PCB_END_LOOP;
-	list_map0(&data->Via, pcb_pin_t, pcb_via_free);
-	PCB_ELEMENT_LOOP(data);
-	{
-		pcb_element_destroy(element);
-	}
-	PCB_END_LOOP;
-
 	PCB_SUBC_LOOP(data);
 	{
 		pcb_subc_free(subc);
 	}
 	PCB_END_LOOP;
 
-	list_map0(&data->Element, pcb_element_t, pcb_element_free);
 	list_map0(&data->Rat, pcb_rat_t, pcb_rat_free);
 
 	for (layer = data->Layer, i = 0; i < data->LayerN; layer++, i++)
 		pcb_layer_free(layer);
 
 	if (!subc) {
-		if (data->element_tree)
-			pcb_r_destroy_tree(&data->element_tree);
 		if (data->subc_tree)
 			pcb_r_destroy_tree(&data->subc_tree);
-		for (i = 0; i < PCB_MAX_ELEMENTNAMES; i++)
-			if (data->name_tree[i])
-				pcb_r_destroy_tree(&data->name_tree[i]);
-		if (data->via_tree)
-			pcb_r_destroy_tree(&data->via_tree);
-		if (data->pin_tree)
-			pcb_r_destroy_tree(&data->pin_tree);
-		if (data->pad_tree)
-			pcb_r_destroy_tree(&data->pad_tree);
 		if (data->padstack_tree)
 			pcb_r_destroy_tree(&data->padstack_tree);
 		if (data->rat_tree)
@@ -283,20 +187,14 @@ void pcb_data_free(pcb_data_t * data)
 	memset(data, 0, sizeof(pcb_data_t));
 }
 
-/* ---------------------------------------------------------------------------
- * returns pcb_true if data area is empty
- */
 pcb_bool pcb_data_is_empty(pcb_data_t *Data)
 {
-	pcb_bool hasNoObjects;
 	pcb_cardinal_t i;
 
-	if (pinlist_length(&Data->Via) != 0) return pcb_false;
 	if (padstacklist_length(&Data->padstack) != 0) return pcb_false;
-	if (elementlist_length(&Data->Element) != 0) return pcb_false;
 	if (pcb_subclist_length(&Data->subc) != 0) return pcb_false;
 	for (i = 0; i < Data->LayerN; i++)
-		if (!pcb_layer_is_empty_(PCB, &(Data->Layer[i])))
+		if (!pcb_layer_is_pure_empty(&(Data->Layer[i])))
 			return pcb_false;
 
 	return pcb_true;
@@ -309,13 +207,6 @@ pcb_box_t *pcb_data_bbox(pcb_box_t *out, pcb_data_t *Data, pcb_bool ignore_float
 	out->X2 = out->Y2 = -PCB_MAX_COORD;
 
 	/* now scan for the lowest/highest X and Y coordinate */
-	PCB_VIA_LOOP(Data);
-	{
-		pcb_pin_bbox(via);
-		if (!ignore_floaters || !PCB_FLAG_TEST(PCB_FLAG_FLOATER, via))
-			pcb_box_bump_box(out, &via->BoundingBox);
-	}
-	PCB_END_LOOP;
 	PCB_PADSTACK_LOOP(Data);
 	{
 		pcb_pstk_bbox(padstack);
@@ -323,13 +214,6 @@ pcb_box_t *pcb_data_bbox(pcb_box_t *out, pcb_data_t *Data, pcb_bool ignore_float
 			pcb_box_bump_box(out, &padstack->BoundingBox);
 	}
 	PCB_END_LOOP;
-	PCB_ELEMENT_LOOP(Data);
-	{
-		pcb_element_bbox(Data, element, pcb_font(PCB, 0, 0));
-		if (!ignore_floaters || !PCB_FLAG_TEST(PCB_FLAG_FLOATER, element))
-			pcb_box_bump_box(out, &element->BoundingBox);
-	}
-	PCB_END_LOOP;
 	PCB_SUBC_LOOP(Data);
 	{
 		pcb_subc_bbox(subc);
@@ -371,8 +255,10 @@ pcb_box_t *pcb_data_bbox(pcb_box_t *out, pcb_data_t *Data, pcb_bool ignore_float
 void pcb_data_set_layer_parents(pcb_data_t *data)
 {
 	pcb_layer_id_t n;
-	for(n = 0; n < PCB_MAX_LAYER; n++)
-		data->Layer[n].parent = data;
+	for(n = 0; n < PCB_MAX_LAYER; n++) {
+		data->Layer[n].parent.data = data;
+		data->Layer[n].parent_type = PCB_PARENT_DATA;
+	}
 }
 
 void pcb_data_bind_board_layers(pcb_board_t *pcb, pcb_data_t *data, int share_rtrees)
@@ -380,7 +266,8 @@ void pcb_data_bind_board_layers(pcb_board_t *pcb, pcb_data_t *data, int share_rt
 	pcb_layer_id_t n;
 	for(n = 0; n < pcb->Data->LayerN; n++) {
 		pcb_layer_real2bound(&data->Layer[n], &pcb->Data->Layer[n], share_rtrees);
-		data->Layer[n].parent = data;
+		data->Layer[n].parent.data = data;
+		data->Layer[n].parent_type = PCB_PARENT_DATA;
 	}
 	data->LayerN = pcb->Data->LayerN;
 }
@@ -391,7 +278,9 @@ void pcb_data_make_layers_bound(pcb_board_t *pcb4layer_groups, pcb_data_t *data)
 	for(n = 0; n < data->LayerN; n++) {
 		pcb_layer_type_t lyt = pcb_layergrp_flags(pcb4layer_groups, data->Layer[n].meta.real.grp);
 		pcb_layer_real2bound_offs(&data->Layer[n], pcb4layer_groups, &data->Layer[n]);
-		data->Layer[n].parent = data;
+		data->Layer[n].parent.data = data;
+		data->Layer[n].parent_type = PCB_PARENT_DATA;
+		data->Layer[n].type = PCB_OBJ_LAYER;
 		data->Layer[n].meta.bound.type = lyt;
 	}
 }
@@ -430,16 +319,11 @@ pcb_board_t *pcb_data_get_top(pcb_data_t *data)
 	return NULL;
 }
 
-void pcb_data_mirror(pcb_data_t *data, pcb_coord_t y_offs, pcb_bool text_too, pcb_bool pstk_smirror)
+void pcb_data_mirror(pcb_data_t *data, pcb_coord_t y_offs, pcb_data_mirror_text_t mtxt, pcb_bool pstk_smirror)
 {
-	PCB_VIA_LOOP(data);
-	{
-		pcb_via_mirror(data, via, y_offs);
-	}
-	PCB_END_LOOP;
 	PCB_PADSTACK_LOOP(data);
 	{
-		pcb_pstk_mirror(padstack, y_offs, pstk_smirror);
+		pcb_pstk_mirror(padstack, y_offs, pstk_smirror, 0);
 	}
 	PCB_END_LOOP;
 	PCB_SUBC_LOOP(data);
@@ -463,12 +347,23 @@ void pcb_data_mirror(pcb_data_t *data, pcb_coord_t y_offs, pcb_bool text_too, pc
 	}
 	PCB_ENDALL_LOOP;
 
-	if (text_too) {
-		PCB_TEXT_ALL_LOOP(data);
-		{
-			pcb_text_flip_side(layer, text, y_offs);
-		}
-		PCB_ENDALL_LOOP;
+	switch(mtxt) {
+		case PCB_TXM_NONE:
+			break;
+		case PCB_TXM_SIDE:
+			PCB_TEXT_ALL_LOOP(data);
+			{
+				pcb_text_flip_side(layer, text, y_offs);
+			}
+			PCB_ENDALL_LOOP;
+			break;
+		case PCB_TXM_COORD:
+			PCB_TEXT_ALL_LOOP(data);
+			{
+				pcb_text_mirror_coords(layer, text, y_offs);
+			}
+			PCB_ENDALL_LOOP;
+			break;
 	}
 }
 
@@ -503,15 +398,10 @@ int pcb_data_normalize(pcb_data_t *data)
 
 void pcb_data_set_parent_globals(pcb_data_t *data, pcb_data_t *new_parent)
 {
-	pcb_pin_t *via;
 	pcb_pstk_t *ps;
 	pcb_subc_t *sc;
 	gdl_iterator_t it;
 
-	pinlist_foreach(&data->Via, &it, via) {
-		via->parent_type = PCB_PARENT_DATA;
-		via->parent.data = new_parent;
-	}
 	padstacklist_foreach(&data->padstack, &it, ps) {
 		ps->parent_type = PCB_PARENT_DATA;
 		ps->parent.data = new_parent;
@@ -532,59 +422,43 @@ void pcb_data_move(pcb_data_t *data, pcb_coord_t dx, pcb_coord_t dy)
 	ctx.move.dx = dx;
 	ctx.move.dy = dy;
 
-	PCB_VIA_LOOP(data);
-	{
-		pcb_object_operation(&MoveFunctions, &ctx, PCB_TYPE_VIA, via, via, via);
-	}
-	PCB_END_LOOP;
 	PCB_PADSTACK_LOOP(data);
 	{
-		pcb_object_operation(&MoveFunctions, &ctx, PCB_TYPE_PSTK, padstack, padstack, padstack);
-	}
-	PCB_END_LOOP;
-	PCB_ELEMENT_LOOP(data);
-	{
-		pcb_object_operation(&MoveFunctions, &ctx, PCB_TYPE_ELEMENT, element, element, element);
+		pcb_object_operation(&MoveFunctions, &ctx, PCB_OBJ_PSTK, padstack, padstack, padstack);
 	}
 	PCB_END_LOOP;
 	PCB_SUBC_LOOP(data);
 	{
-		pcb_object_operation(&MoveFunctions, &ctx, PCB_TYPE_SUBC, subc, subc, subc);
+		pcb_object_operation(&MoveFunctions, &ctx, PCB_OBJ_SUBC, subc, subc, subc);
 	}
 	PCB_END_LOOP;
 	PCB_LINE_ALL_LOOP(data);
 	{
-		pcb_object_operation(&MoveFunctions, &ctx, PCB_TYPE_LINE, layer, line, line);
+		pcb_object_operation(&MoveFunctions, &ctx, PCB_OBJ_LINE, layer, line, line);
 	}
 	PCB_ENDALL_LOOP;
 	PCB_ARC_ALL_LOOP(data);
 	{
-		pcb_object_operation(&MoveFunctions, &ctx, PCB_TYPE_ARC, layer, arc, arc);
+		pcb_object_operation(&MoveFunctions, &ctx, PCB_OBJ_ARC, layer, arc, arc);
 	}
 	PCB_ENDALL_LOOP;
 	PCB_TEXT_ALL_LOOP(data);
 	{
-		pcb_object_operation(&MoveFunctions, &ctx, PCB_TYPE_TEXT, layer, text, text);
+		pcb_object_operation(&MoveFunctions, &ctx, PCB_OBJ_TEXT, layer, text, text);
 	}
 	PCB_ENDALL_LOOP;
 	PCB_POLY_ALL_LOOP(data);
 	{
-		pcb_object_operation(&MoveFunctions, &ctx, PCB_TYPE_POLY, layer, polygon, polygon);
+		pcb_object_operation(&MoveFunctions, &ctx, PCB_OBJ_POLY, layer, polygon, polygon);
 	}
 	PCB_ENDALL_LOOP;
 }
 
 void pcb_data_list_by_flag(pcb_data_t *data, vtp0_t *dst, pcb_objtype_t type, unsigned long mask)
 {
-	if (type & PCB_OBJ_VIA) PCB_VIA_LOOP(data); {
-		if ((mask == 0) || (PCB_FLAG_TEST(mask, via))) vtp0_append(dst, via);
-	} PCB_END_LOOP;
 	if (type & PCB_OBJ_PSTK) PCB_PADSTACK_LOOP(data); {
 		if ((mask == 0) || (PCB_FLAG_TEST(mask, padstack))) vtp0_append(dst, padstack);
 	} PCB_END_LOOP;
-	if (type & PCB_OBJ_ELEMENT) PCB_ELEMENT_LOOP(data); {
-		if ((mask == 0) || (PCB_FLAG_TEST(mask, element))) vtp0_append(dst, element);
-	} PCB_END_LOOP;
 	if (type & PCB_OBJ_SUBC) PCB_SUBC_LOOP(data); {
 		if ((mask == 0) || (PCB_FLAG_TEST(mask, subc))) vtp0_append(dst, subc);
 	} PCB_END_LOOP;
@@ -604,10 +478,7 @@ void pcb_data_list_by_flag(pcb_data_t *data, vtp0_t *dst, pcb_objtype_t type, un
 
 void pcb_data_list_terms(pcb_data_t *data, vtp0_t *dst, pcb_objtype_t type)
 {
-	if (type & PCB_OBJ_VIA) PCB_VIA_LOOP(data); {
-		if (via->term != NULL) vtp0_append(dst, via);
-	} PCB_END_LOOP;
-#warning TODO: subc TODO
+#warning subc TODO: subc in subc
 /*	if (type & PCB_OBJ_SUBC) PCB_SUBC_LOOP(data); {
 		if (subc->term != NULL) vtp0_append(dst, subc);
 	} PCB_END_LOOP;*/
@@ -649,16 +520,9 @@ pcb_r_dir_t pcb_data_r_search(pcb_data_t *data, pcb_objtype_t types, const pcb_b
 	pcb_layer_id_t lid;
 	pcb_r_dir_t res = 0;
 
-	if (!vis_only || PCB->ViaOn)
-		if (types & PCB_OBJ_VIA)  rsearch(data->via_tree);
-
 	if (!vis_only || PCB->RatOn)
 		if (types & PCB_OBJ_RAT)  rsearch(data->rat_tree);
 
-	if (!vis_only || PCB->PinOn) {
-		if (types & PCB_OBJ_PIN)  rsearch(data->pin_tree);
-		if (types & PCB_OBJ_PAD)  rsearch(data->pad_tree);
-	}
 	if (types & PCB_OBJ_PSTK) rsearch(data->padstack_tree);
 
 	if (!vis_only || PCB->SubcOn)
@@ -676,20 +540,9 @@ pcb_r_dir_t pcb_data_r_search(pcb_data_t *data, pcb_objtype_t types, const pcb_b
 		if (types & PCB_OBJ_ARC)  rsearch(ly->arc_tree);
 	}
 
-	/* safe to remove this block with elements */
-	{
-		int n;
-
-		if (types & PCB_OBJ_ELEMENT) rsearch(data->element_tree);
-		if (types & PCB_OBJ_ETEXT)
-			for(n = 0; n < 3; n++)
-				rsearch(data->name_tree[n]);
-	}
-
 	return res;
 }
 
-
 /*** poly clip inhibit mechanism ***/
 void pcb_data_clip_inhibit_inc(pcb_data_t *data)
 {
@@ -711,12 +564,49 @@ void pcb_data_clip_inhibit_dec(pcb_data_t *data, pcb_bool enable_progbar)
 	}
 	data->clip_inhibit--;
 
-	pcb_data_clip_dirty(data, enable_progbar);
+	if (data->clip_inhibit == 0)
+		pcb_data_clip_dirty(data, enable_progbar);
+}
+
+typedef struct {
+	pcb_data_t *data;
+	time_t nextt;
+	int inited, force_all;
+	pcb_cardinal_t at, total;
+} data_clip_all_t;
+
+static void data_clip_all_cb(void *ctx_)
+{
+	data_clip_all_t *ctx = ctx_;
+
+	ctx->at++;
+	if ((ctx->at % 8) == 0) {
+		time_t now = time(NULL);
+		if (now >= ctx->nextt) {
+			ctx->nextt = now+1;
+			if (!ctx->inited) {
+				ctx->total = 0;
+				PCB_POLY_ALL_LOOP(ctx->data); {
+					if (ctx->force_all || polygon->clip_dirty)
+						ctx->total += pcb_poly_num_clears(ctx->data, layer, polygon);
+				} PCB_ENDALL_LOOP;
+				ctx->inited = 1;
+			}
+			if (pcb_gui->progress(ctx->at, ctx->total, "Clipping polygons...") != 0) {
+				if (pcb_gui->confirm_dialog("The only way to cancel poly clipping is to quit pcb-rnd.\nAre you sure you want to quit?", "yes, quit", "no, continue", NULL) != 0) {
+					/* Have to recreate the dialog next time, that's the only way to get it out from the cancel state */
+					pcb_gui->progress(0, 0, NULL);
+				}
+				else
+					exit(1);
+			}
+		}
+	}
 }
 
 void pcb_data_clip_all_poly(pcb_data_t *data, pcb_bool enable_progbar, pcb_bool force_all)
 {
-	pcb_cardinal_t sum = 0, n = 0;
+	data_clip_all_t ctx;
 
 	if (data->clip_inhibit != 0)
 		return;
@@ -725,10 +615,11 @@ void pcb_data_clip_all_poly(pcb_data_t *data, pcb_bool enable_progbar, pcb_bool
 		pcb_data_clip_dirty(subc->data, enable_progbar);
 	} PCB_END_LOOP;
 
-	PCB_POLY_ALL_LOOP(data); {
-		if (force_all || polygon->clip_dirty)
-			sum++;
-	} PCB_ENDALL_LOOP;
+	ctx.data = data;
+	ctx.force_all = force_all;
+	ctx.nextt = time(NULL) + 2;
+	ctx.total = ctx.at = 0;
+	ctx.inited = 0;
 
 	/* have to go in two passes, to make sure that clearing polygons are done
 	   before the polygons that are potentially being cleared - this way we
@@ -736,19 +627,16 @@ void pcb_data_clip_all_poly(pcb_data_t *data, pcb_bool enable_progbar, pcb_bool
 	   calculated, the clearing poly has a contour */
 	PCB_POLY_ALL_LOOP(data); {
 		if ((force_all || polygon->clip_dirty) && (PCB_FLAG_TEST(PCB_FLAG_CLEARPOLYPOLY, polygon)))
-			pcb_poly_init_clip(data, layer, polygon);
-#warning TODO: progbar
-/*		if ((n % 10) == 0) */
-		n++;
+			pcb_poly_init_clip_prog(data, layer, polygon, (enable_progbar ? data_clip_all_cb : NULL), &ctx);
 	} PCB_ENDALL_LOOP;
 
 	PCB_POLY_ALL_LOOP(data); {
 		if ((force_all || polygon->clip_dirty) && (!PCB_FLAG_TEST(PCB_FLAG_CLEARPOLYPOLY, polygon)))
-			pcb_poly_init_clip(data, layer, polygon);
-#warning TODO: progbar
-/*		if ((n % 10) == 0) */
-		n++;
+			pcb_poly_init_clip_prog(data, layer, polygon, (enable_progbar ? data_clip_all_cb : NULL), &ctx);
 	} PCB_ENDALL_LOOP;
+
+	if (enable_progbar)
+		pcb_gui->progress(0, 0, NULL);
 }
 
 void pcb_data_clip_dirty(pcb_data_t *data, pcb_bool enable_progbar)
@@ -762,7 +650,7 @@ void pcb_data_clip_all(pcb_data_t *data, pcb_bool enable_progbar)
 }
 
 
-void pcb_data_flag_change(pcb_data_t *data, pcb_obj_type_t mask, int how, unsigned long flags)
+void pcb_data_flag_change(pcb_data_t *data, pcb_objtype_t mask, int how, unsigned long flags)
 {
 	pcb_any_obj_t *o;
 	pcb_data_it_t it;
