@@ -18,6 +18,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
 #ifndef PCB_DATA_LIST_H
@@ -27,7 +31,7 @@
 #include "data.h"
 
 /* append objects of type, with flags matching mask to dst; pointer are
-   valid only until the first change to data */
+   valid only until the first change to data; if flags is 0, everything matches. */
 void pcb_data_list_by_flag(pcb_data_t *data, vtp0_t *dst, pcb_objtype_t type, unsigned long mask);
 
 /* append objects of type that are terminals; pointer are
