@@ -40,13 +40,9 @@
 #include "rotate.h"
 #include "rtree.h"
 #include "stub_draw.h"
-#include "obj_all.h"
 #include "layer_ui.h"
 
-#include "obj_pad_draw.h"
-#include "obj_pinvia_draw.h"
 #include "obj_pstk_draw.h"
-#include "obj_elem_draw.h"
 #include "obj_line_draw.h"
 #include "obj_arc_draw.h"
 #include "obj_rat_draw.h"
@@ -112,13 +108,14 @@ void pcb_lighten_color(const char *orig, char buf[8], double factor)
 	pcb_snprintf(buf, sizeof("#XXXXXX"), "#%02x%02x%02x", r, g, b);
 }
 
-void pcb_draw_dashed_line(pcb_hid_gc_t GC, pcb_coord_t x1, pcb_coord_t y1, pcb_coord_t x2, pcb_coord_t y2)
+void pcb_draw_dashed_line(pcb_hid_gc_t GC, pcb_coord_t x1, pcb_coord_t y1, pcb_coord_t x2, pcb_coord_t y2, unsigned int segs)
 {
 /* TODO: we need a real geo lib... using double here is plain wrong */
 	double dx = x2-x1, dy = y2-y1;
 	double len_squared = dx*dx + dy*dy;
 	int n;
-	const int segs = 11; /* must be odd */
+
+	segs = (segs << 1) + 1; /* must be odd */
 
 	if (len_squared < 1000000) {
 		/* line too short, just draw it - TODO: magic value; with a proper geo lib this would be gone anyway */
@@ -167,15 +164,7 @@ void pcb_redraw(void)
 
 static void DrawHoles(pcb_bool draw_plated, pcb_bool draw_unplated, const pcb_box_t * drawn_area)
 {
-	int plated = -1;
-
-	if (draw_plated && !draw_unplated)
-		plated = 1;
-	if (!draw_plated && draw_unplated)
-		plated = 0;
-
-	pcb_r_search(PCB->Data->pin_tree, drawn_area, NULL, pcb_hole_draw_callback, &plated, NULL);
-	pcb_r_search(PCB->Data->via_tree, drawn_area, NULL, pcb_hole_draw_callback, &plated, NULL);
+#warning padstack TODO: either remove this or draw padstack holes here
 }
 
 static void DrawEverything_holes(pcb_layergrp_id_t gid, const pcb_box_t *drawn_area)
@@ -201,7 +190,7 @@ static void DrawEverything_holes(pcb_layergrp_id_t gid, const pcb_box_t *drawn_a
  */
 static void DrawEverything(const pcb_box_t * drawn_area)
 {
-	int i, ngroups, side, slk_len;
+	int i, ngroups, slk_len;
 	pcb_layergrp_id_t component, solder, slk[16], gid, side_copper_grp;
 	/* This is the list of layer groups we will draw.  */
 	pcb_layergrp_id_t do_group[PCB_MAX_LAYERGRP];
@@ -237,19 +226,17 @@ static void DrawEverything(const pcb_box_t * drawn_area)
 	solder = component = -1;
 	pcb_layergrp_list(PCB, PCB_LYT_BOTTOM | PCB_LYT_COPPER, &solder, 1);
 	pcb_layergrp_list(PCB, PCB_LYT_TOP | PCB_LYT_COPPER, &component, 1);
-	side_copper_grp = PCB_SWAP_IDENT ? solder : component;
+	side_copper_grp = conf_core.editor.show_solder_side ? solder : component;
 
 
 	/*
 	 * first draw all 'invisible' stuff
 	 */
 	if (!conf_core.editor.check_planes && pcb_layer_gui_set_vlayer(PCB, PCB_VLY_INVISIBLE, 0)) {
-		side = PCB_SWAP_IDENT ? PCB_COMPONENT_SIDE : PCB_SOLDER_SIDE;
 		pcb_draw_silk(PCB_LYT_INVISIBLE_SIDE(), drawn_area);
 
 		pcb_gui->set_drawing_mode(PCB_HID_COMP_RESET, pcb_draw_out.direct, drawn_area);
 		pcb_gui->set_drawing_mode(PCB_HID_COMP_POSITIVE, pcb_draw_out.direct, drawn_area);
-		pcb_r_search(PCB->Data->pad_tree, drawn_area, NULL, pcb_pad_draw_callback, &side, NULL);
 		pcb_gui->set_drawing_mode(PCB_HID_COMP_FLUSH, pcb_draw_out.direct, drawn_area);
 
 		pcb_gui->end_layer();
@@ -282,11 +269,11 @@ static void DrawEverything(const pcb_box_t * drawn_area)
 	if (conf_core.editor.check_planes && pcb_gui->gui)
 		goto finish;
 
-	/* Draw pins, pads, vias below silk */
+	/* Draw padstacks below silk */
 	pcb_gui->set_drawing_mode(PCB_HID_COMP_RESET, pcb_draw_out.direct, drawn_area);
 	pcb_gui->set_drawing_mode(PCB_HID_COMP_POSITIVE, pcb_draw_out.direct, drawn_area);
 	if (pcb_gui->gui)
-		pcb_draw_ppv(PCB_SWAP_IDENT ? solder : component, drawn_area);
+		pcb_draw_ppv(conf_core.editor.show_solder_side ? solder : component, drawn_area);
 	else if (!pcb_gui->holes_after)
 		DrawEverything_holes(side_copper_grp, drawn_area);
 	pcb_gui->set_drawing_mode(PCB_HID_COMP_FLUSH, pcb_draw_out.direct, drawn_area);
@@ -323,9 +310,7 @@ static void DrawEverything(const pcb_box_t * drawn_area)
 	}
 
 	gid = pcb_layergrp_get_top_paste();
-	if (gid < 0)
-		paste_empty = pcb_layer_is_paste_auto_empty(PCB, PCB_COMPONENT_SIDE);
-	else
+	if (gid >= 0)
 		paste_empty = pcb_layergrp_is_empty(PCB, gid);
 	if ((gid >= 0) && (pcb_layer_gui_set_glayer(PCB, gid, paste_empty))) {
 		pcb_draw_paste(PCB_COMPONENT_SIDE, drawn_area);
@@ -333,9 +318,7 @@ static void DrawEverything(const pcb_box_t * drawn_area)
 	}
 
 	gid = pcb_layergrp_get_bottom_paste();
-	if (gid < 0)
-		paste_empty = pcb_layer_is_paste_auto_empty(PCB, PCB_SOLDER_SIDE);
-	else
+	if (gid >= 0)
 		paste_empty = pcb_layergrp_is_empty(PCB, gid);
 	if ((gid >= 0) && (pcb_layer_gui_set_glayer(PCB, gid, paste_empty))) {
 		pcb_draw_paste(PCB_SOLDER_SIDE, drawn_area);
@@ -363,13 +346,10 @@ static void DrawEverything(const pcb_box_t * drawn_area)
 	}
 
 	if (pcb_gui->gui) {
-		/* Draw element Marks */
+		/* Draw subc Marks */
 		pcb_gui->set_drawing_mode(PCB_HID_COMP_RESET, pcb_draw_out.direct, drawn_area);
 		pcb_gui->set_drawing_mode(PCB_HID_COMP_POSITIVE, pcb_draw_out.direct, drawn_area);
 	
-		if (PCB->PinOn)
-			pcb_r_search(PCB->Data->element_tree, drawn_area, NULL, pcb_elem_mark_draw_callback, NULL, NULL);
-
 		if (PCB->SubcOn)
 			pcb_r_search(PCB->Data->subc_tree, drawn_area, NULL, draw_subc_mark_callback, NULL, NULL);
 
@@ -390,7 +370,7 @@ static void DrawEverything(const pcb_box_t * drawn_area)
 		/* Draw pins' and pads' names */
 		pcb_gui->set_drawing_mode(PCB_HID_COMP_RESET, pcb_draw_out.direct, drawn_area);
 		pcb_gui->set_drawing_mode(PCB_HID_COMP_POSITIVE, pcb_draw_out.direct, drawn_area);
-		pcb_draw_ppv_names(PCB_SWAP_IDENT ? solder : component, drawn_area);
+		pcb_draw_pstk_names(conf_core.editor.show_solder_side ? solder : component, drawn_area);
 		pcb_gui->set_drawing_mode(PCB_HID_COMP_FLUSH, pcb_draw_out.direct, drawn_area);
 	}
 
@@ -456,75 +436,23 @@ static void pcb_draw_pstk_holes(pcb_layergrp_id_t group, const pcb_box_t *drawn_
 }
 
 /* ---------------------------------------------------------------------------
- * Draws pins pads and vias - Always draws for non-gui HIDs,
- * otherwise drawing depends on PCB->PinOn and PCB->ViaOn
+ * Draws padstacks - Always draws for non-gui HIDs,
+ * otherwise drawing depends on PCB->pstk_on
  */
 void pcb_draw_ppv(pcb_layergrp_id_t group, const pcb_box_t * drawn_area)
 {
-	int side;
-	unsigned int gflg = pcb_layergrp_flags(PCB, group);
-
-	if (PCB->PinOn || !pcb_gui->gui) {
-		/* draw element pins */
-		pcb_r_search(PCB->Data->pin_tree, drawn_area, NULL, pcb_pin_draw_callback, NULL, NULL);
-
-		/* draw element pads */
-		if (gflg & PCB_LYT_TOP) {
-			side = PCB_COMPONENT_SIDE;
-			pcb_r_search(PCB->Data->pad_tree, drawn_area, NULL, pcb_pad_draw_callback, &side, NULL);
-		}
-
-		if (gflg & PCB_LYT_BOTTOM) {
-			side = PCB_SOLDER_SIDE;
-			pcb_r_search(PCB->Data->pad_tree, drawn_area, NULL, pcb_pad_draw_callback, &side, NULL);
-		}
-	}
-
-	/* draw vias */
-	if (PCB->ViaOn || !pcb_gui->gui) {
-		pcb_r_search(PCB->Data->via_tree, drawn_area, NULL, pcb_via_draw_callback, NULL, NULL);
-		pcb_r_search(PCB->Data->via_tree, drawn_area, NULL, pcb_hole_draw_callback, NULL, NULL);
-	}
-	if (PCB->PinOn || pcb_draw_doing_assy)
-		pcb_r_search(PCB->Data->pin_tree, drawn_area, NULL, pcb_hole_draw_callback, NULL, NULL);
-
-
 	/* draw padstack holes - copper is drawn with each group */
-	if (PCB->ViaOn || !pcb_gui->gui)
+	if (PCB->pstk_on || !pcb_gui->gui)
 		pcb_draw_pstk_holes(group, drawn_area, PCB_PHOLE_PLATED | PCB_PHOLE_UNPLATED | PCB_PHOLE_BB);
 }
 
 /* ---------------------------------------------------------------------------
- * Draws pins' and pads' names - Always draws for non-gui HIDs,
- * otherwise drawing depends on PCB->PinOn and PCB->ViaOn
+ * Draws padstacks' names - Always draws for non-gui HIDs,
+ * otherwise drawing depends on PCB->pstk_on
  */
-void pcb_draw_ppv_names(pcb_layergrp_id_t group, const pcb_box_t * drawn_area)
+void pcb_draw_pstk_names(pcb_layergrp_id_t group, const pcb_box_t *drawn_area)
 {
-	int side;
-	unsigned int gflg = pcb_layergrp_flags(PCB, group);
-
-	if (PCB->PinOn || !pcb_gui->gui) {
-		/* draw element pins' names */
-		pcb_r_search(PCB->Data->pin_tree, drawn_area, NULL, pcb_pin_name_draw_callback, NULL, NULL);
-
-		/* draw element pads' names */
-		if (gflg & PCB_LYT_TOP) {
-			side = PCB_COMPONENT_SIDE;
-			pcb_r_search(PCB->Data->pad_tree, drawn_area, NULL, pcb_pad_name_draw_callback, &side, NULL);
-		}
-
-		if (gflg & PCB_LYT_BOTTOM) {
-			side = PCB_SOLDER_SIDE;
-			pcb_r_search(PCB->Data->pad_tree, drawn_area, NULL, pcb_pad_name_draw_callback, &side, NULL);
-		}
-	}
-
-	if (PCB->ViaOn || !pcb_gui->gui) {
-		/* draw element pins' names */
-		pcb_r_search(PCB->Data->via_tree, drawn_area, NULL, pcb_pin_name_draw_callback, NULL, NULL);
-	}
-
-	if (PCB->PinOn || !pcb_gui->gui) {
+	if (PCB->pstk_on || !pcb_gui->gui) {
 		size_t n;
 		for(n = 0; n < delayed_labels.used; n++)
 			pcb_draw_obj_label(group, delayed_labels.array[n]);
@@ -588,7 +516,7 @@ void pcb_draw_layer(pcb_layer_t *Layer, const pcb_box_t * screen)
 	   the outline layer.  */
 	if ((lflg & PCB_LYT_OUTLINE) && pcb_layer_is_empty_(PCB, Layer)) {
 		pcb_gui->set_color(pcb_draw_out.fgGC, Layer->meta.real.color);
-		pcb_gui->set_line_width(pcb_draw_out.fgGC, PCB->minWid);
+		pcb_gui->set_line_width(pcb_draw_out.fgGC, conf_core.design.min_wid);
 		pcb_gui->draw_rect(pcb_draw_out.fgGC, 0, 0, PCB->MaxWidth, PCB->MaxHeight);
 	}
 
@@ -677,7 +605,7 @@ void pcb_draw_layer_under(pcb_layer_t *Layer, const pcb_box_t *screen, pcb_data_
 
 /* ---------------------------------------------------------------------------
  * draws one layer group.  If the exporter is not a GUI,
- * also draws the pins / pads / vias in this layer group.
+ * also draws the padstacks in this layer group.
  */
 static void DrawLayerGroup(int group, const pcb_box_t *drawn_area, int is_current)
 {
@@ -703,7 +631,7 @@ static void DrawLayerGroup(int group, const pcb_box_t *drawn_area, int is_curren
 	if (n_entries > 1)
 		rv = 1;
 
-	if (gflg & PCB_LYT_COPPER)
+	if ((gflg & PCB_LYT_COPPER) && (PCB->pstk_on))
 		pcb_draw_pstks(group, drawn_area, (CURRENT->meta.real.grp == group), 0);
 
 	/* this draws the holes - must be the last, so holes are drawn over everything else */
@@ -719,37 +647,24 @@ void pcb_erase_obj(int type, void *lptr, void *ptr)
 		return;
 
 	switch (type) {
-	case PCB_TYPE_PSTK:
+	case PCB_OBJ_PSTK:
 		pcb_pstk_invalidate_erase((pcb_pstk_t *) ptr);
 		break;
 
-	case PCB_TYPE_VIA:
-	case PCB_TYPE_PIN:
-		pcb_pin_invalidate_erase((pcb_pin_t *) ptr);
-		break;
-	case PCB_TYPE_TEXT:
-	case PCB_TYPE_ELEMENT_NAME:
+	case PCB_OBJ_TEXT:
 		pcb_text_invalidate_erase((pcb_layer_t *) lptr, (pcb_text_t *) ptr);
 		break;
-	case PCB_TYPE_POLY:
+	case PCB_OBJ_POLY:
 		pcb_poly_invalidate_erase((pcb_poly_t *) ptr);
 		break;
-	case PCB_TYPE_ELEMENT:
-		pcb_elem_invalidate_erase((pcb_element_t *) ptr);
-		break;
-	case PCB_TYPE_SUBC:
+	case PCB_OBJ_SUBC:
 		EraseSubc((pcb_subc_t *)ptr);
 		break;
-	case PCB_TYPE_LINE:
-	case PCB_TYPE_ELEMENT_LINE:
-	case PCB_TYPE_RATLINE:
+	case PCB_OBJ_LINE:
+	case PCB_OBJ_RAT:
 		pcb_line_invalidate_erase((pcb_line_t *) ptr);
 		break;
-	case PCB_TYPE_PAD:
-		pcb_pad_invalidate_erase((pcb_pad_t *) ptr);
-		break;
-	case PCB_TYPE_ARC:
-	case PCB_TYPE_ELEMENT_ARC:
+	case PCB_OBJ_ARC:
 		pcb_arc_invalidate_erase((pcb_arc_t *) ptr);
 		break;
 	default:
@@ -764,12 +679,8 @@ void pcb_draw_obj(pcb_any_obj_t *obj)
 		return;
 
 	switch (obj->type) {
-	case PCB_OBJ_VIA:
-		if (PCB->ViaOn)
-			pcb_via_invalidate_draw((pcb_pin_t *)obj);
-		break;
 	case PCB_OBJ_PSTK:
-		if (PCB->ViaOn)
+		if (PCB->pstk_on)
 			pcb_pstk_invalidate_draw((pcb_pstk_t *)obj);
 		break;
 	case PCB_OBJ_LINE:
@@ -788,32 +699,14 @@ void pcb_draw_obj(pcb_any_obj_t *obj)
 		if (obj->parent.layer->meta.real.vis)
 			pcb_poly_invalidate_draw(obj->parent.layer, (pcb_poly_t *)obj);
 		break;
-	case PCB_OBJ_ELEMENT:
-		if (pcb_silk_on(PCB) && (PCB_FRONT((pcb_element_t *)obj) || PCB->InvisibleObjectsOn))
-			pcb_elem_invalidate_draw((pcb_element_t *)obj);
-		break;
 	case PCB_OBJ_RAT:
 		if (PCB->RatOn)
 			pcb_rat_invalidate_draw((pcb_rat_t *)obj);
 		break;
-	case PCB_OBJ_PIN:
-		if (PCB->PinOn)
-			pcb_pin_invalidate_draw((pcb_pin_t *)obj);
-		break;
-	case PCB_OBJ_PAD:
-		if (PCB->PinOn)
-			pcb_pad_invalidate_draw((pcb_pad_t *)obj);
-		break;
-	case PCB_OBJ_ETEXT:
-		if (pcb_silk_on(PCB) && (PCB_FRONT(obj->parent.element) || PCB->InvisibleObjectsOn))
-			pcb_elem_name_invalidate_draw(obj->parent.element);
-		break;
-	case PCB_OBJ_POINT:
 	case PCB_OBJ_SUBC:
 	case PCB_OBJ_NET:
 	case PCB_OBJ_LAYER:
-	case PCB_OBJ_ELINE:
-	case PCB_OBJ_EARC:
+	case PCB_OBJ_LAYERGRP:
 	case PCB_OBJ_VOID:
 		break;
 	}
@@ -917,7 +810,6 @@ void pcb_hid_expose_pinout(pcb_hid_t * hid, const pcb_hid_expose_ctx_t *ctx)
 	if (ctx->content.obj != NULL) {
 		pcb_draw_doing_pinout = pcb_true;
 		switch(ctx->content.obj->type) {
-			case PCB_OBJ_ELEMENT: pcb_elem_draw((pcb_element_t *)ctx->content.obj); break;
 			case PCB_OBJ_SUBC:    pcb_subc_draw_preview((pcb_subc_t *)ctx->content.obj, &ctx->view); break;
 			default:              pcb_message(PCB_MSG_ERROR, "pcb_hid_expose_pinout: unknown object type %x\n", ctx->content.obj->type);
 		}
