@@ -36,6 +36,7 @@
 #include "build_run.h"
 #include "conf_core.h"
 #include "data.h"
+#include "buffer.h"
 #include "action_helper.h"
 
 #include "plug_io.h"
@@ -50,6 +51,7 @@
 #include "hid_init.h"
 #include "layer_vis.h"
 #include "safe_fs.h"
+#include "tool.h"
 
 /* ---------------------------------------------------------------- */
 static const char pcb_acts_ExecCommand[] = "ExecCommand(command)";
@@ -99,9 +101,9 @@ Loads an entire PCB layout, replacing the current one.
 Loads an entire PCB layout to the paste buffer.
 
 @item ElementToBuffer
-Loads the given element file into the paste buffer.  Element files
-contain only a single @code{Element} definition, such as the
-``newlib'' library uses.
+Loads the given footprint file into the paste buffer. Footprint files
+contain only a single subcircuit definition in one of the various
+supported file formats.
 
 @item Netlist
 Loads a new netlist, replacing any current netlist.
@@ -128,15 +130,15 @@ static int pcb_act_LoadFrom(int argc, const char **argv, pcb_coord_t x, pcb_coor
 
 	if (pcb_strcasecmp(function, "ElementToBuffer") == 0) {
 		pcb_notify_crosshair_change(pcb_false);
-		if (pcb_element_load_to_buffer(PCB_PASTEBUFFER, name, format))
-			pcb_crosshair_set_mode(PCB_MODE_PASTE_BUFFER);
+		if (pcb_buffer_load_footprint(PCB_PASTEBUFFER, name, format))
+			pcb_tool_select_by_id(PCB_MODE_PASTE_BUFFER);
 		pcb_notify_crosshair_change(pcb_true);
 	}
 
 	else if (pcb_strcasecmp(function, "LayoutToBuffer") == 0) {
 		pcb_notify_crosshair_change(pcb_false);
 		if (pcb_buffer_load_layout(PCB, PCB_PASTEBUFFER, name, format))
-			pcb_crosshair_set_mode(PCB_MODE_PASTE_BUFFER);
+			pcb_tool_select_by_id(PCB_MODE_PASTE_BUFFER);
 		pcb_notify_crosshair_change(pcb_true);
 	}
 
@@ -250,7 +252,7 @@ Save all connections to a file.
 List all unused pins to a file.
 
 @item ElementConnections
-Save connections to the element at the cursor to a file.
+Save connections to the subcircuit at the cursor to a file.
 
 @item PasteBuffer
 Save the content of the active Buffer to a file. This is the graphical way to create a footprint.
@@ -265,10 +267,17 @@ static int pcb_act_SaveTo(int argc, const char **argv, pcb_coord_t x, pcb_coord_
 	const char *name;
 	const char *fmt = NULL;
 
+	if (argc < 1)
+		PCB_ACT_FAIL(SaveTo);
+
 	function = argv[0];
 	name = argv[1];
 
 	if (pcb_strcasecmp(function, "Layout") == 0) {
+		if (argc != 1) {
+			pcb_message(PCB_MSG_ERROR, "SaveTo(Layout) doesn't take file name or format - did you mean SaveTo(LayoutAs)?\n");
+			return -1;
+		}
 		if (pcb_save_pcb(PCB->Filename, NULL) == 0)
 			pcb_board_set_changed_flag(pcb_false);
 		if (pcb_gui->notify_filename_changed != NULL)
@@ -316,17 +325,16 @@ static int pcb_act_SaveTo(int argc, const char **argv, pcb_coord_t x, pcb_coord_
 	}
 
 	if (pcb_strcasecmp(function, "ElementConnections") == 0) {
-		pcb_element_t *element;
 		void *ptrtmp;
 		FILE *fp;
 		pcb_bool result;
 		pcb_coord_t x, y;
 
 		pcb_gui->get_coords("Click on an element", &x, &y);
-		if ((pcb_search_screen(x, y, PCB_TYPE_ELEMENT, &ptrtmp, &ptrtmp, &ptrtmp)) != PCB_TYPE_NONE) {
-			element = (pcb_element_t *) ptrtmp;
+		if ((pcb_search_screen(x, y, PCB_OBJ_SUBC, &ptrtmp, &ptrtmp, &ptrtmp)) != PCB_OBJ_VOID) {
+			pcb_subc_t *subc = (pcb_subc_t *) ptrtmp;
 			if ((fp = pcb_check_and_open_file(name, pcb_true, pcb_false, &result, NULL)) != NULL) {
-				pcb_lookup_element_conns(element, fp);
+				pcb_lookup_subc_conns(subc, fp);
 				fclose(fp);
 				pcb_board_set_changed_flag(pcb_true);
 			}
