@@ -30,7 +30,7 @@
 
 /*
  * short description:
- * - lists for pins and vias, lines, arcs, pads and for polygons are created.
+ * - lists for padstacks, lines, arcs, pads and for polygons are created.
  *   Every object that has to be checked is added to its list.
  *   Coarse searching is accomplished with the data rtrees.
  * - there's no 'speed-up' mechanism for polygons because they are not used
@@ -39,20 +39,19 @@
  *   between them. To speed up computation the limit is set to one half
  *   of the thickness of the objects (cause of square pins).
  *
- * PV:  means pin or via (objects that connect layers)
- * LO:  all non PV objects (layer objects like lines, arcs, polygons, pads)
+ * PS:  means padstack
+ * LO:  all non PS objects (layer objects like lines, arcs, polygons, texts)
  *
- * 1. first, the LO or PV at the given coordinates is looked up
- * 2. all LO connections to that PV are looked up next
+ * 1. first, the LO or PS at the given coordinates is looked up
+ * 2. all LO connections to that PS are looked up next
  * 3. lookup of all LOs connected to LOs from (2).
  *    This step is repeated until no more new connections are found.
- * 4. lookup all PVs connected to the LOs from (2) and (3)
- * 5. start again with (1) for all new PVs from (4)
+ * 4. lookup all PSs connected to the LOs from (2) and (3)
+ * 5. start again with (1) for all new PSs from (4)
  *
  */
 
-/* routines to find connections between pins, vias, lines...
- */
+/* routines to find connections between objects */
 #include "config.h"
 
 #include <stdlib.h>
@@ -60,7 +59,6 @@
 #include <setjmp.h>
 #include <assert.h>
 
-#include "const.h"
 #include "math_helper.h"
 #include "conf_core.h"
 #include "data.h"
@@ -80,9 +78,9 @@
 #include "event.h"
 #include "layer_vis.h"
 
-#include "obj_all.h"
-#include "obj_pstk.h"
 #include "obj_text_draw.h"
+#include "obj_pstk.h"
+#include "obj_subc.h"
 #include "obj_subc_parent.h"
 
 #undef DEBUG
@@ -96,9 +94,6 @@
 		fputc('\n', (FP));						\
 	}
 
-#define	PADLIST_ENTRY(L,I)	\
-	(((pcb_pad_t **)PadList[(L)].Data)[(I)])
-
 #define	LINELIST_ENTRY(L,I)	\
 	(((pcb_line_t **)LineList[(L)].Data)[(I)])
 
@@ -111,15 +106,12 @@
 #define	POLYGONLIST_ENTRY(L,I)	\
 	(((pcb_poly_t **)PolygonList[(L)].Data)[(I)])
 
-#define	PVLIST_ENTRY(I)	\
-	(((pcb_pin_t **)PVList.Data)[(I)])
-
 #define	PADSTACKLIST_ENTRY(I)	\
 	(((pcb_pstk_t **)PadstackList.Data)[(I)])
 
 /* ---------------------------------------------------------------------------
  * the two 'dummy' structs for PVs and Pads are necessary for creating
- * connection lists which include the element's name
+ * connection lists which include the subcircuit's name
  */
 typedef struct {
 	void **Data;									/* pointer to index data */
@@ -144,18 +136,14 @@ static pcb_bool User = pcb_false;				/* user action causing this */
 static pcb_bool drc = pcb_false;				/* whether to stop if finding something not found */
 static pcb_bool IsBad = pcb_false;
 static pcb_cardinal_t drcerr_count;		/* count of drc errors */
-static pcb_cardinal_t TotalP, TotalV, NumberOfPads[2], TotalPs;
+static pcb_cardinal_t TotalPs;
 static ListType LineList[PCB_MAX_LAYER+2],	/* list of objects to */
-  PolygonList[PCB_MAX_LAYER+2], ArcList[PCB_MAX_LAYER+2], PadList[2],
-  RatList, PVList, PadstackList;
+  PolygonList[PCB_MAX_LAYER+2], ArcList[PCB_MAX_LAYER+2],
+  RatList, PadstackList;
 
-static pcb_bool LookupLOConnectionsToPVList(pcb_bool);
 static pcb_bool LookupLOConnectionsToPSList(pcb_bool);
 static pcb_bool LookupLOConnectionsToLOList(pcb_bool);
-static pcb_bool LookupPVPSConnectionsToLOList(pcb_bool);
-static pcb_bool LookupPVConnectionsToPVList(void);
 static pcb_bool LookupLOConnectionsToLine(pcb_line_t *, pcb_cardinal_t, pcb_bool);
-static pcb_bool LookupLOConnectionsToPad(pcb_pad_t *, pcb_cardinal_t);
 static pcb_bool LookupLOConnectionsToPolygon(pcb_poly_t *, pcb_cardinal_t);
 static pcb_bool LookupLOConnectionsToArc(pcb_arc_t *, pcb_cardinal_t);
 static pcb_bool LookupLOConnectionsToRatEnd(pcb_point_t *, pcb_cardinal_t);
@@ -163,14 +151,12 @@ static pcb_bool IsRatPointOnLineSpec(pcb_point_t *, pcb_line_t *);
 static pcb_bool IsRatPointOnPoly(pcb_point_t *Point, pcb_poly_t *polygon);
 static pcb_bool IsRatPointOnArcSpec(pcb_point_t *Point, pcb_arc_t *arc);
 static pcb_bool ArcArcIntersect(pcb_arc_t *, pcb_arc_t *);
-static pcb_bool PrintElementConnections(pcb_element_t *, FILE *, pcb_bool);
+static pcb_bool pcb_print_subc_conns(pcb_subc_t *, FILE *, pcb_bool);
 static pcb_bool ListsEmpty(pcb_bool);
 static pcb_bool DoIt(pcb_bool, pcb_bool);
-static void PrintElementNameList(pcb_element_t *, FILE *);
-static void PrintConnectionElementName(pcb_element_t *, FILE *);
-static void PrintConnectionListEntry(char *, pcb_element_t *, pcb_bool, FILE *);
-static void PrintPadConnections(pcb_cardinal_t, FILE *, pcb_bool);
-static void PrintPinConnections(FILE *, pcb_bool);
+static void PrintElementNameList(pcb_subc_t *, FILE *);
+static void pcb_print_conn_subc_name(pcb_subc_t *, FILE *);
+static void pcb_print_conn_list_entry(char *, pcb_subc_t *, pcb_bool, FILE *);
 static void DumpList(void);
 static pcb_bool ListStart(pcb_any_obj_t *obj);
 static pcb_bool SetThing(int, void *, void *, void *);
