@@ -2,7 +2,8 @@
  *
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996, 2005 Thomas Nau
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -19,9 +20,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *  Contact addresses for paper mail and Email:
- *  Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
- *  Thomas.Nau@rz.uni-ulm.de
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
 
@@ -77,7 +79,7 @@ static pcb_bool PVTouchesLine(pcb_line_t *line)
 	else
 		return pcb_true;
 
-	return (pcb_false);
+	return pcb_false;
 }
 
 static pcb_bool LOTouchesLine(pcb_line_t *Line, pcb_cardinal_t LayerGroup)
@@ -105,17 +107,17 @@ static pcb_bool LOTouchesLine(pcb_line_t *Line, pcb_cardinal_t LayerGroup)
 			if (setjmp(info.env) == 0)
 				pcb_r_search(LAYER_PTR(layer)->line_tree, (pcb_box_t *) & info.line, NULL, LOT_Linecallback, &info, NULL);
 			else
-				return (pcb_true);
+				return pcb_true;
 			if (setjmp(info.env) == 0)
 				pcb_r_search(LAYER_PTR(layer)->arc_tree, (pcb_box_t *) & info.line, NULL, LOT_Arccallback, &info, NULL);
 			else
-				return (pcb_true);
+				return pcb_true;
 
 			/* now check all polygons */
 			polylist_foreach(&(PCB->Data->Layer[layer].Polygon), &it, polygon) {
 				if (!PCB_FLAG_TEST(TheFlag, polygon)
 						&& pcb_is_line_in_poly(Line, polygon))
-					return (pcb_true);
+					return pcb_true;
 			}
 		}
 		else {
@@ -127,7 +129,7 @@ static pcb_bool LOTouchesLine(pcb_line_t *Line, pcb_cardinal_t LayerGroup)
 				return pcb_true;
 		}
 	}
-	return (pcb_false);
+	return pcb_false;
 }
 
 /* returns pcb_true if nothing un-found touches the passed line
@@ -138,8 +140,8 @@ static pcb_bool LOTouchesLine(pcb_line_t *Line, pcb_cardinal_t LayerGroup)
 pcb_bool lineClear(pcb_line_t *line, pcb_cardinal_t group)
 {
 	if (LOTouchesLine(line, group))
-		return (pcb_false);
+		return pcb_false;
 	if (PVTouchesLine(line))
-		return (pcb_false);
-	return (pcb_true);
+		return pcb_false;
+	return pcb_true;
 }
