@@ -52,20 +52,6 @@
        (p)->BoundingBox.Y1 -= Bloat; \
        (p)->BoundingBox.Y2 += Bloat;}
 
-#define IS_PV_ON_RAT(PV, Rat) \
-	(pcb_is_point_on_line_end((PV)->X,(PV)->Y, (Rat)))
-
-#define IS_PV_ON_ARC(PV, Arc)	\
-	(PCB_FLAG_TEST(PCB_FLAG_SQUARE, (PV)) ? \
-		pcb_is_arc_in_rectangle( \
-			(PV)->X -MAX(((PV)->Thickness+1)/2 +Bloat,0), (PV)->Y -MAX(((PV)->Thickness+1)/2 +Bloat,0), \
-			(PV)->X +MAX(((PV)->Thickness+1)/2 +Bloat,0), (PV)->Y +MAX(((PV)->Thickness+1)/2 +Bloat,0), \
-			(Arc)) : \
-		pcb_is_point_on_arc((PV)->X,(PV)->Y,MAX((PV)->Thickness/2.0 + Bloat,0.0), (Arc)))
-
-#define	IS_PV_ON_PAD(PV,Pad) \
-	( pcb_is_point_in_pad((PV)->X, (PV)->Y, MAX((PV)->Thickness/2 +Bloat,0), (Pad)))
-
 /* reduce arc start angle and delta to 0..360 */
 static void normalize_angles(pcb_angle_t * sa, pcb_angle_t * d)
 {
@@ -399,10 +385,10 @@ pcb_bool pcb_intersect_line_line(pcb_line_t *Line1, pcb_line_t *Line2)
 	 *  cases where the "real" lines don't intersect but the
 	 *  thick lines touch, and ensures that the dx/dy business
 	 *  below does not cause a divide-by-zero. */
-	if (pcb_is_point_in_pad(Line2->Point1.X, Line2->Point1.Y, MAX(Line2->Thickness / 2 + Bloat, 0), (pcb_pad_t *) Line1)
-			|| pcb_is_point_in_pad(Line2->Point2.X, Line2->Point2.Y, MAX(Line2->Thickness / 2 + Bloat, 0), (pcb_pad_t *) Line1)
-			|| pcb_is_point_in_pad(Line1->Point1.X, Line1->Point1.Y, MAX(Line1->Thickness / 2 + Bloat, 0), (pcb_pad_t *) Line2)
-			|| pcb_is_point_in_pad(Line1->Point2.X, Line1->Point2.Y, MAX(Line1->Thickness / 2 + Bloat, 0), (pcb_pad_t *) Line2))
+	if (pcb_is_point_in_line(Line2->Point1.X, Line2->Point1.Y, MAX(Line2->Thickness / 2 + Bloat, 0), (pcb_any_line_t *) Line1)
+			|| pcb_is_point_in_line(Line2->Point2.X, Line2->Point2.Y, MAX(Line2->Thickness / 2 + Bloat, 0), (pcb_any_line_t *) Line1)
+			|| pcb_is_point_in_line(Line1->Point1.X, Line1->Point1.Y, MAX(Line1->Thickness / 2 + Bloat, 0), (pcb_any_line_t *) Line2)
+			|| pcb_is_point_in_line(Line1->Point2.X, Line1->Point2.Y, MAX(Line1->Thickness / 2 + Bloat, 0), (pcb_any_line_t *) Line2))
 		return pcb_true;
 
 	/* setup some constants */
@@ -428,7 +414,7 @@ pcb_bool pcb_intersect_line_line(pcb_line_t *Line1, pcb_line_t *Line2)
 
 	/* No cross product means parallel lines, or maybe Line2 is
 	 *  zero-length. In either case, since we did a bounding-box
-	 *  check before getting here, the above pcb_is_point_in_pad() checks
+	 *  check before getting here, the above pcb_is_point_in_line() checks
 	 *  will have caught any intersections. */
 	if (r == 0.0)
 		return pcb_false;
@@ -515,11 +501,11 @@ pcb_bool pcb_intersect_line_arc(pcb_line_t *Line, pcb_arc_t *Arc)
 
 	/* check arc end points */
 	pcb_arc_get_end(Arc, 0, &ex, &ey);
-	if (pcb_is_point_in_pad(ex, ey, Arc->Thickness * 0.5 + Bloat, (pcb_pad_t *) Line))
+	if (pcb_is_point_in_line(ex, ey, Arc->Thickness * 0.5 + Bloat, (pcb_any_line_t *) Line))
 		return pcb_true;
 
 	pcb_arc_get_end(Arc, 1, &ex, &ey);
-	if (pcb_is_point_in_pad(ex, ey, Arc->Thickness * 0.5 + Bloat, (pcb_pad_t *) Line))
+	if (pcb_is_point_in_line(ex, ey, Arc->Thickness * 0.5 + Bloat, (pcb_any_line_t *) Line))
 		return pcb_true;
 	return pcb_false;
 }
@@ -610,16 +596,6 @@ pcb_bool pcb_is_line_in_poly(pcb_line_t *Line, pcb_poly_t *Polygon)
 	return pcb_false;
 }
 
-/* ---------------------------------------------------------------------------
- * checks if a pad connects to a non-clearing polygon
- *
- * The polygon is assumed to already have been proven non-clearing
- */
-pcb_bool pcb_is_pad_in_poly(pcb_pad_t *pad, pcb_poly_t *polygon)
-{
-	return pcb_is_line_in_poly((pcb_line_t *) pad, polygon);
-}
-
 /* ---------------------------------------------------------------------------
  * checks if a polygon has a connection to a second one
  *
@@ -695,20 +671,6 @@ pcb_bool pcb_is_poly_in_poly(pcb_poly_t *P1, pcb_poly_t *P2)
 	return pcb_false;
 }
 
-/* ---------------------------------------------------------------------------
- * some of the 'pad' routines are the same as for lines because the 'pad'
- * struct starts with a line struct. See global.h for details
- */
-pcb_bool pcb_intersect_line_pad(pcb_line_t *Line, pcb_pad_t *Pad)
-{
-	return pcb_intersect_line_line((Line), (pcb_line_t *) Pad);
-}
-
-pcb_bool pcb_intersect_arc_pad(pcb_arc_t *Arc, pcb_pad_t *Pad)
-{
-	return pcb_intersect_line_arc((pcb_line_t *) (Pad), (Arc));
-}
-
 pcb_bool BoxBoxIntersection(pcb_box_t *b1, pcb_box_t *b2)
 {
 	if (b2->X2 < b1->X1 || b2->X1 > b1->X2)
@@ -718,84 +680,6 @@ pcb_bool BoxBoxIntersection(pcb_box_t *b1, pcb_box_t *b2)
 	return pcb_true;
 }
 
-static pcb_bool PadPadIntersect(pcb_pad_t *p1, pcb_pad_t *p2)
-{
-	return pcb_intersect_line_pad((pcb_line_t *) p1, p2);
-}
-
-static inline pcb_bool PV_TOUCH_PV(pcb_pin_t *PV1, pcb_pin_t *PV2)
-{
-	double t1, t2;
-	pcb_box_t b1, b2;
-	int shape1, shape2;
-
-	shape1 = PCB_FLAG_SQUARE_GET(PV1);
-	shape2 = PCB_FLAG_SQUARE_GET(PV2);
-
-	if ((shape1 > 1) || (shape2 > 1)) {
-		pcb_polyarea_t *pl1, *pl2;
-		int ret;
-
-		pl1 = pcb_poly_from_pin(PV1, PIN_SIZE(PV1) + Bloat, 0);
-		pl2 = pcb_poly_from_pin(PV2, PIN_SIZE(PV2) + Bloat, 0);
-		ret = pcb_polyarea_touching(pl1, pl2);
-		pcb_polyarea_free(&pl1);
-		pcb_polyarea_free(&pl2);
-		return ret;
-	}
-
-
-	t1 = MAX(PV1->Thickness / 2.0 + Bloat, 0);
-	t2 = MAX(PV2->Thickness / 2.0 + Bloat, 0);
-	if (pcb_is_point_in_pin(PV1->X, PV1->Y, t1, PV2)
-			|| pcb_is_point_in_pin(PV2->X, PV2->Y, t2, PV1))
-		return pcb_true;
-	if (!PCB_FLAG_TEST(PCB_FLAG_SQUARE, PV1) || !PCB_FLAG_TEST(PCB_FLAG_SQUARE, PV2))
-		return pcb_false;
-	/* check for square/square overlap */
-	b1.X1 = PV1->X - t1;
-	b1.X2 = PV1->X + t1;
-	b1.Y1 = PV1->Y - t1;
-	b1.Y2 = PV1->Y + t1;
-	t2 = PV2->Thickness / 2.0;
-	b2.X1 = PV2->X - t2;
-	b2.X2 = PV2->X + t2;
-	b2.Y1 = PV2->Y - t2;
-	b2.Y2 = PV2->Y + t2;
-	return BoxBoxIntersection(&b1, &b2);
-}
-
-pcb_bool pcb_intersect_line_pin(pcb_pin_t *PV, pcb_line_t *Line)
-{
-	if (PCB_FLAG_TEST(PCB_FLAG_SQUARE, PV)) {
-		int shape = PCB_FLAG_SQUARE_GET(PV);
-		if (shape <= 1) {
-			/* the original square case */
-			/* IsLineInRectangle already has Bloat factor */
-			return pcb_is_line_in_rectangle(PV->X - (PIN_SIZE(PV) + 1) / 2,
-															 PV->Y - (PIN_SIZE(PV) + 1) / 2,
-															 PV->X + (PIN_SIZE(PV) + 1) / 2, PV->Y + (PIN_SIZE(PV) + 1) / 2, Line);
-		}
-
-		{
-			/* shaped pin case */
-			pcb_polyarea_t *pl, *lp;
-			int ret;
-
-			pl = pcb_poly_from_pin(PV, PIN_SIZE(PV), 0);
-			lp = pcb_poly_from_line(Line, Line->Thickness + Bloat);
-			ret = pcb_polyarea_touching(lp, pl);
-			pcb_polyarea_free(&pl);
-			pcb_polyarea_free(&lp);
-			return ret;
-		}
-
-	}
-
-	/* the original round pin version */
-	return pcb_is_point_in_pad(PV->X, PV->Y, MAX(PIN_SIZE(PV) / 2.0 + Bloat, 0.0), (pcb_pad_t *) Line);
-}
-
 /* returns whether a round-cap pcb line touches a polygon; assumes bounding
    boxes do touch */
 PCB_INLINE pcb_bool_t pcb_intersect_line_polyline(pcb_pline_t *pl, pcb_coord_t x1, pcb_coord_t y1, pcb_coord_t x2, pcb_coord_t y2, pcb_coord_t thick)
@@ -853,7 +737,7 @@ PCB_INLINE pcb_bool_t pcb_intersect_line_polyline(pcb_pline_t *pl, pcb_coord_t x
 		pcb_line.Flags = shape_line.square ? pcb_flag_make(PCB_FLAG_SQUARE) : pcb_no_flags(); \
 	} while(0)
 
-PCB_INLINE pcb_bool_t pcb_pstk_intersect_line(pcb_pstk_t *ps, pcb_line_t *line)
+pcb_bool_t pcb_pstk_intersect_line(pcb_pstk_t *ps, pcb_line_t *line)
 {
 	pcb_pstk_shape_t *shape = pcb_pstk_shape_at(PCB, ps, line->parent.layer);
 	if (shape == NULL) return pcb_false;
@@ -870,14 +754,14 @@ PCB_INLINE pcb_bool_t pcb_pstk_intersect_line(pcb_pstk_t *ps, pcb_line_t *line)
 		}
 		case PCB_PSSH_CIRC:
 		{
-			pcb_pad_t tmp;
+			pcb_any_line_t tmp;
 			tmp.Point1.X = line->Point1.X;
 			tmp.Point1.Y = line->Point1.Y;
 			tmp.Point2.X = line->Point2.X;
 			tmp.Point2.Y = line->Point2.Y;
 			tmp.Thickness = line->Thickness;
 			tmp.Flags = pcb_no_flags();
-			return pcb_is_point_in_pad(shape->data.circ.x + ps->x, shape->data.circ.y + ps->y, shape->data.circ.dia/2, &tmp);
+			return pcb_is_point_in_line(shape->data.circ.x + ps->x, shape->data.circ.y + ps->y, shape->data.circ.dia/2, &tmp);
 		}
 	}
 	return pcb_false;
@@ -992,14 +876,14 @@ PCB_INLINE pcb_bool_t pstk_shape_int_circ_poly(pcb_pstk_t *p, pcb_pstk_shape_t *
 
 PCB_INLINE pcb_bool_t pstk_shape_int_circ_line(pcb_pstk_t *l, pcb_pstk_shape_t *sl, pcb_pstk_t *c, pcb_pstk_shape_t *sc)
 {
-	pcb_pad_t tmp;
+	pcb_any_line_t tmp;
 	tmp.Point1.X = sl->data.line.x1 + l->x;
 	tmp.Point1.Y = sl->data.line.y1 + l->y;
 	tmp.Point2.X = sl->data.line.x2 + l->x;
 	tmp.Point2.Y = sl->data.line.y2 + l->y;
 	tmp.Thickness = sl->data.line.thickness;
 	tmp.Flags = pcb_no_flags();
-	return pcb_is_point_in_pad(sc->data.circ.x + c->x, sc->data.circ.y + c->y, sc->data.circ.dia/2, &tmp);
+	return pcb_is_point_in_line(sc->data.circ.x + c->x, sc->data.circ.y + c->y, sc->data.circ.dia/2, &tmp);
 }
 
 PCB_INLINE pcb_bool_t pcb_pstk_shape_intersect(pcb_pstk_t *ps1, pcb_pstk_shape_t *shape1, pcb_pstk_t *ps2, pcb_pstk_shape_t *shape2)
@@ -1066,7 +950,6 @@ PCB_INLINE pcb_bool_t pcb_pstk_shape_intersect(pcb_pstk_t *ps1, pcb_pstk_shape_t
 PCB_INLINE pcb_bool_t pcb_pstk_intersect_pstk(pcb_pstk_t *ps1, pcb_pstk_t *ps2)
 {
 	pcb_layer_t *ly;
-	pcb_data_t *data = ps1->parent.data;
 	int n;
 
 	for(n = 0, ly = PCB->Data->Layer; n < PCB->Data->LayerN; n++,ly++) {
@@ -1090,5 +973,19 @@ PCB_INLINE pcb_bool_t pcb_pstk_intersect_pstk(pcb_pstk_t *ps1, pcb_pstk_t *ps2)
 
 PCB_INLINE pcb_bool_t pcb_pstk_intersect_rat(pcb_pstk_t *ps, pcb_rat_t *rat)
 {
-	return ((rat->Point1.X == ps->x) && (rat->Point1.Y == ps->y)) || ((rat->Point2.X == ps->x) && (rat->Point2.Y == ps->y));
+	pcb_board_t *pcb = PCB;
+
+	if ((rat->Point1.X == ps->x) && (rat->Point1.Y == ps->y)) {
+		pcb_layer_t *layer = pcb_get_layer(pcb->Data, pcb->LayerGroups.grp[rat->group1].lid[0]);
+		if ((layer != NULL) && (pcb_pstk_shape_at(pcb, ps, layer) != NULL))
+			return pcb_true;
+	}
+
+	if ((rat->Point2.X == ps->x) && (rat->Point2.Y == ps->y)) {
+		pcb_layer_t *layer = pcb_get_layer(pcb->Data, pcb->LayerGroups.grp[rat->group2].lid[0]);
+		if ((layer != NULL) && (pcb_pstk_shape_at(pcb, ps, layer) != NULL))
+			return pcb_true;
+	}
+
+	return pcb_false;
 }
