@@ -86,17 +86,17 @@ typedef enum {
 	PCB_FLAG_RUBBEREND    = 0x00200,
 	PCB_FLAG_WARN         = 0x00200,
 	PCB_FLAG_USETHERMAL   = 0x00400,
-	PCB_FLAG_ONSILK       = 0x00400, /*!< Obsolete, old files used this to indicate lines drawn on silk. (Used by io_pcb for compatibility.) */
+	PCB_FLAG_ONSILK       = 0x00400, /* Obsolete, old files used this to indicate lines drawn on silk. (Used by io_pcb for compatibility.) */
 	PCB_FLAG_OCTAGON      = 0x00800,
 	PCB_FLAG_DRC          = 0x01000,
 	PCB_FLAG_LOCK         = 0x02000,
 	PCB_FLAG_EDGE2        = 0x04000,
-	PCB_FLAG_VISIT        = 0x08000, /*!< marker to avoid re-visiting an object */
+	PCB_FLAG_VISIT        = 0x08000, /* marker to avoid re-visiting an object */
 	PCB_FLAG_NONETLIST    = 0x10000,
 	PCB_FLAG_MINCUT       = 0x20000, /* used by the mincut short find code */
-	PCB_FLAG_ONPOINT      = 0x40000, /*!< crosshair is on line point or arc point */
+	PCB_FLAG_ONPOINT      = 0x40000, /* crosshair is on line point or arc point */
 	PCB_FLAG_TERMNAME     = 0x80000, 
-	PCB_FLAG_DRC_INTCONN  = 0x100000,/*!< Set for objects are put on the DRC mark because of an intconn */
+	PCB_FLAG_DRC_INTCONN  = 0x100000,/* Set for objects are put on the DRC mark because of an intconn */
 	PCB_FLAG_CLEARPOLYPOLY= 0x200000,
 	PCB_FLAG_DYNTEXT      = 0x400000,
 	PCB_FLAG_FLOATER      = 0x800000
