@@ -45,11 +45,13 @@ action_entry(LayoutToBuffer)
 action_entry(Line)
 action_entry(LineSize)
 action_entry(Lock)
+action_entry(Loose)
 action_entry(Mirror)
 action_entry(Move)
 action_entry(MoveSelected)
 action_entry(Netlist)
 action_entry(None)
+action_entry(Normalize)
 action_entry(Notify)
 action_entry(Object)
 action_entry(PasteBuffer)
@@ -57,7 +59,9 @@ action_entry(PinOrPadName)
 action_entry(Pinout)
 action_entry(Polygon)
 action_entry(PolygonHole)
+action_entry(Pop)
 action_entry(PreviousPoint)
+action_entry(Push)
 action_entry(RatsNest)
 action_entry(Rectangle)
 action_entry(Redraw)
@@ -119,7 +123,5 @@ action_entry(ToggleCheckPlanes)
 action_entry(ToggleUniqueNames)
 action_entry(ToggleStroke)
 action_entry(Via)
-action_entry(ViaDrillingHole)
-action_entry(ViaSize)
 action_entry(Zoom)
 
