@@ -46,13 +46,10 @@ typedef struct pcb_point_s pcb_point_t;
 typedef struct pcb_rat_line_s pcb_rat_t;
 
 typedef struct pcb_poly_s pcb_poly_t;
-typedef struct pcb_pad_s pcb_pad_t;
 typedef struct pcb_pstk_s pcb_pstk_t;
-typedef struct pcb_pin_s pcb_pin_t;
 typedef struct pcb_rtree_s pcb_rtree_t;
 typedef struct pcb_rtree_it_s pcb_rtree_it_t;
 typedef struct pcb_ratspatch_line_s pcb_ratspatch_line_t;
-typedef struct pcb_element_s pcb_element_t;
 typedef struct pcb_subc_s pcb_subc_t;
 typedef struct pcb_text_s pcb_text_t;
 
@@ -63,6 +60,8 @@ typedef union pcb_parent_s pcb_parent_t;
 
 typedef struct pcb_plug_io_s pcb_plug_io_t;
 
+typedef struct pcb_hid_cfg_s pcb_hid_cfg_t;
+
 typedef unsigned int pcb_cardinal_t;
 typedef unsigned char pcb_uint8_t;   /* Don't use in new code. */
 
