@@ -38,6 +38,7 @@
 #include "data.h"
 #include "action_helper.h"
 #include "tool.h"
+#include "grid.h"
 #include "error.h"
 #include "undo.h"
 #include "funchash_core.h"
@@ -53,6 +54,7 @@
 #include "compat_misc.h"
 #include "event.h"
 #include "layer.h"
+#include "layer_ui.h"
 #include "layer_grp.h"
 #include "layer_vis.h"
 #include "attrib.h"
@@ -60,11 +62,7 @@
 #include "operation.h"
 #include "obj_subc_op.h"
 
-#include "obj_elem_draw.h"
-#include "obj_pinvia_draw.h"
-#include "obj_pad_draw.h"
-
-#define CLONE_TYPES PCB_TYPE_LINE | PCB_TYPE_ARC | PCB_TYPE_VIA | PCB_TYPE_POLY
+#define CLONE_TYPES PCB_OBJ_LINE | PCB_OBJ_ARC | PCB_OBJ_POLY
 
 /* --------------------------------------------------------------------------- */
 /* Toggle actions are kept for compatibility; new code should use the conf system instead */
@@ -159,7 +157,7 @@ this allows you to check for isolated regions.
 
 @item ToggleOrthoMove
 If set, the crosshair is only allowed to move orthogonally from its
-previous position.  I.e. you can move an element or line up, down,
+previous position.  I.e. you can move a subcircuit or line up, down,
 left, or right, but not up+left or down+right.
 
 @item ToggleName
@@ -194,12 +192,12 @@ coordinate.
 Toggles whether the grid is displayed or not.
 
 @item Pinout
-Causes the pinout of the element indicated by the cursor to be
+Causes the pinout of the subcircuit indicated by the cursor to be
 displayed, usually in a separate window.
 
 @item PinOrPadName
-Toggles whether the names of pins, pads, or (yes) vias will be
-displayed.  If the cursor is over an element, all of its pins and pads
+Toggles whether the names of terminals will be
+displayed.  If the cursor is over an subcircuit, all of its terminals
 are affected.
 
 @end table
@@ -236,7 +234,7 @@ static int pcb_act_Display(int argc, const char **argv, pcb_coord_t childX, pcb_
 	str_dir = PCB_ACTION_ARG(1);
 
 	id = pcb_funchash_get(function, NULL);
-	if (id == F_SubcID) { /* change the displayed name of elements */
+	if (id == F_SubcID) { /* change the displayed name of subcircuits */
 		if (argc > 0)
 			conf_set(CFR_DESIGN, "editor/subc_id", -1, str_dir, POL_OVERWRITE);
 		else
@@ -296,7 +294,7 @@ static int pcb_act_Display(int argc, const char **argv, pcb_coord_t childX, pcb_
 			break;
 
 		case F_ToggleUniqueNames:
-			conf_toggle_editor(unique_names);
+			pcb_message(PCB_MSG_ERROR, "Unique names/refdes is not supported any more - please use the renumber plugin\n");
 			break;
 
 		case F_ToggleSnapPin:
@@ -402,7 +400,8 @@ static int pcb_act_Display(int argc, const char **argv, pcb_coord_t childX, pcb_
 				PCB->Grid = 1;
 				if (pcb_crosshair_move_absolute(pcb_crosshair.X, pcb_crosshair.Y))
 					pcb_notify_crosshair_change(pcb_true);	/* first notify was in MoveCrosshairAbs */
-				pcb_board_set_grid(oldGrid, pcb_true);
+				pcb_board_set_grid(oldGrid, pcb_true, pcb_crosshair.X, pcb_crosshair.Y);
+				pcb_grid_inval();
 			}
 			break;
 
@@ -412,116 +411,49 @@ static int pcb_act_Display(int argc, const char **argv, pcb_coord_t childX, pcb_
 			pcb_redraw();
 			break;
 
-			/* display the pinout of an element */
+			/* display the pinout of a subcircuit */
 		case F_Pinout:
 			{
 				pcb_subc_t *subc;
 				void *ptrtmp;
 				pcb_coord_t x, y;
 
-				pcb_gui->get_coords(_("Click on an element"), &x, &y);
-				if ((pcb_search_screen(x, y, PCB_TYPE_SUBC, &ptrtmp, &ptrtmp, &ptrtmp)) != PCB_TYPE_NONE) {
+				pcb_gui->get_coords(_("Click on a subcircuit"), &x, &y);
+				if ((pcb_search_screen(x, y, PCB_OBJ_SUBC, &ptrtmp, &ptrtmp, &ptrtmp)) != PCB_OBJ_VOID) {
 					subc = (pcb_subc_t *) ptrtmp;
 					pcb_gui->show_item(subc);
 				}
 				break;
 			}
 
-			/* toggle displaying of pin/pad/via names */
+			/* toggle displaying of terminal names */
 		case F_PinOrPadName:
 			{
 				int type;
 				void *ptr1, *ptr2, *ptr3;
 				pcb_coord_t x, y;
-				pcb_gui->get_coords(_("Click on an element"), &x, &y);
-
-				/* old code for elements */
-				switch (pcb_search_screen(x, y,
-														 PCB_TYPE_ELEMENT | PCB_TYPE_PIN | PCB_TYPE_PAD |
-														 PCB_TYPE_VIA, (void **) &ptr1, (void **) &ptr2, (void **) &ptr3)) {
-				case PCB_TYPE_ELEMENT:
-					PCB_PIN_LOOP((pcb_element_t *) ptr1);
-					{
-						if (PCB_FLAG_TEST(PCB_FLAG_TERMNAME, pin))
-							pcb_pin_name_invalidate_erase(pin);
-						else
-							pcb_pin_name_invalidate_draw(pin);
-						pcb_undo_add_obj_to_flag(pin);
-						PCB_FLAG_TOGGLE(PCB_FLAG_TERMNAME, pin);
-					}
-					PCB_END_LOOP;
-					PCB_PAD_LOOP((pcb_element_t *) ptr1);
-					{
-						if (PCB_FLAG_TEST(PCB_FLAG_TERMNAME, pad))
-							pcb_pad_name_invalidate_erase(pad);
-						else
-							pcb_pad_name_invalidate_draw(pad);
-						pcb_undo_add_obj_to_flag(pad);
-						PCB_FLAG_TOGGLE(PCB_FLAG_TERMNAME, pad);
-					}
-					PCB_END_LOOP;
-					pcb_board_set_changed_flag(pcb_true);
-					pcb_undo_inc_serial();
-					pcb_draw();
-					break;
-
-				case PCB_TYPE_PIN:
-					if (PCB_FLAG_TEST(PCB_FLAG_TERMNAME, (pcb_pin_t *) ptr2))
-						pcb_pin_name_invalidate_erase((pcb_pin_t *) ptr2);
-					else
-						pcb_pin_name_invalidate_draw((pcb_pin_t *) ptr2);
-					pcb_undo_add_obj_to_flag(ptr2);
-					PCB_FLAG_TOGGLE(PCB_FLAG_TERMNAME, (pcb_pin_t *) ptr2);
-					pcb_board_set_changed_flag(pcb_true);
-					pcb_undo_inc_serial();
-					pcb_draw();
-					break;
-
-				case PCB_TYPE_PAD:
-					if (PCB_FLAG_TEST(PCB_FLAG_TERMNAME, (pcb_pad_t *) ptr2))
-						pcb_pad_name_invalidate_erase((pcb_pad_t *) ptr2);
-					else
-						pcb_pad_name_invalidate_draw((pcb_pad_t *) ptr2);
-					pcb_undo_add_obj_to_flag(ptr2);
-					PCB_FLAG_TOGGLE(PCB_FLAG_TERMNAME, (pcb_pad_t *) ptr2);
-					pcb_board_set_changed_flag(pcb_true);
-					pcb_undo_inc_serial();
-					pcb_draw();
-					break;
-				case PCB_TYPE_VIA:
-					if (PCB_FLAG_TEST(PCB_FLAG_TERMNAME, (pcb_pin_t *) ptr2))
-						pcb_via_name_invalidate_erase((pcb_pin_t *) ptr2);
-					else
-						pcb_via_name_invalidate_draw((pcb_pin_t *) ptr2);
-					pcb_undo_add_obj_to_flag(ptr2);
-					PCB_FLAG_TOGGLE(PCB_FLAG_TERMNAME, (pcb_pin_t *) ptr2);
-					pcb_board_set_changed_flag(pcb_true);
-					pcb_undo_inc_serial();
-					pcb_draw();
-					break;
-				}
+				pcb_gui->get_coords(_("Click on a subcircuit"), &x, &y);
 
 				/* toggle terminal ID print for subcircuit parts */
-				type = pcb_search_screen(x, y, PCB_TYPE_SUBC | PCB_TYPE_SUBC_PART | PCB_TYPE_VIA | PCB_TYPE_PSTK | PCB_TYPE_LINE | PCB_TYPE_ARC | PCB_TYPE_POLY | PCB_TYPE_TEXT, (void **)&ptr1, (void **)&ptr2, (void **)&ptr3);
+				type = pcb_search_screen(x, y, PCB_OBJ_SUBC | PCB_OBJ_SUBC_PART | PCB_OBJ_PSTK | PCB_OBJ_LINE | PCB_OBJ_ARC | PCB_OBJ_POLY | PCB_OBJ_TEXT, (void **)&ptr1, (void **)&ptr2, (void **)&ptr3);
 				if (type) {
 					pcb_any_obj_t *obj = ptr2;
 					pcb_opctx_t opctx;
 					
 					switch(type) {
-						case PCB_TYPE_SUBC:
+						case PCB_OBJ_SUBC:
 							opctx.chgflag.pcb = PCB;
 							opctx.chgflag.how = PCB_CHGFLG_TOGGLE;
 							opctx.chgflag.flag = PCB_FLAG_TERMNAME;
-							pcb_subc_op(PCB->Data, (pcb_subc_t *)obj, &ChgFlagFunctions, &opctx);
+							pcb_subc_op(PCB->Data, (pcb_subc_t *)obj, &ChgFlagFunctions, &opctx, 0);
 							pcb_undo_inc_serial();
 							return 0;
 							break;
-						case PCB_TYPE_VIA:
-						case PCB_TYPE_LINE:
-						case PCB_TYPE_ARC:
-						case PCB_TYPE_POLY:
-						case PCB_TYPE_TEXT:
-						case PCB_TYPE_PSTK:
+						case PCB_OBJ_LINE:
+						case PCB_OBJ_ARC:
+						case PCB_OBJ_POLY:
+						case PCB_OBJ_TEXT:
+						case PCB_OBJ_PSTK:
 							pcb_obj_invalidate_label(type, ptr1, ptr2, ptr3);
 							break;
 						default:
@@ -633,34 +565,34 @@ static int pcb_act_Mode(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 		pcb_notify_crosshair_change(pcb_false);
 		switch (pcb_funchash_get(function, NULL)) {
 		case F_Arc:
-			pcb_crosshair_set_mode(PCB_MODE_ARC);
+			pcb_tool_select_by_id(PCB_MODE_ARC);
 			break;
 		case F_Arrow:
-			pcb_crosshair_set_mode(PCB_MODE_ARROW);
+			pcb_tool_select_by_id(PCB_MODE_ARROW);
 			break;
 		case F_Copy:
-			pcb_crosshair_set_mode(PCB_MODE_COPY);
+			pcb_tool_select_by_id(PCB_MODE_COPY);
 			break;
 		case F_InsertPoint:
-			pcb_crosshair_set_mode(PCB_MODE_INSERT_POINT);
+			pcb_tool_select_by_id(PCB_MODE_INSERT_POINT);
 			break;
 		case F_Line:
-			pcb_crosshair_set_mode(PCB_MODE_LINE);
+			pcb_tool_select_by_id(PCB_MODE_LINE);
 			break;
 		case F_Lock:
-			pcb_crosshair_set_mode(PCB_MODE_LOCK);
+			pcb_tool_select_by_id(PCB_MODE_LOCK);
 			break;
 		case F_Move:
-			pcb_crosshair_set_mode(PCB_MODE_MOVE);
+			pcb_tool_select_by_id(PCB_MODE_MOVE);
 			break;
 		case F_None:
-			pcb_crosshair_set_mode(PCB_MODE_NO);
+			pcb_tool_select_by_id(PCB_MODE_NO);
 			break;
 		case F_Cancel:
 			{
 				int saved_mode = conf_core.editor.mode;
-				pcb_crosshair_set_mode(PCB_MODE_NO);
-				pcb_crosshair_set_mode(saved_mode);
+				pcb_tool_select_by_id(PCB_MODE_NO);
+				pcb_tool_select_by_id(saved_mode);
 			}
 			break;
 		case F_Escape:
@@ -677,53 +609,53 @@ static int pcb_act_Mode(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 				case PCB_MODE_RUBBERBAND_MOVE:
 				case PCB_MODE_THERMAL:
 				case PCB_MODE_LOCK:
-					pcb_crosshair_set_mode(PCB_MODE_NO);
-					pcb_crosshair_set_mode(PCB_MODE_ARROW);
+					pcb_tool_select_by_id(PCB_MODE_NO);
+					pcb_tool_select_by_id(PCB_MODE_ARROW);
 					pcb_tool_note.Hit = pcb_tool_note.Click = 0; /* if the mouse button is still pressed, don't start selecting a box */
 					break;
 
 				case PCB_MODE_LINE:
 					if (pcb_crosshair.AttachedLine.State == PCB_CH_STATE_FIRST)
-						pcb_crosshair_set_mode(PCB_MODE_ARROW);
+						pcb_tool_select_by_id(PCB_MODE_ARROW);
 					else {
-						pcb_crosshair_set_mode(PCB_MODE_NO);
-						pcb_crosshair_set_mode(PCB_MODE_LINE);
+						pcb_tool_select_by_id(PCB_MODE_NO);
+						pcb_tool_select_by_id(PCB_MODE_LINE);
 					}
 					break;
 
 				case PCB_MODE_RECTANGLE:
 					if (pcb_crosshair.AttachedBox.State == PCB_CH_STATE_FIRST)
-						pcb_crosshair_set_mode(PCB_MODE_ARROW);
+						pcb_tool_select_by_id(PCB_MODE_ARROW);
 					else {
-						pcb_crosshair_set_mode(PCB_MODE_NO);
-						pcb_crosshair_set_mode(PCB_MODE_RECTANGLE);
+						pcb_tool_select_by_id(PCB_MODE_NO);
+						pcb_tool_select_by_id(PCB_MODE_RECTANGLE);
 					}
 					break;
 
 				case PCB_MODE_POLYGON:
 					if (pcb_crosshair.AttachedLine.State == PCB_CH_STATE_FIRST)
-						pcb_crosshair_set_mode(PCB_MODE_ARROW);
+						pcb_tool_select_by_id(PCB_MODE_ARROW);
 					else {
-						pcb_crosshair_set_mode(PCB_MODE_NO);
-						pcb_crosshair_set_mode(PCB_MODE_POLYGON);
+						pcb_tool_select_by_id(PCB_MODE_NO);
+						pcb_tool_select_by_id(PCB_MODE_POLYGON);
 					}
 					break;
 
 				case PCB_MODE_POLYGON_HOLE:
 					if (pcb_crosshair.AttachedLine.State == PCB_CH_STATE_FIRST)
-						pcb_crosshair_set_mode(PCB_MODE_ARROW);
+						pcb_tool_select_by_id(PCB_MODE_ARROW);
 					else {
-						pcb_crosshair_set_mode(PCB_MODE_NO);
-						pcb_crosshair_set_mode(PCB_MODE_POLYGON_HOLE);
+						pcb_tool_select_by_id(PCB_MODE_NO);
+						pcb_tool_select_by_id(PCB_MODE_POLYGON_HOLE);
 					}
 					break;
 
 				case PCB_MODE_ARC:
 					if (pcb_crosshair.AttachedBox.State == PCB_CH_STATE_FIRST)
-						pcb_crosshair_set_mode(PCB_MODE_ARROW);
+						pcb_tool_select_by_id(PCB_MODE_ARROW);
 					else {
-						pcb_crosshair_set_mode(PCB_MODE_NO);
-						pcb_crosshair_set_mode(PCB_MODE_ARC);
+						pcb_tool_select_by_id(PCB_MODE_NO);
+						pcb_tool_select_by_id(PCB_MODE_ARC);
 					}
 					break;
 
@@ -740,28 +672,32 @@ static int pcb_act_Mode(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 			pcb_notify_mode();
 			break;
 		case F_PasteBuffer:
-			pcb_crosshair_set_mode(PCB_MODE_PASTE_BUFFER);
+			pcb_tool_select_by_id(PCB_MODE_PASTE_BUFFER);
 			break;
 		case F_Polygon:
-			pcb_crosshair_set_mode(PCB_MODE_POLYGON);
+			pcb_tool_select_by_id(PCB_MODE_POLYGON);
 			break;
 		case F_PolygonHole:
-			pcb_crosshair_set_mode(PCB_MODE_POLYGON_HOLE);
+			pcb_tool_select_by_id(PCB_MODE_POLYGON_HOLE);
 			break;
 		case F_Release:
-			if ((pcb_mid_stroke) && (conf_core.editor.enable_stroke))
-				pcb_stub_stroke_finish();
-			else
+			if ((pcb_mid_stroke) && (conf_core.editor.enable_stroke) && (pcb_stub_stroke_finish() == 0)) {
+				/* Ugly hack: returning 1 here will break execution of the
+				   action script, so actions after this one could do things
+				   that would be executed only after non-recognized gestures */
 				pcb_release_mode();
+				return 1;
+			}
+			pcb_release_mode();
 			break;
 		case F_Remove:
-			pcb_crosshair_set_mode(PCB_MODE_REMOVE);
+			pcb_tool_select_by_id(PCB_MODE_REMOVE);
 			break;
 		case F_Rectangle:
-			pcb_crosshair_set_mode(PCB_MODE_RECTANGLE);
+			pcb_tool_select_by_id(PCB_MODE_RECTANGLE);
 			break;
 		case F_Rotate:
-			pcb_crosshair_set_mode(PCB_MODE_ROTATE);
+			pcb_tool_select_by_id(PCB_MODE_ROTATE);
 			break;
 		case F_Stroke:
 			if (conf_core.editor.enable_stroke) {
@@ -772,37 +708,37 @@ static int pcb_act_Mode(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 			   |  a drawing mode, middle mouse button will select objects.
 			 */
 			if (conf_core.editor.mode == PCB_MODE_LINE && pcb_crosshair.AttachedLine.State != PCB_CH_STATE_FIRST) {
-				pcb_crosshair_set_mode(PCB_MODE_LINE);
+				pcb_tool_select_by_id(PCB_MODE_LINE);
 			}
 			else if (conf_core.editor.mode == PCB_MODE_ARC && pcb_crosshair.AttachedBox.State != PCB_CH_STATE_FIRST)
-				pcb_crosshair_set_mode(PCB_MODE_ARC);
+				pcb_tool_select_by_id(PCB_MODE_ARC);
 			else if (conf_core.editor.mode == PCB_MODE_RECTANGLE && pcb_crosshair.AttachedBox.State != PCB_CH_STATE_FIRST)
-				pcb_crosshair_set_mode(PCB_MODE_RECTANGLE);
+				pcb_tool_select_by_id(PCB_MODE_RECTANGLE);
 			else if (conf_core.editor.mode == PCB_MODE_POLYGON && pcb_crosshair.AttachedLine.State != PCB_CH_STATE_FIRST)
-				pcb_crosshair_set_mode(PCB_MODE_POLYGON);
+				pcb_tool_select_by_id(PCB_MODE_POLYGON);
 			else {
-				pcb_crosshair_save_mode();
+				pcb_tool_save();
 				pcb_tool_is_saved = pcb_true;
-				pcb_crosshair_set_mode(PCB_MODE_ARROW);
+				pcb_tool_select_by_id(PCB_MODE_ARROW);
 				pcb_notify_mode();
 			}
 			break;
 		case F_Text:
-			pcb_crosshair_set_mode(PCB_MODE_TEXT);
+			pcb_tool_select_by_id(PCB_MODE_TEXT);
 			break;
 		case F_Thermal:
-			pcb_crosshair_set_mode(PCB_MODE_THERMAL);
+			pcb_tool_select_by_id(PCB_MODE_THERMAL);
 			break;
 		case F_Via:
-			pcb_crosshair_set_mode(PCB_MODE_VIA);
+			pcb_tool_select_by_id(PCB_MODE_VIA);
 			break;
 
-		case F_Restore:						/* restore the last saved mode */
-			pcb_crosshair_restore_mode();
+		case F_Restore:						/* restore the last saved tool */
+			pcb_tool_restore();
 			break;
 
-		case F_Save:								/* save currently selected mode */
-			pcb_crosshair_save_mode();
+		case F_Save:								/* save currently selected tool */
+			pcb_tool_save();
 			break;
 		}
 		pcb_notify_crosshair_change(pcb_true);
@@ -833,29 +769,29 @@ static int pcb_act_CycleDrag(int argc, const char **argv, pcb_coord_t x, pcb_coo
 			over++;
 		}
 
-		if (pcb_search_obj_by_id(PCB->Data, &ptr1, &ptr2, &ptr3, pcb_crosshair.drags[pcb_crosshair.drags_current], PCB_TYPE_LINE) != PCB_TYPE_NONE) {
+		if (pcb_search_obj_by_id(PCB->Data, &ptr1, &ptr2, &ptr3, pcb_crosshair.drags[pcb_crosshair.drags_current], PCB_OBJ_LINE) != PCB_OBJ_VOID) {
 			/* line has two endpoints, check which one is close to the original x;y */
 			pcb_line_t *l = ptr2;
 			if (close_enough(pcb_tool_note.X, l->Point1.X) && close_enough(pcb_tool_note.Y, l->Point1.Y)) {
-				pcb_crosshair.AttachedObject.Type = PCB_TYPE_LINE_POINT;
+				pcb_crosshair.AttachedObject.Type = PCB_OBJ_LINE_POINT;
 				pcb_crosshair.AttachedObject.Ptr1 = ptr1;
 				pcb_crosshair.AttachedObject.Ptr2 = ptr2;
 				pcb_crosshair.AttachedObject.Ptr3 = &l->Point1;
 				goto switched;
 			}
 			if (close_enough(pcb_tool_note.X, l->Point2.X) && close_enough(pcb_tool_note.Y, l->Point2.Y)) {
-				pcb_crosshair.AttachedObject.Type = PCB_TYPE_LINE_POINT;
+				pcb_crosshair.AttachedObject.Type = PCB_OBJ_LINE_POINT;
 				pcb_crosshair.AttachedObject.Ptr1 = ptr1;
 				pcb_crosshair.AttachedObject.Ptr2 = ptr2;
 				pcb_crosshair.AttachedObject.Ptr3 = &l->Point2;
 				goto switched;
 			}
 		}
-		else if (pcb_search_obj_by_id(PCB->Data, &ptr1, &ptr2, &ptr3, pcb_crosshair.drags[pcb_crosshair.drags_current], PCB_TYPE_ARC_POINT) != PCB_TYPE_NONE) {
+		else if (pcb_search_obj_by_id(PCB->Data, &ptr1, &ptr2, &ptr3, pcb_crosshair.drags[pcb_crosshair.drags_current], PCB_OBJ_ARC_POINT) != PCB_OBJ_VOID) {
 			pcb_coord_t ex, ey;
 			pcb_arc_get_end((pcb_arc_t *)ptr2, 0, &ex, &ey);
 			if (close_enough(pcb_tool_note.X, ex) && close_enough(pcb_tool_note.Y, ey)) {
-				pcb_crosshair.AttachedObject.Type = PCB_TYPE_ARC_POINT;
+				pcb_crosshair.AttachedObject.Type = PCB_OBJ_ARC_POINT;
 				pcb_crosshair.AttachedObject.Ptr1 = ptr1;
 				pcb_crosshair.AttachedObject.Ptr2 = ptr2;
 				pcb_crosshair.AttachedObject.Ptr3 = pcb_arc_start_ptr;
@@ -863,29 +799,15 @@ static int pcb_act_CycleDrag(int argc, const char **argv, pcb_coord_t x, pcb_coo
 			}
 			pcb_arc_get_end((pcb_arc_t *)ptr2, 1, &ex, &ey);
 			if (close_enough(pcb_tool_note.X, ex) && close_enough(pcb_tool_note.Y, ey)) {
-				pcb_crosshair.AttachedObject.Type = PCB_TYPE_ARC_POINT;
+				pcb_crosshair.AttachedObject.Type = PCB_OBJ_ARC_POINT;
 				pcb_crosshair.AttachedObject.Ptr1 = ptr1;
 				pcb_crosshair.AttachedObject.Ptr2 = ptr2;
 				pcb_crosshair.AttachedObject.Ptr3 = pcb_arc_end_ptr;
 				goto switched;
 			}
 		}
-		else if (pcb_search_obj_by_id(PCB->Data, &ptr1, &ptr2, &ptr3, pcb_crosshair.drags[pcb_crosshair.drags_current], PCB_TYPE_VIA) != PCB_TYPE_NONE) {
-			pcb_crosshair.AttachedObject.Type = PCB_TYPE_VIA;
-			pcb_crosshair.AttachedObject.Ptr1 = ptr1;
-			pcb_crosshair.AttachedObject.Ptr2 = ptr2;
-			pcb_crosshair.AttachedObject.Ptr3 = ptr3;
-			goto switched;
-		}
-		else if (pcb_search_obj_by_id(PCB->Data, &ptr1, &ptr2, &ptr3, pcb_crosshair.drags[pcb_crosshair.drags_current], PCB_TYPE_PAD) != PCB_TYPE_NONE) {
-			pcb_crosshair.AttachedObject.Type = PCB_TYPE_ELEMENT;
-			pcb_crosshair.AttachedObject.Ptr1 = ptr1;
-			pcb_crosshair.AttachedObject.Ptr2 = ptr1;
-			pcb_crosshair.AttachedObject.Ptr3 = ptr1;
-			goto switched;
-		}
-		else if (pcb_search_obj_by_id(PCB->Data, &ptr1, &ptr2, &ptr3, pcb_crosshair.drags[pcb_crosshair.drags_current], PCB_TYPE_ARC) != PCB_TYPE_NONE) {
-			pcb_crosshair.AttachedObject.Type = PCB_TYPE_ARC;
+		else if (pcb_search_obj_by_id(PCB->Data, &ptr1, &ptr2, &ptr3, pcb_crosshair.drags[pcb_crosshair.drags_current], PCB_OBJ_ARC) != PCB_OBJ_VOID) {
+			pcb_crosshair.AttachedObject.Type = PCB_OBJ_ARC;
 			pcb_crosshair.AttachedObject.Ptr1 = ptr1;
 			pcb_crosshair.AttachedObject.Ptr2 = ptr2;
 			pcb_crosshair.AttachedObject.Ptr3 = ptr3;
@@ -937,7 +859,7 @@ static int pcb_act_Message(int argc, const char **argv, pcb_coord_t x, pcb_coord
 
 static const char pcb_acts_ToggleHideName[] = "ToggleHideName(Object|SelectedElements)";
 
-static const char pcb_acth_ToggleHideName[] = "Toggles the visibility of element names.";
+static const char pcb_acth_ToggleHideName[] = "Toggles the visibility of subcircuit names.";
 
 /* %start-doc actions ToggleHideName
 
@@ -948,48 +870,7 @@ appear on the silk layer when you print the layout.
 
 static int pcb_act_ToggleHideName(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
 {
-	const char *function = PCB_ACTION_ARG(0);
-	if (function && pcb_silk_on(PCB)) {
-		switch (pcb_funchash_get(function, NULL)) {
-		case F_Object:
-			{
-				int type;
-				void *ptr1, *ptr2, *ptr3;
-
-				pcb_gui->get_coords(_("Select an Object"), &x, &y);
-				if ((type = pcb_search_screen(x, y, PCB_TYPE_ELEMENT, &ptr1, &ptr2, &ptr3)) != PCB_TYPE_NONE) {
-					pcb_undo_add_obj_to_flag(ptr2);
-					pcb_elem_name_invalidate_erase((pcb_element_t *) ptr2);
-					PCB_FLAG_TOGGLE(PCB_FLAG_HIDENAME, (pcb_element_t *) ptr2);
-					pcb_elem_name_invalidate_draw((pcb_element_t *) ptr2);
-					pcb_draw();
-					pcb_undo_inc_serial();
-				}
-				break;
-			}
-		case F_SelectedElements:
-		case F_Selected:
-			{
-				pcb_bool changed = pcb_false;
-				PCB_ELEMENT_LOOP(PCB->Data);
-				{
-					if ((PCB_FLAG_TEST(PCB_FLAG_SELECTED, element) || PCB_FLAG_TEST(PCB_FLAG_SELECTED, &PCB_ELEM_TEXT_REFDES(element)))
-							&& (PCB_FRONT(element) || PCB->InvisibleObjectsOn)) {
-						pcb_undo_add_obj_to_flag(element);
-						pcb_elem_name_invalidate_erase(element);
-						PCB_FLAG_TOGGLE(PCB_FLAG_HIDENAME, element);
-						pcb_elem_name_invalidate_draw(element);
-						changed = pcb_true;
-					}
-				}
-				PCB_END_LOOP;
-				if (changed) {
-					pcb_draw();
-					pcb_undo_inc_serial();
-				}
-			}
-		}
-	}
+	pcb_message(PCB_MSG_ERROR, "ToggleHideName: deprecated feature removed with subcircuits; just delete\nthe text object if it should not be on the silk of the final board.\n");
 	return 0;
 }
 
@@ -1086,7 +967,6 @@ static int pcb_act_RouteStyle(int argc, const char **argv, pcb_coord_t x, pcb_co
 			pcb_board_set_via_size(rts->Diameter, pcb_true);
 			pcb_board_set_via_drilling_hole(rts->Hole, pcb_true);
 			pcb_board_set_clearance(rts->Clearance);
-			pcb_event(PCB_EVENT_ROUTE_STYLES_CHANGED, NULL);
 		}
 		else
 			pcb_message(PCB_MSG_ERROR, "Error: invalid route style name or index\n");
@@ -1111,7 +991,14 @@ static int pcb_act_CreateMenu(int argc, const char **argv, pcb_coord_t x, pcb_co
 	}
 
 	if (argc > 0) {
-		pcb_gui->create_menu(argv[0], (argc > 1) ? argv[1] : NULL, (argc > 2) ? argv[2] : NULL, (argc > 3) ? argv[3] : NULL, (argc > 4) ? argv[4] : NULL, (argc > 5) ? argv[5] : NULL);
+		pcb_menu_prop_t props;
+		memset(&props, 0, sizeof(props));
+		props.action = (argc > 1) ? argv[1] : NULL;
+		props.mnemonic = (argc > 2) ? argv[2] : NULL;
+		props.accel = (argc > 3) ? argv[3] : NULL;
+		props.tip = (argc > 4) ? argv[4] : NULL;
+		props.cookie = (argc > 5) ? argv[5] : NULL;
+		pcb_gui->create_menu(argv[0], &props);
 		return 0;
 	}
 
@@ -1186,39 +1073,30 @@ static int pcb_act_SetSame(int argc, const char **argv, pcb_coord_t x, pcb_coord
 	type = pcb_search_screen(x, y, CLONE_TYPES, &ptr1, &ptr2, &ptr3);
 /* set layer current and size from line or arc */
 	switch (type) {
-	case PCB_TYPE_LINE:
+	case PCB_OBJ_LINE:
 		pcb_notify_crosshair_change(pcb_false);
 		set_same_(((pcb_line_t *) ptr2)->Thickness, -1, -1, ((pcb_line_t *) ptr2)->Clearance / 2, NULL);
 		layer = (pcb_layer_t *) ptr1;
 		if (conf_core.editor.mode != PCB_MODE_LINE)
-			pcb_crosshair_set_mode(PCB_MODE_LINE);
+			pcb_tool_select_by_id(PCB_MODE_LINE);
 		pcb_notify_crosshair_change(pcb_true);
 		pcb_event(PCB_EVENT_ROUTE_STYLES_CHANGED, NULL);
 		break;
 
-	case PCB_TYPE_ARC:
+	case PCB_OBJ_ARC:
 		pcb_notify_crosshair_change(pcb_false);
 		set_same_(((pcb_arc_t *) ptr2)->Thickness, -1, -1, ((pcb_arc_t *) ptr2)->Clearance / 2, NULL);
 		layer = (pcb_layer_t *) ptr1;
 		if (conf_core.editor.mode != PCB_MODE_ARC)
-			pcb_crosshair_set_mode(PCB_MODE_ARC);
+			pcb_tool_select_by_id(PCB_MODE_ARC);
 		pcb_notify_crosshair_change(pcb_true);
 		pcb_event(PCB_EVENT_ROUTE_STYLES_CHANGED, NULL);
 		break;
 
-	case PCB_TYPE_POLY:
+	case PCB_OBJ_POLY:
 		layer = (pcb_layer_t *) ptr1;
 		break;
 
-	case PCB_TYPE_VIA:
-		pcb_notify_crosshair_change(pcb_false);
-		set_same_(0, ((pcb_pin_t *) ptr2)->Thickness, ((pcb_pin_t *) ptr2)->DrillingHole, ((pcb_pin_t *) ptr2)->Clearance / 2, NULL);
-		if (conf_core.editor.mode != PCB_MODE_VIA)
-			pcb_crosshair_set_mode(PCB_MODE_VIA);
-		pcb_notify_crosshair_change(pcb_true);
-		pcb_event(PCB_EVENT_ROUTE_STYLES_CHANGED, NULL);
-		break;
-
 	default:
 		return 1;
 	}
@@ -1403,51 +1281,17 @@ static int pcb_act_Cursor(int argc, const char **argv, pcb_coord_t x, pcb_coord_
 		dy = -dy;
 	
 	/* Allow leaving snapped pin/pad/padstack */
-	if (pcb_crosshair.snapped_pad) {
-		pcb_pad_t *pad = pcb_crosshair.snapped_pad;
-		pcb_coord_t width = pad->Thickness;
-		pcb_coord_t height = pcb_distance(pad->Point1.X, pad->Point1.Y, pad->Point2.X, pad->Point2.Y)+width;
-		if (pad->Point1.Y == pad->Point2.Y) {
-			pcb_coord_t tmp = width;
-			width = height;
-			height = tmp;
-		}
-		if (dx < 0) {
-			dx -= width/2;
-		} else if (dx > 0) {
-			dx += width/2;
-		}
-		if (dy < 0) {
-			dy -= height/2;
-		} else if (dy > 0) {
-			dy += height/2;
-		}
-	} else if (pcb_crosshair.snapped_pin) {
-		pcb_pin_t *pin = pcb_crosshair.snapped_pin;
-		pcb_coord_t radius = pin->Thickness/2;
-		if (dx < 0) {
-			dx -= radius;
-		} else if (dx > 0) {
-			dx += radius;
-		}
-		if (dy < 0) {
-			dy -= radius;
-		} else if (dy > 0) {
-			dy += radius;
-		}
-	} else if (pcb_crosshair.snapped_pstk) {
+	if (pcb_crosshair.snapped_pstk) {
 		pcb_pstk_t *ps = pcb_crosshair.snapped_pstk;
 		pcb_coord_t radius = ((ps->BoundingBox.X2 - ps->BoundingBox.X1) + (ps->BoundingBox.Y2 - ps->BoundingBox.Y1))/2;
-		if (dx < 0) {
+		if (dx < 0)
 			dx -= radius;
-		} else if (dx > 0) {
+		else if (dx > 0)
 			dx += radius;
-		}
-		if (dy < 0) {
+		if (dy < 0)
 			dy -= radius;
-		} else if (dy > 0) {
+		else if (dy > 0)
 			dy += radius;
-		}
 	}
 
 	pcb_event_move_crosshair(pcb_crosshair.X + dx, pcb_crosshair.Y + dy);
@@ -1514,9 +1358,9 @@ static int pcb_act_EditLayer(int argc, const char **argv, pcb_coord_t x, pcb_coo
 					val = NULL;
 			}
 			if (val == NULL)
-				ret |= pcb_attribute_remove(&ly->meta.real.Attributes, key);
+				ret |= pcb_attribute_remove(&ly->Attributes, key);
 			else
-				ret |= pcb_attribute_put(&ly->meta.real.Attributes, key, val);
+				ret |= pcb_attribute_put(&ly->Attributes, key, val);
 			free(key);
 			pcb_board_set_changed_flag(pcb_true);
 		}
@@ -1539,7 +1383,7 @@ static int pcb_act_EditLayer(int argc, const char **argv, pcb_coord_t x, pcb_coo
 		attr[1].default_val.int_value = ly->comb & PCB_LYC_SUB;
 		attr[2].default_val.int_value = ly->comb & PCB_LYC_AUTO;
 
-		ar = pcb_gui->attribute_dialog(attr,sizeof(attr)/sizeof(attr[0]), rv, "Edit layer properties", "Edit the properties of a logical layer", NULL);
+		ar = pcb_attribute_dialog(attr,sizeof(attr)/sizeof(attr[0]), rv, "Edit layer properties", "Edit the properties of a logical layer", NULL);
 
 		if (ar == 0) {
 			pcb_layer_combining_t comb = 0;
@@ -1611,13 +1455,12 @@ static int pcb_act_EditGroup(int argc, const char **argv, pcb_coord_t x, pcb_coo
 				return 1;
 			}
 			switch(*sbit) {
-				case '+': g->type |= bit; break;
-				case '-': g->type &= ~bit; break;
+				case '+': g->ltype |= bit; break;
+				case '-': g->ltype &= ~bit; break;
 			}
 			interactive = 0;
 			pcb_board_set_changed_flag(pcb_true);
 		}
-#if 0
 		else if (strncmp(argv[n], "attrib", 6) == 0) {
 			char *key, *val;
 			interactive = 0;
@@ -1631,7 +1474,7 @@ static int pcb_act_EditGroup(int argc, const char **argv, pcb_coord_t x, pcb_coo
 			if (val != NULL) {
 				*val = '\0';
 				val++;
-				if (val == '\0')
+				if (*val == '\0')
 					val = NULL;
 			}
 			if (val == NULL)
@@ -1640,7 +1483,6 @@ static int pcb_act_EditGroup(int argc, const char **argv, pcb_coord_t x, pcb_coo
 				ret |= pcb_attribute_put(&g->Attributes, key, val);
 			free(key);
 		}
-#endif
 		else {
 			pcb_message(PCB_MSG_ERROR, "Invalid EditGroup() command: %s\n", argv[n]);
 			PCB_ACT_FAIL(EditLayer);
@@ -1656,7 +1498,7 @@ static int pcb_act_EditGroup(int argc, const char **argv, pcb_coord_t x, pcb_coo
 
 		attr[0].default_val.str_value = pcb_strdup(g->name);
 
-		ar = pcb_gui->attribute_dialog(attr,sizeof(attr)/sizeof(attr[0]), rv, "Edit layer properties", "Edit the properties of a logical layer", NULL);
+		ar = pcb_attribute_dialog(attr,sizeof(attr)/sizeof(attr[0]), rv, "Edit layer properties", "Edit the properties of a logical layer", NULL);
 
 		if (ar == 0) {
 			if (strcmp(g->name, attr[0].default_val.str_value) != 0) {
@@ -1689,6 +1531,8 @@ visible if it is not already visible
 static int pcb_act_SelectLayer(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
 {
 	pcb_layer_id_t lid;
+	const pcb_menu_layers_t *ml;
+	
 
 	if (pcb_strcasecmp(argv[0], "silk") == 0) {
 		PCB->RatDraw = 0;
@@ -1696,21 +1540,78 @@ static int pcb_act_SelectLayer(int argc, const char **argv, pcb_coord_t x, pcb_c
 			pcb_layervis_change_group_vis(lid, 1, 1);
 		else
 			pcb_message(PCB_MSG_ERROR, "Can't find this-side silk layer\n");
-	}
-	else if (pcb_strcasecmp(argv[0], "rats") == 0) {
-		PCB->RatOn = PCB->RatDraw = 1;
-		pcb_event(PCB_EVENT_LAYERVIS_CHANGED, NULL);
-	}
-	else {
-		PCB->RatDraw = 0;
-		pcb_layervis_change_group_vis(atoi(argv[0])-1, 1, 1);
+		return 0;
 	}
 
+	ml = pcb_menu_layer_find(argv[0]);
+	if (ml != NULL) {
+		pcb_bool *v = (pcb_bool *)((char *)PCB + ml->vis_offs);
+		pcb_bool *s = (pcb_bool *)((char *)PCB + ml->sel_offs);
+		if (ml->sel_offs == 0) {
+			pcb_message(PCB_MSG_ERROR, "Virtual layer '%s' (%s) can not be selected\n", ml->name, ml->abbrev);
+			return 0;
+		}
+		*s = *v = 1;
+		pcb_event(PCB_EVENT_LAYERVIS_CHANGED, NULL);
+		return 0;
+	}
+
+	PCB->RatDraw = 0;
+	pcb_layervis_change_group_vis(atoi(argv[0])-1, 1, 1);
+	pcb_gui->invalidate_all();
+	pcb_event(PCB_EVENT_LAYERVIS_CHANGED, NULL);
 	return 0;
 }
 
+const char pcb_acts_chklayer[] = "ChkLayer(layerid)";
+const char pcb_acth_chklayer[] = "Returns 1 if the specified layer is the active layer";
+/* %start-doc actions ChkLayer
 
-const char pcb_acts_toggleview[] = "ToggleView(1..MAXLAYER)\n" "ToggleView(layername)\n" "ToggleView(Silk|Rats|Pins|Vias|Mask|BackSide)\n" "ToggleView(All, Open|Vis, Set|Clear|Toggle)";
+Returns 1 if the specified layer is the active layer.
+
+%end-doc */
+
+static int pcb_act_ChkLayer(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
+{
+	pcb_layer_id_t lid;
+	pcb_layer_t *ly;
+	char *end;
+	const pcb_menu_layers_t *ml;
+
+	if (argc < 1)
+		PCB_ACT_FAIL(chklayer); /* argv[0] is a must */
+
+	lid = strtol(argv[0], &end, 10);
+	if (*end != '\0') {
+		ml = pcb_menu_layer_find(argv[0]);
+		if (ml != NULL) {
+			if (ml->sel_offs != 0) {
+				pcb_bool *s = (pcb_bool *)((char *)PCB + ml->sel_offs);
+				return *s;
+			}
+			return -1;
+		}
+		pcb_message(PCB_MSG_ERROR, "pcb_act_ChkLayer: '%s' is not a valid layer ID - check your menu file!\n", argv[0]);
+		return -1;
+	}
+
+	/* if any virtual is selected, do not accept CURRENT as selected */
+	for(ml = pcb_menu_layers; ml->name != NULL; ml++) {
+		pcb_bool *s = (pcb_bool *)((char *)PCB + ml->sel_offs);
+		if ((ml->sel_offs != 0) && (*s))
+			return 0;
+	}
+
+	lid--;
+	ly = pcb_get_layer(PCB->Data, lid);
+	if (ly == NULL)
+		return -1;
+
+	return ly == CURRENT;
+}
+
+
+const char pcb_acts_toggleview[] = "ToggleView(1..MAXLAYER)\n" "ToggleView(layername)\n" "ToggleView(Silk|Rats|Pins|Vias|BackSide)\n" "ToggleView(All, Open|Vis, Set|Clear|Toggle)";
 const char pcb_acth_toggleview[] = "Toggle the visibility of the specified layer or layer group.";
 
 /* %start-doc actions ToggleView
@@ -1757,18 +1658,8 @@ static int pcb_act_ToggleView(int argc, const char **argv, pcb_coord_t x, pcb_co
 		else
 			pcb_message(PCB_MSG_ERROR, "Can't find this-side silk layer\n");
 	}
-	else if (pcb_strcasecmp(argv[0], "rats") == 0) {
-		PCB->RatOn = !PCB->RatOn;
-		pcb_gui->invalidate_all();
-		pcb_event(PCB_EVENT_LAYERVIS_CHANGED, NULL);
-	}
-	else if (pcb_strcasecmp(argv[0], "vias") == 0) {
-		PCB->ViaOn = !PCB->ViaOn;
-		pcb_gui->invalidate_all();
-		pcb_event(PCB_EVENT_LAYERVIS_CHANGED, NULL);
-	}
-	else if ((pcb_strcasecmp(argv[0], "pins") == 0) || (pcb_strcasecmp(argv[0], "pads") == 0)) {
-		PCB->PinOn = !PCB->PinOn;
+	else if ((pcb_strcasecmp(argv[0], "padstacks") == 0) || (pcb_strcasecmp(argv[0], "vias") == 0) || (pcb_strcasecmp(argv[0], "pins") == 0) || (pcb_strcasecmp(argv[0], "pads") == 0)) {
+		PCB->pstk_on = !PCB->pstk_on;
 		pcb_gui->invalidate_all();
 		pcb_event(PCB_EVENT_LAYERVIS_CHANGED, NULL);
 	}
@@ -1777,13 +1668,179 @@ static int pcb_act_ToggleView(int argc, const char **argv, pcb_coord_t x, pcb_co
 		pcb_gui->invalidate_all();
 		pcb_event(PCB_EVENT_LAYERVIS_CHANGED, NULL);
 	}
+	else if (strncmp(argv[0], "ui:", 3) == 0) {
+		pcb_layer_t *ly = vtlayer_get(&pcb_uilayer, atoi(argv[0]+3), 0);
+		if (ly == NULL) {
+			pcb_message(PCB_MSG_ERROR, "Invalid ui layer id: '%s'\n", argv[0]);
+			return -1;
+		}
+		ly->meta.real.vis = !ly->meta.real.vis;
+		pcb_gui->invalidate_all();
+		pcb_event(PCB_EVENT_LAYERVIS_CHANGED, NULL);
+	}
 	else {
-		pcb_layervis_change_group_vis(atoi(argv[0])-1, -1, 0);
+		char *end;
+		int id = strtol(argv[0], &end, 10) - 1;
+		if (*end == '\0') { /* integer layer */
+			pcb_layervis_change_group_vis(id, -1, 0);
+			pcb_gui->invalidate_all();
+			pcb_event(PCB_EVENT_LAYERVIS_CHANGED, NULL);
+			return 0;
+		}
+		else { /* virtual menu layer */
+			const pcb_menu_layers_t *ml = pcb_menu_layer_find(argv[0]);
+			if (ml != NULL) {
+				pcb_bool *v = (pcb_bool *)((char *)PCB + ml->vis_offs);
+				*v = !(*v);
+				pcb_gui->invalidate_all();
+				pcb_event(PCB_EVENT_LAYERVIS_CHANGED, NULL);
+				return 0;
+			}
+			pcb_message(PCB_MSG_ERROR, "Invalid layer id: '%s'\n", argv[0]);
+		}
 	}
 
 	return 1;
 }
 
+const char pcb_acts_chkview[] = "ChkView(layerid)\n";
+const char pcb_acth_chkview[] = "Return 1 if layerid is visible.";
+
+/* %start-doc actions ChkView
+Return 1 if layerid is visible. Intended for meu item 'checked' fields.
+%end-doc */
+
+static int pcb_act_ChkView(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
+{
+	pcb_layer_id_t lid;
+	pcb_layer_t *ly;
+	char *end;
+
+	if (argc < 1)
+		PCB_ACT_FAIL(chkview); /* argv[0] is a must */
+
+	if (strncmp(argv[0], "ui:", 3) == 0) {
+		pcb_layer_t *ly = vtlayer_get(&pcb_uilayer, atoi(argv[0]+3), 0);
+		if (ly == NULL)
+			return -1;
+		return ly->meta.real.vis;
+	}
+
+	lid = strtol(argv[0], &end, 10);
+
+	if (*end != '\0') {
+		const pcb_menu_layers_t *ml = pcb_menu_layer_find(argv[0]);
+
+		if (ml != NULL) {
+			pcb_bool *v = (pcb_bool *)((char *)PCB + ml->vis_offs);
+			return *v;
+		}
+
+		pcb_message(PCB_MSG_ERROR, "pcb_act_ChkView: '%s' is not a valid layer ID - check your menu file!\n", argv[0]);
+		return -1;
+	}
+
+	lid--;
+	ly = pcb_get_layer(PCB->Data, lid);
+	if (ly == NULL)
+		return -1;
+
+	return ly->meta.real.vis;
+}
+
+
+static const char pcb_acts_setunits[] = "SetUnits(mm|mil)";
+static const char pcb_acth_setunits[] = "Set the default measurement units.";
+
+/* %start-doc actions SetUnits
+
+@table @code
+
+@item mil
+Sets the display units to mils (1/1000 inch).
+
+@item mm
+Sets the display units to millimeters.
+
+@end table
+
+%end-doc */
+
+static int pcb_act_SetUnits(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
+{
+	const pcb_unit_t *new_unit;
+	if (argc == 0)
+		return 0;
+
+	new_unit = get_unit_struct(argv[0]);
+	pcb_board_set_unit(PCB, new_unit);
+
+	return 0;
+}
+
+static const char pcb_acts_grid[] =
+	"grid(set, [name:]size[@offs][!unit])\n"
+	"grid(+|up)\n" "grid(-|down)\n" "grid(#N)\n" "grid(idx, N)\n";
+static const char pcb_acth_grid[] = "Set the grid.";
+static int pcb_act_grid(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
+{
+	if (argc == 0)
+		return 0;
+
+	if (strcmp(argv[0], "set") == 0) {
+		pcb_grid_t dst;
+		if ((argc < 2) || !pcb_grid_parse(&dst, argv[1]))
+			PCB_ACT_FAIL(grid);
+		pcb_grid_set(PCB, &dst);
+		pcb_grid_free(&dst);
+	}
+	else if ((strcmp(argv[0], "up") == 0) || (strcmp(argv[0], "+") == 0))
+		pcb_grid_list_step(+1);
+	else if ((strcmp(argv[0], "down") == 0) || (strcmp(argv[0], "-") == 0))
+		pcb_grid_list_step(-1);
+	else if (strcmp(argv[0], "idx") == 0) {
+		if (argc < 2)
+			PCB_ACT_FAIL(grid);
+		pcb_grid_list_jump(atoi(argv[1]));
+	}
+	else if (argv[0][0] == '#') {
+		pcb_grid_list_jump(atoi(argv[0]+1));
+	}
+	else
+		PCB_ACT_FAIL(grid);
+
+	return 0;
+}
+
+const char pcb_acts_chkrst[] = "ChkRst(route_style_id)\n";
+const char pcb_acth_chkrst[] = "Return 1 if route_style_id matches pen.";
+
+/* %start-doc actions ChkRst
+Return 1 if route_style_id matches pen.
+%end-doc */
+
+static int pcb_act_ChkRst(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
+{
+	int rid;
+	pcb_route_style_t *rst;
+	char *end;
+
+	if (argc < 1)
+		PCB_ACT_FAIL(chkview); /* argv[0] is a must */
+
+	rid = strtol(argv[0], &end, 10);
+	if (*end != '\0') {
+		pcb_message(PCB_MSG_ERROR, "pcb_act_ChkRst: '%s' is not a valid route style ID - check your menu file!\n", argv[0]);
+		return -1;
+	}
+
+	rst = vtroutestyle_get(&PCB->RouteStyle, rid, 0);
+	if (rst == NULL)
+		return -1;
+
+	return pcb_route_style_match(rst, conf_core.design.line_thickness, conf_core.design.via_thickness, conf_core.design.via_drilling_hole, conf_core.design.clearance, NULL);
+}
+
 pcb_hid_action_t gui_action_list[] = {
 	{"Display", 0, pcb_act_Display,
 	 pcb_acth_Display, pcb_acts_Display}
@@ -1821,12 +1878,18 @@ pcb_hid_action_t gui_action_list[] = {
 	{"SelectLayer", 0, pcb_act_SelectLayer,
 	 pcb_acth_selectlayer, pcb_acts_selectlayer}
 	,
+	{"ChkLayer", 0, pcb_act_ChkLayer,
+	 pcb_acth_chklayer, pcb_acts_chklayer}
+	,
 	{"SwitchHID", 0, pcb_act_SwitchHID,
 	 pcb_acth_SwitchHID, pcb_acts_SwitchHID}
 	,
 	{"ToggleView", 0, pcb_act_ToggleView,
 	 pcb_acth_toggleview, pcb_acts_toggleview}
 	,
+	{"ChkView", 0, pcb_act_ChkView,
+	 pcb_acth_chkview, pcb_acts_chkview}
+	,
 	{"PCBChanged", 0, pcb_act_PCBChanged,
 	 pcb_acth_PCBChanged, pcb_acts_PCBChanged}
 	,
@@ -1846,6 +1909,13 @@ pcb_hid_action_t gui_action_list[] = {
 	,
 	{"EditGroup", 0, pcb_act_EditGroup,
 	 pcb_acth_EditGroup, pcb_acts_EditGroup}
+	,
+	{"Grid", 0, pcb_act_grid, pcb_acth_grid, pcb_acts_grid}
+	,
+	{"SetUnits", 0, pcb_act_SetUnits, pcb_acth_setunits, pcb_acts_setunits}
+	,
+	{"ChkRst", 0, pcb_act_ChkRst,
+	 pcb_acth_chkrst, pcb_acts_chkrst}
 };
 
 PCB_REGISTER_ACTIONS(gui_action_list, NULL)
