@@ -47,7 +47,7 @@ void pcb_hid_remove_attributes_by_cookie(const char *cookie)
 	}
 }
 
-void pcb_hid_parse_command_line(int *argc, char ***argv)
+int pcb_hid_parse_command_line(int *argc, char ***argv)
 {
 	pcb_hid_attr_node_t *ha;
 	int i, e, ok;
@@ -196,7 +196,7 @@ void pcb_hid_parse_command_line(int *argc, char ***argv)
 			goto try_no_arg;
 		}
 		fprintf(stderr, "unrecognized option: %s\n", (*argv)[0]);
-		exit(1);
+		return -1;
 	got_match:;
 	}
 	else {
@@ -209,7 +209,7 @@ void pcb_hid_parse_command_line(int *argc, char ***argv)
 		}
 		else {
 			pcb_message(PCB_MSG_ERROR, "Multiple filenames not supported. First filename was: %s; offending second filename: %s\n", filename, (*argv)[0]);
-			exit(1);
+			return -1;
 		}
 	}
 
@@ -218,6 +218,7 @@ void pcb_hid_parse_command_line(int *argc, char ***argv)
 		(*argv)[0] = filename;
 		(*argc) = 1;
 	}
+	return 0;
 }
 
 void pcb_hid_usage_option(const char *name, const char *help)
@@ -236,7 +237,7 @@ void pcb_hid_usage(pcb_hid_attribute_t * a, int numa)
 	}
 }
 
-int pcb_hid_atrdlg_num_children(pcb_hid_attribute_t *attrs, int start_from, int n_attrs)
+int pcb_hid_attrdlg_num_children(pcb_hid_attribute_t *attrs, int start_from, int n_attrs)
 {
 	int n, level = 1, cnt = 0;
 
@@ -260,3 +261,10 @@ int pcb_hid_atrdlg_num_children(pcb_hid_attribute_t *attrs, int start_from, int
 	}
 	return cnt;
 }
+
+int pcb_attribute_dialog(pcb_hid_attribute_t *attrs, int n_attrs, pcb_hid_attr_val_t *results, const char *title, const char *descr, void *caller_data)
+{
+	if ((pcb_gui == NULL) || (pcb_gui->attribute_dialog == NULL))
+		return -1;
+	return pcb_gui->attribute_dialog(attrs, n_attrs, results, title, descr, caller_data);
+}
\ No newline at end of file
