@@ -31,22 +31,25 @@
 
 #include <liblihata/dom.h>
 #include <stdarg.h>
+#include "global_typedefs.h"
+#include "hid.h"
 
-typedef struct pcb_hid_cfg_s {
+struct pcb_hid_cfg_s {
 	lht_doc_t *doc;
-} pcb_hid_cfg_t;
+};
 
 /* Create a set of resources representing a single menu item
    If action is NULL, it's a drop-down item that has submenus.
    The callback is called after the new lihata node is created.
    NOTE: unlike other cookies, this cookie is strdup()'d. 
    */
-typedef int (*pcb_create_menu_widget_t)(void *ctx, const char *path, const char *name, int is_main, lht_node_t *parent, lht_node_t *menu_item);
-int pcb_hid_cfg_create_menu(pcb_hid_cfg_t *hr, const char *path, const char *action, const char *mnemonic, const char *accel, const char *tip, const char *cookie, pcb_create_menu_widget_t cb, void *cb_ctx);
+typedef int (*pcb_create_menu_widget_t)(void *ctx, const char *path, const char *name, int is_main, lht_node_t *parent, lht_node_t *ins_after, lht_node_t *menu_item);
+int pcb_hid_cfg_create_menu(pcb_hid_cfg_t *hr, const char *path, const pcb_menu_prop_t *props, pcb_create_menu_widget_t cb, void *cb_ctx);
 
 /* Remove a path recursively; call gui_remove() on leaf paths until the subtree
    is consumed (should return 0 on success) */
 int pcb_hid_cfg_remove_menu(pcb_hid_cfg_t *hr, const char *path, int (*gui_remove)(void *ctx, lht_node_t *nd), void *ctx);
+int pcb_hid_cfg_remove_menu_node(pcb_hid_cfg_t *hr, lht_node_t *root, int (*gui_remove)(void *ctx, lht_node_t *nd), void *ctx);
 
 /* Search and load the menu res for hidname; if not found, and embedded_fallback
    is not NULL, parse that string instead. Returns NULL on error */
@@ -62,6 +65,14 @@ const char *pcb_hid_cfg_text_value(lht_doc_t *doc, const char *path);
 lht_node_t *pcb_hid_cfg_get_menu(pcb_hid_cfg_t *hr, const char *menu_path);
 lht_node_t *pcb_hid_cfg_get_menu_at(pcb_hid_cfg_t *hr, lht_node_t *at, const char *menu_path, lht_node_t *(*cb)(void *ctx, lht_node_t *node, const char *path, int rel_level), void *ctx);
 
+/* search all instances of an @anchor menu in the currently active GUI and
+   call cb on the lihata node; path has 128 extra bytes available at the end */
+void pcb_hid_cfg_map_anchor_menus(const char *name, void (*cb)(void *ctx, pcb_hid_cfg_t *cfg, lht_node_t *n, char *path), void *ctx);
+
+/* remove all adjacent anchor menus with matching cookie below anode, the
+   anchor node */
+int pcb_hid_cfg_del_anchor_menus(lht_node_t *anode, const char *cookie);
+
 
 /* Fields are retrieved using this enum so that HIDs don't need to hardwire
    lihata node names */
@@ -78,7 +89,6 @@ typedef enum {
 	PCB_MF_FOREGROUND,
 	PCB_MF_BACKGROUND,
 	PCB_MF_FONT
-/*	MF_RADIO*/
 } pcb_hid_cfg_menufield_t;
 
 /* Return a field of a submenu and optionally fill in field_name with the
@@ -99,8 +109,9 @@ int pcb_hid_cfg_has_submenus(const lht_node_t *submenu);
 
 /* Create a new hash node under parent (optional) and create a flat subtree of
    text nodes from name,value varargs (NULL terminated). This is a shorthand
-   for creating a menu item in a subtree list. */
-lht_node_t *pcb_hid_cfg_create_hash_node(lht_node_t *parent, const char *name, ...);
+   for creating a menu item in a subtree list. If ins_after is not NULL and
+   is under the same parent, the new menu is inserted after ins_after. */
+lht_node_t *pcb_hid_cfg_create_hash_node(lht_node_t *parent, lht_node_t *ins_after, const char *name, ...);
 
 /* Create a flat subtree of text nodes from name,value varargs (NULL
    terminated). This is a shorthand for creating a menu item in a
