@@ -54,6 +54,17 @@ pcb_virt_layer_t pcb_virt_layers[] = {
 	{ NULL, 0 },
 };
 
+const pcb_menu_layers_t pcb_menu_layers[] = {
+	{ "Subcircuits", "subc",     &conf_core.appearance.color.subc,              NULL,       offsetof(pcb_board_t, SubcOn),             0 },
+	{ "Subc. parts", "subcpart", &conf_core.appearance.color.subc,              NULL,       offsetof(pcb_board_t, SubcPartsOn),        0 },
+	{ "Padstacks",   "padstacks",&conf_core.appearance.color.pin,               NULL,       offsetof(pcb_board_t, pstk_on),            0 },
+	{ "Holes",       "holes",    &conf_core.appearance.color.pin,               NULL,       offsetof(pcb_board_t, hole_on),            0 },
+	{ "Pstk. marks", "pstkmark", &conf_core.appearance.color.padstackmark,      NULL,       offsetof(pcb_board_t, padstack_mark_on),   0 },
+	{ "Far side",    "farside",  &conf_core.appearance.color.invisible_objects, NULL,       offsetof(pcb_board_t, InvisibleObjectsOn), 0 },
+	{ "Rats",        "rats",     &conf_core.appearance.color.rat,               "rats",     offsetof(pcb_board_t, RatOn),              offsetof(pcb_board_t, RatDraw) },
+	{ NULL,          NULL,       NULL,                                          NULL,       0}
+};
+
 
 typedef struct {
 	pcb_layer_type_t type;
@@ -111,18 +122,12 @@ do { \
 void pcb_layer_free(pcb_layer_t *layer)
 {
 	if (!layer->is_bound)
-		pcb_attribute_free(&layer->meta.real.Attributes);
+		pcb_attribute_free(&layer->Attributes);
 	PCB_TEXT_LOOP(layer);
 	{
 		free(text->TextString);
 	}
 	PCB_END_LOOP;
-	PCB_LINE_LOOP(layer);
-	{
-		if (line->Number)
-			free(line->Number);
-	}
-	PCB_END_LOOP;
 
 	list_map0(&layer->Line, pcb_line_t, pcb_line_free);
 	list_map0(&layer->Arc,  pcb_arc_t,  pcb_arc_free);
@@ -197,10 +202,7 @@ static pcb_bool pcb_layer_is_empty_glob(pcb_board_t *pcb, pcb_data_t *data, pcb_
 
 pcb_bool pcb_layer_is_empty_(pcb_board_t *pcb, pcb_layer_t *layer)
 {
-	pcb_layer_type_t flags;
-
-	flags = pcb_layer_flags_(layer);
-
+	pcb_layer_type_t flags = pcb_layer_flags_(layer);
 	if (flags == 0)
 		return 1;
 
@@ -208,29 +210,9 @@ pcb_bool pcb_layer_is_empty_(pcb_board_t *pcb, pcb_layer_t *layer)
 	if (!pcb_layer_is_pure_empty(layer))
 		return 0;
 
-	if ((flags & PCB_LYT_COPPER) && (flags & PCB_LYT_TOP)) { /* if our layer is the top copper layer and we have an element pad on it, it's non-empty */
-		PCB_PAD_ALL_LOOP(pcb->Data);
-		{
-			if (!PCB_FLAG_TEST(PCB_FLAG_ONSOLDER, pad))
-				return 0;
-		}
-		PCB_ENDALL_LOOP;
-	}
-
-	if ((flags & PCB_LYT_COPPER) && (flags & PCB_LYT_BOTTOM)) { /* if our layer is the bottom copper layer and we have an element pad on it, it's non-empty */
-		PCB_PAD_ALL_LOOP(pcb->Data);
-		{
-			if (PCB_FLAG_TEST(PCB_FLAG_ONSOLDER, pad))
-				return 0;
-		}
-		PCB_ENDALL_LOOP;
-	}
-
 	if (!pcb_layer_is_empty_glob(pcb, pcb->Data, layer))
 		return 0;
 
-#warning TODO: check top silk and bottom silk for elements
-
 	/* found nothing: layer is empty */
 	return 1;
 }
@@ -247,7 +229,7 @@ pcb_layer_id_t pcb_layer_id(pcb_data_t *Data, pcb_layer_t *Layer)
 	if ((Layer >= pcb_uilayer.array) && (Layer < pcb_uilayer.array + vtlayer_len(&pcb_uilayer)))
 		return (Layer - pcb_uilayer.array) | PCB_LYT_UI;
 
-	if (Layer->parent != Data) {
+	if (Layer->parent.data != Data) {
 		/* the only case this makes sense is when we are resolving a bound layer */
 		if ((Layer->is_bound) && (Layer->meta.bound.real != NULL))
 			return pcb_layer_id(Data, Layer->meta.bound.real);
@@ -282,7 +264,7 @@ unsigned int pcb_layer_flags_(pcb_layer_t *layer)
 
 	/* real layer: have to do a layer stack based lookup; but at least we have a real layer ID  */
 	if (!layer->is_bound) {
-		pcb_data_t *data = layer->parent;
+		pcb_data_t *data = layer->parent.data;
 		pcb_layer_id_t lid;
 
 		if (data == NULL)
@@ -402,7 +384,7 @@ void pcb_layers_reset(pcb_board_t *pcb)
 	/* reset layer groups */
 	for(n = 0; n < PCB_MAX_LAYERGRP; n++) {
 		pcb->LayerGroups.grp[n].len = 0;
-		pcb->LayerGroups.grp[n].type = 0;
+		pcb->LayerGroups.grp[n].ltype = 0;
 		pcb->LayerGroups.grp[n].valid = 0;
 	}
 	pcb->LayerGroups.len = 0;
@@ -440,7 +422,10 @@ pcb_layer_id_t pcb_layer_create(pcb_board_t *pcb, pcb_layergrp_id_t grp, const c
 	}
 	pcb->Data->Layer[id].meta.real.grp = grp;
 
-	pcb->Data->Layer[id].parent = pcb->Data;
+	pcb->Data->Layer[id].parent_type = PCB_PARENT_DATA;
+	pcb->Data->Layer[id].parent.data = pcb->Data;
+	pcb->Data->Layer[id].type = PCB_OBJ_LAYER;
+
 	return id;
 }
 
@@ -508,14 +493,16 @@ static void layer_init(pcb_board_t *pcb, pcb_layer_t *lp, pcb_layer_id_t idx, pc
 	lp->meta.real.color = conf_core.appearance.color.layer[idx];
 	lp->meta.real.selected_color = conf_core.appearance.color.layer_selected[idx];
 	if ((gid >= 0) && (pcb->LayerGroups.grp[gid].len == 0)) { /*When adding the first layer in a group, set up comb flags automatically */
-		switch((pcb->LayerGroups.grp[gid].type) & PCB_LYT_ANYTHING) {
+		switch((pcb->LayerGroups.grp[gid].ltype) & PCB_LYT_ANYTHING) {
 			case PCB_LYT_MASK:  lp->comb = PCB_LYC_AUTO | PCB_LYC_SUB; break;
 			case PCB_LYT_SILK:  lp->comb = PCB_LYC_AUTO;
 			case PCB_LYT_PASTE: lp->comb = PCB_LYC_AUTO;
 			default: break;
 		}
 	}
-	lp->parent = parent;
+	lp->parent.data = parent;
+	lp->parent_type = PCB_PARENT_DATA;
+	lp->type = PCB_OBJ_LAYER;
 }
 
 int pcb_layer_move(pcb_board_t *pcb, pcb_layer_id_t old_index, pcb_layer_id_t new_index, pcb_layergrp_id_t new_in_grp)
@@ -749,10 +736,16 @@ pcb_layer_t *pcb_layer_resolve_binding(pcb_board_t *pcb, pcb_layer_t *src)
 		gid = pcb_layergrp_step(pcb, gid, src->meta.bound.stack_offs, PCB_LYT_COPPER | PCB_LYT_INTERN);
 	}
 	else {
-		if (pcb_layergrp_list(pcb, src->meta.bound.type, &gid, 1) != 1)
+		pcb_layer_type_t lyt = src->meta.bound.type;
+		if ((lyt & PCB_LYT_OUTLINE) && (lyt & PCB_LYT_ANYWHERE)) {
+			lyt = PCB_LYT_OUTLINE;
+			pcb_message(PCB_MSG_WARNING, "Ignoring invalid layer flag combination for %s: outline layer must be global\n", src->name);
+		}
+		if (pcb_layergrp_list(pcb, lyt, &gid, 1) != 1)
 			return NULL;
 	}
 
+	/* target group identified; pick the closest match layer within that group */
 	{
 		pcb_layergrp_t *grp = pcb->LayerGroups.grp+gid;
 		for(l = 0; l < grp->len; l++) {
@@ -791,8 +784,9 @@ pcb_layer_t *pcb_layer_new_bound(pcb_data_t *data, pcb_layer_type_t type, const
 	lay->is_bound = 1;
 	lay->name = pcb_strdup(name);
 	lay->meta.bound.type = type;
-	lay->parent = data;
-
+	lay->parent.data = data;
+	lay->parent_type = PCB_PARENT_DATA;
+	lay->type = PCB_OBJ_LAYER;
 	return lay;
 }
 
@@ -916,7 +910,7 @@ int pcb_layer_gui_set_g_ui(pcb_layer_t *first, int is_empty)
 		return 0;
 
 	if (pcb_gui->set_layer_group != NULL)
-		return pcb_gui->set_layer_group(-1, pcb_layer_id(first->parent, first), PCB_LYT_VIRTUAL | PCB_LYT_UI, is_empty);
+		return pcb_gui->set_layer_group(-1, pcb_layer_id(first->parent.data, first), PCB_LYT_VIRTUAL | PCB_LYT_UI, is_empty);
 
 	/* if the GUI doesn't have a set_layer, assume it wants to draw all layers */
 	return 1;
@@ -925,10 +919,26 @@ int pcb_layer_gui_set_g_ui(pcb_layer_t *first, int is_empty)
 void pcb_layer_edit_attrib(pcb_layer_t *layer)
 {
 	char *buf = pcb_strdup_printf("Layer %s Attributes", layer->name);
-	pcb_gui->edit_attributes(buf, &(layer->meta.real.Attributes));
+	pcb_gui->edit_attributes(buf, &(layer->Attributes));
 	free(buf);
 }
 
+const pcb_menu_layers_t *pcb_menu_layer_find(const char *name_or_abbrev)
+{
+	const pcb_menu_layers_t *ml;
+
+	for(ml = pcb_menu_layers; ml->name != NULL; ml++)
+		if (pcb_strcasecmp(name_or_abbrev, ml->abbrev) == 0)
+			return ml;
+
+	for(ml = pcb_menu_layers; ml->name != NULL; ml++)
+		if (pcb_strcasecmp(name_or_abbrev, ml->name) == 0)
+			return ml;
+
+	return NULL;
+}
+
+
 static pcb_layer_id_t pcb_layer_get_cached(pcb_board_t *pcb, pcb_layer_id_t *cache, unsigned int loc, unsigned int typ)
 {
 	pcb_layergrp_t *g;
@@ -937,7 +947,7 @@ static pcb_layer_id_t pcb_layer_get_cached(pcb_board_t *pcb, pcb_layer_id_t *cac
 		pcb_layergrp_id_t gid = pcb->Data->Layer[*cache].meta.real.grp;
 		if ((gid >= 0) && (gid < pcb->LayerGroups.len)) {
 			g = &(pcb->LayerGroups.grp[gid]);
-			if ((g->type & loc) && (g->type & typ) && (g->lid[0] == *cache))
+			if ((g->ltype & loc) && (g->ltype & typ) && (g->lid[0] == *cache))
 				return *cache;
 		}
 	}
