@@ -40,14 +40,16 @@ typedef struct pcb_layergrp_s pcb_layergrp_t;
  */
 
 struct pcb_layergrp_s {
+	PCB_ANY_OBJ_FIELDS;
 	pcb_cardinal_t len;                    /* number of layer IDs in use */
 	pcb_layer_id_t lid[PCB_MAX_LAYER];     /* lid=layer ID */
 	char *name;                            /* name of the physical layer (independent of the name of the layer groups) */
-	pcb_layer_type_t type;
+	pcb_layer_type_t ltype;
 
 	unsigned valid:1;                      /* 1 if it's a new-style, valid layer group; 0 after loading old files with no layer stackup info */
 	unsigned vis:1;                        /* 1 if layer group is visible on the GUI */
 	unsigned open:1;                       /* 1 if the group is open (expanded) on the GUI */
+	unsigned propedit:1;                   /* temporary: let propedit know this layer is being edited */
 
 	/* internal: temporary states */
 	int intern_id;                         /* for the old layer import mechanism */
@@ -108,11 +110,10 @@ int pcb_layer_add_in_group_(pcb_board_t *pcb, pcb_layergrp_t *grp, pcb_layergrp_
    (.grp = -1), else layers are deleted too */
 int pcb_layergrp_del(pcb_board_t *pcb, pcb_layergrp_id_t gid, int del_layers);
 
-/** Move gfrom to to_before and shift the stack as necessary. Return -1 on range error */
+/* Move gfrom to to_before and shift the stack as necessary. Return -1 on range error */
 int pcb_layergrp_move(pcb_board_t *pcb, pcb_layergrp_id_t gfrom, pcb_layergrp_id_t to_before);
 
-/** Move src onto dst, not shifting the stack, free()'ing and overwriting dst,
-    leaving a gap (0'd slot) at src */
+/* Move src onto dst, not shifting the stack, free()'ing and overwriting dst, leaving a gap (0'd slot) at src */
 int pcb_layergrp_move_onto(pcb_board_t *pcb, pcb_layergrp_id_t dst, pcb_layergrp_id_t src);
 
 
@@ -190,7 +191,7 @@ int pcb_layer_parse_group_string(pcb_board_t *pcb, const char *s, int LayerN, in
 		layer = &data->Layer[number];
 
 /* for parsing old files with old layer descriptions, with no layer groups */
-void pcb_layer_group_setup_default(pcb_layer_stack_t *newg); /* default layer groups, no layers */
+void pcb_layer_group_setup_default(pcb_board_t *pcb); /* default layer groups, no layers */
 void pcb_layer_group_setup_silks(pcb_board_t *pcb); /* make sure we have two silk layers, add them if needed */
 pcb_layergrp_t *pcb_get_grp(pcb_layer_stack_t *stack, pcb_layer_type_t loc, pcb_layer_type_t typ);
 pcb_layergrp_t *pcb_get_grp_new_intern(pcb_board_t *pcb, int intern_id);
