@@ -22,13 +22,18 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
 /* Virtual layers for UI and debug */
 #include "config.h"
+#include "layer.h"
+#include "event.h"
 #define GVT_DONT_UNDEF
 #include "layer_ui.h"
-#include "event.h"
 #include <genvector/genvector_impl.c>
 
 vtlayer_t pcb_uilayer;
@@ -59,18 +64,35 @@ found:;
 	return l;
 }
 
+static void pcb_uilayer_free_(pcb_layer_t *l)
+{
+	pcb_layer_free(l);
+	l->meta.real.cookie = NULL;
+	l->meta.real.color = l->name = NULL;
+	l->meta.real.vis = 0;
+}
+
+void pcb_uilayer_free(pcb_layer_t *ly)
+{
+	int n;
+	for(n = 0; n < vtlayer_len(&pcb_uilayer); n++) {
+		pcb_layer_t *l = &pcb_uilayer.array[n];
+		if (l == ly) {
+			pcb_uilayer_free_(l);
+			break;
+		}
+	}
+	pcb_event(PCB_EVENT_LAYERS_CHANGED, NULL);
+}
+
 void pcb_uilayer_free_all_cookie(const char *cookie)
 {
 	int n;
 	for(n = 0; n < vtlayer_len(&pcb_uilayer); n++) {
 		pcb_layer_t *l = &pcb_uilayer.array[n];
-		if (l->meta.real.cookie == cookie) {
-#warning TODO: free all objects
-			l->meta.real.cookie = NULL;
-			l->meta.real.color = l->name = NULL;
-			l->meta.real.vis = 0;
-			pcb_event(PCB_EVENT_LAYERS_CHANGED, NULL);
-		}
+		if (l->meta.real.cookie == cookie)
+			pcb_uilayer_free_(l);
 	}
+	pcb_event(PCB_EVENT_LAYERS_CHANGED, NULL);
 }
 
