@@ -45,7 +45,7 @@
  * pcb_layervis_restore_stack()
  */
 static struct {
-	pcb_bool ElementOn, InvisibleObjectsOn, PinOn, ViaOn, RatOn;
+	pcb_bool ElementOn, InvisibleObjectsOn, pstk_on, RatOn;
 	int pcb_layer_stack[PCB_MAX_LAYER];
 	pcb_bool LayerOn[PCB_MAX_LAYER];
 	int cnt;
@@ -145,8 +145,7 @@ void pcb_layervis_reset_stack(void)
 		PCB->Data->Layer[i].meta.real.vis = pcb_true;
 	}
 	PCB->InvisibleObjectsOn = pcb_true;
-	PCB->PinOn = pcb_true;
-	PCB->ViaOn = pcb_true;
+	PCB->pstk_on = pcb_true;
 	PCB->SubcOn = pcb_true;
 	PCB->SubcPartsOn = pcb_true;
 	PCB->RatOn = pcb_true;
@@ -181,8 +180,7 @@ void pcb_layervis_save_stack(void)
 	}
 	SavedStack.ElementOn = pcb_silk_on(PCB);
 	SavedStack.InvisibleObjectsOn = PCB->InvisibleObjectsOn;
-	SavedStack.PinOn = PCB->PinOn;
-	SavedStack.ViaOn = PCB->ViaOn;
+	SavedStack.pstk_on = PCB->pstk_on;
 	SavedStack.RatOn = PCB->RatOn;
 	SavedStack.cnt++;
 }
@@ -208,8 +206,7 @@ void pcb_layervis_restore_stack(void)
 		PCB->Data->Layer[i].meta.real.vis = SavedStack.LayerOn[i];
 	}
 	PCB->InvisibleObjectsOn = SavedStack.InvisibleObjectsOn;
-	PCB->PinOn = SavedStack.PinOn;
-	PCB->ViaOn = SavedStack.ViaOn;
+	PCB->pstk_on = SavedStack.pstk_on;
 	PCB->RatOn = SavedStack.RatOn;
 
 	SavedStack.cnt--;
@@ -281,12 +278,15 @@ static void layer_vis_grp_defaults(void *user_data, int argc, pcb_event_arg_t ar
 	/* do not show paste and mask by default - they are distractive */
 	for(gid = 0; gid < pcb_max_group(PCB); gid++) {
 		pcb_layergrp_t *g = &PCB->LayerGroups.grp[gid];
-		if ((g->type & PCB_LYT_MASK) || (g->type & PCB_LYT_PASTE)) {
+		if ((g->ltype & PCB_LYT_MASK) || (g->ltype & PCB_LYT_PASTE)) {
 			int n;
 			g->vis = pcb_false;
 			for(n = 0; n < g->len; n++) {
 				pcb_layer_t *l = pcb_get_layer(PCB->Data, g->lid[n]);
-				l->meta.real.vis = 0;
+				if (l == NULL)
+					pcb_message(PCB_MSG_ERROR, "broken layer groups; layer group references to non-existing layer\n");
+				else
+					l->meta.real.vis = 0;
 			}
 		}
 	}
