@@ -35,6 +35,7 @@
 #include "library.h"
 #include "macro.h"
 #include "compat_misc.h"
+#include "obj_common.h"
 
 #define	STEP_LIBRARYMENU	10
 #define	STEP_LIBRARYENTRY	20
@@ -129,6 +130,7 @@ pcb_lib_menu_t *pcb_lib_net_new(pcb_lib_t *lib, char *name, const char *style)
 		menu->Style = NULL;
 	else
 		menu->Style = pcb_strdup(style);
+	menu->type = PCB_OBJ_NET;
 	return menu;
 }
 
