@@ -55,6 +55,7 @@ static const char *EXPERIMENTAL = NULL;
 #include "flag_str.h"
 #include "plugins.h"
 #include "plug_footprint.h"
+#include "plug_import.h"
 #include "event.h"
 #include "funchash.h"
 #include "conf.h"
@@ -274,6 +275,7 @@ void pcb_main_uninit(void)
 	pcb_io_uninit();
 	pcb_plugin_uninit();
 	pcb_file_loaded_uninit();
+	pcb_import_uninit();
 }
 
 static int arg_match(const char *in, const char *shrt, const char *lng)
@@ -289,6 +291,7 @@ const char *pcb_action_args[] = {
 	NULL, "-dump-actions",    "DumpActions()",      "Print all available actions (script readable) and exit",  NULL,
 	NULL, "-dump-plugins",    "DumpPlugins()",      "Print all available plugins (script readable) and exit",  NULL,
 	NULL, "-dump-plugindirs", "DumpPluginDirs()",   "Print directories plugins might be loaded from and exit", NULL,
+	NULL, "-dump-oflags",     "DumpObjFlags()",     "Print object flags and exit",                             NULL,
 	NULL, "-show-paths",      "PrintPaths()",       "Print all configured paths and exit",                     NULL,
 	NULL, "-dump-config",     "dumpconf(native,1)", "Print the config tree and exit",                          "Config dump not available - make sure you have configured pcb-rnd with --buildin-diag",
 	"V",  "-version",         "PrintVersion()",     "Print version info and exit",                             NULL,
@@ -301,6 +304,57 @@ void print_pup_err(pup_err_stack_t *entry, char *string)
 	pcb_message(PCB_MSG_ERROR, "puplug: %s\n", string);
 }
 
+/* parse arguments using the gui; if fails and fallback is enabled, try the next gui */
+int gui_parse_argumemts(int autopick_gui, int *hid_argc, char **hid_argv[])
+{
+	conf_listitem_t *apg = NULL;
+
+	if ((autopick_gui >= 0) && (conf_core.rc.hid_fallback)) { /* start from the GUI we are initializing first */
+		int n;
+		const char *g;
+
+		conf_loop_list_str(&conf_core.rc.preferred_gui, apg, g, n) {
+			if (n == autopick_gui)
+				break;
+		}
+	}
+
+	for(;;) {
+		int res;
+		if (pcb_gui->get_export_options != NULL)
+			pcb_gui->get_export_options(NULL);
+		res = pcb_gui->parse_arguments(hid_argc, hid_argv);
+		if (res == 0)
+			break; /* HID accepted, don't try anything else */
+		if (res < 0) {
+			fprintf(stderr, "Failed to initialize HID %s (unrecoverable, have to give up)\n", pcb_gui->name);
+			return -1;
+		}
+		fprintf(stderr, "Failed to initialize HID %s (recoverable)\n", pcb_gui->name);
+		if (apg == NULL) {
+			if (conf_core.rc.hid_fallback) {
+				ran_out_of_hids:;
+				fprintf(stderr, "Tried all available HIDs, all failed, giving up.\n");
+			}
+			else
+				fprintf(stderr, "Not trying any other hid as fallback becase rc/hid_fallback is disabled.\n");
+			return -1;
+		}
+
+		/* falling back to the next HID */
+		do {
+			int n;
+			const char *g;
+
+			apg = conf_list_next_str(apg, &g, &n);
+			if (apg == NULL)
+				goto ran_out_of_hids;
+			pcb_gui = pcb_hid_find_gui(g);
+		} while(pcb_gui == NULL);
+	}
+	return 0;
+}
+
 
 int main(int argc, char *argv[])
 {
@@ -316,6 +370,7 @@ int main(int argc, char *argv[])
 	const char *main_action = NULL, *main_action_hint = NULL;
 	char *command_line_pcb = NULL;
 	vtp0_t plugin_cli_conf;
+	int autopick_gui = -1;
 
 #ifdef LOCALEDIR
 	bindtextdomain(GETTEXT_PACKAGE, LOCALEDIR);
@@ -447,6 +502,8 @@ int main(int argc, char *argv[])
 	pcb_pup.error_stack_enable = 1;
 	pup_buildin_load(&pcb_pup, pup_buildins);
 	pup_autoload_dir(&pcb_pup, NULL, NULL);
+	conf_load_extra(NULL, NULL);
+
 	if (pcb_pup.err_stack != NULL) {
 		pcb_message(PCB_MSG_ERROR, "Some of the static linked buildins could not be loaded:\n");
 		pup_err_stack_process_str(&pcb_pup, print_pup_err);
@@ -486,12 +543,11 @@ int main(int argc, char *argv[])
 			}
 			break;
 		default: {
-			conf_listitem_t *i;
-			int n;
 			const char *g;
+			conf_listitem_t *i;
 
 			pcb_gui = NULL;
-			conf_loop_list_str(&conf_core.rc.preferred_gui, i, g, n) {
+			conf_loop_list_str(&conf_core.rc.preferred_gui, i, g, autopick_gui) {
 				pcb_gui = pcb_hid_find_gui(g);
 				if (pcb_gui != NULL)
 					break;
@@ -524,7 +580,8 @@ int main(int argc, char *argv[])
 		exit(res);
 	}
 
-	pcb_gui->parse_arguments(&hid_argc, &hid_argv);
+	if (gui_parse_argumemts(autopick_gui, &hid_argc, &hid_argv) != 0)
+		exit(1);
 
 	/* Create a new PCB object in memory */
 	PCB = pcb_board_new(0);
@@ -546,7 +603,7 @@ int main(int argc, char *argv[])
 	InitHandler();
 	pcb_init_buffers(PCB);
 
-	pcb_crosshair_set_mode(PCB_MODE_ARROW);
+	pcb_tool_select_by_id(PCB_MODE_ARROW);
 
 	if (command_line_pcb) {
 		/* keep filename even if initial load command failed;
@@ -605,7 +662,7 @@ int main(int argc, char *argv[])
 			pcb_gui->parse_arguments(&hid_argc, &hid_argv);
 			if (pcb_gui->gui)
 				pcb_crosshair_init();
-			pcb_crosshair_set_mode(PCB_MODE_ARROW);
+			pcb_tool_select_by_id(PCB_MODE_ARROW);
 			pcb_event(PCB_EVENT_LIBRARY_CHANGED, NULL);
 		}
 	} while(pcb_gui != NULL);
