@@ -1,7 +1,8 @@
 /*
  *                            COPYRIGHT
  *
- *  PCB, interactive printed circuit board design
+ *  pcb-rnd, interactive printed circuit board design
+ *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996,2004,2006 Thomas Nau
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -18,9 +19,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- *  Contact addresses for paper mail and Email:
- *  Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
- *  Thomas.Nau@rz.uni-ulm.de
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
 
@@ -63,6 +65,42 @@ double pcb_get_value(const char *val, const char *units, pcb_bool * absolute, pc
 	return pcb_get_value_ex(val, units, absolute, NULL, "cmil", success);
 }
 
+pcb_bool pcb_get_value_unit(const char *val, pcb_bool *absolute, int unit_strict, double *val_out, const pcb_unit_t **unit_out)
+{
+	int ul = strspn(val, "0123456789.");
+	if ((ul > 0) && (val[ul] != '\0')) {
+		const char *unit = val+ul;
+		const pcb_unit_t *u;
+
+		while(isspace(*unit)) unit++;
+		if (*unit == '\0')
+			goto err;
+		else if ((unit[0] == 'm') && (unit[1] == '\0')) {
+			/* corner case: mil and mm starts with m, and we rarely want to specify anything in meter, so ignore it */
+			goto err;
+		}
+
+		u = get_unit_struct_(unit, unit_strict);
+		if (u != NULL) {
+			pcb_bool succ;
+			double crd;
+
+			crd = pcb_get_value(val, unit, NULL, &succ);
+			if (succ) {
+				*val_out = crd;
+				*unit_out = u;
+				return pcb_true;
+			}
+		}
+	}
+
+	err:;
+		*val_out = 0;
+		*unit_out = NULL;
+		return pcb_false;
+}
+
+
 double pcb_get_value_ex(const char *val, const char *units, pcb_bool * absolute, pcb_unit_list_t extra_units, const char *default_unit, pcb_bool *success)
 {
 	double value;
@@ -173,7 +211,7 @@ char *pcb_strdup_strip_wspace(const char *S)
 	size_t length;
 
 	if (!S || !*S)
-		return (NULL);
+		return NULL;
 
 	/* strip leading blanks */
 	for (p1 = S; *p1 && isspace((int) *p1); p1++);
@@ -187,8 +225,8 @@ char *pcb_strdup_strip_wspace(const char *S)
 		copy = (char *) realloc(NULL, length + 1);
 		strncpy(copy, p1, length);
 		copy[length] = '\0';
-		return (copy);
+		return copy;
 	}
 	else
-		return (NULL);
+		return NULL;
 }
