@@ -26,10 +26,6 @@
  *
  */
 
-
-/* functions used to move pins, elements ...
- */
-
 #include "config.h"
 #include "conf_core.h"
 
@@ -45,18 +41,18 @@
 #include "event.h"
 #include "hid_actions.h"
 #include "compat_misc.h"
-#include "obj_all_op.h"
+#include "obj_arc_op.h"
+#include "obj_line_op.h"
+#include "obj_text_op.h"
+#include "obj_subc_op.h"
+#include "obj_poly_op.h"
 #include "obj_pstk_op.h"
+#include "obj_rat_op.h"
 
 pcb_opfunc_t MoveFunctions = {
 	pcb_lineop_move,
 	pcb_textop_move,
 	pcb_polyop_move,
-	pcb_viaop_move,
-	pcb_elemop_move,
-	pcb_elemop_move_name,
-	NULL,
-	NULL,
 	pcb_lineop_move_point,
 	pcb_polyop_move_point,
 	pcb_arcop_move,
@@ -70,11 +66,6 @@ pcb_opfunc_t MoveFunctions_noclip = {
 	pcb_lineop_move_noclip,
 	pcb_textop_move_noclip,
 	pcb_polyop_move_noclip,
-	pcb_viaop_move_noclip,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
 	NULL,
 	NULL,
 	pcb_arcop_move_noclip,
@@ -88,11 +79,6 @@ pcb_opfunc_t ClipFunctions = {
 	pcb_lineop_clip,
 	pcb_textop_clip,
 	pcb_polyop_clip,
-	pcb_viaop_clip,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
 	NULL,
 	NULL,
 	pcb_arcop_clip,
@@ -108,11 +94,6 @@ static pcb_opfunc_t MoveToLayerFunctions = {
 	pcb_polyop_move_to_layer,
 	NULL,
 	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
 	pcb_arcop_move_to_layer,
 	pcb_ratop_move_to_layer,
 	NULL,
@@ -148,7 +129,7 @@ void *pcb_move_obj_and_rubberband(int Type, void *Ptr1, void *Ptr2, void *Ptr3,
 	pcb_draw_inhibit_inc();
 
 	switch(Type) {
-		case PCB_TYPE_ARC_POINT:
+		case PCB_OBJ_ARC_POINT:
 			{
 				/* Get the initial arc point positions */
 				pcb_arc_t * p_arc = ((pcb_arc_t *)pcb_crosshair.AttachedObject.Ptr2);
@@ -176,18 +157,18 @@ void *pcb_move_obj_and_rubberband(int Type, void *Ptr1, void *Ptr2, void *Ptr3,
 			}
 			break;
 
-		case PCB_TYPE_ARC:
+		case PCB_OBJ_ARC:
 			pcb_event(PCB_EVENT_RUBBER_MOVE, "icccc", 0, DX, DY, DX, DY);
 			pcb_undo_add_obj_to_move(Type, Ptr1, Ptr2, Ptr3, DX, DY);
 			ptr2 = pcb_object_operation(&MoveFunctions, &ctx, Type, Ptr1, Ptr2, Ptr3);
 			break;
 
-		case PCB_TYPE_LINE_POINT :
+		case PCB_OBJ_LINE_POINT :
 			pcb_event(PCB_EVENT_RUBBER_MOVE, "icc", 0, DX, DY);
 			ptr2 = pcb_lineop_move_point_with_route(&ctx, Ptr1, Ptr2, Ptr3);
 			break;
 
-		case PCB_TYPE_LINE:
+		case PCB_OBJ_LINE:
 			{
 				pcb_coord_t dx1 = DX;
 				pcb_coord_t dy1 = DY;
@@ -206,13 +187,13 @@ void *pcb_move_obj_and_rubberband(int Type, void *Ptr1, void *Ptr2, void *Ptr3,
 				/* If the line ends have moved indpendently then move the individual points */
 				if((dx1 != dx2) || (dy1 != dy2)) {
 					/* Move point1 form line */
-					pcb_undo_add_obj_to_move(PCB_TYPE_LINE_POINT,
+					pcb_undo_add_obj_to_move(PCB_OBJ_LINE_POINT,
 							 Ptr1, line, &line->Point1,
 							 dx1, dy1);
 					pcb_lineop_move_point(&ctx, Ptr1, line, &line->Point1);
 
 					/* Move point2 form line */
-					pcb_undo_add_obj_to_move(PCB_TYPE_LINE_POINT,
+					pcb_undo_add_obj_to_move(PCB_OBJ_LINE_POINT,
 								 Ptr1, line, &line->Point2,
 								 dx2, dy2);
 					ctx.move.dx = dx2;
@@ -265,7 +246,7 @@ pcb_bool pcb_move_selected_objs_to_layer(pcb_layer_t *Target)
 	ctx.move.dst_layer = Target;
 	ctx.move.more_to_come = pcb_true;
 
-	changed = pcb_selected_operation(PCB, PCB->Data, &MoveToLayerFunctions, &ctx, pcb_true, PCB_TYPEMASK_ALL);
+	changed = pcb_selected_operation(PCB, PCB->Data, &MoveToLayerFunctions, &ctx, pcb_true, PCB_OBJ_ANY);
 	/* passing pcb_true to above operation causes Undoserial to auto-increment */
 	return changed;
 }
