@@ -200,42 +200,6 @@ pcb_lib_menu_t *pcb_netlist_find_net4pinname(pcb_board_t *pcb, const char *pin)
 	return NULL;
 }
 
-static pcb_lib_menu_t *pcb_netlist_find_net4pin_any(pcb_board_t *pcb, const char *ename, const char *pname)
-{
-	char pinname[256];
-	int len;
-
-	if ((ename == NULL) || (pname == NULL))
-		return NULL;
-
-	len = pcb_snprintf(pinname, sizeof(pinname), "%s-%s", ename, pname);
-	if (len >= sizeof(pinname))
-		return NULL;
-
-	return pcb_netlist_find_net4pinname(pcb, pinname);
-}
-
-pcb_lib_menu_t *pcb_netlist_find_net4pin(pcb_board_t *pcb, const pcb_pin_t *pin)
-{
-	const pcb_element_t *e = pin->Element;
-
-	if (e == NULL)
-		return NULL;
-
-	return pcb_netlist_find_net4pin_any(pcb, e->Name[PCB_ELEMNAME_IDX_REFDES].TextString, pin->Number);
-}
-
-
-pcb_lib_menu_t *pcb_netlist_find_net4pad(pcb_board_t *pcb, const pcb_pad_t *pad)
-{
-	const pcb_element_t *e = pad->Element;
-
-	if (e == NULL)
-		return NULL;
-
-	return pcb_netlist_find_net4pin_any(pcb, e->Name[PCB_ELEMNAME_IDX_REFDES].TextString, pad->Number);
-}
-
 pcb_lib_menu_t *pcb_netlist_find_net4term(pcb_board_t *pcb, const pcb_any_obj_t *term)
 {
 	pcb_data_t *data;
@@ -247,7 +211,7 @@ pcb_lib_menu_t *pcb_netlist_find_net4term(pcb_board_t *pcb, const pcb_any_obj_t
 		return NULL;
 
 	if (term->parent_type == PCB_PARENT_LAYER)
-		data = term->parent.layer->parent;
+		data = term->parent.layer->parent.data;
 	else if (term->parent_type == PCB_PARENT_DATA)
 		data = term->parent.data;
 	else
@@ -282,7 +246,7 @@ pcb_cardinal_t pcb_netlist_net_idx(pcb_board_t *pcb, pcb_lib_menu_t *net)
 /* ---------------------------------------------------------------------------
  * get next slot for a subnet, allocates memory if necessary
  */
-pcb_net_t *pcb_net_new(pcb_netlist_t *Netlist)
+pcb_net_t *pcb_net_new(pcb_board_t *pcb, pcb_netlist_t *Netlist)
 {
 	pcb_net_t *net = Netlist->Net;
 
@@ -293,6 +257,10 @@ pcb_net_t *pcb_net_new(pcb_netlist_t *Netlist)
 		Netlist->Net = net;
 		memset(net + Netlist->NetN, 0, STEP_POINT * sizeof(pcb_net_t));
 	}
+
+	net->type = PCB_OBJ_NET;
+	net->parent_type = PCB_PARENT_BOARD;
+	net->parent.board = pcb;
 	return (net + Netlist->NetN++);
 }
 
