@@ -25,9 +25,6 @@
  *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
-
-/* functions used to create vias, pins ... */
-
 #include "config.h"
 
 #include <stddef.h>
@@ -35,7 +32,6 @@
 #include "conf_core.h"
 #include "flag_str.h"
 #include "global_typedefs.h"
-#include "const.h"
 #include "error.h"
 #include "obj_common.h"
 #include "obj_arc_ui.h"
@@ -44,25 +40,25 @@
 const char *pcb_obj_type_name(pcb_objtype_t type)
 {
 	switch(type) {
-		case PCB_OBJ_VOID:    return "void";
-		case PCB_OBJ_POINT:   return "point";
-		case PCB_OBJ_LINE:    return "line";
-		case PCB_OBJ_TEXT:    return "text";
-		case PCB_OBJ_POLY:    return "polygon";
-		case PCB_OBJ_ARC:     return "arc";
-		case PCB_OBJ_RAT:     return "ratline";
-		case PCB_OBJ_PAD:     return "pad";
-		case PCB_OBJ_PIN:     return "pin";
-		case PCB_OBJ_VIA:     return "via";
-		case PCB_OBJ_PSTK:    return "padstack";
-		case PCB_OBJ_ELEMENT: return "element";
-		case PCB_OBJ_SUBC:    return "subcircuit";
-		case PCB_OBJ_NET:     return "net";
-		case PCB_OBJ_LAYER:   return "layer";
-		case PCB_OBJ_ELINE:
-		case PCB_OBJ_EARC:
-		case PCB_OBJ_ETEXT:
-			break;
+		case PCB_OBJ_VOID:        return "void";
+		case PCB_OBJ_LINE:        return "line";
+		case PCB_OBJ_TEXT:        return "text";
+		case PCB_OBJ_POLY:        return "polygon";
+		case PCB_OBJ_ARC:         return "arc";
+		case PCB_OBJ_RAT:         return "ratline";
+		case PCB_OBJ_PSTK:        return "padstack";
+		case PCB_OBJ_SUBC:        return "subcircuit";
+		case PCB_OBJ_NET:         return "net";
+		case PCB_OBJ_LAYER:       return "layer";
+		case PCB_OBJ_LAYERGRP:    return "layer_group";
+	}
+	switch((pcb_vobjtype_t)type) {
+		case PCB_OBJ_LINE_POINT:  return "line_point";
+		case PCB_OBJ_POLY_POINT:  return "poly_point";
+		case PCB_OBJ_ARC_POINT:   return "arc_point";
+		case PCB_OBJ_SUBC_PART:   return "subc_part";
+		case PCB_OBJ_LOCKED:      return "locked";
+		case PCB_OBJ_FLOATER:     return "floater";
 	}
 	return "<unknown/composite>";
 }
@@ -73,26 +69,21 @@ const char *pcb_obj_type_name(pcb_objtype_t type)
 int pcb_obj_get_bbox(int Type, void *Ptr1, void *Ptr2, void *Ptr3, pcb_box_t *res)
 {
 	switch (Type) {
-	case PCB_TYPE_LINE:
-	case PCB_TYPE_ARC:
-	case PCB_TYPE_TEXT:
-	case PCB_TYPE_POLY:
-	case PCB_TYPE_PAD:
-	case PCB_TYPE_PIN:
-	case PCB_TYPE_PSTK:
-	case PCB_TYPE_ELEMENT_NAME:
+	case PCB_OBJ_LINE:
+	case PCB_OBJ_ARC:
+	case PCB_OBJ_TEXT:
+	case PCB_OBJ_POLY:
+	case PCB_OBJ_PSTK:
 		*res = *(pcb_box_t *)Ptr2;
 		return 0;
-	case PCB_TYPE_VIA:
-	case PCB_TYPE_ELEMENT:
-	case PCB_TYPE_SUBC:
+	case PCB_OBJ_SUBC:
 		*res = *(pcb_box_t *)Ptr1;
 		return 0;
-	case PCB_TYPE_POLY_POINT:
-	case PCB_TYPE_LINE_POINT:
+	case PCB_OBJ_POLY_POINT:
+	case PCB_OBJ_LINE_POINT:
 		*res = *(pcb_box_t *)Ptr3;
 		return 0;
-	case PCB_TYPE_ARC_POINT:
+	case PCB_OBJ_ARC_POINT:
 		return pcb_obj_ui_arc_point_bbox(Type, Ptr1, Ptr2, Ptr3, res);
 	default:
 		pcb_message(PCB_MSG_ERROR, "Request for bounding box of unsupported type %d\n", Type);
@@ -108,9 +99,6 @@ long int ID = 1;
 
 pcb_bool pcb_create_being_lenient = pcb_false;
 
-/* ---------------------------------------------------------------------------
- *  Set the lenience mode.
- */
 void pcb_create_be_lenient(pcb_bool v)
 {
 	pcb_create_being_lenient = v;
@@ -143,10 +131,6 @@ void pcb_obj_add_attribs(void *obj, const pcb_attribute_list_t *src)
 void pcb_obj_center(const pcb_any_obj_t *obj, pcb_coord_t *x, pcb_coord_t *y)
 {
 	switch (obj->type) {
-		case PCB_OBJ_PIN:
-			*x = ((const pcb_pin_t *)(obj))->X;
-			*y = ((const pcb_pin_t *)(obj))->Y;
-			break;
 		case PCB_OBJ_PSTK:
 			*x = ((const pcb_pstk_t *)(obj))->x;
 			*y = ((const pcb_pstk_t *)(obj))->y;
