@@ -38,6 +38,7 @@
 #include "data.h"
 #include "find.h"
 #include "rtree.h"
+#include "macro.h"
 
 static double drc_lines(pcb_point_t *end, pcb_bool way);
 
@@ -195,7 +196,7 @@ void pcb_line_adjust_attached_2lines(pcb_bool way)
 		}
 		else {
 			line->Point2.X = pcb_crosshair.X;
-			line->Point2.Y = line->Point1.Y + SGN(dy) * coord_abs(dx);;
+			line->Point2.Y = line->Point1.Y + SGN(dy) * coord_abs(dx);
 		}
 	}
 }
@@ -206,22 +207,12 @@ struct drc_info {
 	jmp_buf env;
 };
 
-static pcb_r_dir_t drcVia_callback(const pcb_box_t * b, void *cl)
+static pcb_r_dir_t drcPstk_callback(const pcb_box_t *b, void *cl)
 {
-	pcb_pin_t *via = (pcb_pin_t *) b;
-	struct drc_info *i = (struct drc_info *) cl;
+	pcb_pstk_t *ps = (pcb_pstk_t *)b;
+	struct drc_info *i = (struct drc_info *)cl;
 
-	if (!PCB_FLAG_TEST(PCB_FLAG_FOUND, via) && pcb_intersect_line_pin(via, i->line))
-		longjmp(i->env, 1);
-	return PCB_R_DIR_FOUND_CONTINUE;
-}
-
-static pcb_r_dir_t drcPad_callback(const pcb_box_t * b, void *cl)
-{
-	pcb_pad_t *pad = (pcb_pad_t *) b;
-	struct drc_info *i = (struct drc_info *) cl;
-
-	if (PCB_FLAG_TEST(PCB_FLAG_ONSOLDER, pad) == i->solder && !PCB_FLAG_TEST(PCB_FLAG_FOUND, pad) && pcb_intersect_line_pad(i->line, pad))
+	if (!PCB_FLAG_TEST(PCB_FLAG_FOUND, ps) && pcb_pstk_intersect_line(ps, i->line))
 		longjmp(i->env, 1);
 	return PCB_R_DIR_FOUND_CONTINUE;
 }
@@ -269,8 +260,11 @@ static double drc_lines(pcb_point_t *end, pcb_bool way)
 	f = 1.0;
 	s = 0.5;
 	last = -1;
+	line1.type = line2.type = PCB_OBJ_LINE;
 	line1.Flags = line2.Flags = pcb_no_flags();
-	line1.Thickness = conf_core.design.line_thickness + 2 * (PCB->Bloat + 1);
+	line1.parent_type = line2.parent_type = PCB_PARENT_LAYER;
+	line1.parent.layer = line2.parent.layer = CURRENT;
+	line1.Thickness = conf_core.design.line_thickness + 2 * (conf_core.design.bloat + 1);
 	line2.Thickness = line1.Thickness;
 	line1.Clearance = line2.Clearance = 0;
 	line1.Point1.X = pcb_crosshair.AttachedLine.Point1.X;
@@ -356,16 +350,10 @@ static double drc_lines(pcb_point_t *end, pcb_bool way)
 			last2 = length2;
 			if (setjmp(info.env) == 0) {
 				info.line = &line1;
-				pcb_r_search(PCB->Data->via_tree, &line1.BoundingBox, NULL, drcVia_callback, &info, NULL);
-				pcb_r_search(PCB->Data->pin_tree, &line1.BoundingBox, NULL, drcVia_callback, &info, NULL);
-				if (info.solder || comp == group)
-					pcb_r_search(PCB->Data->pad_tree, &line1.BoundingBox, NULL, drcPad_callback, &info, NULL);
+				pcb_r_search(PCB->Data->padstack_tree, &line1.BoundingBox, NULL, drcPstk_callback, &info, NULL);
 				if (two_lines) {
 					info.line = &line2;
-					pcb_r_search(PCB->Data->via_tree, &line2.BoundingBox, NULL, drcVia_callback, &info, NULL);
-					pcb_r_search(PCB->Data->pin_tree, &line2.BoundingBox, NULL, drcVia_callback, &info, NULL);
-					if (info.solder || comp == group)
-						pcb_r_search(PCB->Data->pad_tree, &line2.BoundingBox, NULL, drcPad_callback, &info, NULL);
+					pcb_r_search(PCB->Data->padstack_tree, &line2.BoundingBox, NULL, drcPstk_callback, &info, NULL);
 				}
 				PCB_COPPER_GROUP_LOOP(PCB->Data, group);
 				{
@@ -436,6 +424,9 @@ static void drc_line(pcb_point_t *end)
 	aline.Point1.X = pcb_crosshair.AttachedLine.Point1.X;
 	aline.Point1.Y = pcb_crosshair.AttachedLine.Point1.Y;
 	pcb_line_45(&aline);
+	line.parent_type = PCB_PARENT_LAYER;
+	line.parent.layer = CURRENT;
+	line.type = PCB_OBJ_LINE;
 	line.Point1 = aline.Point1;
 	line.Point2 = aline.Point2;
 
@@ -453,10 +444,7 @@ static void drc_line(pcb_point_t *end)
 	pcb_line_bbox(&line);
 	if (setjmp(info.env) == 0) {
 		info.line = &line;
-		pcb_r_search(PCB->Data->via_tree, &line.BoundingBox, NULL, drcVia_callback, &info, NULL);
-		pcb_r_search(PCB->Data->pin_tree, &line.BoundingBox, NULL, drcVia_callback, &info, NULL);
-		if (info.solder || comp == group)
-			pcb_r_search(PCB->Data->pad_tree, &line.BoundingBox, NULL, drcPad_callback, &info, NULL);
+		pcb_r_search(PCB->Data->padstack_tree, &line.BoundingBox, NULL, drcPstk_callback, &info, NULL);
 		PCB_COPPER_GROUP_LOOP(PCB->Data, group);
 		{
 			info.line = &line;
