@@ -142,14 +142,16 @@ void pcb_poly_rotate90(pcb_poly_t *Polygon, pcb_coord_t X, pcb_coord_t Y, unsign
 
 void pcb_poly_rotate(pcb_layer_t *layer, pcb_poly_t *polygon, pcb_coord_t X, pcb_coord_t Y, double cosa, double sina)
 {
-	pcb_r_delete_entry(layer->polygon_tree, (pcb_box_t *) polygon);
+	if (layer->polygon_tree != NULL)
+		pcb_r_delete_entry(layer->polygon_tree, (pcb_box_t *) polygon);
 	PCB_POLY_POINT_LOOP(polygon);
 	{
 		pcb_rotate(&point->X, &point->Y, X, Y, cosa, sina);
 	}
 	PCB_END_LOOP;
 	pcb_poly_bbox(polygon);
-	pcb_r_insert_entry(layer->polygon_tree, (pcb_box_t *) polygon);
+	if (layer->polygon_tree != NULL)
+		pcb_r_insert_entry(layer->polygon_tree, (pcb_box_t *) polygon);
 }
 
 void pcb_poly_mirror(pcb_layer_t *layer, pcb_poly_t *polygon, pcb_coord_t y_offs)
@@ -293,7 +295,7 @@ void pcb_add_poly_on_layer(pcb_layer_t *Layer, pcb_poly_t *polygon)
 		Layer->polygon_tree = pcb_r_create_tree();
 	pcb_r_insert_entry(Layer->polygon_tree, (pcb_box_t *) polygon);
 	PCB_SET_PARENT(polygon, layer, Layer);
-	pcb_poly_clear_from_poly(Layer->parent, PCB_TYPE_POLY, Layer, polygon);
+	pcb_poly_clear_from_poly(Layer->parent.data, PCB_OBJ_POLY, Layer, polygon);
 }
 
 /* creates a new polygon on a layer */
@@ -327,7 +329,7 @@ pcb_poly_t *pcb_poly_dup(pcb_layer_t *dst, pcb_poly_t *src)
 	pcb_add_poly_on_layer(dst, p);
 	pcb_poly_copy_meta(p, src);
 
-	pcb = pcb_data_get_top(dst->parent);
+	pcb = pcb_data_get_top(dst->parent.data);
 	if (pcb != NULL)
 		pcb_poly_init_clip(pcb->Data, dst, p);
 	return p;
@@ -341,7 +343,7 @@ pcb_poly_t *pcb_poly_dup_at(pcb_layer_t *dst, pcb_poly_t *src, pcb_coord_t dx, p
 	pcb_add_poly_on_layer(dst, p);
 	pcb_poly_copy_meta(p, src);
 
-	pcb = pcb_data_get_top(dst->parent);
+	pcb = pcb_data_get_top(dst->parent.data);
 	if (pcb != NULL)
 		pcb_poly_init_clip(pcb->Data, dst, p);
 	return p;
@@ -485,19 +487,19 @@ void *pcb_polyop_change_clear_size(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_pol
 			return NULL;
 
 		value = MIN(PCB_MAX_LINESIZE, value);
-		if (!ctx->chgsize.is_absolute && ctx->chgsize.value < 0 && value < ctx->chgsize.pcb->Bloat * 2)
+		if (!ctx->chgsize.is_absolute && ctx->chgsize.value < 0 && value < conf_core.design.bloat * 2)
 			value = 0;
-		if (ctx->chgsize.value > 0 && value < ctx->chgsize.pcb->Bloat * 2)
-			value = ctx->chgsize.pcb->Bloat * 2 + 2;
+		if (ctx->chgsize.value > 0 && value < conf_core.design.bloat * 2)
+			value = conf_core.design.bloat * 2 + 2;
 		if (value != poly->Clearance) {
-			pcb_undo_add_obj_to_clear_size(PCB_TYPE_POLY, Layer, poly, poly);
-			pcb_poly_restore_to_poly(ctx->chgsize.pcb->Data, PCB_TYPE_POLY, Layer, poly);
+			pcb_undo_add_obj_to_clear_size(PCB_OBJ_POLY, Layer, poly, poly);
+			pcb_poly_restore_to_poly(ctx->chgsize.pcb->Data, PCB_OBJ_POLY, Layer, poly);
 			pcb_poly_invalidate_erase(poly);
 			pcb_r_delete_entry(Layer->polygon_tree, (pcb_box_t *)poly);
 			poly->Clearance = value;
 			pcb_poly_bbox(poly);
 			pcb_r_insert_entry(Layer->polygon_tree, (pcb_box_t *)poly);
-			pcb_poly_clear_from_poly(ctx->chgsize.pcb->Data, PCB_TYPE_POLY, Layer, poly);
+			pcb_poly_clear_from_poly(ctx->chgsize.pcb->Data, PCB_OBJ_POLY, Layer, poly);
 			pcb_poly_invalidate_draw(Layer, poly);
 			return poly;
 		}
@@ -520,7 +522,7 @@ void *pcb_polyop_change_clear(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_poly_t *
 {
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Polygon))
 		return NULL;
-	pcb_undo_add_obj_to_clear_poly(PCB_TYPE_POLY, Layer, Polygon, Polygon, pcb_true);
+	pcb_undo_add_obj_to_clear_poly(PCB_OBJ_POLY, Layer, Polygon, Polygon, pcb_true);
 	pcb_undo_add_obj_to_flag(Polygon);
 	PCB_FLAG_TOGGLE(PCB_FLAG_CLEARPOLY, Polygon);
 	pcb_poly_init_clip(PCB->Data, Layer, Polygon);
@@ -563,7 +565,7 @@ void *pcb_polyop_insert_point(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_poly_t *
 
 	Polygon->Points[ctx->insert.idx] = save;
 	pcb_board_set_changed_flag(pcb_true);
-	pcb_undo_add_obj_to_insert_point(PCB_TYPE_POLY_POINT, Layer, Polygon, &Polygon->Points[ctx->insert.idx]);
+	pcb_undo_add_obj_to_insert_point(PCB_OBJ_POLY_POINT, Layer, Polygon, &Polygon->Points[ctx->insert.idx]);
 
 	pcb_poly_bbox(Polygon);
 	pcb_r_insert_entry(Layer->polygon_tree, (pcb_box_t *) Polygon);
@@ -583,14 +585,14 @@ void *pcb_polyop_change_join(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_poly_t *p
 		return NULL;
 	pcb_poly_invalidate_erase(poly);
 	if (PCB_FLAG_TEST(PCB_FLAG_CLEARPOLYPOLY, poly)) {
-		pcb_undo_add_obj_to_clear_poly(PCB_TYPE_POLY, Layer, poly, poly, pcb_false);
-		pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_POLY, Layer, poly);
+		pcb_undo_add_obj_to_clear_poly(PCB_OBJ_POLY, Layer, poly, poly, pcb_false);
+		pcb_poly_restore_to_poly(PCB->Data, PCB_OBJ_POLY, Layer, poly);
 	}
 	pcb_undo_add_obj_to_flag(poly);
 	PCB_FLAG_TOGGLE(PCB_FLAG_CLEARPOLYPOLY, poly);
 	if (PCB_FLAG_TEST(PCB_FLAG_CLEARPOLYPOLY, poly)) {
-		pcb_undo_add_obj_to_clear_poly(PCB_TYPE_POLY, Layer, poly, poly, pcb_true);
-		pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_POLY, Layer, poly);
+		pcb_undo_add_obj_to_clear_poly(PCB_OBJ_POLY, Layer, poly, poly, pcb_true);
+		pcb_poly_clear_from_poly(PCB->Data, PCB_OBJ_POLY, Layer, poly);
 	}
 	pcb_poly_invalidate_draw(Layer, poly);
 	return poly;
@@ -679,32 +681,10 @@ void *pcb_polyop_move_to_layer_low(pcb_opctx_t *ctx, pcb_layer_t * Source, pcb_p
 	return polygon;
 }
 
-struct mptlc {
-	pcb_layer_id_t snum, dnum;
-	int type;
-	pcb_poly_t *polygon;
-} mptlc;
-
-pcb_r_dir_t mptl_pin_callback(const pcb_box_t * b, void *cl)
-{
-	struct mptlc *d = (struct mptlc *) cl;
-	pcb_pin_t *pin = (pcb_pin_t *) b;
-	if (!PCB_FLAG_THERM_TEST(d->snum, pin) || !pcb_poly_is_point_in_p(pin->X, pin->Y, pin->Thickness + pin->Clearance + 2, d->polygon))
-		return PCB_R_DIR_NOT_FOUND;
-	if (d->type == PCB_TYPE_PIN)
-		pcb_undo_add_obj_to_flag(pin);
-	else
-		pcb_undo_add_obj_to_flag(pin);
-	PCB_FLAG_THERM_ASSIGN(d->dnum, PCB_FLAG_THERM_GET(d->snum, pin), pin);
-	PCB_FLAG_THERM_CLEAR(d->snum, pin);
-	return PCB_R_DIR_FOUND_CONTINUE;
-}
-
 /* moves a polygon between layers */
 void *pcb_polyop_move_to_layer(pcb_opctx_t *ctx, pcb_layer_t * Layer, pcb_poly_t * Polygon)
 {
 	pcb_poly_t *newone;
-	struct mptlc d;
 
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Polygon)) {
 		pcb_message(PCB_MSG_WARNING, _("Sorry, the object is locked\n"));
@@ -712,17 +692,10 @@ void *pcb_polyop_move_to_layer(pcb_opctx_t *ctx, pcb_layer_t * Layer, pcb_poly_t
 	}
 	if (Layer == ctx->move.dst_layer)
 		return Polygon;
-	pcb_undo_add_obj_to_move_to_layer(PCB_TYPE_POLY, Layer, Polygon, Polygon);
+	pcb_undo_add_obj_to_move_to_layer(PCB_OBJ_POLY, Layer, Polygon, Polygon);
 	if (Layer->meta.real.vis)
 		pcb_poly_invalidate_erase(Polygon);
-	/* Move all of the thermals with the polygon */
-	d.snum = pcb_layer_id(PCB->Data, Layer);
-	d.dnum = pcb_layer_id(PCB->Data, ctx->move.dst_layer);
-	d.polygon = Polygon;
-	d.type = PCB_TYPE_PIN;
-	pcb_r_search(PCB->Data->pin_tree, &Polygon->BoundingBox, NULL, mptl_pin_callback, &d, NULL);
-	d.type = PCB_TYPE_VIA;
-	pcb_r_search(PCB->Data->via_tree, &Polygon->BoundingBox, NULL, mptl_pin_callback, &d, NULL);
+
 	newone = (struct pcb_poly_s *) pcb_polyop_move_to_layer_low(ctx, Layer, Polygon, ctx->move.dst_layer);
 	pcb_poly_init_clip(PCB->Data, ctx->move.dst_layer, newone);
 	if (ctx->move.dst_layer->meta.real.vis)
@@ -786,7 +759,7 @@ void *pcb_polyop_remove(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_poly_t *Polygo
 	pcb_poly_pprestore(Polygon);
 	if (Layer->meta.real.vis)
 		pcb_poly_invalidate_erase(Polygon);
-	pcb_undo_move_obj_to_remove(PCB_TYPE_POLY, Layer, Polygon, Polygon);
+	pcb_undo_move_obj_to_remove(PCB_OBJ_POLY, Layer, Polygon, Polygon);
 	return NULL;
 }
 
@@ -819,7 +792,7 @@ void *pcb_polyop_remove_counter(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_poly_t
 		pcb_poly_invalidate_erase(Polygon);
 
 	/* Copy the polygon to the undo list */
-	pcb_undo_add_obj_to_remove_contour(PCB_TYPE_POLY, Layer, Polygon);
+	pcb_undo_add_obj_to_remove_contour(PCB_OBJ_POLY, Layer, Polygon);
 
 	contour_start = (contour == 0) ? 0 : Polygon->HoleIndex[contour - 1];
 	contour_end = (contour == Polygon->HoleIndexN) ? Polygon->PointN : Polygon->HoleIndex[contour];
@@ -868,7 +841,7 @@ void *pcb_polyop_remove_point(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_poly_t *
 		pcb_poly_invalidate_erase(Polygon);
 
 	/* insert the polygon-point into the undo list */
-	pcb_undo_add_obj_to_remove_point(PCB_TYPE_POLY_POINT, Layer, Polygon, point_idx);
+	pcb_undo_add_obj_to_remove_point(PCB_OBJ_POLY_POINT, Layer, Polygon, point_idx);
 	pcb_r_delete_entry(Layer->polygon_tree, (pcb_box_t *) Polygon);
 
 	/* remove point from list, keep point order */
@@ -907,7 +880,7 @@ void *pcb_polyop_copy(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_poly_t *Polygon)
 	pcb_r_insert_entry(Layer->polygon_tree, (pcb_box_t *) polygon);
 	pcb_poly_init_clip(PCB->Data, Layer, polygon);
 	pcb_poly_invalidate_draw(Layer, polygon);
-	pcb_undo_add_obj_to_create(PCB_TYPE_POLY, Layer, polygon, polygon);
+	pcb_undo_add_obj_to_create(PCB_OBJ_POLY, Layer, polygon, polygon);
 	pcb_poly_ppclear(polygon);
 	return polygon;
 }
@@ -917,9 +890,11 @@ void *pcb_polyop_rotate90(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_poly_t *Poly
 	pcb_poly_pprestore(Polygon);
 	if (Layer->meta.real.vis)
 		pcb_poly_invalidate_erase(Polygon);
-	pcb_r_delete_entry(Layer->polygon_tree, (pcb_box_t *) Polygon);
+	if (Layer->polygon_tree != NULL)
+		pcb_r_delete_entry(Layer->polygon_tree, (pcb_box_t *) Polygon);
 	pcb_poly_rotate90(Polygon, ctx->rotate.center_x, ctx->rotate.center_y, ctx->rotate.number);
-	pcb_r_insert_entry(Layer->polygon_tree, (pcb_box_t *) Polygon);
+	if (Layer->polygon_tree != NULL)
+		pcb_r_insert_entry(Layer->polygon_tree, (pcb_box_t *) Polygon);
 	pcb_poly_init_clip(PCB->Data, Layer, Polygon);
 	if (Layer->meta.real.vis)
 		pcb_poly_invalidate_draw(Layer, Polygon);
@@ -944,7 +919,7 @@ void *pcb_polyop_change_flag(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_poly_t *P
 {
 	static pcb_flag_values_t pcb_poly_flags = 0;
 	if (pcb_poly_flags == 0)
-		pcb_poly_flags = pcb_obj_valid_flags(PCB_TYPE_POLY);
+		pcb_poly_flags = pcb_obj_valid_flags(PCB_OBJ_POLY);
 
 	if ((ctx->chgflag.flag & pcb_poly_flags) != ctx->chgflag.flag)
 		return NULL;
@@ -954,7 +929,7 @@ void *pcb_polyop_change_flag(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_poly_t *P
 
 
 	if (ctx->chgflag.flag & (PCB_FLAG_CLEARPOLY | PCB_FLAG_CLEARPOLYPOLY))
-		pcb_poly_restore_to_poly(ctx->chgflag.pcb->Data, PCB_TYPE_POLY, Polygon->parent.layer, Polygon);
+		pcb_poly_restore_to_poly(ctx->chgflag.pcb->Data, PCB_OBJ_POLY, Polygon->parent.layer, Polygon);
 
 	PCB_FLAG_CHANGE(ctx->chgflag.how, ctx->chgflag.flag, Polygon);
 
@@ -962,7 +937,7 @@ void *pcb_polyop_change_flag(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_poly_t *P
 		pcb_poly_init_clip(ctx->chgflag.pcb->Data, Polygon->parent.layer, Polygon);
 
 	if (ctx->chgflag.flag & (PCB_FLAG_CLEARPOLY | PCB_FLAG_CLEARPOLYPOLY))
-		pcb_poly_clear_from_poly(ctx->chgflag.pcb->Data, PCB_TYPE_POLY, Polygon->parent.layer, Polygon);
+		pcb_poly_clear_from_poly(ctx->chgflag.pcb->Data, PCB_OBJ_POLY, Polygon->parent.layer, Polygon);
 
 	return Polygon;
 }
@@ -1132,6 +1107,9 @@ pcb_r_dir_t pcb_poly_draw_term_callback(const pcb_box_t * b, void *cl)
 	pcb_draw_info_t *i = cl;
 	pcb_poly_t *polygon = (pcb_poly_t *) b;
 
+	if (pcb_hidden_floater((pcb_any_obj_t*)b))
+		return PCB_R_DIR_FOUND_CONTINUE;
+
 	if (!polygon->Clipped)
 		return PCB_R_DIR_NOT_FOUND;
 
