@@ -46,6 +46,7 @@
 #include "search.h"
 #include "undo.h"
 #include "vtpadstack.h"
+#include "event.h"
 
 #define SQR(o) ((o)*(o))
 
@@ -93,7 +94,8 @@ pcb_pstk_t *pcb_pstk_new_tr(pcb_data_t *data, pcb_cardinal_t proto, pcb_coord_t
 	ps->xmirror = xmirror;
 	ps->smirror = smirror;
 	pcb_pstk_add(data, ps);
-	pcb_poly_clear_from_poly(data, PCB_TYPE_PSTK, NULL, ps);
+	pcb_event(PCB_EVENT_NEW_PSTK, "p", ps);
+	pcb_poly_clear_from_poly(data, PCB_OBJ_PSTK, NULL, ps);
 	return ps;
 }
 
@@ -379,7 +381,7 @@ static void pcb_pstk_draw_shape_thin(pcb_hid_gc_t gc, pcb_pstk_t *ps, pcb_pstk_s
 			pcb_draw_wireframe_line(gc, ps->x + shape->data.line.x1, ps->y + shape->data.line.y1, ps->x + shape->data.line.x2, ps->y + shape->data.line.y2, shape->data.line.thickness, shape->data.line.square);
 			break;
 		case PCB_PSSH_CIRC:
-			pcb_gui->draw_arc(gc, ps->x + shape->data.circ.x, ps->y + shape->data.circ.y, shape->data.circ.dia/2, shape->data.circ.dia/2, 0, 360); 
+			pcb_gui->draw_arc(gc, ps->x + shape->data.circ.x, ps->y + shape->data.circ.y, shape->data.circ.dia/2, shape->data.circ.dia/2, 0, 360);
 			break;
 	}
 }
@@ -411,7 +413,7 @@ pcb_r_dir_t pcb_pstk_draw_callback(const pcb_box_t *b, void *cl)
 		if (grp == NULL)
 			set_ps_color(ps, ctx->is_current, ctx->shape_mask);
 		else
-			set_ps_color(ps, ctx->is_current, grp->type);
+			set_ps_color(ps, ctx->is_current, grp->ltype);
 		if (conf_core.editor.thin_draw || conf_core.editor.wireframe_draw) {
 			pcb_gui->set_line_width(pcb_draw_out.fgGC, 0);
 			pcb_pstk_draw_shape_thin(pcb_draw_out.fgGC, ps, shape);
@@ -561,7 +563,7 @@ void pcb_pstk_invalidate_draw(pcb_pstk_t *ps)
 
 static int pcb_pstk_near_box_(pcb_pstk_t *ps, pcb_box_t *box, pcb_pstk_shape_t *shape)
 {
-	pcb_pad_t pad;
+	pcb_any_line_t pad;
 	pcb_vector_t v;
 	pcb_pstk_proto_t *proto = pcb_pstk_get_proto(ps);
 	int isneg = PCB_IS_BOX_NEGATIVE(box), is_in, n;
@@ -691,7 +693,7 @@ int pcb_pstk_near_box(pcb_pstk_t *ps, pcb_box_t *box, pcb_layer_t *layer)
 
 static int pcb_is_point_in_pstk_(pcb_pstk_t *ps, pcb_coord_t x, pcb_coord_t y, pcb_coord_t radius, pcb_pstk_shape_t *shape)
 {
-	pcb_pad_t pad;
+	pcb_any_line_t pad;
 	pcb_vector_t v;
 
 	switch(shape->shape) {
@@ -705,7 +707,7 @@ static int pcb_is_point_in_pstk_(pcb_pstk_t *ps, pcb_coord_t x, pcb_coord_t y, p
 			pad.Thickness = shape->data.line.thickness;
 			pad.Clearance = 0;
 			pad.Flags = pcb_flag_make(shape->data.line.square ? PCB_FLAG_SQUARE : 0);
-			return pcb_is_point_in_pad(x, y, radius, &pad);
+			return pcb_is_point_in_line(x, y, radius, &pad);
 		case PCB_PSSH_POLY:
 			if (shape->data.poly.pa == NULL)
 				pcb_pstk_shape_update_pa(&shape->data.poly);
@@ -757,13 +759,13 @@ int pcb_pstk_drc_check_clearance(pcb_pstk_t *ps, pcb_poly_t *polygon, pcb_coord_
 
 	/* global clearance */
 	if (ps->Clearance > 0)
-		return ps->Clearance < 2 * PCB->Bloat;
+		return ps->Clearance < 2 * conf_core.design.bloat;
 
 	/* else check each shape; it's safest to run this check on the canonical
 	   transformed shape, that's always available */
 	ts = pcb_pstk_get_tshape_(ps->parent.data, ps->proto, 0);
 	for(n = 0; n < ts->len; n++)
-		if ((ts->shape[n].clearance > 0) && (ts->shape[n].clearance < 2 * PCB->Bloat))
+		if ((ts->shape[n].clearance > 0) && (ts->shape[n].clearance < 2 * conf_core.design.bloat))
 			return 1;
 
 	return 0;
@@ -835,28 +837,32 @@ void pcb_pstk_drc_check_and_warn(pcb_pstk_t *ps, pcb_coord_t *err_minring, pcb_c
 		pcb_pstk_tshape_t *ts = pcb_pstk_get_tshape_(ps->parent.data, ps->proto, 0);
 
 		for(n = 0; n < ts->len; n++) {
-			if (pcb_pstk_shape_hole_break(&ts->shape[n], proto->hdia, 2 * PCB->minRing, err_minring)) {
+			if (pcb_pstk_shape_hole_break(&ts->shape[n], proto->hdia, 2 * conf_core.design.min_ring, err_minring)) {
 				break;
 			}
 		}
 	}
 
-	if ((proto->hdia > 0) && (proto->hdia < PCB->minDrill))
+	if ((proto->hdia > 0) && (proto->hdia < conf_core.design.min_drill))
 		*err_minhole = proto->hdia;
 }
 
-void pcb_pstk_mirror(pcb_pstk_t *ps, pcb_coord_t y_offs, int swap_side)
+void pcb_pstk_mirror(pcb_pstk_t *ps, pcb_coord_t y_offs, int swap_side, int disable_xmirror)
 {
 	int xmirror = !ps->xmirror, smirror = (swap_side ? (!ps->smirror) : ps->smirror);
 
+
 	/* change the mirror flag - this will automatically cause mirroring in
 	   every aspect */
-	pcb_pstk_change_instance(ps, NULL, NULL, NULL, &xmirror, &smirror);
+	if (disable_xmirror)
+		pcb_pstk_change_instance(ps, NULL, NULL, NULL, NULL, &smirror);
+	else
+		pcb_pstk_change_instance(ps, NULL, NULL, NULL, &xmirror, &smirror);
 
 	/* if mirror center is not 0, also move, to emulate that the mirror took
 	   place around that point */
-	if (y_offs != 0) {
-		pcb_poly_restore_to_poly(ps->parent.data, PCB_TYPE_PSTK, NULL, ps);
+	if (((y_offs != 0) || (ps->y != 0)) && (y_offs != PCB_PSTK_DONT_MIRROR_COORDS)) {
+		pcb_poly_restore_to_poly(ps->parent.data, PCB_OBJ_PSTK, NULL, ps);
 		pcb_pstk_invalidate_erase(ps);
 		if (ps->parent.data->padstack_tree != NULL)
 			pcb_r_delete_entry(ps->parent.data->padstack_tree, (pcb_box_t *)ps);
@@ -866,7 +872,7 @@ void pcb_pstk_mirror(pcb_pstk_t *ps, pcb_coord_t y_offs, int swap_side)
 
 		if (ps->parent.data->padstack_tree != NULL)
 			pcb_r_insert_entry(ps->parent.data->padstack_tree, (pcb_box_t *)ps);
-		pcb_poly_clear_from_poly(ps->parent.data, PCB_TYPE_PSTK, NULL, ps);
+		pcb_poly_clear_from_poly(ps->parent.data, PCB_OBJ_PSTK, NULL, ps);
 		pcb_pstk_invalidate_draw(ps);
 	}
 }
@@ -944,7 +950,7 @@ static int undo_change_instance_swap(void *udata)
 		return -1;
 	}
 
-	pcb_poly_restore_to_poly(ps->parent.data, PCB_TYPE_PSTK, NULL, ps);
+	pcb_poly_restore_to_poly(ps->parent.data, PCB_OBJ_PSTK, NULL, ps);
 	pcb_pstk_invalidate_erase(ps);
 	if (ps->parent.data->padstack_tree != NULL)
 		pcb_r_delete_entry(ps->parent.data->padstack_tree, (pcb_box_t *)ps);
@@ -962,7 +968,7 @@ static int undo_change_instance_swap(void *udata)
 	pcb_pstk_bbox(ps);
 	if (ps->parent.data->padstack_tree != NULL)
 		pcb_r_insert_entry(ps->parent.data->padstack_tree, (pcb_box_t *)ps);
-	pcb_poly_clear_from_poly(ps->parent.data, PCB_TYPE_PSTK, NULL, ps);
+	pcb_poly_clear_from_poly(ps->parent.data, PCB_OBJ_PSTK, NULL, ps);
 	pcb_pstk_invalidate_draw(ps);
 
 	return 0;
@@ -996,6 +1002,10 @@ int pcb_pstk_change_instance(pcb_pstk_t *ps, pcb_cardinal_t *proto, const pcb_co
 			ps->rot = rot ? *rot : ps->rot;
 			ps->xmirror = xmirror ? *xmirror : ps->xmirror;
 			ps->smirror = smirror ? *smirror : ps->smirror;
+
+			/* re-render the prototype so the new shape is generated and cached */
+			ps->protoi = -1;
+			pcb_pstk_get_tshape(ps);
 			return 0;
 		case PCB_PARENT_BOARD: parent_ID = -1; break;
 		case PCB_PARENT_SUBC: parent_ID = ps->parent.data->parent.subc->ID; break;
@@ -1066,7 +1076,7 @@ void pcb_pstk_rotate(pcb_pstk_t *pstk, pcb_coord_t cx, pcb_coord_t cy, double co
 	pcb_pstkop_rotate(&ctx, pstk);
 }
 
-pcb_coord_t obj_pstk_get_clearance(pcb_board_t *pcb, const pcb_pstk_t *ps, pcb_layer_t *layer)
+pcb_coord_t obj_pstk_get_clearance(pcb_board_t *pcb, pcb_pstk_t *ps, pcb_layer_t *layer)
 {
 	pcb_pstk_shape_t *shp;
 
