@@ -34,7 +34,7 @@
 /* The actual padstack is just a reference to a padstack proto within the same data */
 struct pcb_pstk_s {
 #define thermal thermal_dont_use
-	PCB_ANYOBJECTFIELDS;
+	PCB_ANY_PRIMITIVE_FIELDS;
 #undef thermal
 	pcb_cardinal_t proto;          /* reference to a pcb_pstk_proto_t within pcb_data_t */
 	int protoi;                    /* index of the transformed proto; -1 means invalid; local cache, not saved */
@@ -67,6 +67,7 @@ typedef struct pcb_pstk_proto_s {
 	unsigned in_use:1;             /* 1 if the slot is in use */
 
 	unsigned hplated:1;            /* if > 0, whether the hole is plated */
+	char *name;                    /* optional user assigned name (or NULL) */
 	pcb_coord_t hdia;              /* if > 0, diameter of the hole (else there's no hole) */
 	int htop, hbottom;             /* if hdia > 0, determine the hole's span, counted in copper layer groups from the top or bottom copper layer group */
 
@@ -110,7 +111,7 @@ pcb_pstk_t *pcb_pstk_copy_meta(pcb_pstk_t *dst, pcb_pstk_t *src);
 pcb_pstk_t *pcb_pstk_copy_orient(pcb_pstk_t *dst, pcb_pstk_t *src);
 
 /* Clearance of the padstack on a given layer */
-pcb_coord_t obj_pstk_get_clearance(pcb_board_t *pcb, const pcb_pstk_t *ps, pcb_layer_t *layer);
+pcb_coord_t obj_pstk_get_clearance(pcb_board_t *pcb, pcb_pstk_t *ps, pcb_layer_t *layer);
 
 /*** proto ***/
 
@@ -156,6 +157,10 @@ pcb_cardinal_t pcb_pstk_proto_insert_dup(pcb_data_t *data, const pcb_pstk_proto_
    Returns 0 on success. */
 int pcb_pstk_proto_change_hole(pcb_pstk_proto_t *proto, const int *hplated, const pcb_coord_t *hdia, const int *htop, const int *hbottom);
 
+/* Change the name of a padstack proto; not yet undoable.
+   Returns 0 on success. */
+int pcb_pstk_proto_change_name(pcb_pstk_proto_t *proto, const char *new_name);
+
 /* Find or create a new transformed version of an existing proto */
 pcb_pstk_tshape_t *pcb_pstk_make_tshape(pcb_data_t *data, pcb_pstk_proto_t *proto, double rot, int xmirror, int smirror, int *out_protoi);
 
@@ -182,8 +187,13 @@ int pcb_pstk_get_shape_idx(pcb_pstk_tshape_t *ts, pcb_layer_type_t lyt, pcb_laye
 void pcb_pstk_proto_del_shape(pcb_pstk_proto_t *proto, pcb_layer_type_t lyt, pcb_layer_combining_t comb);
 void pcb_pstk_proto_del_shape_idx(pcb_pstk_proto_t *proto, int idx);
 
-/* Mirror a padstack (useful for sending to the other side - set swap_side to 1 in that case) */
-void pcb_pstk_mirror(pcb_pstk_t *ps, pcb_coord_t y_offs, int swap_side);
+#define PCB_PSTK_DONT_MIRROR_COORDS PCB_MAX_COORD
+/* Mirror a padstack (useful for sending to the other side - set swap_side to 1 in that case)
+   Disabling xmirror is useful if side needs to be swapped but coordinates
+   are already mirrored so they represent the other-side geometry (e.g. when
+   importing from old pcb formats). If y_offs is PCB_PSTK_DONT_MIRROR_COORDS,
+   do not change the y coord */
+void pcb_pstk_mirror(pcb_pstk_t *ps, pcb_coord_t y_offs, int swap_side, int disable_xmirror);
 
 /* Rotate in place (op wrapper) */
 void pcb_pstk_rotate90(pcb_pstk_t *pstk, pcb_coord_t cx, pcb_coord_t cy, int steps);
