@@ -18,6 +18,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
 #include "config.h"
@@ -39,7 +43,7 @@ int pcb_act_padstackconvert(int argc, const char **argv, pcb_coord_t x, pcb_coor
 	pcb_cardinal_t pid;
 	pcb_pstk_proto_t tmp, *p;
 
-	if (argv[0] == NULL)
+	if (argc < 1)
 		PCB_ACT_FAIL(padstackconvert);
 	if (strcmp(argv[0], "selected") == 0) {
 		if (argc > 2) {
