@@ -50,7 +50,7 @@ void *pcb_pstkop_move_buffer(pcb_opctx_t *ctx, pcb_pstk_t *ps)
 
 	npid = pcb_pstk_proto_insert_dup(ctx->buffer.dst, proto, 1);
 
-	pcb_poly_restore_to_poly(ctx->buffer.src, PCB_TYPE_PSTK, NULL, ps);
+	pcb_poly_restore_to_poly(ctx->buffer.src, PCB_OBJ_PSTK, NULL, ps);
 	pcb_r_delete_entry(ctx->buffer.src->padstack_tree, (pcb_box_t *)ps);
 
 	padstacklist_remove(ps);
@@ -64,7 +64,7 @@ void *pcb_pstkop_move_buffer(pcb_opctx_t *ctx, pcb_pstk_t *ps)
 		ctx->buffer.dst->padstack_tree = pcb_r_create_tree();
 
 	pcb_r_insert_entry(ctx->buffer.dst->padstack_tree, (pcb_box_t *)ps);
-	pcb_poly_clear_from_poly(ctx->buffer.dst, PCB_TYPE_PSTK, NULL, ps);
+	pcb_poly_clear_from_poly(ctx->buffer.dst, PCB_OBJ_PSTK, NULL, ps);
 
 	PCB_SET_PARENT(ps, data, ctx->buffer.dst);
 	return ps;
@@ -87,7 +87,7 @@ void *pcb_pstkop_copy(pcb_opctx_t *ctx, pcb_pstk_t *ps)
 
 	pcb_pstk_copy_meta(nps, ps);
 	pcb_pstk_invalidate_draw(nps);
-	pcb_undo_add_obj_to_create(PCB_TYPE_PSTK, data, nps, nps);
+	pcb_undo_add_obj_to_create(PCB_OBJ_PSTK, data, nps, nps);
 	return nps;
 }
 
@@ -105,10 +105,11 @@ void *pcb_pstkop_move(pcb_opctx_t *ctx, pcb_pstk_t *ps)
 	assert(ps->parent_type = PCB_PARENT_DATA);
 
 	pcb_r_delete_entry(data->padstack_tree, (pcb_box_t *)ps);
-	pcb_poly_restore_to_poly(data, PCB_TYPE_PSTK, NULL, ps);
+	pcb_poly_restore_to_poly(data, PCB_OBJ_PSTK, NULL, ps);
 	pcb_pstkop_move_noclip(ctx, ps);
 	pcb_r_insert_entry(data->padstack_tree, (pcb_box_t *)ps);
-	pcb_poly_clear_from_poly(data, PCB_TYPE_PSTK, NULL, ps);
+	pcb_poly_clear_from_poly(data, PCB_OBJ_PSTK, NULL, ps);
+	pcb_subc_part_changed(ps);
 	return ps;
 }
 
@@ -120,12 +121,12 @@ void *pcb_pstkop_clip(pcb_opctx_t *ctx, pcb_pstk_t *ps)
 	if (ctx->clip.restore) {
 		if (data->padstack_tree != NULL)
 			pcb_r_delete_entry(data->padstack_tree, (pcb_box_t *)ps);
-		pcb_poly_restore_to_poly(data, PCB_TYPE_PSTK, NULL, ps);
+		pcb_poly_restore_to_poly(data, PCB_OBJ_PSTK, NULL, ps);
 	}
 	if (ctx->clip.clear) {
 		if (data->padstack_tree != NULL)
 			pcb_r_insert_entry(data->padstack_tree, (pcb_box_t *)ps);
-		pcb_poly_clear_from_poly(data, PCB_TYPE_PSTK, NULL, ps);
+		pcb_poly_clear_from_poly(data, PCB_OBJ_PSTK, NULL, ps);
 	}
 
 	return ps;
@@ -134,7 +135,8 @@ void *pcb_pstkop_clip(pcb_opctx_t *ctx, pcb_pstk_t *ps)
 void *pcb_pstkop_remove(pcb_opctx_t *ctx, pcb_pstk_t *ps)
 {
 	pcb_pstk_invalidate_erase(ps);
-	pcb_undo_move_obj_to_remove(PCB_TYPE_PSTK, ps, ps, ps);
+	pcb_undo_move_obj_to_remove(PCB_OBJ_PSTK, ps, ps, ps);
+	pcb_subc_part_changed(ps);
 	return NULL;
 }
 
@@ -156,14 +158,14 @@ void *pcb_pstkop_change_thermal(pcb_opctx_t *ctx, pcb_pstk_t *ps)
 	pcb = ctx->chgtherm.pcb;
 	layer = pcb_get_layer(pcb->Data, ctx->chgtherm.lid);
 
-	pcb_undo_add_obj_to_clear_poly(PCB_TYPE_PSTK, ps, ps, ps, pcb_false);
-	pcb_poly_restore_to_poly(pcb->Data, PCB_TYPE_PSTK, layer, ps);
+	pcb_undo_add_obj_to_clear_poly(PCB_OBJ_PSTK, ps, ps, ps, pcb_false);
+	pcb_poly_restore_to_poly(pcb->Data, PCB_OBJ_PSTK, layer, ps);
 
 #warning TODO: undo
 	pcb_pstk_set_thermal(ps, ctx->chgtherm.lid, ctx->chgtherm.style);
 
-	pcb_undo_add_obj_to_clear_poly(PCB_TYPE_PSTK, ps, ps, ps, pcb_true);
-	pcb_poly_clear_from_poly(pcb->Data, PCB_TYPE_PSTK, layer, ps);
+	pcb_undo_add_obj_to_clear_poly(PCB_OBJ_PSTK, ps, ps, ps, pcb_true);
+	pcb_poly_clear_from_poly(pcb->Data, PCB_OBJ_PSTK, layer, ps);
 	pcb_pstk_invalidate_draw(ps);
 	return ps;
 }
@@ -176,7 +178,7 @@ void *pcb_pstkop_change_flag(pcb_opctx_t *ctx, pcb_pstk_t *ps)
 		return NULL;
 
 	if (pcb_pstk_flags == 0)
-		pcb_pstk_flags = pcb_obj_valid_flags(PCB_TYPE_PSTK);
+		pcb_pstk_flags = pcb_obj_valid_flags(PCB_OBJ_PSTK);
 
 	if ((ctx->chgflag.flag & pcb_pstk_flags) != ctx->chgflag.flag)
 		return NULL;
@@ -195,7 +197,10 @@ void *pcb_pstkop_rotate(pcb_opctx_t *ctx, pcb_pstk_t *ps)
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, ps))
 		return NULL;
 
-	rot += (double)ctx->rotate.angle;
+	if (ps->xmirror)
+		rot -= (double)ctx->rotate.angle;
+	else
+		rot += (double)ctx->rotate.angle;
 
 	if ((rot > 360.0) || (rot < -360.0))
 		rot = fmod(rot, 360.0);
@@ -207,19 +212,22 @@ void *pcb_pstkop_rotate(pcb_opctx_t *ctx, pcb_pstk_t *ps)
 		pcb_coord_t nx = ps->x, ny = ps->y;
 
 
-		pcb_poly_restore_to_poly(ps->parent.data, PCB_TYPE_PSTK, NULL, ps);
+		pcb_poly_restore_to_poly(ps->parent.data, PCB_OBJ_PSTK, NULL, ps);
 		pcb_pstk_invalidate_erase(ps);
-		pcb_r_delete_entry(ps->parent.data->padstack_tree, (pcb_box_t *)ps);
+		if (ps->parent.data->padstack_tree != NULL)
+			pcb_r_delete_entry(ps->parent.data->padstack_tree, (pcb_box_t *)ps);
 
 		pcb_rotate(&nx, &ny, ctx->rotate.center_x, ctx->rotate.center_y, ctx->rotate.cosa, ctx->rotate.sina);
 		if ((nx != ps->x) || (ny != ps->y))
 			pcb_pstk_move_(ps, nx - ps->x, ny - ps->y);
 
 		pcb_pstk_bbox(ps);
-		pcb_r_insert_entry(ps->parent.data->padstack_tree, (pcb_box_t *)ps);
-		pcb_poly_clear_from_poly(ps->parent.data, PCB_TYPE_PSTK, NULL, ps);
+		if (ps->parent.data->padstack_tree != NULL)
+			pcb_r_insert_entry(ps->parent.data->padstack_tree, (pcb_box_t *)ps);
+		pcb_poly_clear_from_poly(ps->parent.data, PCB_OBJ_PSTK, NULL, ps);
 		pcb_pstk_invalidate_draw(ps);
 
+		pcb_subc_part_changed(ps);
 		return ps;
 	}
 	return NULL;
@@ -270,30 +278,34 @@ void *pcb_pstkop_change_size(pcb_opctx_t *ctx, pcb_pstk_t *ps)
 	if (nproto == PCB_PADSTACK_INVALID)
 		return NULL;
 
-	if (pcb_pstk_change_instance(ps, &nproto, NULL, NULL, NULL, NULL) == 0)
+	if (pcb_pstk_change_instance(ps, &nproto, NULL, NULL, NULL, NULL) == 0) {
+		pcb_subc_part_changed(ps);
 		return ps;
+	}
 
 	return NULL;
 }
 
 void *pcb_pstkop_change_clear_size(pcb_opctx_t *ctx, pcb_pstk_t *ps)
 {
-	pcb_coord_t value = (ctx->chgsize.is_absolute) ? ctx->chgsize.value : ps->Clearance + ctx->chgsize.value;
+	pcb_coord_t value = (ctx->chgsize.is_absolute) ? ctx->chgsize.value / 2 : ps->Clearance + ctx->chgsize.value / 2;
 
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, ps))
 		return NULL;
 	if (value < 0)
 		value = 0;
 	value = MIN(PCB_MAX_LINESIZE, value);
-	if (!ctx->chgsize.is_absolute && (ctx->chgsize.value < 0) && (value < PCB->Bloat * 2))
+	if (!ctx->chgsize.is_absolute && (ctx->chgsize.value < 0) && (value < conf_core.design.bloat))
 		value = 0;
-	if ((ctx->chgsize.value > 0) && (value < PCB->Bloat * 2))
-		value = PCB->Bloat * 2 + 2;
+	if ((ctx->chgsize.value > 0) && (value < conf_core.design.bloat))
+		value = conf_core.design.bloat + 1;
 	if (ps->Clearance == value)
 		return NULL;
 
-	if (pcb_pstk_change_instance(ps, NULL, &value, NULL, NULL, NULL) == 0)
+	if (pcb_pstk_change_instance(ps, NULL, &value, NULL, NULL, NULL) == 0) {
+		pcb_subc_part_changed(ps);
 		return ps;
+	}
 
 	return NULL;
 }
@@ -311,8 +323,8 @@ void *pcb_pstkop_change_2nd_size(pcb_opctx_t *ctx, pcb_pstk_t *ps)
 	pcb_pstk_proto_copy(&proto, pcb_pstk_get_proto(ps));
 	if (!ctx->chgsize.is_absolute) {
 		proto.hdia += ctx->chgsize.value;
-		if (proto.hdia < ctx->chgsize.pcb->minDrill)
-			proto.hdia = ctx->chgsize.pcb->minDrill;
+		if (proto.hdia < conf_core.design.min_drill)
+			proto.hdia = conf_core.design.min_drill;
 	}
 	else
 		proto.hdia = ctx->chgsize.value;
@@ -322,8 +334,10 @@ void *pcb_pstkop_change_2nd_size(pcb_opctx_t *ctx, pcb_pstk_t *ps)
 	if (nproto == PCB_PADSTACK_INVALID)
 		return NULL;
 
-	if (pcb_pstk_change_instance(ps, &nproto, NULL, NULL, NULL, NULL) == 0)
+	if (pcb_pstk_change_instance(ps, &nproto, NULL, NULL, NULL, NULL) == 0) {
+		pcb_subc_part_changed(ps);
 		return ps;
+	}
 
 	return NULL;
 }
