@@ -117,11 +117,11 @@ int pcb_pstk_proto_conv(pcb_data_t *data, pcb_pstk_proto_t *dst, int quiet, vtp0
 	int ret = -1, n, m, i, extra_obj = 0;
 	pcb_any_obj_t **o;
 	pcb_pstk_tshape_t *ts, *ts_src;
-	pcb_pin_t *via = NULL;
 	pcb_pstk_t *pstk = NULL;
 	pcb_pstk_proto_t *prt;
 
 	dst->in_use = 1;
+	dst->name = NULL;
 	pcb_vtpadstack_tshape_init(&dst->tr);
 	dst->hdia = 0;
 	dst->htop = dst->hbottom = 0;
@@ -138,24 +138,8 @@ int pcb_pstk_proto_conv(pcb_data_t *data, pcb_pstk_proto_t *dst, int quiet, vtp0
 			case PCB_OBJ_POLY:
 				ts->len++;
 				break;
-			case PCB_OBJ_VIA:
-				if ((via != NULL) || (pstk != NULL)) {
-					if (!quiet)
-						pcb_message(PCB_MSG_ERROR, "Padstack conversion: multiple vias/padstacks\n");
-					goto quit;
-				}
-				via = *(pcb_pin_t **)o;
-				dst->hdia = via->DrillingHole;
-				dst->hplated = !PCB_FLAG_TEST(PCB_FLAG_HOLE, *o);
-				if ((ox != via->X) || (oy != via->Y)) {
-					pcb_message(PCB_MSG_INFO, "Padstack conversion: adjusting origin to via hole\n");
-					ox = via->X;
-					oy = via->Y;
-				}
-				extra_obj++;
-				break;
 			case PCB_OBJ_PSTK:
-				if ((via != NULL) || (pstk != NULL)) {
+				if (pstk != NULL) {
 					if (!quiet)
 						pcb_message(PCB_MSG_ERROR, "Padstack conversion: multiple vias/padstacks\n");
 					goto quit;
@@ -189,7 +173,7 @@ int pcb_pstk_proto_conv(pcb_data_t *data, pcb_pstk_proto_t *dst, int quiet, vtp0
 		goto quit;
 	}
 
-	if ((ts->len == 0) && (via == NULL) && (pstk == NULL)) {
+	if ((ts->len == 0) && (pstk == NULL)) {
 		if (!quiet)
 			pcb_message(PCB_MSG_ERROR, "Padstack conversion: there are no shapes and there is no via/padstack participating in the conversion; can not create empty padstack\n");
 		goto quit;
@@ -267,29 +251,13 @@ int pcb_pstk_proto_conv(pcb_data_t *data, pcb_pstk_proto_t *dst, int quiet, vtp0
 				goto quit;
 			}
 		}
-		if ((ts->shape[n].layer_mask & PCB_LYT_COPPER) && (ts->shape[n].layer_mask & PCB_LYT_INTERN) && (via == NULL) && (pstk == NULL)) {
+		if ((ts->shape[n].layer_mask & PCB_LYT_COPPER) && (ts->shape[n].layer_mask & PCB_LYT_INTERN) && (pstk == NULL)) {
 			if (!quiet)
 				pcb_message(PCB_MSG_ERROR, "Padstack conversion: can not have internal copper shape if there is no hole\n");
 			goto quit;
 		}
 	}
 
-	/* if there was a via, use the via's shape on layers that are not specified */
-	if (via != NULL) {
-		if (pcb_pstk_get_shape_idx(ts, PCB_LYT_COPPER | PCB_LYT_TOP, 0) == -1)
-			append_circle(ts,  PCB_LYT_COPPER | PCB_LYT_TOP, 0, via->Thickness);
-		if (pcb_pstk_get_shape_idx(ts, PCB_LYT_COPPER | PCB_LYT_INTERN, 0) == -1)
-			append_circle(ts,  PCB_LYT_COPPER | PCB_LYT_INTERN, 0, via->Thickness);
-		if (pcb_pstk_get_shape_idx(ts, PCB_LYT_COPPER | PCB_LYT_BOTTOM, 0) == -1)
-			append_circle(ts,  PCB_LYT_COPPER | PCB_LYT_BOTTOM, 0, via->Thickness);
-		if (via->Mask > 0) {
-			if (pcb_pstk_get_shape_idx(ts, PCB_LYT_MASK | PCB_LYT_BOTTOM, PCB_LYC_SUB | PCB_LYC_AUTO) == -1)
-				append_circle(ts,  PCB_LYT_MASK | PCB_LYT_BOTTOM, PCB_LYC_SUB | PCB_LYC_AUTO, via->Mask);
-			if (pcb_pstk_get_shape_idx(ts, PCB_LYT_MASK | PCB_LYT_TOP, PCB_LYC_SUB | PCB_LYC_AUTO) == -1)
-				append_circle(ts,  PCB_LYT_MASK | PCB_LYT_TOP, PCB_LYC_SUB | PCB_LYC_AUTO, via->Mask);
-		}
-	}
-
 	/* if there was a padstack, use the padstack's shape on layers that are not specified */
 	if (pstk != NULL) {
 		int srci;
@@ -533,12 +501,16 @@ pcb_cardinal_t pcb_pstk_proto_insert_or_free(pcb_data_t *data, pcb_pstk_proto_t
 
 	/* no match, have to register a new one */
 	if (first_free == PCB_PADSTACK_INVALID) {
+		pcb_pstk_proto_t *np;
 		n = pcb_vtpadstack_proto_len(&data->ps_protos);
 		pcb_vtpadstack_proto_append(&data->ps_protos, *proto);
+		np = pcb_vtpadstack_proto_get(&data->ps_protos, n, 0);
+		np->parent = data;
 	}
 	else {
 		memcpy(data->ps_protos.array+first_free, proto, sizeof(pcb_pstk_proto_t));
 		data->ps_protos.array[first_free].in_use = 1;
+		data->ps_protos.array[first_free].parent = data;
 	}
 	memset(proto, 0, sizeof(pcb_pstk_proto_t)); /* make sure a subsequent free() won't do any harm */
 	return n;
@@ -607,6 +579,7 @@ void pcb_pstk_shape_update_pa(pcb_pstk_poly_t *poly)
 	pcb_polyarea_contour_include(poly->pa, pl);
 
 	if (!pcb_poly_valid(poly->pa)) {
+		poly->pa->contours = NULL; /* keep pl safe from pcb_polyarea_free() */
 		pcb_polyarea_free(&poly->pa);
 		
 		poly->pa = pcb_polyarea_create();
@@ -702,6 +675,17 @@ int pcb_pstk_proto_change_hole(pcb_pstk_proto_t *proto, const int *hplated, cons
 	return 0;
 }
 
+int pcb_pstk_proto_change_name(pcb_pstk_proto_t *proto, const char *new_name)
+{
+#warning undo TODO: make this undoable (check how pcb_pstk_proto_change_hole() does it)
+	free(proto->name);
+	if ((new_name == NULL) || (*new_name == '\0'))
+		proto->name = NULL;
+	else
+		proto->name = pcb_strdup(new_name);
+	return 0;
+}
+
 #define TSHAPE_ANGLE_TOL 0.01
 #define tshape_angle_eq(a1, a2) (((a1 - a2) >= -TSHAPE_ANGLE_TOL) && ((a1 - a2) <= TSHAPE_ANGLE_TOL))
 
