@@ -50,6 +50,7 @@
 #include "polygon.h"
 #include "operation.h"
 #include "undo.h"
+#include "macro.h"
 #include "compat_misc.h"
 #include "math_helper.h"
 #include "pcb_minuid.h"
@@ -125,7 +126,9 @@ static pcb_layer_t *pcb_subc_layer_create_buff(pcb_subc_t *sc, pcb_layer_t *src)
 	memcpy(&dst->meta, &src->meta, sizeof(src->meta));
 	dst->is_bound = 1;
 	dst->comb = src->comb;
-	dst->parent = sc->data;
+	dst->parent.data = sc->data;
+	dst->parent_type = PCB_PARENT_DATA;
+	dst->type = PCB_OBJ_LAYER;
 	dst->name = pcb_strdup(src->name);
 	return dst;
 }
@@ -246,7 +249,7 @@ int pcb_subc_get_side(pcb_subc_t *sc, int *on_bottom)
 	return 0;
 }
 
-int pcb_subc_get_host_trans(const pcb_subc_t *sc, pcb_host_trans_t *tr)
+int pcb_subc_get_host_trans(pcb_subc_t *sc, pcb_host_trans_t *tr)
 {
 	int res = 0;
 	double rr;
@@ -293,7 +296,7 @@ static pcb_poly_t *sqline2term(pcb_layer_t *dst, pcb_line_t *line)
 	PCB_FLAG_SET(PCB_FLAG_CLEARPOLYPOLY, poly);
 	pcb_attribute_copy_all(&poly->Attributes, &line->Attributes);
 
-	pcb_poly_init_clip(dst->parent, dst, poly);
+	pcb_poly_init_clip(dst->parent.data, dst, poly);
 	pcb_add_poly_on_layer(dst, poly);
 
 	return poly;
@@ -323,23 +326,6 @@ pcb_layer_t *pcb_loose_subc_layer(pcb_board_t *pcb, pcb_layer_t *layer)
 
 extern unsigned long pcb_obj_type2oldtype(pcb_objtype_t type);
 
-/* Move the pad-side-effect objects to the appropriate layer */
-static void move_pad_side_effect(pcb_any_obj_t *o, pcb_layer_t *top, pcb_layer_t *bottom)
-{
-	pcb_layer_t *source = o->parent.layer;
-	pcb_layer_t *target = (source->meta.bound.type & PCB_LYT_TOP) ? top : bottom;
-	switch(o->type) {
-		case PCB_OBJ_POLY:
-			pcb_polyop_move_to_layer_low(NULL, source, (pcb_poly_t *)o, target);
-			break;
-		case PCB_OBJ_LINE:
-			pcb_lineop_move_to_layer_low(NULL, source, (pcb_line_t *)o, target);
-			break;
-		default:
-			assert(!"internal error: invalid mask/paste side effect");
-	}
-}
-
 static pcb_coord_t read_mask(pcb_any_obj_t *obj)
 {
 	const char *smask = pcb_attribute_get(&obj->Attributes, "elem_smash_pad_mask");
@@ -521,18 +507,6 @@ int pcb_subc_convert_from_buffer(pcb_buffer_t *buffer)
 		}
 	}
 
-	{ /* check if we have pins - they need mask */
-		pcb_pin_t *via;
-
-		for(via = pinlist_first(&buffer->Data->Via); via != NULL; via = pinlist_next(via)) {
-			if (pcb_attribute_get(&via->Attributes, "elem_smash_pad") != NULL) {
-				top_pads++;
-				bottom_pads++;
-				break;
-			}
-		}
-	}
-
 	/* create paste and mask side effects - needed when importing from footprint */
 	{
 		if (top_pads > 0) {
@@ -554,50 +528,8 @@ int pcb_subc_convert_from_buffer(pcb_buffer_t *buffer)
 	}
 
 	{ /* convert globals */
-		pcb_pin_t *via;
 		pcb_pstk_t *ps;
 
-		while((via = pinlist_first(&buffer->Data->Via)) != NULL) {
-			const char *term;
-			pinlist_remove(via);
-			pinlist_append(&sc->data->Via, via);
-			PCB_SET_PARENT(via, data, sc->data);
-			PCB_FLAG_CLEAR(PCB_FLAG_WARN | PCB_FLAG_FOUND | PCB_FLAG_SELECTED, via);
-			term = pcb_attribute_get(&via->Attributes, "term");
-			if (pcb_attribute_get(&via->Attributes, "elem_smash_pad") != NULL) {
-				pcb_coord_t mask = read_mask((pcb_any_obj_t *)via);
-				
-				if (mask == 0)
-					mask = via->Mask;
-
-				if (mask > 0) {
-					if (PCB_FLAG_TEST(PCB_FLAG_SQUARE, via)) {
-						pcb_line_t line;
-						pcb_poly_t *poly;
-						memset(&line, 0, sizeof(line));
-						line.Point1.X = line.Point2.X = via->X;
-						line.Point1.Y = line.Point2.Y = via->Y;
-						line.Thickness = mask;
-						poly = sqline2term(dst_top_mask, &line);
-						if (term != NULL)
-							pcb_attribute_put(&poly->Attributes, "term", term);
-						poly = sqline2term(dst_bottom_mask, &line);
-						if (term != NULL)
-							pcb_attribute_put(&poly->Attributes, "term", term);
-					}
-					else {
-						pcb_line_t *line;
-						line = pcb_line_new(dst_top_mask, via->X, via->Y, via->X, via->Y, mask, 0, pcb_no_flags());
-						if (term != NULL)
-							pcb_attribute_put(&line->Attributes, "term", term);
-						line = pcb_line_new(dst_bottom_mask, via->X, via->Y, via->X, via->Y, mask, 0, pcb_no_flags());
-						if (term != NULL)
-							pcb_attribute_put(&line->Attributes, "term", term);
-					}
-				}
-			}
-		}
-
 		while((ps = padstacklist_first(&buffer->Data->padstack)) != NULL) {
 			const pcb_pstk_proto_t *proto = pcb_pstk_get_proto(ps);
 			padstacklist_remove(ps);
@@ -609,11 +541,6 @@ int pcb_subc_convert_from_buffer(pcb_buffer_t *buffer)
 		}
 	}
 
-	for(n = 0; n < vtp0_len(&paste_pads); n++)
-		move_pad_side_effect(paste_pads.array[n], dst_top_paste, dst_bottom_paste);
-	for(n = 0; n < vtp0_len(&mask_pads); n++)
-		move_pad_side_effect(mask_pads.array[n], dst_top_mask, dst_bottom_mask);
-
 	vtp0_uninit(&mask_pads);
 	vtp0_uninit(&paste_pads);
 
@@ -653,7 +580,7 @@ static void pcb_subc_draw_origin(pcb_hid_gc_t GC, pcb_subc_t *sc, pcb_coord_t DX
 	pcb_gui->draw_line(GC, DX + PCB_EMARK_SIZE, DY, DX, DY + PCB_EMARK_SIZE);
 }
 
-void XORDrawSubc(pcb_subc_t *sc, pcb_coord_t DX, pcb_coord_t DY, int use_curr_side)
+void pcb_xordraw_subc(pcb_subc_t *sc, pcb_coord_t DX, pcb_coord_t DY, int use_curr_side)
 {
 	int n, mirr;
 	pcb_coord_t w = 0, h = 0;
@@ -706,7 +633,7 @@ void XORDrawSubc(pcb_subc_t *sc, pcb_coord_t DX, pcb_coord_t DY, int use_curr_si
 		}
 
 		polylist_foreach(&ly->Polygon, &it, poly)
-			XORPolygon_subc(poly, DX, DY, w, h, mirr);
+			pcb_xordraw_poly_subc(poly, DX, DY, w, h, mirr);
 
 		textlist_foreach(&ly->Text, &it, text) {
 			if (mirr) {
@@ -724,22 +651,9 @@ void XORDrawSubc(pcb_subc_t *sc, pcb_coord_t DX, pcb_coord_t DY, int use_curr_si
 
 	/* draw global objects */
 	{
-		pcb_pin_t *via;
 		pcb_pstk_t *ps;
 		gdl_iterator_t it;
 
-		pinlist_foreach(&sc->data->Via, &it, via) {
-			pcb_coord_t ox, oy;
-			ox = via->X;
-			oy = via->Y;
-			via->X = PCB_CSWAP_X(via->X, w, mirr);
-			via->Y = PCB_CSWAP_Y(via->Y, h, mirr);
-			via->X += DX;
-			via->Y += DY;
-			pcb_gui->thindraw_pcb_pv(pcb_crosshair.GC, pcb_crosshair.GC, via, pcb_true, pcb_false);
-			via->X = ox;
-			via->Y = oy;
-		}
 		padstacklist_foreach(&sc->data->padstack, &it, ps) {
 			pcb_coord_t ox, oy;
 			int oxm, pri;
@@ -882,26 +796,15 @@ pcb_subc_t *pcb_subc_dup_at(pcb_board_t *pcb, pcb_data_t *dst, pcb_subc_t *src,
 
 	/* bind the via rtree so that vias added in this subc show up on the board */
 	if (pcb != NULL) {
-		if (pcb->Data->via_tree == NULL)
-			pcb->Data->via_tree = pcb_r_create_tree();
-		sc->data->via_tree = pcb->Data->via_tree;
 		if (pcb->Data->padstack_tree == NULL)
 			pcb->Data->padstack_tree = pcb_r_create_tree();
 		sc->data->padstack_tree = pcb->Data->padstack_tree;
 	}
 
 	{ /* make a copy of global data */
-		pcb_pin_t *via, *nvia;
 		pcb_pstk_t *ps, *nps;
 		gdl_iterator_t it;
 
-		pinlist_foreach(&src->data->Via, &it, via) {
-			nvia = pcb_via_dup_at(sc->data, via, dx, dy);
-			MAYBE_KEEP_ID(nvia, via);
-			if (nvia != NULL)
-				pcb_box_bump_box_noflt(&sc->BoundingBox, &nvia->BoundingBox);
-		}
-
 		padstacklist_foreach(&src->data->padstack, &it, ps) {
 			pcb_cardinal_t pid = pcb_pstk_proto_insert_dup(sc->data, pcb_pstk_get_proto(ps), 1);
 			nps = pcb_pstk_new_tr(sc->data, pid, ps->x+dx, ps->y+dy, ps->Clearance, ps->Flags, ps->rot, ps->xmirror, ps->smirror);
@@ -968,10 +871,22 @@ void DrawSubc(pcb_subc_t *sc)
 
 
 /* Execute an operation on all children on an sc and update the bbox of the sc */
-void *pcb_subc_op(pcb_data_t *Data, pcb_subc_t *sc, pcb_opfunc_t *opfunc, pcb_opctx_t *ctx)
+void *pcb_subc_op(pcb_data_t *Data, pcb_subc_t *sc, pcb_opfunc_t *opfunc, pcb_opctx_t *ctx, pcb_subc_op_undo_t batch_undo)
 {
 	int n;
 
+	switch(batch_undo) {
+		case PCB_SUBCOP_UNDO_SUBC:
+			pcb_undo_freeze_serial();
+			pcb_undo_freeze_add();
+			break;
+		case PCB_SUBCOP_UNDO_BATCH:
+			pcb_undo_inc_serial();
+			pcb_undo_save_serial();
+			pcb_undo_freeze_serial();
+			break;
+	}
+
 	EraseSubc(sc);
 	if (pcb_data_get_top(Data) != NULL) {
 		if (Data->subc_tree != NULL)
@@ -995,22 +910,22 @@ void *pcb_subc_op(pcb_data_t *Data, pcb_subc_t *sc, pcb_opfunc_t *opfunc, pcb_op
 
 
 		linelist_foreach(&sl->Line, &it, line) {
-			pcb_object_operation(opfunc, ctx, PCB_TYPE_LINE, sl, line, line);
+			pcb_object_operation(opfunc, ctx, PCB_OBJ_LINE, sl, line, line);
 			pcb_box_bump_box_noflt(&sc->BoundingBox, &line->BoundingBox);
 		}
 
 		arclist_foreach(&sl->Arc, &it, arc) {
-			pcb_object_operation(opfunc, ctx, PCB_TYPE_ARC, sl, arc, arc);
+			pcb_object_operation(opfunc, ctx, PCB_OBJ_ARC, sl, arc, arc);
 			pcb_box_bump_box_noflt(&sc->BoundingBox, &arc->BoundingBox);
 		}
 
 		textlist_foreach(&sl->Text, &it, text) {
-			pcb_object_operation(opfunc, ctx, PCB_TYPE_TEXT, sl, text, text);
+			pcb_object_operation(opfunc, ctx, PCB_OBJ_TEXT, sl, text, text);
 			pcb_box_bump_box_noflt(&sc->BoundingBox, &text->BoundingBox);
 		}
 
 		polylist_foreach(&sl->Polygon, &it, poly) {
-			pcb_object_operation(opfunc, ctx, PCB_TYPE_POLY, sl, poly, poly);
+			pcb_object_operation(opfunc, ctx, PCB_OBJ_POLY, sl, poly, poly);
 			pcb_box_bump_box_noflt(&sc->BoundingBox, &poly->BoundingBox);
 		}
 
@@ -1019,17 +934,11 @@ void *pcb_subc_op(pcb_data_t *Data, pcb_subc_t *sc, pcb_opfunc_t *opfunc, pcb_op
 
 	/* execute on globals */
 	{
-		pcb_pin_t *via;
 		pcb_pstk_t *ps;
 		gdl_iterator_t it;
 
-		pinlist_foreach(&sc->data->Via, &it, via) {
-			pcb_object_operation(opfunc, ctx, PCB_TYPE_VIA, via, via, via);
-			pcb_box_bump_box_noflt(&sc->BoundingBox, &via->BoundingBox);
-		}
-
 		padstacklist_foreach(&sc->data->padstack, &it, ps) {
-			pcb_object_operation(opfunc, ctx, PCB_TYPE_PSTK, ps, ps, ps);
+			pcb_object_operation(opfunc, ctx, PCB_OBJ_PSTK, ps, ps, ps);
 			pcb_box_bump_box_noflt(&sc->BoundingBox, &ps->BoundingBox);
 		}
 	}
@@ -1038,6 +947,12 @@ void *pcb_subc_op(pcb_data_t *Data, pcb_subc_t *sc, pcb_opfunc_t *opfunc, pcb_op
 	if (pcb_data_get_top(Data) != NULL)
 		pcb_r_insert_entry(Data->subc_tree, (pcb_box_t *)sc);
 	DrawSubc(sc);
+
+	switch(batch_undo) {
+		case PCB_SUBCOP_UNDO_SUBC: pcb_undo_unfreeze_add(); pcb_undo_unfreeze_serial(); break;
+		case PCB_SUBCOP_UNDO_BATCH: pcb_undo_unfreeze_serial(); break;
+	}
+
 	return sc;
 }
 
@@ -1049,7 +964,7 @@ void *pcb_subcop_copy(pcb_opctx_t *ctx, pcb_subc_t *src)
 
 	sc = pcb_subc_dup_at(PCB, PCB->Data, src, ctx->copy.DeltaX, ctx->copy.DeltaY, pcb_false);
 
-	pcb_undo_add_obj_to_create(PCB_TYPE_SUBC, sc, sc, sc);
+	pcb_undo_add_obj_to_create(PCB_OBJ_SUBC, sc, sc, sc);
 
 	if (ctx->copy.from_outside && conf_core.editor.show_solder_side) {
 		uundo_serial_t last;
@@ -1069,8 +984,6 @@ void *pcb_subcop_copy(pcb_opctx_t *ctx, pcb_subc_t *src)
 }
 
 extern pcb_opfunc_t ClipFunctions, MoveFunctions, MoveFunctions_noclip, Rotate90Functions, RotateFunctions, ChgFlagFunctions, ChangeSizeFunctions, ChangeClearSizeFunctions, Change1stSizeFunctions, Change2ndSizeFunctions;
-extern pcb_opfunc_t ChangeOctagonFunctions, SetOctagonFunctions, ClrOctagonFunctions;
-extern pcb_opfunc_t ChangeSquareFunctions, SetSquareFunctions, ClrSquareFunctions;
 
 /* drag&drop move when not selected */
 void *pcb_subcop_move(pcb_opctx_t *ctx, pcb_subc_t *sc)
@@ -1082,21 +995,21 @@ void *pcb_subcop_move(pcb_opctx_t *ctx, pcb_subc_t *sc)
 	/* restore all pins/pads at once, at the old location */
 	clip.clip.restore = 1; clip.clip.clear = 0;
 	clip.clip.pcb = ctx->move.pcb;
-	pcb_subc_op(data, sc, &ClipFunctions, &clip);
+	pcb_subc_op(data, sc, &ClipFunctions, &clip, PCB_SUBCOP_UNDO_NORMAL);
 
 	/* do the move without messing with the clipping */
-	pcb_subc_op(data, sc, &MoveFunctions_noclip, ctx);
+	pcb_subc_op(data, sc, &MoveFunctions_noclip, ctx, PCB_SUBCOP_UNDO_NORMAL);
 
 	/* clear all pins/pads at once, at the new location */
 	clip.clip.restore = 0; clip.clip.clear = 1;
-	pcb_subc_op(data, sc, &ClipFunctions, &clip);
+	pcb_subc_op(data, sc, &ClipFunctions, &clip, PCB_SUBCOP_UNDO_NORMAL);
 	return sc;
 }
 
 void *pcb_subcop_rotate90(pcb_opctx_t *ctx, pcb_subc_t *sc)
 {
 	pcb_board_t *pcb = pcb_data_get_top(sc->data);
-	return pcb_subc_op((pcb != NULL ? pcb->Data : NULL), sc, &Rotate90Functions, ctx);
+	return pcb_subc_op((pcb != NULL ? pcb->Data : NULL), sc, &Rotate90Functions, ctx, PCB_SUBCOP_UNDO_SUBC);
 }
 
 void *pcb_subcop_rotate(pcb_opctx_t *ctx, pcb_subc_t *sc)
@@ -1105,7 +1018,8 @@ void *pcb_subcop_rotate(pcb_opctx_t *ctx, pcb_subc_t *sc)
 
 	ctx->rotate.pcb = pcb_data_get_top(sc->data);
 	data = (ctx->rotate.pcb != NULL ? ctx->rotate.pcb->Data : NULL);
-	return pcb_subc_op(data, sc, &RotateFunctions, ctx);
+
+	return pcb_subc_op(data, sc, &RotateFunctions, ctx, PCB_SUBCOP_UNDO_SUBC);
 }
 
 void pcb_subc_rotate90(pcb_subc_t *subc, pcb_coord_t cx, pcb_coord_t cy, int steps)
@@ -1144,7 +1058,7 @@ pcb_bool pcb_selected_subc_change_side(void)
 {
 	pcb_bool change = pcb_false;
 
-	if (PCB->PinOn && pcb_silk_on(PCB)) {
+	if (PCB->pstk_on && pcb_silk_on(PCB)) {
 		PCB_SUBC_LOOP(PCB->Data);
 		{
 			if (PCB_FLAG_TEST(PCB_FLAG_SELECTED, subc)) {
@@ -1165,11 +1079,11 @@ static int subc_relocate_layer_objs(pcb_layer_t *dl, pcb_data_t *src_data, pcb_l
 	pcb_arc_t *arc;
 	gdl_iterator_t it;
 	int chg = 0;
-	pcb_data_t *dst_data = dl == NULL ? NULL : dl->parent;
+	pcb_data_t *dst_data = dl == NULL ? NULL : dl->parent.data;
 
 	linelist_foreach(&sl->Line, &it, line) {
 		if (src_has_real_layer) {
-			pcb_poly_restore_to_poly(src_data, PCB_TYPE_LINE, sl, line);
+			pcb_poly_restore_to_poly(src_data, PCB_OBJ_LINE, sl, line);
 			pcb_r_delete_entry(sl->line_tree, (pcb_box_t *)line);
 			chg++;
 		}
@@ -1179,12 +1093,12 @@ static int subc_relocate_layer_objs(pcb_layer_t *dl, pcb_data_t *src_data, pcb_l
 			chg++;
 		}
 		if (dst_is_pcb && (dl != NULL))
-			pcb_poly_clear_from_poly(dst_data, PCB_TYPE_LINE, dl, line);
+			pcb_poly_clear_from_poly(dst_data, PCB_OBJ_LINE, dl, line);
 	}
 
 	arclist_foreach(&sl->Arc, &it, arc) {
 		if (src_has_real_layer) {
-			pcb_poly_restore_to_poly(src_data, PCB_TYPE_ARC, sl, arc);
+			pcb_poly_restore_to_poly(src_data, PCB_OBJ_ARC, sl, arc);
 			pcb_r_delete_entry(sl->arc_tree, (pcb_box_t *)arc);
 			chg++;
 		}
@@ -1194,12 +1108,12 @@ static int subc_relocate_layer_objs(pcb_layer_t *dl, pcb_data_t *src_data, pcb_l
 			chg++;
 		}
 		if (dst_is_pcb && (dl != NULL))
-			pcb_poly_clear_from_poly(dst_data, PCB_TYPE_ARC, dl, arc);
+			pcb_poly_clear_from_poly(dst_data, PCB_OBJ_ARC, dl, arc);
 	}
 
 	textlist_foreach(&sl->Text, &it, text) {
 		if (src_has_real_layer) {
-			pcb_poly_restore_to_poly(src_data, PCB_TYPE_LINE, sl, text);
+			pcb_poly_restore_to_poly(src_data, PCB_OBJ_LINE, sl, text);
 			pcb_r_delete_entry(sl->text_tree, (pcb_box_t *)text);
 			chg++;
 		}
@@ -1209,7 +1123,7 @@ static int subc_relocate_layer_objs(pcb_layer_t *dl, pcb_data_t *src_data, pcb_l
 			chg++;
 		}
 		if (dst_is_pcb && (dl != NULL))
-			pcb_poly_clear_from_poly(dst_data, PCB_TYPE_TEXT, dl, text);
+			pcb_poly_clear_from_poly(dst_data, PCB_OBJ_TEXT, dl, text);
 	}
 
 	polylist_foreach(&sl->Polygon, &it, poly) {
@@ -1242,24 +1156,14 @@ static int subc_relocate_layer_objs(pcb_layer_t *dl, pcb_data_t *src_data, pcb_l
 
 static int subc_relocate_globals(pcb_data_t *dst, pcb_data_t *new_parent, pcb_subc_t *sc, int dst_is_pcb)
 {
-	pcb_pin_t *via;
 	pcb_pstk_t *ps;
 	gdl_iterator_t it;
 	int chg = 0;
 
-	pinlist_foreach(&sc->data->Via, &it, via) {
-		if (sc->data->via_tree != NULL)
-			pcb_r_delete_entry(sc->data->via_tree, (pcb_box_t *)via);
-		PCB_FLAG_CLEAR(PCB_FLAG_WARN | PCB_FLAG_FOUND | PCB_FLAG_SELECTED, via);
-		if ((dst != NULL) && (dst->via_tree != NULL))
-			pcb_r_insert_entry(dst->via_tree, (pcb_box_t *)via);
-		chg++;
-	}
-
 	padstacklist_foreach(&sc->data->padstack, &it, ps) {
 		const pcb_pstk_proto_t *proto = pcb_pstk_get_proto(ps);
 		assert(proto != NULL); /* the prototype must be accessible at the source else we can't add it in the dest */
-		pcb_poly_restore_to_poly(ps->parent.data, PCB_TYPE_PSTK, NULL, ps);
+		pcb_poly_restore_to_poly(ps->parent.data, PCB_OBJ_PSTK, NULL, ps);
 		if (sc->data->padstack_tree != NULL)
 			pcb_r_delete_entry(sc->data->padstack_tree, (pcb_box_t *)ps);
 		PCB_FLAG_CLEAR(PCB_FLAG_WARN | PCB_FLAG_FOUND | PCB_FLAG_SELECTED, ps);
@@ -1270,25 +1174,19 @@ static int subc_relocate_globals(pcb_data_t *dst, pcb_data_t *new_parent, pcb_su
 		ps->protoi = -1;
 		ps->parent.data = new_parent;
 		if (dst_is_pcb)
-			pcb_poly_clear_from_poly(ps->parent.data, PCB_TYPE_PSTK, NULL, ps);
+			pcb_poly_clear_from_poly(ps->parent.data, PCB_OBJ_PSTK, NULL, ps);
 		chg++;
 	}
 
 	/* bind globals */
 	if ((dst != NULL) && (dst_is_pcb)) {
-		if (dst->via_tree == NULL)
-			dst->via_tree = pcb_r_create_tree();
-		sc->data->via_tree = dst->via_tree;
-
 		if (dst->padstack_tree == NULL)
 			dst->padstack_tree = pcb_r_create_tree();
 		sc->data->padstack_tree = dst->padstack_tree;
 		chg++;
 	}
-	else {
-		sc->data->via_tree = NULL;
+	else
 		sc->data->padstack_tree = NULL;
-	}
 	return chg;
 }
 
@@ -1360,8 +1258,15 @@ void *pcb_subcop_add_to_buffer(pcb_opctx_t *ctx, pcb_subc_t *sc)
 {
 	pcb_subc_t *nsc;
 	nsc = pcb_subc_dup_at(NULL, ctx->buffer.dst, sc, 0, 0, pcb_false);
-	if (ctx->buffer.extraflg & PCB_FLAG_SELECTED)
+
+	if (ctx->buffer.extraflg & PCB_FLAG_SELECTED) {
+		pcb_undo_freeze_serial();
+		pcb_undo_freeze_add();
 		pcb_subc_select(NULL, nsc, PCB_CHGFLG_CLEAR, 0);
+		pcb_undo_unfreeze_add();
+		pcb_undo_unfreeze_serial();
+	}
+
 	return nsc;
 }
 
@@ -1436,9 +1341,6 @@ int pcb_subc_rebind(pcb_board_t *pcb, pcb_subc_t *sc)
 
 void pcb_subc_bind_globals(pcb_board_t *pcb, pcb_subc_t *subc)
 {
-	if (pcb->Data->via_tree == NULL)
-		pcb->Data->via_tree = pcb_r_create_tree();
-	subc->data->via_tree = pcb->Data->via_tree;
 	if (pcb->Data->padstack_tree == NULL)
 		pcb->Data->padstack_tree = pcb_r_create_tree();
 	subc->data->padstack_tree = pcb->Data->padstack_tree;
@@ -1447,25 +1349,25 @@ void pcb_subc_bind_globals(pcb_board_t *pcb, pcb_subc_t *subc)
 
 void *pcb_subcop_change_size(pcb_opctx_t *ctx, pcb_subc_t *sc)
 {
-	pcb_subc_op(ctx->chgsize.pcb->Data, sc, &ChangeSizeFunctions, ctx);
+	pcb_subc_op(ctx->chgsize.pcb->Data, sc, &ChangeSizeFunctions, ctx, PCB_SUBCOP_UNDO_BATCH);
 	return sc;
 }
 
 void *pcb_subcop_change_clear_size(pcb_opctx_t *ctx, pcb_subc_t *sc)
 {
-	pcb_subc_op(ctx->chgsize.pcb->Data, sc, &ChangeClearSizeFunctions, ctx);
+	pcb_subc_op(ctx->chgsize.pcb->Data, sc, &ChangeClearSizeFunctions, ctx, PCB_SUBCOP_UNDO_BATCH);
 	return sc;
 }
 
 void *pcb_subcop_change_1st_size(pcb_opctx_t *ctx, pcb_subc_t *sc)
 {
-	pcb_subc_op(ctx->chgsize.pcb->Data, sc, &Change1stSizeFunctions, ctx);
+	pcb_subc_op(ctx->chgsize.pcb->Data, sc, &Change1stSizeFunctions, ctx, PCB_SUBCOP_UNDO_BATCH);
 	return sc;
 }
 
 void *pcb_subcop_change_2nd_size(pcb_opctx_t *ctx, pcb_subc_t *sc)
 {
-	pcb_subc_op(ctx->chgsize.pcb->Data, sc, &Change2ndSizeFunctions, ctx);
+	pcb_subc_op(ctx->chgsize.pcb->Data, sc, &Change2ndSizeFunctions, ctx, PCB_SUBCOP_UNDO_BATCH);
 	return sc;
 }
 
@@ -1503,7 +1405,7 @@ void *pcb_subcop_destroy(pcb_opctx_t *ctx, pcb_subc_t *sc)
 void *pcb_subcop_remove(pcb_opctx_t *ctx, pcb_subc_t *sc)
 {
 	EraseSubc(sc);
-	pcb_undo_move_obj_to_remove(PCB_TYPE_SUBC, sc, sc, sc);
+	pcb_undo_move_obj_to_remove(PCB_OBJ_SUBC, sc, sc, sc);
 	return NULL;
 }
 
@@ -1521,50 +1423,11 @@ void *pcb_subc_remove(pcb_subc_t *sc)
 	return res;
 }
 
-
-void *pcb_subcop_clear_octagon(pcb_opctx_t *ctx, pcb_subc_t *sc)
-{
-	pcb_subc_op(ctx->chgsize.pcb->Data, sc, &ClrOctagonFunctions, ctx);
-	return sc;
-}
-
-void *pcb_subcop_set_octagon(pcb_opctx_t *ctx, pcb_subc_t *sc)
-{
-	pcb_subc_op(ctx->chgsize.pcb->Data, sc, &SetOctagonFunctions, ctx);
-	return sc;
-}
-
-void *pcb_subcop_change_octagon(pcb_opctx_t *ctx, pcb_subc_t *sc)
-{
-	pcb_subc_op(ctx->chgsize.pcb->Data, sc, &ChangeOctagonFunctions, ctx);
-	return sc;
-}
-
-void *pcb_subcop_clear_square(pcb_opctx_t *ctx, pcb_subc_t *sc)
-{
-	pcb_subc_op(ctx->chgsize.pcb->Data, sc, &ClrSquareFunctions, ctx);
-	return sc;
-}
-
-void *pcb_subcop_set_square(pcb_opctx_t *ctx, pcb_subc_t *sc)
-{
-	pcb_subc_op(ctx->chgsize.pcb->Data, sc, &SetSquareFunctions, ctx);
-	return sc;
-}
-
-void *pcb_subcop_change_square(pcb_opctx_t *ctx, pcb_subc_t *sc)
-{
-	pcb_subc_op(ctx->chgsize.pcb->Data, sc, &ChangeSquareFunctions, ctx);
-	return sc;
-}
-
-
-
 void *pcb_subcop_change_flag(pcb_opctx_t *ctx, pcb_subc_t *sc)
 {
 	static pcb_flag_values_t pcb_subc_flags = 0;
 	if (pcb_subc_flags == 0)
-		pcb_subc_flags = pcb_obj_valid_flags(PCB_TYPE_SUBC);
+		pcb_subc_flags = pcb_obj_valid_flags(PCB_OBJ_SUBC);
 
 	if (ctx->chgflag.flag == PCB_FLAG_FLOATER) {
 #warning subc TODO: subc-in-subc: can a whole subc be a floater? - add undo!
@@ -1572,7 +1435,7 @@ void *pcb_subcop_change_flag(pcb_opctx_t *ctx, pcb_subc_t *sc)
 		return sc;
 	}
 
-	pcb_subc_op(ctx->chgflag.pcb->Data, sc, &ChgFlagFunctions, ctx);
+	pcb_subc_op(ctx->chgflag.pcb->Data, sc, &ChgFlagFunctions, ctx, PCB_SUBCOP_UNDO_NORMAL);
 	if ((ctx->chgflag.flag & pcb_subc_flags) == ctx->chgflag.flag)
 		PCB_FLAG_CHANGE(ctx->chgflag.how, ctx->chgflag.flag, sc);
 	return sc;
@@ -1589,7 +1452,7 @@ void pcb_subc_select(pcb_board_t *pcb, pcb_subc_t *sc, pcb_change_flag_t how, in
 	ctx.chgflag.how = how;
 	ctx.chgflag.flag = PCB_FLAG_SELECTED;
 
-	pcb_subc_op((pcb == NULL ? NULL : pcb->Data), sc, &ChgFlagFunctions, &ctx);
+	pcb_subc_op((pcb == NULL ? NULL : pcb->Data), sc, &ChgFlagFunctions, &ctx, PCB_SUBCOP_UNDO_NORMAL);
 	PCB_FLAG_CHANGE(how, PCB_FLAG_SELECTED, sc);
 	if (redraw)
 		DrawSubc(sc);
@@ -1601,7 +1464,7 @@ void pcb_subc_mirror(pcb_data_t *data, pcb_subc_t *subc, pcb_coord_t y_offs, pcb
 	if ((data != NULL) && (data->subc_tree != NULL))
 		pcb_r_delete_entry(data->subc_tree, (pcb_box_t *)subc);
 
-	pcb_data_mirror(subc->data, y_offs, smirror, smirror);
+	pcb_data_mirror(subc->data, y_offs, smirror ? PCB_TXM_SIDE : PCB_TXM_COORD, smirror);
 	pcb_subc_bbox(subc);
 
 	if ((data != NULL) && (data->subc_tree != NULL))
@@ -1646,7 +1509,7 @@ pcb_bool pcb_subc_change_side(pcb_subc_t **subc, pcb_coord_t yoff)
 	newsc2 = pcb_subc_dup_at(pcb, data, newsc, 0, 0, pcb_true);
 	newsc2->ID = newsc->ID;
 	PCB_SET_PARENT(newsc2, data, data);
-	pcb_undo_add_subc_to_otherside(PCB_TYPE_SUBC, newsc2, newsc2, newsc2, yoff);
+	pcb_undo_add_subc_to_otherside(PCB_OBJ_SUBC, newsc2, newsc2, newsc2, yoff);
 
 	*subc = newsc2;
 	pcb_subc_free(newsc);
@@ -1663,6 +1526,10 @@ pcb_r_dir_t draw_subc_mark_callback(const pcb_box_t *b, void *cl)
 	pcb_subc_t *subc = (pcb_subc_t *) b;
 	pcb_box_t *bb = &subc->BoundingBox;
 	int selected = PCB_FLAG_TEST(PCB_FLAG_SELECTED, subc);
+	int freq = conf_core.appearance.subc.dash_freq;
+
+	if (freq > 32)
+		freq = 32;
 
 	pcb_gui->set_color(pcb_draw_out.fgGC, conf_core.appearance.color.element);
 	pcb_gui->set_line_cap(pcb_draw_out.fgGC, Trace_Cap);
@@ -1672,13 +1539,15 @@ pcb_r_dir_t draw_subc_mark_callback(const pcb_box_t *b, void *cl)
 	pcb_subc_draw_origin(pcb_draw_out.fgGC, subc, 0, 0);
 	pcb_gui->set_draw_xor(pcb_draw_out.fgGC, 0);
 
-	pcb_gui->set_line_width(pcb_draw_out.fgGC, 0);
-	pcb_gui->set_draw_xor(pcb_draw_out.fgGC, 1);
-	pcb_draw_dashed_line(pcb_draw_out.fgGC, bb->X1, bb->Y1, bb->X2, bb->Y1);
-	pcb_draw_dashed_line(pcb_draw_out.fgGC, bb->X1, bb->Y1, bb->X1, bb->Y2);
-	pcb_draw_dashed_line(pcb_draw_out.fgGC, bb->X2, bb->Y2, bb->X2, bb->Y1);
-	pcb_draw_dashed_line(pcb_draw_out.fgGC, bb->X2, bb->Y2, bb->X1, bb->Y2);
-	pcb_gui->set_draw_xor(pcb_draw_out.fgGC, 0);
+	if (freq >= 0) {
+		pcb_gui->set_line_width(pcb_draw_out.fgGC, 0);
+		pcb_gui->set_draw_xor(pcb_draw_out.fgGC, 1);
+		pcb_draw_dashed_line(pcb_draw_out.fgGC, bb->X1, bb->Y1, bb->X2, bb->Y1, freq);
+		pcb_draw_dashed_line(pcb_draw_out.fgGC, bb->X1, bb->Y1, bb->X1, bb->Y2, freq);
+		pcb_draw_dashed_line(pcb_draw_out.fgGC, bb->X2, bb->Y2, bb->X2, bb->Y1, freq);
+		pcb_draw_dashed_line(pcb_draw_out.fgGC, bb->X2, bb->Y2, bb->X1, bb->Y2, freq);
+		pcb_gui->set_draw_xor(pcb_draw_out.fgGC, 0);
+	}
 
 	if (subc->refdes != NULL) {
 		pcb_coord_t x0, y0, dx, dy;
@@ -1814,3 +1683,75 @@ pcb_bool pcb_subc_is_empty(pcb_subc_t *subc)
 {
 	return pcb_data_is_empty(subc->data);
 }
+
+pcb_layer_t *pcb_subc_get_layer(pcb_subc_t *sc, pcb_layer_type_t lyt, pcb_layer_combining_t comb, pcb_bool_t alloc, const char *name, pcb_bool req_name_match)
+{
+	int n;
+
+	/* look for an existing layer with matching lyt and comb first */
+	for(n = 0; n < sc->data->LayerN; n++) {
+		if (sc->data->Layer[n].meta.bound.type != lyt)
+			continue;
+		if ((comb != -1) && (sc->data->Layer[n].comb != comb))
+			continue;
+		if (req_name_match && (strcmp(sc->data->Layer[n].name, name) != 0))
+			continue;
+		return &sc->data->Layer[n];
+	}
+
+	if (!alloc)
+		return NULL;
+
+	if (sc->data->LayerN == PCB_MAX_LAYER)
+		return NULL;
+
+	n = sc->data->LayerN++;
+	if (name == NULL)
+		name = "";
+
+	if (comb == -1) {
+		/* "any" means default, for the given layer type */
+		if (lyt & PCB_LYT_MASK)
+			comb = PCB_LYC_SUB;
+		else
+			comb = 0; /* positive, manual */
+	}
+
+	memset(&sc->data->Layer[n], 0, sizeof(sc->data->Layer[n]));
+	sc->data->Layer[n].name = pcb_strdup(name);
+	sc->data->Layer[n].comb = comb;
+	sc->data->Layer[n].is_bound = 1;
+	sc->data->Layer[n].meta.bound.type = lyt;
+	sc->data->Layer[n].parent.data = sc->data;
+	sc->data->Layer[n].parent_type = PCB_PARENT_DATA;
+	sc->data->Layer[n].type = PCB_OBJ_LAYER;
+	return &sc->data->Layer[n];
+}
+
+void pcb_subc_part_changed_(pcb_any_obj_t *obj)
+{
+	pcb_subc_t *sc = pcb_obj_parent_subc(obj);
+	if (sc == NULL)
+		return;
+
+	/* can't do this incrementally: if a boundary object is smaller than before
+	   it has to make the subc bbox smaller too */
+	pcb_subc_bbox(sc);
+}
+
+
+const char *pcb_subc_name(pcb_subc_t *subc, const char *local_name)
+{
+	const char *val;
+
+	if (local_name != NULL) {
+		val = pcb_attribute_get(&subc->Attributes, local_name);
+		if (val != NULL)
+			return val;
+	}
+	val = pcb_attribute_get(&subc->Attributes, "visible_footprint");
+	if (val != NULL)
+		return val;
+	val = pcb_attribute_get(&subc->Attributes, "footprint");
+	return val;
+}
