@@ -31,6 +31,7 @@
 #include <libminuid/libminuid.h>
 #include <genht/htsp.h>
 #include "obj_common.h"
+#include "layer.h"
 #include "global_typedefs.h"
 
 
@@ -42,7 +43,7 @@ typedef enum pcb_subc_cached_s {
 } pcb_subc_cached_t;
 
 struct pcb_subc_s {
-	PCB_ANYOBJECTFIELDS;
+	PCB_ANY_PRIMITIVE_FIELDS;
 	minuid_bin_t uid;
 	pcb_data_t *data;
 	htsp_t terminals;
@@ -68,7 +69,7 @@ pcb_bool pcb_subc_smash_buffer(pcb_buffer_t *buff);
 
 void pcb_subc_mirror(pcb_data_t *data, pcb_subc_t *subc, pcb_coord_t y_offs, pcb_bool smirror);
 
-/* changes the side of the board an element is on; returns pcb_true if done */
+/* changes the side of the board a subc is on; returns pcb_true if done */
 pcb_bool pcb_subc_change_side(pcb_subc_t **subc, pcb_coord_t yoff);
 
 void pcb_subc_rotate(pcb_subc_t *subc, pcb_coord_t cx, pcb_coord_t cy, double cosa, double sina, double angle);
@@ -83,7 +84,7 @@ pcb_bool pcb_selected_subc_change_side(void);
 /* Draw a subcircuit for a preview (silk, copper and outline only) */
 void pcb_subc_draw_preview(const pcb_subc_t *sc, const pcb_box_t *drawn_area);
 
-void XORDrawSubc(pcb_subc_t *sc, pcb_coord_t DX, pcb_coord_t DY, int use_curr_side);
+void pcb_xordraw_subc(pcb_subc_t *sc, pcb_coord_t DX, pcb_coord_t DY, int use_curr_side);
 
 /* Redo the layer binding after the layer binding recipe changed in sc */
 int pcb_subc_rebind(pcb_board_t *pcb, pcb_subc_t *sc);
@@ -91,6 +92,11 @@ int pcb_subc_rebind(pcb_board_t *pcb, pcb_subc_t *sc);
 /* Do the initial global bindings of subc to pcb (rtree links) */
 void pcb_subc_bind_globals(pcb_board_t *pcb, pcb_subc_t *subc);
 
+/* Look up a layer by lyt and comb (and name, if req_name_match is true);
+   if not found and alloc is true, allocate a new layer with the given name.
+   Return NULL on error. */
+pcb_layer_t *pcb_subc_get_layer(pcb_subc_t *sc, pcb_layer_type_t lyt, pcb_layer_combining_t comb, pcb_bool_t alloc, const char *name, pcb_bool req_name_match);
+
 #include "rtree.h"
 pcb_r_dir_t draw_subc_mark_callback(const pcb_box_t *b, void *cl);
 void DrawSubc(pcb_subc_t *sc);
@@ -100,7 +106,7 @@ void EraseSubc(pcb_subc_t *sc);
 int pcb_subc_get_origin(pcb_subc_t *sc, pcb_coord_t *x, pcb_coord_t *y);
 int pcb_subc_get_rotation(pcb_subc_t *sc, double *rot);
 int pcb_subc_get_side(pcb_subc_t *sc, int *on_bottom);
-int pcb_subc_get_host_trans(const pcb_subc_t *sc, pcb_host_trans_t *tr);
+int pcb_subc_get_host_trans(pcb_subc_t *sc, pcb_host_trans_t *tr);
 
 /* Search for the named subc; name is relative path in hierarchy. Returns
    NULL if not found */
@@ -119,6 +125,16 @@ pcb_layer_t *pcb_loose_subc_layer(pcb_board_t *pcb, pcb_layer_t *brd_layer);
 /* Returns whether there's no object in the subc */
 pcb_bool pcb_subc_is_empty(pcb_subc_t *subc);
 
+/* Return the footprint name; if local_name is not NULL, that attribute
+   is queried first; if that doesn't exist, "visible_footprint" and then
+   "footprint" is returned (or NULL if nothing is found) */
+const char *pcb_subc_name(pcb_subc_t *subc, const char *local_name);
+
+
+/* Update the subcircuit (e.g. bbox) after a part of the subcircuit has changed */
+void pcb_subc_part_changed_(pcb_any_obj_t *obj);
+#define pcb_subc_part_changed(obj) pcb_subc_part_changed_((pcb_any_obj_t *)obj)
+
 /*** subc creation helpers ***/
 
 /* Create the aux layer for a subc, set origin to ox;oy and rotation to rot */
