@@ -54,11 +54,11 @@ PCB_INLINE pcb_subc_t *pcb_lobj_parent_subc(pcb_parenttype_t pt, pcb_parent_t *p
 	if (p->layer == NULL)
 		return NULL;
 	
-	if (p->layer->parent == NULL)
+	if (p->layer->parent.data == NULL)
 		return NULL;
 
-	if (p->layer->parent->parent_type == PCB_PARENT_SUBC)
-		return p->layer->parent->parent.subc;
+	if (p->layer->parent.data->parent_type == PCB_PARENT_SUBC)
+		return p->layer->parent.data->parent.subc;
 	return NULL;
 }
 
@@ -66,7 +66,6 @@ PCB_INLINE pcb_subc_t *pcb_lobj_parent_subc(pcb_parenttype_t pt, pcb_parent_t *p
 PCB_INLINE pcb_subc_t *pcb_obj_parent_subc(pcb_any_obj_t *obj)
 {
 	switch(obj->type) {
-		case PCB_OBJ_VIA:
 		case PCB_OBJ_PSTK:
 		case PCB_OBJ_SUBC:
 			return pcb_gobj_parent_subc(obj->parent_type, &obj->parent);
@@ -77,21 +76,6 @@ PCB_INLINE pcb_subc_t *pcb_obj_parent_subc(pcb_any_obj_t *obj)
 		case PCB_OBJ_ARC:
 			return pcb_lobj_parent_subc(obj->parent_type, &obj->parent);
 
-#if 0
-		case PCB_OBJ_RATLINE:
-			/* easy case: can not be in a subc at all */
-			return 0;
-
-		case PCB_OBJ_PIN:
-		case PCB_OBJ_PAD:
-		case PCB_OBJ_ELEMENT_NAME:
-		case PCB_OBJ_ELEMENT:
-		case PCB_OBJ_ELEMENT_LINE:
-		case PCB_OBJ_ELEMENT_ARC:
-			/* easy case: these obsolete constructs can not be in a subc at all */
-			return 0;
-#endif
-
 		default:
 			/* anything else: virtual */
 			return 0;
@@ -103,10 +87,9 @@ PCB_INLINE pcb_bool_t pcb_obj_is_under(pcb_any_obj_t *obj, pcb_data_t *data)
 {
 	for(;;) {
 		switch(obj->parent_type) {
-			case PCB_PARENT_ELEMENT: return pcb_false; /* remove this */
 			case PCB_PARENT_INVALID: return pcb_false;
 			case PCB_PARENT_BOARD:   return pcb_false;
-			case PCB_PARENT_LAYER:   if (obj->parent.layer->parent == data) return pcb_true; break;
+			case PCB_PARENT_LAYER:   if (obj->parent.layer->parent.data == data) return pcb_true; break;
 			case PCB_PARENT_SUBC:    if (obj->parent.subc->data == data) return pcb_true; break;
 			case PCB_PARENT_DATA:    if (obj->parent.data == data) return pcb_true; break;
 		}
