@@ -38,6 +38,7 @@
 #include "obj_subc_parent.h"
 #include "pcb-printf.h"
 #include "undo.h"
+#include "macro.h"
 #include "polygon.h"
 
 static const char core_term_cookie[] = "core-term";
@@ -222,7 +223,7 @@ static int undo_term_rename_swap(void *udata)
 		pcb_attribute_remove(&r->obj->Attributes, "term");
 
 	if (r->obj->type == PCB_OBJ_POLY)
-		pcb_poly_init_clip(r->obj->parent.layer->parent, r->obj->parent.layer, (pcb_poly_t *)r->obj);
+		pcb_poly_init_clip(r->obj->parent.layer->parent.data, r->obj->parent.layer, (pcb_poly_t *)r->obj);
 
 	return res;
 }
@@ -274,7 +275,7 @@ pcb_term_err_t pcb_term_undoable_rename(pcb_board_t *pcb, pcb_any_obj_t *obj, co
 	undo_term_rename_swap(r);
 
 	if (obj->type == PCB_OBJ_POLY)
-		pcb_poly_init_clip(obj->parent.layer->parent, obj->parent.layer, (pcb_poly_t *)obj);
+		pcb_poly_init_clip(obj->parent.layer->parent.data, obj->parent.layer, (pcb_poly_t *)obj);
 
 	pcb_undo_inc_serial();
 	return PCB_TERM_ERR_SUCCESS;
@@ -316,11 +317,8 @@ pcb_any_obj_t *pcb_term_find_name(pcb_board_t *pcb, pcb_data_t *data, pcb_layer_
 	if (PCB_FLAG_TEST(PCB_FLAG_NONETLIST, subc))
 		return NULL;
 
-	/* search for global objects: via and padstack */
+	/* search for global objects: padstack */
 	if (lyt & (PCB_LYT_COPPER | PCB_LYT_MASK | PCB_LYT_OUTLINE)) {
-		PCB_VIA_LOOP(subc->data) {
-			CHECK_TERM_GL(via);
-		} PCB_END_LOOP;
 		PCB_PADSTACK_LOOP(subc->data) {
 			CHECK_TERM_GL(padstack);
 		} PCB_END_LOOP;
