@@ -158,7 +158,7 @@ static int pcb_text_render_str_cb(void *ctx, gds_t *s, const char **input)
 		pcb_attribute_list_t *attr = &text->Attributes;
 		path = key+2;
 		if ((path[0] == 'p') && (memcmp(path, "parent.", 7) == 0)) {
-			pcb_data_t *par = text->parent.layer->parent;
+			pcb_data_t *par = text->parent.layer->parent.data;
 			if (par->parent_type == PCB_PARENT_SUBC)
 				attr = &par->parent.subc->Attributes;
 			else if (par->parent_type == PCB_PARENT_BOARD)
@@ -185,7 +185,10 @@ static unsigned char *pcb_text_render_str(pcb_text_t *text)
 		return (unsigned char *)text->TextString;
 
 	res = (unsigned char *)pcb_strdup_subst(text->TextString, pcb_text_render_str_cb, text, PCB_SUBST_PERCENT | PCB_SUBST_CONF);
-	if (*res == '\0') {
+	if (res == NULL) {
+		res = (unsigned char *)pcb_strdup("<!>");
+	}
+	else if (*res == '\0') {
 		free(res);
 		res = (unsigned char *)pcb_strdup("<?>");
 	}
@@ -231,7 +234,7 @@ void pcb_text_bbox(pcb_font_t *FontPtr, pcb_text_t *Text)
 	/* Calculate the bounding box based on the larger of the thicknesses
 	 * the text might clamped at on silk or copper layers.
 	 */
-	min_final_radius = MAX(PCB->minWid, PCB->minSlk) / 2;
+	min_final_radius = MAX(conf_core.design.min_wid, conf_core.design.min_slk) / 2;
 
 	/* Pre-adjust the line radius for the fact we are initially computing the
 	 * bounds of the un-scaled text, and the thickness clamping applies to
@@ -333,10 +336,10 @@ void pcb_text_bbox(pcb_font_t *FontPtr, pcb_text_t *Text)
 	/* the bounding box covers the extent of influence
 	 * so it must include the clearance values too
 	 */
-	Text->BoundingBox.X1 -= PCB->Bloat;
-	Text->BoundingBox.Y1 -= PCB->Bloat;
-	Text->BoundingBox.X2 += PCB->Bloat;
-	Text->BoundingBox.Y2 += PCB->Bloat;
+	Text->BoundingBox.X1 -= conf_core.design.bloat;
+	Text->BoundingBox.Y1 -= conf_core.design.bloat;
+	Text->BoundingBox.X2 += conf_core.design.bloat;
+	Text->BoundingBox.Y2 += conf_core.design.bloat;
 	pcb_close_box(&Text->BoundingBox);
 	pcb_text_free_str(Text, rendered);
 }
@@ -390,7 +393,7 @@ void *pcb_textop_move_buffer(pcb_opctx_t *ctx, pcb_layer_t *dstly, pcb_text_t *t
 		dstly = &ctx->buffer.dst->Layer[pcb_layer_id(ctx->buffer.src, srcly)];
 
 	pcb_r_delete_entry(srcly->text_tree, (pcb_box_t *) text);
-	pcb_poly_restore_to_poly(ctx->buffer.src, PCB_TYPE_TEXT, srcly, text);
+	pcb_poly_restore_to_poly(ctx->buffer.src, PCB_OBJ_TEXT, srcly, text);
 
 	textlist_remove(text);
 	textlist_append(&dstly->Text, text);
@@ -398,7 +401,7 @@ void *pcb_textop_move_buffer(pcb_opctx_t *ctx, pcb_layer_t *dstly, pcb_text_t *t
 	if (!dstly->text_tree)
 		dstly->text_tree = pcb_r_create_tree();
 	pcb_r_insert_entry(dstly->text_tree, (pcb_box_t *) text);
-	pcb_poly_clear_from_poly(ctx->buffer.dst, PCB_TYPE_TEXT, dstly, text);
+	pcb_poly_clear_from_poly(ctx->buffer.dst, PCB_OBJ_TEXT, dstly, text);
 
 	PCB_SET_PARENT(text, layer, dstly);
 
@@ -414,14 +417,14 @@ void *pcb_textop_change_size(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_text_t *T
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Text))
 		return NULL;
 	if (value <= PCB_MAX_TEXTSCALE && value >= PCB_MIN_TEXTSCALE && value != Text->Scale) {
-		pcb_undo_add_obj_to_size(PCB_TYPE_TEXT, Layer, Text, Text);
+		pcb_undo_add_obj_to_size(PCB_OBJ_TEXT, Layer, Text, Text);
 		pcb_text_invalidate_erase(Layer, Text);
 		pcb_r_delete_entry(Layer->text_tree, (pcb_box_t *) Text);
-		pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_TEXT, Layer, Text);
+		pcb_poly_restore_to_poly(PCB->Data, PCB_OBJ_TEXT, Layer, Text);
 		Text->Scale = value;
 		pcb_text_bbox(pcb_font(PCB, Text->fid, 1), Text);
 		pcb_r_insert_entry(Layer->text_tree, (pcb_box_t *) Text);
-		pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_TEXT, Layer, Text);
+		pcb_poly_clear_from_poly(PCB->Data, PCB_OBJ_TEXT, Layer, Text);
 		pcb_text_invalidate_draw(Layer, Text);
 		return Text;
 	}
@@ -438,13 +441,13 @@ void *pcb_textop_change_name(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_text_t *T
 		return NULL;
 	pcb_text_invalidate_erase(Layer, Text);
 	pcb_r_delete_entry(Layer->text_tree, (pcb_box_t *)Text);
-	pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_TEXT, Layer, Text);
+	pcb_poly_restore_to_poly(PCB->Data, PCB_OBJ_TEXT, Layer, Text);
 	Text->TextString = ctx->chgname.new_name;
 
 	/* calculate size of the bounding box */
 	pcb_text_bbox(pcb_font(PCB, Text->fid, 1), Text);
 	pcb_r_insert_entry(Layer->text_tree, (pcb_box_t *) Text);
-	pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_TEXT, Layer, Text);
+	pcb_poly_clear_from_poly(PCB->Data, PCB_OBJ_TEXT, Layer, Text);
 	pcb_text_invalidate_draw(Layer, Text);
 	return old;
 }
@@ -456,14 +459,14 @@ void *pcb_textop_change_join(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_text_t *T
 		return NULL;
 	pcb_text_invalidate_erase(Layer, Text);
 	if (PCB_FLAG_TEST(PCB_FLAG_CLEARLINE, Text)) {
-		pcb_undo_add_obj_to_clear_poly(PCB_TYPE_TEXT, Layer, Text, Text, pcb_false);
-		pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_TEXT, Layer, Text);
+		pcb_undo_add_obj_to_clear_poly(PCB_OBJ_TEXT, Layer, Text, Text, pcb_false);
+		pcb_poly_restore_to_poly(PCB->Data, PCB_OBJ_TEXT, Layer, Text);
 	}
 	pcb_undo_add_obj_to_flag(Text);
 	PCB_FLAG_TOGGLE(PCB_FLAG_CLEARLINE, Text);
 	if (PCB_FLAG_TEST(PCB_FLAG_CLEARLINE, Text)) {
-		pcb_undo_add_obj_to_clear_poly(PCB_TYPE_TEXT, Layer, Text, Text, pcb_true);
-		pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_TEXT, Layer, Text);
+		pcb_undo_add_obj_to_clear_poly(PCB_OBJ_TEXT, Layer, Text, Text, pcb_true);
+		pcb_poly_clear_from_poly(PCB->Data, PCB_OBJ_TEXT, Layer, Text);
 	}
 	pcb_text_invalidate_draw(Layer, Text);
 	return Text;
@@ -494,7 +497,7 @@ void *pcb_textop_copy(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_text_t *Text)
 											 Text->Y + ctx->copy.DeltaY, Text->Direction, Text->Scale, Text->TextString, pcb_flag_mask(Text->Flags, PCB_FLAG_FOUND));
 	pcb_text_copy_meta(text, Text);
 	pcb_text_invalidate_draw(Layer, text);
-	pcb_undo_add_obj_to_create(PCB_TYPE_TEXT, Layer, text, text);
+	pcb_undo_add_obj_to_create(PCB_OBJ_TEXT, Layer, text, text);
 	return text;
 }
 
@@ -514,10 +517,10 @@ void *pcb_textop_move_noclip(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_text_t *T
 void *pcb_textop_move(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_text_t *Text)
 {
 	pcb_r_delete_entry(Layer->text_tree, (pcb_box_t *) Text);
-	pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_TEXT, Layer, Text);
+	pcb_poly_restore_to_poly(PCB->Data, PCB_OBJ_TEXT, Layer, Text);
 	pcb_textop_move_noclip(ctx, Layer, Text);
 	pcb_r_insert_entry(Layer->text_tree, (pcb_box_t *) Text);
-	pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_TEXT, Layer, Text);
+	pcb_poly_clear_from_poly(PCB->Data, PCB_OBJ_TEXT, Layer, Text);
 	return Text;
 }
 
@@ -525,11 +528,11 @@ void *pcb_textop_clip(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_text_t *Text)
 {
 	if (ctx->clip.restore) {
 		pcb_r_delete_entry(Layer->text_tree, (pcb_box_t *) Text);
-		pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_TEXT, Layer, Text);
+		pcb_poly_restore_to_poly(PCB->Data, PCB_OBJ_TEXT, Layer, Text);
 	}
 	if (ctx->clip.clear) {
 		pcb_r_insert_entry(Layer->text_tree, (pcb_box_t *) Text);
-		pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_TEXT, Layer, Text);
+		pcb_poly_clear_from_poly(PCB->Data, PCB_OBJ_TEXT, Layer, Text);
 	}
 	return Text;
 }
@@ -538,7 +541,7 @@ void *pcb_textop_clip(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_text_t *Text)
 /* moves a text object between layers; lowlevel routines */
 void *pcb_textop_move_to_layer_low(pcb_opctx_t *ctx, pcb_layer_t * Source, pcb_text_t * text, pcb_layer_t * Destination)
 {
-	pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_TEXT, Source, text);
+	pcb_poly_restore_to_poly(PCB->Data, PCB_OBJ_TEXT, Source, text);
 	pcb_r_delete_entry(Source->text_tree, (pcb_box_t *) text);
 
 	textlist_remove(text);
@@ -554,7 +557,7 @@ void *pcb_textop_move_to_layer_low(pcb_opctx_t *ctx, pcb_layer_t * Source, pcb_t
 	if (!Destination->text_tree)
 		Destination->text_tree = pcb_r_create_tree();
 	pcb_r_insert_entry(Destination->text_tree, (pcb_box_t *) text);
-	pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_TEXT, Destination, text);
+	pcb_poly_clear_from_poly(PCB->Data, PCB_OBJ_TEXT, Destination, text);
 
 	PCB_SET_PARENT(text, layer, Destination);
 
@@ -569,7 +572,7 @@ void *pcb_textop_move_to_layer(pcb_opctx_t *ctx, pcb_layer_t * layer, pcb_text_t
 		return NULL;
 	}
 	if (ctx->move.dst_layer != layer) {
-		pcb_undo_add_obj_to_move_to_layer(PCB_TYPE_TEXT, layer, text, text);
+		pcb_undo_add_obj_to_move_to_layer(PCB_OBJ_TEXT, layer, text, text);
 		if (layer->meta.real.vis)
 			pcb_text_invalidate_erase(layer, text);
 		text = pcb_textop_move_to_layer_low(ctx, layer, text, ctx->move.dst_layer);
@@ -598,7 +601,7 @@ void *pcb_textop_remove(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_text_t *Text)
 		pcb_text_invalidate_erase(Layer, Text);
 		pcb_r_delete_entry(Layer->text_tree, (pcb_box_t *)Text);
 	}
-	pcb_undo_move_obj_to_remove(PCB_TYPE_TEXT, Layer, Text, Text);
+	pcb_undo_move_obj_to_remove(PCB_OBJ_TEXT, Layer, Text, Text);
 	return NULL;
 }
 
@@ -638,11 +641,13 @@ void pcb_text_rotate90(pcb_text_t *Text, pcb_coord_t X, pcb_coord_t Y, unsigned
 void *pcb_textop_rotate90(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_text_t *Text)
 {
 	pcb_text_invalidate_erase(Layer, Text);
-	pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_TEXT, Layer, Text);
-	pcb_r_delete_entry(Layer->text_tree, (pcb_box_t *) Text);
+	pcb_poly_restore_to_poly(PCB->Data, PCB_OBJ_TEXT, Layer, Text);
+	if (Layer->text_tree != NULL)
+		pcb_r_delete_entry(Layer->text_tree, (pcb_box_t *) Text);
 	pcb_text_rotate90(Text, ctx->rotate.center_x, ctx->rotate.center_y, ctx->rotate.number);
-	pcb_r_insert_entry(Layer->text_tree, (pcb_box_t *) Text);
-	pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_TEXT, Layer, Text);
+	if (Layer->text_tree != NULL)
+		pcb_r_insert_entry(Layer->text_tree, (pcb_box_t *) Text);
+	pcb_poly_clear_from_poly(PCB->Data, PCB_OBJ_TEXT, Layer, Text);
 	pcb_text_invalidate_draw(Layer, Text);
 	return Text;
 }
@@ -651,8 +656,9 @@ void *pcb_textop_rotate(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_text_t *Text)
 {
 	int steps;
 	pcb_text_invalidate_erase(Layer, Text);
-	pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_TEXT, Layer, Text);
-	pcb_r_delete_entry(Layer->text_tree, (pcb_box_t *) Text);
+	pcb_poly_restore_to_poly(PCB->Data, PCB_OBJ_TEXT, Layer, Text);
+	if (Layer->text_tree != NULL)
+		pcb_r_delete_entry(Layer->text_tree, (pcb_box_t *) Text);
 
 	steps = (int)ctx->rotate.angle / 90;
 	if (steps > 0)
@@ -661,8 +667,9 @@ void *pcb_textop_rotate(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_text_t *Text)
 	pcb_rotate(&Text->X, &Text->Y, ctx->rotate.center_x, ctx->rotate.center_y, ctx->rotate.cosa, ctx->rotate.sina);
 	pcb_text_bbox(NULL, Text);
 
-	pcb_r_insert_entry(Layer->text_tree, (pcb_box_t *) Text);
-	pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_TEXT, Layer, Text);
+	if (Layer->text_tree != NULL)
+		pcb_r_insert_entry(Layer->text_tree, (pcb_box_t *) Text);
+	pcb_poly_clear_from_poly(PCB->Data, PCB_OBJ_TEXT, Layer, Text);
 	pcb_text_invalidate_draw(Layer, Text);
 	return Text;
 }
@@ -679,36 +686,47 @@ void pcb_text_flip_side(pcb_layer_t *layer, pcb_text_t *text, pcb_coord_t y_offs
 		pcb_r_insert_entry(layer->text_tree, (pcb_box_t *) text);
 }
 
+void pcb_text_mirror_coords(pcb_layer_t *layer, pcb_text_t *text, pcb_coord_t y_offs)
+{
+	if (layer->text_tree != NULL)
+		pcb_r_delete_entry(layer->text_tree, (pcb_box_t *) text);
+	text->X = PCB_SWAP_X(text->X);
+	text->Y = PCB_SWAP_Y(text->Y) + y_offs;
+	pcb_text_bbox(pcb_font(PCB, text->fid, 1), text);
+	if (layer->text_tree != NULL)
+		pcb_r_insert_entry(layer->text_tree, (pcb_box_t *) text);
+}
+
 void pcb_text_set_font(pcb_layer_t *layer, pcb_text_t *text, pcb_font_id_t fid)
 {
-	pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_TEXT, layer, text);
+	pcb_poly_restore_to_poly(PCB->Data, PCB_OBJ_TEXT, layer, text);
 	pcb_r_delete_entry(layer->text_tree, (pcb_box_t *) text);
 	text->fid = fid;
 	pcb_text_bbox(pcb_font(PCB, text->fid, 1), text);
 	pcb_r_insert_entry(layer->text_tree, (pcb_box_t *) text);
-	pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_TEXT, layer, text);
+	pcb_poly_clear_from_poly(PCB->Data, PCB_OBJ_TEXT, layer, text);
 }
 
 void pcb_text_update(pcb_layer_t *layer, pcb_text_t *text)
 {
-	pcb_data_t *data = layer->parent;
+	pcb_data_t *data = layer->parent.data;
 	pcb_board_t *pcb = pcb_data_get_top(data);
 
 	if (pcb == NULL)
 		return;
 
-	pcb_poly_restore_to_poly(data, PCB_TYPE_TEXT, layer, text);
+	pcb_poly_restore_to_poly(data, PCB_OBJ_TEXT, layer, text);
 	pcb_r_delete_entry(layer->text_tree, (pcb_box_t *) text);
 	pcb_text_bbox(pcb_font(pcb, text->fid, 1), text);
 	pcb_r_insert_entry(layer->text_tree, (pcb_box_t *) text);
-	pcb_poly_clear_from_poly(data, PCB_TYPE_TEXT, layer, text);
+	pcb_poly_clear_from_poly(data, PCB_OBJ_TEXT, layer, text);
 }
 
 void *pcb_textop_change_flag(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_text_t *Text)
 {
 	static pcb_flag_values_t pcb_text_flags = 0;
 	if (pcb_text_flags == 0)
-		pcb_text_flags = pcb_obj_valid_flags(PCB_TYPE_TEXT);
+		pcb_text_flags = pcb_obj_valid_flags(PCB_OBJ_TEXT);
 
 	if ((ctx->chgflag.flag & pcb_text_flags) != ctx->chgflag.flag)
 		return NULL;
@@ -717,12 +735,12 @@ void *pcb_textop_change_flag(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_text_t *T
 	pcb_undo_add_obj_to_flag(Text);
 
 	if (ctx->chgflag.flag & PCB_FLAG_CLEARLINE)
-		pcb_poly_restore_to_poly(ctx->chgflag.pcb->Data, PCB_TYPE_TEXT, Text->parent.layer, Text);
+		pcb_poly_restore_to_poly(ctx->chgflag.pcb->Data, PCB_OBJ_TEXT, Text->parent.layer, Text);
 
 	PCB_FLAG_CHANGE(ctx->chgflag.how, ctx->chgflag.flag, Text);
 
 	if (ctx->chgflag.flag & PCB_FLAG_CLEARLINE)
-		pcb_poly_clear_from_poly(ctx->chgflag.pcb->Data, PCB_TYPE_TEXT, Text->parent.layer, Text);
+		pcb_poly_clear_from_poly(ctx->chgflag.pcb->Data, PCB_OBJ_TEXT, Text->parent.layer, Text);
 
 	return Text;
 }
@@ -780,9 +798,7 @@ static void draw_text_poly(pcb_text_t *Text, pcb_poly_t *poly, pcb_coord_t x0, i
 }
 
 
-/* ---------------------------------------------------------------------------
- * lowlevel drawing routine for text objects
- */
+/* lowlevel drawing routine for text objects */
 static void DrawTextLowLevel_(pcb_text_t *Text, pcb_coord_t min_line_width, int xordraw, pcb_coord_t xordx, pcb_coord_t xordy)
 {
 	pcb_coord_t x = 0;
@@ -970,9 +986,9 @@ static void pcb_text_draw(pcb_layer_t *layer, pcb_text_t *text, int allow_term_g
 		flg = pcb_layergrp_flags(PCB, layer->meta.real.grp);
 
 	if (flg & PCB_LYT_SILK)
-		min_silk_line = PCB->minSlk;
+		min_silk_line = conf_core.design.min_slk;
 	else
-		min_silk_line = PCB->minWid;
+		min_silk_line = conf_core.design.min_wid;
 
 	pcb_text_draw_(text, min_silk_line, allow_term_gfx);
 }
@@ -997,6 +1013,9 @@ pcb_r_dir_t pcb_text_draw_term_callback(const pcb_box_t * b, void *cl)
 	pcb_layer_t *layer = cl;
 	pcb_text_t *text = (pcb_text_t *) b;
 
+	if (pcb_hidden_floater((pcb_any_obj_t*)b))
+		return PCB_R_DIR_FOUND_CONTINUE;
+
 	if (!PCB->SubcPartsOn && pcb_lobj_parent_subc(text->parent_type, &text->parent))
 		return PCB_R_DIR_FOUND_CONTINUE;
 
@@ -1018,6 +1037,12 @@ void pcb_text_invalidate_draw(pcb_layer_t *Layer, pcb_text_t *Text)
 void pcb_text_draw_xor(pcb_text_t *text, pcb_coord_t x, pcb_coord_t y)
 {
 	DrawTextLowLevel_(text, 0, 1, x, y);
+	if (conf_core.appearance.text_host_bbox) {
+		pcb_gui->draw_line(pcb_crosshair.GC, x + text->BoundingBox.X1, y + text->BoundingBox.Y1, x + text->BoundingBox.X1, y + text->BoundingBox.Y2);
+		pcb_gui->draw_line(pcb_crosshair.GC, x + text->BoundingBox.X1, y + text->BoundingBox.Y1, x + text->BoundingBox.X2, y + text->BoundingBox.Y1);
+		pcb_gui->draw_line(pcb_crosshair.GC, x + text->BoundingBox.X2, y + text->BoundingBox.Y2, x + text->BoundingBox.X1, y + text->BoundingBox.Y2);
+		pcb_gui->draw_line(pcb_crosshair.GC, x + text->BoundingBox.X2, y + text->BoundingBox.Y2, x + text->BoundingBox.X2, y + text->BoundingBox.Y1);
+	}
 }
 
 /*** init ***/
@@ -1038,15 +1063,6 @@ static int pcb_text_font_chg_data(pcb_data_t *data, pcb_font_id_t fid)
 		} PCB_END_LOOP;
 	} PCB_END_LOOP;
 
-	PCB_ELEMENT_LOOP(data); {
-		PCB_ELEMENT_PCB_TEXT_LOOP(element); {
-			if (text->fid == fid) {
-				pcb_element_text_update(PCB, data, element, n);
-				need_redraw = 1;
-			}
-		} PCB_END_LOOP;
-	} PCB_END_LOOP;
-
 	PCB_SUBC_LOOP(data); {
 		int chg = pcb_text_font_chg_data(subc->data, fid);
 		if (chg) {
