@@ -36,14 +36,13 @@
 #include <genvector/gds_char.h>
 
 struct pcb_text_s {
-	PCB_ANYOBJECTFIELDS;
+	PCB_ANY_PRIMITIVE_FIELDS;
 	int Scale;                    /* text scaling in percent */
 	pcb_coord_t X, Y;                   /* origin */
 	pcb_uint8_t Direction;
 	pcb_font_id_t fid;
 	char *TextString;             /* string */
-	void *Element;                /* TODO: remove this in favor of generic parenting */
-	gdl_elem_t link;              /* a text is in a list of a layer or an element */
+	gdl_elem_t link;              /* a text is in a list of a layer */
 };
 
 /* These need to be carefully written to avoid overflows, and return
@@ -65,6 +64,7 @@ void pcb_add_text_on_layer(pcb_layer_t *Layer, pcb_text_t *text, pcb_font_t *PCB
 void pcb_text_bbox(pcb_font_t *FontPtr, pcb_text_t *Text);
 void pcb_text_rotate90(pcb_text_t *Text, pcb_coord_t X, pcb_coord_t Y, unsigned Number);
 void pcb_text_flip_side(pcb_layer_t *layer, pcb_text_t *text, pcb_coord_t y_offs);
+void pcb_text_mirror_coords(pcb_layer_t *layer, pcb_text_t *text, pcb_coord_t y_offs); /* mirror the coords but do not mirror the text itself (no on-solder) */
 void pcb_text_set_font(pcb_layer_t *layer, pcb_text_t *text, pcb_font_id_t fid);
 void pcb_text_update(pcb_layer_t *layer, pcb_text_t *text);
 
