@@ -52,61 +52,19 @@
 #include "compat_nls.h"
 #include "layer_vis.h"
 #include "operation.h"
-
-/* --------------------------------------------------------------------------- */
-
-static pcb_element_t *element_cache = NULL;
-
-static pcb_element_t *find_element_by_refdes(const char *refdes)
-{
-	if (element_cache && PCB_ELEM_NAME_REFDES(element_cache)
-			&& strcmp(PCB_ELEM_NAME_REFDES(element_cache), refdes) == 0)
-		return element_cache;
-
-	PCB_ELEMENT_LOOP(PCB->Data);
-	{
-		if (PCB_ELEM_NAME_REFDES(element)
-				&& strcmp(PCB_ELEM_NAME_REFDES(element), refdes) == 0) {
-			element_cache = element;
-			return element_cache;
-		}
-	}
-	PCB_END_LOOP;
-	return NULL;
-}
-
-#warning subc TODO: remove this when removing element support
-static pcb_attribute_t *lookup_attr(pcb_attribute_list_t *list, const char *name)
-{
-	int i;
-	for (i = 0; i < list->Number; i++)
-		if (strcmp(list->List[i].name, name) == 0)
-			return &list->List[i];
-	return NULL;
-}
-
-#warning subc TODO: remove this when removing element support
-static void delete_attr(pcb_attribute_list_t *list, pcb_attribute_t * attr)
-{
-	int idx = attr - list->List;
-	if (idx < 0 || idx >= list->Number)
-		return;
-	if (list->Number - idx > 1)
-		memmove(attr, attr + 1, (list->Number - idx - 1) * sizeof(pcb_attribute_t));
-	list->Number--;
-}
-
-/* ------------------------------------------------------------ */
+#include "obj_pstk.h"
+#include "macro.h"
+#include "rotate.h"
 
 static const char pcb_acts_Attributes[] = "Attributes(Layout|Layer|Element|Subc)\n" "Attributes(Layer,layername)";
 
 static const char pcb_acth_Attributes[] =
-	"Let the user edit the attributes of the layout, current or given\n" "layer, or selected element or subcircuit.";
+	"Let the user edit the attributes of the layout, current or given\n" "layer, or selected subcircuit.";
 
 /* %start-doc actions Attributes
 
 This just pops up a dialog letting the user edit the attributes of the
-pcb, an element, or a layer.
+pcb, a subcircuit, or a layer.
 
 %end-doc */
 
@@ -153,44 +111,6 @@ static int pcb_act_Attributes(int argc, const char **argv, pcb_coord_t x, pcb_co
 		}
 
 	case F_Element:
-		{
-			int n_found = 0;
-			pcb_element_t *e = NULL;
-			PCB_ELEMENT_LOOP(PCB->Data);
-			{
-				if (PCB_FLAG_TEST(PCB_FLAG_SELECTED, element)) {
-					e = element;
-					n_found++;
-				}
-			}
-			PCB_END_LOOP;
-			if (n_found > 1) {
-				pcb_message(PCB_MSG_ERROR, _("Too many elements selected\n"));
-				return 1;
-			}
-			if (n_found == 0) {
-				void *ptrtmp;
-				pcb_gui->get_coords(_("Click on an element"), &x, &y);
-				if ((pcb_search_screen(x, y, PCB_TYPE_ELEMENT, &ptrtmp, &ptrtmp, &ptrtmp)) != PCB_TYPE_NONE)
-					e = (pcb_element_t *) ptrtmp;
-				else {
-					pcb_message(PCB_MSG_ERROR, _("No element found there\n"));
-					return 1;
-				}
-			}
-
-			if (PCB_ELEM_NAME_REFDES(e)) {
-				buf = (char *) malloc(strlen(PCB_ELEM_NAME_REFDES(e)) + strlen("Element X Attributes"));
-				sprintf(buf, "Element %s Attributes", PCB_ELEM_NAME_REFDES(e));
-			}
-			else {
-				buf = pcb_strdup("Unnamed Element Attributes");
-			}
-			pcb_gui->edit_attributes(buf, &(e->Attributes));
-			free(buf);
-			break;
-		}
-
 	case F_Subc:
 		{
 			int n_found = 0;
@@ -210,7 +130,7 @@ static int pcb_act_Attributes(int argc, const char **argv, pcb_coord_t x, pcb_co
 			if (n_found == 0) {
 				void *ptrtmp;
 				pcb_gui->get_coords(_("Click on a subcircuit"), &x, &y);
-				if ((pcb_search_screen(x, y, PCB_TYPE_SUBC, &ptrtmp, &ptrtmp, &ptrtmp)) != PCB_TYPE_NONE)
+				if ((pcb_search_screen(x, y, PCB_OBJ_SUBC, &ptrtmp, &ptrtmp, &ptrtmp)) != PCB_OBJ_VOID)
 					s = (pcb_subc_t *)ptrtmp;
 				else {
 					pcb_message(PCB_MSG_ERROR, _("No subcircuit found there\n"));
@@ -239,14 +159,14 @@ static int pcb_act_Attributes(int argc, const char **argv, pcb_coord_t x, pcb_co
 
 static const char pcb_acts_DisperseElements[] = "DisperseElements(All|Selected)";
 
-static const char pcb_acth_DisperseElements[] = "Disperses elements.";
+static const char pcb_acth_DisperseElements[] = "Disperses subcircuits.";
 
 /* %start-doc actions DisperseElements
 
-Normally this is used when starting a board, by selecting all elements
-and then dispersing them.  This scatters the elements around the board
+Normally this is used when starting a board, by selecting all subcircuits
+and then dispersing them.  This scatters the subcircuits around the board
 so that you can pick individual ones, rather than have all the
-elements at the same 0,0 coordinate and thus impossible to choose
+subcircuits at the same 0,0 coordinate and thus impossible to choose
 from.
 
 %end-doc */
@@ -298,7 +218,7 @@ static void disperse_obj(pcb_board_t *pcb, pcb_any_obj_t *obj, pcb_coord_t ox, p
 		*maxy = newy2;
 		if (*maxy > PCB->MaxHeight - GAP) {
 			*maxy = GAP;
-			pcb_message(PCB_MSG_WARNING, "The board is too small for hosting all elements,\ndiesperse restarted from the top.\nExpect overlapping elements\n");
+			pcb_message(PCB_MSG_WARNING, "The board is too small for hosting all subcircuits,\ndiesperse restarted from the top.\nExpect overlapping subcircuits\n");
 		}
 	}
 }
@@ -332,20 +252,6 @@ static int pcb_act_DisperseElements(int argc, const char **argv, pcb_coord_t x,
 	}
 
 	pcb_draw_inhibit_inc();
-	PCB_ELEMENT_LOOP(PCB->Data);
-	{
-		if (!PCB_FLAG_TEST(PCB_FLAG_LOCK, element) && (all || PCB_FLAG_TEST(PCB_FLAG_SELECTED, element))) {
-			disperse_obj(PCB, (pcb_any_obj_t *)element, element->MarkX, element->MarkY, &dx, &dy, &minx, &miny, &maxy);
-			
-			/* move the element */
-			pcb_element_move(PCB->Data, element, dx, dy);
-
-			/* and add to the undo list so we can undo this operation */
-			pcb_undo_add_obj_to_move(PCB_TYPE_ELEMENT, NULL, NULL, element, dx, dy);
-		}
-	}
-	PCB_END_LOOP;
-
 	PCB_SUBC_LOOP(PCB->Data);
 	{
 		if (!PCB_FLAG_TEST(PCB_FLAG_LOCK, subc) && (all || PCB_FLAG_TEST(PCB_FLAG_SELECTED, subc))) {
@@ -355,7 +261,7 @@ static int pcb_act_DisperseElements(int argc, const char **argv, pcb_coord_t x,
 				oy = (subc->BoundingBox.Y1 + subc->BoundingBox.Y2)/2;
 			}
 			disperse_obj(PCB, (pcb_any_obj_t *)subc, ox, oy, &dx, &dy, &minx, &miny, &maxy);
-			pcb_move_obj(PCB_TYPE_SUBC, subc, subc, subc, dx, dy);
+			pcb_move_obj(PCB_OBJ_SUBC, subc, subc, subc, dx, dy);
 		}
 	}
 	PCB_END_LOOP;
@@ -376,14 +282,14 @@ static int pcb_act_DisperseElements(int argc, const char **argv, pcb_coord_t x,
 
 static const char pcb_acts_Flip[] = "Flip(Object|Selected|SelectedElements)";
 
-static const char pcb_acth_Flip[] = "Flip an element to the opposite side of the board.";
+static const char pcb_acth_Flip[] = "Flip a subcircuit to the opposite side of the board.";
 
 /* %start-doc actions Flip
 
-Note that the location of the element will be symmetric about the
+Note that the location of the subcircuit will be symmetric about the
 cursor location; i.e. if the part you are pointing at will still be at
-the same spot once the element is on the other side.  When flipping
-multiple elements, this retains their positions relative to each
+the same spot once the subcircuit is on the other side.  When flipping
+multiple subcircuits, this retains their positions relative to each
 other, not their absolute positions on the board.
 
 %end-doc */
@@ -391,21 +297,13 @@ other, not their absolute positions on the board.
 static int pcb_act_Flip(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
 {
 	const char *function = PCB_ACTION_ARG(0);
-	pcb_element_t *element;
 	void *ptrtmp;
 	int err = 0;
 
 	if (function) {
 		switch (pcb_funchash_get(function, NULL)) {
 		case F_Object:
-			if ((pcb_search_screen(x, y, PCB_TYPE_ELEMENT, &ptrtmp, &ptrtmp, &ptrtmp)) != PCB_TYPE_NONE) {
-				element = (pcb_element_t *) ptrtmp;
-				pcb_undo_save_serial();
-				pcb_element_change_side(element, 2 * pcb_crosshair.Y - PCB->MaxHeight);
-				pcb_undo_inc_serial();
-				pcb_draw();
-			}
-			if ((pcb_search_screen(x, y, PCB_TYPE_SUBC, &ptrtmp, &ptrtmp, &ptrtmp)) != PCB_TYPE_NONE) {
+			if ((pcb_search_screen(x, y, PCB_OBJ_SUBC, &ptrtmp, &ptrtmp, &ptrtmp)) != PCB_OBJ_VOID) {
 				pcb_subc_t *subc = (pcb_subc_t *)ptrtmp;
 				pcb_undo_save_serial();
 				pcb_subc_change_side(&subc, 2 * pcb_crosshair.Y - PCB->MaxHeight);
@@ -417,7 +315,6 @@ static int pcb_act_Flip(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 		case F_SelectedElements:
 			pcb_undo_save_serial();
 			pcb_selected_subc_change_side();
-			pcb_selected_element_change_side();
 			pcb_undo_inc_serial();
 			pcb_draw();
 			break;
@@ -461,7 +358,7 @@ static int pcb_act_MoveObject(int argc, const char **argv, pcb_coord_t x, pcb_co
 	nx = pcb_get_value(x_str, units, &absolute2, NULL);
 
 	type = pcb_search_screen(x, y, PCB_MOVE_TYPES, &ptr1, &ptr2, &ptr3);
-	if (type == PCB_TYPE_NONE) {
+	if (type == PCB_OBJ_VOID) {
 		pcb_message(PCB_MSG_ERROR, _("Nothing found under crosshair\n"));
 		return 1;
 	}
@@ -472,7 +369,7 @@ static int pcb_act_MoveObject(int argc, const char **argv, pcb_coord_t x, pcb_co
 	pcb_event(PCB_EVENT_RUBBER_RESET, NULL);
 	if (conf_core.editor.rubber_band_mode)
 		pcb_event(PCB_EVENT_RUBBER_LOOKUP_LINES, "ippp", type, ptr1, ptr2, ptr3);
-	if (type == PCB_TYPE_ELEMENT)
+	if (type == PCB_OBJ_SUBC)
 		pcb_event(PCB_EVENT_RUBBER_LOOKUP_RATS, "ippp", type, ptr1, ptr2, ptr3);
 	pcb_move_obj_and_rubberband(type, ptr1, ptr2, ptr3, nx, ny);
 	pcb_board_set_changed_flag(pcb_true);
@@ -487,7 +384,7 @@ static const char pcb_acth_MoveToCurrentLayer[] = "Moves objects to the current
 
 /* %start-doc actions MoveToCurrentLayer
 
-Note that moving an element from a component layer to a solder layer,
+Note that moving an subcircuit from a component layer to a solder layer,
 or from solder to component, won't automatically flip it.  Use the
 @code{Flip()} action to do that.
 
@@ -504,9 +401,28 @@ static int pcb_act_MoveToCurrentLayer(int argc, const char **argv, pcb_coord_t x
 				void *ptr1, *ptr2, *ptr3;
 
 				pcb_gui->get_coords(_("Select an Object"), &x, &y);
-				if ((type = pcb_search_screen(x, y, PCB_MOVETOLAYER_TYPES | PCB_LOOSE_SUBC, &ptr1, &ptr2, &ptr3)) != PCB_TYPE_NONE)
-					if (pcb_move_obj_to_layer(type, ptr1, ptr2, ptr3, CURRENT, pcb_false))
+				if ((type = pcb_search_screen(x, y, PCB_MOVETOLAYER_TYPES | PCB_LOOSE_SUBC, &ptr1, &ptr2, &ptr3)) != PCB_OBJ_VOID) {
+					pcb_layer_t *target = CURRENT;
+					pcb_any_obj_t *o = ptr2;
+
+					/* if object is part of a subc (must be a floater!), target layer
+					   shall be within the subc too else we would move out the object from
+					   under the subc to under the board as an unwanted side effect */
+					if ((o->parent_type == PCB_PARENT_LAYER) && (o->parent.layer->parent.data->parent_type == PCB_PARENT_SUBC)) {
+						pcb_subc_t *subc= o->parent.layer->parent.data->parent.subc;
+						pcb_layer_type_t lyt = pcb_layer_flags_(CURRENT);
+						int old_len = subc->data->LayerN;
+						target = pcb_subc_get_layer(subc, lyt, CURRENT->comb, 1, CURRENT->name, 0);
+						if (target == NULL) {
+							pcb_message(PCB_MSG_ERROR, "Failed to find or allocate the matching subc layer\n");
+							break;
+						}
+						if (old_len != subc->data->LayerN)
+							pcb_subc_rebind(PCB, subc); /* had to alloc a new layer */
+					}
+					if (pcb_move_obj_to_layer(type, ptr1, ptr2, ptr3, target, pcb_false))
 						pcb_board_set_changed_flag(pcb_true);
+				}
 				break;
 			}
 
@@ -530,20 +446,20 @@ static const char pcb_acth_ElementList[] = "Adds the given element if it doesn't
 @table @code
 
 @item Start
-Indicates the start of an element list; call this before any Need
+Indicates the start of an subcircuit list; call this before any Need
 actions.
 
 @item Need
-Searches the board for an element with a matching refdes.
+Searches the board for an subcircuit with a matching refdes.
 
 If found, the value and footprint are updated.
 
-If not found, a new element is created with the given footprint and value.
+If not found, a new subcircuit is created with the given footprint and value.
 
 @item Done
-Compares the list of elements needed since the most recent
-@code{start} with the list of elements actually on the board.  Any
-elements that weren't listed are selected, so that the user may delete
+Compares the list of subcircuits needed since the most recent
+@code{start} with the list of subcircuits actually on the board.  Any
+subcircuits that weren't listed are selected, so that the user may delete
 them.
 
 @end table
@@ -562,7 +478,7 @@ static int parse_layout_attribute_units(const char *name, int def)
 
 static int subc_differs(pcb_subc_t *sc, const char *expect_name)
 {
-	const char *got_name = pcb_attribute_get(&sc->Attributes, "name");
+	const char *got_name = pcb_attribute_get(&sc->Attributes, "footprint");
 	if ((expect_name != NULL) && (*expect_name == '\0'))
 		expect_name = NULL;
 	if ((got_name != NULL) && (*got_name == '\0'))
@@ -576,37 +492,35 @@ static int subc_differs(pcb_subc_t *sc, const char *expect_name)
 
 static int pcb_act_ElementList(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
 {
-	pcb_element_t *e = NULL;
 	pcb_subc_t *sc;
 	const char *refdes, *value, *footprint;
 	const char *args[3];
 	const char *function = argv[0];
-	char *old;
+	int fx, fy, fs;
 
 #ifdef DEBUG
 	printf("Entered pcb_act_ElementList, executing function %s\n", function);
 #endif
 
 	if (pcb_strcasecmp(function, "start") == 0) {
-		PCB_ELEMENT_LOOP(PCB->Data);
+		PCB_SUBC_LOOP(PCB->Data);
 		{
-			PCB_FLAG_CLEAR(PCB_FLAG_FOUND, element);
+			PCB_FLAG_CLEAR(PCB_FLAG_FOUND, subc);
 		}
 		PCB_END_LOOP;
-		element_cache = NULL;
 		number_of_footprints_not_found = 0;
 		return 0;
 	}
 
 	if (pcb_strcasecmp(function, "done") == 0) {
-		PCB_ELEMENT_LOOP(PCB->Data);
+		PCB_SUBC_LOOP(PCB->Data);
 		{
-			if (PCB_FLAG_TEST(PCB_FLAG_FOUND, element)) {
-				PCB_FLAG_CLEAR(PCB_FLAG_FOUND, element);
+			if (PCB_FLAG_TEST(PCB_FLAG_FOUND, subc)) {
+				PCB_FLAG_CLEAR(PCB_FLAG_FOUND, subc);
 			}
-			else if (!PCB_EMPTY_STRING_P(PCB_ELEM_NAME_REFDES(element))) {
+			else if (!PCB_EMPTY_STRING_P(subc->refdes)) {
 				/* Unnamed elements should remain untouched */
-				PCB_FLAG_SET(PCB_FLAG_SELECTED, element);
+				PCB_FLAG_SET(PCB_FLAG_SELECTED, subc);
 			}
 		}
 		PCB_END_LOOP;
@@ -632,16 +546,23 @@ static int pcb_act_ElementList(int argc, const char **argv, pcb_coord_t x, pcb_c
 	args[1] = refdes;
 	args[2] = value;
 
+	/* turn of flip to avoid mirror/rotat confusion */
+	fx = conf_core.editor.view.flip_x;
+	fy = conf_core.editor.view.flip_y;
+	fs = conf_core.editor.show_solder_side;
+	conf_force_set_bool(conf_core.editor.view.flip_x, 0);
+	conf_force_set_bool(conf_core.editor.view.flip_y, 0);
+	conf_force_set_bool(conf_core.editor.show_solder_side, 0);
+
 #ifdef DEBUG
 	printf("  ... footprint = %s\n", footprint);
 	printf("  ... refdes = %s\n", refdes);
 	printf("  ... value = %s\n", value);
 #endif
 
-	e = find_element_by_refdes(refdes);
 	sc = pcb_subc_by_refdes(PCB->Data, refdes);
 
-	if ((e == NULL) && (sc == NULL)) {
+	if (sc == NULL) {
 		pcb_coord_t nx, ny, d;
 
 #ifdef DEBUG
@@ -680,17 +601,12 @@ static int pcb_act_ElementList(int argc, const char **argv, pcb_coord_t x, pcb_c
 		if (pcb_buffer_copy_to_layout(PCB, nx, ny))
 			pcb_board_set_changed_flag(pcb_true);
 	}
-	else if (
-			(e && PCB_ELEM_NAME_DESCRIPTION(e) && strcmp(PCB_ELEM_NAME_DESCRIPTION(e), footprint) != 0) 
-			|| (sc && subc_differs(sc, footprint))
-		)
-	{
+	else if (sc && subc_differs(sc, footprint)) {
 #ifdef DEBUG
 		printf("  ... Footprint on board, but different from footprint loaded.\n");
 #endif
-		int orig_on_top, orig_rotstep, paste_ok = 0;
+		int orig_on_top, paste_ok = 0;
 		pcb_coord_t orig_cx, orig_cy;
-		pcb_element_t *pe;
 		double orig_rot;
 
 		/* Different footprint, we need to swap them out.  */
@@ -699,49 +615,18 @@ static int pcb_act_ElementList(int argc, const char **argv, pcb_coord_t x, pcb_c
 			return 1;
 		}
 
-		/* save the rotation value of the original element/subc */
-		if (e != NULL) {
-			orig_rotstep = pcb_element_get_orientation(e);
-			orig_rot = orig_rotstep * 90.0;
-			orig_cx = e->MarkX;
-			orig_cy = e->MarkY;
-			orig_on_top = PCB_FRONT(e);
-		}
-		else {
+		{
 			orig_rot = 0.0;
 			orig_cx = 0;
 			orig_cy = 0;
 			orig_on_top = 0;
 			pcb_subc_get_rotation(sc, &orig_rot);
-			orig_rotstep = pcb_round(orig_rot / 90.0);
 			pcb_subc_get_origin(sc, &orig_cx, &orig_cy);
 			pcb_subc_get_side(sc, &orig_on_top);
 			orig_on_top = !orig_on_top;
 		}
 
-		pe = elementlist_first(&(PCB_PASTEBUFFER->Data->Element));
-		if (pe != NULL) {
-			/* replace with element */
-			int pr, i;
-
-			if (!orig_on_top)
-				pcb_element_mirror(PCB_PASTEBUFFER->Data, pe, pe->MarkY * 2 - PCB->MaxHeight);
-
-			pr = pcb_element_get_orientation(pe);
-
-			if (orig_rotstep != pr)
-				pcb_element_rotate90(PCB_PASTEBUFFER->Data, pe, pe->MarkX, pe->MarkY, (orig_rotstep - pr + 4) % 4);
-
-			for (i = 0; i < PCB_MAX_ELEMENTNAMES; i++) {
-				pe->Name[i].X = e->Name[i].X - orig_cx + pe->MarkX;
-				pe->Name[i].Y = e->Name[i].Y - orig_cy + pe->MarkY;
-				pe->Name[i].Direction = e->Name[i].Direction;
-				pe->Name[i].Scale = e->Name[i].Scale;
-			}
-
-			paste_ok = 1;
-		}
-		else {
+		{
 			/* replace with subc */
 			pcb_subc_t *psc;
 
@@ -772,8 +657,6 @@ static int pcb_act_ElementList(int argc, const char **argv, pcb_coord_t x, pcb_c
 		}
 
 		if (paste_ok) {
-			if (e != NULL)
-				pcb_element_remove(e);
 			if (sc != NULL)
 				pcb_subc_remove(sc);
 			if (pcb_buffer_copy_to_layout(PCB, orig_cx, orig_cy))
@@ -782,18 +665,6 @@ static int pcb_act_ElementList(int argc, const char **argv, pcb_coord_t x, pcb_c
 	}
 
 	/* Now reload footprint */
-	element_cache = NULL;
-	e = find_element_by_refdes(refdes);
-	if (e != NULL) {
-		old = pcb_element_text_change(PCB, PCB->Data, e, PCB_ELEMNAME_IDX_REFDES, pcb_strdup(refdes));
-		if (old)
-			free(old);
-		old = pcb_element_text_change(PCB, PCB->Data, e, PCB_ELEMNAME_IDX_VALUE, pcb_strdup(value));
-		if (old)
-			free(old);
-		PCB_FLAG_SET(PCB_FLAG_FOUND, e);
-	}
-
 	sc = pcb_subc_by_refdes(PCB->Data, refdes);
 	if (sc != NULL) {
 /*		pcb_attribute_put(&sc->Attributes, "refdes", refdes);*/
@@ -801,16 +672,18 @@ static int pcb_act_ElementList(int argc, const char **argv, pcb_coord_t x, pcb_c
 		PCB_FLAG_SET(PCB_FLAG_FOUND, sc);
 	}
 
-
 #ifdef DEBUG
 	printf(" ... Leaving pcb_act_ElementList.\n");
 #endif
 
+	conf_force_set_bool(conf_core.editor.view.flip_x, fx);
+	conf_force_set_bool(conf_core.editor.view.flip_y, fy);
+	conf_force_set_bool(conf_core.editor.show_solder_side, fs);
+
 	return 0;
 }
 
 /* ---------------------------------------------------------------- */
-#warning subc TODO: remove this when removing element support
 static const char pcb_acts_ElementSetAttr[] = "ElementSetAttr(refdes,name[,value])";
 
 static const char pcb_acth_ElementSetAttr[] = "Sets or clears an element-specific attribute.";
@@ -825,10 +698,8 @@ not specified, the given attribute is removed if present.
 
 static int pcb_act_ElementSetAttr(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
 {
-	pcb_element_t *e = NULL;
 	pcb_subc_t *sc;
 	const char *refdes, *name, *value;
-	pcb_attribute_t *attr;
 
 	if (argc < 2) {
 		PCB_ACT_FAIL(ElementSetAttr);
@@ -840,47 +711,15 @@ static int pcb_act_ElementSetAttr(int argc, const char **argv, pcb_coord_t x, pc
 
 
 	sc = pcb_subc_by_refdes(PCB->Data, refdes);
-	if (sc != NULL) {
-		attr = lookup_attr(&sc->Attributes, name);
-
-		if (attr && value) {
-			free(attr->value);
-			attr->value = pcb_strdup(value);
-		}
-		if (attr && !value)
-			delete_attr(&sc->Attributes, attr);
-		if (!attr && value)
-			pcb_attribute_put(&sc->Attributes, name, value);
+	if (sc == NULL) {
+		pcb_message(PCB_MSG_ERROR, "Can't find subcircuit with refdes '%s'\n", refdes);
 		return 0;
 	}
 
-	PCB_ELEMENT_LOOP(PCB->Data);
-	{
-		if (PCB_NSTRCMP(refdes, PCB_ELEM_NAME_REFDES(element)) == 0) {
-			e = element;
-			break;
-		}
-	}
-	PCB_END_LOOP;
-
-	if (!e) {
-		pcb_message(PCB_MSG_ERROR, _("Cannot change attribute of %s - element not found\n"), refdes);
-		return 1;
-	}
-
-	attr = lookup_attr(&e->Attributes, name);
-
-	if (attr && value) {
-		free(attr->value);
-		attr->value = pcb_strdup(value);
-	}
-	if (attr && !value) {
-		delete_attr(&e->Attributes, attr);
-	}
-	if (!attr && value) {
-		pcb_attribute_put(&e->Attributes, name, value);
-	}
-
+	if (value != NULL)
+		pcb_attribute_put(&sc->Attributes, name, value);
+	else
+		pcb_attribute_remove(&sc->Attributes, name);
 	return 0;
 }
 
@@ -888,7 +727,7 @@ static int pcb_act_ElementSetAttr(int argc, const char **argv, pcb_coord_t x, pc
 
 static const char pcb_acts_RipUp[] = "RipUp(All|Selected|Element)";
 
-static const char pcb_acth_RipUp[] = "Ripup auto-routed tracks, or convert an element to parts.";
+static const char pcb_acth_RipUp[] = "Ripup auto-routed tracks";
 
 /* %start-doc actions RipUp
 
@@ -901,10 +740,6 @@ Removes all lines and vias which were created by the autorouter.
 Removes all selected lines and vias which were created by the
 autorouter.
 
-@item Element
-Converts the element under the cursor to parts (vias and lines).  Note
-that this uses the highest numbered paste buffer.
-
 @end table
 
 %end-doc */
@@ -920,7 +755,7 @@ static int pcb_act_RipUp(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 			PCB_LINE_ALL_LOOP(PCB->Data);
 			{
 				if (PCB_FLAG_TEST(PCB_FLAG_AUTO, line) && !PCB_FLAG_TEST(PCB_FLAG_LOCK, line)) {
-					pcb_remove_object(PCB_TYPE_LINE, layer, line, line);
+					pcb_remove_object(PCB_OBJ_LINE, layer, line, line);
 					changed = pcb_true;
 				}
 			}
@@ -928,15 +763,16 @@ static int pcb_act_RipUp(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 			PCB_ARC_ALL_LOOP(PCB->Data);
 			{
 				if (PCB_FLAG_TEST(PCB_FLAG_AUTO, arc) && !PCB_FLAG_TEST(PCB_FLAG_LOCK, arc)) {
-					pcb_remove_object(PCB_TYPE_ARC, layer, arc, arc);
+					pcb_remove_object(PCB_OBJ_ARC, layer, arc, arc);
 					changed = pcb_true;
 				}
 			}
 			PCB_ENDALL_LOOP;
-			PCB_VIA_LOOP(PCB->Data);
+
+			PCB_PADSTACK_LOOP(PCB->Data);
 			{
-				if (PCB_FLAG_TEST(PCB_FLAG_AUTO, via) && !PCB_FLAG_TEST(PCB_FLAG_LOCK, via)) {
-					pcb_remove_object(PCB_TYPE_VIA, via, via, via);
+				if (PCB_FLAG_TEST(PCB_FLAG_AUTO, padstack) && !PCB_FLAG_TEST(PCB_FLAG_LOCK, padstack)) {
+					pcb_remove_object(PCB_OBJ_PSTK, padstack, padstack, padstack);
 					changed = pcb_true;
 				}
 			}
@@ -952,17 +788,17 @@ static int pcb_act_RipUp(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 			{
 				if (PCB_FLAGS_TEST(PCB_FLAG_AUTO | PCB_FLAG_SELECTED, line)
 						&& !PCB_FLAG_TEST(PCB_FLAG_LOCK, line)) {
-					pcb_remove_object(PCB_TYPE_LINE, layer, line, line);
+					pcb_remove_object(PCB_OBJ_LINE, layer, line, line);
 					changed = pcb_true;
 				}
 			}
 			PCB_ENDALL_LOOP;
-			if (PCB->ViaOn)
-				PCB_VIA_LOOP(PCB->Data);
+			if (PCB->pstk_on)
+			PCB_PADSTACK_LOOP(PCB->Data);
 			{
-				if (PCB_FLAGS_TEST(PCB_FLAG_AUTO | PCB_FLAG_SELECTED, via)
-						&& !PCB_FLAG_TEST(PCB_FLAG_LOCK, via)) {
-					pcb_remove_object(PCB_TYPE_VIA, via, via, via);
+				if (PCB_FLAGS_TEST(PCB_FLAG_AUTO | PCB_FLAG_SELECTED, padstack)
+						&& !PCB_FLAG_TEST(PCB_FLAG_LOCK, padstack)) {
+					pcb_remove_object(PCB_OBJ_PSTK, padstack, padstack, padstack);
 					changed = pcb_true;
 				}
 			}
@@ -972,29 +808,6 @@ static int pcb_act_RipUp(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 				pcb_board_set_changed_flag(pcb_true);
 			}
 			break;
-		case F_Element:
-			{
-				void *ptr1, *ptr2, *ptr3;
-				pcb_coord_t gx, gy;
-				pcb_gui->get_coords("Click on an element", &gx, &gy);
-				if (pcb_search_screen(gx, gy, PCB_TYPE_ELEMENT, &ptr1, &ptr2, &ptr3) != PCB_TYPE_NONE) {
-					pcb_tool_note.Buffer = conf_core.editor.buffer_number;
-					pcb_buffer_set_number(PCB_MAX_BUFFER - 1);
-					pcb_buffer_clear(PCB, PCB_PASTEBUFFER);
-					pcb_copy_obj_to_buffer(PCB, PCB_PASTEBUFFER->Data, PCB->Data, PCB_TYPE_ELEMENT, ptr1, ptr2, ptr3);
-					pcb_element_smash_buffer(PCB_PASTEBUFFER);
-					PCB_PASTEBUFFER->X = 0;
-					PCB_PASTEBUFFER->Y = 0;
-					pcb_undo_save_serial();
-					pcb_erase_obj(PCB_TYPE_ELEMENT, ptr1, ptr1);
-					pcb_undo_move_obj_to_remove(PCB_TYPE_ELEMENT, ptr1, ptr2, ptr3);
-					pcb_undo_restore_serial();
-					pcb_buffer_copy_to_layout(PCB, 0, 0);
-					pcb_buffer_set_number(pcb_tool_note.Buffer);
-					pcb_board_set_changed_flag(pcb_true);
-				}
-			}
-			break;
 		}
 	}
 	return 0;
@@ -1003,76 +816,11 @@ static int pcb_act_RipUp(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 /* ---------------------------------------------------------------------------  */
 
 static const char pcb_acts_MinMaskGap[] = "MinMaskGap(delta)\n" "MinMaskGap(Selected, delta)";
-
-static const char pcb_acth_MinMaskGap[] = "Ensures the mask is a minimum distance from pins and pads.";
-
-/* %start-doc actions MinMaskGap
-
-Checks all specified pins and/or pads, and increases the mask if
-needed to ensure a minimum distance between the pin or pad edge and
-the mask edge.
-
-%end-doc */
-
+static const char pcb_acth_MinMaskGap[] = "Ensures the mask is a minimum distance from pins and pads. Not supported anymore.";
 static int pcb_act_MinMaskGap(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
 {
-	const char *function = PCB_ACTION_ARG(0);
-	const char *delta = PCB_ACTION_ARG(1);
-	const char *units = PCB_ACTION_ARG(2);
-	pcb_bool absolute;
-	pcb_coord_t value;
-	int flags;
-
-	if (!function)
-		return 1;
-	if (pcb_strcasecmp(function, "Selected") == 0)
-		flags = PCB_FLAG_SELECTED;
-	else {
-		units = delta;
-		delta = function;
-		flags = 0;
-	}
-	value = 2 * pcb_get_value(delta, units, &absolute, NULL);
-
-	pcb_undo_save_serial();
-	PCB_ELEMENT_LOOP(PCB->Data);
-	{
-		PCB_PIN_LOOP(element);
-		{
-			if (!PCB_FLAGS_TEST(flags, pin))
-				continue;
-			if (pin->Mask < pin->Thickness + value) {
-				pcb_chg_obj_mask_size(PCB_TYPE_PIN, element, pin, 0, pin->Thickness + value, 1);
-				pcb_undo_restore_serial();
-			}
-		}
-		PCB_END_LOOP;
-		PCB_PAD_LOOP(element);
-		{
-			if (!PCB_FLAGS_TEST(flags, pad))
-				continue;
-			if (pad->Mask < pad->Thickness + value) {
-				pcb_chg_obj_mask_size(PCB_TYPE_PAD, element, pad, 0, pad->Thickness + value, 1);
-				pcb_undo_restore_serial();
-			}
-		}
-		PCB_END_LOOP;
-	}
-	PCB_END_LOOP;
-#warning padstack TODO
-	PCB_VIA_LOOP(PCB->Data);
-	{
-		if (!PCB_FLAGS_TEST(flags, via))
-			continue;
-		if (via->Mask && via->Mask < via->Thickness + value) {
-			pcb_chg_obj_mask_size(PCB_TYPE_VIA, via, 0, 0, via->Thickness + value, 1);
-			pcb_undo_restore_serial();
-		}
-	}
-	PCB_END_LOOP;
-	pcb_undo_restore_serial();
-	pcb_undo_inc_serial();
-	return 0;
+	pcb_message(PCB_MSG_ERROR, "Feature not supported; use padstackedit()\n");
+	return 1;
 }
 
 /* ---------------------------------------------------------------------------  */
@@ -1089,6 +837,59 @@ polygon edges.
 
 %end-doc */
 
+static void minclr(pcb_data_t *data, pcb_coord_t value, int flags)
+{
+	PCB_SUBC_LOOP(data);
+	{
+		if (!PCB_FLAGS_TEST(flags, subc))
+			continue;
+		minclr(subc->data, value, 0);
+	}
+	PCB_END_LOOP;
+
+	PCB_PADSTACK_LOOP(data);
+	{
+		if (!PCB_FLAGS_TEST(flags, padstack))
+			continue;
+		if (padstack->Clearance < value) {
+			pcb_chg_obj_clear_size(PCB_OBJ_PSTK, padstack, 0, 0, value, 1);
+			pcb_undo_restore_serial();
+		}
+	}
+	PCB_END_LOOP;
+
+	PCB_LINE_ALL_LOOP(data);
+	{
+		if (!PCB_FLAGS_TEST(flags, line))
+			continue;
+		if ((line->Clearance != 0) && (line->Clearance < value)) {
+			pcb_chg_obj_clear_size(PCB_OBJ_LINE, layer, line, 0, value, 1);
+			pcb_undo_restore_serial();
+		}
+	}
+	PCB_ENDALL_LOOP;
+	PCB_ARC_ALL_LOOP(data);
+	{
+		if (!PCB_FLAGS_TEST(flags, arc))
+			continue;
+		if ((arc->Clearance != 0) && (arc->Clearance < value)) {
+			pcb_chg_obj_clear_size(PCB_OBJ_ARC, layer, arc, 0, value, 1);
+			pcb_undo_restore_serial();
+		}
+	}
+	PCB_ENDALL_LOOP;
+	PCB_POLY_ALL_LOOP(data);
+	{
+		if (!PCB_FLAGS_TEST(flags, polygon))
+			continue;
+		if ((polygon->Clearance != 0) && (polygon->Clearance < value)) {
+			pcb_chg_obj_clear_size(PCB_OBJ_POLY, layer, polygon, 0, value, 1);
+			pcb_undo_restore_serial();
+		}
+	}
+	PCB_ENDALL_LOOP;
+}
+
 static int pcb_act_MinClearGap(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
 {
 	const char *function = PCB_ACTION_ARG(0);
@@ -1110,80 +911,12 @@ static int pcb_act_MinClearGap(int argc, const char **argv, pcb_coord_t x, pcb_c
 	value = 2 * pcb_get_value(delta, units, &absolute, NULL);
 
 	pcb_undo_save_serial();
-	PCB_ELEMENT_LOOP(PCB->Data);
-	{
-		PCB_PIN_LOOP(element);
-		{
-			if (!PCB_FLAGS_TEST(flags, pin))
-				continue;
-			if (pin->Clearance < value) {
-				pcb_chg_obj_clear_size(PCB_TYPE_PIN, element, pin, 0, value, 1);
-				pcb_undo_restore_serial();
-			}
-		}
-		PCB_END_LOOP;
-		PCB_PAD_LOOP(element);
-		{
-			if (!PCB_FLAGS_TEST(flags, pad))
-				continue;
-			if (pad->Clearance < value) {
-				pcb_chg_obj_clear_size(PCB_TYPE_PAD, element, pad, 0, value, 1);
-				pcb_undo_restore_serial();
-			}
-		}
-		PCB_END_LOOP;
-	}
-	PCB_END_LOOP;
-#warning padstack TODO
-	PCB_VIA_LOOP(PCB->Data);
-	{
-		if (!PCB_FLAGS_TEST(flags, via))
-			continue;
-		if (via->Clearance < value) {
-			pcb_chg_obj_clear_size(PCB_TYPE_VIA, via, 0, 0, value, 1);
-			pcb_undo_restore_serial();
-		}
-	}
-	PCB_END_LOOP;
-	PCB_LINE_ALL_LOOP(PCB->Data);
-	{
-		if (!PCB_FLAGS_TEST(flags, line))
-			continue;
-		if ((line->Clearance != 0) && (line->Clearance < value)) {
-			pcb_chg_obj_clear_size(PCB_TYPE_LINE, layer, line, 0, value, 1);
-			pcb_undo_restore_serial();
-		}
-	}
-	PCB_ENDALL_LOOP;
-	PCB_ARC_ALL_LOOP(PCB->Data);
-	{
-		if (!PCB_FLAGS_TEST(flags, arc))
-			continue;
-		if ((arc->Clearance != 0) && (arc->Clearance < value)) {
-			pcb_chg_obj_clear_size(PCB_TYPE_ARC, layer, arc, 0, value, 1);
-			pcb_undo_restore_serial();
-		}
-	}
-	PCB_ENDALL_LOOP;
+	minclr(PCB->Data, value, flags);
 	pcb_undo_restore_serial();
 	pcb_undo_inc_serial();
 	return 0;
 }
 
-/* ---------------------------------------------------------------------------  */
-int pcb_act_ListRotations(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
-{
-	PCB_ELEMENT_LOOP(PCB->Data);
-	{
-		printf("%d %s\n", pcb_element_get_orientation(element), PCB_ELEM_NAME_REFDES(element));
-	}
-	PCB_END_LOOP;
-#warning subc TODO
-#warning cleanup TODO: move this to oldactions maybe?
-	return 0;
-}
-
-
 /* --------------------------------------------------------------------------- */
 
 static const char movelayer_syntax[] = "MoveLayer(old,new)";
@@ -1360,13 +1093,33 @@ static int pcb_act_CreateText(int argc, const char **argv, pcb_coord_t x, pcb_co
 	return 0;
 }
 
-static const char pcb_acts_subc[] = "subc(hash, [board|selected])\n";
+static const char pcb_acts_subc[] =
+	"subc(hash, [board|selected])\n"
+	"subc(loose, on|off|toggle|check)\n"
+	;
 static const char pcb_acth_subc[] = "Various operations on subc";
 static int pcb_act_subc(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
 {
 	if (argc == 0)
 		PCB_ACT_FAIL(subc);
 	switch (pcb_funchash_get(argv[0], NULL)) {
+		case F_Loose:
+			if ((argc < 2) || (strcmp(argv[1], "toggle") == 0))
+				PCB->loose_subc = !PCB->loose_subc;
+			else if (strcmp(argv[1], "on") == 0)
+				PCB->loose_subc = 1;
+			else if (strcmp(argv[1], "off") == 0)
+				PCB->loose_subc = 0;
+			else if (strcmp(argv[1], "check") == 0)
+				return PCB->loose_subc;
+			else {
+				PCB_ACT_FAIL(subc);
+				return 1;
+			}
+			/* have to manually trigger the update as it is not a conf item */
+			if ((pcb_gui != NULL) && (pcb_gui->update_menu_checkbox != NULL))
+				pcb_gui->update_menu_checkbox(NULL);
+			return 0;
 		case F_Hash:
 			{
 				int selected_only = 0;
@@ -1440,6 +1193,32 @@ static int pcb_act_subc(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 	return 0;
 }
 
+static const char pcb_acts_Rotate90[] = "pcb_move_obj(steps)";
+static const char pcb_acth_Rotate90[] = "Rotates the object under the crosshair by 90 degree steps.";
+
+/* %start-doc actions Rotate90
+
+Rotates the object under the crosshair by 90 degree @code{steps}.
+
+%end-doc */
+
+static int pcb_act_Rotate90(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
+{
+	const char *ssteps = PCB_ACTION_ARG(0);
+	int steps = atoi(ssteps);
+
+	if (conf_core.editor.show_solder_side)
+		steps = -steps;
+
+	steps = steps % 4;
+	if (steps < 0)
+		steps = 4+steps;
+
+	pcb_screen_obj_rotate90(x, y, steps);
+
+	return 0;
+}
+
 pcb_hid_action_t object_action_list[] = {
 	{"Attributes", 0, pcb_act_Attributes,
 	 pcb_acth_Attributes, pcb_acts_Attributes}
@@ -1450,9 +1229,6 @@ pcb_hid_action_t object_action_list[] = {
 	{"Flip", N_("Click on Object or Flip Point"), pcb_act_Flip,
 	 pcb_acth_Flip, pcb_acts_Flip}
 	,
-	{"ListRotations", 0, pcb_act_ListRotations,
-	 0, 0}
-	,
 	{"MoveObject", N_("Select an Object"), pcb_act_MoveObject,
 	 pcb_acth_MoveObject, pcb_acts_MoveObject}
 	,
@@ -1482,6 +1258,9 @@ pcb_hid_action_t object_action_list[] = {
 	,
 	{"CreateText", 0, pcb_act_CreateText,
 	 pcb_acth_CreateText, pcb_acts_CreateText}
+	,
+	{"Rotate90", N_("Select an Object"), pcb_act_Rotate90,
+	 pcb_acth_Rotate90, pcb_acts_Rotate90}
 };
 
 PCB_REGISTER_ACTIONS(object_action_list, NULL)
