@@ -91,6 +91,10 @@ void pcb_fp_init()
 void pcb_fp_uninit()
 {
 	htsp_entry_t *e;
+
+	if (pcb_plug_fp_chain != NULL)
+		pcb_message(PCB_MSG_ERROR, "pcb_plug_fp_chain is not empty; a plugin did not remove itself from the chain. Fix your plugins!\n");
+
 	pcb_fp_free_children(&pcb_library);
 	if (fp_tags != NULL) {
 		for (e = htsp_first(fp_tags); e; e = htsp_next(fp_tags, e))
@@ -110,7 +114,24 @@ const char *pcb_fp_tagname(const void *tagid)
 FILE *pcb_fp_fopen(const char *path, const char *name, pcb_fp_fopen_ctx_t *fctx)
 {
 	FILE *res = NULL;
-	PCB_HOOK_CALL(pcb_plug_fp_t, pcb_plug_fp_chain, fp_fopen, res, != NULL, (self, path, name, fctx));
+	if (strchr(path, ':') != NULL)  {
+		char *tmp, *next, *curr;
+
+		curr = tmp = pcb_strdup(path);
+		while((res == NULL) && (curr != NULL)) {
+			next = strchr(curr, ':');
+			if (next != NULL) {
+				*next= '\0';
+				next++;
+			}
+
+			PCB_HOOK_CALL(pcb_plug_fp_t, pcb_plug_fp_chain, fp_fopen, res, != NULL, (self, curr, name, fctx));
+			curr = next;
+		}
+		free(tmp);
+	}
+	else
+		PCB_HOOK_CALL(pcb_plug_fp_t, pcb_plug_fp_chain, fp_fopen, res, != NULL, (self, path, name, fctx));
 	return res;
 }
 
