@@ -33,8 +33,9 @@
 
 /* file save, load, merge ... routines */
 
-#warning TODO: do not hardwire this, make a function to decide
-#define DEFAULT_FMT "pcb"
+/* If emergency format is not configured but emergency file name is, default
+   to saving in the native format */
+#define DEFAULT_EMERGENCY_FMT "lihata"
 
 #include "config.h"
 #include "conf_core.h"
@@ -67,6 +68,7 @@
 #include "safe_fs.h"
 #include "plug_footprint.h"
 #include "build_run.h"
+#include "macro.h"
 
 /* for opendir */
 #include "compat_inc.h"
@@ -231,12 +233,13 @@ int pcb_parse_pcb(pcb_board_t *Ptr, const char *Filename, const char *fmt, int l
 
 	if (load_settings)
 		pcb_event(PCB_EVENT_LOAD_POST, "si", Filename, res);
+	pcb_event(PCB_EVENT_ROUTE_STYLES_CHANGED, NULL);
 
 	plug_io_err(res, "load pcb", Filename);
 	return res;
 }
 
-int pcb_parse_element(pcb_data_t *Ptr, const char *Filename, const char *fmt)
+int pcb_parse_footprint(pcb_data_t *Ptr, const char *Filename, const char *fmt)
 {
 	int res = -1, len, n;
 	pcb_find_io_t available[PCB_IO_MAX_FORMATS];
@@ -256,9 +259,9 @@ int pcb_parse_element(pcb_data_t *Ptr, const char *Filename, const char *fmt)
 
 	/* try all plugins that said it could handle the file */
 	for(n = 0; n < len; n++) {
-		if ((available[n].plug->parse_element == NULL) || (!accepts[n])) /* can't parse or doesn't want to parse this file */
+		if ((available[n].plug->parse_footprint == NULL) || (!accepts[n])) /* can't parse or doesn't want to parse this file */
 			continue;
-		res = available[n].plug->parse_element(available[n].plug, Ptr, Filename);
+		res = available[n].plug->parse_footprint(available[n].plug, Ptr, Filename);
 		if (res == 0) {
 			if (Ptr->loader == NULL) /* if the loader didn't set this (to some more fine grained, e.g. depending on file format version) */
 				Ptr->loader = available[n].plug;
@@ -384,7 +387,7 @@ int pcb_write_buffer(FILE *f, pcb_buffer_t *buff, const char *fmt, pcb_bool elem
 	return res;
 }
 
-int pcb_write_element_data(FILE *f, pcb_data_t *e, const char *fmt)
+int pcb_write_footprint_data(FILE *f, pcb_data_t *e, const char *fmt)
 {
 	int res, newfmt = 0;
 	pcb_plug_io_t *p = e->loader;
@@ -394,8 +397,8 @@ int pcb_write_element_data(FILE *f, pcb_data_t *e, const char *fmt)
 		newfmt = 1;
 	}
 
-	if ((p != NULL) && (p->write_element != NULL))
-		res = p->write_element(p, f, e);
+	if ((p != NULL) && (p->write_footprint != NULL))
+		res = p->write_footprint(p, f, e);
 
 	if ((res == 0) && (newfmt))
 		e->loader = p;
@@ -730,10 +733,10 @@ int pcb_save_buffer_elements(const char *Filename, const char *fmt)
 {
 	int result;
 
-	if (PCB_SWAP_IDENT)
+	if (conf_core.editor.show_solder_side)
 		pcb_buffers_flip_side(PCB);
 	result = pcb_write_pipe(Filename, pcb_false, fmt, pcb_true);
-	if (PCB_SWAP_IDENT)
+	if (conf_core.editor.show_solder_side)
 		pcb_buffers_flip_side(PCB);
 	return result;
 }
@@ -762,9 +765,15 @@ int pcb_save_pcb(const char *file, const char *fmt)
 int pcb_load_pcb(const char *file, const char *fmt, pcb_bool require_font, int how)
 {
 	int res = real_load_pcb(file, fmt, pcb_false, require_font, how);
-	if (res == 0)
+	if (res == 0) {
 		pcb_file_loaded_set_at("design", "main", file, PCB->is_footprint ? "footprint" : "board");
-
+		if (PCB->is_footprint) {
+			pcb_box_t b;
+			/* a footprint has no board size set, need to invent one */
+			pcb_data_bbox(&b, PCB->Data, 0);
+			pcb_board_resize(b.X2*1.5, b.Y2*1.5);
+		}
+	}
 	return res;
 }
 
@@ -811,7 +820,7 @@ void pcb_save_in_tmp(void)
 
 	/* memory might have been released before this function is called */
 	if (PCB && PCB->Changed && (conf_core.rc.emergency_name != NULL) && (*conf_core.rc.emergency_name != '\0')) {
-		const char *fmt = conf_core.rc.emergency_format == NULL ? DEFAULT_FMT : conf_core.rc.emergency_name;
+		const char *fmt = conf_core.rc.emergency_format == NULL ? DEFAULT_EMERGENCY_FMT : conf_core.rc.emergency_format;
 		sprintf(filename, conf_core.rc.emergency_name, (long int)pcb_getpid());
 		pcb_message(PCB_MSG_INFO, _("Trying to save your layout in '%s'\n"), filename);
 		pcb_write_pcb_file(filename, pcb_true, fmt, pcb_true, pcb_false);
@@ -936,7 +945,7 @@ static int pcb_write_file(FILE *fp, pcb_bool thePcb, const char *old_path, const
 {
 	if (thePcb) {
 		if (PCB->is_footprint)
-			return pcb_write_element_data(fp, PCB->Data, fmt);
+			return pcb_write_footprint_data(fp, PCB->Data, fmt);
 		return pcb_write_pcb(fp, old_path, new_path, fmt, emergency);
 	}
 	return pcb_write_buffer(fp, PCB_PASTEBUFFER, fmt, elem_only);
@@ -1087,6 +1096,10 @@ pcb_cardinal_t pcb_io_incompat_save(pcb_data_t *data, pcb_any_obj_t *obj, const
 
 void pcb_io_uninit(void)
 {
+	if (pcb_plug_io_chain != NULL) {
+		pcb_message(PCB_MSG_ERROR, "pcb_plug_io_chain is not empty; a plugin did not remove itself from the chain. Fix your plugins!\n");
+		pcb_message(PCB_MSG_ERROR, "head: desc='%s'\n", pcb_plug_io_chain->description);
+	}
 	free(last_design_dir);
 	last_design_dir = NULL;
 }
