@@ -49,7 +49,6 @@
 #include "undo.h"
 #include "layer.h"
 #include "compat_nls.h"
-#include "obj_all.h"
 #include "obj_poly_draw.h"
 #include "polygon_selfi.h"
 
@@ -169,7 +168,7 @@ static pcb_polyarea_t *biggest(pcb_polyarea_t * p)
 	n = p;
 	do {
 #if 0
-		if (n->contours->area < PCB->IsleArea) {
+		if (n->contours->area < conf_core.design.poly_isle_area) {
 			n->b->f = n->f;
 			n->f->b = n->b;
 			pcb_poly_contour_del(&n->contours);
@@ -324,31 +323,6 @@ pcb_polyarea_t *pcb_poly_from_rect(pcb_coord_t x1, pcb_coord_t x2, pcb_coord_t y
 	return pcb_poly_from_contour(contour);
 }
 
-/* set up x and y multiplier for an octa poly, depending on square pin style */
-#warning padstack TODO: move this out to lib_compat_helper when removing pinvias
-void pcb_poly_square_pin_factors(int style, double *xm, double *ym)
-{
-	int i;
-	const double factor = 2.0;
-
-	/* reset multipliers */
-	for (i = 0; i < 8; i++) {
-		xm[i] = 1;
-		ym[i] = 1;
-	}
-
-	style--;
-	if (style & 1)
-		xm[0] = xm[1] = xm[6] = xm[7] = factor;
-	if (style & 2)
-		xm[2] = xm[3] = xm[4] = xm[5] = factor;
-	if (style & 4)
-		ym[4] = ym[5] = ym[6] = ym[7] = factor;
-	if (style & 8)
-		ym[0] = ym[1] = ym[2] = ym[3] = factor;
-}
-
-
 pcb_polyarea_t *pcb_poly_from_octagon(pcb_coord_t x, pcb_coord_t y, pcb_coord_t radius, int style)
 {
 	pcb_pline_t *contour = NULL;
@@ -693,77 +667,6 @@ pcb_polyarea_t *pcb_poly_from_line(pcb_line_t * L, pcb_coord_t thick)
 	return np;
 }
 
-/* make a rounded-corner rectangle */
-pcb_polyarea_t *SquarePadPoly(pcb_pad_t * pad, pcb_coord_t clear)
-{
-	pcb_pline_t *contour = NULL;
-	pcb_polyarea_t *np = NULL;
-	pcb_vector_t v;
-	double d;
-	double tx, ty;
-	double cx, cy;
-	pcb_pad_t _t = *pad, *t = &_t;
-	pcb_pad_t _c = *pad, *c = &_c;
-	int halfthick = (pad->Thickness + 1) / 2;
-	int halfclear = (clear + 1) / 2;
-
-	d = sqrt(PCB_SQUARE(pad->Point1.X - pad->Point2.X) + PCB_SQUARE(pad->Point1.Y - pad->Point2.Y));
-	if (d != 0) {
-		double a = halfthick / d;
-		tx = (t->Point1.Y - t->Point2.Y) * a;
-		ty = (t->Point2.X - t->Point1.X) * a;
-		a = halfclear / d;
-		cx = (c->Point1.Y - c->Point2.Y) * a;
-		cy = (c->Point2.X - c->Point1.X) * a;
-
-		t->Point1.X -= ty;
-		t->Point1.Y += tx;
-		t->Point2.X += ty;
-		t->Point2.Y -= tx;
-		c->Point1.X -= cy;
-		c->Point1.Y += cx;
-		c->Point2.X += cy;
-		c->Point2.Y -= cx;
-	}
-	else {
-		tx = halfthick;
-		ty = 0;
-		cx = halfclear;
-		cy = 0;
-
-		t->Point1.Y += tx;
-		t->Point2.Y -= tx;
-		c->Point1.Y += cx;
-		c->Point2.Y -= cx;
-	}
-
-	v[0] = c->Point1.X - tx;
-	v[1] = c->Point1.Y - ty;
-	if ((contour = pcb_poly_contour_new(v)) == NULL)
-		return 0;
-	pcb_poly_frac_circle_end(contour, (t->Point1.X - tx), (t->Point1.Y - ty), v, 4);
-
-	v[0] = t->Point2.X - cx;
-	v[1] = t->Point2.Y - cy;
-	pcb_poly_vertex_include(contour->head.prev, pcb_poly_node_create(v));
-	pcb_poly_frac_circle_end(contour, (t->Point2.X - tx), (t->Point2.Y - ty), v, 4);
-
-	v[0] = c->Point2.X + tx;
-	v[1] = c->Point2.Y + ty;
-	pcb_poly_vertex_include(contour->head.prev, pcb_poly_node_create(v));
-	pcb_poly_frac_circle_end(contour, (t->Point2.X + tx), (t->Point2.Y + ty), v, 4);
-
-	v[0] = t->Point1.X + cx;
-	v[1] = t->Point1.Y + cy;
-	pcb_poly_vertex_include(contour->head.prev, pcb_poly_node_create(v));
-	pcb_poly_frac_circle_end(contour, (t->Point1.X + tx), (t->Point1.Y + ty), v, 4);
-
-	/* now we have the line contour */
-	if (!(np = pcb_poly_from_contour(contour)))
-		return NULL;
-	return np;
-}
-
 /* clear np1 from the polygon - should be inline with -O3 */
 static int Subtract(pcb_polyarea_t * np1, pcb_poly_t * p, pcb_bool fnp)
 {
@@ -807,72 +710,11 @@ int pcb_poly_subtract(pcb_polyarea_t *np1, pcb_poly_t *p, pcb_bool fnp)
 	return Subtract(np1, p, fnp);
 }
 
-/* create a polygon of the pin clearance */
-pcb_polyarea_t *pcb_poly_from_pin(pcb_pin_t * pin, pcb_coord_t thick, pcb_coord_t clear)
-{
-	int size;
-
-	if (PCB_FLAG_TEST(PCB_FLAG_SQUARE, pin)) {
-		if (PCB_FLAG_SQUARE_GET(pin) <= 1) {
-			size = (thick + 1) / 2;
-			return RoundRect(pin->X - size, pin->X + size, pin->Y - size, pin->Y + size, (clear + 1) / 2);
-		}
-		else {
-			size = (thick + clear + 1) / 2;
-			return pcb_poly_from_octagon(pin->X, pin->Y, size + size, PCB_FLAG_SQUARE_GET(pin));
-		}
-
-	}
-	else {
-		size = (thick + clear + 1) / 2;
-		if (PCB_FLAG_TEST(PCB_FLAG_OCTAGON, pin)) {
-			if (PCB_FLAG_SQUARE_GET(pin) <= 1)
-				size /= 2;
-			return pcb_poly_from_octagon(pin->X, pin->Y, size + size, PCB_FLAG_SQUARE_GET(pin));
-		}
-	}
-	return pcb_poly_from_circle(pin->X, pin->Y, size);
-}
-
 pcb_polyarea_t *pcb_poly_from_box_bloated(pcb_box_t * box, pcb_coord_t bloat)
 {
 	return pcb_poly_from_rect(box->X1 - bloat, box->X2 + bloat, box->Y1 - bloat, box->Y2 + bloat);
 }
 
-/* return the clearance polygon for a pin */
-static pcb_polyarea_t *pin_clearance_poly(pcb_cardinal_t layernum, pcb_board_t *pcb, pcb_pin_t * pin)
-{
-	pcb_polyarea_t *np;
-	if (PCB_FLAG_THERM_TEST(layernum, pin))
-		np = pcb_thermal_area_pin(pcb, pin, layernum);
-	else
-		np = pcb_poly_from_pin(pin, PIN_SIZE(pin), pin->Clearance);
-	return np;
-}
-
-/* remove the pin clearance from the polygon */
-static int SubtractPin(pcb_data_t * d, pcb_pin_t * pin, pcb_layer_t * l, pcb_poly_t * p)
-{
-	pcb_polyarea_t *np;
-	pcb_layer_id_t i;
-
-	if (pin->Clearance == 0)
-		return 0;
-	i = pcb_layer_id(d, l);
-	np = pin_clearance_poly(i, pcb_data_get_top(d), pin);
-
-	if (PCB_FLAG_THERM_TEST(i, pin)) {
-		if (!np)
-			return 0;
-	}
-	else {
-		if (!np)
-			return -1;
-	}
-
-	return Subtract(np, p, pcb_true);
-}
-
 /* remove the padstack clearance from the polygon */
 static int SubtractPadstack(pcb_data_t *d, pcb_pstk_t *ps, pcb_layer_t *l, pcb_poly_t *p)
 {
@@ -927,28 +769,11 @@ static int SubtractText(pcb_text_t * text, pcb_poly_t * p)
 
 	if (!PCB_FLAG_TEST(PCB_FLAG_CLEARLINE, text))
 		return 0;
-	if (!(np = RoundRect(b->X1 + PCB->Bloat, b->X2 - PCB->Bloat, b->Y1 + PCB->Bloat, b->Y2 - PCB->Bloat, PCB->Bloat)))
+	if (!(np = RoundRect(b->X1 + conf_core.design.bloat, b->X2 - conf_core.design.bloat, b->Y1 + conf_core.design.bloat, b->Y2 - conf_core.design.bloat, conf_core.design.bloat)))
 		return -1;
 	return Subtract(np, p, pcb_true);
 }
 
-static int SubtractPad(pcb_pad_t * pad, pcb_poly_t * p)
-{
-	pcb_polyarea_t *np = NULL;
-
-	if (pad->Clearance == 0)
-		return 0;
-	if (PCB_FLAG_TEST(PCB_FLAG_SQUARE, pad)) {
-		if (!(np = SquarePadPoly(pad, pad->Thickness + pad->Clearance)))
-			return -1;
-	}
-	else {
-		if (!(np = pcb_poly_from_line((pcb_line_t *) pad, pad->Thickness + pad->Clearance)))
-			return -1;
-	}
-	return Subtract(np, p, pcb_true);
-}
-
 struct cpInfo {
 	const pcb_box_t *other;
 	pcb_data_t *data;
@@ -969,44 +794,18 @@ static void subtract_accumulated(struct cpInfo *info, pcb_poly_t *polygon)
 	info->batch_size = 0;
 }
 
-static pcb_r_dir_t pin_sub_callback(const pcb_box_t * b, void *cl)
-{
-	pcb_pin_t *pin = (pcb_pin_t *) b;
-	struct cpInfo *info = (struct cpInfo *) cl;
-	pcb_poly_t *polygon;
-	pcb_polyarea_t *np;
-	pcb_polyarea_t *merged;
-	pcb_layer_id_t i;
+static int pcb_poly_clip_noop = 0;
+static void *pcb_poly_clip_prog_ctx;
+static void (*pcb_poly_clip_prog)(void *ctx) = NULL;
 
-	/* don't subtract the object that was put back! */
-	if (b == info->other)
-		return PCB_R_DIR_NOT_FOUND;
-	polygon = info->polygon;
-
-	if (pin->Clearance == 0)
-		return PCB_R_DIR_NOT_FOUND;
-	i = pcb_layer_id(info->data, info->layer);
-	if (PCB_FLAG_THERM_TEST(i, pin)) {
-		np = pcb_thermal_area_pin(pcb_data_get_top(info->data), pin, i);
-		if (!np)
-			return PCB_R_DIR_FOUND_CONTINUE;
-	}
-	else {
-		np = pcb_poly_from_pin(pin, PIN_SIZE(pin), pin->Clearance);
-		if (!np)
-			longjmp(info->env, 1);
-	}
-
-	pcb_polyarea_boolean_free(info->accumulate, np, &merged, PCB_PBO_UNITE);
-	info->accumulate = merged;
-
-	info->batch_size++;
-
-	if (info->batch_size == SUBTRACT_PIN_VIA_BATCH_SIZE)
-		subtract_accumulated(info, polygon);
-
-	return PCB_R_DIR_FOUND_CONTINUE;
-}
+/* call the progress report callback and return if no-op is set */
+#define POLY_CLIP_PROG() \
+do { \
+	if (pcb_poly_clip_prog != NULL) \
+		pcb_poly_clip_prog(pcb_poly_clip_prog_ctx); \
+	if (pcb_poly_clip_noop) \
+		return PCB_R_DIR_FOUND_CONTINUE; \
+} while(0)
 
 static pcb_r_dir_t padstack_sub_callback(const pcb_box_t *b, void *cl)
 {
@@ -1025,17 +824,19 @@ static pcb_r_dir_t padstack_sub_callback(const pcb_box_t *b, void *cl)
 	/* ps->Clearance == 0 doesn't mean no clearance because of the per shape clearances */
 	if (!PCB_FLAG_TEST(PCB_FLAG_CLEARLINE, ps))
 		return PCB_R_DIR_NOT_FOUND;
+
 	i = pcb_layer_id(info->data, info->layer);
 
 	np = pcb_thermal_area_pstk(pcb_data_get_top(info->data), ps, i);
 	if (np == 0)
 			return PCB_R_DIR_FOUND_CONTINUE;
 
+	info->batch_size++;
+	POLY_CLIP_PROG();
+
 	pcb_polyarea_boolean_free(info->accumulate, np, &merged, PCB_PBO_UNITE);
 	info->accumulate = merged;
 
-	info->batch_size++;
-
 	if (info->batch_size == SUBTRACT_PADSTACK_BATCH_SIZE)
 		subtract_accumulated(info, polygon);
 
@@ -1053,7 +854,11 @@ static pcb_r_dir_t arc_sub_callback(const pcb_box_t * b, void *cl)
 		return PCB_R_DIR_NOT_FOUND;
 	if (!PCB_NONPOLY_HAS_CLEARANCE(arc))
 		return PCB_R_DIR_NOT_FOUND;
+
+	POLY_CLIP_PROG();
+
 	polygon = info->polygon;
+
 	if (SubtractArc(arc, polygon) < 0)
 		longjmp(info->env, 1);
 	return PCB_R_DIR_FOUND_CONTINUE;
@@ -1066,7 +871,7 @@ static pcb_polyarea_t *poly_sub_callback_line(pcb_coord_t x1, pcb_coord_t y1, pc
 	static int inited = 0;
 
 	if (!inited) {
-		lin.type = PCB_TYPE_LINE;
+		lin.type = PCB_OBJ_LINE;
 		lin.Flags = pcb_no_flags();
 		PCB_FLAG_SET(PCB_FLAG_CLEARLINE, &lin);
 		lin.Thickness = 0;
@@ -1205,32 +1010,14 @@ static pcb_r_dir_t poly_sub_callback(const pcb_box_t *b, void *cl)
 	if (!PCB_POLY_HAS_CLEARANCE(subpoly))
 		return PCB_R_DIR_NOT_FOUND;
 
+	POLY_CLIP_PROG();
+
 	if (SubtractPolyPoly(subpoly, polygon) < 0)
 		longjmp(info->env, 1);
 
 	return PCB_R_DIR_FOUND_CONTINUE;
 }
 
-static pcb_r_dir_t pad_sub_callback(const pcb_box_t * b, void *cl)
-{
-	pcb_pad_t *pad = (pcb_pad_t *) b;
-	struct cpInfo *info = (struct cpInfo *) cl;
-	pcb_poly_t *polygon;
-
-	/* don't subtract the object that was put back! */
-	if (b == info->other)
-		return PCB_R_DIR_NOT_FOUND;
-	if (pad->Clearance == 0)
-		return PCB_R_DIR_NOT_FOUND;
-	polygon = info->polygon;
-	if (PCB_XOR(PCB_FLAG_TEST(PCB_FLAG_ONSOLDER, pad), !info->solder)) {
-		if (SubtractPad(pad, polygon) < 0)
-			longjmp(info->env, 1);
-		return PCB_R_DIR_FOUND_CONTINUE;
-	}
-	return PCB_R_DIR_NOT_FOUND;
-}
-
 static pcb_r_dir_t line_sub_callback(const pcb_box_t * b, void *cl)
 {
 	pcb_line_t *line = (pcb_line_t *) b;
@@ -1246,6 +1033,8 @@ static pcb_r_dir_t line_sub_callback(const pcb_box_t * b, void *cl)
 		return PCB_R_DIR_NOT_FOUND;
 	polygon = info->polygon;
 
+	POLY_CLIP_PROG();
+
 	np = line_clearance_poly(-1, NULL, line);
 	if (!np)
 		longjmp(info->env, 1);
@@ -1271,20 +1060,25 @@ static pcb_r_dir_t text_sub_callback(const pcb_box_t * b, void *cl)
 		return PCB_R_DIR_NOT_FOUND;
 	if (!PCB_FLAG_TEST(PCB_FLAG_CLEARLINE, text))
 		return PCB_R_DIR_NOT_FOUND;
+
+	POLY_CLIP_PROG();
+
 	polygon = info->polygon;
 	if (SubtractText(text, polygon) < 0)
 		longjmp(info->env, 1);
 	return PCB_R_DIR_FOUND_CONTINUE;
 }
 
-static int clearPoly(pcb_data_t *Data, pcb_layer_t *Layer, pcb_poly_t * polygon, const pcb_box_t * here, pcb_coord_t expand)
+static pcb_cardinal_t clearPoly(pcb_data_t *Data, pcb_layer_t *Layer, pcb_poly_t *polygon, const pcb_box_t *here, pcb_coord_t expand, int noop)
 {
-	int r = 0, seen;
+	pcb_cardinal_t r = 0;
+	int seen;
 	pcb_box_t region;
 	struct cpInfo info;
 	pcb_layergrp_id_t group;
 	unsigned int gflg;
 	pcb_layer_type_t lf;
+	int old_noop;
 
 	lf = pcb_layer_flags_(Layer);
 	if (!(lf & PCB_LYT_COPPER)) { /* also handles lf == 0 */
@@ -1295,6 +1089,9 @@ static int clearPoly(pcb_data_t *Data, pcb_layer_t *Layer, pcb_poly_t * polygon,
 	if (!PCB_FLAG_TEST(PCB_FLAG_CLEARPOLY, polygon))
 		return 0;
 
+	old_noop = pcb_poly_clip_noop;
+	pcb_poly_clip_noop = noop;
+
 	group = pcb_layer_get_group_(Layer);
 	gflg = pcb_layergrp_flags(PCB, group);
 	info.solder = (gflg & PCB_LYT_BOTTOM);
@@ -1313,10 +1110,6 @@ static int clearPoly(pcb_data_t *Data, pcb_layer_t *Layer, pcb_poly_t * polygon,
 		r = 0;
 		info.accumulate = NULL;
 		info.batch_size = 0;
-		if (info.solder || (gflg & PCB_LYT_TOP)) {
-			pcb_r_search(Data->pad_tree, &region, NULL, pad_sub_callback, &info, &seen);
-			r += seen;
-		}
 		PCB_COPPER_GROUP_LOOP(Data, group);
 		{
 			pcb_r_search(layer->line_tree, &region, NULL, line_sub_callback, &info, &seen);
@@ -1330,15 +1123,13 @@ static int clearPoly(pcb_data_t *Data, pcb_layer_t *Layer, pcb_poly_t * polygon,
 			r += seen;
 		}
 		PCB_END_LOOP;
-		pcb_r_search(Data->via_tree, &region, NULL, pin_sub_callback, &info, &seen);
-		r += seen;
-		pcb_r_search(Data->pin_tree, &region, NULL, pin_sub_callback, &info, &seen);
-		r += seen;
 		pcb_r_search(Data->padstack_tree, &region, NULL, padstack_sub_callback, &info, &seen);
 		r += seen;
 		subtract_accumulated(&info, polygon);
 	}
-	polygon->NoHolesValid = 0;
+	if (!noop)
+		polygon->NoHolesValid = 0;
+	pcb_poly_clip_noop = old_noop;
 	return r;
 }
 
@@ -1378,22 +1169,6 @@ fail:
 	return 0;
 }
 
-static int UnsubtractPin(pcb_pin_t * pin, pcb_layer_t * l, pcb_poly_t * p)
-{
-	pcb_polyarea_t *np;
-
-	/* overlap a bit to prevent gaps from rounding errors */
-	np = pcb_poly_from_box_bloated(&pin->BoundingBox, UNSUBTRACT_BLOAT * 400000);
-
-	if (!np)
-		return 0;
-	if (!Unsubtract(np, p))
-		return 0;
-
-	clearPoly(PCB->Data, l, p, (const pcb_box_t *) pin, 2 * UNSUBTRACT_BLOAT * 400000);
-	return 1;
-}
-
 static int UnsubtractPadstack(pcb_data_t *data, pcb_pstk_t *ps, pcb_layer_t *l, pcb_poly_t *p)
 {
 	pcb_polyarea_t *np;
@@ -1406,7 +1181,7 @@ static int UnsubtractPadstack(pcb_data_t *data, pcb_pstk_t *ps, pcb_layer_t *l,
 	if (!Unsubtract(np, p))
 		return 0;
 
-	clearPoly(PCB->Data, l, p, (const pcb_box_t *)ps, 2 * UNSUBTRACT_BLOAT * 400000);
+	clearPoly(PCB->Data, l, p, (const pcb_box_t *)ps, 2 * UNSUBTRACT_BLOAT * 400000, 0);
 	return 1;
 }
 
@@ -1426,7 +1201,7 @@ static int UnsubtractArc(pcb_arc_t * arc, pcb_layer_t * l, pcb_poly_t * p)
 		return 0;
 	if (!Unsubtract(np, p))
 		return 0;
-	clearPoly(PCB->Data, l, p, (const pcb_box_t *) arc, 2 * UNSUBTRACT_BLOAT);
+	clearPoly(PCB->Data, l, p, (const pcb_box_t *) arc, 2 * UNSUBTRACT_BLOAT, 0);
 	return 1;
 }
 
@@ -1444,7 +1219,7 @@ static int UnsubtractLine(pcb_line_t * line, pcb_layer_t * l, pcb_poly_t * p)
 		return 0;
 	if (!Unsubtract(np, p))
 		return 0;
-	clearPoly(PCB->Data, l, p, (const pcb_box_t *) line, 2 * UNSUBTRACT_BLOAT);
+	clearPoly(PCB->Data, l, p, (const pcb_box_t *) line, 2 * UNSUBTRACT_BLOAT, 0);
 	return 1;
 }
 
@@ -1462,31 +1237,18 @@ static int UnsubtractText(pcb_text_t * text, pcb_layer_t * l, pcb_poly_t * p)
 		return -1;
 	if (!Unsubtract(np, p))
 		return 0;
-	clearPoly(PCB->Data, l, p, (const pcb_box_t *) text, 2 * UNSUBTRACT_BLOAT);
-	return 1;
-}
-
-static int UnsubtractPad(pcb_pad_t * pad, pcb_layer_t * l, pcb_poly_t * p)
-{
-	pcb_polyarea_t *np;
-
-	/* overlap a bit to prevent notches from rounding errors */
-	np = pcb_poly_from_box_bloated(&pad->BoundingBox, UNSUBTRACT_BLOAT);
-
-	if (!np)
-		return 0;
-	if (!Unsubtract(np, p))
-		return 0;
-	clearPoly(PCB->Data, l, p, (const pcb_box_t *) pad, 2 * UNSUBTRACT_BLOAT);
+	clearPoly(PCB->Data, l, p, (const pcb_box_t *) text, 2 * UNSUBTRACT_BLOAT, 0);
 	return 1;
 }
 
 static pcb_bool inhibit = pcb_false;
 
-int pcb_poly_init_clip(pcb_data_t *Data, pcb_layer_t *layer, pcb_poly_t * p)
+int pcb_poly_init_clip_prog(pcb_data_t *Data, pcb_layer_t *layer, pcb_poly_t *p, void (*cb)(void *ctx), void *ctx)
 {
 	pcb_board_t *pcb;
 	pcb_bool need_full;
+	void (*old_cb)(void *ctx);
+	void *old_ctx;
 
 	if (inhibit)
 		return 0;
@@ -1520,13 +1282,43 @@ int pcb_poly_init_clip(pcb_data_t *Data, pcb_layer_t *layer, pcb_poly_t * p)
 	if (pcb != NULL)
 		Data = pcb->Data;
 
+	old_cb = pcb_poly_clip_prog;
+	old_ctx = pcb_poly_clip_prog_ctx;
+	pcb_poly_clip_prog = cb;
+	pcb_poly_clip_prog_ctx = ctx;
+
 	if (PCB_FLAG_TEST(PCB_FLAG_CLEARPOLY, p))
-		clearPoly(Data, layer, p, NULL, 0);
+		clearPoly(Data, layer, p, NULL, 0, 0);
 	else
 		p->NoHolesValid = 0;
+
+	pcb_poly_clip_prog = old_cb;
+	pcb_poly_clip_prog_ctx = old_ctx;
 	return 1;
 }
 
+int pcb_poly_init_clip(pcb_data_t *Data, pcb_layer_t *layer, pcb_poly_t *p)
+{
+	return pcb_poly_init_clip_prog(Data, layer, p, NULL, NULL);
+}
+
+pcb_cardinal_t pcb_poly_num_clears(pcb_data_t *data, pcb_layer_t *layer, pcb_poly_t *polygon)
+{
+	pcb_cardinal_t res;
+	void (*old_cb)(void *ctx);
+
+	if (layer->is_bound)
+		layer = layer->meta.bound.real;
+
+	old_cb = pcb_poly_clip_prog;
+	pcb_poly_clip_prog = NULL;
+
+	res = clearPoly(data, layer, polygon, NULL, 0, 1);
+
+	pcb_poly_clip_prog = old_cb;
+	return res;
+}
+
 /* --------------------------------------------------------------------------
  * remove redundant polygon points. Any point that lies on the straight
  * line between the points on either side of it is redundant.
@@ -1551,7 +1343,7 @@ pcb_bool pcb_poly_remove_excess_points(pcb_layer_t *Layer, pcb_poly_t *Polygon)
 		line.Point2 = Polygon->Points[next];
 		line.Thickness = 0;
 		if (pcb_is_point_on_line(p->X, p->Y, 0.0, &line)) {
-			pcb_remove_object(PCB_TYPE_POLY_POINT, Layer, Polygon, p);
+			pcb_remove_object(PCB_OBJ_POLY_POINT, Layer, Polygon, p);
 			changed = pcb_true;
 		}
 	}
@@ -1619,7 +1411,6 @@ void pcb_polygon_go_to_prev_point(void)
 	case 1:
 		pcb_crosshair.AttachedPolygon.PointN = 0;
 		pcb_crosshair.AttachedLine.State = PCB_CH_STATE_FIRST;
-		pcb_added_lines = 0;
 		break;
 
 		/* back-up one point */
@@ -1723,10 +1514,9 @@ void pcb_polygon_copy_attached_to_layer(void)
 
 	/* reset state of attached line */
 	pcb_crosshair.AttachedLine.State = PCB_CH_STATE_FIRST;
-	pcb_added_lines = 0;
 
 	/* add to undo list */
-	pcb_undo_add_obj_to_create(PCB_TYPE_POLY, layer, polygon, polygon);
+	pcb_undo_add_obj_to_create(PCB_OBJ_POLY, layer, polygon, polygon);
 	pcb_undo_inc_serial();
 }
 
@@ -1781,7 +1571,7 @@ void pcb_polygon_hole_create_from_attached(void)
 	pcb_undo_save_serial();
 	Flags = ((pcb_poly_t *) pcb_crosshair.AttachedObject.Ptr2)->Flags;
 	pcb_poly_to_polygons_on_layer(PCB->Data, (pcb_layer_t *) pcb_crosshair.AttachedObject.Ptr1, result, Flags);
-	pcb_remove_object(PCB_TYPE_POLY,
+	pcb_remove_object(PCB_OBJ_POLY,
 							 pcb_crosshair.AttachedObject.Ptr1, pcb_crosshair.AttachedObject.Ptr2, pcb_crosshair.AttachedObject.Ptr3);
 	pcb_undo_restore_serial();
 	pcb_undo_inc_serial();
@@ -1790,7 +1580,6 @@ void pcb_polygon_hole_create_from_attached(void)
 	memset(&pcb_crosshair.AttachedPolygon, 0, sizeof(pcb_poly_t));
 	pcb_crosshair.AttachedLine.State = PCB_CH_STATE_FIRST;
 	pcb_crosshair.AttachedObject.State = PCB_CH_STATE_FIRST;
-	pcb_added_lines = 0;
 }
 
 /* find polygon holes in range, then call the callback function for
@@ -1830,35 +1619,26 @@ static pcb_r_dir_t subtract_plow(pcb_data_t *Data, pcb_layer_t *Layer, pcb_poly_
 	if (!Polygon->Clipped)
 		return 0;
 	switch (type) {
-	case PCB_TYPE_PIN:
-	case PCB_TYPE_VIA:
-		SubtractPin(Data, (pcb_pin_t *) ptr2, Layer, Polygon);
-		Polygon->NoHolesValid = 0;
-		return PCB_R_DIR_FOUND_CONTINUE;
-	case PCB_TYPE_PSTK:
+	case PCB_OBJ_PSTK:
 		SubtractPadstack(Data, (pcb_pstk_t *) ptr2, Layer, Polygon);
 		Polygon->NoHolesValid = 0;
 		return PCB_R_DIR_FOUND_CONTINUE;
-	case PCB_TYPE_LINE:
+	case PCB_OBJ_LINE:
 		SubtractLine((pcb_line_t *) ptr2, Polygon);
 		Polygon->NoHolesValid = 0;
 		return PCB_R_DIR_FOUND_CONTINUE;
-	case PCB_TYPE_ARC:
+	case PCB_OBJ_ARC:
 		SubtractArc((pcb_arc_t *) ptr2, Polygon);
 		Polygon->NoHolesValid = 0;
 		return PCB_R_DIR_FOUND_CONTINUE;
-	case PCB_TYPE_POLY:
+	case PCB_OBJ_POLY:
 		if (ptr2 != Polygon) {
 			SubtractPolyPoly((pcb_poly_t *) ptr2, Polygon);
 			Polygon->NoHolesValid = 0;
 			return PCB_R_DIR_FOUND_CONTINUE;
 		}
 		break;
-	case PCB_TYPE_PAD:
-		SubtractPad((pcb_pad_t *) ptr2, Polygon);
-		Polygon->NoHolesValid = 0;
-		return PCB_R_DIR_FOUND_CONTINUE;
-	case PCB_TYPE_TEXT:
+	case PCB_OBJ_TEXT:
 		SubtractText((pcb_text_t *) ptr2, Polygon);
 		Polygon->NoHolesValid = 0;
 		return PCB_R_DIR_FOUND_CONTINUE;
@@ -1869,29 +1649,22 @@ static pcb_r_dir_t subtract_plow(pcb_data_t *Data, pcb_layer_t *Layer, pcb_poly_
 static pcb_r_dir_t add_plow(pcb_data_t *Data, pcb_layer_t *Layer, pcb_poly_t *Polygon, int type, void *ptr1, void *ptr2)
 {
 	switch (type) {
-	case PCB_TYPE_PIN:
-	case PCB_TYPE_VIA:
-		UnsubtractPin((pcb_pin_t *) ptr2, Layer, Polygon);
-		return PCB_R_DIR_FOUND_CONTINUE;
-	case PCB_TYPE_PSTK:
+	case PCB_OBJ_PSTK:
 		UnsubtractPadstack(Data, (pcb_pstk_t *) ptr2, Layer, Polygon);
 		return PCB_R_DIR_FOUND_CONTINUE;
-	case PCB_TYPE_LINE:
+	case PCB_OBJ_LINE:
 		UnsubtractLine((pcb_line_t *) ptr2, Layer, Polygon);
 		return PCB_R_DIR_FOUND_CONTINUE;
-	case PCB_TYPE_ARC:
+	case PCB_OBJ_ARC:
 		UnsubtractArc((pcb_arc_t *) ptr2, Layer, Polygon);
 		return PCB_R_DIR_FOUND_CONTINUE;
-	case PCB_TYPE_POLY:
+	case PCB_OBJ_POLY:
 		if (ptr2 != Polygon) {
 			UnsubtractPolyPoly((pcb_poly_t *) ptr2, Polygon);
 			return PCB_R_DIR_FOUND_CONTINUE;
 		}
 		break;
-	case PCB_TYPE_PAD:
-		UnsubtractPad((pcb_pad_t *) ptr2, Layer, Polygon);
-		return PCB_R_DIR_FOUND_CONTINUE;
-	case PCB_TYPE_TEXT:
+	case PCB_OBJ_TEXT:
 		UnsubtractText((pcb_text_t *) ptr2, Layer, Polygon);
 		return PCB_R_DIR_FOUND_CONTINUE;
 	}
@@ -1935,7 +1708,7 @@ int
 pcb_poly_plows(pcb_data_t * Data, int type, void *ptr1, void *ptr2,
 						 pcb_r_dir_t (*call_back) (pcb_data_t *data, pcb_layer_t *lay, pcb_poly_t *poly, int type, void *ptr1, void *ptr2))
 {
-	pcb_box_t sb = ((pcb_pin_t *) ptr2)->BoundingBox;
+	pcb_box_t sb = ((pcb_any_obj_t *) ptr2)->BoundingBox;
 	int r = 0, seen;
 	struct plow_info info;
 
@@ -1948,32 +1721,7 @@ pcb_poly_plows(pcb_data_t * Data, int type, void *ptr1, void *ptr2,
 	info.data = Data;
 	info.callback = call_back;
 	switch (type) {
-	case PCB_TYPE_PIN:
-	case PCB_TYPE_VIA:
-		if (pcb_data_get_top(Data) == NULL) /* don't do it if not on a board */
-			break;
-		if (type == PCB_TYPE_PIN || ptr1 == ptr2 || ptr1 == NULL) {
-			LAYER_LOOP(Data, pcb_max_layer);
-			{
-				if (!(pcb_layer_flags_(layer) & PCB_LYT_COPPER))
-					continue;
-				info.layer = layer;
-				pcb_r_search(layer->polygon_tree, &sb, NULL, plow_callback, &info, &seen);
-				r += seen;
-			}
-			PCB_END_LOOP;
-		}
-		else {
-			PCB_COPPER_GROUP_LOOP(Data, pcb_layer_get_group(PCB, pcb_layer_id(Data, ((pcb_layer_t *) ptr1))));
-			{
-				info.layer = layer;
-				pcb_r_search(layer->polygon_tree, &sb, NULL, plow_callback, &info, &seen);
-				r += seen;
-			}
-			PCB_END_LOOP;
-		}
-		break;
-	case PCB_TYPE_PSTK:
+	case PCB_OBJ_PSTK:
 		if (Data->parent_type != PCB_PARENT_BOARD)
 			return 0;
 		if (ptr1 == NULL) { /* no layer specified: run on all layers */
@@ -1992,19 +1740,19 @@ pcb_poly_plows(pcb_data_t * Data, int type, void *ptr1, void *ptr2,
 		if (!PCB_FLAG_TEST(PCB_FLAG_CLEARLINE, (pcb_pstk_t *)ptr2))
 			return 0;
 		goto doit;
-	case PCB_TYPE_POLY:
+	case PCB_OBJ_POLY:
 		if (!PCB_POLY_HAS_CLEARANCE((pcb_poly_t *) ptr2))
 			return 0;
 		goto doit;
 
-	case PCB_TYPE_LINE:
-	case PCB_TYPE_ARC:
+	case PCB_OBJ_LINE:
+	case PCB_OBJ_ARC:
 		/* the cast works equally well for lines and arcs */
 		if (!PCB_NONPOLY_HAS_CLEARANCE((pcb_line_t *) ptr2))
 			return 0;
 		goto doit;
 
-	case PCB_TYPE_TEXT:
+	case PCB_OBJ_TEXT:
 		/* text has no clearance property */
 		if (!PCB_FLAG_TEST(PCB_FLAG_CLEARLINE, (pcb_line_t *) ptr2))
 			return 0;
@@ -2021,38 +1769,6 @@ pcb_poly_plows(pcb_data_t * Data, int type, void *ptr1, void *ptr2,
 		}
 		PCB_END_LOOP;
 		break;
-	case PCB_TYPE_PAD:
-		{
-			pcb_layergrp_id_t SLayer, CLayer, group;
-	
-			SLayer = CLayer = -1;
-			pcb_layergrp_list(PCB, PCB_LYT_BOTTOM | PCB_LYT_COPPER, &SLayer, 1);
-			pcb_layergrp_list(PCB, PCB_LYT_TOP | PCB_LYT_COPPER, &CLayer, 1);
-			group = PCB_FLAG_TEST(PCB_FLAG_ONSOLDER, (pcb_pad_t *) ptr2) ? SLayer : CLayer;
-			PCB_COPPER_GROUP_LOOP(Data, group);
-			{
-				info.layer = layer;
-				pcb_r_search(layer->polygon_tree, &sb, NULL, plow_callback, &info, &seen);
-				r += seen;
-			}
-			PCB_END_LOOP;
-		}
-		break;
-
-	case PCB_TYPE_ELEMENT:
-		{
-			PCB_PIN_LOOP((pcb_element_t *) ptr1);
-			{
-				pcb_poly_plows(Data, PCB_TYPE_PIN, ptr1, pin, call_back);
-			}
-			PCB_END_LOOP;
-			PCB_PAD_LOOP((pcb_element_t *) ptr1);
-			{
-				pcb_poly_plows(Data, PCB_TYPE_PAD, ptr1, pad, call_back);
-			}
-			PCB_END_LOOP;
-		}
-		break;
 	}
 	return r;
 }
@@ -2064,7 +1780,7 @@ void pcb_poly_restore_to_poly(pcb_data_t * Data, int type, void *ptr1, void *ptr
 		dt = dt->parent.subc->parent.data;
 	if ((dt == NULL) || (dt->parent_type != PCB_PARENT_BOARD)) /* clear/restore only on boards */
 		return;
-	if (type == PCB_TYPE_POLY)
+	if (type == PCB_OBJ_POLY)
 		pcb_poly_init_clip(dt, (pcb_layer_t *) ptr1, (pcb_poly_t *) ptr2);
 	pcb_poly_plows(dt, type, ptr1, ptr2, add_plow);
 }
@@ -2076,7 +1792,7 @@ void pcb_poly_clear_from_poly(pcb_data_t * Data, int type, void *ptr1, void *ptr
 		dt = dt->parent.subc->parent.data;
 	if ((dt == NULL) || (dt->parent_type != PCB_PARENT_BOARD)) /* clear/restore only on boards */
 		return;
-	if (type == PCB_TYPE_POLY)
+	if (type == PCB_OBJ_POLY)
 		pcb_poly_init_clip(dt, (pcb_layer_t *) ptr1, (pcb_poly_t *) ptr2);
 	pcb_poly_plows(dt, type, ptr1, ptr2, subtract_plow);
 }
@@ -2241,7 +1957,7 @@ pcb_bool pcb_poly_morph(pcb_layer_t *layer, pcb_poly_t *poly)
 		pcb_vnode_t *v;
 		pcb_poly_t *newone;
 
-		if (p->contours->area > PCB->IsleArea) {
+		if (p->contours->area > conf_core.design.poly_isle_area) {
 			newone = pcb_poly_new(layer, poly->Clearance, flags);
 			if (!newone)
 				return pcb_false;
@@ -2254,7 +1970,7 @@ pcb_bool pcb_poly_morph(pcb_layer_t *layer, pcb_poly_t *poly)
 			newone->BoundingBox.X2 = p->contours->xmax + 1;
 			newone->BoundingBox.Y1 = p->contours->ymin;
 			newone->BoundingBox.Y2 = p->contours->ymax + 1;
-			pcb_undo_add_obj_to_create(PCB_TYPE_POLY, layer, newone, newone);
+			pcb_undo_add_obj_to_create(PCB_OBJ_POLY, layer, newone, newone);
 			newone->Clipped = p;
 			p = p->f;									/* go to next pline */
 			newone->Clipped->b = newone->Clipped->f = newone->Clipped;	/* unlink from others */
@@ -2363,7 +2079,7 @@ void pcb_poly_to_polygons_on_layer(pcb_data_t * Destination, pcb_layer_t * Layer
 
 		pcb_poly_invalidate_draw(Layer, Polygon);
 		/* add to undo list */
-		pcb_undo_add_obj_to_create(PCB_TYPE_POLY, Layer, Polygon, Polygon);
+		pcb_undo_add_obj_to_create(PCB_OBJ_POLY, Layer, Polygon, Polygon);
 	}
 	while ((pa = pa->f) != Input);
 
