@@ -115,7 +115,7 @@ int pcb_vect_inters2(pcb_vector_t A, pcb_vector_t B, pcb_vector_t C, pcb_vector_
 
 #ifndef NDEBUG
 #include <stdarg.h>
-static void DEBUGP(const char *fmt, ...)
+PCB_INLINE void DEBUGP(const char *fmt, ...)
 {
 	va_list ap;
 	va_start(ap, fmt);
@@ -123,7 +123,7 @@ static void DEBUGP(const char *fmt, ...)
 	va_end(ap);
 }
 #else
-static void DEBUGP(const char *fmt, ...) { }
+PCB_INLINE void DEBUGP(const char *fmt, ...) { }
 #endif
 
 /* ///////////////////////////////////////////////////////////////////////////// * /
@@ -521,7 +521,7 @@ static int adjust_tree(pcb_rtree_t * tree, struct seg *s)
 	q->box.X2 = max(q->v->point[0], q->v->next->point[0]) + 1;
 	q->box.Y1 = min(q->v->point[1], q->v->next->point[1]);
 	q->box.Y2 = max(q->v->point[1], q->v->next->point[1]) + 1;
-	pcb_r_insert_entry(tree, (const pcb_box_t *) q, 1);
+	pcb_r_insert_entry(tree, (const pcb_box_t *) q);
 	q = (seg *) malloc(sizeof(struct seg));
 	if (!q)
 		return 1;
@@ -532,8 +532,9 @@ static int adjust_tree(pcb_rtree_t * tree, struct seg *s)
 	q->box.X2 = max(q->v->point[0], q->v->next->point[0]) + 1;
 	q->box.Y1 = min(q->v->point[1], q->v->next->point[1]);
 	q->box.Y2 = max(q->v->point[1], q->v->next->point[1]) + 1;
-	pcb_r_insert_entry(tree, (const pcb_box_t *) q, 1);
+	pcb_r_insert_entry(tree, (const pcb_box_t *) q);
 	pcb_r_delete_entry(tree, (const pcb_box_t *) s);
+	free(s);
 	return 0;
 }
 
@@ -634,7 +635,7 @@ static void *make_edge_tree(pcb_pline_t * pb)
 {
 	struct seg *s;
 	pcb_vnode_t *bv;
-	pcb_rtree_t *ans = pcb_r_create_tree(NULL, 0, 0);
+	pcb_rtree_t *ans = pcb_r_create_tree();
 	bv = &pb->head;
 	do {
 		s = (seg *) malloc(sizeof(struct seg));
@@ -657,7 +658,7 @@ static void *make_edge_tree(pcb_pline_t * pb)
 		}
 		s->v = bv;
 		s->p = pb;
-		pcb_r_insert_entry(ans, (const pcb_box_t *) s, 1);
+		pcb_r_insert_entry(ans, (const pcb_box_t *) s);
 	}
 	while ((bv = bv->next) != &pb->head);
 	return (void *) ans;
@@ -1090,8 +1091,8 @@ static void InsCntr(jmp_buf * e, pcb_pline_t * c, pcb_polyarea_t ** dst)
 		newp->f->b = newp->b->f = newp;
 	}
 	newp->contours = c;
-	newp->contour_tree = pcb_r_create_tree(NULL, 0, 0);
-	pcb_r_insert_entry(newp->contour_tree, (pcb_box_t *) c, 0);
+	newp->contour_tree = pcb_r_create_tree();
+	pcb_r_insert_entry(newp->contour_tree, (pcb_box_t *) c);
 	c->next = NULL;
 }																/* InsCntr */
 
@@ -1117,7 +1118,7 @@ PutContour(jmp_buf * e, pcb_pline_t * cntr, pcb_polyarea_t ** contours, pcb_plin
 			if (owner != parent) {
 				if (owner != NULL)
 					pcb_r_delete_entry(owner->contour_tree, (pcb_box_t *) cntr);
-				pcb_r_insert_entry(parent->contour_tree, (pcb_box_t *) cntr, 0);
+				pcb_r_insert_entry(parent->contour_tree, (pcb_box_t *) cntr);
 			}
 		}
 		else {
@@ -1209,7 +1210,7 @@ void pcb_poly_insert_holes(jmp_buf * e, pcb_polyarea_t * dest, pcb_pline_t ** sr
 	/* make a polyarea info table */
 	/* make an rtree of polyarea info table */
 	all_pa_info = (struct polyarea_info *) malloc(sizeof(struct polyarea_info) * num_polyareas);
-	tree = pcb_r_create_tree(NULL, 0, 0);
+	tree = pcb_r_create_tree();
 	i = 0;
 	curc = dest;
 	do {
@@ -1218,7 +1219,7 @@ void pcb_poly_insert_holes(jmp_buf * e, pcb_polyarea_t * dest, pcb_pline_t ** sr
 		all_pa_info[i].BoundingBox.X2 = curc->contours->xmax;
 		all_pa_info[i].BoundingBox.Y2 = curc->contours->ymax;
 		all_pa_info[i].pa = curc;
-		pcb_r_insert_entry(tree, (const pcb_box_t *) &all_pa_info[i], 0);
+		pcb_r_insert_entry(tree, (const pcb_box_t *) &all_pa_info[i]);
 		i++;
 	}
 	while ((curc = curc->f) != dest);
@@ -1313,7 +1314,7 @@ void pcb_poly_insert_holes(jmp_buf * e, pcb_polyarea_t * dest, pcb_pline_t ** sr
 			/* link at front of hole list */
 			curh->next = container->next;
 			container->next = curh;
-			pcb_r_insert_entry(pa_info->pa->contour_tree, (pcb_box_t *) curh, 0);
+			pcb_r_insert_entry(pa_info->pa->contour_tree, (pcb_box_t *) curh);
 
 		}
 	}
@@ -2317,6 +2318,7 @@ void pcb_poly_contour_del(pcb_pline_t ** c)
 	/* FIXME -- strict aliasing violation.  */
 	if ((*c)->tree) {
 		pcb_rtree_t *r = (*c)->tree;
+		pcb_r_free_tree_data(r, free);
 		pcb_r_destroy_tree(&r);
 	}
 	free(*c), *c = NULL;
@@ -2467,7 +2469,7 @@ pcb_bool pcb_polyarea_copy0(pcb_polyarea_t ** dst, const pcb_polyarea_t * src)
 		*dst = (pcb_polyarea_t *) calloc(1, sizeof(pcb_polyarea_t));
 	if (*dst == NULL)
 		return pcb_false;
-	(*dst)->contour_tree = pcb_r_create_tree(NULL, 0, 0);
+	(*dst)->contour_tree = pcb_r_create_tree();
 
 	return pcb_polyarea_copy1(*dst, src);
 }
@@ -2482,7 +2484,7 @@ pcb_bool pcb_polyarea_copy1(pcb_polyarea_t * dst, const pcb_polyarea_t * src)
 	for (cur = src->contours; cur != NULL; cur = cur->next) {
 		if (!pcb_poly_contour_copy(last, cur))
 			return pcb_false;
-		pcb_r_insert_entry(dst->contour_tree, (pcb_box_t *) * last, 0);
+		pcb_r_insert_entry(dst->contour_tree, (pcb_box_t *) * last);
 		last = &(*last)->next;
 	}
 	return pcb_true;
@@ -2535,7 +2537,7 @@ pcb_bool pcb_polyarea_contour_include(pcb_polyarea_t * p, pcb_pline_t * c)
 		p->contours->next = c;
 		c->next = tmp;
 	}
-	pcb_r_insert_entry(p->contour_tree, (pcb_box_t *) c, 0);
+	pcb_r_insert_entry(p->contour_tree, (pcb_box_t *) c);
 	return pcb_true;
 }
 
@@ -2788,7 +2790,7 @@ void pcb_polyarea_init(pcb_polyarea_t * p)
 {
 	p->f = p->b = p;
 	p->contours = NULL;
-	p->contour_tree = pcb_r_create_tree(NULL, 0, 0);
+	p->contour_tree = pcb_r_create_tree();
 }
 
 pcb_polyarea_t *pcb_polyarea_create(void)
@@ -2849,31 +2851,91 @@ static pcb_bool inside_sector(pcb_vnode_t * pn, pcb_vector_t p2)
 }																/* inside_sector */
 
 /* returns pcb_true if bad contour */
-pcb_bool pcb_polyarea_contour_check(pcb_pline_t * a)
+typedef struct {
+	int marks, lines;
+#ifndef NDEBUG
+	pcb_coord_t x[8], y[8];
+	pcb_coord_t x1[8], y1[8], x2[8], y2[8];
+	char msg[256];
+#endif
+} pa_chk_res_t;
+
+
+#ifndef NDEBUG
+#define PA_CHK_MARK(x_, y_) \
+do { \
+	if (res->marks < sizeof(res->x) / sizeof(res->x[0])) { \
+		res->x[res->marks] = x_; \
+		res->y[res->marks] = y_; \
+		res->marks++; \
+	} \
+} while(0)
+#define PA_CHK_LINE(x1_, y1_, x2_, y2_) \
+do { \
+	if (res->lines < sizeof(res->x1) / sizeof(res->x1[0])) { \
+		res->x1[res->lines] = x1_; \
+		res->y1[res->lines] = y1_; \
+		res->x2[res->lines] = x2_; \
+		res->y2[res->lines] = y2_; \
+		res->lines++; \
+	} \
+} while(0)
+#else
+#define PA_CHK_MARK(x, y)
+#define PA_CHK_LINE(x1, y1, x2, y2)
+#endif
+
+
+PCB_INLINE pcb_bool PA_CHK_ERROR(pa_chk_res_t *res, const char *fmt, ...)
+{
+#ifndef NDEBUG
+	va_list ap;
+	va_start(ap, fmt);
+	pcb_vsnprintf(res->msg, sizeof(res->msg), fmt, ap);
+	va_end(ap);
+#endif
+	return pcb_true;
+}
+
+pcb_bool pcb_polyarea_contour_check_(pcb_pline_t *a, pa_chk_res_t *res)
 {
 	pcb_vnode_t *a1, *a2, *hit1, *hit2;
 	pcb_vector_t i1, i2;
 	int icnt;
 
+#ifndef NDEBUG
+	*res->msg = '\0';
+#endif
+	res->marks = res->lines = 0;
+
 	assert(a != NULL);
 	a1 = &a->head;
 	do {
 		a2 = a1;
 		do {
 			if (!node_neighbours(a1, a2) && (icnt = pcb_vect_inters2(a1->point, a1->next->point, a2->point, a2->next->point, i1, i2)) > 0) {
-				if (icnt > 1)
-					return pcb_true;
+				if (icnt > 1) {
+					PA_CHK_MARK(a1->point[0], a1->point[1]);
+					PA_CHK_MARK(a2->point[0], a2->point[1]);
+					return PA_CHK_ERROR(res, "icnt > 1 (%d) at %mm;%mm or  %mm;%mm", icnt, a1->point[0], a1->point[1], a2->point[0], a2->point[1]);
+				}
 
-				if (pcb_vect_dist2(i1, a1->point) < EPSILON)
+#warning TODO: ugly workaround: test where exactly the intersection happens and tune the endpoint of the line
+				/* EPSILON^2 for endpoint matching; the bool algebra code is not
+				   perfect and causes tiny self intersections at the end of sharp
+				   spikes. Accept at most 10 nanometer of such intersection */
+#				define ENDP_EPSILON 100
+
+				if (pcb_vect_dist2(i1, a1->point) < ENDP_EPSILON)
 					hit1 = a1;
-				else if (pcb_vect_dist2(i1, a1->next->point) < EPSILON)
+				else if (pcb_vect_dist2(i1, a1->next->point) < ENDP_EPSILON)
 					hit1 = a1->next;
 				else
 					hit1 = NULL;
 
-				if (pcb_vect_dist2(i1, a2->point) < EPSILON)
+				if (pcb_vect_dist2(i1, a2->point) < ENDP_EPSILON)
 					hit2 = a2;
-				else if (pcb_vect_dist2(i1, a2->next->point) < EPSILON)
+				else if (pcb_vect_dist2(i1, a2->next->point) < ENDP_EPSILON)
 					hit2 = a2->next;
 				else
 					hit2 = NULL;
@@ -2882,24 +2944,35 @@ pcb_bool pcb_polyarea_contour_check(pcb_pline_t * a)
 					/* If the intersection didn't land on an end-point of either
 					 * line, we know the lines cross and we return pcb_true.
 					 */
-					return pcb_true;
+					PA_CHK_LINE(a1->point[0], a1->point[1], a1->next->point[0], a1->next->point[1]);
+					PA_CHK_LINE(a2->point[0], a2->point[1], a2->next->point[0], a2->next->point[1]);
+					return PA_CHK_ERROR(res, "lines cross between %mm;%mm and %mm;%mm", a1->point[0], a1->point[1], a2->point[0], a2->point[1]);
 				}
 				else if (hit1 == NULL) {
 					/* An end-point of the second line touched somewhere along the
 					   length of the first line. Check where the second line leads. */
-					if (inside_sector(hit2, a1->point) != inside_sector(hit2, a1->next->point))
-						return pcb_true;
+					if (inside_sector(hit2, a1->point) != inside_sector(hit2, a1->next->point)) {
+						PA_CHK_MARK(a1->point[0], a1->point[1]);
+						PA_CHK_MARK(hit2->point[0], hit2->point[1]);
+						return PA_CHK_ERROR(res, "lines is inside sector (1) at %mm;%mm", a1->point[0], a1->point[1]);
+					}
 				}
 				else if (hit2 == NULL) {
 					/* An end-point of the first line touched somewhere along the
 					   length of the second line. Check where the first line leads. */
-					if (inside_sector(hit1, a2->point) != inside_sector(hit1, a2->next->point))
-						return pcb_true;
+					if (inside_sector(hit1, a2->point) != inside_sector(hit1, a2->next->point)) {
+						PA_CHK_MARK(a2->point[0], a2->point[1]);
+						PA_CHK_MARK(hit1->point[0], hit1->point[1]);
+						return PA_CHK_ERROR(res, "lines is inside sector (2) at %mm;%mm", a2->point[0], a2->point[1]);
+					}
 				}
 				else {
 					/* Both lines intersect at an end-point. Check where they lead. */
-					if (inside_sector(hit1, hit2->prev->point) != inside_sector(hit1, hit2->next->point))
-						return pcb_true;
+					if (inside_sector(hit1, hit2->prev->point) != inside_sector(hit1, hit2->next->point)) {
+						PA_CHK_MARK(hit1->point[0], hit2->point[1]);
+						PA_CHK_MARK(hit2->point[0], hit2->point[1]);
+						return PA_CHK_ERROR(res, "lines is inside sector (3) at %mm;%mm or %mm;%mm", hit1->point[0], hit1->point[1], hit2->point[0], hit2->point[1]);
+					}
 				}
 			}
 		}
@@ -2909,6 +2982,12 @@ pcb_bool pcb_polyarea_contour_check(pcb_pline_t * a)
 	return pcb_false;
 }
 
+pcb_bool pcb_polyarea_contour_check(pcb_pline_t *a)
+{
+	pa_chk_res_t res;
+	return pcb_polyarea_contour_check_(a, &res);
+}
+
 void pcb_polyarea_bbox(pcb_polyarea_t * p, pcb_box_t * b)
 {
 	pcb_pline_t *n;
@@ -2933,7 +3012,7 @@ void pcb_polyarea_bbox(pcb_polyarea_t * p, pcb_box_t * b)
 }
 
 #ifndef NDEBUG
-static void pcb_poly_valid_report(pcb_pline_t *c, pcb_vnode_t *pl)
+static void pcb_poly_valid_report(pcb_pline_t *c, pcb_vnode_t *pl, pa_chk_res_t *chk)
 {
 	pcb_vnode_t *v, *n;
 	pcb_coord_t minx = COORD_MAX, miny = COORD_MAX, maxx = -COORD_MAX, maxy = -COORD_MAX;
@@ -2941,13 +3020,8 @@ static void pcb_poly_valid_report(pcb_pline_t *c, pcb_vnode_t *pl)
 #define update_minmax(min, max, val) \
 	if (val < min) min = val; \
 	if (val > max) max = val;
-
-	if (c->Flags.orient == PCB_PLF_INV)
-		DEBUGP("failed orient\n");
-	if (pcb_polyarea_contour_check(c))
-		DEBUGP("failed self-intersection\n");
-
-
+	if (chk != NULL)
+		pcb_fprintf(stderr, "Details: %s\n", chk->msg);
 	pcb_fprintf(stderr, "!!!animator start\n");
 	v = pl;
 	do {
@@ -2958,6 +3032,7 @@ static void pcb_poly_valid_report(pcb_pline_t *c, pcb_vnode_t *pl)
 		update_minmax(miny, maxy, n->point[1]);
 	}
 	while ((v = v->next) != pl);
+	pcb_fprintf(stderr, "scale 1 -1\n");
 	pcb_fprintf(stderr, "viewport %mm %mm - %mm %mm\n", minx, miny, maxx, maxy);
 	pcb_fprintf(stderr, "frame\n");
 	v = pl;
@@ -2966,8 +3041,26 @@ static void pcb_poly_valid_report(pcb_pline_t *c, pcb_vnode_t *pl)
 		pcb_fprintf(stderr, "line %#mm %#mm %#mm %#mm\n", v->point[0], v->point[1], n->point[0], n->point[1]);
 	}
 	while ((v = v->next) != pl);
-	pcb_fprintf(stderr, "flush\n");
-	pcb_fprintf(stderr, "!!!animator end\n");
+
+	if ((chk != NULL) && (chk->marks > 0)) {
+		int n, MR=PCB_MM_TO_COORD(0.05);
+		fprintf(stderr, "color #770000\n");
+		for(n = 0; n < chk->marks; n++) {
+			pcb_fprintf(stderr, "line %#mm %#mm %#mm %#mm\n", chk->x[n]-MR, chk->y[n]-MR, chk->x[n]+MR, chk->y[n]+MR);
+			pcb_fprintf(stderr, "line %#mm %#mm %#mm %#mm\n", chk->x[n]-MR, chk->y[n]+MR, chk->x[n]+MR, chk->y[n]-MR);
+		}
+	}
+
+	if ((chk != NULL) && (chk->lines > 0)) {
+		int n;
+		fprintf(stderr, "color #990000\n");
+		for(n = 0; n < chk->lines; n++)
+			pcb_fprintf(stderr, "line %#mm %#mm %#mm %#mm\n", chk->x1[n], chk->y1[n], chk->x2[n], chk->y2[n]);
+	}
+
+	fprintf(stderr, "flush\n");
+	fprintf(stderr, "!!!animator end\n");
+
 #undef update_minmax
 }
 #endif
@@ -2976,22 +3069,46 @@ static void pcb_poly_valid_report(pcb_pline_t *c, pcb_vnode_t *pl)
 pcb_bool pcb_poly_valid(pcb_polyarea_t * p)
 {
 	pcb_pline_t *c;
+	pa_chk_res_t chk;
 
 	if ((p == NULL) || (p->contours == NULL))
 		return pcb_false;
 
-	if (p->contours->Flags.orient == PCB_PLF_INV || pcb_polyarea_contour_check(p->contours)) {
+	if (p->contours->Flags.orient == PCB_PLF_INV) {
 #ifndef NDEBUG
-		DEBUGP("Invalid Outer pcb_pline_t\n");
-		pcb_poly_valid_report(p->contours, &p->contours->head);
+		pcb_fprintf(stderr, "Invalid Outer pcb_pline_t: failed orient\n");
+		pcb_poly_valid_report(p->contours, &p->contours->head, NULL);
 #endif
 		return pcb_false;
 	}
-	for (c = p->contours->next; c != NULL; c = c->next) {
-		if (c->Flags.orient == PCB_PLF_DIR || pcb_polyarea_contour_check(c) || !pcb_poly_contour_in_contour(p->contours, c)) {
+
+	if (pcb_polyarea_contour_check_(p->contours, &chk)) {
 #ifndef NDEBUG
-			DEBUGP("Invalid Inner pcb_pline_t orient = %d\n", c->Flags.orient);
-			pcb_poly_valid_report(c, &c->head);
+		pcb_fprintf(stderr, "Invalid Outer pcb_pline_t: failed contour check\n");
+		pcb_poly_valid_report(p->contours, &p->contours->head, &chk);
+#endif
+		return pcb_false;
+	}
+
+	for (c = p->contours->next; c != NULL; c = c->next) {
+		if (c->Flags.orient == PCB_PLF_DIR) {
+#ifndef NDEBUG
+			pcb_fprintf(stderr, "Invalid Inner: pcb_pline_t orient = %d\n", c->Flags.orient);
+			pcb_poly_valid_report(c, &c->head, NULL);
+#endif
+			return pcb_false;
+		}
+		if (pcb_polyarea_contour_check_(c, &chk)) {
+#ifndef NDEBUG
+			pcb_fprintf(stderr, "Invalid Inner: failed contour check\n");
+			pcb_poly_valid_report(c, &c->head, &chk);
+#endif
+			return pcb_false;
+		}
+		if (!pcb_poly_contour_in_contour(p->contours, c)) {
+#ifndef NDEBUG
+			pcb_fprintf(stderr, "Invalid Inner: overlap with outer\n");
+			pcb_poly_valid_report(c, &c->head, NULL);
 #endif
 			return pcb_false;
 		}
@@ -3204,11 +3321,12 @@ int pcb_vect_inters2(pcb_vector_t p1, pcb_vector_t p2, pcb_vector_t q1, pcb_vect
 
 /*
  * pcb_pline_isect_line()
- * (C) 2017 Tibor 'Igor2' Palinkas
+ * (C) 2017, 2018 Tibor 'Igor2' Palinkas
 */
 
 typedef struct {
 	pcb_vector_t l1, l2;
+	pcb_coord_t cx, cy;
 } pline_isect_line_t;
 
 static pcb_r_dir_t pline_isect_line_cb(const pcb_box_t * b, void *cl)
@@ -3217,13 +3335,16 @@ static pcb_r_dir_t pline_isect_line_cb(const pcb_box_t * b, void *cl)
 	struct seg *s = (struct seg *)b;
 	pcb_vector_t S1, S2;
 
-	if (pcb_vect_inters2(s->v->point, s->v->next->point, ctx->l1, ctx->l2, S1, S2))
+	if (pcb_vect_inters2(s->v->point, s->v->next->point, ctx->l1, ctx->l2, S1, S2)) {
+		ctx->cx = S1[0];
+		ctx->cy = S1[1];
 		return PCB_R_DIR_CANCEL; /* found */
+	}
 
 	return PCB_R_DIR_NOT_FOUND;
 }
 
-pcb_bool pcb_pline_isect_line(pcb_pline_t *pl, pcb_coord_t lx1, pcb_coord_t ly1, pcb_coord_t lx2, pcb_coord_t ly2)
+pcb_bool pcb_pline_isect_line(pcb_pline_t *pl, pcb_coord_t lx1, pcb_coord_t ly1, pcb_coord_t lx2, pcb_coord_t ly2, pcb_coord_t *cx, pcb_coord_t *cy)
 {
 	pline_isect_line_t ctx;
 	pcb_box_t lbx;
@@ -3237,7 +3358,12 @@ pcb_bool pcb_pline_isect_line(pcb_pline_t *pl, pcb_coord_t lx1, pcb_coord_t ly1,
 	if (pl->tree == NULL)
 		pl->tree = (pcb_rtree_t *) make_edge_tree(pl);
 
-	return pcb_r_search(pl->tree, &lbx, NULL, pline_isect_line_cb, &ctx, NULL) == PCB_R_DIR_CANCEL;
+	if (pcb_r_search(pl->tree, &lbx, NULL, pline_isect_line_cb, &ctx, NULL) == PCB_R_DIR_CANCEL) {
+		if (cx != NULL) *cx = ctx.cx;
+		if (cy != NULL) *cy = ctx.cy;
+		return pcb_true;
+	}
+	return pcb_false;
 }
 
 /*
@@ -3393,7 +3519,7 @@ void pcb_polyarea_move(pcb_polyarea_t *pa1, pcb_coord_t dx, pcb_coord_t dy)
 		}
 		if (pa->contour_tree != NULL)
 			pcb_r_destroy_tree(&pa->contour_tree);
-		pa->contour_tree = pcb_r_create_tree(NULL, 0, 0);
+		pa->contour_tree = pcb_r_create_tree();
 		for(pl = pa->contours; pl != NULL; pl = pl->next) {
 			pcb_vnode_t *v;
 			int cnt2 = 0;
@@ -3410,15 +3536,18 @@ void pcb_polyarea_move(pcb_polyarea_t *pa1, pcb_coord_t dx, pcb_coord_t dy)
 			pl->ymin += dy;
 			pl->xmax += dx;
 			pl->ymax += dy;
-			if (pl->tree != NULL)
+			if (pl->tree != NULL) {
+				pcb_r_free_tree_data(pl->tree, free);
 				pcb_r_destroy_tree(&pl->tree);
+			}
 			pl->tree = (pcb_rtree_t *)make_edge_tree(pl);
 
-			pcb_r_insert_entry(pa->contour_tree, (pcb_box_t *)pl, 0);
+			pcb_r_insert_entry(pa->contour_tree, (pcb_box_t *)pl);
 		}
 	}
 }
 
+#include "polygon_selfi.c"
 
 /* how about expanding polygons so that edges can be arcs rather than
  * lines. Consider using the third coordinate to store the radius of the
