@@ -50,6 +50,7 @@
 #include "compat_misc.h"
 #include "netlist.h"
 #include "compat_nls.h"
+#include "macro.h"
 #include <genvector/vtp0.h>
 #include "obj_rat_draw.h"
 #include "obj_term.h"
@@ -61,10 +62,6 @@
 #define TRIEDFIRST 0x1
 #define BESTFOUND 0x2
 
-/* ---------------------------------------------------------------------------
- * some forward declarations
- */
-static pcb_bool pcb_term_find_name_ppt(const char *, const char *, pcb_connection_t *, pcb_bool);
 static pcb_bool ParseConnection(const char *, char *, char *);
 static pcb_bool DrawShortestRats(pcb_netlist_t *,
 														 void (*)(register pcb_connection_t *, register pcb_connection_t *, register pcb_route_style_t *));
@@ -91,50 +88,41 @@ static pcb_layergrp_id_t rat_padstack_side(pcb_pstk_t *padstack)
 	return Sgrp; /* ultimate fallback */
 }
 
-/* ---------------------------------------------------------------------------
- * parse a connection description from a string
- * puts the element name in the string and the pin number in
+/* parse a connection description from a string
+ * puts the subcircuit name in the string and the pin number in
  * the number.  If a valid connection is found, it returns the
  * number of characters processed from the string, otherwise
- * it returns 0
- */
-static pcb_bool ParseConnection(const char *InString, char *ElementName, char *PinNum)
+ * it returns 0 */
+static pcb_bool ParseConnection(const char *InString, char *refdes, char *termid)
 {
 	int i, j;
 
 	/* copy element name portion */
 	for (j = 0; InString[j] != '\0' && InString[j] != '-'; j++)
-		ElementName[j] = InString[j];
+		refdes[j] = InString[j];
 	if (InString[j] == '-') {
-		for (i = j; i > 0 && ElementName[i - 1] >= 'a'; i--);
-		ElementName[i] = '\0';
+		for (i = j; i > 0 && refdes[i - 1] >= 'a'; i--);
+		refdes[i] = '\0';
 		for (i = 0, j++; InString[j] != '\0'; i++, j++)
-			PinNum[i] = InString[j];
-		PinNum[i] = '\0';
+			termid[i] = InString[j];
+		termid[i] = '\0';
 		return pcb_false;
 	}
 	else {
-		ElementName[j] = '\0';
-		pcb_message(PCB_MSG_ERROR, _("Bad net-list format encountered near: \"%s\"\n"), ElementName);
+		refdes[j] = '\0';
+		pcb_message(PCB_MSG_ERROR, _("Bad net-list format encountered near: \"%s\"\n"), refdes);
 		return pcb_true;
 	}
 }
 
-/* ---------------------------------------------------------------------------
- * Find a particular terminal from an element/subc name and pin/pad/terminal number
- */
-#warning term TODO: once pins and pads are gone, move this to obj_term.c
-static pcb_bool pcb_term_find_name_ppt(const char *ElementName, const char *PinNum, pcb_connection_t * conn, pcb_bool Same)
+/* Find a particular terminal from an subc name and terminal number */
+static pcb_bool pcb_term_find_name_term(const char *refdes, const char *termid, pcb_connection_t *conn, pcb_bool Same)
 {
-	pcb_element_t *element;
-	gdl_iterator_t it;
-	pcb_pad_t *pad;
-	pcb_pin_t *pin;
 	pcb_any_obj_t *obj;
 
 	/* first check for subcircuits; this is the only one thing we'll need to do
 	   once elements are removed */
-	obj = pcb_term_find_name(PCB, PCB->Data, PCB_LYT_COPPER, ElementName, PinNum, Same, (pcb_subc_t **)&conn->ptr1, &conn->group);
+	obj = pcb_term_find_name(PCB, PCB->Data, PCB_LYT_COPPER, refdes, termid, Same, (pcb_subc_t **)&conn->ptr1, &conn->group);
 	if (obj != NULL) {
 		conn->obj = obj;
 		pcb_obj_center(obj, &conn->X, &conn->Y);
@@ -144,65 +132,33 @@ static pcb_bool pcb_term_find_name_ppt(const char *ElementName, const char *PinN
 		return pcb_true;
 	}
 
-	if ((element = pcb_search_elem_by_name(PCB->Data, ElementName)) == NULL)
-		return pcb_false;
-
-	padlist_foreach(&element->Pad, &it, pad) {
-		if (PCB_NSTRCMP(PinNum, pad->Number) == 0 && (!Same || !PCB_FLAG_TEST(PCB_FLAG_DRC, pad))) {
-			conn->ptr1 = element;
-			conn->obj = (pcb_any_obj_t *)pad;
-			conn->group = PCB_FLAG_TEST(PCB_FLAG_ONSOLDER, pad) ? Sgrp : Cgrp;
-
-			if (PCB_FLAG_TEST(PCB_FLAG_EDGE2, pad)) {
-				conn->X = pad->Point2.X;
-				conn->Y = pad->Point2.Y;
-			}
-			else {
-				conn->X = pad->Point1.X;
-				conn->Y = pad->Point1.Y;
-			}
-			return pcb_true;
-		}
-	}
-
-	padlist_foreach(&element->Pin, &it, pin) {
-		if (!PCB_FLAG_TEST(PCB_FLAG_HOLE, pin) && pin->Number && PCB_NSTRCMP(PinNum, pin->Number) == 0 && (!Same || !PCB_FLAG_TEST(PCB_FLAG_DRC, pin))) {
-			conn->ptr1 = element;
-			conn->obj = (pcb_any_obj_t *)pin;
-			conn->group = Sgrp;			/* any layer will do */
-			conn->X = pin->X;
-			conn->Y = pin->Y;
-			return pcb_true;
-		}
-	}
-
 	return pcb_false;
 }
 
 pcb_bool pcb_rat_seek_pad(pcb_lib_entry_t * entry, pcb_connection_t * conn, pcb_bool Same)
 {
 	int j;
-	char ElementName[256];
-	char PinNum[256];
+	char refdes[256];
+	char termid[256];
 
-	if (ParseConnection(entry->ListEntry, ElementName, PinNum))
+	if (ParseConnection(entry->ListEntry, refdes, termid))
 		return pcb_false;
-	for (j = 0; PinNum[j] != '\0'; j++);
+	for (j = 0; termid[j] != '\0'; j++);
 	if (j == 0) {
-		pcb_message(PCB_MSG_ERROR, _("Error! Netlist file is missing pin!\n" "white space after \"%s-\"\n"), ElementName);
+		pcb_message(PCB_MSG_ERROR, _("Error! Netlist file is missing pin!\n" "white space after \"%s-\"\n"), refdes);
 		badnet = pcb_true;
 	}
 	else {
-		if (pcb_term_find_name_ppt(ElementName, PinNum, conn, Same))
+		if (pcb_term_find_name_term(refdes, termid, conn, Same))
 			return pcb_true;
 		if (Same)
 			return pcb_false;
-		if (PinNum[j - 1] < '0' || PinNum[j - 1] > '9') {
+		if (termid[j - 1] < '0' || termid[j - 1] > '9') {
 			pcb_message(PCB_MSG_WARNING, "WARNING! Pin number ending with '%c'"
-							" encountered in netlist file\n" "Probably a bad netlist file format\n", PinNum[j - 1]);
+							" encountered in netlist file\n" "Probably a bad netlist file format\n", termid[j - 1]);
 		}
 	}
-	pcb_message(PCB_MSG_WARNING, _("Can't find %s pin %s called for in netlist.\n"), ElementName, PinNum);
+	pcb_message(PCB_MSG_WARNING, _("Can't find %s pin %s called for in netlist.\n"), refdes, termid);
 	return pcb_false;
 }
 
@@ -210,7 +166,6 @@ static const char *get_refdes(void *ptr1)
 {
 	pcb_any_obj_t *obj = ptr1;
 	switch(obj->type) {
-		case PCB_OBJ_ELEMENT: return PCB_ELEM_NAME_REFDES((pcb_element_t *)ptr1);
 		case PCB_OBJ_SUBC:
 			if (((pcb_subc_t *)ptr1)->refdes != NULL)
 				return ((pcb_subc_t *)ptr1)->refdes;
@@ -221,15 +176,13 @@ static const char *get_refdes(void *ptr1)
 	return "<invalid>";
 }
 
+#warning padstack TODO: remove this bloat
 static const char *get_termid(pcb_any_obj_t *obj)
 {
-	switch(obj->type) {
-		case PCB_OBJ_PIN: return ((pcb_pin_t *)obj)->Number;
-		case PCB_OBJ_PAD: return ((pcb_pad_t *)obj)->Number;
-		default: return obj->term;
-	}
+	return obj->term;
 }
 
+#warning padstack TODO: consider using some data.h call instead of this local implementation
 static void clear_drc_flag(int clear_ratconn)
 {
 	pcb_rtree_it_t it;
@@ -237,28 +190,10 @@ static void clear_drc_flag(int clear_ratconn)
 	int li;
 	pcb_layer_t *l;
 
-	for(n = pcb_r_first(PCB->Data->pin_tree, &it); n != NULL; n = pcb_r_next(&it)) {
-		if (clear_ratconn)
-			((pcb_pin_t *)n)->ratconn = NULL;
-		PCB_FLAG_CLEAR(PCB_FLAG_DRC | PCB_FLAG_DRC_INTCONN, (pcb_pin_t *)n);
-	}
-	pcb_r_end(&it);
-
-	for(n = pcb_r_first(PCB->Data->via_tree, &it); n != NULL; n = pcb_r_next(&it))
-		PCB_FLAG_CLEAR(PCB_FLAG_DRC | PCB_FLAG_DRC_INTCONN, (pcb_pin_t *)n);
-	pcb_r_end(&it);
-
 	for(n = pcb_r_first(PCB->Data->padstack_tree, &it); n != NULL; n = pcb_r_next(&it))
 		PCB_FLAG_CLEAR(PCB_FLAG_DRC | PCB_FLAG_DRC_INTCONN, (pcb_pstk_t *)n);
 	pcb_r_end(&it);
 
-	for(n = pcb_r_first(PCB->Data->pad_tree, &it); n != NULL; n = pcb_r_next(&it)) {
-		if (clear_ratconn)
-			((pcb_pad_t *)n)->ratconn = NULL;
-		PCB_FLAG_CLEAR(PCB_FLAG_DRC | PCB_FLAG_DRC_INTCONN, (pcb_pad_t *)n);
-	}
-	pcb_r_end(&it);
-
 	for(li = 0, l = PCB->Data->Layer; li < PCB->Data->LayerN; li++,l++) {
 		for(n = pcb_r_first(l->line_tree, &it); n != NULL; n = pcb_r_next(&it))
 			PCB_FLAG_CLEAR(PCB_FLAG_DRC | PCB_FLAG_DRC_INTCONN, (pcb_line_t *)n);
@@ -305,7 +240,7 @@ pcb_netlist_t *pcb_rat_proc_netlist(pcb_lib_t *net_menu)
 				badnet = pcb_true;
 				continue;
 			}
-			net = pcb_net_new(Wantlist);
+			net = pcb_net_new(PCB, Wantlist);
 			if (menu->Style) {
 				int idx = pcb_route_style_lookup(&PCB->RouteStyle, -1, -1, -1, -1, menu->Style);
 				if (idx >= 0)
@@ -316,9 +251,9 @@ pcb_netlist_t *pcb_rat_proc_netlist(pcb_lib_t *net_menu)
 			PCB_ENTRY_LOOP(menu);
 			{
 				if (pcb_rat_seek_pad(entry, &LastPoint, pcb_false)) {
-					if (PCB_FLAG_TEST(PCB_FLAG_DRC, (pcb_pin_t *) LastPoint.obj))
-						pcb_message(PCB_MSG_ERROR, _
-										("Error! Element %s pin %s appears multiple times in the netlist file.\n"),
+					if (PCB_FLAG_TEST(PCB_FLAG_DRC, (pcb_any_obj_t *)LastPoint.obj))
+						pcb_message(PCB_MSG_ERROR,
+										"Error! Subcircuit %s terminal %s appears multiple times in the netlist file.\n",
 										get_refdes(LastPoint.ptr1), get_termid(LastPoint.obj));
 					else {
 						connection = pcb_rat_connection_alloc(net);
@@ -326,7 +261,7 @@ pcb_netlist_t *pcb_rat_proc_netlist(pcb_lib_t *net_menu)
 						/* indicate expect net */
 						connection->menu = menu;
 						/* mark as visited */
-						PCB_FLAG_SET(PCB_FLAG_DRC, (pcb_pin_t *) LastPoint.obj);
+						PCB_FLAG_SET(PCB_FLAG_DRC, (pcb_any_obj_t *)LastPoint.obj);
 						LastPoint.obj->ratconn = (void *)menu;
 					}
 				}
@@ -339,7 +274,7 @@ pcb_netlist_t *pcb_rat_proc_netlist(pcb_lib_t *net_menu)
 					/* indicate expect net */
 					connection->menu = menu;
 					/* mark as visited */
-					PCB_FLAG_SET(PCB_FLAG_DRC, (pcb_pin_t *) LastPoint.obj);
+					PCB_FLAG_SET(PCB_FLAG_DRC, (pcb_any_obj_t *)LastPoint.obj);
 					LastPoint.obj->ratconn = (void *)menu;
 				}
 			}
@@ -354,10 +289,7 @@ pcb_netlist_t *pcb_rat_proc_netlist(pcb_lib_t *net_menu)
 	return Wantlist;
 }
 
-/*
- * copy all connections from one net into another
- * and then remove the first net from its netlist
- */
+/* copy all connections from one net into another and then remove the first net from its netlist */
 static void TransferNet(pcb_netlist_t *Netl, pcb_net_t *SourceNet, pcb_net_t *DestNet)
 {
 	pcb_connection_t *conn;
@@ -413,17 +345,6 @@ static void **find_shorts_in_subc(pcb_subc_t *subc_in, vtp0_t *generic, pcb_lib_
 	if (PCB_FLAG_TEST(PCB_FLAG_NONETLIST, subc_in))
 		return menu;
 
-	PCB_VIA_LOOP(subc_in->data);
-	{
-		if (via->term == NULL)
-			continue;
-		if (PCB_FLAG_TEST(PCB_FLAG_DRC, via)) {
-			*warn = pcb_true;
-			menu = found_short((pcb_any_obj_t *)subc_in, (pcb_any_obj_t *)via, generic, theNet, menu);
-		}
-	}
-	PCB_END_LOOP;
-
 	PCB_PADSTACK_LOOP(subc_in->data);
 	{
 		if (padstack->term == NULL)
@@ -493,33 +414,12 @@ static pcb_bool CheckShorts(pcb_lib_menu_t *theNet)
 	pcb_bool warn = pcb_false;
 	vtp0_t generic;
 	/* the first connection was starting point so
-	 * the menu is always non-null
-	 */
+	 * the menu is always non-null */
 	void **menu;
 
 	vtp0_init(&generic);
 	menu = vtp0_alloc_append(&generic, 1);
 	*menu = theNet;
-	PCB_PIN_ALL_LOOP(PCB->Data);
-	{
-		pcb_element_t *e = pin->Element;
-/* TODO: should be: !PCB_FLAG_TEST(PCB_FLAG_NONETLIST, (pcb_element_t *)pin->Element)*/
-		if ((PCB_FLAG_TEST(PCB_FLAG_DRC, pin)) && (!(e->Flags.f & PCB_FLAG_NONETLIST)) && (!(e->Name->Flags.f & PCB_FLAG_NONETLIST))) {
-			warn = pcb_true;
-			menu = found_short((pcb_any_obj_t *)element, (pcb_any_obj_t *)pin, &generic, theNet, menu);
-		}
-	}
-	PCB_ENDALL_LOOP;
-	PCB_PAD_ALL_LOOP(PCB->Data);
-	{
-		pcb_element_t *e = pad->Element;
-/* TODO: should be: !PCB_FLAG_TEST(PCB_FLAG_NONETLIST, (pcb_element_t *)pad->Element)*/
-		if ((PCB_FLAG_TEST(PCB_FLAG_DRC, pad)) && (!(e->Flags.f & PCB_FLAG_NONETLIST)) && (!(e->Name->Flags.f & PCB_FLAG_NONETLIST))) {
-			warn = pcb_true;
-			menu = found_short((pcb_any_obj_t *)element, (pcb_any_obj_t *)pad, &generic, theNet, menu);
-		}
-	}
-	PCB_ENDALL_LOOP;
 	PCB_SUBC_LOOP(PCB->Data);
 	{
 		menu = find_shorts_in_subc(subc, &generic, theNet, menu, &warn);
@@ -578,20 +478,6 @@ static void gather_subnet_objs(pcb_data_t *data, pcb_netlist_t *Netl, pcb_net_t
 		}
 	}
 	PCB_ENDALL_LOOP;
-	PCB_VIA_LOOP(data);
-	{
-		if (PCB_FLAG_TEST(PCB_FLAG_DRC, via)) {
-			conn = pcb_rat_connection_alloc(a);
-			conn->X = via->X;
-			conn->Y = via->Y;
-			conn->ptr1 = via;
-			conn->obj = (pcb_any_obj_t *)via;
-			conn->group = Sgrp;
-			if PCB_FLAG_TEST(PCB_FLAG_DRC_INTCONN, via)
-				PCB_FLAG_CLEAR(PCB_FLAG_DRC | PCB_FLAG_DRC_INTCONN, via);
-		}
-	}
-	PCB_END_LOOP;
 	PCB_PADSTACK_LOOP(data);
 	{
 		if (PCB_FLAG_TEST(PCB_FLAG_DRC, padstack)) {
@@ -615,12 +501,9 @@ static void gather_subnet_objs(pcb_data_t *data, pcb_netlist_t *Netl, pcb_net_t
 	PCB_END_LOOP;
 }
 
-/* ---------------------------------------------------------------------------
- * Determine existing interconnections of the net and gather into sub-nets
- *
- * initially the netlist has each connection in its own individual net
- * afterwards there can be many fewer nets with multiple connections each
- */
+/* Determine existing interconnections of the net and gather into sub-nets.
+ * Initially the netlist has each connection in its own individual net
+ * afterwards there can be many fewer nets with multiple connections each */
 static pcb_bool GatherSubnets(pcb_netlist_t *Netl, pcb_bool NoWarn, pcb_bool AndRats)
 {
 	pcb_net_t *a, *b;
@@ -634,12 +517,12 @@ static pcb_bool GatherSubnets(pcb_netlist_t *Netl, pcb_bool NoWarn, pcb_bool And
 
 		/* now anybody connected to the first point has PCB_FLAG_DRC set */
 		/* so move those to this subnet */
-		PCB_FLAG_CLEAR(PCB_FLAG_DRC, (pcb_pin_t *) a->Connection[0].obj);
+		PCB_FLAG_CLEAR(PCB_FLAG_DRC, (pcb_any_obj_t *)a->Connection[0].obj);
 		for (n = m + 1; n < Netl->NetN; n++) {
 			b = &Netl->Net[n];
 			/* There can be only one connection in net b */
-			if (PCB_FLAG_TEST(PCB_FLAG_DRC, (pcb_pin_t *) b->Connection[0].obj)) {
-				PCB_FLAG_CLEAR(PCB_FLAG_DRC, (pcb_pin_t *) b->Connection[0].obj);
+			if (PCB_FLAG_TEST(PCB_FLAG_DRC, (pcb_any_obj_t *)b->Connection[0].obj)) {
+				PCB_FLAG_CLEAR(PCB_FLAG_DRC, (pcb_any_obj_t *)b->Connection[0].obj);
 				TransferNet(Netl, b, a);
 				/* back up since new subnet is now at old index */
 				n--;
@@ -654,16 +537,12 @@ static pcb_bool GatherSubnets(pcb_netlist_t *Netl, pcb_bool NoWarn, pcb_bool And
 	return Warned;
 }
 
-/* ---------------------------------------------------------------------------
- * Draw a rat net (tree) having the shortest lines
- * this also frees the subnet memory as they are consumed
- *
+/* Draw a rat net (tree) having the shortest lines
+ * this also frees the subnet memory as they are consumed.
  * Note that the Netl we are passed is NOT the main netlist - it's the
  * connectivity for ONE net.  It represents the CURRENT connectivity
  * state for the net, with each Netl->Net[N] representing one
- * copper-connected subset of the net.
- */
-
+ * copper-connected subset of the net. */
 static pcb_bool
 DrawShortestRats(pcb_netlist_t *Netl,
 								 void (*funcp) (register pcb_connection_t *, register pcb_connection_t *, register pcb_route_style_t *))
@@ -745,7 +624,7 @@ DrawShortestRats(pcb_netlist_t *Netl,
 					if (conn1->obj->type == PCB_OBJ_POLY &&
 							(polygon = (pcb_poly_t *) conn1->obj) &&
 							!(distance == 0 &&
-								firstpoint && firstpoint->obj->type == PCB_OBJ_VIA) && pcb_poly_is_point_in_p_ignore_holes(conn2->X, conn2->Y, polygon)) {
+								firstpoint && firstpoint->obj->type == PCB_OBJ_PSTK) && pcb_poly_is_point_in_p_ignore_holes(conn2->X, conn2->Y, polygon)) {
 						distance = 0;
 						firstpoint = conn2;
 						secondpoint = conn1;
@@ -755,7 +634,7 @@ DrawShortestRats(pcb_netlist_t *Netl,
 					else if (conn2->obj->type == PCB_OBJ_POLY &&
 									 (polygon = (pcb_poly_t *) conn2->obj) &&
 									 !(distance == 0 &&
-										 firstpoint && firstpoint->obj->type == PCB_OBJ_VIA) && pcb_poly_is_point_in_p_ignore_holes(conn1->X, conn1->Y, polygon)) {
+										 firstpoint && firstpoint->obj->type == PCB_OBJ_PSTK) && pcb_poly_is_point_in_p_ignore_holes(conn1->X, conn1->Y, polygon)) {
 						distance = 0;
 						firstpoint = conn1;
 						secondpoint = conn2;
@@ -789,7 +668,7 @@ DrawShortestRats(pcb_netlist_t *Netl,
 																 firstpoint->group, secondpoint->group, conf_core.appearance.rat_thickness, pcb_no_flags())) != NULL) {
 					if (distance == 0)
 						PCB_FLAG_SET(PCB_FLAG_VIA, line);
-					pcb_undo_add_obj_to_create(PCB_TYPE_RATLINE, line, line, line);
+					pcb_undo_add_obj_to_create(PCB_OBJ_RAT, line, line, line);
 					pcb_rat_invalidate_draw(line);
 					changed = pcb_true;
 				}
@@ -852,8 +731,8 @@ pcb_rat_add_all(pcb_bool SelectedOnly,
 	{
 		PCB_CONNECTION_LOOP(net);
 		{
-			if (!SelectedOnly || PCB_FLAG_TEST(PCB_FLAG_SELECTED, (pcb_pin_t *) connection->obj)) {
-				lonesome = pcb_net_new(Nets);
+			if (!SelectedOnly || PCB_FLAG_TEST(PCB_FLAG_SELECTED, (pcb_any_obj_t *)connection->obj)) {
+				lonesome = pcb_net_new(PCB, Nets);
 				onepin = pcb_rat_connection_alloc(lonesome);
 				*onepin = *connection;
 				lonesome->Style = net->Style;
@@ -937,8 +816,8 @@ pcb_netlist_list_t pcb_rat_collect_subnets(pcb_bool SelectedOnly)
 		Nets = pcb_netlist_new(&result);
 		PCB_CONNECTION_LOOP(net);
 		{
-			if (!SelectedOnly || PCB_FLAG_TEST(PCB_FLAG_SELECTED, (pcb_pin_t *) connection->obj)) {
-				lonesome = pcb_net_new(Nets);
+			if (!SelectedOnly || PCB_FLAG_TEST(PCB_FLAG_SELECTED, (pcb_any_obj_t *)connection->obj)) {
+				lonesome = pcb_net_new(PCB, Nets);
 				onepin = pcb_rat_connection_alloc(lonesome);
 				*onepin = *connection;
 				lonesome->Style = net->Style;
@@ -954,10 +833,7 @@ pcb_netlist_list_t pcb_rat_collect_subnets(pcb_bool SelectedOnly)
 	return result;
 }
 
-/*
- * Check to see if a particular name is the name of an already existing rats
- * line
- */
+/* Check to see if a particular name is the name of an already existing rats line */
 static int rat_used(char *name)
 {
 	if (name == NULL)
@@ -989,15 +865,15 @@ pcb_rat_t *pcb_rat_add_net(void)
 			&& pcb_crosshair.AttachedLine.Point1.Y == pcb_crosshair.AttachedLine.Point2.Y)
 		return NULL;
 
-	found = pcb_search_obj_by_location(PCB_TYPE_PAD | PCB_TYPE_PIN | PCB_TYPE_PSTK | PCB_TYPE_SUBC_PART, &ptr1, &ptr2, &ptr3,
+	found = pcb_search_obj_by_location(PCB_OBJ_PSTK | PCB_OBJ_SUBC_PART, &ptr1, &ptr2, &ptr3,
 																 pcb_crosshair.AttachedLine.Point1.X, pcb_crosshair.AttachedLine.Point1.Y, 5);
-	if (found == PCB_TYPE_NONE) {
+	if (found == PCB_OBJ_VOID) {
 		pcb_message(PCB_MSG_ERROR, _("No pad/pin under rat line\n"));
 		return NULL;
 	}
 	rd = get_refdes(ptr1);
 	if ((rd == NULL) || (*rd == 0) || (*rd == '<')) {
-		pcb_message(PCB_MSG_ERROR, _("You must name the starting element first\n"));
+		pcb_message(PCB_MSG_ERROR, _("You must name the starting subcricuit first\n"));
 		return NULL;
 	}
 
@@ -1006,20 +882,20 @@ pcb_rat_t *pcb_rat_add_net(void)
 	pcb_layergrp_list(PCB, PCB_LYT_TOP | PCB_LYT_COPPER, &Cgrp, 1);
 
 	/* will work for pins to since the FLAG is common */
-	group1 = (PCB_FLAG_TEST(PCB_FLAG_ONSOLDER, (pcb_pad_t *) ptr2) ? Sgrp : Cgrp);
+	group1 = (PCB_FLAG_TEST(PCB_FLAG_ONSOLDER, (pcb_any_obj_t *) ptr2) ? Sgrp : Cgrp);
 	strcpy(name1, pcb_connection_name(ptr2));
-	found = pcb_search_obj_by_location(PCB_TYPE_PAD | PCB_TYPE_PIN | PCB_TYPE_PSTK | PCB_TYPE_SUBC_PART, &ptr1, &ptr2, &ptr3,
+	found = pcb_search_obj_by_location(PCB_OBJ_PSTK | PCB_OBJ_SUBC_PART, &ptr1, &ptr2, &ptr3,
 																 pcb_crosshair.AttachedLine.Point2.X, pcb_crosshair.AttachedLine.Point2.Y, 5);
-	if (found == PCB_TYPE_NONE) {
+	if (found == PCB_OBJ_VOID) {
 		pcb_message(PCB_MSG_ERROR, _("No pad/pin under rat line\n"));
 		return NULL;
 	}
 	rd = get_refdes(ptr1);
 	if ((rd == NULL) || (*rd == 0) || (*rd == '<')) {
-		pcb_message(PCB_MSG_ERROR, _("You must name the ending element first\n"));
+		pcb_message(PCB_MSG_ERROR, _("You must name the ending subcircuit first\n"));
 		return NULL;
 	}
-	group2 = (PCB_FLAG_TEST(PCB_FLAG_ONSOLDER, (pcb_pad_t *) ptr2) ? Sgrp : Cgrp);
+	group2 = (PCB_FLAG_TEST(PCB_FLAG_ONSOLDER, (pcb_any_obj_t *) ptr2) ? Sgrp : Cgrp);
 	name2 = pcb_connection_name(ptr2);
 
 	menu = pcb_netnode_to_netname(name1);
@@ -1081,32 +957,20 @@ char *pcb_connection_name(pcb_any_obj_t *obj)
 	const char *num, *parent = NULL;
 	pcb_subc_t *subc;
 
-	switch (obj->type) {
-	case PCB_OBJ_PIN:
-		num = ((pcb_pin_t *)obj)->Number;
-		parent = PCB_ELEM_NAME_REFDES((pcb_element_t*)((pcb_pin_t *)obj)->Element);
-		break;
-	case PCB_OBJ_PAD:
-		num = ((pcb_pad_t *)obj)->Number;
-		parent = PCB_ELEM_NAME_REFDES((pcb_element_t*)((pcb_pad_t *)obj)->Element);
-		break;
-	default:
-		if (obj->term == NULL)
-			return NULL;
-		num = obj->term;
-		subc = pcb_obj_parent_subc(obj);
-		if (subc != NULL)
-			parent = subc->refdes;
-	}
+	if (obj->term == NULL)
+		return NULL;
+	num = obj->term;
+	subc = pcb_obj_parent_subc(obj);
+	if (subc != NULL)
+		parent = subc->refdes;
+
 	strcpy(name, PCB_UNKNOWN(parent));
 	strcat(name, "-");
 	strcat(name, PCB_UNKNOWN(num));
 	return name;
 }
 
-/* ---------------------------------------------------------------------------
- * get next slot for a connection, allocates memory if necessary
- */
+/* get next slot for a connection, allocates memory if necessary */
 pcb_connection_t *pcb_rat_connection_alloc(pcb_net_t *Net)
 {
 	pcb_connection_t *con = Net->Connection;
