@@ -25,9 +25,6 @@
  *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  *
  */
-
-/* functions used to remove vias, pins ... */
-
 #include "config.h"
 #include "conf_core.h"
 
@@ -36,18 +33,18 @@
 #include "remove.h"
 #include "select.h"
 #include "undo.h"
-#include "obj_all_op.h"
+#include "obj_arc_op.h"
+#include "obj_line_op.h"
+#include "obj_poly_op.h"
+#include "obj_text_op.h"
+#include "obj_rat_op.h"
+#include "obj_subc_op.h"
 #include "obj_pstk_op.h"
 
 static pcb_opfunc_t RemoveFunctions = {
 	pcb_lineop_remove,
 	pcb_textop_remove,
 	pcb_polyop_remove,
-	pcb_viaop_remove,
-	pcb_elemop_remove,
-	NULL,
-	NULL,
-	NULL,
 	pcb_lineop_remove_point,
 	pcb_polyop_remove_point,
 	pcb_arcop_remve,
@@ -61,11 +58,6 @@ static pcb_opfunc_t DestroyFunctions = {
 	pcb_lineop_destroy,
 	pcb_textop_destroy,
 	pcb_polyop_destroy,
-	pcb_viaop_destroy,
-	pcb_elemop_destroy,
-	NULL,
-	NULL,
-	NULL,
 	NULL,
 	pcb_polyop_destroy_point,
 	pcb_arcop_destroy,
@@ -86,7 +78,7 @@ pcb_bool pcb_remove_selected(void)
 	ctx.remove.pcb = PCB;
 	ctx.remove.destroy_target = NULL;
 
-	if (pcb_selected_operation(PCB, PCB->Data, &RemoveFunctions, &ctx, pcb_false, PCB_TYPEMASK_ALL)) {
+	if (pcb_selected_operation(PCB, PCB->Data, &RemoveFunctions, &ctx, pcb_false, PCB_OBJ_ANY & (~PCB_OBJ_SUBC_PART))) {
 		pcb_undo_inc_serial();
 		pcb_draw();
 		return pcb_true;
