@@ -51,8 +51,8 @@ static pcb_box_t SearchBox;
 static pcb_layer_t *SearchLayer;
 
 /* ---------------------------------------------------------------------------
- * The first parameter includes PCB_TYPE_LOCKED if we
- * want to include locked object in the search and PCB_TYPE_SUBC_PART if
+ * The first parameter includes PCB_OBJ_LOCKED if we
+ * want to include locked object in the search and PCB_OBJ_SUBC_PART if
  * objects that are part of a subcircuit should be found.
  */
 static pcb_bool SearchLineByLocation(unsigned long, unsigned long, pcb_layer_t **, pcb_line_t **, pcb_line_t **);
@@ -60,14 +60,9 @@ static pcb_bool SearchArcByLocation(unsigned long, unsigned long, pcb_layer_t **
 static pcb_bool SearchRatLineByLocation(unsigned long, unsigned long, pcb_rat_t **, pcb_rat_t **, pcb_rat_t **);
 static pcb_bool SearchTextByLocation(unsigned long, unsigned long, pcb_layer_t **, pcb_text_t **, pcb_text_t **);
 static pcb_bool SearchPolygonByLocation(unsigned long, unsigned long, pcb_layer_t **, pcb_poly_t **, pcb_poly_t **);
-static pcb_bool SearchPinByLocation(unsigned long, unsigned long, pcb_element_t **, pcb_pin_t **, pcb_pin_t **);
-static pcb_bool SearchPadByLocation(unsigned long, unsigned long, pcb_element_t **, pcb_pad_t **, pcb_pad_t **, pcb_bool);
-static pcb_bool SearchViaByLocation(unsigned long, unsigned long, pcb_pin_t **, pcb_pin_t **, pcb_pin_t **);
-static pcb_bool SearchElementNameByLocation(unsigned long, unsigned long, pcb_element_t **, pcb_text_t **, pcb_text_t **, pcb_bool);
 static pcb_bool SearchLinePointByLocation(unsigned long, unsigned long, pcb_layer_t **, pcb_line_t **, pcb_point_t **);
 static pcb_bool SearchArcPointByLocation(unsigned long, unsigned long, pcb_layer_t **, pcb_arc_t **, int **);
 static pcb_bool SearchPointByLocation(unsigned long, unsigned long, unsigned long, pcb_layer_t **, pcb_poly_t **, pcb_point_t **);
-static pcb_bool SearchElementByLocation(unsigned long, unsigned long, pcb_element_t **, pcb_element_t **, pcb_element_t **, pcb_bool);
 static pcb_bool SearchSubcByLocation(unsigned long, unsigned long, pcb_subc_t **, pcb_subc_t **, pcb_subc_t **, pcb_bool);
 
 /* Return not-found for subc parts and locked items unless objst says otherwise
@@ -77,60 +72,24 @@ static pcb_bool SearchSubcByLocation(unsigned long, unsigned long, pcb_subc_t **
 do { \
 	if ((req_flag != 0) && (!PCB_FLAG_TEST(req_flag, obj))) \
 		reject; \
-	if (!(objst & PCB_TYPE_SUBC_PART) && (pcb_ ## locality ## obj_parent_subc(obj->parent_type, &obj->parent))) \
+	if (!(objst & PCB_OBJ_SUBC_PART) && (pcb_ ## locality ## obj_parent_subc(obj->parent_type, &obj->parent))) \
 		reject; \
-	if (!(objst & PCB_TYPE_LOCKED) && (PCB_FLAG_TEST(objst & PCB_FLAG_LOCK, locked_obj))) \
+	if (!(objst & PCB_OBJ_LOCKED) && (PCB_FLAG_TEST(objst & PCB_FLAG_LOCK, locked_obj))) \
 		reject; \
 } while(0)
 
 #define TEST_OBJST(objst, req_flag, locality, obj, locked_obj) \
 	TEST_OBJST_(objst, req_flag, locality, obj, locked_obj, return PCB_R_DIR_NOT_FOUND)
 
-/* ---------------------------------------------------------------------------
- * searches a via
- */
 struct ans_info {
 	void **ptr1, **ptr2, **ptr3;
 	pcb_bool BackToo;
 	double area;
 	unsigned long objst, req_flag;
 	int on_current;
+	pcb_layer_type_t on_lyt;
 };
 
-static pcb_r_dir_t pinorvia_callback(const pcb_box_t * box, void *cl)
-{
-	struct ans_info *i = (struct ans_info *) cl;
-	pcb_pin_t *pin = (pcb_pin_t *) box;
-	pcb_any_obj_t *ptr1 = pin->Element ? pin->Element : pin;
-
-	TEST_OBJST(i->objst, i->req_flag, g, pin, ptr1);
-
-	if (!pcb_is_point_in_pin(PosX, PosY, SearchRadius, pin))
-		return PCB_R_DIR_NOT_FOUND;
-	*i->ptr1 = ptr1;
-	*i->ptr2 = *i->ptr3 = pin;
-	return PCB_R_DIR_CANCEL; /* found, stop searching */
-}
-
-static pcb_bool SearchViaByLocation(unsigned long objst, unsigned long req_flag, pcb_pin_t ** Via, pcb_pin_t ** Dummy1, pcb_pin_t ** Dummy2)
-{
-	struct ans_info info;
-
-	/* search only if via-layer is visible */
-	if (!PCB->ViaOn)
-		return pcb_false;
-
-	info.ptr1 = (void **) Via;
-	info.ptr2 = (void **) Dummy1;
-	info.ptr3 = (void **) Dummy2;
-	info.objst = objst;
-	info.req_flag = req_flag;
-
-	if (pcb_r_search(PCB->Data->via_tree, &SearchBox, NULL, pinorvia_callback, &info, NULL) != PCB_R_DIR_NOT_FOUND)
-		return pcb_true;
-	return pcb_false;
-}
-
 /* ---------------------------------------------------------------------------
  * searches a padstack
  */
@@ -144,7 +103,9 @@ static pcb_r_dir_t padstack_callback(const pcb_box_t *box, void *cl)
 
 	if (!pcb_is_point_in_pstk(PosX, PosY, SearchRadius, ps, NULL))
 		return PCB_R_DIR_NOT_FOUND;
-	if ((i->on_current) && (pcb_pstk_shape_at(PCB, ps, CURRENT) == NULL))
+	if ((i->on_current) && (pcb_pstk_shape_at(PCB, ps, CURRENT) == NULL) && (!pcb_pstk_bb_drills(PCB, ps, CURRENT->meta.real.grp, NULL)))
+		return PCB_R_DIR_NOT_FOUND;
+	if ((i->on_lyt != 0) && (pcb_pstk_shape(ps, i->on_lyt, 0) == NULL))
 		return PCB_R_DIR_NOT_FOUND;
 	*i->ptr2 = *i->ptr3 = ps;
 	assert(ps->parent_type == PCB_PARENT_DATA);
@@ -156,12 +117,12 @@ static pcb_r_dir_t padstack_callback(const pcb_box_t *box, void *cl)
 	return PCB_R_DIR_CANCEL; /* found, stop searching */
 }
 
-static pcb_bool SearchPadstackByLocation(unsigned long objst, unsigned long req_flag, pcb_pstk_t **ps, pcb_pstk_t **Dummy1, pcb_pstk_t **Dummy2, int on_current)
+static pcb_bool SearchPadstackByLocation(unsigned long objst, unsigned long req_flag, pcb_pstk_t **ps, pcb_pstk_t **Dummy1, pcb_pstk_t **Dummy2, int on_current, pcb_layer_type_t on_lyt)
 {
 	struct ans_info info;
 
 	/* search only if via-layer is visible */
-	if (!PCB->ViaOn)
+	if (!PCB->pstk_on)
 		return pcb_false;
 
 	info.ptr1 = (void **)ps;
@@ -170,74 +131,13 @@ static pcb_bool SearchPadstackByLocation(unsigned long objst, unsigned long req_
 	info.objst = objst;
 	info.req_flag = req_flag;
 	info.on_current = on_current;
+	info.on_lyt = on_lyt;
 
 	if (pcb_r_search(PCB->Data->padstack_tree, &SearchBox, NULL, padstack_callback, &info, NULL) != PCB_R_DIR_NOT_FOUND)
 		return pcb_true;
 	return pcb_false;
 }
 
-/* ---------------------------------------------------------------------------
- * searches a pin
- * starts with the newest element
- */
-static pcb_bool SearchPinByLocation(unsigned long objst, unsigned long req_flag, pcb_element_t ** Element, pcb_pin_t ** Pin, pcb_pin_t ** Dummy)
-{
-	struct ans_info info;
-
-	/* search only if pin-layer is visible */
-	if (!PCB->PinOn)
-		return pcb_false;
-	info.ptr1 = (void **) Element;
-	info.ptr2 = (void **) Pin;
-	info.ptr3 = (void **) Dummy;
-	info.objst = objst;
-	info.req_flag = req_flag;
-
-	if (pcb_r_search(PCB->Data->pin_tree, &SearchBox, NULL, pinorvia_callback, &info, NULL)  != PCB_R_DIR_NOT_FOUND)
-		return pcb_true;
-	return pcb_false;
-}
-
-static pcb_r_dir_t pad_callback(const pcb_box_t * b, void *cl)
-{
-	pcb_pad_t *pad = (pcb_pad_t *) b;
-	struct ans_info *i = (struct ans_info *) cl;
-	pcb_any_obj_t *ptr1 = pad->Element;
-
-	TEST_OBJST(i->objst, i->req_flag, g, pad, ptr1);
-
-	if (PCB_FRONT(pad) || i->BackToo) {
-		if (pcb_is_point_in_pad(PosX, PosY, SearchRadius, pad)) {
-			*i->ptr1 = ptr1;
-			*i->ptr2 = *i->ptr3 = pad;
-			return PCB_R_DIR_CANCEL; /* found */
-		}
-	}
-	return PCB_R_DIR_NOT_FOUND;
-}
-
-/* ---------------------------------------------------------------------------
- * searches a pad
- * starts with the newest element
- */
-static pcb_bool SearchPadByLocation(unsigned long objst, unsigned long req_flag, pcb_element_t ** Element, pcb_pad_t ** Pad, pcb_pad_t ** Dummy, pcb_bool BackToo)
-{
-	struct ans_info info;
-
-	/* search only if pin-layer is visible */
-	if (!PCB->PinOn)
-		return pcb_false;
-	info.ptr1 = (void **) Element;
-	info.ptr2 = (void **) Pad;
-	info.ptr3 = (void **) Dummy;
-	info.objst = objst;
-	info.req_flag = req_flag;
-	info.BackToo = (BackToo && PCB->InvisibleObjectsOn);
-	if (pcb_r_search(PCB->Data->pad_tree, &SearchBox, NULL, pad_callback, &info, NULL) != PCB_R_DIR_NOT_FOUND)
-		return pcb_true;
-	return pcb_false;
-}
-
 /* ---------------------------------------------------------------------------
  * searches ordinary line on the SearchLayer
  */
@@ -257,7 +157,7 @@ static pcb_r_dir_t line_callback(const pcb_box_t * box, void *cl)
 
 	TEST_OBJST(i->objst, i->req_flag, l, l, l);
 
-	if (!pcb_is_point_in_pad(PosX, PosY, SearchRadius, (pcb_pad_t *) l))
+	if (!pcb_is_point_in_line(PosX, PosY, SearchRadius, (pcb_any_line_t *)l))
 		return PCB_R_DIR_NOT_FOUND;
 	*i->Line = l;
 	*i->Point = (pcb_point_t *) l;
@@ -540,10 +440,10 @@ static pcb_r_dir_t polypoint_callback(const pcb_box_t *box, void *cl)
 	double d;
 	pcb_data_t *dt;
 
-	dt = polygon->parent.layer->parent; /* polygon -> layer -> data */
+	dt = polygon->parent.layer->parent.data; /* polygon -> layer -> data */
 	if ((dt != NULL) && (dt->parent_type == PCB_PARENT_SUBC)) {
 		/* do not find subc part poly points if not explicitly requested */
-		if (!(ctx->Type & PCB_TYPE_SUBC_PART))
+		if (!(ctx->Type & PCB_OBJ_SUBC_PART))
 			return PCB_R_DIR_NOT_FOUND;
 
 		/* don't find subc poly points even as subc part unless we are editing a subc */
@@ -584,107 +484,21 @@ static pcb_bool SearchPointByLocation(unsigned long Type, unsigned long objst, u
 	return pcb_false;
 }
 
-static pcb_r_dir_t name_callback(const pcb_box_t * box, void *cl)
-{
-	pcb_text_t *text = (pcb_text_t *) box;
-	struct ans_info *i = (struct ans_info *) cl;
-	pcb_element_t *element = (pcb_element_t *) text->Element;
-	double newarea;
-
-	TEST_OBJST(i->objst, i->req_flag, l, text, text);
-
-	if ((PCB_FRONT(element) || i->BackToo) && !PCB_FLAG_TEST(PCB_FLAG_HIDENAME, element) && PCB_POINT_IN_BOX(PosX, PosY, &text->BoundingBox)) {
-		/* use the text with the smallest bounding box */
-		newarea = (text->BoundingBox.X2 - text->BoundingBox.X1) * (double) (text->BoundingBox.Y2 - text->BoundingBox.Y1);
-		if (newarea < i->area) {
-			i->area = newarea;
-			*i->ptr1 = element;
-			*i->ptr2 = *i->ptr3 = text;
-		}
-		return PCB_R_DIR_FOUND_CONTINUE;
-	}
-	return PCB_R_DIR_NOT_FOUND;
-}
-
-/* ---------------------------------------------------------------------------
- * searches the name of an element
- */
-static pcb_bool
-SearchElementNameByLocation(unsigned long objst, unsigned long req_flag, pcb_element_t ** Element, pcb_text_t ** Text, pcb_text_t ** Dummy, pcb_bool BackToo)
-{
-	struct ans_info info;
-
-	/* package layer have to be switched on */
-	if (pcb_silk_on(PCB)) {
-		info.ptr1 = (void **) Element;
-		info.ptr2 = (void **) Text;
-		info.ptr3 = (void **) Dummy;
-		info.area = PCB_SQUARE(PCB_MAX_COORD);
-		info.BackToo = (BackToo && PCB->InvisibleObjectsOn);
-		info.objst = objst;
-		info.req_flag = req_flag;
-
-		if (pcb_r_search(PCB->Data->name_tree[PCB_ELEMNAME_IDX_VISIBLE()], &SearchBox, NULL, name_callback, &info, NULL))
-			return pcb_true;
-	}
-	return pcb_false;
-}
-
-static pcb_r_dir_t element_callback(const pcb_box_t * box, void *cl)
-{
-	pcb_element_t *element = (pcb_element_t *) box;
-	struct ans_info *i = (struct ans_info *) cl;
-	double newarea;
-
-	TEST_OBJST(i->objst, i->req_flag, g, element, element);
-
-	if ((PCB_FRONT(element) || i->BackToo) && PCB_POINT_IN_BOX(PosX, PosY, &element->VBox) && pcb_element_silk_vis(element)) {
-		/* use the element with the smallest bounding box */
-		newarea = (element->VBox.X2 - element->VBox.X1) * (double) (element->VBox.Y2 - element->VBox.Y1);
-		if (newarea < i->area) {
-			i->area = newarea;
-			*i->ptr1 = *i->ptr2 = *i->ptr3 = element;
-			return PCB_R_DIR_FOUND_CONTINUE;
-		}
-	}
-	return PCB_R_DIR_NOT_FOUND;
-}
-
-/* ---------------------------------------------------------------------------
- * searches an element
- * if more than one element matches, the smallest one is taken
- */
-static pcb_bool
-SearchElementByLocation(unsigned long objst, unsigned long req_flag, pcb_element_t ** Element, pcb_element_t ** Dummy1, pcb_element_t ** Dummy2, pcb_bool BackToo)
-{
-	struct ans_info info;
-
-	/* Both package layers have to be switched on */
-	if (pcb_silk_on(PCB) && PCB->PinOn) {
-		info.ptr1 = (void **) Element;
-		info.ptr2 = (void **) Dummy1;
-		info.ptr3 = (void **) Dummy2;
-		info.area = PCB_SQUARE(PCB_MAX_COORD);
-		info.BackToo = (BackToo && PCB->InvisibleObjectsOn);
-		info.objst = objst;
-		info.req_flag = req_flag;
-
-		if (pcb_r_search(PCB->Data->element_tree, &SearchBox, NULL, element_callback, &info, NULL))
-			return pcb_true;
-	}
-	return pcb_false;
-}
 
 static pcb_r_dir_t subc_callback(const pcb_box_t *box, void *cl)
 {
 	pcb_subc_t *subc = (pcb_subc_t *) box;
 	struct ans_info *i = (struct ans_info *) cl;
 	double newarea;
+	int subc_on_bottom = 0, front;
+
+	pcb_subc_get_side(subc, &subc_on_bottom);
+	front = subc_on_bottom == conf_core.editor.show_solder_side;
 
 	TEST_OBJST(i->objst, i->req_flag, g, subc, subc);
 
-	if ((PCB_FRONT(subc) || i->BackToo) && PCB_POINT_IN_BOX(PosX, PosY, &subc->BoundingBox)) {
-		/* use the element with the smallest bounding box */
+	if ((front || i->BackToo) && PCB_POINT_IN_BOX(PosX, PosY, &subc->BoundingBox)) {
+		/* use the subcircuit with the smallest bounding box */
 		newarea = (subc->BoundingBox.X2 - subc->BoundingBox.X1) * (double) (subc->BoundingBox.Y2 - subc->BoundingBox.Y1);
 		if (newarea < i->area) {
 			i->area = newarea;
@@ -766,25 +580,6 @@ static pcb_bool SearchSubcFloaterByLocation(unsigned long objst, unsigned long r
 	return pcb_false;
 }
 
-
-pcb_bool pcb_is_point_in_pin(pcb_coord_t X, pcb_coord_t Y, pcb_coord_t Radius, pcb_pin_t *pin)
-{
-	pcb_coord_t t = PIN_SIZE(pin) / 2;
-	if (PCB_FLAG_TEST(PCB_FLAG_SQUARE, pin)) {
-		pcb_box_t b;
-
-		b.X1 = pin->X - t;
-		b.X2 = pin->X + t;
-		b.Y1 = pin->Y - t;
-		b.Y2 = pin->Y + t;
-		if (pcb_is_point_in_box(X, Y, &b, Radius))
-			return pcb_true;
-	}
-	else if (pcb_distance(pin->X, pin->Y, X, Y) <= Radius + t)
-		return pcb_true;
-	return pcb_false;
-}
-
 /* for checking if a rat-line end is on a PV */
 pcb_bool pcb_is_point_on_line_end(pcb_coord_t X, pcb_coord_t Y, pcb_rat_t *Line)
 {
@@ -1060,10 +855,10 @@ pcb_bool pcb_is_arc_in_rectangle(pcb_coord_t X1, pcb_coord_t Y1, pcb_coord_t X2,
 }
 
 /* ---------------------------------------------------------------------------
- * Check if a circle of Radius with center at (X, Y) intersects a Pad.
- * Written to enable arbitrary pad directions; for rounded pads, too.
+ * Check if a circle of Radius with center at (X, Y) intersects a line.
+ * Written to enable arbitrary line directions; for rounded/square lines, too.
  */
-pcb_bool pcb_is_point_in_pad(pcb_coord_t X, pcb_coord_t Y, pcb_coord_t Radius, pcb_pad_t *Pad)
+pcb_bool pcb_is_point_in_line(pcb_coord_t X, pcb_coord_t Y, pcb_coord_t Radius, pcb_any_line_t *Pad)
 {
 	double r, Sin, Cos;
 	pcb_coord_t x;
@@ -1071,7 +866,7 @@ pcb_bool pcb_is_point_in_pad(pcb_coord_t X, pcb_coord_t Y, pcb_coord_t Radius, p
 	/* Also used from line_callback with line type smaller than pad type;
 	   use the smallest common subset; ->Thickness is still ok. */
 	pcb_coord_t t2 = (Pad->Thickness + 1) / 2, range;
-	pcb_any_line_t pad = *(pcb_any_line_t *) Pad;
+	pcb_any_line_t pad = *Pad;
 
 
 	/* series of transforms saving range */
@@ -1278,22 +1073,34 @@ pcb_line_t *pcb_line_center_cross_point(pcb_layer_t *layer, pcb_coord_t x, pcb_c
 	return NULL; /* found nothing */
 }
 
+pcb_layer_type_t pstk_vis_layers(pcb_board_t *pcb, pcb_layer_type_t material)
+{
+	pcb_layer_type_t res = 0, lyt;
+	pcb_layer_id_t lid;
+	for(lid = 0; lid < pcb->Data->LayerN; lid++) {
+		pcb_layer_t *ly = &pcb->Data->Layer[lid];
+		lyt = pcb_layer_flags_(ly);
+		if ((ly->meta.real.vis) && (lyt & material))
+			res |= (lyt & PCB_LYT_ANYWHERE);
+	}
+	return res | material;
+}
 
 /* ---------------------------------------------------------------------------
  * searches for any kind of object or for a set of object types
  * the calling routine passes two pointers to allocated memory for storing
  * the results.
- * A type value is returned too which is PCB_TYPE_NONE if no objects has been found.
+ * A type value is returned too which is PCB_OBJ_VOID if no objects has been found.
  * A set of object types is passed in.
  * The object is located by it's position.
  *
  * The layout is checked in the following order:
- *   polygon-point, pin, via, line, text, elementname, polygon, element
+ *   polygon-point, padstack, line, text, polygon, subcircuit
  *
- * Note that if Type includes PCB_TYPE_LOCKED, then the search includes
+ * Note that if Type includes PCB_OBJ_LOCKED, then the search includes
  * locked items.  Otherwise, locked items are ignored.
 
- * Note that if Type includes PCB_TYPE_SUBC_PART, then the search includes
+ * Note that if Type includes PCB_OBJ_SUBC_PART, then the search includes
  * objects that are part of subcircuits, else they are ignored.
  */
 static int pcb_search_obj_by_location_(unsigned long Type, void **Result1, void **Result2, void **Result3, pcb_coord_t X, pcb_coord_t Y, pcb_coord_t Radius, unsigned long req_flag)
@@ -1302,8 +1109,8 @@ static int pcb_search_obj_by_location_(unsigned long Type, void **Result1, void
 	void **pr1 = &r1, **pr2 = &r2, **pr3 = &r3;
 	int i;
 	double HigherBound = 0;
-	int HigherAvail = PCB_TYPE_NONE;
-	int objst = Type & (PCB_TYPE_LOCKED | PCB_TYPE_SUBC_PART);
+	int HigherAvail = PCB_OBJ_VOID;
+	int objst = Type & (PCB_OBJ_LOCKED | PCB_OBJ_SUBC_PART);
 
 	/* setup variables used by local functions */
 	PosX = X;
@@ -1319,66 +1126,49 @@ static int pcb_search_obj_by_location_(unsigned long Type, void **Result1, void
 		SearchBox = pcb_point_box(X, Y);
 	}
 
-	if (conf_core.editor.lock_names) {
-		Type &= ~(PCB_TYPE_ELEMENT_NAME | PCB_TYPE_TEXT);
-	}
-	if (conf_core.editor.hide_names) {
-		Type &= ~PCB_TYPE_ELEMENT_NAME;
-	}
 	if (conf_core.editor.only_names) {
-		Type &= (PCB_TYPE_ELEMENT_NAME | PCB_TYPE_TEXT);
+		Type &= PCB_OBJ_TEXT;
 	}
+
 	if (conf_core.editor.thin_draw || conf_core.editor.thin_draw_poly) {
-		Type &= ~PCB_TYPE_POLY;
+		Type &= ~PCB_OBJ_POLY;
 	}
 
-	if (Type & PCB_TYPE_RATLINE && PCB->RatOn &&
+	if (Type & PCB_OBJ_RAT && PCB->RatOn &&
 			SearchRatLineByLocation(objst, req_flag, (pcb_rat_t **) Result1, (pcb_rat_t **) Result2, (pcb_rat_t **) Result3))
-		return PCB_TYPE_RATLINE;
+		return PCB_OBJ_RAT;
 
-	if (Type & PCB_TYPE_VIA && SearchViaByLocation(objst, req_flag, (pcb_pin_t **) Result1, (pcb_pin_t **) Result2, (pcb_pin_t **) Result3))
-		return PCB_TYPE_VIA;
+	if (Type & PCB_OBJ_PSTK && SearchPadstackByLocation(objst, req_flag, (pcb_pstk_t **) Result1, (pcb_pstk_t **) Result2, (pcb_pstk_t **) Result3, 1, 0))
+		return PCB_OBJ_PSTK;
 
-	if (Type & PCB_TYPE_PSTK && SearchPadstackByLocation(objst, req_flag, (pcb_pstk_t **) Result1, (pcb_pstk_t **) Result2, (pcb_pstk_t **) Result3, 1))
-		return PCB_TYPE_PSTK;
+	if (Type & PCB_OBJ_PSTK && SearchPadstackByLocation(objst, req_flag, (pcb_pstk_t **) Result1, (pcb_pstk_t **) Result2, (pcb_pstk_t **) Result3, 0, PCB_LYT_VISIBLE_SIDE() | PCB_LYT_COPPER))
+		return PCB_OBJ_PSTK;
 
-	if (Type & PCB_TYPE_PSTK && SearchPadstackByLocation(objst, req_flag, (pcb_pstk_t **) Result1, (pcb_pstk_t **) Result2, (pcb_pstk_t **) Result3, 0))
-		return PCB_TYPE_PSTK;
+	if (Type & PCB_OBJ_PSTK && SearchPadstackByLocation(objst, req_flag, (pcb_pstk_t **) Result1, (pcb_pstk_t **) Result2, (pcb_pstk_t **) Result3, 0, (PCB_LYT_VISIBLE_SIDE() | PCB_LYT_ANYTHING) & pstk_vis_layers(PCB, PCB_LYT_ANYTHING)))
+		return PCB_OBJ_PSTK;
 
-	if (Type & PCB_TYPE_PIN && SearchPinByLocation(objst, req_flag, (pcb_element_t **) pr1, (pcb_pin_t **) pr2, (pcb_pin_t **) pr3))
-		HigherAvail = PCB_TYPE_PIN;
+	if (Type & PCB_OBJ_PSTK && SearchPadstackByLocation(objst, req_flag, (pcb_pstk_t **) Result1, (pcb_pstk_t **) Result2, (pcb_pstk_t **) Result3, 0, pstk_vis_layers(PCB, PCB_LYT_MASK)))
+		return PCB_OBJ_PSTK;
 
-	if (!HigherAvail && Type & PCB_TYPE_PAD &&
-			SearchPadByLocation(objst, req_flag, (pcb_element_t **) pr1, (pcb_pad_t **) pr2, (pcb_pad_t **) pr3, pcb_false))
-		HigherAvail = PCB_TYPE_PAD;
+	if (Type & PCB_OBJ_PSTK && SearchPadstackByLocation(objst, req_flag, (pcb_pstk_t **) Result1, (pcb_pstk_t **) Result2, (pcb_pstk_t **) Result3, 0, pstk_vis_layers(PCB, PCB_LYT_COPPER)))
+		return PCB_OBJ_PSTK;
 
-	if (!HigherAvail && Type & PCB_TYPE_ELEMENT_NAME &&
-			SearchElementNameByLocation(objst, req_flag, (pcb_element_t **) pr1, (pcb_text_t **) pr2, (pcb_text_t **) pr3, pcb_false)) {
-		pcb_box_t *box = &((pcb_text_t *) r2)->BoundingBox;
-		HigherBound = (double) (box->X2 - box->X1) * (double) (box->Y2 - box->Y1);
-		HigherAvail = PCB_TYPE_ELEMENT_NAME;
-	}
+	if (Type & PCB_OBJ_PSTK && SearchPadstackByLocation(objst, req_flag, (pcb_pstk_t **) Result1, (pcb_pstk_t **) Result2, (pcb_pstk_t **) Result3, 0, pstk_vis_layers(PCB, PCB_LYT_PASTE)))
+		return PCB_OBJ_PSTK;
 
-	if (!HigherAvail && (Type & PCB_TYPE_SUBC_FLOATER) && (Type & PCB_TYPE_TEXT) &&
+	if (!HigherAvail && (Type & PCB_OBJ_FLOATER) && (Type & PCB_OBJ_TEXT) &&
 			SearchSubcFloaterByLocation(objst, req_flag, (pcb_subc_t **)pr1, (pcb_text_t **) pr2, pr3, pcb_false)) {
 		*Result1 = ((pcb_text_t *)r2)->parent.layer;
 		*Result2 = r2;
 		*Result3 = r3;
-		return PCB_TYPE_TEXT;
+		return PCB_OBJ_TEXT;
 	}
 
-	if (!HigherAvail && Type & PCB_TYPE_ELEMENT &&
-			SearchElementByLocation(objst, req_flag, (pcb_element_t **) pr1, (pcb_element_t **) pr2, (pcb_element_t **) pr3, pcb_false)) {
-		pcb_box_t *box = &((pcb_element_t *) r1)->BoundingBox;
-		HigherBound = (double) (box->X2 - box->X1) * (double) (box->Y2 - box->Y1);
-		HigherAvail = PCB_TYPE_ELEMENT;
-	}
-
-	if (!HigherAvail && Type & PCB_TYPE_SUBC && PCB->SubcOn &&
+	if (!HigherAvail && Type & PCB_OBJ_SUBC && PCB->SubcOn &&
 			SearchSubcByLocation(objst, req_flag, (pcb_subc_t **) pr1, (pcb_subc_t **) pr2, (pcb_subc_t **) pr3, pcb_false)) {
 		pcb_box_t *box = &((pcb_subc_t *) r1)->BoundingBox;
 		HigherBound = (double) (box->X2 - box->X1) * (double) (box->Y2 - box->Y1);
-		HigherAvail = PCB_TYPE_SUBC;
+		HigherAvail = PCB_OBJ_SUBC;
 	}
 
 	for (i = -1; i < pcb_max_layer + 1; i++) {
@@ -1395,34 +1185,34 @@ static int pcb_search_obj_by_location_(unsigned long Type, void **Result1, void
 				continue;
 		}
 		if (SearchLayer->meta.real.vis) {
-			if ((HigherAvail & (PCB_TYPE_PIN | PCB_TYPE_PAD)) == 0 &&
-					Type & PCB_TYPE_POLY_POINT &&
+			if ((HigherAvail & (PCB_OBJ_PSTK)) == 0 &&
+					Type & PCB_OBJ_POLY_POINT &&
 					SearchPointByLocation(Type, objst, req_flag, (pcb_layer_t **) Result1, (pcb_poly_t **) Result2, (pcb_point_t **) Result3))
-				return PCB_TYPE_POLY_POINT;
+				return PCB_OBJ_POLY_POINT;
 
-			if ((HigherAvail & (PCB_TYPE_PIN | PCB_TYPE_PAD)) == 0 &&
-					Type & PCB_TYPE_LINE_POINT &&
+			if ((HigherAvail & (PCB_OBJ_PSTK)) == 0 &&
+					Type & PCB_OBJ_LINE_POINT &&
 					SearchLinePointByLocation(objst, req_flag, (pcb_layer_t **) Result1, (pcb_line_t **) Result2, (pcb_point_t **) Result3))
-				return PCB_TYPE_LINE_POINT;
+				return PCB_OBJ_LINE_POINT;
 
-			if ((HigherAvail & (PCB_TYPE_PIN | PCB_TYPE_PAD)) == 0 &&
-					Type & PCB_TYPE_ARC_POINT &&
+			if ((HigherAvail & (PCB_OBJ_PSTK)) == 0 &&
+					Type & PCB_OBJ_ARC_POINT &&
 					SearchArcPointByLocation(objst, req_flag, (pcb_layer_t **) Result1, (pcb_arc_t **) Result2, (int **) Result3))
-				return PCB_TYPE_ARC_POINT;
+				return PCB_OBJ_ARC_POINT;
 
-			if ((HigherAvail & (PCB_TYPE_PIN | PCB_TYPE_PAD)) == 0 && Type & PCB_TYPE_LINE
+			if ((HigherAvail & (PCB_OBJ_PSTK)) == 0 && Type & PCB_OBJ_LINE
 					&& SearchLineByLocation(objst, req_flag, (pcb_layer_t **) Result1, (pcb_line_t **) Result2, (pcb_line_t **) Result3))
-				return PCB_TYPE_LINE;
+				return PCB_OBJ_LINE;
 
-			if ((HigherAvail & (PCB_TYPE_PIN | PCB_TYPE_PAD)) == 0 && Type & PCB_TYPE_ARC &&
+			if ((HigherAvail & (PCB_OBJ_PSTK)) == 0 && Type & PCB_OBJ_ARC &&
 					SearchArcByLocation(objst, req_flag, (pcb_layer_t **) Result1, (pcb_arc_t **) Result2, (pcb_arc_t **) Result3))
-				return PCB_TYPE_ARC;
+				return PCB_OBJ_ARC;
 
-			if ((HigherAvail & (PCB_TYPE_PIN | PCB_TYPE_PAD)) == 0 && Type & PCB_TYPE_TEXT
+			if ((HigherAvail & (PCB_OBJ_PSTK)) == 0 && Type & PCB_OBJ_TEXT
 					&& SearchTextByLocation(objst, req_flag, (pcb_layer_t **) Result1, (pcb_text_t **) Result2, (pcb_text_t **) Result3))
-				return PCB_TYPE_TEXT;
+				return PCB_OBJ_TEXT;
 
-			if (Type & PCB_TYPE_POLY &&
+			if (Type & PCB_OBJ_POLY &&
 					SearchPolygonByLocation(objst, req_flag, (pcb_layer_t **) Result1, (pcb_poly_t **) Result2, (pcb_poly_t **) Result3)) {
 				if (HigherAvail) {
 					pcb_box_t *box = &(*(pcb_poly_t **) Result2)->BoundingBox;
@@ -1430,84 +1220,56 @@ static int pcb_search_obj_by_location_(unsigned long Type, void **Result1, void
 					if (HigherBound < area)
 						break;
 					else
-						return PCB_TYPE_POLY;
+						return PCB_OBJ_POLY;
 				}
 				else
-					return PCB_TYPE_POLY;
+					return PCB_OBJ_POLY;
 			}
 		}
 	}
 	/* return any previously found objects */
-	if (HigherAvail & PCB_TYPE_PIN) {
+	if (HigherAvail & PCB_OBJ_PSTK) {
 		*Result1 = r1;
 		*Result2 = r2;
 		*Result3 = r3;
-		return PCB_TYPE_PIN;
+		return PCB_OBJ_PSTK;
 	}
 
-	if (HigherAvail & PCB_TYPE_PAD) {
+	if (HigherAvail & PCB_OBJ_SUBC) {
 		*Result1 = r1;
 		*Result2 = r2;
 		*Result3 = r3;
-		return PCB_TYPE_PAD;
-	}
-
-	if (HigherAvail & PCB_TYPE_ELEMENT_NAME) {
-		*Result1 = r1;
-		*Result2 = r2;
-		*Result3 = r3;
-		return PCB_TYPE_ELEMENT_NAME;
-	}
-
-	if (HigherAvail & PCB_TYPE_ELEMENT) {
-		*Result1 = r1;
-		*Result2 = r2;
-		*Result3 = r3;
-		return PCB_TYPE_ELEMENT;
-	}
-
-	if (HigherAvail & PCB_TYPE_SUBC) {
-		*Result1 = r1;
-		*Result2 = r2;
-		*Result3 = r3;
-		return PCB_TYPE_SUBC;
+		return PCB_OBJ_SUBC;
 	}
 
 	/* search the 'invisible objects' last */
 	if (!PCB->InvisibleObjectsOn)
-		return PCB_TYPE_NONE;
+		return PCB_OBJ_VOID;
 
-	if (Type & PCB_TYPE_PAD &&
-			SearchPadByLocation(objst, req_flag, (pcb_element_t **) Result1, (pcb_pad_t **) Result2, (pcb_pad_t **) Result3, pcb_true))
-		return PCB_TYPE_PAD;
-
-	if (Type & PCB_TYPE_ELEMENT_NAME &&
-			SearchElementNameByLocation(objst, req_flag, (pcb_element_t **) Result1, (pcb_text_t **) Result2, (pcb_text_t **) Result3, pcb_true))
-		return PCB_TYPE_ELEMENT_NAME;
-
-	if ((Type & PCB_TYPE_SUBC_FLOATER) && (Type & PCB_TYPE_TEXT) &&
+	if ((Type & PCB_OBJ_FLOATER) && (Type & PCB_OBJ_TEXT) &&
 			SearchSubcFloaterByLocation(objst, req_flag, (pcb_subc_t **)pr1, (pcb_text_t **) pr2, pr3, pcb_true)) {
 		*Result1 = ((pcb_text_t *)r2)->parent.layer;
 		*Result2 = r2;
 		*Result3 = r3;
-		return PCB_TYPE_TEXT;
+		return PCB_OBJ_TEXT;
 	}
 
-	if (Type & PCB_TYPE_ELEMENT &&
-			SearchElementByLocation(objst, req_flag, (pcb_element_t **) Result1, (pcb_element_t **) Result2, (pcb_element_t **) Result3, pcb_true))
-		return PCB_TYPE_ELEMENT;
-
-	if (Type & PCB_TYPE_SUBC && PCB->SubcOn &&
+	if (Type & PCB_OBJ_SUBC && PCB->SubcOn &&
 			SearchSubcByLocation(objst, req_flag, (pcb_subc_t **) Result1, (pcb_subc_t **) Result2, (pcb_subc_t **) Result3, pcb_true))
-		return PCB_TYPE_SUBC;
+		return PCB_OBJ_SUBC;
 
-	return PCB_TYPE_NONE;
+	return PCB_OBJ_VOID;
 }
 
 int pcb_search_obj_by_location(unsigned long Type, void **Result1, void **Result2, void **Result3, pcb_coord_t X, pcb_coord_t Y, pcb_coord_t Radius)
 {
-	int res = pcb_search_obj_by_location_(Type, Result1, Result2, Result3, X, Y, Radius, 0);
-	if ((res != PCB_TYPE_NONE) && (res != PCB_TYPE_SUBC))
+	int res;
+
+	if ((conf_core.editor.lock_names) || (conf_core.editor.hide_names))
+		Type &= ~ PCB_OBJ_FLOATER;
+
+	res = pcb_search_obj_by_location_(Type, Result1, Result2, Result3, X, Y, Radius, 0);
+	if ((res != PCB_OBJ_VOID) && (res != PCB_OBJ_SUBC))
 		return res;
 
 	/* found a subc because it was still the best option over all the plain
@@ -1516,10 +1278,10 @@ int pcb_search_obj_by_location(unsigned long Type, void **Result1, void **Result
 	if ((!conf_core.editor.lock_names) && (!conf_core.editor.hide_names)) {
 		int fres;
 		void *fr1, *fr2, *fr3;
-		Type &= ~PCB_TYPE_SUBC;
-		Type |= PCB_TYPE_SUBC_PART;
+		Type &= ~PCB_OBJ_SUBC;
+		Type |= PCB_OBJ_SUBC_PART;
 		fres = pcb_search_obj_by_location_(Type, &fr1, &fr2, &fr3, X, Y, Radius, PCB_FLAG_FLOATER);
-		if (fres != PCB_TYPE_NONE) {
+		if (fres != PCB_OBJ_VOID) {
 			*Result1 = fr1;
 			*Result2 = fr2;
 			*Result3 = fr3;
@@ -1537,73 +1299,73 @@ int pcb_search_obj_by_location(unsigned long Type, void **Result1, void **Result
  * buffer or on the remove list.
  * The calling routine passes two pointers to allocated memory for storing
  * the results.
- * A type value is returned too which is PCB_TYPE_NONE if no objects has been found.
+ * A type value is returned too which is PCB_OBJ_VOID if no objects has been found.
  */
 static int pcb_search_obj_by_id_(pcb_data_t *Base, void **Result1, void **Result2, void **Result3, int ID, int type)
 {
-	if (type == PCB_TYPE_LINE || type == PCB_TYPE_LINE_POINT) {
+	if (type == PCB_OBJ_LINE || type == PCB_OBJ_LINE_POINT) {
 		PCB_LINE_ALL_LOOP(Base);
 		{
 			if (line->ID == ID) {
 				*Result1 = (void *) layer;
 				*Result2 = *Result3 = (void *) line;
-				return PCB_TYPE_LINE;
+				return PCB_OBJ_LINE;
 			}
 			if (line->Point1.ID == ID) {
 				*Result1 = (void *) layer;
 				*Result2 = (void *) line;
 				*Result3 = (void *) &line->Point1;
-				return PCB_TYPE_LINE_POINT;
+				return PCB_OBJ_LINE_POINT;
 			}
 			if (line->Point2.ID == ID) {
 				*Result1 = (void *) layer;
 				*Result2 = (void *) line;
 				*Result3 = (void *) &line->Point2;
-				return PCB_TYPE_LINE_POINT;
+				return PCB_OBJ_LINE_POINT;
 			}
 		}
 		PCB_ENDALL_LOOP;
 	}
-	if (type == PCB_TYPE_ARC || type == PCB_TYPE_ARC_POINT) {
+	if (type == PCB_OBJ_ARC || type == PCB_OBJ_ARC_POINT) {
 		PCB_ARC_ALL_LOOP(Base);
 		{
 			if (arc->ID == ID) {
 				*Result1 = (void *) layer;
 				*Result2 = *Result3 = (void *) arc;
-				return PCB_TYPE_ARC;
+				return PCB_OBJ_ARC;
 			}
 		}
 		PCB_ENDALL_LOOP;
 	}
 
-	if (type == PCB_TYPE_TEXT) {
+	if (type == PCB_OBJ_TEXT) {
 		PCB_TEXT_ALL_LOOP(Base);
 		{
 			if (text->ID == ID) {
 				*Result1 = (void *) layer;
 				*Result2 = *Result3 = (void *) text;
-				return PCB_TYPE_TEXT;
+				return PCB_OBJ_TEXT;
 			}
 		}
 		PCB_ENDALL_LOOP;
 	}
 
-	if (type == PCB_TYPE_POLY || type == PCB_TYPE_POLY_POINT) {
+	if (type == PCB_OBJ_POLY || type == PCB_OBJ_POLY_POINT) {
 		PCB_POLY_ALL_LOOP(Base);
 		{
 			if (polygon->ID == ID) {
 				*Result1 = (void *) layer;
 				*Result2 = *Result3 = (void *) polygon;
-				return PCB_TYPE_POLY;
+				return PCB_OBJ_POLY;
 			}
-			if (type == PCB_TYPE_POLY_POINT)
+			if (type == PCB_OBJ_POLY_POINT)
 				PCB_POLY_POINT_LOOP(polygon);
 			{
 				if (point->ID == ID) {
 					*Result1 = (void *) layer;
 					*Result2 = (void *) polygon;
 					*Result3 = (void *) point;
-					return PCB_TYPE_POLY_POINT;
+					return PCB_OBJ_POLY_POINT;
 				}
 			}
 			PCB_END_LOOP;
@@ -1611,46 +1373,35 @@ static int pcb_search_obj_by_id_(pcb_data_t *Base, void **Result1, void **Result
 		PCB_ENDALL_LOOP;
 	}
 
-	if (type == PCB_TYPE_VIA) {
-		PCB_VIA_LOOP(Base);
-		{
-			if (via->ID == ID) {
-				*Result1 = *Result2 = *Result3 = (void *) via;
-				return PCB_TYPE_VIA;
-			}
-		}
-		PCB_END_LOOP;
-	}
-
-	if (type == PCB_TYPE_PSTK) {
+	if (type == PCB_OBJ_PSTK) {
 		PCB_PADSTACK_LOOP(Base);
 		{
 			if (padstack->ID == ID) {
 				*Result1 = *Result2 = *Result3 = (void *)padstack;
-				return PCB_TYPE_PSTK;
+				return PCB_OBJ_PSTK;
 			}
 		}
 		PCB_END_LOOP;
 	}
 
-	if (type == PCB_TYPE_RATLINE || type == PCB_TYPE_LINE_POINT) {
+	if (type == PCB_OBJ_RAT || type == PCB_OBJ_LINE_POINT) {
 		PCB_RAT_LOOP(Base);
 		{
 			if (line->ID == ID) {
 				*Result1 = *Result2 = *Result3 = (void *) line;
-				return PCB_TYPE_RATLINE;
+				return PCB_OBJ_RAT;
 			}
 			if (line->Point1.ID == ID) {
 				*Result1 = (void *) NULL;
 				*Result2 = (void *) line;
 				*Result3 = (void *) &line->Point1;
-				return PCB_TYPE_LINE_POINT;
+				return PCB_OBJ_LINE_POINT;
 			}
 			if (line->Point2.ID == ID) {
 				*Result1 = (void *) NULL;
 				*Result2 = (void *) line;
 				*Result3 = (void *) &line->Point2;
-				return PCB_TYPE_LINE_POINT;
+				return PCB_OBJ_LINE_POINT;
 			}
 		}
 		PCB_END_LOOP;
@@ -1660,110 +1411,50 @@ static int pcb_search_obj_by_id_(pcb_data_t *Base, void **Result1, void **Result
 	PCB_SUBC_LOOP(Base);
 	{
 		int res;
-		if (type == PCB_TYPE_SUBC) {
+		if (type == PCB_OBJ_SUBC) {
 			if (subc->ID == ID) {
 				*Result1 = *Result2 = *Result3 = (void *)subc;
-				return PCB_TYPE_SUBC;
+				return PCB_OBJ_SUBC;
 			}
 		}
 
 		res = pcb_search_obj_by_id_(subc->data, Result1, Result2, Result3, ID, type);
-		if (res != PCB_TYPE_NONE)
+		if (res != PCB_OBJ_VOID)
 			return res;
 	}
 	PCB_END_LOOP;
 
-	if (type == PCB_TYPE_ELEMENT || type == PCB_TYPE_PAD || type == PCB_TYPE_PIN
-			|| type == PCB_TYPE_ELEMENT_LINE || type == PCB_TYPE_ELEMENT_NAME || type == PCB_TYPE_ELEMENT_ARC)
-		/* check pins and elementnames too */
-		PCB_ELEMENT_LOOP(Base);
-	{
-		if (element->ID == ID) {
-			*Result1 = *Result2 = *Result3 = (void *) element;
-			return PCB_TYPE_ELEMENT;
-		}
-		if (type == PCB_TYPE_ELEMENT_LINE)
-			PCB_ELEMENT_PCB_LINE_LOOP(element);
-		{
-			if (line->ID == ID) {
-				*Result1 = (void *) element;
-				*Result2 = *Result3 = (void *) line;
-				return PCB_TYPE_ELEMENT_LINE;
-			}
-		}
-		PCB_END_LOOP;
-		if (type == PCB_TYPE_ELEMENT_ARC)
-			PCB_ARC_LOOP(element);
-		{
-			if (arc->ID == ID) {
-				*Result1 = (void *) element;
-				*Result2 = *Result3 = (void *) arc;
-				return PCB_TYPE_ELEMENT_ARC;
-			}
-		}
-		PCB_END_LOOP;
-		if (type == PCB_TYPE_ELEMENT_NAME)
-			PCB_ELEMENT_PCB_TEXT_LOOP(element);
-		{
-			if (text->ID == ID) {
-				*Result1 = (void *) element;
-				*Result2 = *Result3 = (void *) text;
-				return PCB_TYPE_ELEMENT_NAME;
-			}
-		}
-		PCB_END_LOOP;
-		if (type == PCB_TYPE_PIN)
-			PCB_PIN_LOOP(element);
-		{
-			if (pin->ID == ID) {
-				*Result1 = (void *) element;
-				*Result2 = *Result3 = (void *) pin;
-				return PCB_TYPE_PIN;
-			}
-		}
-		PCB_END_LOOP;
-		if (type == PCB_TYPE_PAD)
-			PCB_PAD_LOOP(element);
-		{
-			if (pad->ID == ID) {
-				*Result1 = (void *) element;
-				*Result2 = *Result3 = (void *) pad;
-				return PCB_TYPE_PAD;
-			}
-		}
-		PCB_END_LOOP;
-	}
-	PCB_END_LOOP;
-
-	return PCB_TYPE_NONE;
+	return PCB_OBJ_VOID;
 }
 
 int pcb_search_obj_by_id(pcb_data_t *Base, void **Result1, void **Result2, void **Result3, int ID, int type)
 {
 	int res = pcb_search_obj_by_id_(Base, Result1, Result2, Result3, ID, type);
-	if (res == PCB_TYPE_NONE)
+	if (res == PCB_OBJ_VOID)
 		pcb_message(PCB_MSG_ERROR, "hace: Internal error, search for ID %d failed\n", ID);
 	return res;
 }
 
-
-/* ---------------------------------------------------------------------------
- * searches for an element by its refdes.
- * Return the element or NULL if not found
- */
-pcb_element_t *pcb_search_elem_by_name(pcb_data_t *Base, const char *Name)
+int pcb_search_obj_by_id_buf2(pcb_data_t *Base, void **Result1, void **Result2, void **Result3, int ID, int type)
 {
-	pcb_element_t *result = NULL;
+	int bid = 0;
+	long res;
 
-	PCB_ELEMENT_LOOP(Base);
-	{
-		if (element->Name[1].TextString && PCB_NSTRCMP(element->Name[1].TextString, Name) == 0) {
-			result = element;
-			return result;
-		}
+	res = pcb_search_obj_by_id_(Base, Result1, Result2, Result3, ID, type);
+	if (res != PCB_OBJ_VOID)
+		return res;
+
+	while(res == PCB_OBJ_VOID) {
+		res = pcb_search_obj_by_id_(pcb_buffers[bid].Data, Result1, Result2, Result3, ID, type);
+		if (res != PCB_OBJ_VOID)
+			return res;
+		bid++;
+		if (bid >= PCB_MAX_BUFFER)
+			break;
 	}
-	PCB_END_LOOP;
-	return result;
+
+	pcb_message(PCB_MSG_ERROR, "hace: Internal error, search for ID %d failed\n", ID);
+	return PCB_OBJ_VOID;
 }
 
 /* ---------------------------------------------------------------------------
