@@ -46,44 +46,11 @@
 #include "draw.h"
 #include "remove.h"
 #include "copy.h"
+#include "grid.h"
 #include "hid_attrib.h"
 #include "compat_misc.h"
 #include "compat_nls.h"
 
-/* --------------------------------------------------------------------------- */
-/* Ask the user for a search pattern */
-static char *gui_get_pat(pcb_search_method_t * method)
-{
-	const char *methods[] = { "regexp", "list of names", NULL };
-	pcb_hid_attribute_t attrs[2];
-#define nattr sizeof(attrs)/sizeof(attrs[0])
-	static pcb_hid_attr_val_t results[nattr] = { 0 };
-
-	memset(attrs, 0, sizeof(attrs));
-
-	attrs[0].name = "Pattern";
-	attrs[0].help_text = "Name/refdes pattern";
-	attrs[0].type = PCB_HATT_STRING;
-	attrs[0].default_val.str_value = results[0].str_value;
-
-	attrs[1].name = "Method";
-	attrs[1].help_text = "method of search: either regular expression or a list of full names separated by |";
-	attrs[1].type = PCB_HATT_ENUM;
-	attrs[1].enumerations = methods;
-	attrs[1].default_val.int_value = results[1].int_value;
-
-	pcb_gui->attribute_dialog(attrs, nattr, results, "Find element", "Find element by name", NULL);
-
-	*method = results[1].int_value;
-	if (results[0].str_value == NULL)
-		return NULL;
-	return pcb_strdup(results[0].str_value);
-#undef nattr
-}
-
-
-/* --------------------------------------------------------------------------- */
-
 static const char pcb_acts_Select[] =
 	"Select(Object|ToggleObject)\n"
 	"Select(All|Block|Connection)\n"
@@ -109,7 +76,7 @@ Selects all objects on the board.
 Selects all connections with the ``found'' flag set.
 
 @item Convert
-Converts the selected objects to an element.  This uses the highest
+Converts the selected objects to a subcircuit.  This uses the highest
 numbered paste buffer.
 
 @end table
@@ -120,8 +87,6 @@ static int pcb_act_Select(int argc, const char **argv, pcb_coord_t x, pcb_coord_
 {
 	const char *function = PCB_ACTION_ARG(0);
 	if (function) {
-		int type;
-
 		switch (pcb_funchash_get(function, NULL)) { /* select objects by their names */
 
 			/* select a single object */
@@ -174,24 +139,6 @@ static int pcb_act_Select(int argc, const char **argv, pcb_coord_t x, pcb_coord_
 			break;
 
 		case F_Convert:
-			{
-				pcb_coord_t x, y;
-				pcb_tool_note.Buffer = conf_core.editor.buffer_number;
-				pcb_buffer_set_number(PCB_MAX_BUFFER - 1);
-				pcb_buffer_clear(PCB, PCB_PASTEBUFFER);
-				pcb_gui->get_coords(_("Select the Element's Mark Location"), &x, &y);
-				x = pcb_grid_fit(x, PCB->Grid, PCB->GridOffsetX);
-				y = pcb_grid_fit(y, PCB->Grid, PCB->GridOffsetY);
-				pcb_buffer_add_selected(PCB, PCB_PASTEBUFFER, x, y, pcb_true);
-				pcb_undo_save_serial();
-				pcb_remove_selected();
-				pcb_element_convert_from_buffer(PCB_PASTEBUFFER);
-				pcb_undo_restore_serial();
-				pcb_buffer_copy_to_layout(PCB, x, y);
-				pcb_buffer_set_number(pcb_tool_note.Buffer);
-			}
-			break;
-
 		case F_ConvertSubc:
 			{
 				pcb_coord_t x, y;
@@ -247,7 +194,6 @@ static int pcb_act_Unselect(int argc, const char **argv, pcb_coord_t x, pcb_coor
 {
 	const char *function = PCB_ACTION_ARG(0);
 	if (function) {
-		int type;
 		switch (pcb_funchash_get(function, NULL)) {
 
 			/* all objects in block */
