@@ -25,6 +25,6 @@
  */
 
 extern pcb_bool pcb_mid_stroke;
-extern void (*pcb_stub_stroke_record)(int ev_x, int ev_y);
+extern void (*pcb_stub_stroke_record)(pcb_coord_t ev_x, pcb_coord_t ev_y);
 extern void (*pcb_stub_stroke_start)(void);
-extern void (*pcb_stub_stroke_finish)(void);
+extern int (*pcb_stub_stroke_finish)(void);
