@@ -28,16 +28,29 @@
 
 #include "tool.h"
 
+#include "action_helper.h"
 #include "board.h"
 #include "conf_core.h"
+#include "crosshair.h"
+#include "compat_nls.h"
+#include "data.h"
+#include "draw.h"
 #include "error.h"
 #include "event.h"
+#include "find.h"
+#include "grid.h"
+#include "undo.h"
 
-#warning tool TODO: remove this when pcb_crosshair_set_mode() got moved here
-#include "crosshair.h"
+
+pcb_toolid_t pcb_tool_prev_id;
+pcb_toolid_t pcb_tool_next_id;
+static int save_position = 0;
+static int save_stack[PCB_MAX_MODESTACK_DEPTH];
 
 static void default_tool_reg(void);
 static void default_tool_unreg(void);
+static void init_current_tool(void);
+static void uninit_current_tool(void);
 
 void pcb_tool_init(void)
 {
@@ -93,15 +106,45 @@ int pcb_tool_select_by_name(const char *name)
 	pcb_toolid_t id = pcb_tool_lookup(name);
 	if (id == PCB_TOOLID_INVALID)
 		return -1;
-	pcb_crosshair_set_mode(id);
-	return 0;
+	return pcb_tool_select_by_id(id);
 }
 
 int pcb_tool_select_by_id(pcb_toolid_t id)
 {
+	char id_s[32];
+	static pcb_bool recursing = pcb_false;
+	
 	if ((id < 0) || (id > vtp0_len(&pcb_tools)))
 		return -1;
-	pcb_crosshair_set_mode(id);
+	
+	/* protect the cursor while changing the mode
+	 * perform some additional stuff depending on the new mode
+	 * reset 'state' of attached objects
+	 */
+	if (recursing)
+		return -1;
+	recursing = pcb_true;
+	
+	if (PCB->RatDraw && !pcb_tool_get(id)->allow_when_drawing_ratlines) {
+		pcb_message(PCB_MSG_WARNING, _("That mode is NOT allowed when drawing ratlines!\n"));
+		id = PCB_MODE_NO;
+	}
+	
+	pcb_tool_prev_id = conf_core.editor.mode;
+	pcb_tool_next_id = id;
+	uninit_current_tool();
+	sprintf(id_s, "%d", id);
+	conf_set(CFR_DESIGN, "editor/mode", -1, id_s, POL_OVERWRITE);
+	init_current_tool();
+
+	recursing = pcb_false;
+
+	/* force a crosshair grid update because the valid range
+	 * may have changed
+	 */
+	pcb_notify_crosshair_change(pcb_false);
+	pcb_crosshair_move_relative(0, 0);
+	pcb_notify_crosshair_change(pcb_true);
 	return 0;
 }
 
@@ -118,10 +161,28 @@ int pcb_tool_select_highest(void)
 	}
 	if (bestn == PCB_TOOLID_INVALID)
 		return -1;
-	pcb_crosshair_set_mode(bestn);
+	return pcb_tool_select_by_id(bestn);
+}
+
+int pcb_tool_save(void)
+{
+	save_stack[save_position] = conf_core.editor.mode;
+	if (save_position < PCB_MAX_MODESTACK_DEPTH - 1)
+		save_position++;
+	else
+		return -1;
 	return 0;
 }
 
+int pcb_tool_restore(void)
+{
+	if (save_position == 0) {
+		pcb_message(PCB_MSG_ERROR, "hace: underflow of restore mode\n");
+		return -1;
+	}
+	return pcb_tool_select_by_id(save_stack[--save_position]);
+}
+
 /**** current tool function wrappers ****/
 #define wrap(func, err_ret, prefix, args) \
 	do { \
@@ -137,6 +198,16 @@ int pcb_tool_select_highest(void)
 #define wrap_void(func, args)           wrap(func, return,  ;,      args)
 #define wrap_retv(func, err_ret, args)  wrap(func, err_ret, return, args)
 
+static void init_current_tool(void)
+{
+	wrap_void(init, ());
+}
+
+static void uninit_current_tool(void)
+{
+	wrap_void(uninit, ());
+}
+
 void pcb_tool_notify_mode(void)
 {
 	wrap_void(notify_mode, ());
@@ -176,37 +247,28 @@ pcb_bool pcb_tool_is_saved = pcb_false;
 static void get_grid_lock_coordinates(int type, void *ptr1, void *ptr2, void *ptr3, pcb_coord_t * x, pcb_coord_t * y)
 {
 	switch (type) {
-	case PCB_TYPE_VIA:
-		*x = ((pcb_pin_t *) ptr2)->X;
-		*y = ((pcb_pin_t *) ptr2)->Y;
-		break;
-	case PCB_TYPE_LINE:
+	case PCB_OBJ_LINE:
 		*x = ((pcb_line_t *) ptr2)->Point1.X;
 		*y = ((pcb_line_t *) ptr2)->Point1.Y;
 		break;
-	case PCB_TYPE_TEXT:
-	case PCB_TYPE_ELEMENT_NAME:
+	case PCB_OBJ_TEXT:
 		*x = ((pcb_text_t *) ptr2)->X;
 		*y = ((pcb_text_t *) ptr2)->Y;
 		break;
-	case PCB_TYPE_ELEMENT:
-		*x = ((pcb_element_t *) ptr2)->MarkX;
-		*y = ((pcb_element_t *) ptr2)->MarkY;
-		break;
-	case PCB_TYPE_POLY:
+	case PCB_OBJ_POLY:
 		*x = ((pcb_poly_t *) ptr2)->Points[0].X;
 		*y = ((pcb_poly_t *) ptr2)->Points[0].Y;
 		break;
 
-	case PCB_TYPE_LINE_POINT:
-	case PCB_TYPE_POLY_POINT:
+	case PCB_OBJ_LINE_POINT:
+	case PCB_OBJ_POLY_POINT:
 		*x = ((pcb_point_t *) ptr3)->X;
 		*y = ((pcb_point_t *) ptr3)->Y;
 		break;
-	case PCB_TYPE_ARC:
+	case PCB_OBJ_ARC:
 		pcb_arc_get_end((pcb_arc_t *) ptr2, 0, x, y);
 		break;
-	case PCB_TYPE_ARC_POINT:
+	case PCB_OBJ_ARC_POINT:
 		if (ptr3 != NULL) /* need to check because: if snap off, there's no known endpoint (leave x;y as is, then) */
 			pcb_arc_get_end((pcb_arc_t *) ptr2, ((*(int **)ptr3) != pcb_arc_start_ptr), x, y);
 		break;
@@ -246,10 +308,9 @@ void pcb_tool_attach_for_copy(pcb_coord_t PlaceX, pcb_coord_t PlaceY, pcb_bool d
 	if (do_rubberband && conf_core.editor.rubber_band_mode)
 		pcb_event(PCB_EVENT_RUBBER_LOOKUP_LINES, "ippp", pcb_crosshair.AttachedObject.Type, pcb_crosshair.AttachedObject.Ptr1, pcb_crosshair.AttachedObject.Ptr2, pcb_crosshair.AttachedObject.Ptr3);
 	if (do_rubberband &&
-			(pcb_crosshair.AttachedObject.Type == PCB_TYPE_ELEMENT ||
-			 pcb_crosshair.AttachedObject.Type == PCB_TYPE_SUBC ||
-			 pcb_crosshair.AttachedObject.Type == PCB_TYPE_VIA ||
-			 pcb_crosshair.AttachedObject.Type == PCB_TYPE_LINE || pcb_crosshair.AttachedObject.Type == PCB_TYPE_LINE_POINT))
+			(pcb_crosshair.AttachedObject.Type == PCB_OBJ_SUBC ||
+			 pcb_crosshair.AttachedObject.Type == PCB_OBJ_PSTK ||
+			 pcb_crosshair.AttachedObject.Type == PCB_OBJ_LINE || pcb_crosshair.AttachedObject.Type == PCB_OBJ_LINE_POINT))
 		pcb_event(PCB_EVENT_RUBBER_LOOKUP_RATS, "ippp", pcb_crosshair.AttachedObject.Type, pcb_crosshair.AttachedObject.Ptr1, pcb_crosshair.AttachedObject.Ptr2, pcb_crosshair.AttachedObject.Ptr3);
 }
 
@@ -270,6 +331,21 @@ void pcb_tool_notify_block(void)
 	pcb_notify_crosshair_change(pcb_true);
 }
 
+pcb_bool pcb_tool_should_snap_offgrid_line(pcb_layer_t *layer, pcb_line_t *line)
+{
+	/* Allow snapping to off-grid lines when drawing new lines (on
+	 * the same layer), and when moving a line end-point
+	 * (but don't snap to the same line)
+	 */
+	if ((conf_core.editor.mode == PCB_MODE_LINE && CURRENT == layer) ||
+			(conf_core.editor.mode == PCB_MODE_MOVE
+			 && pcb_crosshair.AttachedObject.Type == PCB_OBJ_LINE_POINT
+			 && pcb_crosshair.AttachedObject.Ptr1 == layer
+			 && pcb_crosshair.AttachedObject.Ptr2 != line))
+		return pcb_true;
+	else
+		return pcb_false;
+}
 
 #warning tool TODO: move this out to a tool plugin
 
