@@ -32,6 +32,27 @@
 #include "global_typedefs.h"
 #include "pcb_bool.h"
 
+typedef enum {
+	PCB_MODE_NO              = -1,  /* no mode selected - panning - TODO: remove this in favor of the default mode (it's the same as the arrow mode) */
+	PCB_MODE_VIA             = 15,  /* draw vias */
+	PCB_MODE_LINE            = 5,   /* draw lines */
+	PCB_MODE_RECTANGLE       = 10,  /* create rectangles */
+	PCB_MODE_POLYGON         = 8,   /* draw filled polygons */
+	PCB_MODE_PASTE_BUFFER    = 2,   /* paste objects from buffer */
+	PCB_MODE_TEXT            = 13,  /* create text objects */
+	PCB_MODE_ROTATE          = 12,  /* rotate objects */
+	PCB_MODE_REMOVE          = 11,  /* remove objects */
+	PCB_MODE_MOVE            = 7,   /* move objects */
+	PCB_MODE_COPY            = 3,   /* copy objects */
+	PCB_MODE_INSERT_POINT    = 4,   /* insert point into line/polygon */
+	PCB_MODE_RUBBERBAND_MOVE = 16,  /* move objects and attached lines */
+	PCB_MODE_THERMAL         = 14,  /* toggle thermal layer flag */
+	PCB_MODE_ARC             = 0,   /* draw arcs */
+	PCB_MODE_ARROW           = 1,   /* selection with arrow mode */
+	PCB_MODE_LOCK            = 6,   /* lock/unlock objects */
+	PCB_MODE_POLYGON_HOLE    = 9    /* cut holes in filled polygons */
+} pcb_mode_t;
+
 typedef int pcb_toolid_t;
 #define PCB_TOOLID_INVALID (-1)
 
@@ -41,12 +62,16 @@ typedef struct pcb_tool_s {
 	unsigned int priority;        /* lower values are higher priorities; escaping mode will try to select the highest prio tool */
 
 	/* tool implementation */
+	void     (*init)(void);
+	void     (*uninit)(void);
 	void     (*notify_mode)(void);
 	void     (*release_mode)(void);
 	void     (*adjust_attached_objects)(void);
 	void     (*draw_attached)(void);
 	pcb_bool (*undo_act)(void);
 	pcb_bool (*redo_act)(void);
+	
+	pcb_bool allow_when_drawing_ratlines;
 } pcb_tool_t;
 
 vtp0_t pcb_tools;
@@ -70,6 +95,9 @@ int pcb_tool_select_by_name(const char *name);
 int pcb_tool_select_by_id(pcb_toolid_t id);
 int pcb_tool_select_highest(void);
 
+int pcb_tool_save(void);
+int pcb_tool_restore(void);
+
 /**** Tool function wrappers; calling these will operate on the current tool 
       as defined in conf_core.editor.mode ****/
 
@@ -96,14 +124,17 @@ typedef struct {
 
 extern pcb_tool_note_t pcb_tool_note;
 extern pcb_bool pcb_tool_is_saved;
+extern pcb_toolid_t pcb_tool_prev_id;
+extern pcb_toolid_t pcb_tool_next_id;
 
 void pcb_tool_attach_for_copy(pcb_coord_t PlaceX, pcb_coord_t PlaceY, pcb_bool do_rubberband);
 void pcb_tool_notify_block(void);	/* create first or second corner of a marked block (when clicked) */
+pcb_bool pcb_tool_should_snap_offgrid_line(pcb_layer_t *layer, pcb_line_t *line);
 
 
 /**** Low level, for internal use ****/
 
 /* Get the tool pointer of a tool by id */
-#define pcb_tool_get(id) ((const pcb_tool_t *)vtp0_get(&pcb_tools, id, 0))
+#define pcb_tool_get(id) ((const pcb_tool_t *)*vtp0_get(&pcb_tools, id, 0))
 
 #endif
