@@ -44,10 +44,35 @@
 #include "search.h"
 #include "tool.h"
 #include "undo.h"
+#include "macro.h"
 
 #include "obj_arc_draw.h"
 
 
+void pcb_tool_arc_init(void)
+{
+	pcb_notify_crosshair_change(pcb_false);
+	if (pcb_tool_prev_id == PCB_MODE_LINE && pcb_crosshair.AttachedLine.State != PCB_CH_STATE_FIRST) {
+		pcb_crosshair.AttachedLine.State = PCB_CH_STATE_FIRST;
+		pcb_crosshair.AttachedBox.State = PCB_CH_STATE_SECOND;
+		pcb_crosshair.AttachedBox.Point1.X = pcb_crosshair.AttachedBox.Point2.X = pcb_crosshair.AttachedLine.Point1.X;
+		pcb_crosshair.AttachedBox.Point1.Y = pcb_crosshair.AttachedBox.Point2.Y = pcb_crosshair.AttachedLine.Point1.Y;
+		pcb_adjust_attached_objects();
+	}
+	pcb_notify_crosshair_change(pcb_true);
+}
+
+void pcb_tool_arc_uninit(void)
+{
+	pcb_notify_crosshair_change(pcb_false);
+	pcb_added_lines = 0;
+	if (pcb_tool_next_id != PCB_MODE_LINE) {
+		pcb_crosshair.AttachedBox.State = PCB_CH_STATE_FIRST;
+		pcb_crosshair_set_local_ref(0, 0, pcb_false);
+	}
+	pcb_notify_crosshair_change(pcb_true);
+}
+
 void pcb_tool_arc_notify_mode(void)
 {
 	switch (pcb_crosshair.AttachedBox.State) {
@@ -91,7 +116,7 @@ void pcb_tool_arc_notify_mode(void)
 				pcb_arc_get_end(arc, 1, &pcb_crosshair.AttachedBox.Point2.X, &pcb_crosshair.AttachedBox.Point2.Y);
 				pcb_crosshair.AttachedBox.Point1.X = pcb_crosshair.AttachedBox.Point2.X;
 				pcb_crosshair.AttachedBox.Point1.Y = pcb_crosshair.AttachedBox.Point2.Y;
-				pcb_undo_add_obj_to_create(PCB_TYPE_ARC, CURRENT, arc, arc);
+				pcb_undo_add_obj_to_create(PCB_OBJ_ARC, CURRENT, arc, arc);
 				pcb_undo_inc_serial();
 				pcb_added_lines++;
 				pcb_arc_invalidate_draw(CURRENT, arc);
@@ -111,10 +136,10 @@ void pcb_tool_arc_adjust_attached_objects(void)
 void pcb_tool_arc_draw_attached(void)
 {
 	if (pcb_crosshair.AttachedBox.State != PCB_CH_STATE_FIRST) {
-		XORDrawAttachedArc(conf_core.design.line_thickness);
+		pcb_xordraw_attached_arc(conf_core.design.line_thickness);
 		if (conf_core.editor.show_drc) {
 			pcb_gui->set_color(pcb_crosshair.GC, conf_core.appearance.color.cross);
-			XORDrawAttachedArc(conf_core.design.line_thickness + 2 * (PCB->Bloat + 1));
+			pcb_xordraw_attached_arc(conf_core.design.line_thickness + 2 * (conf_core.design.bloat + 1));
 			pcb_gui->set_color(pcb_crosshair.GC, conf_core.appearance.color.crosshair);
 		}
 	}
@@ -133,7 +158,7 @@ void pcb_tool_arc_draw_attached(void)
       pcb_draw_wireframe_line(pcb_crosshair.GC,
                               pcb_crosshair.X, pcb_crosshair.Y,
                               pcb_crosshair.X, pcb_crosshair.Y,
-                              conf_core.design.line_thickness + (2 * PCB->Bloat),0 );	
+                              conf_core.design.line_thickness + (2 * conf_core.design.bloat),0 );	
 		}
 	}
 
@@ -149,7 +174,7 @@ pcb_bool pcb_tool_arc_undo_act(void)
 	if (pcb_crosshair.AttachedBox.State == PCB_CH_STATE_THIRD) {
 		void *ptr1, *ptr2, *ptr3;
 		/* guaranteed to succeed */
-		pcb_search_obj_by_location(PCB_TYPE_ARC, &ptr1, &ptr2, &ptr3,
+		pcb_search_obj_by_location(PCB_OBJ_ARC, &ptr1, &ptr2, &ptr3,
 													 pcb_crosshair.AttachedBox.Point1.X, pcb_crosshair.AttachedBox.Point1.Y, 0);
 		pcb_arc_get_end((pcb_arc_t *) ptr2, 0, &pcb_crosshair.AttachedBox.Point2.X, &pcb_crosshair.AttachedBox.Point2.Y);
 		pcb_crosshair.AttachedBox.Point1.X = pcb_crosshair.AttachedBox.Point2.X;
@@ -163,10 +188,14 @@ pcb_bool pcb_tool_arc_undo_act(void)
 
 pcb_tool_t pcb_tool_arc = {
 	"arc", NULL, 100,
+	pcb_tool_arc_init,
+	pcb_tool_arc_uninit,
 	pcb_tool_arc_notify_mode,
 	NULL,
 	pcb_tool_arc_adjust_attached_objects,
 	pcb_tool_arc_draw_attached,
 	pcb_tool_arc_undo_act,
-	NULL
+	NULL,
+	
+	pcb_false
 };
