@@ -48,6 +48,15 @@
 #include "undo.h"
 
 
+void pcb_tool_arrow_uninit(void)
+{
+	pcb_notify_crosshair_change(pcb_false);
+	pcb_crosshair.AttachedObject.Type = PCB_OBJ_VOID;
+	pcb_crosshair.AttachedObject.State = PCB_CH_STATE_FIRST;
+	pcb_crosshair.AttachedBox.State = PCB_CH_STATE_FIRST;
+	pcb_notify_crosshair_change(pcb_true);
+}
+
 /* Called some time after the click if there was a release but no second click
 	 a.k.a. finalize single click (some things are already done in pcb_notify_mode
 	 at the initial click event) */
@@ -63,16 +72,16 @@ static void click_timer_cb(pcb_hidval_t hv)
 			pcb_buffer_add_selected(PCB, PCB_PASTEBUFFER, pcb_tool_note.X, pcb_tool_note.Y, pcb_true);
 			pcb_undo_save_serial();
 			pcb_remove_selected();
-			pcb_crosshair_save_mode();
+			pcb_tool_save();
 			pcb_tool_is_saved = pcb_true;
-			pcb_crosshair_set_mode(PCB_MODE_PASTE_BUFFER);
+			pcb_tool_select_by_id(PCB_MODE_PASTE_BUFFER);
 		}
 		else if (pcb_tool_note.Hit && !pcb_gui->shift_is_pressed()) {
 			pcb_box_t box;
 
-			pcb_crosshair_save_mode();
+			pcb_tool_save();
 			pcb_tool_is_saved = pcb_true;
-			pcb_crosshair_set_mode(pcb_gui->control_is_pressed()? PCB_MODE_COPY : PCB_MODE_MOVE);
+			pcb_tool_select_by_id(pcb_gui->control_is_pressed()? PCB_MODE_COPY : PCB_MODE_MOVE);
 			pcb_crosshair.AttachedObject.Ptr1 = pcb_tool_note.ptr1;
 			pcb_crosshair.AttachedObject.Ptr2 = pcb_tool_note.ptr2;
 			pcb_crosshair.AttachedObject.Ptr3 = pcb_tool_note.ptr3;
@@ -128,17 +137,17 @@ void pcb_tool_arrow_notify_mode(void)
 	 * (pcb_tool_note.Moving) or clicked on a MOVE_TYPE
 	 * (pcb_tool_note.Hit)
 	 */
-	for (test = (PCB_SELECT_TYPES | PCB_MOVE_TYPES | PCB_TYPE_SUBC_FLOATER | PCB_LOOSE_SUBC) & ~PCB_TYPE_RATLINE; test; test &= ~type) {
+	for (test = (PCB_SELECT_TYPES | PCB_MOVE_TYPES | PCB_OBJ_FLOATER | PCB_LOOSE_SUBC) & ~PCB_OBJ_RAT; test; test &= ~type) {
 		type = pcb_search_screen(pcb_tool_note.X, pcb_tool_note.Y, test, &ptr1, &ptr2, &ptr3);
-		if (!pcb_tool_note.Hit && (type & PCB_MOVE_TYPES) && !PCB_FLAG_TEST(PCB_FLAG_LOCK, (pcb_pin_t *) ptr2)) {
+		if (!pcb_tool_note.Hit && (type & PCB_MOVE_TYPES) && !PCB_FLAG_TEST(PCB_FLAG_LOCK, (pcb_any_obj_t *) ptr2)) {
 			pcb_tool_note.Hit = type;
 			pcb_tool_note.ptr1 = ptr1;
 			pcb_tool_note.ptr2 = ptr2;
 			pcb_tool_note.ptr3 = ptr3;
 		}
-		if (!pcb_tool_note.Moving && (type & (PCB_SELECT_TYPES | PCB_LOOSE_SUBC)) && PCB_FLAG_TEST(PCB_FLAG_SELECTED, (pcb_pin_t *) ptr2))
+		if (!pcb_tool_note.Moving && (type & (PCB_SELECT_TYPES | PCB_LOOSE_SUBC)) && PCB_FLAG_TEST(PCB_FLAG_SELECTED, (pcb_any_obj_t *) ptr2))
 			pcb_tool_note.Moving = pcb_true;
-		if ((pcb_tool_note.Hit && pcb_tool_note.Moving) || type == PCB_TYPE_NONE)
+		if ((pcb_tool_note.Hit && pcb_tool_note.Moving) || type == PCB_OBJ_VOID)
 			return;
 	}
 }
@@ -202,10 +211,14 @@ void pcb_tool_arrow_adjust_attached_objects(void)
 
 pcb_tool_t pcb_tool_arrow = {
 	"arrow", NULL, 10,
+	NULL,
+	pcb_tool_arrow_uninit,
 	pcb_tool_arrow_notify_mode,
 	pcb_tool_arrow_release_mode,
 	pcb_tool_arrow_adjust_attached_objects,
 	NULL,
 	NULL,
-	NULL
+	NULL,
+	
+	pcb_true
 };
