@@ -46,62 +46,34 @@
 #include "tool.h"
 #include "undo.h"
 
-#include "obj_elem_draw.h"
+void pcb_tool_buffer_init(void)
+{
+	pcb_crosshair_range_to_buffer();
+}
 
+void pcb_tool_buffer_uninit(void)
+{
+	pcb_crosshair_set_range(0, 0, PCB->MaxWidth, PCB->MaxHeight);
+}
 
 void pcb_tool_buffer_notify_mode(void)
 {
 	void *ptr1, *ptr2, *ptr3;
-	pcb_text_t estr[PCB_MAX_ELEMENTNAMES];
-	pcb_element_t *e = 0;
 	pcb_subc_t *orig_subc = NULL;
 
 	if (pcb_gui->shift_is_pressed()) {
-		int type = pcb_search_screen(pcb_tool_note.X, pcb_tool_note.Y, PCB_TYPE_ELEMENT | PCB_TYPE_SUBC, &ptr1, &ptr2,
-														&ptr3);
-		if (type == PCB_TYPE_ELEMENT) {
-			e = (pcb_element_t *) ptr1;
-			if (e) {
-				int i;
-
-				memcpy(estr, e->Name, PCB_MAX_ELEMENTNAMES * sizeof(pcb_text_t));
-				for (i = 0; i < PCB_MAX_ELEMENTNAMES; ++i)
-					estr[i].TextString = estr[i].TextString ? pcb_strdup(estr[i].TextString) : NULL;
-				pcb_element_remove(e);
-			}
-		}
-		if (type == PCB_TYPE_SUBC) {
+		int type = pcb_search_screen(pcb_tool_note.X, pcb_tool_note.Y, PCB_OBJ_SUBC, &ptr1, &ptr2, &ptr3);
+		if (type == PCB_OBJ_SUBC) {
 			orig_subc = (pcb_subc_t *) ptr1;
 			pcb_subc_remove(orig_subc);
 		}
 	}
 	if (pcb_buffer_copy_to_layout(PCB, pcb_tool_note.X, pcb_tool_note.Y))
 		pcb_board_set_changed_flag(pcb_true);
-	if (e) {
-		int type = pcb_search_screen(pcb_tool_note.X, pcb_tool_note.Y, PCB_TYPE_ELEMENT | PCB_TYPE_SUBC, &ptr1, &ptr2, &ptr3);
-		if (type == PCB_TYPE_ELEMENT && ptr1) {
-			int i, save_n;
-			e = (pcb_element_t *) ptr1;
-
-			save_n = PCB_ELEMNAME_IDX_VISIBLE();
-
-			for (i = 0; i < PCB_MAX_ELEMENTNAMES; i++) {
-				if (i == save_n)
-					pcb_elem_name_invalidate_erase(e);
-				pcb_r_delete_entry(PCB->Data->name_tree[i], (pcb_box_t *) & (e->Name[i]));
-				memcpy(&(e->Name[i]), &(estr[i]), sizeof(pcb_text_t));
-				e->Name[i].Element = e;
-				pcb_text_bbox(pcb_font(PCB, 0, 1), &(e->Name[i]));
-				pcb_r_insert_entry(PCB->Data->name_tree[i], (pcb_box_t *) & (e->Name[i]));
-				if (i == save_n)
-					pcb_elem_name_invalidate_draw(e);
-			}
-		}
-	}
 
 	if (orig_subc != NULL) {
-		int type = pcb_search_screen(pcb_tool_note.X, pcb_tool_note.Y, PCB_TYPE_SUBC, &ptr1, &ptr2, &ptr3);
-		if (type == PCB_TYPE_SUBC && (ptr1 != NULL)) {
+		int type = pcb_search_screen(pcb_tool_note.X, pcb_tool_note.Y, PCB_OBJ_SUBC, &ptr1, &ptr2, &ptr3);
+		if (type == PCB_OBJ_SUBC && (ptr1 != NULL)) {
 			int n;
 			pcb_attribute_list_t *dst = &(((pcb_subc_t *)ptr1)->Attributes), *src = &orig_subc->Attributes;
 			for (n = 0; n < src->Number; n++)
@@ -125,15 +97,19 @@ void pcb_tool_buffer_release_mode(void)
 
 void pcb_tool_buffer_draw_attached(void)
 {
-	XORDrawBuffer(PCB_PASTEBUFFER);
+	pcb_xordraw_buffer(PCB_PASTEBUFFER);
 }
 
 pcb_tool_t pcb_tool_buffer = {
 	"buffer", NULL, 100,
+	pcb_tool_buffer_init,
+	pcb_tool_buffer_uninit,
 	pcb_tool_buffer_notify_mode,
 	pcb_tool_buffer_release_mode,
 	NULL,
 	pcb_tool_buffer_draw_attached,
 	NULL,
-	NULL
+	NULL,
+	
+	pcb_true
 };
