@@ -50,17 +50,52 @@
 #include "undo.h"
 
 #include "obj_line_draw.h"
-#include "obj_pinvia_draw.h"
+#include "obj_pstk_draw.h"
 #include "obj_rat_draw.h"
 
+#warning padstack TODO: remove this when via is removed and the padstack is created from style directly
+#include "src_plugins/lib_compat_help/pstk_compat.h"
 
 static pcb_layer_t *last_layer;
 
 
+void pcb_tool_line_init(void)
+{
+	pcb_notify_crosshair_change(pcb_false);
+	if (pcb_tool_prev_id == PCB_MODE_ARC && pcb_crosshair.AttachedBox.State != PCB_CH_STATE_FIRST) {
+		pcb_crosshair.AttachedBox.State = PCB_CH_STATE_FIRST;
+		pcb_crosshair.AttachedLine.State = PCB_CH_STATE_SECOND;
+		pcb_crosshair.AttachedLine.Point1.X = pcb_crosshair.AttachedLine.Point2.X = pcb_crosshair.AttachedBox.Point1.X;
+		pcb_crosshair.AttachedLine.Point1.Y = pcb_crosshair.AttachedLine.Point2.Y = pcb_crosshair.AttachedBox.Point1.Y;
+		pcb_adjust_attached_objects();
+	}
+	else {
+		if (conf_core.editor.auto_drc) {
+			if (pcb_reset_conns(pcb_true)) {
+				pcb_undo_inc_serial();
+				pcb_draw();
+			}
+		}
+	}
+	pcb_notify_crosshair_change(pcb_true);
+}
+
+void pcb_tool_line_uninit(void)
+{
+	pcb_notify_crosshair_change(pcb_false);
+	pcb_added_lines = 0;
+	pcb_route_reset(&pcb_crosshair.Route);
+	if (pcb_tool_next_id != PCB_MODE_ARC) {
+		pcb_crosshair.AttachedLine.State = PCB_CH_STATE_FIRST;
+		pcb_crosshair_set_local_ref(0, 0, pcb_false);
+	}
+	pcb_notify_crosshair_change(pcb_true);
+}
+
 /* creates points of a line (when clicked) */
 static void notify_line(void)
 {
-	int type = PCB_TYPE_NONE;
+	int type = PCB_OBJ_VOID;
 	void *ptr1, *ptr2, *ptr3;
 
 	if (!pcb_marked.status || conf_core.editor.local_ref)
@@ -68,39 +103,19 @@ static void notify_line(void)
 	switch (pcb_crosshair.AttachedLine.State) {
 	case PCB_CH_STATE_FIRST:						/* first point */
 #warning subc TODO: this should work on heavy terminals too!
-		if (PCB->RatDraw && pcb_search_screen(pcb_crosshair.X, pcb_crosshair.Y, PCB_TYPE_PAD | PCB_TYPE_PIN | PCB_TYPE_PSTK | PCB_TYPE_SUBC_PART, &ptr1, &ptr1, &ptr1) == PCB_TYPE_NONE) {
+		if (PCB->RatDraw && pcb_search_screen(pcb_crosshair.X, pcb_crosshair.Y, PCB_OBJ_PSTK | PCB_OBJ_SUBC_PART, &ptr1, &ptr1, &ptr1) == PCB_OBJ_VOID) {
 			pcb_gui->beep();
 			break;
 		}
 		if (conf_core.editor.auto_drc) {
-			type = pcb_search_screen(pcb_crosshair.X, pcb_crosshair.Y, PCB_TYPE_PIN | PCB_TYPE_PAD | PCB_TYPE_VIA | PCB_TYPE_PSTK | PCB_TYPE_SUBC_PART, &ptr1, &ptr2, &ptr3);
+			type = pcb_search_screen(pcb_crosshair.X, pcb_crosshair.Y, PCB_OBJ_PSTK | PCB_OBJ_SUBC_PART, &ptr1, &ptr2, &ptr3);
 			pcb_lookup_conn(pcb_crosshair.X, pcb_crosshair.Y, pcb_true, 1, PCB_FLAG_FOUND);
 		}
-		if (type == PCB_TYPE_PSTK) {
+		if (type == PCB_OBJ_PSTK) {
 			pcb_pstk_t *pstk = (pcb_pstk_t *)ptr2;
 			pcb_crosshair.AttachedLine.Point1.X = pstk->x;
 			pcb_crosshair.AttachedLine.Point1.Y = pstk->y;
 		}
-		else if (type == PCB_TYPE_PIN || type == PCB_TYPE_VIA) {
-			pcb_crosshair.AttachedLine.Point1.X = pcb_crosshair.AttachedLine.Point2.X = ((pcb_pin_t *) ptr2)->X;
-			pcb_crosshair.AttachedLine.Point1.Y = pcb_crosshair.AttachedLine.Point2.Y = ((pcb_pin_t *) ptr2)->Y;
-		}
-		else if (type == PCB_TYPE_PAD) {
-			pcb_pad_t *pad = (pcb_pad_t *) ptr2;
-			double d1 = pcb_distance(pcb_crosshair.X, pcb_crosshair.Y, pad->Point1.X, pad->Point1.Y);
-			double d2 = pcb_distance(pcb_crosshair.X, pcb_crosshair.Y, pad->Point2.X, pad->Point2.Y);
-			double dm = pcb_distance(pcb_crosshair.X, pcb_crosshair.Y, (pad->Point1.X + pad->Point2.X) / 2, (pad->Point1.Y + pad->Point2.Y)/2);
-			if ((dm <= d1) && (dm <= d2)) { /* prefer to snap to the middle of a pin if that's the closest */
-				pcb_crosshair.AttachedLine.Point1.X = pcb_crosshair.AttachedLine.Point2.X = pcb_crosshair.X;
-				pcb_crosshair.AttachedLine.Point1.Y = pcb_crosshair.AttachedLine.Point2.Y = pcb_crosshair.Y;
-			}
-			else if (d2 < d1) { /* else select the closest endpoint */
-				pcb_crosshair.AttachedLine.Point1 = pcb_crosshair.AttachedLine.Point2 = pad->Point2;
-			}
-			else {
-				pcb_crosshair.AttachedLine.Point1 = pcb_crosshair.AttachedLine.Point2 = pad->Point1;
-			}
-		}
 		else {
 			pcb_crosshair.AttachedLine.Point1.X = pcb_crosshair.AttachedLine.Point2.X = pcb_crosshair.X;
 			pcb_crosshair.AttachedLine.Point1.Y = pcb_crosshair.AttachedLine.Point2.Y = pcb_crosshair.Y;
@@ -132,7 +147,7 @@ void pcb_tool_line_notify_mode(void)
 	 * the file after saving.
 	 */
 	if (pcb_crosshair.X == pcb_crosshair.AttachedLine.Point1.X && pcb_crosshair.Y == pcb_crosshair.AttachedLine.Point1.Y) {
-		pcb_crosshair_set_mode(PCB_MODE_LINE);
+		pcb_tool_select_by_id(PCB_MODE_LINE);
 		return;
 	}
 
@@ -140,7 +155,7 @@ void pcb_tool_line_notify_mode(void)
 		pcb_rat_t *line;
 		if ((line = pcb_rat_add_net())) {
 			pcb_added_lines++;
-			pcb_undo_add_obj_to_create(PCB_TYPE_RATLINE, line, line, line);
+			pcb_undo_add_obj_to_create(PCB_OBJ_RAT, line, line, line);
 			pcb_undo_inc_serial();
 			pcb_rat_invalidate_draw(line);
 			pcb_crosshair.AttachedLine.Point1.X = pcb_crosshair.AttachedLine.Point2.X;
@@ -151,30 +166,29 @@ void pcb_tool_line_notify_mode(void)
 	}
 	else if(pcb_crosshair.Route.size > 0)
 	{
-		pcb_pin_t *via = NULL;
+		pcb_pstk_t *ps = NULL;
 
 		/* place a via if vias are visible, the layer is
 			 in a new group since the last line and there
 			 isn't a pin already here */
-		if (conf_core.editor.auto_via && PCB->ViaOn
+		if (conf_core.editor.auto_via && PCB->pstk_on
 				&& pcb_layer_get_group_(CURRENT) != pcb_layer_get_group_(last_layer)
-				&& pcb_search_obj_by_location(PCB_TYPEMASK_PIN, &ptr1, &ptr2, &ptr3,
+				&& pcb_search_obj_by_location(PCB_OBJ_CLASS_PIN, &ptr1, &ptr2, &ptr3,
 																			pcb_crosshair.AttachedLine.Point1.X,
 																			pcb_crosshair.AttachedLine.Point1.Y,
 																			conf_core.design.via_thickness / 2) ==
-																				PCB_TYPE_NONE
+																				PCB_OBJ_VOID
 				&& (pcb_layer_flags_(CURRENT) & PCB_LYT_COPPER)
 				&& (pcb_layer_flags_(last_layer) & PCB_LYT_COPPER)
 				&& (!PCB->is_footprint)
-				&& (via =	pcb_via_new(PCB->Data,
+#warning pdstk TODO #21: do not work in comp mode, use a pstk proto - scconfig also has TODO #21, fix it there too
+				&& ((ps = pcb_pstk_new_compat_via(PCB->Data,
 															pcb_crosshair.AttachedLine.Point1.X,
 															pcb_crosshair.AttachedLine.Point1.Y,
-															conf_core.design.via_thickness,
-															2 * conf_core.design.clearance, 0, 
-															conf_core.design.via_drilling_hole, NULL, 
-															pcb_no_flags())) != NULL) {
-					pcb_obj_add_attribs(via, PCB->pen_attr);
-					pcb_undo_add_obj_to_create(PCB_TYPE_VIA, via, via, via);
+				conf_core.design.via_drilling_hole, conf_core.design.via_thickness, conf_core.design.clearance,
+			0, PCB_PSTK_COMPAT_ROUND, pcb_true)) != NULL)) {
+					pcb_obj_add_attribs(ps, PCB->pen_attr);
+					pcb_undo_add_obj_to_create(PCB_OBJ_PSTK, ps, ps, ps);
 		}
 
 		/* Add the route to the design */
@@ -195,8 +209,8 @@ void pcb_tool_line_notify_mode(void)
 			pcb_marked.Y = pcb_crosshair.Route.end_point.Y;
 		}
 
-		if(via)
-			pcb_via_invalidate_draw(via);
+		if (ps)
+			pcb_pstk_invalidate_draw(ps);
 
 		pcb_draw();
 		pcb_undo_inc_serial();
@@ -239,33 +253,33 @@ void pcb_tool_line_notify_mode(void)
 																	 2 * conf_core.design.clearance,
 																	 pcb_flag_make(maybe_found_flag |
 																						 (conf_core.editor.clear_line ? PCB_FLAG_CLEARLINE : 0)))) != NULL) {
-			pcb_pin_t *via;
+			pcb_pstk_t *ps = NULL;
 
 			pcb_added_lines++;
 			pcb_obj_add_attribs(line, PCB->pen_attr);
-			pcb_undo_add_obj_to_create(PCB_TYPE_LINE, CURRENT, line, line);
+			pcb_undo_add_obj_to_create(PCB_OBJ_LINE, CURRENT, line, line);
 			pcb_line_invalidate_draw(CURRENT, line);
 			/* place a via if vias are visible, the layer is
 				 in a new group since the last line and there
 				 isn't a pin already here */
-			if (PCB->ViaOn 
+			if (PCB->pstk_on
 					&& pcb_layer_get_group_(CURRENT) != pcb_layer_get_group_(last_layer) 
 					&& (pcb_layer_flags_(CURRENT) & PCB_LYT_COPPER)
 					&& (pcb_layer_flags_(last_layer) & PCB_LYT_COPPER)
 					&& (!PCB->is_footprint)
-					&& pcb_search_obj_by_location(PCB_TYPEMASK_PIN, &ptr1, &ptr2, &ptr3,
+					&& pcb_search_obj_by_location(PCB_OBJ_CLASS_PIN, &ptr1, &ptr2, &ptr3,
 																 pcb_crosshair.AttachedLine.Point1.X,
 																 pcb_crosshair.AttachedLine.Point1.Y,
-																 conf_core.design.via_thickness / 2) == PCB_TYPE_NONE
-					&& (via =
-							pcb_via_new(PCB->Data,
-													 pcb_crosshair.AttachedLine.Point1.X,
-													 pcb_crosshair.AttachedLine.Point1.Y,
-													 conf_core.design.via_thickness,
-													 2 * conf_core.design.clearance, 0, conf_core.design.via_drilling_hole, NULL, pcb_no_flags())) != NULL) {
-				pcb_obj_add_attribs(via, PCB->pen_attr);
-				pcb_undo_add_obj_to_create(PCB_TYPE_VIA, via, via, via);
-				pcb_via_invalidate_draw(via);
+																 conf_core.design.via_thickness / 2) == PCB_OBJ_VOID
+#warning pdstk TODO #21: do not work in comp mode, use a pstk proto - scconfig also has TODO #21, fix it there too
+				&& ((ps = pcb_pstk_new_compat_via(PCB->Data,
+															pcb_crosshair.AttachedLine.Point1.X,
+															pcb_crosshair.AttachedLine.Point1.Y,
+															conf_core.design.via_drilling_hole, conf_core.design.via_thickness, conf_core.design.clearance,
+															0, PCB_PSTK_COMPAT_ROUND, pcb_true)) != NULL)) {
+				pcb_obj_add_attribs(ps, PCB->pen_attr);
+				pcb_undo_add_obj_to_create(PCB_OBJ_PSTK, ps, ps, ps);
+				pcb_pstk_invalidate_draw(ps);
 			}
 			/* copy the coordinates */
 			pcb_crosshair.AttachedLine.Point1.X = pcb_crosshair.AttachedLine.Point2.X;
@@ -285,7 +299,7 @@ void pcb_tool_line_notify_mode(void)
 																						 (conf_core.editor.clear_line ? PCB_FLAG_CLEARLINE : 0)))) != NULL) {
 			pcb_added_lines++;
 			pcb_obj_add_attribs(line, PCB->pen_attr);
-			pcb_undo_add_obj_to_create(PCB_TYPE_LINE, CURRENT, line, line);
+			pcb_undo_add_obj_to_create(PCB_OBJ_LINE, CURRENT, line, line);
 			pcb_undo_inc_serial();
 			pcb_line_invalidate_draw(CURRENT, line);
 			/* move to new start point */
@@ -352,7 +366,7 @@ void pcb_tool_line_draw_attached(void)
 			pcb_draw_wireframe_line(pcb_crosshair.GC,
 															pcb_crosshair.X, pcb_crosshair.Y,
 															pcb_crosshair.X, pcb_crosshair.Y, 
-															conf_core.design.line_thickness + (2 * PCB->Bloat),0 );
+															conf_core.design.line_thickness + (2 * conf_core.design.bloat), 0);
 		}
 	}
 }
@@ -373,7 +387,7 @@ pcb_bool pcb_tool_line_undo_act(void)
 		pcb_line_t *ptr2;
 		ptrtmp = &pcb_crosshair.AttachedLine; /* a workaround for the line undo bug */
 		/* this search is guaranteed to succeed */
-		pcb_search_obj_by_location(PCB_TYPE_LINE | PCB_TYPE_RATLINE, &ptr1,
+		pcb_search_obj_by_location(PCB_OBJ_LINE | PCB_OBJ_RAT, &ptr1,
 													 &ptrtmp, &ptr3, pcb_crosshair.AttachedLine.Point1.X, pcb_crosshair.AttachedLine.Point1.Y, 0);
 		ptr2 = (pcb_line_t *) ptrtmp;
 
@@ -395,7 +409,7 @@ pcb_bool pcb_tool_line_undo_act(void)
 		/* check if an intermediate point was removed */
 		if (type & PCB_UNDO_REMOVE) {
 			/* this search should find the restored line */
-			pcb_search_obj_by_location(PCB_TYPE_LINE | PCB_TYPE_RATLINE, &ptr1,
+			pcb_search_obj_by_location(PCB_OBJ_LINE | PCB_OBJ_RAT, &ptr1,
 														 &ptrtmp, &ptr3, pcb_crosshair.AttachedLine.Point2.X, pcb_crosshair.AttachedLine.Point2.Y, 0);
 			ptr2 = (pcb_line_t *) ptrtmp;
 			if (conf_core.editor.auto_drc) {
@@ -414,7 +428,7 @@ pcb_bool pcb_tool_line_undo_act(void)
 		}
 		else {
 			/* this search is guaranteed to succeed too */
-			pcb_search_obj_by_location(PCB_TYPE_LINE | PCB_TYPE_RATLINE, &ptr1,
+			pcb_search_obj_by_location(PCB_OBJ_LINE | PCB_OBJ_RAT, &ptr1,
 														 &ptrtmp, &ptr3, pcb_crosshair.AttachedLine.Point1.X, pcb_crosshair.AttachedLine.Point1.Y, 0);
 			ptr2 = (pcb_line_t *) ptrtmp;
 			last_layer = (pcb_layer_t *) ptr1;
@@ -442,10 +456,14 @@ pcb_bool pcb_tool_line_redo_act(void)
 
 pcb_tool_t pcb_tool_line = {
 	"line", NULL, 100,
+	pcb_tool_line_init,
+	pcb_tool_line_uninit,
 	pcb_tool_line_notify_mode,
 	NULL,
 	pcb_tool_line_adjust_attached_objects,
 	pcb_tool_line_draw_attached,
 	pcb_tool_line_undo_act,
-	pcb_tool_line_redo_act
+	pcb_tool_line_redo_act,
+	
+	pcb_true
 };
