@@ -44,48 +44,24 @@
 #include "tool.h"
 #include "tool_lock.h"
 
-#include "obj_elem_draw.h"
-
+#define PCB_OBJ_CLASS_LOCK (PCB_OBJ_PSTK | PCB_OBJ_LINE | PCB_OBJ_ARC | PCB_OBJ_POLY | PCB_OBJ_SUBC | PCB_OBJ_TEXT | PCB_OBJ_LOCKED)
 
 void pcb_tool_lock_notify_mode(void)
 {
 	void *ptr1, *ptr2, *ptr3;
 	int type;
 	
-	type = pcb_search_screen(pcb_tool_note.X, pcb_tool_note.Y, PCB_TYPEMASK_LOCK, &ptr1, &ptr2, &ptr3);
-	if (type == PCB_TYPE_ELEMENT) {
-		pcb_element_t *element = (pcb_element_t *) ptr2;
+	type = pcb_search_screen(pcb_tool_note.X, pcb_tool_note.Y, PCB_OBJ_CLASS_LOCK, &ptr1, &ptr2, &ptr3);
 
-		PCB_FLAG_TOGGLE(PCB_FLAG_LOCK, element);
-		PCB_PIN_LOOP(element);
-		{
-			PCB_FLAG_TOGGLE(PCB_FLAG_LOCK, pin);
-			PCB_FLAG_CLEAR(PCB_FLAG_SELECTED, pin);
-		}
-		PCB_END_LOOP;
-		PCB_PAD_LOOP(element);
-		{
-			PCB_FLAG_TOGGLE(PCB_FLAG_LOCK, pad);
-			PCB_FLAG_CLEAR(PCB_FLAG_SELECTED, pad);
-		}
-		PCB_END_LOOP;
-		PCB_FLAG_CLEAR(PCB_FLAG_SELECTED, element);
-		/* always re-draw it since I'm too lazy
-		 * to tell if a selected flag changed
-		 */
-		pcb_elem_invalidate_draw(element);
-		pcb_draw();
-		pcb_hid_actionl("Report", "Object", NULL);
-	}
-	else if (type == PCB_TYPE_SUBC) {
+	if (type == PCB_OBJ_SUBC) {
 		pcb_subc_t *subc = (pcb_subc_t *)ptr2;
-		pcb_flag_change(PCB, PCB_CHGFLG_TOGGLE, PCB_FLAG_LOCK, PCB_TYPE_SUBC, ptr1, ptr2, ptr3);
+		pcb_flag_change(PCB, PCB_CHGFLG_TOGGLE, PCB_FLAG_LOCK, PCB_OBJ_SUBC, ptr1, ptr2, ptr3);
 
 		DrawSubc(subc);
 		pcb_draw();
 		pcb_hid_actionl("Report", "Subc", NULL);
 	}
-	else if (type != PCB_TYPE_NONE) {
+	else if (type != PCB_OBJ_VOID) {
 		pcb_text_t *thing = (pcb_text_t *) ptr3;
 		PCB_FLAG_TOGGLE(PCB_FLAG_LOCK, thing);
 		if (PCB_FLAG_TEST(PCB_FLAG_LOCK, thing)
@@ -101,10 +77,14 @@ void pcb_tool_lock_notify_mode(void)
 
 pcb_tool_t pcb_tool_lock = {
 	"lock", NULL, 100,
+	NULL,
+	NULL,
 	pcb_tool_lock_notify_mode,
 	NULL,
 	NULL,
 	NULL,
 	NULL,
-	NULL
+	NULL,
+	
+	pcb_true
 };
