@@ -43,8 +43,18 @@
 #include "tool.h"
 
 
+void pcb_tool_move_uninit(void)
+{
+	pcb_notify_crosshair_change(pcb_false);
+	pcb_crosshair.AttachedObject.Type = PCB_OBJ_VOID;
+	pcb_crosshair.AttachedObject.State = PCB_CH_STATE_FIRST;
+	pcb_notify_crosshair_change(pcb_true);
+}
+
 void pcb_tool_move_notify_mode(void)
 {
+	pcb_coord_t dx, dy;
+
 	switch (pcb_crosshair.AttachedObject.State) {
 		/* first notify, lookup object */
 	case PCB_CH_STATE_FIRST:
@@ -54,10 +64,10 @@ void pcb_tool_move_notify_mode(void)
 			pcb_crosshair.AttachedObject.Type =
 				pcb_search_screen(pcb_tool_note.X, pcb_tool_note.Y, types,
 										 &pcb_crosshair.AttachedObject.Ptr1, &pcb_crosshair.AttachedObject.Ptr2, &pcb_crosshair.AttachedObject.Ptr3);
-			if (pcb_crosshair.AttachedObject.Type != PCB_TYPE_NONE) {
-				if (PCB_FLAG_TEST(PCB_FLAG_LOCK, (pcb_pin_t *) pcb_crosshair.AttachedObject.Ptr2)) {
+			if (pcb_crosshair.AttachedObject.Type != PCB_OBJ_VOID) {
+				if (PCB_FLAG_TEST(PCB_FLAG_LOCK, (pcb_any_obj_t *) pcb_crosshair.AttachedObject.Ptr2)) {
 					pcb_message(PCB_MSG_WARNING, _("Sorry, the object is locked\n"));
-					pcb_crosshair.AttachedObject.Type = PCB_TYPE_NONE;
+					pcb_crosshair.AttachedObject.Type = PCB_OBJ_VOID;
 				}
 				else
 					pcb_tool_attach_for_copy(pcb_tool_note.X, pcb_tool_note.Y, pcb_true);
@@ -67,17 +77,18 @@ void pcb_tool_move_notify_mode(void)
 
 		/* second notify, move object */
 	case PCB_CH_STATE_SECOND:
-		pcb_move_obj_and_rubberband(pcb_crosshair.AttachedObject.Type,
-														pcb_crosshair.AttachedObject.Ptr1,
-														pcb_crosshair.AttachedObject.Ptr2,
-														pcb_crosshair.AttachedObject.Ptr3,
-														pcb_tool_note.X - pcb_crosshair.AttachedObject.X, pcb_tool_note.Y - pcb_crosshair.AttachedObject.Y);
-		pcb_crosshair_set_local_ref(0, 0, pcb_false);
-		pcb_board_set_changed_flag(pcb_true);
+		dx = pcb_tool_note.X - pcb_crosshair.AttachedObject.X;
+		dy = pcb_tool_note.Y - pcb_crosshair.AttachedObject.Y;
+		if ((dx != 0) || (dy != 0)) {
+			pcb_move_obj_and_rubberband(pcb_crosshair.AttachedObject.Type, pcb_crosshair.AttachedObject.Ptr1, pcb_crosshair.AttachedObject.Ptr2, pcb_crosshair.AttachedObject.Ptr3, dx, dy);
+			pcb_crosshair_set_local_ref(0, 0, pcb_false);
+			pcb_board_set_changed_flag(pcb_true);
+		}
 
 		/* reset identifiers */
-		pcb_crosshair.AttachedObject.Type = PCB_TYPE_NONE;
+		pcb_crosshair.AttachedObject.Type = PCB_OBJ_VOID;
 		pcb_crosshair.AttachedObject.State = PCB_CH_STATE_FIRST;
+		pcb_crosshair_set_range(0, 0, PCB->MaxWidth, PCB->MaxHeight);
 		break;
 	}
 }
@@ -92,7 +103,7 @@ void pcb_tool_move_release_mode (void)
 
 void pcb_tool_move_draw_attached(void)
 {
-	XORDrawMoveOrCopy();
+	pcb_xordraw_movecopy();
 }
 
 pcb_bool pcb_tool_move_undo_act(void)
@@ -105,10 +116,14 @@ pcb_bool pcb_tool_move_undo_act(void)
 
 pcb_tool_t pcb_tool_move = {
 	"move", NULL, 100,
+	NULL,
+	pcb_tool_move_uninit,
 	pcb_tool_move_notify_mode,
 	pcb_tool_move_release_mode,
 	NULL,
 	pcb_tool_move_draw_attached,
 	pcb_tool_move_undo_act,
-	NULL
+	NULL,
+	
+	pcb_true
 };
