@@ -44,6 +44,14 @@
 #include "tool.h"
 
 
+void pcb_tool_polyhole_uninit(void)
+{
+	pcb_notify_crosshair_change(pcb_false);
+	pcb_crosshair.AttachedPolygon.PointN = 0;
+	pcb_crosshair.AttachedLine.State = PCB_CH_STATE_FIRST;
+	pcb_notify_crosshair_change(pcb_true);
+}
+
 void pcb_tool_polyhole_notify_mode(void)
 {
 	switch (pcb_crosshair.AttachedObject.State) {
@@ -51,10 +59,10 @@ void pcb_tool_polyhole_notify_mode(void)
 	case PCB_CH_STATE_FIRST:
 		pcb_crosshair.AttachedPolygon_pts = 0;
 		pcb_crosshair.AttachedObject.Type =
-			pcb_search_screen(pcb_tool_note.X, pcb_tool_note.Y, PCB_TYPE_POLY,
+			pcb_search_screen(pcb_tool_note.X, pcb_tool_note.Y, PCB_OBJ_POLY,
 									 &pcb_crosshair.AttachedObject.Ptr1, &pcb_crosshair.AttachedObject.Ptr2, &pcb_crosshair.AttachedObject.Ptr3);
 
-		if (pcb_crosshair.AttachedObject.Type == PCB_TYPE_NONE) {
+		if (pcb_crosshair.AttachedObject.Type == PCB_OBJ_VOID) {
 			pcb_message(PCB_MSG_WARNING, "The first point of a polygon hole must be on a polygon.\n");
 			break; /* don't start doing anything if clicket out of polys */
 		}
@@ -62,7 +70,7 @@ void pcb_tool_polyhole_notify_mode(void)
 		if (PCB_FLAG_TEST(PCB_FLAG_LOCK, (pcb_poly_t *)
 									pcb_crosshair.AttachedObject.Ptr2)) {
 			pcb_message(PCB_MSG_WARNING, _("Sorry, the object is locked\n"));
-			pcb_crosshair.AttachedObject.Type = PCB_TYPE_NONE;
+			pcb_crosshair.AttachedObject.Type = PCB_OBJ_VOID;
 			break;
 		}
 		else
@@ -146,7 +154,7 @@ void pcb_tool_polyhole_draw_attached(void)
 
 	/* draw attached polygon only if in PCB_MODE_POLYGON or PCB_MODE_POLYGON_HOLE */
 	if (pcb_crosshair.AttachedPolygon.PointN > 1) {
-		XORPolygon(&pcb_crosshair.AttachedPolygon, 0, 0, 1);
+		pcb_xordraw_poly(&pcb_crosshair.AttachedPolygon, 0, 0, 1);
 	}
 }
 
@@ -171,10 +179,14 @@ pcb_bool pcb_tool_polyhole_redo_act(void)
 
 pcb_tool_t pcb_tool_polyhole = {
 	"polyhole", NULL, 100,
+	NULL,
+	pcb_tool_polyhole_uninit,
 	pcb_tool_polyhole_notify_mode,
 	NULL,
 	pcb_tool_polyhole_adjust_attached_objects,
 	pcb_tool_polyhole_draw_attached,
 	pcb_tool_polyhole_undo_act,
-	pcb_tool_polyhole_redo_act
+	pcb_tool_polyhole_redo_act,
+	
+	pcb_false
 };
