@@ -46,6 +46,13 @@
 #include "obj_poly_draw.h"
 
 
+void pcb_tool_rectangle_uninit(void)
+{
+	pcb_notify_crosshair_change(pcb_false);
+	pcb_crosshair.AttachedBox.State = PCB_CH_STATE_FIRST;
+	pcb_notify_crosshair_change(pcb_true);
+}
+
 void pcb_tool_rectangle_notify_mode(void)
 {
 	/* do update of position */
@@ -73,7 +80,7 @@ void pcb_tool_rectangle_notify_mode(void)
 																								 2 * conf_core.design.clearance,
 																								 pcb_flag_make(flags))) != NULL) {
 			pcb_obj_add_attribs(polygon, PCB->pen_attr);
-			pcb_undo_add_obj_to_create(PCB_TYPE_POLY, layer, polygon, polygon);
+			pcb_undo_add_obj_to_create(PCB_OBJ_POLY, layer, polygon, polygon);
 			pcb_undo_inc_serial();
 			pcb_poly_invalidate_draw(layer, polygon);
 			pcb_draw();
@@ -107,10 +114,14 @@ pcb_bool pcb_tool_rectangle_anydo_act(void)
 
 pcb_tool_t pcb_tool_rectangle = {
 	"rectangle", NULL, 100,
+	NULL,
+	pcb_tool_rectangle_uninit,
 	pcb_tool_rectangle_notify_mode,
 	NULL,
 	pcb_tool_rectangle_adjust_attached_objects,
 	NULL,
 	pcb_tool_rectangle_anydo_act,
-	pcb_tool_rectangle_anydo_act
+	pcb_tool_rectangle_anydo_act,
+	
+	pcb_false
 };
