@@ -51,21 +51,20 @@ void pcb_tool_remove_notify_mode(void)
 	pcb_any_obj_t *obj;
 	int type;
 	
-	if ((type = pcb_search_screen(pcb_tool_note.X, pcb_tool_note.Y, PCB_REMOVE_TYPES | PCB_LOOSE_SUBC, &ptr1, &ptr2, &ptr3)) != PCB_TYPE_NONE) {
+	if ((type = pcb_search_screen(pcb_tool_note.X, pcb_tool_note.Y, PCB_REMOVE_TYPES | PCB_LOOSE_SUBC, &ptr1, &ptr2, &ptr3)) != PCB_OBJ_VOID) {
 		if (PCB_FLAG_TEST(PCB_FLAG_LOCK, (pcb_line_t *) ptr2)) {
 			pcb_message(PCB_MSG_WARNING, _("Sorry, the object is locked\n"));
 			return;
 		}
 
-		if ((type == PCB_TYPE_SUBC) && (PCB->is_footprint)) {
-			pcb_message(PCB_MSG_WARNING, "Can not remove the subcircuit being edited in the footprint edit mode\n");
-			return;
+		if (type == PCB_OBJ_SUBC) {
+			if(PCB->is_footprint) {
+				pcb_message(PCB_MSG_WARNING, "Can not remove the subcircuit being edited in the footprint edit mode\n");
+				return;
+			}
+			pcb_event(PCB_EVENT_RUBBER_REMOVE_SUBC, "ppp", ptr1, ptr2, ptr3);
 		}
 
-#warning subc TODO: rewrite this to subc when elements are removed
-		if (type == PCB_TYPE_ELEMENT)
-			pcb_event(PCB_EVENT_RUBBER_REMOVE_ELEMENT, "ppp", ptr1, ptr2, ptr3);
-
 		/* preserve original parent over the board layer pcb_search_screen operated on -
 		   this is essential for undo: it needs to put back the object to the original
 		   layer (e.g. inside a subc) instead of on the board layer */
@@ -83,10 +82,14 @@ void pcb_tool_remove_notify_mode(void)
 
 pcb_tool_t pcb_tool_remove = {
 	"remove", NULL, 100,
+	NULL,
+	NULL,
 	pcb_tool_remove_notify_mode,
 	NULL,
 	NULL,
 	NULL,
 	NULL,
-	NULL
+	NULL,
+	
+	pcb_true
 };
