@@ -60,7 +60,7 @@ void pcb_tool_text_notify_mode(void)
 				flag |= PCB_FLAG_ONSOLDER;
 			if ((text = pcb_text_new(pcb_loose_subc_layer(PCB, CURRENT), pcb_font(PCB, conf_core.design.text_font_id, 1), pcb_tool_note.X,
 																pcb_tool_note.Y, 0, conf_core.design.text_scale, string, pcb_flag_make(flag))) != NULL) {
-				pcb_undo_add_obj_to_create(PCB_TYPE_TEXT, CURRENT, text, text);
+				pcb_undo_add_obj_to_create(PCB_OBJ_TEXT, CURRENT, text, text);
 				pcb_undo_inc_serial();
 				pcb_text_invalidate_draw(CURRENT, text);
 				pcb_draw();
@@ -93,11 +93,15 @@ void pcb_tool_text_draw_attached(void)
 
 pcb_tool_t pcb_tool_text = {
 	"text", NULL, 100,
+	NULL,
+	NULL,
 	pcb_tool_text_notify_mode,
 	NULL,
 	NULL,
 	pcb_tool_text_draw_attached,
 	NULL,
-	NULL
+	NULL,
+	
+	pcb_false
 };
 
