@@ -44,21 +44,6 @@
 #include "thermal.h"
 #include "tool.h"
 
-static void tool_thermal_on_pinvia(int type, void *ptr1, void *ptr2, void *ptr3)
-{
-	if (pcb_gui->shift_is_pressed()) {
-		int tstyle = PCB_FLAG_THERM_GET(INDEXOFCURRENT, (pcb_pin_t *) ptr3);
-		tstyle++;
-		if (tstyle > 5)
-			tstyle = 1;
-		pcb_chg_obj_thermal(type, ptr1, ptr2, ptr3, tstyle, INDEXOFCURRENT);
-	}
-	else if (PCB_FLAG_THERM_GET(INDEXOFCURRENT, (pcb_pin_t *) ptr3))
-		pcb_chg_obj_thermal(type, ptr1, ptr2, ptr3, 0, INDEXOFCURRENT);
-	else
-		pcb_chg_obj_thermal(type, ptr1, ptr2, ptr3, PCB->ThermStyle, INDEXOFCURRENT);
-}
-
 void pcb_tool_thermal_on_pstk(pcb_pstk_t *ps, unsigned long lid)
 {
 	unsigned char *th, newth = 0;
@@ -100,7 +85,7 @@ void pcb_tool_thermal_on_pstk(pcb_pstk_t *ps, unsigned long lid)
 			newth = cycle[0]; /* new thermal, use default */
 	}
 
-	pcb_chg_obj_thermal(PCB_TYPE_PSTK, ps, ps, ps, newth, lid);
+	pcb_chg_obj_thermal(PCB_OBJ_PSTK, ps, ps, ps, newth, lid);
 }
 
 
@@ -109,21 +94,23 @@ void pcb_tool_thermal_notify_mode(void)
 	void *ptr1, *ptr2, *ptr3;
 	int type;
 
-	if (((type = pcb_search_screen(pcb_tool_note.X, pcb_tool_note.Y, PCB_TYPEMASK_PIN, &ptr1, &ptr2, &ptr3)) != PCB_TYPE_NONE)
-			&& !PCB_FLAG_TEST(PCB_FLAG_HOLE, (pcb_pin_t *) ptr3)) {
-		if (type == PCB_TYPE_PSTK)
+	if (((type = pcb_search_screen(pcb_tool_note.X, pcb_tool_note.Y, PCB_OBJ_CLASS_PIN, &ptr1, &ptr2, &ptr3)) != PCB_OBJ_VOID)
+			&& !PCB_FLAG_TEST(PCB_FLAG_HOLE, (pcb_any_obj_t *) ptr3)) {
+		if (type == PCB_OBJ_PSTK)
 			pcb_tool_thermal_on_pstk((pcb_pstk_t *)ptr2, INDEXOFCURRENT);
-		else
-			tool_thermal_on_pinvia(type, ptr1, ptr2, ptr3);
 	}
 }
 
 pcb_tool_t pcb_tool_thermal = {
 	"thermal", NULL, 100,
+	NULL,
+	NULL,
 	pcb_tool_thermal_notify_mode,
 	NULL,
 	NULL,
 	NULL,
 	NULL,
-	NULL
+	NULL,
+	
+	pcb_false
 };
