@@ -45,7 +45,6 @@
 #include "tool_thermal.h"
 #include "undo.h"
 
-#include "obj_pinvia_draw.h"
 #include "obj_pstk_draw.h"
 
 #warning padstack TODO: remove this when via is removed and the padstack is created from style directly
@@ -54,12 +53,17 @@
 void pcb_tool_via_notify_mode(void)
 {
 
-	if (!PCB->ViaOn) {
+	if (!PCB->pstk_on) {
 		pcb_message(PCB_MSG_WARNING, _("You must turn via visibility on before\n" "you can place vias\n"));
 		return;
 	}
 
-#warning padstack TODO: do not work in compatibility mode, use a padstack proto
+	if (conf_core.design.via_drilling_hole >= conf_core.design.via_thickness) {
+		pcb_message(PCB_MSG_ERROR, "Can't place via: invalid via geometry (hole too large for via size)\n");
+		return;
+	}
+
+#warning pdstk TODO #21: do not work in comp mode, use a pstk proto - scconfig also has TODO #21, fix it there too
 	{
 		pcb_pstk_t *ps = pcb_pstk_new_compat_via(PCB->Data, pcb_tool_note.X, pcb_tool_note.Y,
 			conf_core.design.via_drilling_hole, conf_core.design.via_thickness, conf_core.design.clearance,
@@ -68,7 +72,7 @@ void pcb_tool_via_notify_mode(void)
 			return;
 
 		pcb_obj_add_attribs(ps, PCB->pen_attr);
-		pcb_undo_add_obj_to_create(PCB_TYPE_PSTK, ps, ps, ps);
+		pcb_undo_add_obj_to_create(PCB_OBJ_PSTK, ps, ps, ps);
 
 		if (pcb_gui->shift_is_pressed())
 			pcb_tool_thermal_on_pstk(ps, INDEXOFCURRENT);
@@ -79,34 +83,37 @@ void pcb_tool_via_notify_mode(void)
 	}
 }
 
+static void xor_draw_fake_via(pcb_coord_t x, pcb_coord_t y, pcb_coord_t dia, pcb_coord_t clearance)
+{
+	pcb_coord_t r = (dia/2)+clearance;
+	pcb_gui->set_line_cap(pcb_crosshair.GC, Round_Cap);
+	pcb_gui->set_line_width(pcb_crosshair.GC, 0);
+	pcb_gui->draw_arc(pcb_crosshair.GC, x, y, r, r, 0, 360);
+}
+
+
 void pcb_tool_via_draw_attached(void)
 {
-	/* Make a dummy via structure to draw from */
-	pcb_pin_t via;
-	via.X = pcb_crosshair.X;
-	via.Y = pcb_crosshair.Y;
-	via.Thickness = conf_core.design.via_thickness;
-	via.Clearance = 2 * conf_core.design.clearance;
-	via.DrillingHole = conf_core.design.via_drilling_hole;
-	via.Mask = 0;
-	via.Flags = pcb_no_flags();
-
-	pcb_gui->thindraw_pcb_pv(pcb_crosshair.GC, pcb_crosshair.GC, &via, pcb_true, pcb_false);
-
+#warning padstack TODO: replace this when route style has a prototype
+	xor_draw_fake_via(pcb_crosshair.X, pcb_crosshair.Y, conf_core.design.via_thickness, 0);
 	if (conf_core.editor.show_drc) {
 		/* XXX: Naughty cheat - use the mask to draw DRC clearance! */
 		pcb_gui->set_color(pcb_crosshair.GC, conf_core.appearance.color.cross);
-		XORDrawPinViaDRCOutline(&via,PCB->Bloat);
+		xor_draw_fake_via(pcb_crosshair.X, pcb_crosshair.Y, conf_core.design.via_thickness, conf_core.design.clearance);
 		pcb_gui->set_color(pcb_crosshair.GC, conf_core.appearance.color.crosshair);
 	}
 }
 
 pcb_tool_t pcb_tool_via = {
 	"via", NULL, 100,
+	NULL,
+	NULL,
 	pcb_tool_via_notify_mode,
 	NULL,
 	NULL,
 	pcb_tool_via_draw_attached,
 	NULL,
-	NULL
+	NULL,
+	
+	pcb_false
 };
