@@ -64,7 +64,6 @@
 #include "compat_misc.h"
 #include "compat_nls.h"
 
-#include "obj_elem_draw.h"
 #include "obj_poly_draw.h"
 
 static pcb_bool between_increment_and_restore = pcb_false;
@@ -82,10 +81,7 @@ void *pcb_undo_alloc(pcb_board_t *pcb, const uundo_oper_t *oper, size_t data_len
 
 int pcb_undo(pcb_bool draw)
 {
-	int res, unique;
-
-	unique = conf_core.editor.unique_names;
-	conf_force_set_bool(conf_core.editor.unique_names, 0);
+	int res;
 
 	pcb_undo_and_draw = draw;
 
@@ -123,9 +119,6 @@ int pcb_undo(pcb_bool draw)
 	else if (pcb_undo_and_draw)
 		pcb_draw();
 
-	/* restore the unique flag setting */
-	conf_force_set_bool(conf_core.editor.unique_names, unique);
-
 	return res;
 }
 
@@ -158,7 +151,9 @@ int pcb_redo(pcb_bool draw)
 	}
 
 	pcb_undo_lock(); /* lock undo module to prevent from loops */
+	uundo_freeze_add(&pcb_uundo);
 	res = uundo_redo(&pcb_uundo);
+	uundo_unfreeze_add(&pcb_uundo);
 	pcb_undo_unlock();
 
 	if (res != 0)
@@ -274,3 +269,25 @@ void undo_dump(void)
 	uundo_dump(&pcb_uundo, NULL, NULL);
 }
 #endif
+
+void pcb_undo_freeze_serial(void)
+{
+	uundo_freeze_serial(&pcb_uundo);
+}
+
+void pcb_undo_unfreeze_serial(void)
+{
+	uundo_unfreeze_serial(&pcb_uundo);
+}
+
+
+void pcb_undo_freeze_add(void)
+{
+	uundo_freeze_add(&pcb_uundo);
+}
+
+void pcb_undo_unfreeze_add(void)
+{
+	uundo_unfreeze_add(&pcb_uundo);
+}
+
