@@ -64,7 +64,6 @@
 #include "compat_misc.h"
 #include "compat_nls.h"
 
-#include "obj_elem_draw.h"
 #include "obj_poly_draw.h"
 #include "obj_subc_parent.h"
 
@@ -87,16 +86,14 @@ static pcb_bool UndoRemoveContour(UndoListTypePtr);
 static pcb_bool UndoInsertContour(UndoListTypePtr);
 static pcb_bool UndoMoveToLayer(UndoListTypePtr);
 static pcb_bool UndoFlag(UndoListTypePtr);
-static pcb_bool UndoMirror(UndoListTypePtr);
 static pcb_bool UndoOtherSide(UndoListTypePtr);
 static pcb_bool UndoChangeSize(UndoListTypePtr);
-static pcb_bool UndoChange2ndSize(UndoListTypePtr);
 static pcb_bool UndoChangeAngles(UndoListTypePtr);
 static pcb_bool UndoChangeRadii(UndoListTypePtr);
 static pcb_bool UndoChangeClearSize(UndoListTypePtr);
-static pcb_bool UndoChangeMaskSize(UndoListTypePtr);
 static pcb_bool UndoClearPoly(UndoListTypePtr);
 
+#define PCB_OBJECT_ID(p) (((pcb_any_obj_t *) p)->ID)
 
 static void pcb_undo_old_free(void *udata);
 static int pcb_undo_old_undo(void *udata);
@@ -152,7 +149,7 @@ static pcb_bool UndoRotate90(UndoListTypePtr Entry)
 
 	/* lookup entry by it's ID */
 	type = pcb_search_obj_by_id(PCB->Data, &ptr1, &ptr2, &ptr3, Entry->ID, Entry->Kind);
-	if (type != PCB_TYPE_NONE) {
+	if (type != PCB_OBJ_VOID) {
 		pcb_obj_rotate90(type, ptr1, ptr2, ptr3,
 								 Entry->Data.Rotate.CenterX, Entry->Data.Rotate.CenterY, (4 - Entry->Data.Rotate.Steps) & 0x03);
 		Entry->Data.Rotate.Steps = (4 - Entry->Data.Rotate.Steps) & 0x03;
@@ -172,7 +169,7 @@ static pcb_bool UndoRotate(UndoListTypePtr Entry)
 
 	/* lookup entry by it's ID */
 	type = pcb_search_obj_by_id(PCB->Data, &ptr1, &ptr2, &ptr3, Entry->ID, Entry->Kind);
-	if (type != PCB_TYPE_NONE) {
+	if (type != PCB_OBJ_VOID) {
 		pcb_obj_rotate(type, ptr1, ptr2, ptr3, Entry->Data.Rotate.CenterX, Entry->Data.Rotate.CenterY, -(Entry->Data.Angle));
 		Entry->Data.Angle = -(Entry->Data.Angle);
 		return pcb_true;
@@ -190,7 +187,7 @@ static pcb_bool UndoClearPoly(UndoListTypePtr Entry)
 	int type;
 
 	type = pcb_search_obj_by_id(PCB->Data, &ptr1, &ptr2, &ptr3, Entry->ID, Entry->Kind);
-	if (type != PCB_TYPE_NONE) {
+	if (type != PCB_OBJ_VOID) {
 		if (Entry->Data.ClearPoly.Clear)
 			pcb_poly_restore_to_poly(PCB->Data, type, Entry->Data.ClearPoly.Layer, ptr3);
 		else
@@ -212,54 +209,13 @@ static pcb_bool UndoChangeName(UndoListTypePtr Entry)
 
 	/* lookup entry by it's ID */
 	type = pcb_search_obj_by_id(PCB->Data, &ptr1, &ptr2, &ptr3, Entry->ID, Entry->Kind);
-	if (type != PCB_TYPE_NONE) {
+	if (type != PCB_OBJ_VOID) {
 		Entry->Data.ChangeName.Name = (char *) (pcb_chg_obj_name(type, ptr1, ptr2, ptr3, Entry->Data.ChangeName.Name));
 		return pcb_true;
 	}
 	return pcb_false;
 }
 
-/* ---------------------------------------------------------------------------
- * recovers an object from a 'change oinnum' operation
- * returns pcb_true if anything has been recovered
- */
-static pcb_bool UndoChangePinnum(UndoListTypePtr Entry)
-{
-	void *ptr1, *ptr2, *ptr3;
-	int type;
-
-	/* lookup entry by it's ID */
-	type = pcb_search_obj_by_id(PCB->Data, &ptr1, &ptr2, &ptr3, Entry->ID, Entry->Kind);
-	if (type != PCB_TYPE_NONE) {
-		Entry->Data.ChangeName.Name = (char *) (pcb_chg_obj_pinnum(type, ptr1, ptr2, ptr3, Entry->Data.ChangeName.Name));
-		return pcb_true;
-	}
-	return pcb_false;
-}
-
-/* ---------------------------------------------------------------------------
- * recovers an object from a 2ndSize change operation
- */
-static pcb_bool UndoChange2ndSize(UndoListTypePtr Entry)
-{
-	void *ptr1, *ptr2, *ptr3;
-	int type;
-	pcb_coord_t swap;
-
-	/* lookup entry by ID */
-	type = pcb_search_obj_by_id(PCB->Data, &ptr1, &ptr2, &ptr3, Entry->ID, Entry->Kind);
-	if (type != PCB_TYPE_NONE) {
-		swap = ((pcb_pin_t *) ptr2)->DrillingHole;
-		if (pcb_undo_and_draw)
-			pcb_erase_obj(type, ptr1, ptr2);
-		((pcb_pin_t *) ptr2)->DrillingHole = Entry->Data.Size;
-		Entry->Data.Size = swap;
-		pcb_draw_obj((pcb_any_obj_t *)ptr2);
-		return pcb_true;
-	}
-	return pcb_false;
-}
-
 /* ---------------------------------------------------------------------------
  * recovers an object from a ChangeAngles change operation
  */
@@ -271,7 +227,7 @@ static pcb_bool UndoChangeAngles(UndoListTypePtr Entry)
 
 	/* lookup entry by ID */
 	type = pcb_search_obj_by_id(PCB->Data, &ptr1, &ptr2, &ptr3, Entry->ID, Entry->Kind);
-	if (type == PCB_TYPE_ARC) {
+	if (type == PCB_OBJ_ARC) {
 		pcb_layer_t *Layer = (pcb_layer_t *) ptr1;
 		pcb_arc_t *a = (pcb_arc_t *) ptr2;
 		pcb_r_delete_entry(Layer->arc_tree, (pcb_box_t *) a);
@@ -302,7 +258,7 @@ static pcb_bool UndoChangeRadii(UndoListTypePtr Entry)
 
 	/* lookup entry by ID */
 	type = pcb_search_obj_by_id(PCB->Data, &ptr1, &ptr2, &ptr3, Entry->ID, Entry->Kind);
-	if (type == PCB_TYPE_ARC) {
+	if (type == PCB_OBJ_ARC) {
 		pcb_layer_t *Layer = (pcb_layer_t *) ptr1;
 		pcb_arc_t *a = (pcb_arc_t *) ptr2;
 		pcb_r_delete_entry(Layer->arc_tree, (pcb_box_t *) a);
@@ -333,12 +289,12 @@ static pcb_bool UndoChangeClearSize(UndoListTypePtr Entry)
 
 	/* lookup entry by ID */
 	type = pcb_search_obj_by_id(PCB->Data, &ptr1, &ptr2, &ptr3, Entry->ID, Entry->Kind);
-	if (type != PCB_TYPE_NONE) {
-		swap = ((pcb_pin_t *) ptr2)->Clearance;
+	if (type != PCB_OBJ_VOID) {
+		swap = ((pcb_line_t *) ptr2)->Clearance;
 		pcb_poly_restore_to_poly(PCB->Data, type, ptr1, ptr2);
 		if (pcb_undo_and_draw)
 			pcb_erase_obj(type, ptr1, ptr2);
-		((pcb_pin_t *) ptr2)->Clearance = Entry->Data.Size;
+		((pcb_line_t *) ptr2)->Clearance = Entry->Data.Size;
 		pcb_poly_clear_from_poly(PCB->Data, type, ptr1, ptr2);
 		Entry->Data.Size = swap;
 		if (pcb_undo_and_draw)
@@ -348,34 +304,6 @@ static pcb_bool UndoChangeClearSize(UndoListTypePtr Entry)
 	return pcb_false;
 }
 
-/* ---------------------------------------------------------------------------
- * recovers an object from a mask size change operation
- */
-static pcb_bool UndoChangeMaskSize(UndoListTypePtr Entry)
-{
-	void *ptr1, *ptr2, *ptr3;
-	int type;
-	pcb_coord_t swap;
-
-	/* lookup entry by ID */
-	type = pcb_search_obj_by_id(PCB->Data, &ptr1, &ptr2, &ptr3, Entry->ID, Entry->Kind);
-	if (type & (PCB_TYPE_VIA | PCB_TYPE_PIN | PCB_TYPE_PAD)) {
-		swap = (type == PCB_TYPE_PAD ? ((pcb_pad_t *) ptr2)->Mask : ((pcb_pin_t *) ptr2)->Mask);
-		if (pcb_undo_and_draw)
-			pcb_erase_obj(type, ptr1, ptr2);
-		if (type == PCB_TYPE_PAD)
-			((pcb_pad_t *) ptr2)->Mask = Entry->Data.Size;
-		else
-			((pcb_pin_t *) ptr2)->Mask = Entry->Data.Size;
-		Entry->Data.Size = swap;
-		if (pcb_undo_and_draw)
-			pcb_draw_obj((pcb_any_obj_t *)ptr2);
-		return pcb_true;
-	}
-	return pcb_false;
-}
-
-
 /* ---------------------------------------------------------------------------
  * recovers an object from a Size change operation
  */
@@ -387,19 +315,15 @@ static pcb_bool UndoChangeSize(UndoListTypePtr Entry)
 
 	/* lookup entry by ID */
 	type = pcb_search_obj_by_id(PCB->Data, &ptr1, &ptr2, &ptr3, Entry->ID, Entry->Kind);
-		if (type == PCB_TYPE_ELEMENT_NAME)
-			ptr1e = NULL;
-		else
-			ptr1e = ptr1;
+	ptr1e = ptr1;
 
-	if (type != PCB_TYPE_NONE) {
-		/* Wow! can any object be treated as a pin type for size change?? */
-		/* pins, vias, lines, and arcs can. Text can't but it has it's own mechanism */
-		swap = ((pcb_pin_t *) ptr2)->Thickness;
+	if (type != PCB_OBJ_VOID) {
+		/* Size change for lines and arcs can. Text has it's own mechanism */
+		swap = ((pcb_line_t *) ptr2)->Thickness;
 		pcb_poly_restore_to_poly(PCB->Data, type, ptr1, ptr2);
 		if ((pcb_undo_and_draw) && (ptr1e != NULL))
 			pcb_erase_obj(type, ptr1e, ptr2);
-		((pcb_pin_t *) ptr2)->Thickness = Entry->Data.Size;
+		((pcb_line_t *) ptr2)->Thickness = Entry->Data.Size;
 		Entry->Data.Size = swap;
 		pcb_poly_clear_from_poly(PCB->Data, type, ptr1, ptr2);
 		if (pcb_undo_and_draw)
@@ -421,19 +345,15 @@ static pcb_bool UndoFlag(UndoListTypePtr Entry)
 
 	/* lookup entry by ID */
 	type = pcb_search_obj_by_id(PCB->Data, &ptr1, &ptr2, &ptr3, Entry->ID, Entry->Kind);
-	if (type != PCB_TYPE_NONE) {
+	if (type != PCB_OBJ_VOID) {
 		pcb_flag_t f1, f2;
-		pcb_pin_t *pin = (pcb_pin_t *) ptr2;
+		pcb_any_obj_t *obj = (pcb_any_obj_t *)ptr2;
 
-		if ((type == PCB_TYPE_ELEMENT) || (type == PCB_TYPE_ELEMENT_NAME))
-			ptr1e = NULL;
-		else
-			ptr1e = ptr1;
-
-		swap = pin->Flags;
+		ptr1e = ptr1;
+		swap = obj->Flags;
 
 		must_redraw = 0;
-		f1 = pcb_flag_mask(pin->Flags, ~DRAW_FLAGS);
+		f1 = pcb_flag_mask(obj->Flags, ~DRAW_FLAGS);
 		f2 = pcb_flag_mask(Entry->Data.Flags, ~DRAW_FLAGS);
 
 		if (!PCB_FLAG_EQ(f1, f2))
@@ -442,7 +362,7 @@ static pcb_bool UndoFlag(UndoListTypePtr Entry)
 		if (pcb_undo_and_draw && must_redraw && (ptr1e != NULL))
 			pcb_erase_obj(type, ptr1e, ptr2);
 
-		pin->Flags = Entry->Data.Flags;
+		obj->Flags = Entry->Data.Flags;
 
 		Entry->Data.Flags = swap;
 
@@ -451,31 +371,7 @@ static pcb_bool UndoFlag(UndoListTypePtr Entry)
 		return pcb_true;
 	}
 	pcb_message(PCB_MSG_ERROR, "hace Internal error: Can't find ID %d type %08x\n", Entry->ID, Entry->Kind);
-	pcb_message(PCB_MSG_ERROR, "for UndoFlag Operation. Previous flags: %s\n", pcb_strflg_f2s(Entry->Data.Flags, 0, NULL));
-	return pcb_false;
-}
-
-/* ---------------------------------------------------------------------------
- * recovers an object from a mirror operation
- * returns pcb_true if anything has been recovered
- */
-static pcb_bool UndoMirror(UndoListTypePtr Entry)
-{
-	void *ptr1, *ptr2, *ptr3;
-	int type;
-
-	/* lookup entry by ID */
-	type = pcb_search_obj_by_id(PCB->Data, &ptr1, &ptr2, &ptr3, Entry->ID, Entry->Kind);
-	if (type == PCB_TYPE_ELEMENT) {
-		pcb_element_t *element = (pcb_element_t *) ptr3;
-		if (pcb_undo_and_draw)
-			pcb_elem_invalidate_erase(element);
-		pcb_element_mirror(PCB->Data, element, Entry->Data.Move.DY);
-		if (pcb_undo_and_draw)
-			pcb_elem_invalidate_draw(element);
-		return pcb_true;
-	}
-	pcb_message(PCB_MSG_ERROR, "hace Internal error: UndoMirror on object type %d\n", type);
+	pcb_message(PCB_MSG_ERROR, "for UndoFlag Operation. Previous flags: %s\n", pcb_strflg_f2s(Entry->Data.Flags, 0, NULL, 0));
 	return pcb_false;
 }
 
@@ -490,7 +386,7 @@ static pcb_bool UndoOtherSide(UndoListTypePtr Entry)
 
 	/* lookup entry by ID */
 	type = pcb_search_obj_by_id(PCB->Data, &ptr1, &ptr2, &ptr3, Entry->ID, Entry->Kind);
-	if (type == PCB_TYPE_SUBC) {
+	if (type == PCB_OBJ_SUBC) {
 		pcb_subc_t *subc = (pcb_subc_t *)ptr3;
 		if (pcb_undo_and_draw)
 			EraseSubc(subc);
@@ -514,7 +410,8 @@ static pcb_bool UndoCopyOrCreate(UndoListTypePtr Entry)
 
 	/* lookup entry by it's ID */
 	type = pcb_search_obj_by_id(PCB->Data, &ptr1, &ptr2, &ptr3, Entry->ID, Entry->Kind);
-	if (type != PCB_TYPE_NONE) {
+	if (type != PCB_OBJ_VOID) {
+		Removed *r = &Entry->Data.Removed;
 		if (!pcb_removelist)
 			pcb_removelist = pcb_buffer_new(NULL);
 		if (pcb_undo_and_draw)
@@ -522,6 +419,7 @@ static pcb_bool UndoCopyOrCreate(UndoListTypePtr Entry)
 		/* in order to make this re-doable we move it to the pcb_removelist */
 		pcb_move_obj_to_buffer(PCB, pcb_removelist, PCB->Data, type, ptr1, ptr2, ptr3);
 		Entry->Type = PCB_UNDO_REMOVE;
+		r->p_subc_id = 0;
 		return pcb_true;
 	}
 	return pcb_false;
@@ -538,7 +436,7 @@ static pcb_bool UndoMove(UndoListTypePtr Entry)
 
 	/* lookup entry by it's ID */
 	type = pcb_search_obj_by_id(PCB->Data, &ptr1, &ptr2, &ptr3, Entry->ID, Entry->Kind);
-	if (type != PCB_TYPE_NONE) {
+	if (type != PCB_OBJ_VOID) {
 		pcb_move_obj(type, ptr1, ptr2, ptr3, -Entry->Data.Move.DX, -Entry->Data.Move.DY);
 		Entry->Data.Move.DX *= -1;
 		Entry->Data.Move.DY *= -1;
@@ -560,10 +458,10 @@ static pcb_bool UndoRemove(UndoListTypePtr Entry)
 
 	/* lookup entry by it's ID */
 	type = pcb_search_obj_by_id(pcb_removelist, &ptr1, &ptr2, &ptr3, Entry->ID, Entry->Kind);
-	if (type != PCB_TYPE_NONE) {
+	if (type != PCB_OBJ_VOID) {
 		if (r->p_subc_id > 0) { /* need to use a subc layer - putting back a floater */
 			void *p1, *p2, *p3;
-			if (pcb_search_obj_by_id(PCB->Data, &p1, &p2, &p3, r->p_subc_id, PCB_TYPE_SUBC) != 0) {
+			if (pcb_search_obj_by_id(PCB->Data, &p1, &p2, &p3, r->p_subc_id, PCB_OBJ_SUBC) != 0) {
 				pcb_subc_t *subc = p2;
 				if (r->p_subc_layer < subc->data->LayerN) {
 					data = subc->data;
@@ -592,7 +490,7 @@ static pcb_bool UndoMoveToLayer(UndoListTypePtr Entry)
 
 	/* lookup entry by it's ID */
 	type = pcb_search_obj_by_id(PCB->Data, &ptr1, &ptr2, &ptr3, Entry->ID, Entry->Kind);
-	if (type != PCB_TYPE_NONE) {
+	if (type != PCB_OBJ_VOID) {
 		swap = pcb_layer_id(PCB->Data, (pcb_layer_t *) ptr1);
 		pcb_move_obj_to_layer(type, ptr1, ptr2, ptr3, LAYER_PTR(Entry->Data.MoveToLayer.OriginalLayer), pcb_true);
 		Entry->Data.MoveToLayer.OriginalLayer = swap;
@@ -613,15 +511,15 @@ static pcb_bool UndoRemovePoint(UndoListTypePtr Entry)
 	int type;
 
 	/* lookup entry (polygon not point was saved) by it's ID */
-	assert(Entry->Kind == PCB_TYPE_POLY);
+	assert(Entry->Kind == PCB_OBJ_POLY);
 	type = pcb_search_obj_by_id(PCB->Data, (void **) &layer, (void **) &polygon, &ptr3, Entry->ID, Entry->Kind);
 	switch (type) {
-	case PCB_TYPE_POLY:						/* restore the removed point */
+	case PCB_OBJ_POLY:						/* restore the removed point */
 		{
 			/* recover the point */
 			if (pcb_undo_and_draw && layer->meta.real.vis)
 				pcb_poly_invalidate_erase(polygon);
-			pcb_insert_point_in_object(PCB_TYPE_POLY, layer, polygon,
+			pcb_insert_point_in_object(PCB_OBJ_POLY, layer, polygon,
 														&Entry->Data.RemovedPoint.Index,
 														Entry->Data.RemovedPoint.X,
 														Entry->Data.RemovedPoint.Y, pcb_true, Entry->Data.RemovedPoint.last_in_contour);
@@ -631,7 +529,7 @@ static pcb_bool UndoRemovePoint(UndoListTypePtr Entry)
 				pcb_poly_invalidate_draw(layer, polygon);
 			Entry->Type = PCB_UNDO_INSERT_POINT;
 			Entry->ID = Entry->Data.RemovedPoint.ID;
-			Entry->Kind = PCB_TYPE_POLY_POINT;
+			Entry->Kind = PCB_OBJ_POLY_POINT;
 			return pcb_true;
 		}
 
@@ -654,11 +552,11 @@ static pcb_bool UndoInsertPoint(UndoListTypePtr Entry)
 	pcb_cardinal_t hole;
 	pcb_bool last_in_contour = pcb_false;
 
-	assert(Entry->Kind == PCB_TYPE_POLY_POINT);
+	assert(Entry->Kind == PCB_OBJ_POLY_POINT);
 	/* lookup entry by it's ID */
 	type = pcb_search_obj_by_id(PCB->Data, (void **) &layer, (void **) &polygon, (void **) &pnt, Entry->ID, Entry->Kind);
 	switch (type) {
-	case PCB_TYPE_POLY_POINT:			/* removes an inserted polygon point */
+	case PCB_OBJ_POLY_POINT:			/* removes an inserted polygon point */
 		{
 			if (pcb_undo_and_draw && layer->meta.real.vis)
 				pcb_poly_invalidate_erase(polygon);
@@ -679,10 +577,10 @@ static pcb_bool UndoInsertPoint(UndoListTypePtr Entry)
 			Entry->Data.RemovedPoint.Y = pnt->Y;
 			Entry->Data.RemovedPoint.ID = pnt->ID;
 			Entry->ID = polygon->ID;
-			Entry->Kind = PCB_TYPE_POLY;
+			Entry->Kind = PCB_OBJ_POLY;
 			Entry->Type = PCB_UNDO_REMOVE_POINT;
 			Entry->Data.RemovedPoint.Index = point_idx;
-			pcb_destroy_object(PCB->Data, PCB_TYPE_POLY_POINT, layer, polygon, pnt);
+			pcb_destroy_object(PCB->Data, PCB_OBJ_POLY_POINT, layer, polygon, pnt);
 			if (pcb_undo_and_draw && layer->meta.real.vis)
 				pcb_poly_invalidate_draw(layer, polygon);
 			return pcb_true;
@@ -703,11 +601,11 @@ static pcb_bool UndoSwapCopiedObject(UndoListTypePtr Entry)
 
 	/* lookup entry by it's ID */
 	type = pcb_search_obj_by_id(pcb_removelist, &ptr1, &ptr2, &ptr3, Entry->Data.CopyID, Entry->Kind);
-	if (type == PCB_TYPE_NONE)
+	if (type == PCB_OBJ_VOID)
 		return pcb_false;
 
 	type = pcb_search_obj_by_id(PCB->Data, &ptr1b, &ptr2b, &ptr3b, Entry->ID, Entry->Kind);
-	if (type == PCB_TYPE_NONE)
+	if (type == PCB_OBJ_VOID)
 		return pcb_false;
 
 	obj = (pcb_any_obj_t *) ptr2;
@@ -723,7 +621,7 @@ static pcb_bool UndoSwapCopiedObject(UndoListTypePtr Entry)
 		DrawRecoveredObject((pcb_any_obj_t *)ptr2);
 
 	obj = (pcb_any_obj_t *) pcb_move_obj_to_buffer(PCB, PCB->Data, pcb_removelist, type, ptr1, ptr2, ptr3);
-	if (Entry->Kind == PCB_TYPE_POLY)
+	if (Entry->Kind == PCB_OBJ_POLY)
 		pcb_poly_init_clip(PCB->Data, (pcb_layer_t *) ptr1b, (pcb_poly_t *) obj);
 	return pcb_true;
 }
@@ -734,7 +632,7 @@ static pcb_bool UndoSwapCopiedObject(UndoListTypePtr Entry)
  */
 static pcb_bool UndoRemoveContour(UndoListTypePtr Entry)
 {
-	assert(Entry->Kind == PCB_TYPE_POLY);
+	assert(Entry->Kind == PCB_OBJ_POLY);
 	return UndoSwapCopiedObject(Entry);
 }
 
@@ -744,7 +642,7 @@ static pcb_bool UndoRemoveContour(UndoListTypePtr Entry)
  */
 static pcb_bool UndoInsertContour(UndoListTypePtr Entry)
 {
-	assert(Entry->Kind == PCB_TYPE_POLY);
+	assert(Entry->Kind == PCB_OBJ_POLY);
 	return UndoSwapCopiedObject(Entry);
 }
 
@@ -826,11 +724,6 @@ static int pcb_undo_old_undo(void *ptr_)
 			return 0;
 		break;
 
-	case PCB_UNDO_CHANGEPINNUM:
-		if (UndoChangePinnum(ptr))
-			return 0;
-		break;
-
 	case PCB_UNDO_CREATE:
 		if (UndoCopyOrCreate(ptr))
 			return 0;
@@ -901,16 +794,6 @@ static int pcb_undo_old_undo(void *ptr_)
 			return 0;
 		break;
 
-	case PCB_UNDO_CHANGEMASKSIZE:
-		if (UndoChangeMaskSize(ptr))
-			return 0;
-		break;
-
-	case PCB_UNDO_CHANGE2NDSIZE:
-		if (UndoChange2ndSize(ptr))
-			return 0;
-		break;
-
 	case PCB_UNDO_CHANGEANGLES:
 		if (UndoChangeAngles(ptr))
 			return 0;
@@ -931,11 +814,6 @@ static int pcb_undo_old_undo(void *ptr_)
 			return 0;
 		break;
 
-	case PCB_UNDO_MIRROR:
-		if (UndoMirror(ptr))
-			return 0;
-		break;
-
 	case PCB_UNDO_OTHERSIDE:
 		if (UndoOtherSide(ptr))
 			return 0;
@@ -958,19 +836,6 @@ void pcb_undo_add_obj_to_clear_poly(int Type, void *Ptr1, void *Ptr2, void *Ptr3
 	}
 }
 
-/* ---------------------------------------------------------------------------
- * adds an object to the list of mirrored objects
- */
-void pcb_undo_add_obj_to_mirror(int Type, void *Ptr1, void *Ptr2, void *Ptr3, pcb_coord_t yoff)
-{
-	UndoListTypePtr undo;
-
-	if (!Locked) {
-		undo = GetUndoSlot(PCB_UNDO_MIRROR, PCB_OBJECT_ID(Ptr3), Type);
-		undo->Data.Move.DY = yoff;
-	}
-}
-
 /* ---------------------------------------------------------------------------
  * adds an subc to the list of objects tossed to the other side
  */
@@ -1072,12 +937,12 @@ void pcb_undo_add_obj_to_remove_point(int Type, void *Ptr1, void *Ptr2, pcb_card
 
 	if (!Locked) {
 		switch (Type) {
-		case PCB_TYPE_POLY_POINT:
+		case PCB_OBJ_POLY_POINT:
 			{
 				/* save the ID of the parent object; else it will be
 				 * impossible to recover the point
 				 */
-				undo = GetUndoSlot(PCB_UNDO_REMOVE_POINT, PCB_OBJECT_ID(polygon), PCB_TYPE_POLY);
+				undo = GetUndoSlot(PCB_UNDO_REMOVE_POINT, PCB_OBJECT_ID(polygon), PCB_OBJ_POLY);
 				undo->Data.RemovedPoint.X = polygon->Points[index].X;
 				undo->Data.RemovedPoint.Y = polygon->Points[index].Y;
 				undo->Data.RemovedPoint.ID = polygon->Points[index].ID;
@@ -1169,19 +1034,6 @@ void pcb_undo_add_obj_to_change_name(int Type, void *Ptr1, void *Ptr2, void *Ptr
 	}
 }
 
-/* ---------------------------------------------------------------------------
- * adds an object to the list of objects with changed pinnums
- */
-void pcb_undo_add_obj_to_change_pinnum(int Type, void *Ptr1, void *Ptr2, void *Ptr3, char *OldName)
-{
-	UndoListTypePtr undo;
-
-	if (!Locked) {
-		undo = GetUndoSlot(PCB_UNDO_CHANGEPINNUM, PCB_OBJECT_ID(Ptr3), Type);
-		undo->Data.ChangeName.Name = OldName;
-	}
-}
-
 /* ---------------------------------------------------------------------------
  * adds an object to the list of objects moved to another layer
  */
@@ -1230,23 +1082,13 @@ void pcb_undo_add_obj_to_size(int Type, void *ptr1, void *ptr2, void *ptr3)
 	if (!Locked) {
 		undo = GetUndoSlot(PCB_UNDO_CHANGESIZE, PCB_OBJECT_ID(ptr2), Type);
 		switch (Type) {
-		case PCB_TYPE_PIN:
-		case PCB_TYPE_VIA:
-			undo->Data.Size = ((pcb_pin_t *) ptr2)->Thickness;
-			break;
-		case PCB_TYPE_LINE:
-		case PCB_TYPE_ELEMENT_LINE:
+		case PCB_OBJ_LINE:
 			undo->Data.Size = ((pcb_line_t *) ptr2)->Thickness;
 			break;
-		case PCB_TYPE_TEXT:
-		case PCB_TYPE_ELEMENT_NAME:
+		case PCB_OBJ_TEXT:
 			undo->Data.Size = ((pcb_text_t *) ptr2)->Scale;
 			break;
-		case PCB_TYPE_PAD:
-			undo->Data.Size = ((pcb_pad_t *) ptr2)->Thickness;
-			break;
-		case PCB_TYPE_ARC:
-		case PCB_TYPE_ELEMENT_ARC:
+		case PCB_OBJ_ARC:
 			undo->Data.Size = ((pcb_arc_t *) ptr2)->Thickness;
 			break;
 		}
@@ -1263,58 +1105,16 @@ void pcb_undo_add_obj_to_clear_size(int Type, void *ptr1, void *ptr2, void *ptr3
 	if (!Locked) {
 		undo = GetUndoSlot(PCB_UNDO_CHANGECLEARSIZE, PCB_OBJECT_ID(ptr2), Type);
 		switch (Type) {
-		case PCB_TYPE_PIN:
-		case PCB_TYPE_VIA:
-			undo->Data.Size = ((pcb_pin_t *) ptr2)->Clearance;
-			break;
-		case PCB_TYPE_LINE:
+		case PCB_OBJ_LINE:
 			undo->Data.Size = ((pcb_line_t *) ptr2)->Clearance;
 			break;
-		case PCB_TYPE_PAD:
-			undo->Data.Size = ((pcb_pad_t *) ptr2)->Clearance;
-			break;
-		case PCB_TYPE_ARC:
+		case PCB_OBJ_ARC:
 			undo->Data.Size = ((pcb_arc_t *) ptr2)->Clearance;
 			break;
 		}
 	}
 }
 
-/* ---------------------------------------------------------------------------
- * adds an object to the list of objects with Size changes
- */
-void pcb_undo_add_obj_to_mask_size(int Type, void *ptr1, void *ptr2, void *ptr3)
-{
-	UndoListTypePtr undo;
-
-	if (!Locked) {
-		undo = GetUndoSlot(PCB_UNDO_CHANGEMASKSIZE, PCB_OBJECT_ID(ptr2), Type);
-		switch (Type) {
-		case PCB_TYPE_PIN:
-		case PCB_TYPE_VIA:
-			undo->Data.Size = ((pcb_pin_t *) ptr2)->Mask;
-			break;
-		case PCB_TYPE_PAD:
-			undo->Data.Size = ((pcb_pad_t *) ptr2)->Mask;
-			break;
-		}
-	}
-}
-
-/* ---------------------------------------------------------------------------
- * adds an object to the list of objects with 2ndSize changes
- */
-void pcb_undo_add_obj_to_2nd_size(int Type, void *ptr1, void *ptr2, void *ptr3)
-{
-	UndoListTypePtr undo;
-
-	if (!Locked) {
-		undo = GetUndoSlot(PCB_UNDO_CHANGE2NDSIZE, PCB_OBJECT_ID(ptr2), Type);
-		if (Type == PCB_TYPE_PIN || Type == PCB_TYPE_VIA)
-			undo->Data.Size = ((pcb_pin_t *) ptr2)->DrillingHole;
-	}
-}
-
 /* ---------------------------------------------------------------------------
  * adds an object to the list of changed angles.  Note that you must
  * call this before changing the angles, passing the new start/delta.
@@ -1443,17 +1243,13 @@ const char *undo_type2str(int type)
 		case PCB_UNDO_MOVETOLAYER: return "movetolayer";
 		case PCB_UNDO_FLAG: return "flag";
 		case PCB_UNDO_CHANGESIZE: return "changesize";
-		case PCB_UNDO_CHANGE2NDSIZE: return "change2ndsize";
-		case PCB_UNDO_MIRROR: return "mirror";
 		case PCB_UNDO_OTHERSIDE: return "otherside";
 		case PCB_UNDO_CHANGECLEARSIZE: return "chngeclearsize";
-		case PCB_UNDO_CHANGEMASKSIZE: return "changemasksize";
 		case PCB_UNDO_CHANGEANGLES: return "changeangles";
 		case PCB_UNDO_CHANGERADII: return "changeradii";
 		case PCB_UNDO_LAYERMOVE: return "layermove";
 		case PCB_UNDO_CLEAR: return "clear";
 		case PCB_UNDO_NETLISTCHANGE: return "netlistchange";
-		case PCB_UNDO_CHANGEPINNUM: return "changepinnum";
 	}
 	sprintf(buff, "Unknown %d", type);
 	return buff;
@@ -1469,12 +1265,11 @@ static void pcb_undo_old_free(void *ptr_)
 
 	switch (ptr->Type) {
 		case PCB_UNDO_CHANGENAME:
-		case PCB_UNDO_CHANGEPINNUM:
 			free(ptr->Data.ChangeName.Name);
 			break;
 		case PCB_UNDO_REMOVE:
 			type = pcb_search_obj_by_id(pcb_removelist, &ptr1, &ptr2, &ptr3, ptr->ID, ptr->Kind);
-			if (type != PCB_TYPE_NONE)
+			if (type != PCB_OBJ_VOID)
 				pcb_destroy_object(pcb_removelist, type, ptr1, ptr2, ptr3);
 			break;
 		default:
