@@ -96,30 +96,8 @@ struct pcb_unit_s {
 	const char *alias[1];
 };
 
-struct pcb_increments_s {
-	const char *suffix;
-	/* key g and <shift>g value  */
-	pcb_coord_t grid;
-	pcb_coord_t grid_min;
-	pcb_coord_t grid_max;
-	/* key s and <shift>s value  */
-	pcb_coord_t size;
-	pcb_coord_t size_min;
-	pcb_coord_t size_max;
-	/* key l and <shift>l value  */
-	pcb_coord_t line;
-	pcb_coord_t line_min;
-	pcb_coord_t line_max;
-	/* key k and <shift>k value  */
-	pcb_coord_t clear;
-	pcb_coord_t clear_min;
-	pcb_coord_t clear_max;
-};
-
 typedef struct pcb_unit_s pcb_unit_t;
-typedef struct pcb_increments_s pcb_increments_t;
 extern pcb_unit_t pcb_units[];
-extern pcb_increments_t pcb_increments[];
 
 /* Look up a given suffix in the units array. Pluralized units are supported.
    The _ version allows strict (full name match) lookup if strict iz non-zero. */
@@ -130,7 +108,6 @@ const pcb_unit_t *get_unit_struct_(const char *suffix, int strict);
 const pcb_unit_t *get_unit_struct_by_allow(enum pcb_allow_e allow);
 
 /* Return the list of units and number of units (obsolete, do not use) */
-const pcb_unit_t *get_unit_list(void);
 int pcb_get_n_units(void);
 
 /* Return the idxth unit or NULL (bounds check) */
@@ -145,10 +122,6 @@ double pcb_unit_to_factor(const pcb_unit_t * unit);
 /* Convert a given unit to pcb coords; clamp at the end of the ranges */
 pcb_coord_t pcb_unit_to_coord(const pcb_unit_t * unit, double x);
 
-/* Looks up a given suffix in the increments array. Pluralized units
-   are not supported. Returns NULL on error. */
-pcb_increments_t *pcb_get_increments_struct(const char *suffix);
-
 /* Bring an angle into [0, 360) range */
 pcb_angle_t pcb_normalize_angle(pcb_angle_t a);
 
@@ -171,4 +144,10 @@ void pcb_units_init(void);
 #define	PCB_SWAP_ANGLE(a)		(-(a))
 #define	PCB_SWAP_DELTA(d)		(-(d))
 
+PCB_INLINE pcb_coord_t pcb_coord_abs(pcb_coord_t c)
+{
+	if (c < 0) return -c;
+	return c;
+}
+
 #endif
