@@ -193,6 +193,9 @@ lht_err_t lht_dom_list_append(lht_node_t *dest, lht_node_t *node);
 /* insert a detached node before the first element of the list */
 lht_err_t lht_dom_list_insert(lht_node_t *dest, lht_node_t *node);
 
+/* insert a detached node after dest (dest's parent must be a list) */
+lht_err_t lht_dom_list_insert_after(lht_node_t *dest, lht_node_t *node);
+
 /* returns the number of elements on the list */
 int lht_dom_list_len(lht_node_t *list);
 
