@@ -33,6 +33,13 @@
 
 #define CHK "Broken integrity: "
 
+#define check_type(obj, exp_type) \
+	do { \
+		pcb_any_obj_t *__obj__ = (pcb_any_obj_t *)(obj); \
+		if (__obj__->type != exp_type) \
+			pcb_message(PCB_MSG_ERROR, CHK "%s %ld type proken (%d != %d)\n", pcb_obj_type_name(exp_type), __obj__->ID, __obj__->type, exp_type); \
+	} while(0)
+
 #define check_parent(name, obj, pt, prnt) \
 	do { \
 		if (obj->parent_type != pt) \
@@ -47,14 +54,6 @@
 			pcb_message(PCB_MSG_ERROR, CHK "%s " name " %ld broken empty attribute list\n", whose, (obj)->ID); \
 	} while(0)
 
-#define chk_layer_attr(ly) \
-	do { \
-		if (!(ly)->is_bound) { \
-			if (((ly)->meta.real.Attributes.Number > 0) && ((ly)->meta.real.Attributes.List == NULL)) \
-				pcb_message(PCB_MSG_ERROR, CHK "%s layer %s broken empty attribute list\n", whose, (ly)->name); \
-		} \
-	} while(0)
-
 #define check_field_eq(name, obj, st1, st2, fld, fmt) \
 	do { \
 		if ((st1)->fld != (st2)->fld) \
@@ -63,35 +62,6 @@
 
 static void chk_layers(const char *whose, pcb_data_t *data, pcb_parenttype_t pt, void *parent, int name_chk);
 
-
-static void chk_element(const char *whose, pcb_element_t *elem)
-{
-	int n;
-	pcb_pin_t *pin;
-	pcb_pad_t *pad;
-	pcb_line_t *lin;
-	pcb_arc_t *arc;
-
-	for(pin = pinlist_first(&elem->Pin); pin != NULL; pin = pinlist_next(pin))
-		check_parent("pin", pin, PCB_PARENT_ELEMENT, elem);
-
-	for(pad = padlist_first(&elem->Pad); pad != NULL; pad = padlist_next(pad))
-		check_parent("pad", pad, PCB_PARENT_ELEMENT, elem);
-
-	for(lin = linelist_first(&elem->Line); lin != NULL; lin = linelist_next(lin))
-		check_parent("line", lin, PCB_PARENT_ELEMENT, elem);
-
-	for(arc = arclist_first(&elem->Arc); arc != NULL; arc = arclist_next(arc))
-		check_parent("arc", arc, PCB_PARENT_ELEMENT, elem);
-
-	for(n = 1; n < PCB_MAX_ELEMENTNAMES; n++) {
-		check_field_eq("element name", elem, &elem->Name[n], &elem->Name[0], X, "%mm");
-		check_field_eq("element name", elem, &elem->Name[n], &elem->Name[0], Y, "%mm");
-		check_field_eq("element name", elem, &elem->Name[n], &elem->Name[0], Direction, "%d");
-		check_field_eq("element name", elem, &elem->Name[n], &elem->Name[0], fid, "%d");
-	}
-}
-
 static void chk_term(const char *whose, pcb_any_obj_t *obj)
 {
 	const char *aterm = pcb_attribute_get(&obj->Attributes, "term");
@@ -160,15 +130,12 @@ static void chk_subc_cache(pcb_subc_t *subc)
 static void chk_subc(const char *whose, pcb_subc_t *subc)
 {
 	int n;
-	pcb_pin_t *via;
 	pcb_pstk_t *ps;
 
 	chk_layers("subc", subc->data, PCB_PARENT_SUBC, subc, 0);
 	chk_subc_cache(subc);
 
 	/* check term chaches */
-	for(via = pinlist_first(&subc->data->Via); via != NULL; via = pinlist_next(via))
-		chk_term("via", (pcb_any_obj_t *)via);
 	for(ps = padstacklist_first(&subc->data->padstack); ps != NULL; ps = padstacklist_next(ps))
 		chk_term("padstack", (pcb_any_obj_t *)ps);
 
@@ -208,67 +175,67 @@ static void chk_layers(const char *whose, pcb_data_t *data, pcb_parenttype_t pt,
 		pcb_message(PCB_MSG_ERROR, CHK "%s data: parent proken (%p != %p)\n", whose, data->parent, parent);
 
 
-	for(n = 0; n < pcb_max_layer; n++) {
+	for(n = 0; n < data->LayerN; n++) {
 		pcb_line_t *lin;
 		pcb_text_t *txt;
 		pcb_arc_t *arc;
 		pcb_poly_t *poly;
 	
 		/* check layers */
-		if (data->Layer[n].parent != data)
+		if (data->Layer[n].parent.data != data)
 			pcb_message(PCB_MSG_ERROR, CHK "%s layer %ld/%s parent proken (%p != %p)\n", whose, n, data->Layer[n].name, data->Layer[n].parent, data);
 		if (name_chk && ((data->Layer[n].name == NULL) || (*data->Layer[n].name == '\0')))
 			pcb_message(PCB_MSG_ERROR, CHK "%s layer %ld has invalid name\n", whose, n);
-		chk_layer_attr(&data->Layer[n]);
+		check_type(&data->Layer[n], PCB_OBJ_LAYER);
+		check_parent("layer", (&data->Layer[n]), PCB_PARENT_DATA, data);
+		chk_attr("layer", &data->Layer[n]);
+
+		if (data->Layer[n].is_bound) {
+			if ((data->Layer[n].meta.bound.type & PCB_LYT_OUTLINE) && (data->Layer[n].meta.bound.type & PCB_LYT_ANYWHERE))
+				pcb_message(PCB_MSG_ERROR, CHK "%s layer %ld/%s is a non-global outline (bound layer)\n", whose, n, data->Layer[n].name);
+		}
 
 		/* check layer objects */
 		for(lin = linelist_first(&data->Layer[n].Line); lin != NULL; lin = linelist_next(lin)) {
 			check_parent("line", lin, PCB_PARENT_LAYER, &data->Layer[n]);
+			check_type(lin, PCB_OBJ_LINE);
 			chk_attr("line", lin);
 		}
 
 		for(txt = textlist_first(&data->Layer[n].Text); txt != NULL; txt = textlist_next(txt)) {
 			check_parent("text", txt, PCB_PARENT_LAYER, &data->Layer[n]);
+			check_type(txt, PCB_OBJ_TEXT);
 			chk_attr("text", txt);
 		}
 
 		for(poly = polylist_first(&data->Layer[n].Polygon); poly != NULL; poly = polylist_next(poly)) {
 			check_parent("polygon", poly, PCB_PARENT_LAYER, &data->Layer[n]);
+			check_type(poly, PCB_OBJ_POLY);
 			chk_attr("polygon", poly);
 		}
 
 		for(arc = arclist_first(&data->Layer[n].Arc); arc != NULL; arc = arclist_next(arc)) {
 			check_parent("arc", arc, PCB_PARENT_LAYER, &data->Layer[n]);
+			check_type(arc, PCB_OBJ_ARC);
 			chk_attr("arc", arc);
 		}
 	}
 
 	/* check global objects */
 	{
-		pcb_pin_t *via;
-		pcb_element_t *elem;
 		pcb_subc_t *subc;
 		pcb_pstk_t *ps;
 
-		for(via = pinlist_first(&data->Via); via != NULL; via = pinlist_next(via)) {
-			check_parent("via", via, PCB_PARENT_DATA, data);
-			chk_attr("via", via);
-		}
-
 		for(ps = padstacklist_first(&data->padstack); ps != NULL; ps = padstacklist_next(ps)) {
 			check_parent("padstack", ps, PCB_PARENT_DATA, data);
+			check_type(ps, PCB_OBJ_PSTK);
 			chk_attr("padstack", ps);
 			chk_term("padstack", (pcb_any_obj_t *)ps);
 		}
 
-		for(elem = elementlist_first(&data->Element); elem != NULL; elem = elementlist_next(elem)) {
-			check_parent("element", elem, PCB_PARENT_DATA, data);
-			chk_element(whose, elem);
-			chk_attr("element", elem);
-		}
-
 		for(subc = pcb_subclist_first(&data->subc); subc != NULL; subc = pcb_subclist_next(subc)) {
 			check_parent("subc", subc, PCB_PARENT_DATA, data);
+			check_type(subc, PCB_OBJ_SUBC);
 			chk_subc(whose, subc);
 			chk_attr("subc", subc);
 		}
@@ -276,9 +243,26 @@ static void chk_layers(const char *whose, pcb_data_t *data, pcb_parenttype_t pt,
 #warning subc TODO: check buffers: parents
 }
 
+static void chk_layergrps(pcb_board_t *pcb)
+{
+	pcb_layergrp_id_t n;
+	const char *whose = "board";
+
+	for(n = 0; n < pcb->LayerGroups.len; n++) {
+		pcb_layergrp_t *grp = &pcb->LayerGroups.grp[n];
+		check_parent("layer_group", grp, PCB_PARENT_BOARD, pcb);
+		check_type(grp, PCB_OBJ_LAYERGRP);
+		if ((grp->ltype & PCB_LYT_OUTLINE) && (grp->ltype & PCB_LYT_ANYWHERE))
+			pcb_message(PCB_MSG_ERROR, CHK "layer group %ld/%s is a non-global outline\n", n, grp->name);
+	}
+}
+
+
 void pcb_check_integrity(pcb_board_t *pcb)
 {
 	int n;
+
+	chk_layergrps(pcb);
 	chk_layers("board", pcb->Data, PCB_PARENT_BOARD, pcb, 1);
 
 	for (n = 0; n < PCB_MAX_BUFFER; n++) {
