@@ -33,12 +33,14 @@
 #include "hid_actions.h"
 #include "hid_dad.h"
 #include "action_helper.h"
+#include "plugins.h"
 
 /* include them all for static inlines */
 #include "dlg_test.c"
 #include "dlg_layer_binding.c"
 #include "dlg_flag_edit.c"
 #include "dlg_padstack.c"
+#include "dlg_about.c"
 
 pcb_hid_action_t dialogs_action_list[] = {
 	{"dlg_test", 0, pcb_act_dlg_test,
@@ -48,7 +50,9 @@ pcb_hid_action_t dialogs_action_list[] = {
 	{"FlagEdit", 0, pcb_act_FlagEdit,
 	 pcb_acth_FlagEdit, pcb_acts_FlagEdit},
 	{"PadstackEdit", 0, pcb_act_PadstackEdit,
-	 pcb_acth_PadstackEdit, pcb_acts_PadstackEdit}
+	 pcb_acth_PadstackEdit, pcb_acts_PadstackEdit},
+	{"About2", 0, pcb_act_About,
+	 pcb_acth_About, pcb_acts_About}
 };
 
 static const char *dialogs_cookie = "dialogs plugin";
@@ -65,6 +69,7 @@ void pplg_uninit_dialogs(void)
 #include "dolists.h"
 int pplg_init_dialogs(void)
 {
+	PCB_API_CHK_VER;
 	PCB_REGISTER_ACTIONS(dialogs_action_list, dialogs_cookie)
 	return 0;
 }
