@@ -26,7 +26,6 @@
 
 #include "board.h"
 #include "data.h"
-#include "const.h"
 #include "conf_core.h"
 #include "compat_misc.h"
 #include "obj_subc.h"
@@ -201,8 +200,8 @@ static int pcb_act_LayerBinding(int argc, const char **argv, pcb_coord_t x, pcb_
 		int type;
 		void *ptr1, *ptr2, *ptr3;
 		pcb_gui->get_coords("Click on object to change size of", &x, &y);
-		type = pcb_search_screen(x, y, PCB_TYPE_SUBC, &ptr1, &ptr2, &ptr3);
-		if (type != PCB_TYPE_SUBC) {
+		type = pcb_search_screen(x, y, PCB_OBJ_SUBC, &ptr1, &ptr2, &ptr3);
+		if (type != PCB_OBJ_SUBC) {
 			pcb_message(PCB_MSG_ERROR, "No subc under the cursor\n");
 			return -1;
 		}
