@@ -38,17 +38,23 @@ typedef struct pse_proto_layer_s {
 	pcb_coord_t auto_bloat;
 } pse_proto_layer_t;
 
+#define PSE_MASK_BLOAT PCB_MIL_TO_COORD(4)
+
 static const pse_proto_layer_t pse_layer[] = {
 	{"top paste",            PCB_LYT_TOP | PCB_LYT_PASTE,     PCB_LYC_AUTO,             {2,-1}, 0},
-	{"top mask",             PCB_LYT_TOP | PCB_LYT_MASK,      PCB_LYC_SUB|PCB_LYC_AUTO, {2,-1}, PCB_MIL_TO_COORD(4)},
+	{"top mask",             PCB_LYT_TOP | PCB_LYT_MASK,      PCB_LYC_SUB|PCB_LYC_AUTO, {2,-1}, PSE_MASK_BLOAT},
 	{"top copper",           PCB_LYT_TOP | PCB_LYT_COPPER,    0,                        {4,3},  0},
 	{"any internal copper",  PCB_LYT_INTERN | PCB_LYT_COPPER, 0,                        {2,4},  0},
 	{"bottom copper",        PCB_LYT_BOTTOM | PCB_LYT_COPPER, 0,                        {2,3},  0},
-	{"bottom mask",          PCB_LYT_BOTTOM | PCB_LYT_MASK,   PCB_LYC_SUB|PCB_LYC_AUTO, {4,-1}, PCB_MIL_TO_COORD(4)},
-	{"bottom paste",         PCB_LYT_BOTTOM | PCB_LYT_PASTE,  PCB_LYC_AUTO,             {4,-1},  0}
+	{"bottom mask",          PCB_LYT_BOTTOM | PCB_LYT_MASK,   PCB_LYC_SUB|PCB_LYC_AUTO, {4,-1}, PSE_MASK_BLOAT},
+	{"bottom paste",         PCB_LYT_BOTTOM | PCB_LYT_PASTE,  PCB_LYC_AUTO,             {4,-1}, 0}
 };
 #define pse_num_layers (sizeof(pse_layer) / sizeof(pse_layer[0]))
 
+static const char *shapes[] = { "circle", "square", NULL };
+static const char *sides[] = { "all (top, bottom, intern)", "top & bottom only", "top only", "bottom only", "none", NULL };
+static pcb_layer_type_t sides_lyt[] = { PCB_LYT_TOP | PCB_LYT_BOTTOM | PCB_LYT_INTERN, PCB_LYT_TOP | PCB_LYT_BOTTOM, PCB_LYT_TOP, PCB_LYT_BOTTOM, 0 };
+
 typedef struct pse_s {
 	pcb_hid_attribute_t *attrs;
 	pcb_board_t *pcb;
@@ -56,17 +62,18 @@ typedef struct pse_s {
 	int tab;
 
 	/* widget IDs */
-	int tab_instance, tab_prototype;
 	int but_instance, but_prototype;
 	int proto_id, clearance, rot, xmirror, smirror;
 	int proto_shape[pse_num_layers];
 	int proto_info[pse_num_layers];
 	int proto_change[pse_num_layers];
 	pcb_coord_t proto_clr[pse_num_layers];
+	int prname;
 	int hole_header;
 	int hdia, hplated;
 	int htop_val, htop_text, htop_layer;
 	int hbot_val, hbot_text, hbot_layer;
+	int gen_shp, gen_size, gen_drill, gen_sides, gen_expose, gen_paste, gen_do;
 
 	/* sub-dialog: shape change */
 	void *parent_hid_ctx;
@@ -77,24 +84,6 @@ typedef struct pse_s {
 	int shrink, amount, grow;
 } pse_t;
 
-static void pse_tab_update(void *hid_ctx, pse_t *pse)
-{
-	switch(pse->tab) {
-		case 0:
-			pcb_gui->attr_dlg_widget_hide(hid_ctx, pse->tab_instance, pcb_false);
-			pcb_gui->attr_dlg_widget_hide(hid_ctx, pse->tab_prototype, pcb_true);
-			pcb_gui->attr_dlg_widget_state(hid_ctx, pse->but_instance, pcb_false);
-			pcb_gui->attr_dlg_widget_state(hid_ctx, pse->but_prototype, pcb_true);
-			break;
-		case 1:
-			pcb_gui->attr_dlg_widget_hide(hid_ctx, pse->tab_instance, pcb_true);
-			pcb_gui->attr_dlg_widget_hide(hid_ctx, pse->tab_prototype, pcb_false);
-			pcb_gui->attr_dlg_widget_state(hid_ctx, pse->but_instance, pcb_true);
-			pcb_gui->attr_dlg_widget_state(hid_ctx, pse->but_prototype, pcb_false);
-			break;
-	}
-}
-
 /* build a group/layer name string in tmp */
 char *pse_group_string(pcb_board_t *pcb, pcb_layergrp_t *grp, char *out, int size)
 {
@@ -117,6 +106,7 @@ char *pse_group_string(pcb_board_t *pcb, pcb_layergrp_t *grp, char *out, int siz
 static void pse_ps2dlg(void *hid_ctx, pse_t *pse)
 {
 	char tmp[256], *s;
+	const char *prn = "";
 	int n;
 	pcb_pstk_proto_t *proto;
 	pcb_layergrp_id_t top_gid, bottom_gid;
@@ -128,7 +118,9 @@ static void pse_ps2dlg(void *hid_ctx, pse_t *pse)
 	bottom_grp = pcb_get_layergrp(pse->pcb, bottom_gid);
 
 	/* instance */
-	sprintf(tmp, "#%ld, %d", (long int)pse->ps->proto, pse->ps->protoi);
+	if ((proto != NULL) && (proto->name != NULL))
+		prn = proto->name;
+	pcb_snprintf(tmp, sizeof(tmp), "#%ld:%d (%s)", (long int)pse->ps->proto, pse->ps->protoi, prn);
 	PCB_DAD_SET_VALUE(hid_ctx, pse->proto_id, str_value, tmp);
 	PCB_DAD_SET_VALUE(hid_ctx, pse->clearance, coord_value, pse->ps->Clearance);
 	PCB_DAD_SET_VALUE(hid_ctx, pse->rot, real_value, pse->ps->rot);
@@ -182,6 +174,9 @@ static void pse_ps2dlg(void *hid_ctx, pse_t *pse)
 	}
 	PCB_DAD_SET_VALUE(hid_ctx, pse->hole_header, str_value, s);
 
+	free(pse->attrs[pse->prname].default_val.str_value);
+	pse->attrs[pse->prname].default_val.str_value = NULL;
+	PCB_DAD_SET_VALUE(hid_ctx, pse->prname, str_value, pcb_strdup(proto->name == NULL ? "" : proto->name));
 	PCB_DAD_SET_VALUE(hid_ctx, pse->hdia, coord_value, proto->hdia);
 	PCB_DAD_SET_VALUE(hid_ctx, pse->hplated, int_value, proto->hplated);
 	PCB_DAD_SET_VALUE(hid_ctx, pse->htop_val, int_value, proto->htop);
@@ -203,20 +198,6 @@ static void pse_ps2dlg(void *hid_ctx, pse_t *pse)
 
 }
 
-static void pse_tab_ps(void *hid_ctx, void *caller_data, pcb_hid_attribute_t *attr)
-{
-	pse_t *pse = caller_data;
-	pse->tab = 0;
-	pse_tab_update(hid_ctx, pse);
-}
-
-static void pse_tab_proto(void *hid_ctx, void *caller_data, pcb_hid_attribute_t *attr)
-{
-	pse_t *pse = caller_data;
-	pse->tab = 1;
-	pse_tab_update(hid_ctx, pse);
-}
-
 static void pse_chg_instance(void *hid_ctx, void *caller_data, pcb_hid_attribute_t *attr)
 {
 	pse_t *pse = caller_data;
@@ -239,6 +220,34 @@ static void pse_chg_instance(void *hid_ctx, void *caller_data, pcb_hid_attribute
 	pcb_gui->invalidate_all();
 }
 
+static void pse_chg_prname(void *hid_ctx, void *caller_data, pcb_hid_attribute_t *attr)
+{
+	pse_t *pse = caller_data;
+	pcb_pstk_proto_t *proto = pcb_pstk_get_proto(pse->ps);
+	static int lock = 0;
+	const char *new_name = pse->attrs[pse->prname].default_val.str_value;
+
+	if ((lock != 0) || (proto == NULL))
+		return;
+
+	if (proto->name == NULL) {
+		if ((new_name == NULL) || (*new_name == '\0'))
+			return;
+	}
+	else {
+		if (strcmp(proto->name, new_name) == 0)
+			return;
+	}
+
+	pcb_pstk_proto_change_name(proto, new_name);
+
+	lock++;
+	pse_ps2dlg(hid_ctx, pse); /* to get calculated text fields updated */
+	lock--;
+
+	pcb_gui->invalidate_all();
+}
+
 static void pse_chg_hole(void *hid_ctx, void *caller_data, pcb_hid_attribute_t *attr)
 {
 	pse_t *pse = caller_data;
@@ -375,7 +384,7 @@ static void pse_shape_copy(void *hid_ctx, void *caller_data, pcb_hid_attribute_t
 		return;
 	}
 
-	pcb_pstk_shape_derive(proto, dst_idx, src_idx, pse_layer[pse->editing_shape].auto_bloat, pse_layer[pse->editing_shape].mask, pse_layer[pse->editing_shape].comb);
+	pcb_pstk_shape_derive(proto, dst_idx, src_idx, 0, pse_layer[pse->editing_shape].mask, pse_layer[pse->editing_shape].comb);
 
 	pse_ps2dlg(pse->parent_hid_ctx, pse);
 	pcb_gui->invalidate_all();
@@ -486,12 +495,95 @@ static void pse_chg_shape(void *hid_ctx, void *caller_data, pcb_hid_attribute_t
 	PCB_DAD_FREE(dlg);
 }
 
+/* Auto gen shape on a single layer */
+static void pse_gen_shape(pcb_pstk_tshape_t *ts, pcb_layer_type_t lyt, int shape, pcb_coord_t size)
+{
+	int idx = ts->len;
+
+	ts->len++;
+	ts->shape = realloc(ts->shape, ts->len * sizeof(pcb_pstk_shape_t));
+
+	ts->shape[idx].layer_mask = lyt;
+	ts->shape[idx].comb = 0;
+	ts->shape[idx].clearance = 0;
+
+	switch(shape) {
+		case 0:
+			ts->shape[idx].shape = PCB_PSSH_CIRC;
+			ts->shape[idx].data.circ.x = ts->shape[idx].data.circ.y = 0;
+			ts->shape[idx].data.circ.dia = size;
+			break;
+		case 1:
+			ts->shape[idx].shape = PCB_PSSH_POLY;
+			pcb_pstk_shape_alloc_poly(&ts->shape[idx].data.poly, 4);
+			ts->shape[idx].data.poly.x[0] = -size/2;
+			ts->shape[idx].data.poly.y[0] = -size/2;
+			ts->shape[idx].data.poly.x[1] = ts->shape[idx].data.poly.x[0];
+			ts->shape[idx].data.poly.y[1] = ts->shape[idx].data.poly.y[0] + size;
+			ts->shape[idx].data.poly.x[2] = ts->shape[idx].data.poly.x[0] + size;
+			ts->shape[idx].data.poly.y[2] = ts->shape[idx].data.poly.y[0] + size;
+			ts->shape[idx].data.poly.x[3] = ts->shape[idx].data.poly.x[0] + size;
+			ts->shape[idx].data.poly.y[3] = ts->shape[idx].data.poly.y[0];
+			break;
+	}
+}
+
+/* Auto derive shape from the related copper layer */
+static void pse_drv_shape(pcb_pstk_proto_t *proto, pcb_pstk_tshape_t *ts, pcb_layer_type_t lyt, int paste)
+{
+	int srci = (lyt & PCB_LYT_TOP) ? 0 : 1;
+	pcb_pstk_shape_derive(proto, -1, srci, PSE_MASK_BLOAT, lyt | PCB_LYT_MASK, PCB_LYC_SUB|PCB_LYC_AUTO);
+	if (paste)
+		pcb_pstk_shape_derive(proto, -1, srci, 0, lyt | PCB_LYT_PASTE, PCB_LYC_AUTO);
+}
+
+/* Auto gen shapes for all layers the user selected on the dialog plus add the hole */
+static void pse_gen(void *hid_ctx, void *caller_data, pcb_hid_attribute_t *attr)
+{
+	pse_t *pse = caller_data;
+	pcb_pstk_proto_t proto;
+	int sides = pse->attrs[pse->gen_sides].default_val.int_value;
+	int shape = pse->attrs[pse->gen_shp].default_val.int_value;
+	int expose = pse->attrs[pse->gen_expose].default_val.int_value;
+	int paste = pse->attrs[pse->gen_paste].default_val.int_value;
+	pcb_coord_t size = pse->attrs[pse->gen_size].default_val.coord_value;
+	pcb_layer_type_t lyt = sides_lyt[sides];
+	pcb_pstk_tshape_t *ts;
+	pcb_cardinal_t pid;
+
+	memset(&proto, 0, sizeof(proto));
+
+	ts = pcb_vtpadstack_tshape_alloc_append(&proto.tr, 1);
+	ts->rot = 0.0;
+	ts->xmirror = 0;
+	ts->smirror = 0;
+	ts->len = 0;
+
+	if (lyt & PCB_LYT_TOP)     pse_gen_shape(ts, PCB_LYT_COPPER | PCB_LYT_TOP, shape, size);
+	if (lyt & PCB_LYT_BOTTOM)  pse_gen_shape(ts, PCB_LYT_COPPER | PCB_LYT_BOTTOM, shape, size);
+	if (lyt & PCB_LYT_INTERN)  pse_gen_shape(ts, PCB_LYT_COPPER | PCB_LYT_INTERN, shape, size);
+	if (expose) {
+		if (lyt & PCB_LYT_TOP)    pse_drv_shape(&proto, ts, PCB_LYT_TOP, paste);
+		if (lyt & PCB_LYT_BOTTOM) pse_drv_shape(&proto, ts, PCB_LYT_BOTTOM, paste);
+	}
+
+	proto.hdia = pse->attrs[pse->gen_drill].default_val.coord_value;
+	proto.hplated = 1;
+
+	pid = pcb_pstk_proto_insert_dup(pse->pcb->Data, &proto, 1);
+	pcb_pstk_change_instance(pse->ps, &pid, NULL, NULL, NULL, NULL);
+
+	pse_ps2dlg(hid_ctx, pse);
+	PCB_DAD_SET_VALUE(hid_ctx, pse->tab, int_value, 1); /* switch to the prototype view where the new attributes are visible */
+}
+
 static const char pcb_acts_PadstackEdit[] = "PadstackEdit(object)\n";
 static const char pcb_acth_PadstackEdit[] = "interactive pad stack editor";
 static int pcb_act_PadstackEdit(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
 {
-	int n;
+	int n, target_tab = -1;
 	pse_t pse;
+	const char *tabs[] = { "this instance", "prototype", "generate common geometry", NULL };
 	PCB_DAD_DECL(dlg);
 
 	memset(&pse, 0, sizeof(pse));
@@ -500,8 +592,8 @@ static int pcb_act_PadstackEdit(int argc, const char **argv, pcb_coord_t x, pcb_
 		void *ptr1, *ptr2 = NULL, *ptr3;
 		long type;
 		pcb_gui->get_coords("Click on a padstack to edit", &x, &y);
-		type = pcb_search_screen(x, y, PCB_TYPE_PSTK | PCB_TYPE_SUBC_PART | PCB_LOOSE_SUBC, &ptr1, &ptr2, &ptr3);
-		if (type != PCB_TYPE_PSTK) {
+		type = pcb_search_screen(x, y, PCB_OBJ_PSTK | PCB_OBJ_SUBC_PART | PCB_LOOSE_SUBC, &ptr1, &ptr2, &ptr3);
+		if (type != PCB_OBJ_PSTK) {
 			pcb_message(PCB_MSG_ERROR, "Need a padstack.\n");
 			return 1;
 		}
@@ -510,28 +602,20 @@ static int pcb_act_PadstackEdit(int argc, const char **argv, pcb_coord_t x, pcb_
 	else
 		PCB_ACT_FAIL(PadstackEdit);
 
+	if (argc > 0)
+		target_tab = atoi(argv[1]);
+
 	pse.pcb = pcb_data_get_top(pse.ps->parent.data);
 	if (pse.pcb == NULL)
 		pse.pcb = PCB;
 
-	PCB_DAD_BEGIN_VBOX(dlg);
-		PCB_DAD_BEGIN_HBOX(dlg);
-			PCB_DAD_BUTTON(dlg, "this instance");
-				pse.but_instance = PCB_DAD_CURRENT(dlg);
-				PCB_DAD_CHANGE_CB(dlg, pse_tab_ps);
-				PCB_DAD_HELP(dlg, "Change the properties of\nthis one padstack instance");
-			PCB_DAD_BUTTON(dlg, "prototype");
-				pse.but_prototype = PCB_DAD_CURRENT(dlg);
-				PCB_DAD_CHANGE_CB(dlg, pse_tab_proto);
-				PCB_DAD_HELP(dlg, "Change the properties of\nthe padstack prototype used by\nthis padstack\n(affects all padstacks using the same prototype)");
-		PCB_DAD_END(dlg);
-
-		/* this instance */
+	PCB_DAD_BEGIN_TABBED(dlg, tabs);
+		pse.tab = PCB_DAD_CURRENT(dlg);
+		/* Tab 0: this instance */
 		PCB_DAD_BEGIN_VBOX(dlg);
-			pse.tab_instance = PCB_DAD_CURRENT(dlg);
 			PCB_DAD_BEGIN_VBOX(dlg);
 				PCB_DAD_COMPFLAG(dlg, PCB_HATF_FRAME);
-				PCB_DAD_LABEL(dlg, "Setting that affect only this padstack instance");
+				PCB_DAD_LABEL(dlg, "Settings that affect only this padstack instance");
 				PCB_DAD_BEGIN_HBOX(dlg);
 					PCB_DAD_LABEL(dlg, "prototype");
 					PCB_DAD_BUTTON(dlg, "#5");
@@ -564,11 +648,10 @@ static int pcb_act_PadstackEdit(int argc, const char **argv, pcb_coord_t x, pcb_
 			PCB_DAD_END(dlg);
 		PCB_DAD_END(dlg);
 
-		/* prototype */
+		/* Tab 1: prototype */
 		PCB_DAD_BEGIN_VBOX(dlg);
-			pse.tab_prototype = PCB_DAD_CURRENT(dlg);
 			PCB_DAD_BEGIN_VBOX(dlg);
-				PCB_DAD_LABEL(dlg, "Setting that affect all padstacks with the same prototype");
+				PCB_DAD_LABEL(dlg, "Settings that affect all padstacks with the same prototype");
 				PCB_DAD_COMPFLAG(dlg, PCB_HATF_FRAME);
 				PCB_DAD_LABEL(dlg, "Pad geometry per layer type:");
 				PCB_DAD_BEGIN_TABLE(dlg, 5);
@@ -591,7 +674,15 @@ static int pcb_act_PadstackEdit(int argc, const char **argv, pcb_coord_t x, pcb_
 							PCB_DAD_HELP(dlg, "local, per layer type clearance\n(only when global padstack clearance is 0)");
 					}
 				PCB_DAD_END(dlg);
-			
+
+				PCB_DAD_BEGIN_HBOX(dlg);
+					PCB_DAD_COMPFLAG(dlg, PCB_HATF_FRAME);
+					PCB_DAD_LABEL(dlg, "Prototype name:");
+					PCB_DAD_STRING(dlg);
+						PCB_DAD_CHANGE_CB(dlg, pse_chg_prname);
+						pse.prname = PCB_DAD_CURRENT(dlg);
+				PCB_DAD_END(dlg);
+
 				PCB_DAD_LABEL(dlg, "Hole properties:");
 					pse.hole_header = PCB_DAD_CURRENT(dlg);
 
@@ -640,14 +731,59 @@ static int pcb_act_PadstackEdit(int argc, const char **argv, pcb_coord_t x, pcb_
 				PCB_DAD_END(dlg);
 			PCB_DAD_END(dlg);
 		PCB_DAD_END(dlg);
+
+		/* Tab 2: generate common geometry */
+		PCB_DAD_BEGIN_VBOX(dlg);
+			PCB_DAD_BEGIN_VBOX(dlg);
+				PCB_DAD_LABEL(dlg, "Generate a new prototype using a few numeric input");
+
+				PCB_DAD_BEGIN_TABLE(dlg, 2);
+					PCB_DAD_LABEL(dlg, "Copper shape:");
+					PCB_DAD_ENUM(dlg, shapes);
+						pse.gen_shp = PCB_DAD_CURRENT(dlg);
+
+					PCB_DAD_LABEL(dlg, "Size (cirlce diameter or square side):");
+					PCB_DAD_COORD(dlg, "");
+						pse.gen_size = PCB_DAD_CURRENT(dlg);
+						PCB_DAD_MINVAL(dlg, 1);
+						PCB_DAD_MAXVAL(dlg, PCB_MM_TO_COORD(1000));
+
+					PCB_DAD_LABEL(dlg, "Drill diameter (0 means no hole):");
+					PCB_DAD_COORD(dlg, "");
+						pse.gen_drill = PCB_DAD_CURRENT(dlg);
+						PCB_DAD_MINVAL(dlg, 1);
+						PCB_DAD_MAXVAL(dlg, PCB_MM_TO_COORD(1000));
+
+					PCB_DAD_LABEL(dlg, "Copper shapes on:");
+					PCB_DAD_ENUM(dlg, sides);
+						pse.gen_sides = PCB_DAD_CURRENT(dlg);
+
+					PCB_DAD_LABEL(dlg, "Expose top/bottom copper:");
+					PCB_DAD_BOOL(dlg, "");
+						pse.gen_expose = PCB_DAD_CURRENT(dlg);
+
+					PCB_DAD_LABEL(dlg, "Paste exposed copper:");
+					PCB_DAD_BOOL(dlg, "");
+						pse.gen_paste = PCB_DAD_CURRENT(dlg);
+
+					PCB_DAD_LABEL(dlg, "");
+					PCB_DAD_BUTTON(dlg, "Generate!");
+						pse.gen_do = PCB_DAD_CURRENT(dlg);
+						PCB_DAD_CHANGE_CB(dlg, pse_gen);
+				PCB_DAD_END(dlg);
+			PCB_DAD_END(dlg);
+		PCB_DAD_END(dlg);
 	PCB_DAD_END(dlg);
 
 	PCB_DAD_NEW(dlg, "dlg_padstack_edit", "Edit padstack", &pse, pcb_true, NULL);
 	pse.attrs = dlg;
-	pse_tab_update(dlg_hid_ctx, &pse);
 	pse_ps2dlg(dlg_hid_ctx, &pse);
+	if (target_tab > 0)
+		PCB_DAD_SET_VALUE(dlg_hid_ctx, pse.tab, int_value, target_tab);
 	PCB_DAD_RUN(dlg);
 
+	free(pse.attrs[pse.prname].default_val.str_value);
+
 	PCB_DAD_FREE(dlg);
 	return 0;
 }
