@@ -28,6 +28,7 @@
 #include "move.h"
 #include "draw.h"
 #include "plugins.h"
+#include "macro.h"
 #include "action_helper.h"
 #include "hid_actions.h"
 #include "compat_misc.h"
@@ -350,7 +351,7 @@ static int align(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
 			else
 				dx = 0;
 			pcb_subc_move(subc, dx, dy, 1);
-			pcb_undo_add_obj_to_move(PCB_TYPE_SUBC, NULL, NULL, subc, dx, dy);
+			pcb_undo_add_obj_to_move(PCB_OBJ_SUBC, NULL, NULL, subc, dx, dy);
 			changed = 1;
 		}
 	}
@@ -515,7 +516,7 @@ static int distribute(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
 			else
 				dx = 0;
 			pcb_subc_move(subc, dx, dy, 1);
-			pcb_undo_add_obj_to_move(PCB_TYPE_SUBC, NULL, NULL, subc, dx, dy);
+			pcb_undo_add_obj_to_move(PCB_OBJ_SUBC, NULL, NULL, subc, dx, dy);
 			changed = 1;
 		}
 		/* in gaps mode, accumulate part widths */
@@ -555,6 +556,7 @@ void pplg_uninit_distalign(void)
 #include "dolists.h"
 int pplg_init_distalign(void)
 {
+	PCB_API_CHK_VER;
 	PCB_REGISTER_ACTIONS(distalign_action_list, distalign_cookie);
 	return 0;
 }
