@@ -54,8 +54,9 @@
 #include "hid_actions.h"
 #include "djopt_conf.h"
 #include "obj_line.h"
-#include "obj_pinvia.h"
 #include "event.h"
+#include "macro.h"
+#include "obj_pstk_inlines.h"
 
 conf_djopt_t conf_djopt;
 
@@ -66,7 +67,7 @@ static const char *djopt_cookie = "djopt";
 #define selected(x) PCB_FLAG_TEST(PCB_FLAG_SELECTED, (x))
 #define autorouted(x) PCB_FLAG_TEST(PCB_FLAG_AUTO, (x))
 
-#define SB (PCB->Bloat+1)
+#define SB (conf_core.design.bloat+1)
 
 /* must be 2^N-1 */
 #define INC 7
@@ -91,9 +92,7 @@ typedef struct corner_s {
 	struct corner_s *next;
 	int x, y;
 	int net;
-	pcb_pin_t *via;
-	pcb_pad_t *pad;
-	pcb_pin_t *pin;
+	pcb_pstk_t *via, *pad, *pin;
 	int miter;
 	int n_lines;
 	struct line_s **lines;
@@ -124,22 +123,17 @@ static char layer_type[PCB_MAX_LAYER];
 
 static const char *element_name_for(corner_s * c)
 {
-	PCB_ELEMENT_LOOP(PCB->Data);
+	PCB_SUBC_LOOP(PCB->Data);
 	{
-		PCB_PIN_LOOP(element);
+		PCB_PADSTACK_LOOP(subc->data);
 		{
-			if (pin == c->pin)
-				return element->Name[1].TextString;
-		}
-		PCB_END_LOOP;
-		PCB_PAD_LOOP(element);
-		{
-			if (pad == c->pad)
-				return element->Name[1].TextString;
+			if ((padstack == c->pin) || (padstack == c->pad))
+				return subc->refdes;
 		}
 		PCB_END_LOOP;
 	}
 	PCB_END_LOOP;
+
 	return "unknown";
 }
 
@@ -159,13 +153,13 @@ static const char *corner_name(corner_s * c)
 	bp = buf[bn] + strlen(buf[bn]);
 
 	if (c->pin)
-		pcb_sprintf(bp, "pin %s:%s at %#mD", element_name_for(c), c->pin->Number, c->x, c->y);
+		pcb_sprintf(bp, "pin %s:%s at %#mD", element_name_for(c), c->pin->term, c->x, c->y);
 	else if (c->via)
 		pcb_sprintf(bp, "via at %#mD", c->x, c->y);
 	else if (c->pad) {
 		pcb_sprintf(bp, "pad %s:%s at %#mD %#mD-%#mD",
-								element_name_for(c), c->pad->Number, c->x, c->y,
-								c->pad->Point1.X, c->pad->Point1.Y, c->pad->Point2.X, c->pad->Point2.Y);
+								element_name_for(c), c->pad->term, c->x, c->y,
+								c->pad->BoundingBox.X1, c->pad->BoundingBox.Y1, c->pad->BoundingBox.X2, c->pad->BoundingBox.Y2);
 	}
 	else
 		pcb_sprintf(bp, "at %#mD", c->x, c->y);
@@ -444,7 +438,7 @@ static pcb_line_t *create_pcb_line(int layer, int x1, int y1, int x2, int y2, in
 
 	from = (char *) linelist_first(&lyr->Line);
 	nl = pcb_line_new(PCB->Data->Layer + layer, x1, y1, x2, y2, thick, clear, flags);
-	pcb_undo_add_obj_to_create(PCB_TYPE_LINE, lyr, nl, nl);
+	pcb_undo_add_obj_to_create(PCB_OBJ_LINE, lyr, nl, nl);
 
 	to = (char *) linelist_first(&lyr->Line);
 	if (from != to) {
@@ -490,7 +484,7 @@ static void new_line(corner_s * s, corner_s * e, int layer, pcb_line_t * example
 		pcb_line_t *nl;
 		dprintf
 			("New line \033[35m%#mD to %#mD from l%d t%#mS c%#mS f%s\033[0m\n",
-			 s->x, s->y, e->x, e->y, layer, example->Thickness, example->Clearance, pcb_strflg_f2s(example->Flags, PCB_TYPE_LINE, NULL));
+			 s->x, s->y, e->x, e->y, layer, example->Thickness, example->Clearance, pcb_strflg_f2s(example->Flags, PCB_OBJ_LINE, NULL, 0));
 		nl = create_pcb_line(layer, s->x, s->y, e->x, e->y, example->Thickness, example->Clearance, example->Flags);
 
 		if (!nl)
@@ -579,12 +573,16 @@ static int line_in_rect(rect_s * r, line_s * l)
 
 static int corner_radius(corner_s * c)
 {
+	pcb_pstk_t *p = NULL;
 	int diam = 0;
 	int i;
-	if (c->pin)
-		diam = djmax(c->pin->Thickness, diam);
-	if (c->via)
-		diam = djmax(c->via->Thickness, diam);
+
+	if (c->pin) p = c->pin;
+	if (c->via) p = c->via;
+	if (c->pad) p = c->pad;
+	if (p != NULL)
+		diam = djmax(p->BoundingBox.X2 - p->BoundingBox.X1, p->BoundingBox.Y2 - p->BoundingBox.Y1);
+
 	for (i = 0; i < c->n_lines; i++)
 		if (c->lines[i]->line)
 			diam = djmax(c->lines[i]->line->Thickness, diam);
@@ -641,13 +639,13 @@ static void move_line_to_layer(line_s * l, int layer)
 	ls = LAYER_PTR(l->layer);
 	ld = LAYER_PTR(layer);
 
-	pcb_move_obj_to_layer(PCB_TYPE_LINE, ls, l->line, 0, ld, 0);
+	pcb_move_obj_to_layer(PCB_OBJ_LINE, ls, l->line, 0, ld, 0);
 	l->layer = layer;
 }
 
 static void remove_via_at(corner_s * c)
 {
-	pcb_remove_object(PCB_TYPE_VIA, c->via, 0, 0);
+	pcb_remove_object(PCB_OBJ_PSTK, c->via, 0, 0);
 	c->via = 0;
 }
 
@@ -699,7 +697,7 @@ static void merge_corners(corner_s * c1, corner_s * c2)
 
 static void move_corner(corner_s * c, int x, int y)
 {
-	pcb_pin_t *via;
+	pcb_pstk_t *via;
 	int i;
 	corner_s *pad;
 
@@ -712,17 +710,17 @@ static void move_corner(corner_s * c, int x, int y)
 	c->y = y;
 	via = c->via;
 	if (via) {
-		pcb_move_obj(PCB_TYPE_VIA, via, via, via, x - via->X, y - via->Y);
-		dprintf("via move %#mD to %#mD\n", via->X, via->Y, x, y);
+		pcb_move_obj(PCB_OBJ_PSTK, via, via, via, x - via->x, y - via->y);
+		dprintf("via move %#mD to %#mD\n", via->x, via->y, x, y);
 	}
 	for (i = 0; i < c->n_lines; i++) {
 		pcb_line_t *tl = c->lines[i]->line;
 		if (tl) {
 			if (c->lines[i]->s == c) {
-				pcb_move_obj(PCB_TYPE_LINE_POINT, LAYER_PTR(c->lines[i]->layer), tl, &tl->Point1, x - (tl->Point1.X), y - (tl->Point1.Y));
+				pcb_move_obj(PCB_OBJ_LINE_POINT, LAYER_PTR(c->lines[i]->layer), tl, &tl->Point1, x - (tl->Point1.X), y - (tl->Point1.Y));
 			}
 			else {
-				pcb_move_obj(PCB_TYPE_LINE_POINT, LAYER_PTR(c->lines[i]->layer), tl, &tl->Point2, x - (tl->Point2.X), y - (tl->Point2.Y));
+				pcb_move_obj(PCB_OBJ_LINE_POINT, LAYER_PTR(c->lines[i]->layer), tl, &tl->Point2, x - (tl->Point2.X), y - (tl->Point2.Y));
 			}
 			dprintf("Line %p moved to %#mD %#mD\n", (void *) tl, tl->Point1.X, tl->Point1.Y, tl->Point2.X, tl->Point2.Y);
 		}
@@ -814,7 +812,7 @@ static int split_line(line_s * l, corner_s * c)
 	add_line_to_corner(l, c);
 	add_line_to_corner(ls, c);
 
-	pcb_move_obj(PCB_TYPE_LINE_POINT, LAYER_PTR(l->layer), l->line, &l->line->Point2,
+	pcb_move_obj(PCB_OBJ_LINE_POINT, LAYER_PTR(l->layer), l->line, &l->line->Point2,
 						 c->x - (l->line->Point2.X), c->y - (l->line->Point2.Y));
 
 	return 1;
@@ -900,10 +898,11 @@ static int canonicalize_line(line_s * l)
 			if ((x1 < c->x && c->x < x2)
 					&& (y1 < c->y && c->y < y2)
 					&& intersecting_layers(l->layer, c->layer)) {
-				int th = c->pin ? c->pin->Thickness : c->via->Thickness;
+				pcb_pstk_t *p = c->pin ? c->pin : c->via;
+				int th = djmax(p->BoundingBox.X2 - p->BoundingBox.X1, p->BoundingBox.Y2 - p->BoundingBox.Y1);
 				th /= 2;
 				if (dist(l->s->x, l->s->y, c->x, c->y) > th
-						&& dist(l->e->x, l->e->y, c->x, c->y) > th && pcb_intersect_line_pin(c->pin ? c->pin : c->via, l->line)) {
+						&& dist(l->e->x, l->e->y, c->x, c->y) > th && pcb_pstk_intersect_line(p, l->line)) {
 					return split_line(l, c);
 				}
 			}
@@ -943,9 +942,8 @@ static int simple_optimize_corner(corner_s * c)
 		if (selected(c->via))
 			dprintf("via check: line[0] layer %d at %#mD nl %d\n", c->lines[0]->layer, c->x, c->y, c->n_lines);
 		/* We can't delete vias that connect to power planes, or vias
-		   that aren't tented (assume they're test points).  */
-		if (!PCB_FLAG_THERM_TEST_ANY(c->via)
-				&& c->via->Mask == 0) {
+		   have terminal name (assume they're test points).  */
+		if (!PCB_FLAG_THERM_TEST_ANY(c->via) && (c->via->term == NULL)) {
 			for (i = 1; i < c->n_lines; i++) {
 				if (selected(c->via))
 					dprintf("           line[%d] layer %d %#mD to %#mD\n",
@@ -1744,7 +1742,8 @@ static int vianudge()
 			continue;
 
 		/* Now look for clearance in the new position */
-		vr = c->via->Thickness / 2 + SB + 1;
+		vr = djmax(c->via->BoundingBox.X2 - c->via->BoundingBox.X1, c->via->BoundingBox.Y2 - c->via->BoundingBox.Y1);
+		vr = vr / 2 + SB + 1;
 		for (c3 = corners; c3; c3 = c3->next) {
 			if (DELETED(c3))
 				continue;
@@ -2122,7 +2121,7 @@ static line_s *choose_example_line(corner_s * c1, corner_s * c2)
 							c[ci]->lines[li]->s->x, c[ci]->lines[li]->s->y,
 							c[ci]->lines[li]->e->x, c[ci]->lines[li]->e->y,
 							c[ci]->lines[li]->line->Thickness,
-							c[ci]->lines[li]->line->Clearance, pcb_strflg_f2s(c[ci]->lines[li]->line->Flags, PCB_TYPE_LINE, NULL));
+							c[ci]->lines[li]->line->Clearance, pcb_strflg_f2s(c[ci]->lines[li]->line->Flags, PCB_OBJ_LINE, NULL, 0));
 			/* Pads are disqualified, as we want to mimic a trace line. */
 			if (c[ci]->lines[li]->line == (pcb_line_t *) c[ci]->pad) {
 				dprintf("  bad, pad\n");
@@ -2185,7 +2184,7 @@ static void pinsnap()
 	corner_s *best_c[PCB_MAX_LAYER + 1];
 	int l, got_one;
 	int left = 0, right = 0, top = 0, bottom = 0;
-	pcb_pin_t *pin;
+	pcb_pstk_t *pin;
 	int again = 1;
 
 	int close = 0;
@@ -2205,40 +2204,16 @@ static void pinsnap()
 			if (!(c->pin || c->via || c->pad))
 				continue;
 
-			pin = 0;
+			pin = NULL;
 
 			dprintf("\ncorner %s\n", corner_name(c));
-			if (c->pin || c->via) {
-				pin = c->pin ? c->pin : c->via;
-				close = pin->Thickness / 2;
-				left = c->x - close;
-				right = c->x + close;
-				bottom = c->y - close;
-				top = c->y + close;
-			}
-			else if (c->pad) {
-				close = c->pad->Thickness / 2 + 1;
-				left = djmin(c->pad->Point1.X, c->pad->Point2.X) - close;
-				right = djmax(c->pad->Point1.X, c->pad->Point2.X) + close;
-				bottom = djmin(c->pad->Point1.Y, c->pad->Point2.Y) - close;
-				top = djmax(c->pad->Point1.Y, c->pad->Point2.Y) + close;
-				if (c->pad->Point1.X == c->pad->Point2.X) {
-					int hy = (c->pad->Point1.Y + c->pad->Point2.Y) / 2;
-					dprintf("pad y %#mS %#mS hy %#mS c %#mS\n", c->pad->Point1.Y, c->pad->Point2.Y, hy, c->y);
-					if (c->y < hy)
-						top = hy;
-					else
-						bottom = hy + 1;
-				}
-				else {
-					int hx = (c->pad->Point1.X + c->pad->Point2.X) / 2;
-					dprintf("pad x %#mS %#mS hx %#mS c %#mS\n", c->pad->Point1.X, c->pad->Point2.X, hx, c->x);
-					if (c->x < hx)
-						right = hx;
-					else
-						left = hx + 1;
-				}
-			}
+			if (c->pin) pin = c->pin;
+			if (c->via) pin = c->via;
+			if (c->pad) pin = c->pad;
+			left = pin->BoundingBox.X1;
+			right = pin->BoundingBox.X2;
+			top = pin->BoundingBox.Y1;
+			bottom = pin->BoundingBox.Y2;
 
 			dprintf("%s x %#mS-%#mS y %#mS-%#mS\n", corner_name(c), left, right, bottom, top);
 			for (l = 0; l <= pcb_max_layer; l++) {
@@ -2250,15 +2225,17 @@ static void pinsnap()
 			got_one = 0;
 			for (c2 = corners; c2; c2 = c2->next) {
 				int lt;
+				pcb_coord_t thick;
 
 				if (DELETED(c2))
 					continue;
+				thick = djmax(pin->BoundingBox.X2 - pin->BoundingBox.X1, pin->BoundingBox.Y2 - pin->BoundingBox.Y1);
 				lt = corner_radius(c2);
 				if (c2->n_lines && c2 != c && !(c2->pin || c2->pad || c2->via)
 						&& intersecting_layers(c->layer, c2->layer)
 						&& c2->x >= left - lt && c2->x <= right + lt && c2->y >= bottom - lt && c2->y <= top + lt) {
 					int d = dist(c->x, c->y, c2->x, c2->y);
-					if (pin && d > pin->Thickness / 2 + lt)
+					if (pin && d > thick / 2 + lt)
 						continue;
 					if (c2->n_lines == 1) {
 						got_one++;
@@ -2355,19 +2332,20 @@ static void pinsnap()
 	}
 }
 
-static int pad_orient(pcb_pad_t * p)
+static int pstk_orient(pcb_pstk_t *p)
 {
-	if (p->Point1.X == p->Point2.X)
+	pcb_coord_t dx = p->BoundingBox.X2 - p->BoundingBox.X1;
+	pcb_coord_t dy = p->BoundingBox.Y2 - p->BoundingBox.Y1;
+	if (dx < dy * 3)
 		return O_VERT;
-	if (p->Point1.Y == p->Point2.Y)
+	if (dx > dy * 3)
 		return O_HORIZ;
 	return DIAGONAL;
 }
 
-static void padcleaner()
+static void padcleaner_(pcb_data_t *data)
 {
 	line_s *l, *nextl;
-	int close;
 	rect_s r;
 
 	dprintf("\ndj: padcleaner\n");
@@ -2386,33 +2364,46 @@ static void padcleaner()
 		if (l->s->pad && l->s->pad == l->e->pad)
 			continue;
 
-		PCB_PAD_ALL_LOOP(PCB->Data);
+		PCB_PADSTACK_LOOP(data);
 		{
-			int layerflag = PCB_FLAG_TEST(PCB_FLAG_ONSOLDER, element) ? LT_SOLDER : LT_COMPONENT;
+			pcb_pstk_shape_t *shape = NULL;
 
-			if (layer_type[l->layer] != layerflag)
+			/* check if the padstack has copper on the given side */
+			switch(layer_type[l->layer]) {
+				case LT_SOLDER:     shape = pcb_pstk_shape(padstack, PCB_LYT_BOTTOM | PCB_LYT_COPPER, 0); break;
+				case LT_COMPONENT:  shape = pcb_pstk_shape(padstack, PCB_LYT_TOP | PCB_LYT_COPPER, 0); break;
+			}
+			if (shape == NULL)
 				continue;
 
 			empty_rect(&r);
-			close = pad->Thickness / 2 + 1;
-			add_point_to_rect(&r, pad->Point1.X, pad->Point1.Y, close - SB / 2);
-			add_point_to_rect(&r, pad->Point2.X, pad->Point2.Y, close - SB / 2);
+			add_point_to_rect(&r, padstack->BoundingBox.X1, padstack->BoundingBox.Y1, 0);
+			add_point_to_rect(&r, padstack->BoundingBox.X2, padstack->BoundingBox.Y2, 0);
 			if (pin_in_rect(&r, l->s->x, l->s->y, 0)
 					&& pin_in_rect(&r, l->e->x, l->e->y, 0)
-					&& ORIENT(line_orient(l, 0)) == pad_orient(pad)) {
+					&& ORIENT(line_orient(l, 0)) == pstk_orient(padstack)) {
 				dprintf
-					("padcleaner %#mD-%#mD %#mS vs line %#mD-%#mD %#mS\n",
-					 pad->Point1.X, pad->Point1.Y, pad->Point2.X, pad->Point2.Y,
-					 pad->Thickness, l->s->x, l->s->y, l->e->x, l->e->y, l->line->Thickness);
+					("padcleaner %#mD-%#mD vs line %#mD-%#mD %#mS\n",
+					 padstack->BoundingBox.X1, padstack->BoundingBox.Y1, padstack->BoundingBox.X2, padstack->BoundingBox.Y2,
+					 l->s->x, l->s->y, l->e->x, l->e->y, l->line->Thickness);
 				remove_line(l);
 				goto next_line;
 			}
 		}
-		PCB_ENDALL_LOOP;
+		PCB_END_LOOP;
 	next_line:;
 	}
 }
 
+static void padcleaner(void)
+{
+	padcleaner_(PCB->Data);
+	PCB_SUBC_LOOP(PCB->Data); {
+		padcleaner_(subc->data);
+	} PCB_END_LOOP;
+}
+
+
 static void grok_layer_groups()
 {
 	int i, j, f;
@@ -2517,42 +2508,32 @@ static int pcb_act_DJopt(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 
 	grok_layer_groups();
 
-	PCB_ELEMENT_LOOP(PCB->Data);
-	PCB_PIN_LOOP(element);
-	{
-		c = find_corner(pin->X, pin->Y, -1);
-		c->pin = pin;
-	}
-	PCB_END_LOOP;
-	PCB_PAD_LOOP(element);
-	{
-		int layern = PCB_FLAG_TEST(PCB_FLAG_ONSOLDER, pad) ? solder_layer : component_layer;
-		line_s *ls = (line_s *) malloc(sizeof(line_s));
-		ls->next = lines;
-		lines = ls;
-		ls->is_pad = 1;
-		ls->s = find_corner(pad->Point1.X, pad->Point1.Y, layern);
-		ls->s->pad = pad;
-		ls->e = find_corner(pad->Point2.X, pad->Point2.Y, layern);
-		ls->e->pad = pad;
-		ls->layer = layern;
-		ls->line = (pcb_line_t *) pad;
-		add_line_to_corner(ls, ls->s);
-		add_line_to_corner(ls, ls->e);
+	PCB_SUBC_LOOP(PCB->Data);
+		PCB_PADSTACK_LOOP(subc->data);
+		{
+			pcb_pstk_proto_t *proto;
+			proto = pcb_pstk_get_proto(padstack);
+			if (proto == NULL)
+				continue;
 
-	}
+			c = find_corner(padstack->x, padstack->y, -1);
+			if (proto->hdia <= 0)
+				c->pad = padstack;
+			else
+				c->pin = padstack;
+		}
+		PCB_END_LOOP;
 	PCB_END_LOOP;
-	PCB_END_LOOP;
-	PCB_VIA_LOOP(PCB->Data);
+
+	PCB_PADSTACK_LOOP(PCB->Data);
 	/* hace don't mess with vias that have thermals */
-	/* but then again don't bump into them
-	   if (!PCB_FLAG_TEST(ALLTHERMFLAGS, via))
-	 */
+	/* but then again don't bump into them if (!PCB_FLAG_TEST(ALLTHERMFLAGS, via)) */
 	{
-		c = find_corner(via->X, via->Y, -1);
-		c->via = via;
+		c = find_corner(padstack->x, padstack->y, -1);
+		c->via = padstack;
 	}
 	PCB_END_LOOP;
+
 	check(0, 0);
 
 	if (PCB_NSTRCMP(arg, "splitlines") == 0) {
@@ -2652,6 +2633,8 @@ void pplg_uninit_djopt(void)
 #include "dolists.h"
 int pplg_init_djopt(void)
 {
+	PCB_API_CHK_VER;
+
 #define conf_reg(field,isarray,type_name,cpath,cname,desc,flags) \
 	conf_reg_field(conf_djopt, field,isarray,type_name,cpath,cname,desc,flags);
 #include "djopt_conf_fields.h"
