@@ -37,11 +37,11 @@
 #include "data.h"
 #include "draw.h"
 #include "../report/drill.h"
-#include "obj_all.h"
 #include "plugins.h"
 #include "stub_draw.h"
 #include "draw_fab_conf.h"
 #include "conf_core.h"
+#include "macro.h"
 #include "obj_pstk_inlines.h"
 
 #include "obj_text_draw.h"
@@ -151,11 +151,11 @@ static void drill_sym(pcb_hid_gc_t gc, int idx, int x, int y)
 	}
 }
 
-static int count_drill_lines(DrillInfoTypePtr AllDrills)
+static int count_drill_lines(pcb_drill_info_t *AllDrills)
 {
 	int n, ds = 0;
 	for (n = AllDrills->DrillN - 1; n >= 0; n--) {
-		DrillTypePtr drill = &(AllDrills->Drill[n]);
+		pcb_drill_t *drill = &(AllDrills->Drill[n]);
 		if (drill->PinCount + drill->ViaCount > drill->UnplatedCount)
 			ds++;
 		if (drill->UnplatedCount)
@@ -167,7 +167,7 @@ static int count_drill_lines(DrillInfoTypePtr AllDrills)
 
 static int DrawFab_overhang(void)
 {
-	DrillInfoTypePtr AllDrills = GetDrillInfo(PCB->Data);
+	pcb_drill_info_t *AllDrills = drill_get_info(PCB->Data);
 	int ds = count_drill_lines(AllDrills);
 	if (ds < 4)
 		ds = 4;
@@ -176,14 +176,14 @@ static int DrawFab_overhang(void)
 
 static void DrawFab(pcb_hid_gc_t gc, const pcb_hid_expose_ctx_t *e)
 {
-	DrillInfoTypePtr AllDrills;
+	pcb_drill_info_t *AllDrills;
 	int i, n, yoff, total_drills = 0, ds = 0, found;
 	char utcTime[64];
-	AllDrills = GetDrillInfo(PCB->Data);
+	AllDrills = drill_get_info(PCB->Data);
 	if (conf_core.editor.grid_unit->family == PCB_UNIT_IMPERIAL)
-		RoundDrillInfo(AllDrills, PCB_MIL_TO_COORD(1));
+		drill_round_info(AllDrills, PCB_MIL_TO_COORD(1));
 	else
-		RoundDrillInfo(AllDrills, PCB_MM_TO_COORD(0.01));
+		drill_round_info(AllDrills, PCB_MM_TO_COORD(0.01));
 	yoff = -TEXT_LINE;
 
 	/* count how many drill description lines will be needed */
@@ -202,7 +202,7 @@ static void DrawFab(pcb_hid_gc_t gc, const pcb_hid_expose_ctx_t *e)
 
 	for (n = AllDrills->DrillN - 1; n >= 0; n--) {
 		int plated_sym = -1, unplated_sym = -1;
-		DrillTypePtr drill = &(AllDrills->Drill[n]);
+		pcb_drill_t *drill = &(AllDrills->Drill[n]);
 		if (drill->PinCount + drill->ViaCount > drill->UnplatedCount)
 			plated_sym = --ds;
 		if (drill->UnplatedCount)
@@ -211,13 +211,8 @@ static void DrawFab(pcb_hid_gc_t gc, const pcb_hid_expose_ctx_t *e)
 		for (i = 0; i < drill->PinN; i++) {
 			int unplated = 1;
 			pcb_coord_t x, y;
-			if ((drill->hole[i]->type == PCB_OBJ_PIN) || (drill->hole[i]->type == PCB_OBJ_VIA)) {
-				pcb_pin_t *pin = (pcb_pin_t *)drill->hole[i];
-				unplated = PCB_FLAG_TEST(PCB_FLAG_HOLE, pin);
-				x = pin->X;
-				y = pin->Y;
-			}
-			else if (drill->hole[i]->type == PCB_OBJ_PSTK) {
+			assert(drill->hole[i]->type == PCB_OBJ_PSTK);
+			{
 				pcb_pstk_t *ps = (pcb_pstk_t *)drill->hole[i];
 				pcb_pstk_proto_t *proto = pcb_pstk_get_proto(ps);
 				unplated = !proto->hplated;
@@ -335,6 +330,7 @@ void pplg_uninit_draw_fab(void)
 
 int pplg_init_draw_fab(void)
 {
+	PCB_API_CHK_VER;
 	pcb_stub_draw_fab = DrawFab;
 	pcb_stub_draw_fab_overhang = DrawFab_overhang;
 
