@@ -35,9 +35,7 @@
 #include "data.h"
 #include "draw.h"
 #include "font.h"
-#include "const.h"
 #include "hid_actions.h"
-#include "obj_all.h"
 #include "plugins.h"
 #include "stub_draw.h"
 #include "pcb-printf.h"
@@ -206,21 +204,9 @@ static pcb_bool pcb_mouse_fontsel(void *widget, pcb_hid_mouse_ev_t kind, pcb_coo
 				}
 				else {
 					switch(fontsel_txt_type) {
-						case PCB_TYPE_TEXT:
+						case PCB_OBJ_TEXT:
 							pcb_text_set_font(fontsel_layer, fontsel_txt, fid);
 							break;
-						case PCB_TYPE_ELEMENT_NAME:
-							{
-								int which;
-								pcb_element_t *e = (pcb_element_t *)fontsel_layer;
-								for(which = 0; which <= PCB_ELEMNAME_IDX_VALUE; which++) {
-									if (&e->Name[which] == fontsel_txt) {
-										pcb_element_text_set_font(PCB, PCB->Data, e, which, fid);
-										break;
-									}
-								}
-							}
-							break;
 					}
 					pcb_gui->invalidate_all();
 				}
@@ -257,6 +243,7 @@ void pplg_uninit_draw_fontsel(void)
 
 int pplg_init_draw_fontsel(void)
 {
+	PCB_API_CHK_VER;
 	pcb_stub_draw_fontsel = pcb_draw_fontsel;
 	pcb_stub_draw_fontsel_mouse_ev = pcb_mouse_fontsel;
 	pcb_stub_draw_fontsel_text_obj = &fontsel_txt;
