@@ -43,8 +43,6 @@ By Josh Jordan and Dan McMahill, modified from bom.c
 #include <time.h>
 #include <genvector/gds_char.h>
 
-#include <glib.h>
-
 #include "board.h"
 #include "data.h"
 #include "error.h"
@@ -68,11 +66,12 @@ By Josh Jordan and Dan McMahill, modified from bom.c
 #include "hid_helper.h"
 #include "plugins.h"
 #include "obj_line.h"
-#include "obj_pinvia.h"
 #include "obj_pstk_inlines.h"
 
 static const char *dsn_cookie = "dsn exporter";
 
+#define GRP_NAME(grp_) ((grp_) - PCB->LayerGroups.grp), ((grp_)->name)
+
 static pcb_coord_t trackwidth = 8;  /* user options defined in export dialog */
 static pcb_coord_t clearance = 8;
 static pcb_coord_t viawidth = 45;
@@ -118,135 +117,37 @@ static pcb_hid_attribute_t *dsn_get_export_options(int *n)
 	return dsn_options;
 }
 
-
-/* this function is mostly ripped from bom.c */
-static pcb_point_t get_centroid(pcb_element_t * element)
-{
-	pcb_point_t centroid;
-	double sumx = 0.0, sumy = 0.0;
-	int pin_cnt = 0;
-
-	PCB_PIN_LOOP(element);
-	{
-		sumx += (double) pin->X;
-		sumy += (double) pin->Y;
-		pin_cnt++;
-	}
-	PCB_END_LOOP;
-
-	PCB_PAD_LOOP(element);
-	{
-		sumx += (pad->Point1.X + pad->Point2.X) / 2.0;
-		sumy += (pad->Point1.Y + pad->Point2.Y) / 2.0;
-		pin_cnt++;
-	}
-	PCB_END_LOOP;
-
-	if (pin_cnt > 0) {
-		centroid.X = sumx / (double) pin_cnt;
-		centroid.Y = PCB->MaxHeight - (sumy / (double) pin_cnt);
-	}
-	else {
-		centroid.X = 0;
-		centroid.Y = 0;
-	}
-	return centroid;
-}
-
-static GList *layerlist = NULL;  /* contain routing layers */
-
 static void print_structure(FILE * fp)
 {
 	pcb_layergrp_id_t group, top_group, bot_group;
-	pcb_layer_id_t top_layer, bot_layer;
 
 	pcb_layergrp_list(PCB, PCB_LYT_TOP | PCB_LYT_COPPER, &top_group, 1);
 	pcb_layergrp_list(PCB, PCB_LYT_BOTTOM | PCB_LYT_COPPER, &bot_group, 1);
 
-	top_layer = PCB->LayerGroups.grp[top_group].lid[0];
-	bot_layer = PCB->LayerGroups.grp[bot_group].lid[0];
-
-
-	g_list_free(layerlist);				/* might be around from the last export */
-
-	if (PCB->Data->Layer[top_layer].meta.real.vis) {
-		layerlist = g_list_append(layerlist, &PCB->Data->Layer[top_layer]);
-	}
-	else {
-		pcb_message(PCB_MSG_WARNING, "WARNING! DSN export does not include the top layer. "
-						 "Router will consider an inner layer to be the \"top\" layer.\n");
-	}
-
-	for (group = 0; group < pcb_max_group(PCB); group++) {
-		pcb_layer_t *first_layer;
-		unsigned int gflg = pcb_layergrp_flags(PCB, group);
-
-		if (gflg & PCB_LYT_SILK)
-			continue;
-
-		if (group == top_group || group == bot_group)
-			continue;
-
-		if (PCB->LayerGroups.grp[group].len < 1)
-			continue;
-
-		first_layer = &PCB->Data->Layer[PCB->LayerGroups.grp[group].lid[0]];
-		if (!first_layer->meta.real.vis)
-			continue;
-
-		layerlist = g_list_append(layerlist, first_layer);
-
-		if (group < top_group) {
-			pcb_message(PCB_MSG_WARNING, "WARNING! DSN export moved layer group with the \"%s\" layer "
-							 "after the top layer group.  DSN files must have the top " "layer first.\n", first_layer->name);
-		}
-
-		if (group > bot_group) {
-			pcb_message(PCB_MSG_WARNING, "WARNING! DSN export moved layer group with the \"%s\" layer "
-							 "before the bottom layer group.  DSN files must have the " "bottom layer last.\n", first_layer->name);
-		}
-
-		PCB_COPPER_GROUP_LOOP(PCB->Data, group);
-		{
-			if (entry > 0) {
-				pcb_message(PCB_MSG_WARNING, "WARNING! DSN export squashed layer \"%s\" into layer "
-								 "\"%s\", DSN files do not have layer groups.", layer->name, first_layer->name);
-			}
-		}
-		PCB_END_LOOP;
-	}
-
-	if (PCB->Data->Layer[bot_layer].meta.real.vis) {
-		layerlist = g_list_append(layerlist, &PCB->Data->Layer[bot_layer]);
-	}
-	else {
-		pcb_message(PCB_MSG_WARNING, "WARNING! DSN export does not include the bottom layer. "
-						 "Router will consider an inner layer to be the \"bottom\" layer.\n");
-	}
-
 	fprintf(fp, "  (structure\n");
-
-	for (GList * iter = layerlist; iter; iter = g_list_next(iter)) {
-		pcb_layer_t *layer = iter->data;
+	for (group = 0; group < pcb_max_group(PCB); group++) {
+		int ni;
+		pcb_layergrp_t *g = &PCB->LayerGroups.grp[group];
 		char *layeropts = pcb_strdup("(type signal)");
 		
-		if (!(pcb_layer_flags_(layer) & PCB_LYT_COPPER))
+		if (!(g->ltype & PCB_LYT_COPPER))
 			continue;
-		
-		/* see if layer has same name as a net and make it a power layer */
+
+#warning TODO: revise this; use attributes instead
+		/* see if group has same name as a net and make it a power layer */
 		/* loop thru all nets */
-		for (int ni = 0; ni < PCB->NetlistLib[PCB_NETLIST_EDITED].MenuN; ni++) {
+		for(ni = 0; ni < PCB->NetlistLib[PCB_NETLIST_EDITED].MenuN; ni++) {
 			char *nname;
 			nname = PCB->NetlistLib[PCB_NETLIST_EDITED].Menu[ni].Name + 2;
-			if (!strcmp(layer->name, nname)) {
-				g_free(layeropts);
-				layeropts = pcb_strdup_printf("(type power) (use_net \"%s\")", layer->name);
+			if (!strcmp(g->name, nname)) {
+				free(layeropts);
+				layeropts = pcb_strdup_printf("(type power) (use_net \"%s\")", g->name);
 			}
 		}
-		fprintf(fp, "    (layer \"%s\"\n", layer->name);
+		fprintf(fp, "    (layer \"%d__%s\"\n", GRP_NAME(g));
 		fprintf(fp, "      %s\n", layeropts);
 		fprintf(fp, "    )\n");
-		g_free(layeropts);
+		free(layeropts);
 	}
 
 	/* PCB outline */
@@ -269,22 +170,6 @@ static void print_structure(FILE * fp)
 static void print_placement(FILE * fp)
 {
 	fprintf(fp, "  (placement\n");
-	PCB_ELEMENT_LOOP(PCB->Data);
-	{
-		char *ename;
-		pcb_point_t ecentroid = get_centroid(element);
-		char *side = PCB_FLAG_TEST(PCB_FLAG_ONSOLDER, element) ? "back" : "front";
-		ename = PCB_ELEM_NAME_REFDES(element);
-		if (ename != NULL)
-			ename = pcb_strdup(ename);
-		else
-			ename = pcb_strdup("null");
-		pcb_fprintf(fp, "    (component %d\n", element->ID);
-		pcb_fprintf(fp, "      (place \"%s\" %.6mm %.6mm %s 0 (PN 0))\n", ename, ecentroid.X, ecentroid.Y, side);
-		pcb_fprintf(fp, "    )\n");
-		g_free(ename);
-	}
-	PCB_END_LOOP;
 
 	PCB_SUBC_LOOP(PCB->Data);
 	{
@@ -292,11 +177,10 @@ static void print_placement(FILE * fp)
 		pcb_coord_t ox, oy;
 		char *side;
 		int res;
-		pcb_layer_type_t lyt = PCB_LYT_TOP;
+		int subc_on_solder = 0;
 
-		if (subc->aux_layer != NULL)
-			lyt = pcb_layer_flags_(subc->aux_layer);
-		side = (lyt & PCB_LYT_BOTTOM) ? "back" : "front";
+		pcb_subc_get_side(subc, &subc_on_solder);
+		side = subc_on_solder ? "back" : "front";
 
 		res = pcb_subc_get_origin(subc, &ox, &oy);
 		assert(res == 0);
@@ -307,108 +191,36 @@ static void print_placement(FILE * fp)
 		pcb_fprintf(fp, "    (component %d\n", subc->ID);
 		pcb_fprintf(fp, "      (place \"%s\" %.6mm %.6mm %s 0 (PN 0))\n", ename, ox, PCB->MaxHeight - oy, side);
 		pcb_fprintf(fp, "    )\n");
-		g_free(ename);
+		free(ename);
 	}
 	PCB_END_LOOP;
 
-	PCB_VIA_LOOP(PCB->Data);
+#warning padstack TODO: check if real shapes are exported
+	PCB_PADSTACK_LOOP(PCB->Data);
 	{ /* add mounting holes */
-		pcb_fprintf(fp, "    (component %d\n", via->ID);
-		pcb_fprintf(fp, "      (place %d %.6mm %.6mm %s 0 (PN 0))\n", via->ID, via->X, (PCB->MaxHeight - via->Y), "front");
+		pcb_fprintf(fp, "    (component %d\n", padstack->ID);
+		pcb_fprintf(fp, "      (place %d %.6mm %.6mm %s 0 (PN 0))\n", padstack->ID, padstack->x, (PCB->MaxHeight - padstack->y), "front");
 		pcb_fprintf(fp, "    )\n");
 	}
 	PCB_END_LOOP;
+
 	fprintf(fp, "  )\n");
 }
 
-static void add_padstack(GList **pads, char *padstack)
-{
-	if (!g_list_find_custom(*pads, padstack, (GCompareFunc) strcmp))
-		*pads = g_list_append(*pads, padstack);
-	else
-		free(padstack);
-}
-
-static void print_pin(FILE *fp, GList **pads, pcb_pin_t *pin, pcb_point_t centroid, int partsidesign, int partside)
-{
-	char *padstack;
-	pcb_coord_t ty;
-	pcb_coord_t pinthickness;
-	pcb_coord_t lx, ly;  /* hold local pin coordinates */
-	ty = PCB->MaxHeight - pin->Y;
-	pinthickness = pin->Thickness;
-	if (PCB_FLAG_TEST(PCB_FLAG_SQUARE, pin))
-		padstack = pcb_strdup_printf("Th_square_%mI", pinthickness);
-	else
-		padstack = pcb_strdup_printf("Th_round_%mI", pinthickness);
-	lx = (pin->X - centroid.X) * partsidesign;
-	ly = (centroid.Y - ty) * -1;
-
-	if (!pin->Number) { /* if pin is null just make it a keepout */
-		for (GList * iter = layerlist; iter; iter = g_list_next(iter)) {
-			pcb_layer_t *lay = iter->data;
-			pcb_fprintf(fp, "      (keepout \"\" (circle \"%s\" %.6mm %.6mm %.6mm))\n", lay->name, pinthickness, lx, ly);
-		}
-	}
-	else {
-		pcb_fprintf(fp, "      (pin %s \"%s\" %.6mm %.6mm)\n", padstack, pin->Number, lx, ly);
-	}
-
-	add_padstack(pads, padstack);
-}
-
-static void print_pad(FILE *fp, GList **pads, pcb_pad_t *pad, pcb_point_t centroid, int partsidesign, int partside)
-{
-	gchar *padstack;
-	pcb_coord_t xlen, ylen, xc, yc, p1y, p2y;
-	pcb_coord_t lx, ly;  /* store local coordinates for pins */
-	p1y = PCB->MaxHeight - pad->Point1.Y;
-	p2y = PCB->MaxHeight - pad->Point2.Y;
-	/* pad dimensions are unusual-
-	   the width is thickness and length is point difference plus thickness */
-	xlen = ABS(pad->Point1.X - pad->Point2.X);
-	if (xlen == 0) {
-		xlen = pad->Thickness;
-		ylen = ABS(p1y - p2y) + pad->Thickness;
-	}
-	else {
-		ylen = pad->Thickness;
-		xlen += pad->Thickness;
-	}
-	xc = (pad->Point1.X + pad->Point2.X) / 2;
-	yc = (p1y + p2y) / 2;
-	lx = (xc - centroid.X) * partsidesign;
-	ly = (centroid.Y - yc) * -1;
-	padstack = pcb_strdup_printf("Smd_rect_%mIx%mI", xlen, ylen);
-
-	if (!pad->Number) {				/* if pad is null just make it a keepout */
-		pcb_layer_t *lay;
-		lay = g_list_nth_data(layerlist, partside);
-		pcb_fprintf(fp, "      (keepout \"\" (rect \"%s\" %.6mm %.6mm %.6mm %.6mm))\n",
-								lay->name, lx - xlen / 2, ly - ylen / 2, lx + xlen / 2, ly + ylen / 2);
-	}
-	else {
-		pcb_fprintf(fp, "      (pin %s \"%s\" %.6mm %.6mm)\n", padstack, pad->Number, lx, ly);
-	}
-
-	add_padstack(pads, padstack);
-}
-
-static void print_polyshape(gds_t *term_shapes, pcb_pstk_poly_t *ply, pcb_coord_t ox, pcb_coord_t oy, const char *layer_name, int partsidesign)
+static void print_polyshape(gds_t *term_shapes, pcb_pstk_poly_t *ply, pcb_coord_t ox, pcb_coord_t oy, pcb_layergrp_t *grp, int partsidesign)
 {
 	char tmp[512];
 	int fld;
-	pcb_coord_t x, y;
 	int n;
 
-	pcb_snprintf(tmp, sizeof(tmp), "        (polygon \"%s\" 0", layer_name);
+	pcb_snprintf(tmp, sizeof(tmp), "        (polygon \"%d__%s\" 0", GRP_NAME(grp));
 	gds_append_str(term_shapes, tmp);
 
 	fld = 0;
 	for(n = 0; n < ply->len; n++) {
 		if ((fld % 3) == 0)
 			gds_append_str(term_shapes, "\n       ");
-		pcb_snprintf(tmp, sizeof(tmp), " %.6mm %.6mm", (ply->x[n] - ox), -(ply->y[n] - oy));
+		pcb_snprintf(tmp, sizeof(tmp), " %.6mm %.6mm", (ply->x[n] - ox) * partsidesign, -(ply->y[n] - oy));
 		gds_append_str(term_shapes, tmp);
 		fld++;
 	}
@@ -416,7 +228,7 @@ static void print_polyshape(gds_t *term_shapes, pcb_pstk_poly_t *ply, pcb_coord_
 	gds_append_str(term_shapes, "\n        )\n");
 }
 
-static void print_lineshape(gds_t *term_shapes, pcb_pstk_line_t *lin, pcb_coord_t ox, pcb_coord_t oy, const char *layer_name, int partsidesign)
+static void print_lineshape(gds_t *term_shapes, pcb_pstk_line_t *lin, pcb_coord_t ox, pcb_coord_t oy, pcb_layergrp_t *grp, int partsidesign)
 {
 	char tmp[512];
 	int fld;
@@ -424,7 +236,7 @@ static void print_lineshape(gds_t *term_shapes, pcb_pstk_line_t *lin, pcb_coord_
 	int n;
 	pcb_line_t ltmp;
 
-	pcb_snprintf(tmp, sizeof(tmp), "        (polygon \"%s\" 0", layer_name);
+	pcb_snprintf(tmp, sizeof(tmp), "        (polygon \"%d__%s\" 0", GRP_NAME(grp));
 	gds_append_str(term_shapes, tmp);
 
 	memset(&ltmp, 0, sizeof(ltmp));
@@ -441,7 +253,7 @@ static void print_lineshape(gds_t *term_shapes, pcb_pstk_line_t *lin, pcb_coord_
 	for(n = 0; n < 4; n++) {
 		if ((fld % 3) == 0)
 			gds_append_str(term_shapes, "\n       ");
-		pcb_snprintf(tmp, sizeof(tmp), " %.6mm %.6mm", (x[n] - ox), -(y[n] - oy));
+		pcb_snprintf(tmp, sizeof(tmp), " %.6mm %.6mm", (x[n] - ox) * partsidesign, -(y[n] - oy));
 		gds_append_str(term_shapes, tmp);
 		fld++;
 	}
@@ -449,11 +261,11 @@ static void print_lineshape(gds_t *term_shapes, pcb_pstk_line_t *lin, pcb_coord_
 	gds_append_str(term_shapes, "\n        )\n");
 }
 
-static void print_circshape(gds_t *term_shapes, pcb_pstk_circ_t *circ, pcb_coord_t ox, pcb_coord_t oy, const char *layer_name, int partsidesign)
+static void print_circshape(gds_t *term_shapes, pcb_pstk_circ_t *circ, pcb_coord_t ox, pcb_coord_t oy, pcb_layergrp_t *grp, int partsidesign)
 {
 	char tmp[512];
 
-	pcb_snprintf(tmp, sizeof(tmp), "        (circle \"%s\"", layer_name);
+	pcb_snprintf(tmp, sizeof(tmp), "        (circle \"%d__%s\"", GRP_NAME(grp));
 	gds_append_str(term_shapes, tmp);
 
 #warning padstack TODO: this ignores circle center offset
@@ -462,7 +274,7 @@ static void print_circshape(gds_t *term_shapes, pcb_pstk_circ_t *circ, pcb_coord
 	gds_append_str(term_shapes, tmp);
 }
 
-static void print_polyline(gds_t *term_shapes, pcb_poly_it_t *it, pcb_pline_t *pl, pcb_coord_t ox, pcb_coord_t oy, const char *layer_name, int partsidesign)
+static void print_polyline(gds_t *term_shapes, pcb_poly_it_t *it, pcb_pline_t *pl, pcb_coord_t ox, pcb_coord_t oy, pcb_layergrp_t *grp, int partsidesign)
 {
 	char tmp[512];
 	int fld;
@@ -470,14 +282,14 @@ static void print_polyline(gds_t *term_shapes, pcb_poly_it_t *it, pcb_pline_t *p
 	int go;
 
 	if (pl != NULL) {
-		pcb_snprintf(tmp, sizeof(tmp), "(polygon \"%s\" 0", layer_name);
+		pcb_snprintf(tmp, sizeof(tmp), "(polygon \"%d__%s\" 0", GRP_NAME(grp));
 		gds_append_str(term_shapes, tmp);
 
 		fld = 0;
 		for(go = pcb_poly_vect_first(it, &x, &y); go; go = pcb_poly_vect_next(it, &x, &y)) {
 			if ((fld % 3) == 0)
 				gds_append_str(term_shapes, "\n       ");
-			pcb_snprintf(tmp, sizeof(tmp), " %.6mm %.6mm", (x - ox) * partsidesign, y - oy);
+			pcb_snprintf(tmp, sizeof(tmp), " %.6mm %.6mm", (x - ox) * partsidesign, -(y - oy));
 			gds_append_str(term_shapes, tmp);
 			fld++;
 		}
@@ -486,15 +298,17 @@ static void print_polyline(gds_t *term_shapes, pcb_poly_it_t *it, pcb_pline_t *p
 	}
 }
 
-static void print_term_poly(FILE *fp, gds_t *term_shapes, pcb_poly_t *poly, pcb_coord_t ox, pcb_coord_t oy, int term_side, int partsidesign)
+static void print_term_poly(FILE *fp, gds_t *term_shapes, pcb_poly_t *poly, pcb_coord_t ox, pcb_coord_t oy, int term_on_bottom, int partsidesign)
 {
 	if (poly->term != NULL) {
-		pcb_layer_t *lay = g_list_nth_data(layerlist, term_side);
+		pcb_layergrp_id_t gid = term_on_bottom ? pcb_layergrp_get_bottom_copper() : pcb_layergrp_get_top_copper();
+		pcb_layergrp_t *grp = pcb_get_layergrp(PCB, gid);
 		char *padstack = pcb_strdup_printf("Term_poly_%ld", poly->ID);
 		pcb_poly_it_t it;
 		pcb_polyarea_t *pa;
 
-		pcb_fprintf(fp, "      (pin %s \"%s\" %.6mm %.6mm)\n", padstack, poly->term, ox, oy);
+
+		pcb_fprintf(fp, "      (pin %s \"%s\" %.6mm %.6mm)\n", padstack, poly->term, 0, 0);
 
 		gds_append_str(term_shapes, "    (padstack ");
 		gds_append_str(term_shapes, padstack);
@@ -507,7 +321,7 @@ static void print_term_poly(FILE *fp, gds_t *term_shapes, pcb_poly_t *poly, pcb_
 
 			pl = pcb_poly_contour(&it);
 
-			print_polyline(term_shapes, &it, pl, ox, oy, lay->name, partsidesign);
+			print_polyline(term_shapes, &it, pl, ox, oy, grp, partsidesign);
 		}
 
 		gds_append_str(term_shapes, "      )\n");
@@ -516,169 +330,113 @@ static void print_term_poly(FILE *fp, gds_t *term_shapes, pcb_poly_t *poly, pcb_
 	}
 }
 
-void print_pstk_shape(gds_t *term_shapes, pcb_pstk_t *padstack, int lid, pcb_coord_t ox, pcb_coord_t oy, int partsidesign)
+void print_pstk_shape(gds_t *term_shapes, pcb_pstk_t *padstack, pcb_layergrp_id_t gid, pcb_coord_t ox, pcb_coord_t oy, int partsidesign)
 {
 	pcb_pstk_shape_t *shp;
-	pcb_layer_t *lay = g_list_nth_data(layerlist, lid);
-	pcb_layer_type_t lyt = pcb_layer_flags_(lay);
-		pcb_poly_it_t it;
-
-	if (!(lyt & PCB_LYT_COPPER))
-		return;
+	pcb_layergrp_t *grp = pcb_get_layergrp(PCB, gid);
+	pcb_layer_type_t lyt = grp->ltype;
 
 	shp = pcb_pstk_shape(padstack, lyt, 0);
 	if (shp == NULL)
 		return;
 
+	/* if the subc is placed on the other side, need to invert the output layerstack as well */
+	if (partsidesign < 0) {
+		pcb_layergrp_id_t n, offs = 0;
+
+		/* determine copper offset from the top */
+		for(n = 0; (n < PCB->LayerGroups.len) && (n != gid); n++)
+			if (PCB->LayerGroups.grp[n].type & PCB_LYT_COPPER)
+				offs++;
+
+		/* count it back from the bottom and set grp */
+		for(n = PCB->LayerGroups.len-1; (n > 0) && (n != gid); n--) {
+			if (PCB->LayerGroups.grp[n].type & PCB_LYT_COPPER) {
+				if (offs == 0) {
+					grp = &PCB->LayerGroups.grp[n];
+					break;
+				}
+				offs--;
+			}
+		}
+	}
+
 	switch(shp->shape) {
 		case PCB_PSSH_POLY:
-			print_polyshape(term_shapes, &shp->data.poly, ox, oy, lay->name, partsidesign);
+			print_polyshape(term_shapes, &shp->data.poly, ox, oy, grp, partsidesign);
 			break;
 		case PCB_PSSH_LINE:
-			print_lineshape(term_shapes, &shp->data.line, ox, oy, lay->name, partsidesign);
+			print_lineshape(term_shapes, &shp->data.line, ox, oy, grp, partsidesign);
 			break;
 		case PCB_PSSH_CIRC:
-			print_circshape(term_shapes, &shp->data.circ, ox, oy, lay->name, partsidesign);
+			print_circshape(term_shapes, &shp->data.circ, ox, oy, grp, partsidesign);
 			break;
 	}
 }
 
 static void print_library(FILE * fp)
 {
-	GList *pads = NULL, *iter; /* contain unique pad names */
-	gchar *padstack;
 	gds_t term_shapes;
 
 	gds_init(&term_shapes);
 
 	fprintf(fp, "  (library\n");
-	PCB_ELEMENT_LOOP(PCB->Data);
-	{
-		int partside = PCB_FLAG_TEST(PCB_FLAG_ONSOLDER, element) ? g_list_length(layerlist) - 1 : 0;
-		int partsidesign = PCB_FLAG_TEST(PCB_FLAG_ONSOLDER, element) ? -1 : 1;
-		pcb_point_t centroid = get_centroid(element);
-		fprintf(fp, "    (image %ld\n", element->ID); /* map every element by ID */
-		/* loop thru pins and pads to add to image */
-		PCB_PIN_LOOP(element);
-		{
-			print_pin(fp, &pads, pin, centroid, partsidesign, partside);
-		}
-		PCB_END_LOOP;
-
-		PCB_PAD_LOOP(element);
-		{
-			print_pad(fp, &pads, pad, centroid, partsidesign, partside);
-		}
-		PCB_END_LOOP;
-		fprintf(fp, "    )\n");
-	}
-	PCB_END_LOOP;
 
 	PCB_SUBC_LOOP(PCB->Data);
 	{
 		pcb_coord_t ox, oy;
-		pcb_point_t centroid;
-		int partside, partsidesign, subc_on_solder = 0;
+		int partsidesign, subc_on_solder = 0;
+		pcb_layer_type_t lyt_side;
 
 		pcb_subc_get_side(subc, &subc_on_solder);
-		partside = subc_on_solder ? g_list_length(layerlist) - 1 : 0;
 		partsidesign = subc_on_solder ? -1 : 1;
+		lyt_side = subc_on_solder ? PCB_LYT_BOTTOM : PCB_LYT_TOP;
 
 		pcb_subc_get_origin(subc, &ox, &oy);
-		centroid.X = ox;
-		centroid.Y = oy;
 
 		fprintf(fp, "    (image %ld\n", subc->ID); /* map every subc by ID */
 		PCB_POLY_COPPER_LOOP(subc->data);
 		{
 			pcb_layer_type_t lyt = pcb_layer_flags_(layer);
-			if ((lyt & PCB_LYT_COPPER) && ((lyt & PCB_LYT_TOP) || (lyt & PCB_LYT_BOTTOM))) {
-				int termside = (lyt & PCB_LYT_BOTTOM) ? g_list_length(layerlist) - 1 : 0;
-				print_term_poly(fp, &term_shapes, polygon, ox, oy, termside, partsidesign);
-			}
+			if ((lyt & PCB_LYT_COPPER) && ((lyt & PCB_LYT_TOP) || (lyt & PCB_LYT_BOTTOM)))
+				print_term_poly(fp, &term_shapes, polygon, ox, oy, !(lyt & lyt_side), partsidesign);
 		}
 		PCB_ENDALL_LOOP;
 
 		PCB_PADSTACK_LOOP(subc->data);
 		{
-			int n;
+			pcb_layergrp_id_t group;
 			char *pid = pcb_strdup_printf("Pstk_shape_%ld", padstack->ID);
-			pcb_fprintf(fp, "      (pin %s \"%s\" %.6mm %.6mm)\n", pid, padstack->term, -(padstack->x-ox), padstack->y-oy);
+
+			pcb_fprintf(fp, "      (pin %s \"%s\" %.6mm %.6mm)\n", pid, padstack->term, (padstack->x-ox)*partsidesign, -(padstack->y-oy));
 
 			gds_append_str(&term_shapes, "    (padstack ");
 			gds_append_str(&term_shapes, pid);
 			gds_append_str(&term_shapes, "\n");
 
 			gds_append_str(&term_shapes, "      (shape\n");
-			for(n = 0; n < g_list_length(layerlist); n++)
-				print_pstk_shape(&term_shapes, padstack, n, 0, 0, partsidesign);
+
+			for (group = 0; group < pcb_max_group(PCB); group++) {
+				pcb_layergrp_t *g = &PCB->LayerGroups.grp[group];
+				if (g->ltype & PCB_LYT_COPPER)
+					print_pstk_shape(&term_shapes, padstack, group, 0, 0, partsidesign);
+			}
+
 			gds_append_str(&term_shapes, "      )\n");
 			gds_append_str(&term_shapes, "      (attach off)\n");
 			gds_append_str(&term_shapes, "    )\n");
 		}
 		PCB_END_LOOP;
 
-		PCB_VIA_LOOP(subc->data);
-		{
-			print_pin(fp, &pads, via, centroid, partsidesign, partside);
-		}
-		PCB_END_LOOP;
-
 		fprintf(fp, "    )\n");
 	}
 	PCB_END_LOOP;
 
-	PCB_VIA_LOOP(PCB->Data);
-	{ /* add mounting holes and vias */
-		fprintf(fp, "    (image %ld\n", via->ID); /* map every via by ID */
-		/* for mounting holes, clearance is added to thickness for higher total clearance */
-		padstack = pcb_strdup_printf("Th_round_%mI", via->Thickness + via->Clearance);
-		fprintf(fp, "      (pin %s 1 0 0)\n", padstack);	/* only 1 pin, 0,0 puts it right on component placement spot */
-		fprintf(fp, "    )\n");
-
-		add_padstack(&pads, padstack);
-	}
-	PCB_END_LOOP;
-
-	/* loop thru padstacks and define them all */
-	for (iter = pads; iter; iter = g_list_next(iter)) {
-		pcb_coord_t dim1, dim2;
-		long int dim1l, dim2l;
-		padstack = iter->data;
-		fprintf(fp, "    (padstack %s\n", padstack);
-
-		/* print info about pad here */
-		if (sscanf(padstack, "Smd_rect_%ldx%ld", &dim1l, &dim2l) == 2) { /* then pad is smd */
-			dim1 = dim1l;
-			dim2 = dim2l;
-			pcb_fprintf(fp,
-									"      (shape (rect \"%s\" %.6mm %.6mm %.6mm %.6mm))\n",
-									((pcb_layer_t *) (g_list_first(layerlist)->data))->name, dim1 / -2, dim2 / -2, dim1 / 2, dim2 / 2);
-		}
-		else if (sscanf(padstack, "Th_square_%ld", &dim1l) == 1) {
-			dim1 = dim1l;
-			pcb_fprintf(fp, "      (shape (rect signal %.6mm %.6mm %.6mm %.6mm))\n", dim1 / -2, dim1 / -2, dim1 / 2, dim1 / 2);
-		}
-		else {
-			sscanf(padstack, "Th_round_%ld", &dim1l);
-			dim1 = dim1l;
-			pcb_fprintf(fp, "      (shape (circle signal %.6mm))\n", dim1);
-		}
-		fprintf(fp, "      (attach off)\n");
-		fprintf(fp, "    )\n");
-	}
-
-	/* add padstack for via */
-	pcb_fprintf(fp, "    (padstack via_%ld_%ld\n", viawidth, viadrill);
-	pcb_fprintf(fp, "      (shape (circle signal %.6mm))\n", viawidth);
-	pcb_fprintf(fp, "      (attach off)\n    )\n");
-
 	/* add padstack for terminals */
 	pcb_fprintf(fp, "%s", term_shapes.array);
-
 	pcb_fprintf(fp, "  )\n");
-	g_list_foreach(pads, (GFunc)free, NULL);
-	g_list_free(pads);
+
+
 	gds_uninit(&term_shapes);
 }
 
@@ -725,21 +483,27 @@ static void print_network(FILE * fp)
 
 static void print_wires(FILE * fp)
 {
-	GList *iter;
-	pcb_layer_t *lay;
+	int group;
 	fprintf(fp, "    (wiring\n");
 
-	for (iter = layerlist; iter; iter = g_list_next(iter)) {
-		lay = iter->data;
-		PCB_LINE_LOOP(lay);
-		{
-			pcb_fprintf(fp,
-									"        (wire (path %s %.6mm %.6mm %.6mm %.6mm %.6mm)\n",
-									lay->name, line->Thickness, line->Point1.X,
-									(PCB->MaxHeight - line->Point1.Y), line->Point2.X, (PCB->MaxHeight - line->Point2.Y));
-			fprintf(fp, "            (type protect))\n");
+	for (group = 0; group < pcb_max_group(PCB); group++) {
+		pcb_layergrp_t *g = &PCB->LayerGroups.grp[group];
+		pcb_cardinal_t n;
+		if (!(g->ltype & PCB_LYT_COPPER))
+			continue;
+		for(n = 0; n < g->len; n++) {
+			pcb_layer_t *lay = pcb_get_layer(PCB->Data, g->lid[n]);
+
+			PCB_LINE_LOOP(lay);
+			{
+				pcb_fprintf(fp,
+					"        (wire (path %d__%s %.6mm %.6mm %.6mm %.6mm %.6mm)\n", GRP_NAME(g), line->Thickness,
+					line->Point1.X, (PCB->MaxHeight - line->Point1.Y),
+					line->Point2.X, (PCB->MaxHeight - line->Point2.Y));
+				fprintf(fp, "            (type protect))\n");
+			}
+			PCB_END_LOOP;
 		}
-		PCB_END_LOOP;
 	}
 	fprintf(fp, "\n    )\n)\n"); /* close all braces */
 }
@@ -831,9 +595,9 @@ static void dsn_do_export(pcb_hid_attr_val_t * options)
 	PrintSPECCTRA();
 }
 
-static void dsn_parse_arguments(int *argc, char ***argv)
+static int dsn_parse_arguments(int *argc, char ***argv)
 {
-	pcb_hid_parse_command_line(argc, argv);
+	return pcb_hid_parse_command_line(argc, argv);
 }
 
 int pplg_check_ver_export_dsn(int ver_needed) { return 0; }
@@ -846,6 +610,7 @@ void pplg_uninit_export_dsn(void)
 #include "dolists.h"
 int pplg_init_export_dsn(void)
 {
+	PCB_API_CHK_VER;
 	memset(&dsn_hid, 0, sizeof(pcb_hid_t));
 	pcb_hid_nogui_init(&dsn_hid);
 
