@@ -319,10 +319,10 @@ static void dxf_do_export(pcb_hid_attr_val_t * options)
 	fclose(dxf_ctx.f);
 }
 
-static void dxf_parse_arguments(int *argc, char ***argv)
+static int dxf_parse_arguments(int *argc, char ***argv)
 {
 	pcb_hid_register_attributes(dxf_attribute_list, sizeof(dxf_attribute_list) / sizeof(dxf_attribute_list[0]), dxf_cookie, 0);
-	pcb_hid_parse_command_line(argc, argv);
+	return pcb_hid_parse_command_line(argc, argv);
 }
 
 static int dxf_set_layer_group(pcb_layergrp_id_t group, pcb_layer_id_t layer, unsigned int flags, int is_empty)
@@ -457,7 +457,7 @@ static void dxf_calibrate(double xval, double yval)
 	return;
 }
 
-static void dxf_set_crosshair(int x, int y, int a)
+static void dxf_set_crosshair(pcb_coord_t x, pcb_coord_t y, int a)
 {
 }
 
@@ -480,6 +480,8 @@ void pplg_uninit_export_dxf(void)
 
 int pplg_init_export_dxf(void)
 {
+	PCB_API_CHK_VER;
+
 	memset(&dxf_hid, 0, sizeof(pcb_hid_t));
 
 	pcb_hid_nogui_init(&dxf_hid);
