@@ -158,14 +158,6 @@ static int layer_map(unsigned int lflg, int *fidoly_next, const char *lyname)
 	return *fidoly_next;
 }
 
-static void write_custom_element(FILE *f, pcb_element_t *e)
-{
-	char *msg = pcb_strdup_printf("Can't export custom footprint for %s yet\n", e->Name[PCB_ELEMNAME_IDX_REFDES].TextString);
-	pcb_io_incompat_save(e->parent.data, (pcb_any_obj_t *)e, msg, "element omitted - add the footprint type on the footprint list!");
-	free(msg);
-
-}
-
 static void write_custom_subc(FILE *f, pcb_subc_t *sc)
 {
 	char *msg = pcb_strdup_printf("Can't export custom footprint for %s yet\n", sc->refdes);
@@ -294,21 +286,6 @@ static void fidocadj_do_export(pcb_hid_attr_val_t * options)
 		PCB_END_LOOP;
 	}
 
-	PCB_VIA_LOOP(PCB->Data) {
-		fprintf(f, "pa %ld %ld", crd(via->X), crd(via->Y));
-		if (PCB_FLAG_TEST(PCB_FLAG_SQUARE, via)) {
-			if (!((PCB_FLAG_SQUARE_GET(via) == 0) || (PCB_FLAG_SQUARE_GET(via) == 1))) {
-#warning TODO: find out the orientation
-				fprintf(f, "%ld %ld %ld 2\n", crd(via->Thickness), crd(via->Thickness*2), crd(via->DrillingHole)); /* rounded corner rectangle */
-			}
-			else
-				fprintf(f, "%ld %ld %ld 1\n", crd(via->Thickness), crd(via->Thickness), crd(via->DrillingHole)); /* rectangle with sharp corners */
-		}
-		else
-			fprintf(f, "%ld %ld %ld 0\n", crd(via->Thickness), crd(via->Thickness), crd(via->DrillingHole)); /* circular */
-	}
-	PCB_END_LOOP;
-
 	PCB_PADSTACK_LOOP(PCB->Data) {
 		int oshape;
 		pcb_coord_t x, y, drill_dia, pad_dia, clearance, mask;
@@ -332,15 +309,6 @@ static void fidocadj_do_export(pcb_hid_attr_val_t * options)
 	}
 	PCB_END_LOOP;
 
-	PCB_ELEMENT_LOOP(PCB->Data) {
-		const char *fp = element->Name[PCB_ELEMNAME_IDX_DESCRIPTION].TextString;
-		if (have_lib && (htsi_get(&lib_names, fp)))
-			fprintf(f, "MC %ld %ld %d 0 %s\n", crd(element->MarkX), crd(element->MarkY), 0, fp);
-		else
-			write_custom_element(f, element);
-	}
-	PCB_END_LOOP;
-
 	PCB_SUBC_LOOP(PCB->Data) {
 		const char *fp = pcb_attribute_get(&subc->Attributes, "footprint");
 		if ((fp != NULL) && have_lib && (htsi_get(&lib_names, fp))) {
@@ -376,10 +344,10 @@ static void fidocadj_do_export(pcb_hid_attr_val_t * options)
 	}
 }
 
-static void fidocadj_parse_arguments(int *argc, char ***argv)
+static int fidocadj_parse_arguments(int *argc, char ***argv)
 {
 	pcb_hid_register_attributes(fidocadj_attribute_list, sizeof(fidocadj_attribute_list) / sizeof(fidocadj_attribute_list[0]), fidocadj_cookie, 0);
-	pcb_hid_parse_command_line(argc, argv);
+	return pcb_hid_parse_command_line(argc, argv);
 }
 
 static int fidocadj_usage(const char *topic)
@@ -401,6 +369,8 @@ void pplg_uninit_export_fidocadj(void)
 
 int pplg_init_export_fidocadj(void)
 {
+	PCB_API_CHK_VER;
+
 	memset(&fidocadj_hid, 0, sizeof(pcb_hid_t));
 
 	pcb_hid_nogui_init(&fidocadj_hid);
