@@ -226,10 +226,10 @@ struct drill_struct *sort_drill(struct drill_struct *drill, int n_drill)
 
 /* *** Main export callback ************************************************ */
 
-static void gcode_parse_arguments(int *argc, char ***argv)
+static int gcode_parse_arguments(int *argc, char ***argv)
 {
 	pcb_hid_register_attributes(gcode_attribute_list, sizeof(gcode_attribute_list) / sizeof(gcode_attribute_list[0]), gcode_cookie, 0);
-	pcb_hid_parse_command_line(argc, argv);
+	return pcb_hid_parse_command_line(argc, argv);
 }
 
 static pcb_hid_attribute_t *gcode_get_export_options(int *n)
@@ -396,7 +396,7 @@ static void gcode_do_export(pcb_hid_attr_val_t * options)
 		fprintf(stderr, "ERROR:  dpi may not be < 0\n");
 		return;
 	}
-	unit = &(get_unit_list()[options[HA_unit].int_value]);
+	unit = &(pcb_units[options[HA_unit].int_value]);
 	metric = (unit->family == PCB_UNIT_METRIC);
 	scale = metric ? 1.0 / pcb_coord_to_unit(unit, PCB_MM_TO_COORD(1.0))
 		: 1.0 / pcb_coord_to_unit(unit, PCB_INCH_TO_COORD(1.0));
@@ -868,7 +868,7 @@ static void gcode_calibrate(double xval, double yval)
 	CRASH("gcode_calibrate");
 }
 
-static void gcode_set_crosshair(int x, int y, int a)
+static void gcode_set_crosshair(pcb_coord_t x, pcb_coord_t y, int a)
 {
 }
 
@@ -894,6 +894,7 @@ void pplg_uninit_export_gcode(void)
 
 int pplg_init_export_gcode(void)
 {
+	PCB_API_CHK_VER;
 	memset(&gcode_hid, 0, sizeof(pcb_hid_t));
 
 	pcb_hid_nogui_init(&gcode_hid);
