@@ -23,6 +23,7 @@
 #include "hid_helper.h"
 #include "compat_misc.h"
 #include "safe_fs.h"
+#include "macro.h"
 
 #include "hid.h"
 #include "hid_nogui.h"
@@ -44,7 +45,7 @@ const char *gerber_cookie = "gerber HID";
 
 static pcb_hid_attribute_t *gerber_get_export_options(int *n);
 static void gerber_do_export(pcb_hid_attr_val_t * options);
-static void gerber_parse_arguments(int *argc, char ***argv);
+static int gerber_parse_arguments(int *argc, char ***argv);
 static pcb_hid_gc_t gerber_make_gc(void);
 static void gerber_destroy_gc(pcb_hid_gc_t gc);
 static void gerber_set_color(pcb_hid_gc_t gc, const char *name);
@@ -57,7 +58,7 @@ static void gerber_draw_rect(pcb_hid_gc_t gc, pcb_coord_t x1, pcb_coord_t y1, pc
 static void gerber_fill_circle(pcb_hid_gc_t gc, pcb_coord_t cx, pcb_coord_t cy, pcb_coord_t radius);
 static void gerber_fill_rect(pcb_hid_gc_t gc, pcb_coord_t x1, pcb_coord_t y1, pcb_coord_t x2, pcb_coord_t y2);
 static void gerber_calibrate(double xval, double yval);
-static void gerber_set_crosshair(int x, int y, int action);
+static void gerber_set_crosshair(pcb_coord_t x, pcb_coord_t y, int action);
 static void gerber_fill_polygon_offs(pcb_hid_gc_t gc, int n_coords, pcb_coord_t *x, pcb_coord_t *y, pcb_coord_t dx, pcb_coord_t dy);
 static void gerber_fill_polygon(pcb_hid_gc_t gc, int n_coords, pcb_coord_t * x, pcb_coord_t * y);
 
@@ -650,10 +651,10 @@ static void gerber_do_export(pcb_hid_attr_val_t * options)
 	conf_update(NULL, -1); /* resotre forced sets */
 }
 
-static void gerber_parse_arguments(int *argc, char ***argv)
+static int gerber_parse_arguments(int *argc, char ***argv)
 {
 	pcb_hid_register_attributes(gerber_options, NUM_OPTIONS, gerber_cookie, 0);
-	pcb_hid_parse_command_line(argc, argv);
+	return pcb_hid_parse_command_line(argc, argv);
 }
 
 static int drill_sort(const void *va, const void *vb)
@@ -853,9 +854,9 @@ emit_outline:
 		else if (!outline_layer) {
 			pcb_hid_gc_t gc = pcb_gui->make_gc();
 			if (flags & PCB_LYT_SILK)
-				pcb_gui->set_line_width(gc, PCB->minSlk);
+				pcb_gui->set_line_width(gc, conf_core.design.min_slk);
 			else if (group >= 0)
-				pcb_gui->set_line_width(gc, PCB->minWid);
+				pcb_gui->set_line_width(gc, conf_core.design.min_wid);
 			else
 				pcb_gui->set_line_width(gc, AUTO_OUTLINE_WIDTH);
 			pcb_gui->draw_line(gc, 0, 0, PCB->MaxWidth, 0);
@@ -1283,7 +1284,7 @@ static int gerber_usage(const char *topic)
 	return 0;
 }
 
-static void gerber_set_crosshair(int x, int y, int action)
+static void gerber_set_crosshair(pcb_coord_t x, pcb_coord_t y, int action)
 {
 }
 
@@ -1296,6 +1297,7 @@ void pplg_uninit_export_gerber(void)
 
 int pplg_init_export_gerber(void)
 {
+	PCB_API_CHK_VER;
 	memset(&gerber_hid, 0, sizeof(gerber_hid));
 
 	pcb_hid_nogui_init(&gerber_hid);
