@@ -467,10 +467,10 @@ static void nelma_write_objects(FILE * out)
 
 /* *** Main export callback ************************************************ */
 
-static void nelma_parse_arguments(int *argc, char ***argv)
+static int nelma_parse_arguments(int *argc, char ***argv)
 {
 	pcb_hid_register_attributes(nelma_attribute_list, sizeof(nelma_attribute_list) / sizeof(nelma_attribute_list[0]), nelma_cookie, 0);
-	pcb_hid_parse_command_line(argc, argv);
+	return pcb_hid_parse_command_line(argc, argv);
 }
 
 static pcb_hid_attribute_t *nelma_get_export_options(int *n)
@@ -954,7 +954,7 @@ static void nelma_calibrate(double xval, double yval)
 	CRASH("nelma_calibrate");
 }
 
-static void nelma_set_crosshair(int x, int y, int a)
+static void nelma_set_crosshair(pcb_coord_t x, pcb_coord_t y, int a)
 {
 }
 
@@ -980,6 +980,8 @@ int pplg_uninit_export_nelma(void)
 
 int pplg_init_export_nelma(void)
 {
+	PCB_API_CHK_VER;
+
 	memset(&nelma_hid, 0, sizeof(pcb_hid_t));
 
 	pcb_hid_nogui_init(&nelma_hid);
