@@ -77,9 +77,10 @@ typedef struct {
 	int lg_next;
 	int clayer; /* current layer (lg index really) */
 	long oid; /* unique object ID - we need some unique variable names, keep on counting them */
-	long pad_id; /* unique pad ID for the same reason (a single padstack object may make multiple pads) */
+	long port_id; /* unique port ID for similar reasons */
 	pcb_coord_t ox, oy;
 	unsigned warn_subc_term:1;
+	unsigned warn_port_pstk:1;
 } wctx_t;
 
 static FILE *f = NULL;
@@ -148,6 +149,10 @@ pcb_hid_attribute_t openems_attribute_list[] = {
 	 PCB_HATT_STRING, 0, 0, {0, "SetGaussExcite(FDTD, f_max/2, f_max/2)", 0}, 0, 0},
 #define HA_excite 14
 
+	{"port-resistance", "default port resistance",
+	 PCB_HATT_REAL, 0, 1000, {0, 0, 50}, 0, 0}
+#define HA_def_port_res 15
+
 };
 
 #define NUM_OPTIONS (sizeof(openems_attribute_list)/sizeof(openems_attribute_list[0]))
@@ -249,7 +254,7 @@ static void print_lparm(wctx_t *ctx, pcb_layergrp_t *grp, const char *attr, int
 	}
 #endif
 
-	opt = (grp->type & PCB_LYT_COPPER) ? cop_opt : subs_opt;
+	opt = (grp->ltype & PCB_LYT_COPPER) ? cop_opt : subs_opt;
 	assert(opt >= 0);
 	if (is_str)
 		pcb_fprintf(ctx->f, "%s", ctx->options[opt].str_value);
@@ -270,9 +275,9 @@ static void openems_write_layers(wctx_t *ctx)
 	/* linear map of copper and substrate layers */
 	for(gid = 0; gid < ctx->pcb->LayerGroups.len; gid++) {
 		pcb_layergrp_t *grp = &ctx->pcb->LayerGroups.grp[gid];
-		int iscop = (grp->type & PCB_LYT_COPPER);
+		int iscop = (grp->ltype & PCB_LYT_COPPER);
 
-		if (!(iscop) && !(grp->type & PCB_LYT_SUBSTRATE))
+		if (!(iscop) && !(grp->ltype & PCB_LYT_SUBSTRATE))
 			continue;
 		ctx->lg_ems2pcb[next] = gid;
 		ctx->lg_pcb2ems[gid] = next;
@@ -353,108 +358,119 @@ static void openems_write_outline(wctx_t *ctx)
 	/* create all substrate layers using this polygon*/
 	for(n = 1; n < ctx->lg_next; n++) {
 		pcb_layergrp_t *grp = &ctx->pcb->LayerGroups.grp[ctx->lg_ems2pcb[n]];
-		if (grp->type & PCB_LYT_SUBSTRATE)
+		if (grp->ltype & PCB_LYT_SUBSTRATE)
 			fprintf(ctx->f, "CSX = AddPcbrndPoly(CSX, PCBRND, %d, outline_xy, 1);\n", n);
 	}
 
 	fprintf(ctx->f, "\n");
 }
 
-static void openems_write_testpoint_(wctx_t *ctx, pcb_coord_t x, pcb_coord_t y, int layer, const char *refdes, const char *term)
+static void openems_vport_write(wctx_t *ctx, pcb_any_obj_t *o, pcb_coord_t x, pcb_coord_t y, pcb_layergrp_id_t gid1, pcb_layergrp_id_t gid2, const char *port_name)
 {
-	long oid = ctx->oid++;
-	pcb_coord_t sx = PCB_MM_TO_COORD(0.1), sy = PCB_MM_TO_COORD(0.1);
+	char *end, *s, *safe_name = pcb_strdup(port_name);
+	const char *att;
+	double resistance = ctx->options[HA_def_port_res].real_value;
+	int act = 1;
 
-	pcb_fprintf(ctx->f, "points%ld(1, 1) = %mm; points%ld(2, 1) = %mm;\n", oid, x-sx, oid, -(y-sy));
-	pcb_fprintf(ctx->f, "points%ld(1, 2) = %mm; points%ld(2, 2) = %mm;\n", oid, x+sx, oid, -(y-sy));
-	pcb_fprintf(ctx->f, "points%ld(1, 3) = %mm; points%ld(2, 3) = %mm;\n", oid, x+sx, oid, -(y+sy));
-	pcb_fprintf(ctx->f, "points%ld(1, 4) = %mm; points%ld(2, 4) = %mm;\n", oid, x-sx, oid, -(y+sy));
-	fprintf(ctx->f, "refdes = '%s';\n", refdes);
-	fprintf(ctx->f, "pad.number = '%s';\n", term);
-	fprintf(ctx->f, "pad.id = '%ld';\n", ++ctx->pad_id);
-	fprintf(ctx->f, "PCBRND = RegPcbrndPad(PCBRND, %d, points%ld, refdes, pad);\n", layer, oid);
-	fprintf(ctx->f, "[pad_points layer_number] = LookupPcbrndPort(PCBRND, refdes, pad);\n");
-	fprintf(ctx->f, "[ start stop] = CalcPcbrndPoly2Port(PCBRND, points%ld, layer_number);\n", oid);
-}
+	ctx->port_id++;
 
-
-static void openems_write_testpoint_on(wctx_t *ctx, const char *refdes, const char *termid, pcb_layergrp_id_t gid, pcb_coord_t x, pcb_coord_t y)
-{
-	int layer;
-
-	layer = ctx->lg_pcb2ems[gid];
-	if (layer <= 0) {
-		pcb_message(PCB_MSG_ERROR, "Can't determine EMS layer for pad (%$mm;%$mm)\n", x, y);
-		return;
+	att = pcb_attribute_get(&o->Attributes, "openems::resistance");
+	if (att != NULL) {
+		double tmp = strtod(att, &end);
+		if (*end == '\0')
+			resistance = tmp;
+		else
+			pcb_message(PCB_MSG_WARNING, "Ignoring invalid openems::resistance value for port %s: '%s' (must be a number without suffix)\n", port_name, att);
 	}
 
-	openems_write_testpoint_(ctx, x, y, layer, refdes, termid);
+	att = pcb_attribute_get(&o->Attributes, "openems::active");
+	if (att != NULL) {
+		if (pcb_strcasecmp(att, "true") == 0)
+			act = 1;
+		else if (pcb_strcasecmp(att, "false") == 0)
+			act = 0;
+		else
+			pcb_message(PCB_MSG_WARNING, "Ignoring invalid openems::active value for port %s: '%s' (must be true or false)\n", port_name, att);
+	}
+
+	for(s = safe_name; *s != '\0'; s++)
+		if (!isalnum(*s))
+			*s = '_';
+
+	pcb_fprintf(ctx->f, "\npoint(1, 1) = %mm; point(2, 1) = %mm;\n",x, y);
+	fprintf(ctx->f, "start_layer = %d;\n", ctx->lg_pcb2ems[gid1]);
+	fprintf(ctx->f, "stop_layer = %d;\n", ctx->lg_pcb2ems[gid2]);
+	fprintf(ctx->f, "[%s_start, %s_stop] = CalcPcbrnd2PortV(PCBRND, point, start_layer, stop_layer);\n", safe_name, safe_name);
+	fprintf(ctx->f, "[CSX, port{%ld}] = AddLumpedPort(CSX, 999, %ld, %f, %s_start, %s_stop, [0 0 -1], %s);\n", ctx->port_id, ctx->port_id, resistance, safe_name, safe_name, act ? "true" : "false");
+
+	free(safe_name);
 }
 
-static void openems_write_testpoint(wctx_t *ctx, pcb_any_obj_t *o, pcb_coord_t x, pcb_coord_t y)
+pcb_layergrp_id_t openems_vport_main_group_pstk(pcb_board_t *pcb, pcb_pstk_t *ps, int *gstep, const char *port_name)
 {
-	pcb_layergrp_id_t gid;
-	const char *refdes = NULL;
-	pcb_subc_t *sc;
+	int top, bot, intern;
+	pcb_layergrp_id_t gid1;
 
-	sc = pcb_obj_parent_subc(o);
-	if (sc != NULL)
-		refdes = sc->refdes;
-
-	if (refdes == NULL)
-		refdes = "none";
-
-	if (o->type == PCB_OBJ_PSTK) { /* light terminal: padstack */
-		for(gid = 0; gid < ctx->pcb->LayerGroups.len; gid++) { /* put testpoint on all interesting layers */
-			pcb_layergrp_t *grp = &ctx->pcb->LayerGroups.grp[gid];
-			pcb_layer_id_t lid;
-			pcb_pstk_shape_t *sh;
-
-			if (grp->len <= 0) /* group has no layers -> probably substrate */
-				continue;
-
-			if (!(grp->type & PCB_LYT_COPPER)) /* testpoint goes on copper only */
-				continue;
-
-			if (!(grp->type & PCB_LYT_TOP) && !(grp->type & PCB_LYT_BOTTOM)) /* do not put testpoints on inner layers */
-				continue;
-
-			lid = grp->lid[0];
-			if (lid < 0) /* invalid layer in group, what?! */
-				continue;
-
-			sh = pcb_pstk_shape_at(ctx->pcb, (pcb_pstk_t *)o, &ctx->pcb->Data->Layer[lid]);
-			if (sh == NULL) /* padstack has no shape on layer */
-				continue;
-
-			openems_write_testpoint_on(ctx, refdes, o->term, gid, x, y);
-		}
+	top = (pcb_pstk_shape(ps, PCB_LYT_COPPER | PCB_LYT_TOP, 0) != NULL);
+	bot = (pcb_pstk_shape(ps, PCB_LYT_COPPER | PCB_LYT_BOTTOM, 0) != NULL);
+	intern = (pcb_pstk_shape(ps, PCB_LYT_INTERN | PCB_LYT_BOTTOM, 0) != NULL);
+	if (intern) {
+		pcb_message(PCB_MSG_ERROR, "Can not export openems vport %s: it has internal copper\n(must be either top or bottom copper)\n", port_name);
+		return -1;
 	}
-	else { /* heavy terminal: plain layer object */
-		assert(o->parent_type == PCB_PARENT_LAYER);
-
-		gid = pcb_layer_get_group_(o->parent.layer);
-		if (gid < 0) {
-			pcb_message(PCB_MSG_ERROR, "Can't determine pad layer (%$mm;%$mm)\n", x, y);
-			return;
-		}
-
-		openems_write_testpoint_on(ctx, refdes, o->term, gid, x, y);
+	if (top && bot) {
+		pcb_message(PCB_MSG_ERROR, "Can not export openems vport %s: it has both top and bottom copper\n", port_name);
+		return -1;
 	}
+	if (!top && !bot) {
+		pcb_message(PCB_MSG_ERROR, "Can not export openems vport %s: it does not have copper either on top or bottom\n", port_name);
+		return -1;
+	}
+
+	/* pick main group */
+	if (top) {
+		gid1 = pcb_layergrp_get_top_copper();
+		*gstep = +1;
+	}
+	else {
+		gid1 = pcb_layergrp_get_bottom_copper();
+		*gstep = -1;
+	}
+	if (gid1 < 0) {
+		pcb_message(PCB_MSG_ERROR, "Can not export openems vport %s: can not find top or bottom layer group ID\n", port_name);
+		return -1;
+	}
+
+	return gid1;
 }
 
+pcb_layergrp_id_t openems_vport_aux_group(pcb_board_t *pcb, pcb_layergrp_id_t gid1, int gstep, const char *port_name)
+{
+	pcb_layergrp_id_t gid2;
+
+	for(gid2 = gid1 + gstep; (gid2 >= 0) && (gid2 <= pcb->LayerGroups.len); gid2 += gstep)
+		if (pcb->LayerGroups.grp[gid2].ltype & PCB_LYT_COPPER)
+			return gid2;
+
+	pcb_message(PCB_MSG_ERROR, "Can not export openems vport %s: can not find pair layer\n", port_name);
+	return -1;
+}
 
 #define TPMASK (PCB_OBJ_LINE | PCB_OBJ_PSTK | PCB_OBJ_SUBC)
 static void openems_write_testpoints(wctx_t *ctx, pcb_data_t *data)
 {
 	pcb_any_obj_t *o;
 	pcb_data_it_t it;
-	
+
 	for(o = pcb_data_first(&it, data, TPMASK); o != NULL; o = pcb_data_next(&it)) {
+		const char *port_name;
 		if (o->type == PCB_OBJ_SUBC)
 			openems_write_testpoints(ctx, ((pcb_subc_t *)o)->data);
-		if (o->term == NULL)
+
+		port_name = pcb_attribute_get(&o->Attributes, "openems::vport");
+		if (port_name == NULL)
 			continue;
+
 		if (o->type == PCB_OBJ_SUBC) {
 			if (!ctx->warn_subc_term)
 				pcb_message(PCB_MSG_ERROR, "Subcircuit being a terminal is not supported.\n");
@@ -462,10 +478,38 @@ static void openems_write_testpoints(wctx_t *ctx, pcb_data_t *data)
 			continue;
 		}
 
-		/* place the test pad */
+		/* place the vertical port */
 		switch(o->type) {
-			case PCB_OBJ_PSTK: openems_write_testpoint(ctx, o, ((pcb_pstk_t *)o)->x, ((pcb_pstk_t *)o)->y); break;
-			default: openems_write_testpoint(ctx, o, (o->BoundingBox.X1+o->BoundingBox.X2)/2, (o->BoundingBox.Y1+o->BoundingBox.Y2)/2);
+			case PCB_OBJ_PSTK:
+				{
+					int gstep;
+					pcb_layergrp_id_t gid1, gid2;
+					pcb_pstk_t *ps = (pcb_pstk_t *)o;
+
+					if (port_name == NULL)
+						break;
+
+					gid1 = openems_vport_main_group_pstk(ctx->pcb, ps, &gstep, port_name);
+					if (gid1 < 0)
+						break;
+
+					gid2 = openems_vport_aux_group(ctx->pcb, gid1, gstep, port_name);
+					if (gid2 < 0)
+						break;
+
+#warning TODO: check if there is copper object on hid2 at x;y
+
+					if (pcb_attribute_get(&o->Attributes, "openems::vport-reverse") == NULL)
+						openems_vport_write(ctx, (pcb_any_obj_t *)ps, ps->x, ps->y, gid1, gid2, port_name);
+					else
+						openems_vport_write(ctx, (pcb_any_obj_t *)ps, ps->x, ps->y, gid2, gid1, port_name);
+				}
+				break;
+				default:
+					if (!ctx->warn_port_pstk)
+						pcb_message(PCB_MSG_ERROR, "Only padstacks can be openems ports at the moment\n");
+					ctx->warn_port_pstk = 1;
+					break;
 		}
 	}
 }
@@ -606,10 +650,10 @@ static void openems_do_export(pcb_hid_attr_val_t * options)
 	f = NULL;
 }
 
-static void openems_parse_arguments(int *argc, char ***argv)
+static int openems_parse_arguments(int *argc, char ***argv)
 {
 	pcb_hid_register_attributes(openems_attribute_list, sizeof(openems_attribute_list) / sizeof(openems_attribute_list[0]), openems_cookie, 0);
-	pcb_hid_parse_command_line(argc, argv);
+	return pcb_hid_parse_command_line(argc, argv);
 }
 
 static int openems_set_layer_group(pcb_layergrp_id_t group, pcb_layer_id_t layer, unsigned int flags, int is_empty)
@@ -736,7 +780,7 @@ static void openems_calibrate(double xval, double yval)
 	return;
 }
 
-static void openems_set_crosshair(int x, int y, int a)
+static void openems_set_crosshair(pcb_coord_t x, pcb_coord_t y, int a)
 {
 }
 
@@ -766,6 +810,8 @@ void pplg_uninit_export_openems(void)
 
 int pplg_init_export_openems(void)
 {
+	PCB_API_CHK_VER;
+
 	memset(&openems_hid, 0, sizeof(pcb_hid_t));
 
 	pcb_hid_nogui_init(&openems_hid);
