@@ -347,10 +347,10 @@ static void openscad_do_export(pcb_hid_attr_val_t * options)
 	f = NULL;
 }
 
-static void openscad_parse_arguments(int *argc, char ***argv)
+static int openscad_parse_arguments(int *argc, char ***argv)
 {
 	pcb_hid_register_attributes(openscad_attribute_list, sizeof(openscad_attribute_list) / sizeof(openscad_attribute_list[0]), openscad_cookie, 0);
-	pcb_hid_parse_command_line(argc, argv);
+	return pcb_hid_parse_command_line(argc, argv);
 }
 
 
@@ -580,7 +580,7 @@ static void openscad_calibrate(double xval, double yval)
 	return;
 }
 
-static void openscad_set_crosshair(int x, int y, int a)
+static void openscad_set_crosshair(pcb_coord_t x, pcb_coord_t y, int a)
 {
 }
 
@@ -661,6 +661,8 @@ void pplg_uninit_export_openscad(void)
 
 int pplg_init_export_openscad(void)
 {
+	PCB_API_CHK_VER;
+
 	memset(&openscad_hid, 0, sizeof(pcb_hid_t));
 
 	pcb_hid_nogui_init(&openscad_hid);
