@@ -109,12 +109,6 @@ static int scad_draw_outline(void)
 	return 0;
 }
 
-static void scad_draw_drill(const pcb_pin_t *pin)
-{
-	pcb_fprintf(f, "	translate([%mm,%mm,0])\n", TRX_(pin->X), TRY_(pin->Y));
-	pcb_fprintf(f, "		cylinder(r=%mm, h=4, center=true, $fn=30);\n", pin->DrillingHole/2);
-}
-
 static void scad_draw_pstk(const pcb_pstk_t *ps)
 {
 	pcb_pstk_proto_t *proto = pcb_pstk_get_proto(ps);
@@ -138,19 +132,10 @@ static void scad_draw_drills(void)
 
 	fprintf(f, "module pcb_drill() {\n");
 
-	for(obj = pcb_r_first(PCB->Data->via_tree, &it); obj != NULL; obj = pcb_r_next(&it))
-		scad_draw_drill((pcb_pin_t *)obj);
-	pcb_r_end(&it);
-
 	for(obj = pcb_r_first(PCB->Data->padstack_tree, &it); obj != NULL; obj = pcb_r_next(&it))
 		scad_draw_pstk((pcb_pstk_t *)obj);
 	pcb_r_end(&it);
 
-	PCB_PIN_ALL_LOOP(PCB->Data); {
-		scad_draw_drill(pin);
-	} PCB_ENDALL_LOOP;
-
-
 	fprintf(f, "}\n");
 }
 
