@@ -1168,10 +1168,10 @@ static void png_do_export(pcb_hid_attr_val_t * options)
 	free(black);
 }
 
-static void png_parse_arguments(int *argc, char ***argv)
+static int png_parse_arguments(int *argc, char ***argv)
 {
 	pcb_hid_register_attributes(png_attribute_list, sizeof(png_attribute_list) / sizeof(png_attribute_list[0]), png_cookie, 0);
-	pcb_hid_parse_command_line(argc, argv);
+	return pcb_hid_parse_command_line(argc, argv);
 }
 
 
@@ -1606,7 +1606,7 @@ static void png_fill_rect(pcb_hid_gc_t gc, pcb_coord_t x1, pcb_coord_t y1, pcb_c
 
 static void png_draw_line_(gdImagePtr im, pcb_hid_gc_t gc, pcb_coord_t x1, pcb_coord_t y1, pcb_coord_t x2, pcb_coord_t y2)
 {
-	if (x1 == x2 && y1 == y2) {
+	if (x1 == x2 && y1 == y2 && !photo_mode) {
 		pcb_coord_t w = gc->width / 2;
 		if (gc->cap != Square_Cap)
 			png_fill_circle(gc, x1, y1, w);
@@ -1827,7 +1827,7 @@ static void png_calibrate(double xval, double yval)
 	CRASH("png_calibrate");
 }
 
-static void png_set_crosshair(int x, int y, int a)
+static void png_set_crosshair(pcb_coord_t x, pcb_coord_t y, int a)
 {
 }
 
@@ -1850,6 +1850,8 @@ void pplg_uninit_export_png(void)
 
 int pplg_init_export_png(void)
 {
+	PCB_API_CHK_VER;
+
 	memset(&png_hid, 0, sizeof(pcb_hid_t));
 
 	pcb_hid_nogui_init(&png_hid);
