@@ -31,7 +31,7 @@
 /*----------------------------------------------------------------------------*/
 static pcb_hid_attribute_t *eps_get_export_options(int *n);
 static void eps_do_export(pcb_hid_attr_val_t * options);
-static void eps_parse_arguments(int *argc, char ***argv);
+static int eps_parse_arguments(int *argc, char ***argv);
 static int eps_set_layer_group(pcb_layergrp_id_t group, pcb_layer_id_t layer, unsigned int flags, int is_empty);
 static pcb_hid_gc_t eps_make_gc(void);
 static void eps_destroy_gc(pcb_hid_gc_t gc);
@@ -47,7 +47,7 @@ static void eps_fill_circle(pcb_hid_gc_t gc, pcb_coord_t cx, pcb_coord_t cy, pcb
 static void eps_fill_polygon(pcb_hid_gc_t gc, int n_coords, pcb_coord_t * x, pcb_coord_t * y);
 static void eps_fill_polygon_ffs(pcb_hid_gc_t gc, int n_coords, pcb_coord_t *x, pcb_coord_t *y, pcb_coord_t dx, pcb_coord_t dy);
 static void eps_calibrate(double xval, double yval);
-static void eps_set_crosshair(int x, int y, int action);
+static void eps_set_crosshair(pcb_coord_t x, pcb_coord_t y, int action);
 /*----------------------------------------------------------------------------*/
 
 typedef struct hid_gc_s {
@@ -134,9 +134,9 @@ Limit the bounds of the EPS file to the visible items.
 
 PCB_REGISTER_ATTRIBUTES(eps_attribute_list, ps_cookie)
 
-		 static pcb_hid_attr_val_t eps_values[NUM_OPTIONS];
+static pcb_hid_attr_val_t eps_values[NUM_OPTIONS];
 
-		 static pcb_hid_attribute_t *eps_get_export_options(int *n)
+static pcb_hid_attribute_t *eps_get_export_options(int *n)
 {
 	static char *last_made_filename = 0;
 
@@ -337,10 +337,10 @@ static void eps_do_export(pcb_hid_attr_val_t * options)
 	fclose(f);
 }
 
-static void eps_parse_arguments(int *argc, char ***argv)
+static int eps_parse_arguments(int *argc, char ***argv)
 {
 	pcb_hid_register_attributes(eps_attribute_list, sizeof(eps_attribute_list) / sizeof(eps_attribute_list[0]), ps_cookie, 0);
-	pcb_hid_parse_command_line(argc, argv);
+	return pcb_hid_parse_command_line(argc, argv);
 }
 
 static int is_mask;
@@ -613,7 +613,7 @@ static void eps_calibrate(double xval, double yval)
 	CRASH("eps_calibrate");
 }
 
-static void eps_set_crosshair(int x, int y, int action)
+static void eps_set_crosshair(pcb_coord_t x, pcb_coord_t y, int action)
 {
 }
 
