@@ -332,10 +332,10 @@ static void svg_do_export(pcb_hid_attr_val_t * options)
 	f = NULL;
 }
 
-static void svg_parse_arguments(int *argc, char ***argv)
+static int svg_parse_arguments(int *argc, char ***argv)
 {
 	pcb_hid_register_attributes(svg_attribute_list, sizeof(svg_attribute_list) / sizeof(svg_attribute_list[0]), svg_cookie, 0);
-	pcb_hid_parse_command_line(argc, argv);
+	return pcb_hid_parse_command_line(argc, argv);
 }
 
 static int svg_set_layer_group(pcb_layergrp_id_t group, pcb_layer_id_t layer, unsigned int flags, int is_empty)
@@ -787,7 +787,7 @@ static void svg_calibrate(double xval, double yval)
 	return;
 }
 
-static void svg_set_crosshair(int x, int y, int a)
+static void svg_set_crosshair(pcb_coord_t x, pcb_coord_t y, int a)
 {
 }
 
@@ -810,6 +810,8 @@ void pplg_uninit_export_svg(void)
 
 int pplg_init_export_svg(void)
 {
+	PCB_API_CHK_VER;
+
 	memset(&svg_hid, 0, sizeof(pcb_hid_t));
 
 	pcb_hid_nogui_init(&svg_hid);
