@@ -34,11 +34,11 @@
 #include "hid_actions.h"
 #include "hid_dad.h"
 #include "action_helper.h"
+#include "plugins.h"
 
 #include "board.h"
 #include "compat_fs.h"
 #include "compat_misc.h"
-#include "const.h"
 #include "conf_core.h"
 #include "data.h"
 #include "buffer.h"
@@ -70,13 +70,13 @@ typedef struct extedit_method_s {
 } extedit_method_t;
 
 static extedit_method_t methods[] = {
-	{"pcb-rnd",         PCB_TYPE_SUBC | PCB_TYPE_ELEMENT, EEF_LIHATA, "pcb-rnd \"%f\""},
-	{"editor",          PCB_TYPE_SUBC | PCB_TYPE_ELEMENT, EEF_LIHATA, "xterm -e editor \"%f\""},
+	{"pcb-rnd",         PCB_OBJ_SUBC, EEF_LIHATA, "pcb-rnd \"%f\""},
+	{"editor",          PCB_OBJ_SUBC, EEF_LIHATA, "xterm -e editor \"%f\""},
 	{NULL, 0, 0, NULL}
 };
 
 /* accept these objects for external editing */
-#define EXTEDIT_TYPES (PCB_TYPE_SUBC | PCB_TYPE_ELEMENT)
+#define EXTEDIT_TYPES (PCB_OBJ_SUBC)
 
 #include "extedit_dad.c"
 
@@ -307,6 +307,7 @@ void pplg_uninit_extedit(void)
 #include "dolists.h"
 int pplg_init_extedit(void)
 {
+	PCB_API_CHK_VER;
 	PCB_REGISTER_ACTIONS(extedit_action_list, extedit_cookie)
 	return 0;
 }
