@@ -1,3 +1,29 @@
+/*
+ *                            COPYRIGHT
+ *
+ *  pcb-rnd, interactive printed circuit board design
+ *  Copyright (C) 2016, 2017, 2018 Tibor 'Igor2' Palinkas
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ *
+ *  Contact:
+ *    Project page: http://repo.hu/projects/pcb-rnd
+ *    lead developer: email to pcb-rnd (at) igor2.repo.hu
+ *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
+ */
+
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
@@ -159,3 +185,45 @@ int md5_cmp_free(const char *last_fn, char *md5_last, char *md5_new)
 	free(md5_new);
 	return changed;
 }
+
+int fp_wget_search_(char *out, int out_len, FILE *f, const char *fn)
+{
+	char *line, line_[8192];
+
+	*out = '\0';
+
+	if (f == NULL)
+		return -1;
+
+	while((line = fgets(line_, sizeof(line_), f)) != NULL) {
+		char *sep;
+		sep = strchr(line, '|');
+		if (sep == NULL)
+			continue;
+		*sep = '\0';
+		if ((strstr(line, fn) != NULL) && (strlen(line) < out_len)) {
+			strcpy(out, line);
+			return 0;
+		}
+	}
+	return -1;
+}
+
+
+int fp_wget_search(char *out, int out_len, char *name, int offline, const char *url, const char *cache)
+{
+	FILE *f_idx;
+	int fctx;
+	fp_get_mode mode = offline ? FP_WGET_OFFLINE : 0;
+
+	if (fp_wget_open(url, cache, &f_idx, &fctx, mode) == 0) {
+		if (fp_wget_search_(out, out_len, f_idx, name) != 0) {
+			fp_wget_close(&f_idx, &fctx);
+			return -1;
+		}
+	}
+	else
+		return -1;
+	fp_wget_close(&f_idx, &fctx);
+	return 0;
+}
