@@ -68,13 +68,13 @@ void gpmi_hid_do_export(pcb_hid_attr_val_t * options)
 	h->result = NULL;
 }
 
-void gpmi_hid_parse_arguments(int *pcbargc, char ***pcbargv)
+int gpmi_hid_parse_arguments(int *pcbargc, char ***pcbargv)
 {
 	/* Do nothing for now */
-	pcb_hid_parse_command_line(pcbargc, pcbargv);
+	return pcb_hid_parse_command_line(pcbargc, pcbargv);
 }
 
-void gpmi_hid_set_crosshair(int x, int y, int cursor_action)
+void gpmi_hid_set_crosshair(pcb_coord_t x, pcb_coord_t y, int cursor_action)
 {
 	/* Do nothing */
 }
@@ -165,14 +165,3 @@ void gpmi_hid_fill_rect(pcb_hid_gc_t gc, pcb_coord_t x1, pcb_coord_t y1, pcb_coo
 	gpmi_event(h->module, HIDE_fill_rect, h, gc, x1, y1, x2, y2);
 }
 
-void gpmi_hid_fill_pcb_pv(pcb_hid_gc_t fg_gc, pcb_hid_gc_t bg_gc, pcb_pin_t *pad, pcb_bool drawHole, pcb_bool mask)
-{
-	gpmi_hid_t *h = hid_gpmi_data_get(pcb_exporter);
-	gpmi_event(h->module, HIDE_fill_pcb_pv, h, fg_gc, bg_gc, pad, drawHole, mask);
-}
-
-void gpmi_hid_fill_pcb_pad(pcb_hid_gc_t gc, pcb_pad_t * pad, pcb_bool clear, pcb_bool mask)
-{
-	gpmi_hid_t *h = hid_gpmi_data_get(pcb_exporter);
-	gpmi_event(h->module, HIDE_fill_pcb_pad, h, gc, pad, clear, mask);
-}
