@@ -1,12 +1,12 @@
 #include <stdlib.h>
 #include <assert.h>
 #include "layout.h"
-#include "src/const.h"
 #include "src/board.h"
 #include "src/undo.h"
 #include "src/conf_core.h"
 #include "src/layer.h"
 #include "src/compat_misc.h"
+#include "src_plugins/lib_compat_help/pstk_compat.h"
 
 
 typedef struct flag_tr_s {
@@ -73,7 +73,7 @@ static void *layout_create_line_(pcb_layer_t *layer, int x1, int y1, int x2, int
 
 	line = pcb_line_new(layer, x1, y1, x2, y2, thickness, clearance, get_flags(flags));
 	if (line != NULL) {
-		pcb_undo_add_obj_to_create(PCB_TYPE_LINE, layer, line, line);
+		pcb_undo_add_obj_to_create(PCB_OBJ_LINE, layer, line, line);
 		return line;
 	}
 	return NULL;
@@ -93,10 +93,10 @@ static void *layout_create_via_(int x, int y, int thickness, int clearance, int
 {
 	void *pin;
 
-	pin = pcb_via_new(PCB->Data, x, y, thickness, clearance, mask, hole, name, get_flags(flags));
+	pin = pcb_pstk_new_compat_via(PCB->Data, x, y, hole, thickness, clearance, mask, PCB_PSTK_COMPAT_ROUND, 1);
 
 	if (pin != NULL) {
-		pcb_undo_add_obj_to_create(PCB_TYPE_VIA, pin, pin, pin);
+		pcb_undo_add_obj_to_create(PCB_OBJ_PSTK, pin, pin, pin);
 		return pin;
 	}
 	return NULL;
@@ -113,7 +113,7 @@ static void *layout_create_arc_(pcb_layer_t *layer, int x, int y, int width, int
 	void *arc;
 	arc = pcb_arc_new(layer, x, y, width, height, sa, dir, thickness, clearance, get_flags(flags));
 	if (arc != NULL) {
-		pcb_undo_add_obj_to_create(PCB_TYPE_ARC, layer, arc, arc);
+		pcb_undo_add_obj_to_create(PCB_OBJ_ARC, layer, arc, arc);
 		return 0;
 	}
 	return NULL;
