@@ -112,8 +112,6 @@ typedef struct layout_object_s {
 		pcb_text_t    *t;
 		pcb_poly_t *p;
 		pcb_arc_t     *a;
-		pcb_pin_t     *v;
-		pcb_pin_t     *pin;
 	} obj;
 	pcb_layer_id_t layer;
 } layout_object_t;
@@ -257,19 +255,6 @@ typedef struct dctx_s {
 	void *gc;
 } dctx_t;
 
-/* -- debug draw GC -- */
-/* Initialize debug drawing; returns 1 if worked, 0 if denied */
-int debug_draw_request(void);
-
-/* Flush the drawing */
-void debug_draw_flush(void);
-
-/* Finish (close) drawing */
-void debug_draw_finish(dctx_t *ctx);
-
-/* Get the draw context of debug draw */
-dctx_t *debug_draw_dctx(void);
-
 /* -- draw on a GC -- */
 
 /* Debug draw style: set drawing color */
@@ -295,10 +280,6 @@ void fill_circle(dctx_t *ctx, int cx_, int cy_, int radius_);
 void fill_polygon(dctx_t *ctx, int n_ints_, int *x_, int *y_);
 void fill_pcb_polygon(dctx_t *ctx, pcb_poly_t *poly, const pcb_box_t *clip_box);
 void thindraw_pcb_polygon(dctx_t *ctx, pcb_poly_t *poly, const pcb_box_t *clip_box);
-void fill_pcb_pad(dctx_t *ctx, pcb_pad_t *pad, pcb_bool clip, pcb_bool mask);
-void thindraw_pcb_pad(dctx_t *ctx, pcb_pad_t *pad, pcb_bool clip, pcb_bool mask);
-void fill_pcb_pv(pcb_hid_gc_t fg_gc, pcb_hid_gc_t bg_gc, pcb_pin_t *pv, pcb_bool drawHole, pcb_bool mask);
-void thindraw_pcb_pv(pcb_hid_gc_t fg_gc, pcb_hid_gc_t bg_gc, pcb_pin_t *pv, pcb_bool drawHole, pcb_bool mask);
 void fill_rect(dctx_t *ctx, int x1_, int y1_, int x2_, int y2_);
 */
 
