@@ -33,6 +33,8 @@ typedef struct hid_gc_s {
 
 static pcb_hid_attribute_t *batch_get_export_options(int *n_ret)
 {
+	if (n_ret != NULL)
+		*n_ret = 0;
 	return 0;
 }
 
@@ -143,9 +145,9 @@ static void batch_do_exit(pcb_hid_t *hid)
 	batch_stay = 0;
 }
 
-static void batch_parse_arguments(int *argc, char ***argv)
+static int batch_parse_arguments(int *argc, char ***argv)
 {
-	pcb_hid_parse_command_line(argc, argv);
+	return pcb_hid_parse_command_line(argc, argv);
 }
 
 static void batch_invalidate_lr(pcb_coord_t l, pcb_coord_t r, pcb_coord_t t, pcb_coord_t b)
@@ -237,7 +239,7 @@ static void batch_get_coords(const char *msg, pcb_coord_t * x, pcb_coord_t * y)
 {
 }
 
-static void batch_set_crosshair(int x, int y, int action)
+static void batch_set_crosshair(pcb_coord_t x, pcb_coord_t y, int action)
 {
 }
 
@@ -286,7 +288,7 @@ static void batch_show_item(void *item)
 {
 }
 
-static void batch_create_menu(const char *menu, const char *action, const char *mnemonic, const char *accel, const char *tip, const char *cookie)
+static void batch_create_menu(const char *menu_path, const pcb_menu_prop_t *props)
 {
 }
 
@@ -331,6 +333,7 @@ void pplg_uninit_hid_batch(void)
 
 int pplg_init_hid_batch(void)
 {
+	PCB_API_CHK_VER;
 	memset(&batch_hid, 0, sizeof(pcb_hid_t));
 
 	pcb_hid_nogui_init(&batch_hid);
