@@ -7,6 +7,7 @@
 #include "clip.h"
 #include "data.h"
 #include "layer.h"
+#include "grid.h"
 #include "hid_draw_helpers.h"
 #include "hid_attrib.h"
 #include "hid_helper.h"
@@ -76,7 +77,7 @@ static const gchar *get_color_name(pcb_gtk_color_t * color)
 	return tmp;
 }
 
-/** Returns TRUE only if \p color_string has been allocated to \p color. */
+/* Returns TRUE only if color_string has been allocated to color. */
 static pcb_bool map_color_string(const char *color_string, pcb_gtk_color_t * color)
 {
 	static GdkColormap *colormap = NULL;
@@ -1374,26 +1375,6 @@ static gboolean ghid_gdk_preview_expose(GtkWidget * widget, pcb_gtk_expose_t *ev
 	return FALSE;
 }
 
-static pcb_hid_t *ghid_gdk_request_debug_draw(void)
-{
-	/* No special setup requirements, drawing goes into
-	 * the backing pixmap. */
-	return &gtk2_gdk_hid;
-}
-
-static void ghid_gdk_flush_debug_draw(void)
-{
-	ghid_gdk_screen_update();
-	gdk_flush();
-}
-
-static void ghid_gdk_finish_debug_draw(void)
-{
-	ghid_gdk_flush_debug_draw();
-	/* No special tear down requirements
-	 */
-}
-
 static void draw_lead_user(render_priv_t *priv)
 {
 	GdkWindow *window = gtk_widget_get_window(gport->drawing_area);
@@ -1479,9 +1460,5 @@ void ghid_gdk_install(pcb_gtk_common_t *common, pcb_hid_t *hid)
 		hid->fill_polygon = ghid_gdk_fill_polygon;
 		hid->fill_polygon_offs = ghid_gdk_fill_polygon_offs;
 		hid->fill_rect = ghid_gdk_fill_rect;
-
-		hid->request_debug_draw = ghid_gdk_request_debug_draw;
-		hid->flush_debug_draw = ghid_gdk_flush_debug_draw;
-		hid->finish_debug_draw = ghid_gdk_finish_debug_draw;
 	}
 }
