@@ -244,7 +244,7 @@ static int Save(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
 
 	if (pcb_strcasecmp(function, "Layout") == 0)
 		if (PCB->Filename)
-			return pcb_hid_actionl("SaveTo", "Layout", PCB->Filename, NULL);
+			return pcb_hid_actionl("SaveTo", "Layout", NULL);
 
 	setup_fsb_dialog();
 
@@ -881,7 +881,7 @@ static int attribute_dialog_add(lesstif_attr_dlg_t *ctx, Widget real_parent, att
 		case PCB_HATT_BEGIN_TABLE:
 			/* create content table */
 			numcol = ctx->attrs[i].pcb_hatt_table_cols;
-			len = pcb_hid_atrdlg_num_children(ctx->attrs, i+1, ctx->n_attrs);
+			len = pcb_hid_attrdlg_num_children(ctx->attrs, i+1, ctx->n_attrs);
 			numch = len  / numcol + !!(len % numcol);
 			w = pcb_motif_box(parent, XmStrCast(ctx->attrs[i].name), 't', numch, (ctx->attrs[i].pcb_hatt_flags & PCB_HATF_FRAME), (ctx->attrs[i].pcb_hatt_flags & PCB_HATF_SCROLL));
 
@@ -1138,7 +1138,7 @@ void *lesstif_attr_dlg_new(pcb_hid_attribute_t *attrs, int n_attrs, pcb_hid_attr
 
 	if (!PCB_HATT_IS_COMPOSITE(attrs[0].type)) {
 		stdarg_n = 0;
-		main_tbl = pcb_motif_box(topform, XmStrCast("layout"), 't', pcb_hid_atrdlg_num_children(ctx->attrs, 0, ctx->n_attrs), 0, 0);
+		main_tbl = pcb_motif_box(topform, XmStrCast("layout"), 't', pcb_hid_attrdlg_num_children(ctx->attrs, 0, ctx->n_attrs), 0, 0);
 		XtManageChild(main_tbl);
 		attribute_dialog_add(ctx, main_tbl, NULL, 0, 1);
 	}
@@ -1391,7 +1391,7 @@ static int PrintCalibrate(int argc, const char **argv, pcb_coord_t x, pcb_coord_
 {
 	pcb_hid_t *printer = pcb_hid_find_printer();
 	printer->calibrate(0.0, 0.0);
-	if (pcb_gui->attribute_dialog(printer_calibrate_attrs, 3,
+	if (pcb_attribute_dialog(printer_calibrate_attrs, 3,
 														printer_calibrate_values,
 														"Printer Calibration Values", "Enter calibration values for your printer", NULL))
 		return 1;
@@ -1509,13 +1509,7 @@ static void sizes_set()
 {
 	PCB->MaxWidth = sz_str2val(sz_pcb_w, 1);
 	PCB->MaxHeight = sz_str2val(sz_pcb_h, 1);
-	PCB->Bloat = sz_str2val(sz_bloat, 1);
-	PCB->Shrink = sz_str2val(sz_shrink, 1);
-	PCB->minWid = sz_str2val(sz_drc_wid, 1);
-	PCB->minSlk = sz_str2val(sz_drc_slk, 1);
-	PCB->minDrill = sz_str2val(sz_drc_drill, 1);
-	PCB->minRing = sz_str2val(sz_drc_ring, 1);
-#warning think these over - are these only for new designs amd we keep real values in PCB-> ?
+
 	conf_set_design("design/text_scale", "%s", sz_text);
 	conf_set_design("design/bloat", "%s", sz_bloat);
 	conf_set_design("design/shrink", "%s", sz_shrink);
@@ -1535,12 +1529,6 @@ void lesstif_sizes_reset()
 		return;
 	sz_val2str(sz_pcb_w, PCB->MaxWidth, 1);
 	sz_val2str(sz_pcb_h, PCB->MaxHeight, 1);
-	sz_val2str(sz_bloat, PCB->Bloat, 1);
-	sz_val2str(sz_shrink, PCB->Shrink, 1);
-	sz_val2str(sz_drc_wid, PCB->minWid, 1);
-	sz_val2str(sz_drc_slk, PCB->minSlk, 1);
-	sz_val2str(sz_drc_drill, PCB->minDrill, 1);
-	sz_val2str(sz_drc_ring, PCB->minRing, 1);
 	sz_val2str(sz_text, conf_core.design.text_scale, 0);
 
 	ls = pcb_strdup_printf(_("Units are %s."), conf_core.editor.grid_unit->in_suffix);
@@ -1706,7 +1694,7 @@ static int pcb_act_fontsel(int argc, const char **argv, pcb_coord_t x, pcb_coord
 			int type;
 			void *ptr1, *ptr2, *ptr3;
 			pcb_gui->get_coords(_("Select an Object"), &x, &y);
-			if ((type = pcb_search_screen(x, y, PCB_CHANGENAME_TYPES, &ptr1, &ptr2, &ptr3)) != PCB_TYPE_NONE) {
+			if ((type = pcb_search_screen(x, y, PCB_CHANGENAME_TYPES, &ptr1, &ptr2, &ptr3)) != PCB_OBJ_VOID) {
 /*				pcb_undo_save_serial();*/
 				lesstif_show_fontsel_edit(ptr1, ptr2, type);
 			}
