@@ -18,6 +18,7 @@
 #include "lesstif.h"
 #include "stdarg.h"
 #include "event.h"
+#include "tool.h"
 
 static Widget library_dialog = 0;
 static Widget library_list, libnode_list;
@@ -64,8 +65,8 @@ static void libnode_select(Widget w, void *v, XmListCallbackStruct * cbs)
 {
 	pcb_fplibrary_t *e = picks.array[last_pick];
 	e = &e->data.dir.children.array[cbs->item_position - 1];
-	if (pcb_element_load_to_buffer(PCB_PASTEBUFFER, e->data.fp.loc_info, NULL))
-		pcb_crosshair_set_mode(PCB_MODE_PASTE_BUFFER);
+	if (pcb_buffer_load_footprint(PCB_PASTEBUFFER, e->data.fp.loc_info, NULL))
+		pcb_tool_select_by_id(PCB_MODE_PASTE_BUFFER);
 }
 
 static int build_library_dialog()
