@@ -18,6 +18,7 @@
 #include "hid_cfg.h"
 #include "hid_cfg_action.h"
 #include "hid_cfg_input.h"
+#include "conf_hid.h"
 #include "lesstif.h"
 #include "paths.h"
 #include "hid_actions.h"
@@ -30,6 +31,7 @@
 
 Widget lesstif_menubar;
 pcb_hid_cfg_t *lesstif_cfg;
+conf_hid_id_t lesstif_menuconf_id = -1;
 
 #ifndef R_OK
 /* Common value for systems that don't define it.  */
@@ -59,329 +61,6 @@ static int GetXY(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
 
 /*-----------------------------------------------------------------------------*/
 
-#define LB_RATS	(PCB_MAX_LAYER+0)
-#define LB_NUMPICK (LB_RATS+1)
-/* more */
-#define LB_BACK	(PCB_MAX_LAYER+1)
-#define LB_SUBC	(PCB_MAX_LAYER+2)
-#define LB_SUBC_PARTS	(PCB_MAX_LAYER+3)
-#define LB_PSTK_MARKS	(PCB_MAX_LAYER+4)
-#define LB_HOLES	(PCB_MAX_LAYER+5)
-#define LB_NUM  (PCB_MAX_LAYER+6)
-
-typedef struct {
-	Widget w[LB_NUM];
-	int is_pick;
-} LayerButtons;
-
-static LayerButtons *layer_button_list = 0;
-static int num_layer_buttons = 0;
-static int fg_colors[LB_NUM];
-static int bg_color;
-
-extern Widget lesstif_m_layer;
-
-void LesstifLayersChanged(void *user_data, int argc, pcb_event_arg_t argv[])
-{
-	int l, i, set;
-	const char *name;
-	int current_layer;
-
-	if (!layer_button_list)
-		return;
-	if (PCB && PCB->Data) {
-		pcb_data_t *d = PCB->Data;
-		for (i = 0; i < PCB_MAX_LAYER; i++) {
-			pcb_layer_type_t lyt = pcb_layer_flags(PCB, i);
-#warning layer TODO: hardwired layer colors
-			if (lyt & PCB_LYT_SILK) fg_colors[i] = lesstif_parse_color(conf_core.appearance.color.element);
-			else if (lyt & PCB_LYT_MASK) fg_colors[i] = lesstif_parse_color(conf_core.appearance.color.mask);
-			else if (lyt & PCB_LYT_PASTE) fg_colors[i] = lesstif_parse_color(conf_core.appearance.color.paste);
-			else fg_colors[i] = lesstif_parse_color(d->Layer[i].meta.real.color);
-		}
-
-		fg_colors[LB_RATS] = lesstif_parse_color(conf_core.appearance.color.rat);
-		fg_colors[LB_SUBC] = lesstif_parse_color(conf_core.appearance.color.subc);
-		fg_colors[LB_SUBC_PARTS] = lesstif_parse_color(conf_core.appearance.color.subc);
-		fg_colors[LB_PSTK_MARKS] = lesstif_parse_color(conf_core.appearance.color.subc);
-		fg_colors[LB_HOLES] = lesstif_parse_color(conf_core.appearance.color.via);
-		fg_colors[LB_BACK] = lesstif_parse_color(conf_core.appearance.color.invisible_objects);
-		bg_color = lesstif_parse_color(conf_core.appearance.color.background);
-	}
-	else {
-		for (i = 0; i < PCB_MAX_LAYER; i++)
-			fg_colors[i] = lesstif_parse_color(conf_core.appearance.color.layer[i]);
-		fg_colors[LB_RATS] = lesstif_parse_color(conf_core.appearance.color.rat);
-		fg_colors[LB_SUBC] = lesstif_parse_color(conf_core.appearance.color.subc);
-		fg_colors[LB_SUBC_PARTS] = lesstif_parse_color(conf_core.appearance.color.subc);
-		fg_colors[LB_PSTK_MARKS] = lesstif_parse_color(conf_core.appearance.color.subc);
-		fg_colors[LB_HOLES] = lesstif_parse_color(conf_core.appearance.color.via);
-		fg_colors[LB_BACK] = lesstif_parse_color(conf_core.appearance.color.invisible_objects);
-		bg_color = lesstif_parse_color(conf_core.appearance.color.background);
-	}
-
-	if (PCB->RatDraw)
-		current_layer = LB_RATS;
-	else
-		current_layer = pcb_layer_stack[0];
-
-	for (l = 0; l < num_layer_buttons; l++) {
-		LayerButtons *lb = layer_button_list + l;
-		for (i = 0; i < (lb->is_pick ? LB_NUMPICK : LB_NUM); i++) {
-			switch (i) {
-			case LB_RATS:
-				set = PCB->RatOn;
-				break;
-			case LB_SUBC:
-				set = PCB->SubcOn;
-				break;
-			case LB_SUBC_PARTS:
-				set = PCB->SubcPartsOn;
-				break;
-			case LB_PSTK_MARKS:
-				set = PCB->padstack_mark_on;
-				break;
-			case LB_BACK:
-				set = PCB->InvisibleObjectsOn;
-				break;
-			default:									/* layers */
-				set = PCB->Data->Layer[i].meta.real.vis;
-				break;
-			}
-
-			stdarg_n = 0;
-			if (i < PCB_MAX_LAYER && PCB->Data->Layer[i].name) {
-				XmString s = XmStringCreatePCB(PCB->Data->Layer[i].name);
-				stdarg(XmNlabelString, s);
-			}
-			if (!lb->is_pick) {
-				if (set) {
-					stdarg(XmNforeground, bg_color);
-					stdarg(XmNbackground, fg_colors[i]);
-				}
-				else {
-					stdarg(XmNforeground, fg_colors[i]);
-					stdarg(XmNbackground, bg_color);
-				}
-				stdarg(XmNset, set);
-			}
-			else {
-				stdarg(XmNforeground, bg_color);
-				stdarg(XmNbackground, fg_colors[i]);
-				stdarg(XmNset, current_layer == i ? True : False);
-			}
-			XtSetValues(lb->w[i], stdarg_args, stdarg_n);
-
-			if (i >= pcb_max_layer && i < PCB_MAX_LAYER)
-				XtUnmanageChild(lb->w[i]);
-			else
-				XtManageChild(lb->w[i]);
-		}
-	}
-	if (lesstif_m_layer) {
-		switch (current_layer) {
-		case LB_RATS:
-			name = "Rats";
-			break;
-		default:
-			name = PCB->Data->Layer[current_layer].name;
-			break;
-		}
-		stdarg_n = 0;
-		stdarg(XmNbackground, fg_colors[current_layer]);
-		stdarg(XmNforeground, bg_color);
-		stdarg(XmNlabelString, XmStringCreatePCB(name));
-		XtSetValues(lesstif_m_layer, stdarg_args, stdarg_n);
-	}
-
-	lesstif_update_layer_groups();
-
-	return;
-}
-
-static void show_one_layer_button(int layer, int set)
-{
-	int l;
-
-	stdarg_n = 0;
-	if (set) {
-		stdarg(XmNforeground, bg_color);
-		stdarg(XmNbackground, fg_colors[layer]);
-	}
-	else {
-		stdarg(XmNforeground, fg_colors[layer]);
-		stdarg(XmNbackground, bg_color);
-	}
-	stdarg(XmNset, set);
-
-	for (l = 0; l < num_layer_buttons; l++) {
-		LayerButtons *lb = layer_button_list + l;
-		if (!lb->is_pick)
-			XtSetValues(lb->w[layer], stdarg_args, stdarg_n);
-	}
-}
-
-static void layer_button_callback(Widget w, int layer, XmPushButtonCallbackStruct * pbcs)
-{
-	int l, set;
-	switch (layer) {
-	case LB_RATS:
-		set = PCB->RatOn = !PCB->RatOn;
-		break;
-	case LB_SUBC:
-		set = PCB->SubcOn = !PCB->SubcOn;
-		break;
-	case LB_SUBC_PARTS:
-		set = PCB->SubcPartsOn = !PCB->SubcPartsOn;
-		break;
-	case LB_PSTK_MARKS:
-		set = PCB->padstack_mark_on = !PCB->padstack_mark_on;
-		break;
-	case LB_HOLES:
-		set = PCB->hole_on = !PCB->hole_on;
-		break;
-	case LB_BACK:
-		set = PCB->InvisibleObjectsOn = !PCB->InvisibleObjectsOn;
-		break;
-	default:											/* layers */
-		set = PCB->Data->Layer[layer].meta.real.vis = !PCB->Data->Layer[layer].meta.real.vis;
-		break;
-	}
-
-	show_one_layer_button(layer, set);
-	if (layer < pcb_max_layer) {
-		int i;
-		pcb_layergrp_id_t group = pcb_layer_get_group(PCB, layer);
-		for (i = 0; i < PCB->LayerGroups.grp[group].len; i++) {
-			l = PCB->LayerGroups.grp[group].lid[i];
-			if (l != layer) {
-				show_one_layer_button(l, set);
-				PCB->Data->Layer[l].meta.real.vis = set;
-			}
-		}
-	}
-	lesstif_invalidate_all();
-}
-
-static void layerpick_button_callback(Widget w, int layer, XmPushButtonCallbackStruct * pbcs)
-{
-	int l, i;
-	const char *name;
-	PCB->RatDraw = (layer == LB_RATS);
-	if (layer < pcb_max_layer)
-		pcb_layervis_change_group_vis(layer, 1, 1);
-	for (l = 0; l < num_layer_buttons; l++) {
-		LayerButtons *lb = layer_button_list + l;
-		if (!lb->is_pick)
-			continue;
-		for (i = 0; i < LB_NUMPICK; i++)
-			XmToggleButtonSetState(lb->w[i], layer == i, False);
-	}
-	switch (layer) {
-	case LB_RATS:
-		name = "Rats";
-		break;
-	default:
-		name = PCB->Data->Layer[layer].name;
-		break;
-	}
-	stdarg_n = 0;
-	stdarg(XmNbackground, fg_colors[layer]);
-	stdarg(XmNforeground, bg_color);
-	stdarg(XmNlabelString, XmStringCreatePCB(name));
-	XtSetValues(lesstif_m_layer, stdarg_args, stdarg_n);
-	lesstif_invalidate_all();
-}
-
-static void insert_layerview_buttons(Widget menu)
-{
-	int i, s;
-	LayerButtons *lb;
-
-	num_layer_buttons++;
-	s = num_layer_buttons * sizeof(LayerButtons);
-	if (layer_button_list)
-		layer_button_list = (LayerButtons *) realloc(layer_button_list, s);
-	else
-		layer_button_list = (LayerButtons *) malloc(s);
-	lb = layer_button_list + num_layer_buttons - 1;
-
-	for (i = 0; i < LB_NUM; i++) {
-		static char namestr[] = "Label ";
-		const char *name = namestr;
-		Widget btn;
-		namestr[5] = 'A' + i;
-		switch (i) {
-		case LB_RATS:
-			name = "Rat Lines";
-			break;
-		case LB_SUBC:
-			name = "Subcircuits";
-			break;
-		case LB_SUBC_PARTS:
-			name = "Subc. parts";
-			break;
-		case LB_PSTK_MARKS:
-			name = "Pstk. Marks";
-			break;
-		case LB_HOLES:
-			name = "Holes";
-			break;
-		case LB_BACK:
-			name = "Far Side";
-			break;
-		}
-		stdarg_n = 0;
-		btn = XmCreateToggleButton(menu, XmStrCast(name), stdarg_args, stdarg_n);
-		XtManageChild(btn);
-		XtAddCallback(btn, XmNvalueChangedCallback, (XtCallbackProc) layer_button_callback, (XtPointer) (size_t) i);
-		lb->w[i] = btn;
-	}
-	lb->is_pick = 0;
-	LesstifLayersChanged(0, 0, 0);
-}
-
-static void insert_layerpick_buttons(Widget menu)
-{
-	int i, s;
-	LayerButtons *lb;
-
-	num_layer_buttons++;
-	s = num_layer_buttons * sizeof(LayerButtons);
-	if (layer_button_list)
-		layer_button_list = (LayerButtons *) realloc(layer_button_list, s);
-	else
-		layer_button_list = (LayerButtons *) malloc(s);
-	lb = layer_button_list + num_layer_buttons - 1;
-
-	for (i = 0; i < LB_NUMPICK; i++) {
-		static char namestr[] = "Label ";
-		const char *name = namestr;
-		char av[30];
-		Widget btn;
-		namestr[5] = 'A' + i;
-		switch (i) {
-		case LB_RATS:
-			name = "Rat Lines";
-			strcpy(av, "SelectLayer(Rats)");
-			break;
-		default:
-			sprintf(av, "SelectLayer(%d)", i + 1);
-			break;
-		}
-		stdarg_n = 0;
-		stdarg(XmNindicatorType, XmONE_OF_MANY);
-		btn = XmCreateToggleButton(menu, XmStrCast(name), stdarg_args, stdarg_n);
-		XtManageChild(btn);
-		XtAddCallback(btn, XmNvalueChangedCallback, (XtCallbackProc) layerpick_button_callback, (XtPointer) (size_t) i);
-		lb->w[i] = btn;
-	}
-	lb->is_pick = 1;
-	LesstifLayersChanged(0, 0, 0);
-}
-
-/*-----------------------------------------------------------------------------*/
-
 typedef struct {
 	Widget w;
 	const char *flagname;
@@ -425,7 +104,7 @@ static int del_widget_flag(int idx)
 	return 0;
 }
 
-void lesstif_update_widget_flags()
+void lesstif_update_widget_flags(const char *cookie)
 {
 	int i;
 	for (i = 0; i < n_wflags; i++) {
@@ -471,34 +150,6 @@ PCB_REGISTER_ACTIONS(lesstif_menu_action_list, lesstif_cookie)
 
 static int need_xy = 0, have_xy = 0, action_x, action_y;
 
-#if 0
-typedef struct ToggleItem {
-	struct ToggleItem *next;
-	Widget w;
-	char *group, *item;
-	XtCallbackProc callback;
-	lht_node_t *node;
-} ToggleItem;
-static ToggleItem *toggle_items = 0;
-
-static void radio_callback(Widget toggle, ToggleItem * me, XmToggleButtonCallbackStruct * cbs)
-{
-	if (!cbs->set)								/* uh uh, can't turn it off */
-		XmToggleButtonSetState(toggle, 1, 0);
-	else {
-		ToggleItem *ti;
-		for (ti = toggle_items; ti; ti = ti->next)
-			if (strcmp(me->group, ti->group) == 0) {
-				if (me->item == ti->item || strcmp(me->item, ti->item) == 0)
-					XmToggleButtonSetState(ti->w, 1, 0);
-				else
-					XmToggleButtonSetState(ti->w, 0, 0);
-			}
-		me->callback(toggle, me->node, cbs);
-	}
-}
-#endif
-
 int lesstif_button_event(Widget w, XEvent * e)
 {
 	have_xy = 1;
@@ -577,7 +228,7 @@ static void note_accelerator(const lht_node_t *node)
 	anode = pcb_hid_cfg_menu_field(node, PCB_MF_ACTION, NULL);
 	knode = pcb_hid_cfg_menu_field(node, PCB_MF_ACCELERATOR, NULL);
 	if ((anode != NULL) && (knode != NULL))
-		pcb_hid_cfg_keys_add_by_desc(&lesstif_keymap, knode, anode, NULL, 0);
+		pcb_hid_cfg_keys_add_by_desc(&lesstif_keymap, knode, anode);
 	else
 		pcb_hid_cfg_error(node, "No action specified for key accel\n");
 }
@@ -588,8 +239,6 @@ int lesstif_key_event(XKeyEvent * e)
 	KeySym sym;
 	int slen;
 	int mods = 0;
-	static pcb_hid_cfg_keyseq_t *seq[32];
-	static int seq_len = 0;
 
 	if (e->state & ShiftMask)
 		mods |= PCB_M_Shift;
@@ -604,24 +253,23 @@ int lesstif_key_event(XKeyEvent * e)
 		e->state &= ~ShiftMask;
 	slen = XLookupString(e, buf, sizeof(buf), &sym, NULL);
 
-	/* Ignore these.  */
 	switch (sym) {
-	case XK_Shift_L:
-	case XK_Shift_R:
-	case XK_Control_L:
-	case XK_Control_R:
-	case XK_Caps_Lock:
-	case XK_Shift_Lock:
-	case XK_Meta_L:
-	case XK_Meta_R:
-	case XK_Alt_L:
-	case XK_Alt_R:
-	case XK_Super_L:
-	case XK_Super_R:
-	case XK_Hyper_L:
-	case XK_Hyper_R:
-	case XK_ISO_Level3_Shift:
-		return 1;
+	/* Ignore these.  */
+		case XK_Shift_L:    case XK_Shift_R:
+		case XK_Control_L:  case XK_Control_R:
+		case XK_Caps_Lock:  case XK_Shift_Lock:
+		case XK_Meta_L:     case XK_Meta_R:
+		case XK_Alt_L:      case XK_Alt_R:
+		case XK_Super_L:    case XK_Super_R:
+		case XK_Hyper_L:    case XK_Hyper_R:
+		case XK_ISO_Level3_Shift:
+			return 1;
+
+		case XK_KP_Add: sym = '+'; break;
+		case XK_KP_Subtract: sym = '-'; break;
+		case XK_KP_Multiply: sym = '*'; break;
+		case XK_KP_Divide: sym = '/'; break;
+		case XK_KP_Enter: sym = XK_Return; break;
 	}
 
 /* TODO#3: this works only on US keyboard */
@@ -639,7 +287,7 @@ int lesstif_key_event(XKeyEvent * e)
 
 /*	printf("KEY lookup: mod=%x sym=%x/%d\n", mods, sym, slen); */
 #warning TODO#3: pass on raw and translated keys
-	slen = pcb_hid_cfg_keys_input(&lesstif_keymap, mods, sym, sym, seq, &seq_len);
+	slen = pcb_hid_cfg_keys_input(&lesstif_keymap, mods, sym, sym);
 	if (slen <= 0)
 		return 1;
 
@@ -653,12 +301,12 @@ int lesstif_key_event(XKeyEvent * e)
 
 	/* Parsing actions may not return until more user interaction
 	   happens.  */
-	pcb_hid_cfg_keys_action(seq, slen);
+	pcb_hid_cfg_keys_action(&lesstif_keymap);
 
 	return 1;
 }
 
-static void add_node_to_menu(Widget menu, lht_node_t *node, XtCallbackProc callback, int level);
+static void add_node_to_menu(Widget menu, lht_node_t *ins_after, lht_node_t *node, XtCallbackProc callback, int level);
 
 typedef struct {
 	Widget sub;     /* the open menu pane that hosts all the submenus */
@@ -716,11 +364,40 @@ static void add_res2menu_main(Widget menu, lht_node_t *node, XtCallbackProc call
 		lht_node_t *i;
 		i = pcb_hid_cfg_menu_field(node, PCB_MF_SUBMENU, NULL);
 		for(i = i->data.list.first; i != NULL; i = i->next)
-			add_node_to_menu(md->sub, i, callback, 1);
+			add_node_to_menu(md->sub, NULL, i, callback, 1);
 	}
 }
 
-static void add_res2menu_named(Widget menu, lht_node_t *node, XtCallbackProc callback, int level)
+/* when ins_after is specified, calculate where the new node needs to be
+   inserted and set the XmNpositionIndex accordingly */
+static void set_ins_after(Widget menu, lht_node_t *ins_after)
+{
+	WidgetList ch;
+	Cardinal n, nch;
+	lht_node_t *nd;
+
+	if (ins_after == NULL)
+		return;
+
+	XtVaGetValues(menu, XmNchildren, &ch, XmNnumChildren, &nch, NULL);
+	assert(ins_after->parent->type == LHT_LIST);
+	for(n = 0, nd = ins_after->parent->data.list.first; n < nch; n++,nd = nd->next) {
+		short pos;
+		
+		XtVaGetValues(ch[n], XmNpositionIndex, &pos, NULL);
+		if (nd == ins_after) {
+			stdarg(XmNpositionIndex, pos);
+			break;
+		}
+	}
+}
+
+static void lesstif_confchg_checkbox(conf_native_t *cfg, int arr_idx)
+{
+	lesstif_update_widget_flags(NULL);
+}
+
+static void add_res2menu_named(Widget menu, lht_node_t *ins_after, lht_node_t *node, XtCallbackProc callback, int level)
 {
 	const char *v;
 	lht_node_t *act, *kacc;
@@ -763,6 +440,7 @@ static void add_res2menu_named(Widget menu, lht_node_t *node, XtCallbackProc cal
 
 	v = node->name;
 	stdarg(XmNlabelString, XmStringCreatePCB(pcb_strdup(v)));
+	set_ins_after(menu, ins_after);
 
 	md = menu_data_alloc();
 	if (pcb_hid_cfg_has_submenus(node)) {
@@ -781,34 +459,13 @@ static void add_res2menu_named(Widget menu, lht_node_t *node, XtCallbackProc cal
 
 		/* assume submenu is a list, pcb_hid_cfg_has_submenus() already checked that */
 		for(i = submenu_node->data.list.first; i != NULL; i = i->next)
-			add_node_to_menu(md->sub, i, callback, level+1);
+			add_node_to_menu(md->sub, NULL, i, callback, level+1);
 	}
 	else {
 		/* doesn't have submenu */
 		const char *checked = pcb_hid_cfg_menu_field_str(node, PCB_MF_CHECKED);
 		const char *label = pcb_hid_cfg_menu_field_str(node, PCB_MF_SENSITIVE);
-#if 0
-/* Do not support radio for now: the gtk HID doesn't have it either */
-		Resource *radio = resource_subres(node->v[i].subres, "radio");
-		if (radio) {
-			ToggleItem *ti = (ToggleItem *) malloc(sizeof(ToggleItem));
-			ti->next = toggle_items;
-			ti->group = radio->v[0].value;
-			ti->item = radio->v[1].value;
-			ti->callback = callback;
-			ti->node = node->v[i].subres;
-			toggle_items = ti;
 
-			if (resource_value(node->v[i].subres, "set")) {
-				stdarg(XmNset, True);
-			}
-			stdarg(XmNindicatorType, XmONE_OF_MANY);
-			btn = XmCreateToggleButton(menu, "menubutton", args, n);
-			ti->w = btn;
-			XtAddCallback(btn, XmNvalueChangedCallback, (XtCallbackProc) radio_callback, (XtPointer) ti);
-		}
-		else
-#endif
 		act = pcb_hid_cfg_menu_field(node, PCB_MF_ACTION, NULL);
 		if (checked) {
 			if (strchr(checked, '='))
@@ -824,14 +481,41 @@ static void add_res2menu_named(Widget menu, lht_node_t *node, XtCallbackProc cal
 			md->btn = XmCreateLabel(menu, XmStrCast("menulabel"), stdarg_args, stdarg_n);
 		}
 		else {
+
 			md->btn = XmCreatePushButton(menu, XmStrCast("menubutton"), stdarg_args, stdarg_n);
 			XtAddCallback(md->btn, XmNactivateCallback, callback, (XtPointer) act);
 		}
 
 		v = pcb_hid_cfg_menu_field_str(node, PCB_MF_CHECKED);
-		if (v != NULL)
+		if (v != NULL) {
+			const char *uo;
+
 			md->wflag_idx = note_widget_flag(md->btn, XmNset, v);
 
+			/* set up the update-on callback */
+			uo = pcb_hid_cfg_menu_field_str(node, PCB_MF_UPDATE_ON);
+			if (uo == NULL)
+				uo = pcb_hid_cfg_menu_field_str(node, PCB_MF_CHECKED);
+			if (uo != NULL) {
+				static conf_hid_callbacks_t cbs;
+				static int cbs_inited = 0;
+				conf_native_t *nat = conf_get_field(uo);
+
+				if (nat != NULL) {
+					if (!cbs_inited) {
+						memset(&cbs, 0, sizeof(conf_hid_callbacks_t));
+						cbs.val_change_post = lesstif_confchg_checkbox;
+						cbs_inited = 1;
+					}
+					conf_hid_set_cb(nat, lesstif_menuconf_id, &cbs);
+				}
+				else {
+					if (*uo != '\0')
+						pcb_message(PCB_MSG_WARNING, "Checkbox menu item %s not updated on any conf change - try to use the update_on field\n", checked);
+				}
+			}
+		}
+
 		v = pcb_hid_cfg_menu_field_str(node, PCB_MF_ACTIVE);
 		if (v != NULL)
 			note_widget_flag(md->btn, XmNsensitive, v);
@@ -847,12 +531,7 @@ static void add_res2menu_text_special(Widget menu, lht_node_t *node, XtCallbackP
 	Widget btn = NULL;
 	stdarg_n = 0;
 	if (*node->data.text.value == '@') {
-		if (strcmp(node->data.text.value, "@layerview") == 0)
-			insert_layerview_buttons(menu);
-		if (strcmp(node->data.text.value, "@layerpick") == 0)
-			insert_layerpick_buttons(menu);
-		if (strcmp(node->data.text.value, "@routestyles") == 0)
-			lesstif_insert_style_buttons(menu);
+		/* ignore anchors */
 	}
 	else if ((strcmp(node->data.text.value, "-") == 0) || (strcmp(node->data.text.value, "-"))) {
 		btn = XmCreateSeparator(menu, XmStrCast("sep"), stdarg_args, stdarg_n);
@@ -860,7 +539,7 @@ static void add_res2menu_text_special(Widget menu, lht_node_t *node, XtCallbackP
 	}
 }
 
-static void add_node_to_menu(Widget in_menu, lht_node_t *node, XtCallbackProc callback, int level)
+static void add_node_to_menu(Widget in_menu, lht_node_t *ins_after, lht_node_t *node, XtCallbackProc callback, int level)
 {
 	if (level == 0) {
 		add_res2menu_main(in_menu, node, callback);
@@ -868,15 +547,15 @@ static void add_node_to_menu(Widget in_menu, lht_node_t *node, XtCallbackProc ca
 	}
 
 	switch(node->type) {
-		case LHT_HASH: add_res2menu_named(in_menu, node, callback, level); break;
+		case LHT_HASH: add_res2menu_named(in_menu, ins_after, node, callback, level); break;
 		case LHT_TEXT: add_res2menu_text_special(in_menu, node, callback); break;
 		default: /* ignore them */;
 	}
 }
 
 extern char *lesstif_pcbmenu_path;
-extern const char *lesstif_menu_default;
-
+extern const char *pcb_menu_default;
+extern pcb_hid_t lesstif_hid;
 
 Widget lesstif_menu(Widget parent, const char *name, Arg * margs, int mn)
 {
@@ -888,7 +567,8 @@ Widget lesstif_menu(Widget parent, const char *name, Arg * margs, int mn)
 	screen = DefaultScreen(display);
 	cmap = DefaultColormap(display, screen);
 
-	lesstif_cfg = pcb_hid_cfg_load("lesstif", 0, lesstif_menu_default);
+	lesstif_cfg = pcb_hid_cfg_load("lesstif", 0, pcb_menu_default);
+	lesstif_hid.hid_cfg = lesstif_cfg;
 	if (lesstif_cfg == NULL) {
 		pcb_message(PCB_MSG_ERROR, "FATAL: can't load the lesstif menu res either from file or from hardwired default.");
 		abort();
@@ -899,7 +579,7 @@ Widget lesstif_menu(Widget parent, const char *name, Arg * margs, int mn)
 		if (mr->type == LHT_LIST) {
 			lht_node_t *n;
 			for(n = mr->data.list.first; n != NULL; n = n->next)
-				add_node_to_menu(mb, n, (XtCallbackProc) callback, 0);
+				add_node_to_menu(mb, NULL, n, (XtCallbackProc) callback, 0);
 		}
 		else
 			pcb_hid_cfg_error(mr, "/main_menu should be a list");
@@ -911,22 +591,22 @@ Widget lesstif_menu(Widget parent, const char *name, Arg * margs, int mn)
 	return mb;
 }
 
-static int lesstif_create_menu_widget(void *ctx, const char *path, const char *name, int is_main, lht_node_t *parent, lht_node_t *menu_item)
+static int lesstif_create_menu_widget(void *ctx, const char *path, const char *name, int is_main, lht_node_t *parent, lht_node_t *ins_after, lht_node_t *menu_item)
 {
 	Widget w = (is_main) ? lesstif_menubar : ((menu_data_t *)parent->user_data)->sub;
 
 	if (strncmp(path, "/popups", 7) == 0)
 		return -1; /* there's no popup support in lesstif */
 
-	add_node_to_menu(w, menu_item, (XtCallbackProc) callback, is_main ? 0 : 2);
+	add_node_to_menu(w, ins_after, menu_item, (XtCallbackProc) callback, is_main ? 0 : 2);
 
 	return 0;
 }
 
 
-void lesstif_create_menu(const char *menu_path, const char *action, const char *mnemonic, const char *accel, const char *tip, const char *cookie)
+void lesstif_create_menu(const char *menu_path, const pcb_menu_prop_t *props)
 {
-	pcb_hid_cfg_create_menu(lesstif_cfg, menu_path, action, mnemonic, accel, tip, cookie, lesstif_create_menu_widget, NULL);
+	pcb_hid_cfg_create_menu(lesstif_cfg, menu_path, props, lesstif_create_menu_widget, NULL);
 }
 
 void lesstif_remove_menu(const char *menu_path)
@@ -934,7 +614,21 @@ void lesstif_remove_menu(const char *menu_path)
 	pcb_hid_cfg_remove_menu(lesstif_cfg, menu_path, del_menu, NULL);
 }
 
+void lesstif_remove_menu_node(lht_node_t *node)
+{
+	pcb_hid_cfg_remove_menu_node(lesstif_cfg, node, del_menu, NULL);
+}
+
+static const char *lesstif_menu_cookie = "hid_lesstif_menu";
+void lesstif_init_menu(void)
+{
+	if (lesstif_menuconf_id < 0)
+		lesstif_menuconf_id = conf_hid_reg(lesstif_menu_cookie, NULL);
+}
+
 void lesstif_uninit_menu(void)
 {
+	conf_hid_unreg(lesstif_menu_cookie);
 	XtDestroyWidget(lesstif_menubar);
+	lesstif_menuconf_id = -1;
 }
