@@ -31,6 +31,8 @@ typedef struct hid_gc_s {
 
 static pcb_hid_attribute_t *remote_get_export_options(int *n_ret)
 {
+	if (n_ret != NULL)
+		*n_ret = 0;
 	return 0;
 }
 
@@ -114,9 +116,9 @@ static void remote_do_exit(pcb_hid_t *hid)
 	remote_stay = 0;
 }
 
-static void remote_parse_arguments(int *argc, char ***argv)
+static int remote_parse_arguments(int *argc, char ***argv)
 {
-	pcb_hid_parse_command_line(argc, argv);
+	return pcb_hid_parse_command_line(argc, argv);
 }
 
 static void remote_invalidate_lr(pcb_coord_t l, pcb_coord_t r, pcb_coord_t t, pcb_coord_t b)
@@ -318,7 +320,7 @@ static void remote_get_coords(const char *msg, pcb_coord_t * x, pcb_coord_t * y)
 {
 }
 
-static void remote_set_crosshair(int x, int y, int action)
+static void remote_set_crosshair(pcb_coord_t x, pcb_coord_t y, int action)
 {
 }
 
@@ -424,6 +426,8 @@ void pplg_uninit_hid_remote(void)
 
 int pplg_init_hid_remote(void)
 {
+	PCB_API_CHK_VER;
+
 	memset(&remote_hid, 0, sizeof(pcb_hid_t));
 
 	pcb_hid_nogui_init(&remote_hid);
