@@ -45,6 +45,8 @@
 #include "hid.h"
 #include "plugins.h"
 
+#include "src_plugins/lib_compat_help/pstk_compat.h"
+
 static const char *dsn_cookie = "dsn importer";
 
 typedef enum {
@@ -52,6 +54,38 @@ typedef enum {
 	TYPE_SESSION
 } dsn_type_t;
 
+static pcb_layer_id_t ses_layer_by_name(pcb_board_t *pcb, const char *name)
+{
+	char *end;
+	long gid;
+	pcb_layergrp_t *grp;
+
+	gid = strtol(name, &end, 10);
+	if ((end[0] != '_') || (end[1] != '_'))
+		return -1;
+
+	grp = pcb_get_layergrp(pcb, gid);
+	if (grp == NULL)
+		return -1;
+
+	if (strcmp(end+2, grp->name) != 0) {
+		pcb_message(PCB_MSG_ERROR, "layer (group) name mismatch: group %ld should be '%s' but is '%s'\nses file not for this board?\n", gid, grp->name, end+2);
+		return -1;
+	}
+
+	if (grp->len <= 0) {
+		pcb_message(PCB_MSG_ERROR, "layer (group) '%s' has no layers\nses file not for this board?\n", name);
+		return -1;
+	}
+
+	if (!(grp->ltype & PCB_LYT_COPPER)) {
+		pcb_message(PCB_MSG_ERROR, "layer (group) type %s should a copper layer group\nses file not for this board?\n", name);
+		return -1;
+	}
+
+	return grp->lid[0];
+}
+
 static void parse_polyline(long int *nlines, pcb_coord_t clear, const gsxl_node_t *n, const char *unit, int workaround0)
 {
 	const gsxl_node_t *c;
@@ -69,7 +103,7 @@ static void parse_polyline(long int *nlines, pcb_coord_t clear, const gsxl_node_
 		return;
 	}
 
-	lid = pcb_layer_by_name(PCB->Data, slayer);
+	lid = ses_layer_by_name(PCB, slayer);
 	if (lid < 0) {
 		pcb_message(PCB_MSG_ERROR, "import_dsn: skipping polyline because layer name is invalid: %s\n", slayer);
 		return;
@@ -96,7 +130,7 @@ static void parse_polyline(long int *nlines, pcb_coord_t clear, const gsxl_node_
 		if (pn > 0) {
 			/*pcb_line_t *line = */pcb_line_new_merge(layer, lx, PCB->MaxHeight - ly,
 				x, PCB->MaxHeight - y, thick, clear, pcb_flag_make(PCB_FLAG_AUTO | PCB_FLAG_CLEARLINE));
-/*				pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_LINE, layer, line);*/
+/*				pcb_poly_clear_from_poly(PCB->Data, PCB_OBJ_LINE, layer, line);*/
 /*				pcb_printf("LINE: %$mm %$mm .. %$mm %$mm\n", lx, ly, x, y);*/
 		}
 		lx = x;
@@ -162,7 +196,10 @@ static void parse_via(pcb_coord_t clear, const gsxl_node_t *via, dsn_type_t type
 		return;
 	}
 
-	pcb_via_new(PCB->Data, x, PCB->MaxHeight - y, dia, clear, 0, drill, 0, pcb_flag_make(PCB_FLAG_AUTO));
+	{
+		pcb_pstk_t *ps = pcb_pstk_new_compat_via(PCB->Data, x, PCB->MaxHeight - y, drill, dia, clear, 0, PCB_PSTK_COMPAT_ROUND, 1);
+		PCB_FLAG_SET(PCB_FLAG_AUTO, ps);
+	}
 }
 
 static const char load_dsn_syntax[] = "LoadDsnFrom(filename)";
@@ -314,6 +351,7 @@ void pplg_uninit_import_dsn(void)
 #include "dolists.h"
 int pplg_init_import_dsn(void)
 {
+	PCB_API_CHK_VER;
 	PCB_REGISTER_ACTIONS(dsn_action_list, dsn_cookie)
 	return 0;
 }
