@@ -81,10 +81,12 @@ int pplg_check_ver_import_edif(int ver_needed) { return 0; }
 
 void pplg_uninit_import_edif(void)
 {
+	PCB_HOOK_UNREGISTER(pcb_plug_import_t, pcb_plug_import_chain, &import_edif);
 }
 
 int pplg_init_import_edif(void)
 {
+	PCB_API_CHK_VER;
 
 	/* register the IO hook */
 	import_edif.plugin_data = NULL;
