@@ -67,18 +67,21 @@ int pplg_check_ver_io_autotrax(int ver_needed) { return 0; }
 void pplg_uninit_io_autotrax(void)
 {
 	pcb_hid_remove_actions_by_cookie(autotrax_cookie);
+	PCB_HOOK_UNREGISTER(pcb_plug_io_t, pcb_plug_io_chain, &io_autotrax);
 }
 
 int pplg_init_io_autotrax(void)
 {
+	PCB_API_CHK_VER;
+
 	io_autotrax.plugin_data = NULL;
 	io_autotrax.fmt_support_prio = io_autotrax_fmt;
 	io_autotrax.test_parse = io_autotrax_test_parse;
 	io_autotrax.parse_pcb = io_autotrax_read_pcb;
-	io_autotrax.parse_element = NULL;
+	io_autotrax.parse_footprint = NULL;
 	io_autotrax.parse_font = NULL;
 	io_autotrax.write_buffer = NULL;
-	io_autotrax.write_element = NULL;
+	io_autotrax.write_footprint = NULL;
 	io_autotrax.write_pcb = io_autotrax_write_pcb;
 	io_autotrax.default_fmt = "Protel autotrax";
 	io_autotrax.description = "Protel autotrax and easytrax";
