@@ -40,14 +40,12 @@
 #include "data.h"
 #include "read.h"
 #include "layer.h"
-#include "const.h"
 #include "netlist.h"
 #include "polygon.h"
 #include "misc_util.h" /* for distance calculations */
 #include "conf_core.h"
 #include "move.h"
 #include "macro.h"
-#include "obj_all.h"
 #include "safe_fs.h"
 #include "rotate.h"
 #include "../src_plugins/boardflip/boardflip.h"
@@ -783,7 +781,7 @@ static int autotrax_create_layers(read_state_t *st)
 	pcb_layergrp_t *g;
 	pcb_layer_id_t id;
 
-	pcb_layer_group_setup_default(&st->pcb->LayerGroups);
+	pcb_layer_group_setup_default(st->pcb);
 
 	st->protel_to_stackup[7] = autotrax_reg_layer(st, "top silk", PCB_LYT_SILK | PCB_LYT_TOP);
 	st->protel_to_stackup[8] = autotrax_reg_layer(st, "bottom silk", PCB_LYT_SILK | PCB_LYT_BOTTOM);
@@ -822,7 +820,7 @@ static int autotrax_create_layers(read_state_t *st)
 
 	g = pcb_get_grp_new_intern(st->pcb, -1);
 	g->name = pcb_strdup("outline"); /* equivalent to keepout = layer 12 in autotrax */
-	g->type = PCB_LYT_OUTLINE; /* and includes cutouts */
+	g->ltype = PCB_LYT_OUTLINE; /* and includes cutouts */
 	st->protel_to_stackup[12] = autotrax_reg_layer(st, "outline", PCB_LYT_OUTLINE);
 
 	pcb_layergrp_inhibit_dec();
