@@ -56,22 +56,26 @@ void pplg_uninit_io_eagle(void)
 {
 	/* Runs once when the plugin is unloaded. TODO: free plugin-globals here. */
 	pcb_hid_remove_actions_by_cookie(eagle_cookie);
+	PCB_HOOK_UNREGISTER(pcb_plug_io_t, pcb_plug_io_chain, &io_eagle_xml);
+	PCB_HOOK_UNREGISTER(pcb_plug_io_t, pcb_plug_io_chain, &io_eagle_bin);
+	PCB_HOOK_UNREGISTER(pcb_plug_io_t, pcb_plug_io_chain, &io_eagle_dru);
 }
 
 #include "dolists.h"
 
 int pplg_init_io_eagle(void)
 {
+	PCB_API_CHK_VER;
 
 	/* register the IO hook */
 	io_eagle_xml.plugin_data = NULL;
 	io_eagle_xml.fmt_support_prio = io_eagle_fmt;
 	io_eagle_xml.test_parse = io_eagle_test_parse_xml;
 	io_eagle_xml.parse_pcb = io_eagle_read_pcb_xml;
-/*	io_eagle_xml.parse_element = NULL;
+/*	io_eagle_xml.parse_footprint = NULL;
 	io_eagle_xml.parse_font = NULL;
 	io_eagle_xml.write_buffer = io_eagle_write_buffer;
-	io_eagle_xml.write_element = io_eagle_write_element;
+	io_eagle_xml.write_footprint = io_eagle_write_element;
 	io_eagle_xml.write_pcb = io_eagle_write_pcb;*/
 	io_eagle_xml.default_fmt = "eagle";
 	io_eagle_xml.description = "eagle xml";
@@ -87,10 +91,10 @@ int pplg_init_io_eagle(void)
 	io_eagle_bin.fmt_support_prio = io_eagle_fmt;
 	io_eagle_bin.test_parse = io_eagle_test_parse_bin;
 	io_eagle_bin.parse_pcb = io_eagle_read_pcb_bin;
-/*	io_eagle_bin.parse_element = NULL;
+/*	io_eagle_bin.parse_footprint = NULL;
 	io_eagle_bin.parse_font = NULL;
 	io_eagle_bin.write_buffer = io_eagle_write_buffer;
-	io_eagle_bin.write_element = io_eagle_write_element;
+	io_eagle_bin.write_footprint = io_eagle_write_element;
 	io_eagle_bin.write_pcb = io_eagle_write_pcb;*/
 	io_eagle_bin.default_fmt = "eagle";
 	io_eagle_bin.description = "eagle bin";
@@ -106,10 +110,10 @@ int pplg_init_io_eagle(void)
 	io_eagle_dru.fmt_support_prio = io_eagle_fmt;
 	io_eagle_dru.test_parse = io_eagle_test_parse_dru;
 	io_eagle_dru.parse_pcb = io_eagle_read_pcb_dru;
-	io_eagle_dru.parse_element = NULL;
+	io_eagle_dru.parse_footprint = NULL;
 	io_eagle_dru.parse_font = NULL;
 	io_eagle_dru.write_buffer = NULL;
-	io_eagle_dru.write_element = NULL;
+	io_eagle_dru.write_footprint = NULL;
 	io_eagle_dru.write_pcb = /*io_eagle_write_pcb_dru*/ NULL;
 	io_eagle_dru.default_fmt = "eagle";
 	io_eagle_dru.description = "eagle dru";
