@@ -1478,7 +1478,7 @@ static void st_init(read_state_t *st)
 {
 	htip_init(&st->layers, longhash, longkeyeq);
 	htsp_init(&st->libs, strhash, strkeyeq);
-	pcb_layer_group_setup_default(&st->pcb->LayerGroups);
+	pcb_layer_group_setup_default(st->pcb);
 }
 
 static void st_uninit(read_state_t *st)
@@ -1506,29 +1506,9 @@ static void st_uninit(read_state_t *st)
 
 static int post_process_thermals(read_state_t *st)
 {
-	/*do something in here */
-	pcb_trace("Entering pin loop...\n");
-	PCB_PIN_ALL_LOOP(st->pcb->Data);
-	{
-		pcb_trace("Pin ID: %d \n", pin->ID);
-	}
-	PCB_ENDALL_LOOP;
-	pcb_trace("Entering pad loop...\n");
-	PCB_PAD_ALL_LOOP(st->pcb->Data);
-	{
-		pcb_trace("Pad ID: %d \n", pad->ID);
-	}
-	PCB_ENDALL_LOOP;
-	pcb_trace("Entering via loop...\n");
-	PCB_VIA_LOOP(st->pcb->Data);
-	{
-		pcb_trace("Via ID: %d \n", via->ID);
-	}
-	PCB_END_LOOP;
-	pcb_trace("Entering padstack loop...\n");
+#warning TODO: process thermals
 	PCB_PADSTACK_LOOP(st->pcb->Data);
 	{
-		pcb_trace("Padstack ID: %d \n", padstack->ID);
 	}
 	PCB_END_LOOP;
 	return 0;
