@@ -158,20 +158,24 @@ int pplg_check_ver_io_hyp(int ver_needed)
 void pplg_uninit_io_hyp(void)
 {
 	pcb_hid_remove_actions_by_cookie(hyp_cookie);
+	PCB_HOOK_UNREGISTER(pcb_plug_io_t, pcb_plug_io_chain, &io_hyp);
+
 }
 
 #include "dolists.h"
 int pplg_init_io_hyp(void)
 {
+	PCB_API_CHK_VER;
+
 	/* register the IO hook */
 	io_hyp.plugin_data = NULL;
 	io_hyp.fmt_support_prio = io_hyp_fmt;
 	io_hyp.test_parse = io_hyp_test_parse;
 	io_hyp.parse_pcb = io_hyp_read_pcb;
-/*	io_hyp.parse_element = NULL;
+/*	io_hyp.parse_footprint = NULL;
 	io_hyp.parse_font = NULL;
 	io_hyp.write_buffer = io_hyp_write_buffer;
-	io_hyp.write_element = io_hyp_write_element;*/
+	io_hyp.write_footprint = io_hyp_write_element;*/
 	io_hyp.write_pcb = io_hyp_write_pcb;
 	io_hyp.default_fmt = "hyp";
 	io_hyp.description = "hyperlynx";
