@@ -26,7 +26,6 @@
 #include "hyp_y.h"
 #include "error.h"
 #include "pcb-printf.h"
-#include "obj_all.h"
 #include "flag_str.h"
 #include "polygon.h"
 #include "board.h"
@@ -802,7 +801,7 @@ void hyp_reset_layers()
 
 	pcb_layers_reset(PCB);
 
-	pcb_layer_group_setup_default(&PCB->LayerGroups);
+	pcb_layer_group_setup_default(PCB);
 
 	/* set up dual layer board: top and bottom copper and silk */
 
