@@ -251,7 +251,7 @@ static const char *hyp_pstk_cache(hyp_wr_t *wr, pcb_pstk_proto_t *proto, int pri
 			loc = (shp->layer_mask & PCB_LYT_ANYWHERE);
 			for(l = 0; l < wr->pcb->LayerGroups.len; l++) {
 				pcb_layergrp_t *lg = &wr->pcb->LayerGroups.grp[l];
-				pcb_layer_type_t lyt = lg->type;
+				pcb_layer_type_t lyt = lg->ltype;
 				if ((lyt & PCB_LYT_COPPER) && (lyt & loc))
 					hyp_pstk_shape(wr, hyp_grp_name(wr, lg, NULL), shp);
 			}
@@ -280,7 +280,7 @@ static void write_poly(hyp_wr_t * wr, pcb_poly_t * poly)
 
 	if (poly->Clipped == NULL) {
 		pcb_layer_t *l = poly->parent.layer;
-		pcb_poly_init_clip(l->parent, l, poly);
+		pcb_poly_init_clip(l->parent.data, l, poly);
 	}
 
 	if (poly->Clipped == NULL)
@@ -396,14 +396,14 @@ static int write_lstack(hyp_wr_t * wr)
 		pcb_layergrp_t *grp = &wr->pcb->LayerGroups.grp[n];
 		const char *name = grp->name;
 
-		if (grp->type & PCB_LYT_COPPER) {
+		if (grp->ltype & PCB_LYT_COPPER) {
 			pcb_fprintf(wr->f, "  (SIGNAL T=0.003500 L=%[4])\n", hyp_grp_name(wr, grp, name));
-			if (grp->type & PCB_LYT_TOP)
+			if (grp->ltype & PCB_LYT_TOP)
 				wr->ln_top = name;
-			else if (grp->type & PCB_LYT_BOTTOM)
+			else if (grp->ltype & PCB_LYT_BOTTOM)
 				wr->ln_bottom = name;
 		}
-		else if (grp->type & PCB_LYT_SUBSTRATE) {
+		else if (grp->ltype & PCB_LYT_SUBSTRATE) {
 			char tmp[128];
 			if (name == NULL) {
 				sprintf(tmp, "dielectric layer %d", n);
@@ -491,11 +491,6 @@ static int write_nets(hyp_wr_t * wr)
 			case PCB_OBJ_PSTK:
 				write_pstk(wr, (pcb_pstk_t *) o->obj);
 
-			case PCB_OBJ_PIN:
-			case PCB_OBJ_VIA:
-			case PCB_OBJ_PAD:
-				break;
-
 			case PCB_OBJ_POLY:
 				write_poly(wr, (pcb_poly_t *) o->obj);
 				break;
@@ -504,14 +499,10 @@ static int write_nets(hyp_wr_t * wr)
 				break;									/* not yet done */
 
 			case PCB_OBJ_TEXT:
-			case PCB_OBJ_ELEMENT:
 			case PCB_OBJ_SUBC:
-			case PCB_OBJ_POINT:
 			case PCB_OBJ_NET:
 			case PCB_OBJ_LAYER:
-			case PCB_OBJ_ELINE:
-			case PCB_OBJ_EARC:
-			case PCB_OBJ_ETEXT:
+			case PCB_OBJ_LAYERGRP:
 			case PCB_OBJ_VOID:
 				break;									/* silently ignore these */
 			}
