@@ -38,7 +38,6 @@
 #include "data.h"
 #include "read.h"
 #include "layer.h"
-#include "const.h"
 #include "netlist.h"
 #include "polygon.h"
 #include "plug_footprint.h"
@@ -46,7 +45,6 @@
 #include "conf_core.h"
 #include "move.h"
 #include "macro.h"
-#include "obj_all.h"
 #include "rotate.h"
 #include "safe_fs.h"
 #include "attrib.h"
@@ -1186,7 +1184,7 @@ static int kicad_parse_layer_definitions(read_state_t *st, gsxl_node_t *subtree)
 	}
 	else { /* we are just below the top level or root of the tree, so this must be a layer definitions section */
 		pcb_layergrp_inhibit_inc();
-		pcb_layer_group_setup_default(&st->pcb->LayerGroups);
+		pcb_layer_group_setup_default(st->pcb);
 
 		/* set up the hash for implicit layers */
 		res = 0;
@@ -1446,7 +1444,7 @@ static int kicad_parse_module(read_state_t *st, gsxl_node_t *subtree)
 	char *end, *textLabel, *text;
 	char *pinName, *moduleName;
 	const char *subc_layer_str;
-	pcb_subc_t *subc;
+	pcb_subc_t *subc = NULL;
 	pcb_layer_t *subc_layer;
 	pcb_layer_type_t smd_side;
 
