@@ -35,9 +35,7 @@
 #include "data.h"
 #include "write.h"
 #include "layer.h"
-#include "const.h"
 #include "netlist.h"
-#include "obj_all.h"
 #include "obj_pstk_inlines.h"
 
 #include "../src_plugins/lib_compat_help/pstk_compat.h"
@@ -189,7 +187,7 @@ static int kicad_map_layers(wctx_t *ctx)
 
 	for(n = 0; n < ctx->pcb->LayerGroups.len; n++) {
 		pcb_layergrp_t *grp = &ctx->pcb->LayerGroups.grp[n];
-		if ((mapped[n] == 0) && !(grp->type & PCB_LYT_SUBSTRATE))
+		if ((mapped[n] == 0) && !(grp->ltype & PCB_LYT_SUBSTRATE))
 			pcb_io_incompat_save(ctx->pcb->Data, NULL, "Failed to map layer for export - layer omitted", grp->name);
 		else if (mapped[n] > 1)
 			pcb_io_incompat_save(ctx->pcb->Data, NULL, "Mapped layer for export multiple times", grp->name);
@@ -483,37 +481,9 @@ static void kicad_print_layer(wctx_t *ctx, pcb_layer_t *ly, const klayer_t *kly,
 static void kicad_print_pstks(wctx_t *ctx, pcb_data_t *Data, int ind, pcb_coord_t dx, pcb_coord_t dy)
 {
 	gdl_iterator_t it;
-	pcb_pin_t *via;
 	pcb_pstk_t *ps;
 	int is_subc = Data->parent_type == PCB_PARENT_SUBC;
 
-	/* write information about vias */
-	pinlist_foreach(&Data->Via, &it, via) {
-		fprintf(ctx->f, "%*s", ind, "");
-		if (is_subc && (via->term == NULL)) {
-			pcb_io_incompat_save(Data, (pcb_any_obj_t *)via, "can't export non-terminal via in subcircuit, omitting the object", NULL);
-			continue;
-		}
-		if (!is_subc && (via->term != NULL)) {
-			pcb_io_incompat_save(Data, (pcb_any_obj_t *)via, "can't export terminal info for a via outside of a subcircuit (omitting terminal info)", NULL);
-			continue;
-		}
-		if (is_subc) {
-			pcb_fprintf(ctx->f, "(pad %s thru_hole %s (at %.3mm %.3mm) (size %.3mm %.3mm) (drill %.3mm) (layers %s %s))\n",
-				via->term, (PCB_FLAG_TEST(PCB_FLAG_SQUARE, via) ? "rect" : "oval"),
-				via->X + dx, via->Y + dy,
-				via->Thickness, via->Thickness,
-				via->DrillingHole,
-				kicad_sexpr_layer_to_text(ctx, 0), kicad_sexpr_layer_to_text(ctx, 15)); /* skip (net 0) for now */
-		}
-		else {
-			pcb_fprintf(ctx->f, "(via (at %.3mm %.3mm) (size %.3mm) (layers %s %s))\n",
-				via->X + dx, via->Y + dy,
-				via->Thickness,
-				kicad_sexpr_layer_to_text(ctx, 0), kicad_sexpr_layer_to_text(ctx, 15)); /* skip (net 0) for now */
-		}
-	}
-
 	padstacklist_foreach(&Data->padstack, &it, ps) {
 		int klayer_from = 0, klayer_to = 15;
 		pcb_coord_t x, y, drill_dia, pad_dia, clearance, mask, x1, y1, x2, y2, thickness;
@@ -692,7 +662,9 @@ static int kicad_print_subcs(wctx_t *ctx, pcb_data_t *Data, pcb_cardinal_t ind,
 	const char *currentElementRef;
 	const char *currentElementVal;
 
-	elementlist_dedup_initializer(ededup);
+#warning TODO: revise this for subc
+/*	elementlist_dedup_initializer(ededup);*/
+
 	/* Now initialize the group with defaults */
 	unm_init(&group1);
 
@@ -780,8 +752,10 @@ static int kicad_print_subcs(wctx_t *ctx, pcb_data_t *Data, pcb_cardinal_t ind,
 	}
 	/* Release unique name utility memory */
 	unm_uninit(&group1);
+
+#warning TODO: revise this for subc
 	/* free the state used for deduplication */
-	elementlist_dedup_free(ededup);
+/*	elementlist_dedup_free(ededup);*/
 
 	return 0;
 }
@@ -799,7 +773,7 @@ int io_kicad_write_element(pcb_plug_io_t *ctx, FILE *FP, pcb_data_t *Data)
 {
 	wctx_t wctx;
 
-	if (elementlist_length(&Data->Element) > 1) {
+	if (pcb_subclist_length(&Data->subc) > 1) {
 		pcb_message(PCB_MSG_ERROR, "Can't save multiple modules (footprints) in a single s-experssion mod file\n");
 		return -1;
 	}
