@@ -34,9 +34,8 @@
 #include "data.h"
 #include "write.h"
 #include "layer.h"
-#include "const.h"
 #include "netlist.h"
-#include "obj_all.h"
+#include "macro.h"
 #include "obj_pstk_inlines.h"
 
 #include "../io_kicad/uniq_name.h"
@@ -64,7 +63,7 @@ static int io_kicad_legacy_write_subc_index(FILE *FP, pcb_data_t *Data)
 
 	unm_init(&group1);
 
-	subclist_foreach(&Data->Element, &eit, subc) {
+	subclist_foreach(&Data->subc, &eit, subc) {
 		if (pcb_data_is_empty(subc->data))
 			continue;
 
@@ -651,18 +650,19 @@ static int write_kicad_legacy_layout_polygons(FILE *FP, pcb_cardinal_t number, p
 
 int io_kicad_legacy_write_buffer(pcb_plug_io_t *ctx, FILE *FP, pcb_buffer_t *buff, pcb_bool elem_only)
 {
-	if (elementlist_length(&buff->Data->Element) == 0) {
-		pcb_message(PCB_MSG_ERROR, "Buffer has no elements!\n");
+	if (pcb_subclist_length(&buff->Data->subc) == 0) {
+		pcb_message(PCB_MSG_ERROR, "Buffer has no subcircuits!\n");
 		return -1;
 	}
 
+#warning TODO: don't hardwire dates
 	fputs("PCBNEW-LibModule-V1	jan 01 jan 2016 00:00:01 CET\n", FP);
 	fputs("Units mm\n", FP);
 	fputs("$INDEX\n", FP);
 	io_kicad_legacy_write_subc_index(FP, buff->Data);
 	fputs("$EndINDEX\n", FP);
 
-	pcb_write_element_data(FP, buff->Data, "kicadl");
+	pcb_write_footprint_data(FP, buff->Data, "kicadl");
 
 	return 0;
 }
