@@ -33,6 +33,7 @@
 #include <libminuid/libminuid.h>
 #include "config.h"
 #include "board.h"
+#include "conf_core.h"
 #include "data.h"
 #include "plugins.h"
 #include "plug_io.h"
@@ -58,6 +59,12 @@
 #define CFMT "%.08$$mH"
 /*#define CFMT "%$$mn"*/
 
+/* Note: this works because of using str_flag compat_types */
+#define PCB_OBJ_VIA PCB_OBJ_PSTK
+#define PCB_OBJ_PIN PCB_OBJ_PSTK
+#define PCB_OBJ_PAD PCB_OBJ_PSTK
+#define PCB_OBJ_ELEMENT PCB_OBJ_SUBC
+
 static int io_lihata_full_tree = 0;
 static int wrver;
 
@@ -138,24 +145,29 @@ static lht_node_t *build_board_meta(pcb_board_t *pcb)
 	lht_dom_hash_put(meta, grp);
 	lht_dom_hash_put(grp, build_textf("x", CFMT, pcb->MaxWidth));
 	lht_dom_hash_put(grp, build_textf("y", CFMT, pcb->MaxHeight));
-	lht_dom_hash_put(grp, build_textf("isle_area_nm2", "%f", pcb->IsleArea));
+	if (wrver < 5) {
+		lht_dom_hash_put(grp, build_textf("isle_area_nm2", "%f", conf_core.design.poly_isle_area));
+	}
 	lht_dom_hash_put(grp, build_textf("thermal_scale", "%f", pcb->ThermScale));
 
+	if (wrver < 5) {
+		grp = lht_dom_node_alloc(LHT_HASH, "drc");
+		lht_dom_hash_put(meta, grp);
+		lht_dom_hash_put(grp, build_textf("bloat",     CFMT, conf_core.design.bloat));
+		lht_dom_hash_put(grp, build_textf("shrink",    CFMT, conf_core.design.shrink));
+		lht_dom_hash_put(grp, build_textf("min_width", CFMT, conf_core.design.min_wid));
+		lht_dom_hash_put(grp, build_textf("min_silk",  CFMT, conf_core.design.min_slk));
+		lht_dom_hash_put(grp, build_textf("min_drill", CFMT, conf_core.design.min_drill));
+		lht_dom_hash_put(grp, build_textf("min_ring",  CFMT, conf_core.design.min_ring));
+	}
 
-	grp = lht_dom_node_alloc(LHT_HASH, "drc");
-	lht_dom_hash_put(meta, grp);
-	lht_dom_hash_put(grp, build_textf("bloat",     CFMT, pcb->Bloat));
-	lht_dom_hash_put(grp, build_textf("shrink",    CFMT, pcb->Shrink));
-	lht_dom_hash_put(grp, build_textf("min_width", CFMT, pcb->minWid));
-	lht_dom_hash_put(grp, build_textf("min_silk",  CFMT, pcb->minSlk));
-	lht_dom_hash_put(grp, build_textf("min_drill", CFMT, pcb->minDrill));
-	lht_dom_hash_put(grp, build_textf("min_ring",  CFMT, pcb->minRing));
-
-	grp = lht_dom_node_alloc(LHT_HASH, "cursor");
-	lht_dom_hash_put(meta, grp);
-	lht_dom_hash_put(grp, build_textf("x", CFMT, pcb->CursorX));
-	lht_dom_hash_put(grp, build_textf("y", CFMT, pcb->CursorY));
-	lht_dom_hash_put(grp, build_textf("zoom", "%.6f", pcb->Zoom));
+	if (wrver < 5) {
+		grp = lht_dom_node_alloc(LHT_HASH, "cursor");
+		lht_dom_hash_put(meta, grp);
+		lht_dom_hash_put(grp, build_textf("x", CFMT, pcb->CursorX));
+		lht_dom_hash_put(grp, build_textf("y", CFMT, pcb->CursorY));
+		lht_dom_hash_put(grp, build_textf("zoom", "%.6f", pcb->Zoom));
+	}
 
 	return meta;
 }
@@ -191,7 +203,8 @@ static lht_node_t *build_flags(pcb_flag_t *f, int object_type, int intconn)
 
 	/* create normal flag nodes */
 	for (n = 0; n < pcb_object_flagbits_len; n++) {
-		if ((pcb_object_flagbits[n].object_types & object_type) && (PCB_FLAG_TEST(pcb_object_flagbits[n].mask, &fh))) {
+		long my_types = pcb_object_flagbits[n].object_types | ((wrver <= 4) ? pcb_object_flagbits[n].compat_types : 0);
+		if ((my_types & object_type) && (PCB_FLAG_TEST(pcb_object_flagbits[n].mask, &fh))) {
 			lht_dom_hash_put(hsh, build_text(pcb_object_flagbits[n].name, "1"));
 			PCB_FLAG_CLEAR(pcb_object_flagbits[n].mask, &fh);
 			added++;
@@ -242,6 +255,52 @@ static lht_node_t *build_flags(pcb_flag_t *f, int object_type, int intconn)
 	return hsh;
 }
 
+static void obj_attr_flag_warn(pcb_any_obj_t *obj)
+{
+	int warned = 0;
+
+	if (wrver < 5) {
+		if (pcb_attribute_get(&obj->Attributes, "intnoconn") != NULL) {
+			warned = 1;
+			pcb_message(PCB_MSG_WARNING, "pcb-rnd versions only reading file older than lihata v5 may ignore the intnoconn flag\n");
+		}
+	}
+	if (wrver < 3) {
+		if (pcb_attribute_get(&obj->Attributes, "intconn") != NULL) {
+			warned = 1;
+			pcb_message(PCB_MSG_WARNING, "pcb-rnd versions only reading file older than lihata v3 may ignore the intconn flag\n");
+		}
+	}
+
+	if (warned)
+		pcb_message(PCB_MSG_WARNING, "^^^ in %s #%ld\n", pcb_obj_type_name(obj->type), obj->ID);
+}
+
+/* Write the thermal list of heavy terminals; put the resulting "thermal"
+   node in ha:dst */
+void build_thermal_heavy(lht_node_t *dst, pcb_any_obj_t *o)
+{
+	lht_node_t *th;
+	pcb_thermal_t tin = o->thermal;
+	const char *bit;
+
+	assert(dst->type == LHT_HASH);
+
+	if (o->thermal == 0) {
+		lht_dom_hash_put(dst, dummy_node("thermal"));
+		return;
+	}
+
+	if (wrver < 4)
+		pcb_io_incompat_save(NULL, o, "lihata boards before version v4 did not support heavy terminal vias\n", "Either save in lihata v4+ or do not use heavy terminal thermals");
+
+	th = lht_dom_node_alloc(LHT_LIST, "thermal");
+	lht_dom_hash_put(dst, th);
+	while((bit = pcb_thermal_bits2str(&tin)) != NULL)
+		lht_dom_list_append(th, build_textf(NULL, bit));
+}
+
+
 static lht_node_t *build_line(pcb_line_t *line, int local_id, pcb_coord_t dx, pcb_coord_t dy, int simple)
 {
 	char buff[128];
@@ -251,8 +310,9 @@ static lht_node_t *build_line(pcb_line_t *line, int local_id, pcb_coord_t dx, pc
 	obj = lht_dom_node_alloc(LHT_HASH, buff);
 
 	if (!simple) {
+		obj_attr_flag_warn((pcb_any_obj_t *)line);
 		lht_dom_hash_put(obj, build_attributes(&line->Attributes));
-		lht_dom_hash_put(obj, build_flags(&line->Flags, PCB_TYPE_LINE, line->intconn));
+		lht_dom_hash_put(obj, build_flags(&line->Flags, PCB_OBJ_LINE, line->intconn));
 		lht_dom_hash_put(obj, build_textf("clearance", CFMT, line->Clearance));
 	}
 
@@ -262,6 +322,8 @@ static lht_node_t *build_line(pcb_line_t *line, int local_id, pcb_coord_t dx, pc
 	lht_dom_hash_put(obj, build_textf("x2", CFMT, line->Point2.X+dx));
 	lht_dom_hash_put(obj, build_textf("y2", CFMT, line->Point2.Y+dy));
 
+	build_thermal_heavy(obj, (pcb_any_obj_t*)line);
+
 	return obj;
 }
 
@@ -308,8 +370,10 @@ static lht_node_t *build_rat(pcb_rat_t *rat)
 	sprintf(buff, "rat.%ld", rat->ID);
 	obj = lht_dom_node_alloc(LHT_HASH, buff);
 
+	obj_attr_flag_warn((pcb_any_obj_t *)rat);
+
 	lht_dom_hash_put(obj, build_attributes(&rat->Attributes));
-	lht_dom_hash_put(obj, build_flags(&rat->Flags, PCB_TYPE_LINE, rat->intconn));
+	lht_dom_hash_put(obj, build_flags(&rat->Flags, PCB_OBJ_LINE, rat->intconn));
 	lht_dom_hash_put(obj, build_textf("x1", CFMT, rat->Point1.X));
 	lht_dom_hash_put(obj, build_textf("y1", CFMT, rat->Point1.Y));
 	lht_dom_hash_put(obj, build_textf("x2", CFMT, rat->Point2.X));
@@ -328,8 +392,10 @@ static lht_node_t *build_arc(pcb_arc_t *arc, pcb_coord_t dx, pcb_coord_t dy)
 	sprintf(buff, "arc.%ld", arc->ID);
 	obj = lht_dom_node_alloc(LHT_HASH, buff);
 
+	obj_attr_flag_warn((pcb_any_obj_t *)arc);
+
 	lht_dom_hash_put(obj, build_attributes(&arc->Attributes));
-	lht_dom_hash_put(obj, build_flags(&arc->Flags, PCB_TYPE_ARC, arc->intconn));
+	lht_dom_hash_put(obj, build_flags(&arc->Flags, PCB_OBJ_ARC, arc->intconn));
 	lht_dom_hash_put(obj, build_textf("thickness", CFMT, arc->Thickness));
 	lht_dom_hash_put(obj, build_textf("clearance", CFMT, arc->Clearance));
 	lht_dom_hash_put(obj, build_textf("x", CFMT, arc->X+dx));
@@ -339,27 +405,8 @@ static lht_node_t *build_arc(pcb_arc_t *arc, pcb_coord_t dx, pcb_coord_t dy)
 	lht_dom_hash_put(obj, build_textf("astart", "%ma", arc->StartAngle));
 	lht_dom_hash_put(obj, build_textf("adelta", "%ma", arc->Delta));
 
-	return obj;
-}
+	build_thermal_heavy(obj, (pcb_any_obj_t*)arc);
 
-static lht_node_t *build_pin(pcb_pin_t *pin, int is_via, pcb_coord_t dx, pcb_coord_t dy)
-{
-	char buff[128];
-	lht_node_t *obj;
-
-	sprintf(buff, "%s.%ld", is_via ? "via" : "pin", pin->ID);
-	obj = lht_dom_node_alloc(LHT_HASH, buff);
-
-	lht_dom_hash_put(obj, build_attributes(&pin->Attributes));
-	lht_dom_hash_put(obj, build_flags(&pin->Flags, PCB_TYPE_VIA, pin->intconn));
-	lht_dom_hash_put(obj, build_textf("thickness", CFMT, pin->Thickness));
-	lht_dom_hash_put(obj, build_textf("clearance", CFMT, pin->Clearance));
-	lht_dom_hash_put(obj, build_textf("mask", CFMT, pin->Mask));
-	lht_dom_hash_put(obj, build_textf("hole", CFMT, pin->DrillingHole));
-	lht_dom_hash_put(obj, build_textf("x", CFMT, pin->X+dx));
-	lht_dom_hash_put(obj, build_textf("y", CFMT, pin->Y+dy));
-	lht_dom_hash_put(obj, build_text("name", pin->Name));
-	lht_dom_hash_put(obj, build_text("number", pin->Number));
 	return obj;
 }
 
@@ -385,8 +432,10 @@ static lht_node_t *build_pstk_pinvia(pcb_data_t *data, pcb_pstk_t *ps, pcb_bool
 
 	flg = pcb_pstk_compat_pinvia_flag(ps, cshape);
 
+	obj_attr_flag_warn((pcb_any_obj_t *)ps);
+
 	lht_dom_hash_put(obj, build_attributes(&ps->Attributes));
-	lht_dom_hash_put(obj, build_flags(&flg, PCB_TYPE_VIA, ps->intconn));
+	lht_dom_hash_put(obj, build_flags(&flg, PCB_OBJ_VIA, ps->intconn));
 	lht_dom_hash_put(obj, build_textf("thickness", CFMT, pad_dia));
 	lht_dom_hash_put(obj, build_textf("clearance", CFMT, clearance));
 	lht_dom_hash_put(obj, build_textf("mask", CFMT, mask));
@@ -422,8 +471,10 @@ static lht_node_t *build_pstk_pad(pcb_data_t *data, pcb_pstk_t *ps, pcb_coord_t
 	if (nopaste)
 		flg.f |= PCB_FLAG_NOPASTE;
 
+	obj_attr_flag_warn((pcb_any_obj_t *)ps);
+
 	lht_dom_hash_put(obj, build_attributes(&ps->Attributes));
-	lht_dom_hash_put(obj, build_flags(&flg, PCB_TYPE_PAD, ps->intconn));
+	lht_dom_hash_put(obj, build_flags(&flg, PCB_OBJ_PAD, ps->intconn));
 	lht_dom_hash_put(obj, build_textf("thickness", CFMT, thickness));
 	lht_dom_hash_put(obj, build_textf("clearance", CFMT, clearance));
 	lht_dom_hash_put(obj, build_textf("mask", CFMT, mask));
@@ -436,28 +487,6 @@ static lht_node_t *build_pstk_pad(pcb_data_t *data, pcb_pstk_t *ps, pcb_coord_t
 	return obj;
 }
 
-static lht_node_t *build_pad(pcb_pad_t *pad, pcb_coord_t dx, pcb_coord_t dy)
-{
-	char buff[128];
-	lht_node_t *obj;
-
-	sprintf(buff, "pad.%ld", pad->ID);
-	obj = lht_dom_node_alloc(LHT_HASH, buff);
-
-	lht_dom_hash_put(obj, build_attributes(&pad->Attributes));
-	lht_dom_hash_put(obj, build_flags(&pad->Flags, PCB_TYPE_PAD, pad->intconn));
-	lht_dom_hash_put(obj, build_textf("thickness", CFMT, pad->Thickness));
-	lht_dom_hash_put(obj, build_textf("clearance", CFMT, pad->Clearance));
-	lht_dom_hash_put(obj, build_textf("mask", CFMT, pad->Mask));
-	lht_dom_hash_put(obj, build_textf("x1", CFMT, pad->Point1.X+dx));
-	lht_dom_hash_put(obj, build_textf("y1", CFMT, pad->Point1.Y+dy));
-	lht_dom_hash_put(obj, build_textf("x2", CFMT, pad->Point2.X+dx));
-	lht_dom_hash_put(obj, build_textf("y2", CFMT, pad->Point2.Y+dy));
-	lht_dom_hash_put(obj, build_text("name", pad->Name));
-	lht_dom_hash_put(obj, build_text("number", pad->Number));
-	return obj;
-}
-
 static lht_node_t *build_polygon(pcb_poly_t *poly)
 {
 	char buff[128];
@@ -467,11 +496,15 @@ static lht_node_t *build_polygon(pcb_poly_t *poly)
 	sprintf(buff, "polygon.%ld", poly->ID);
 	obj = lht_dom_node_alloc(LHT_HASH, buff);
 
+	obj_attr_flag_warn((pcb_any_obj_t *)poly);
+
 	lht_dom_hash_put(obj, build_attributes(&poly->Attributes));
-	lht_dom_hash_put(obj, build_flags(&poly->Flags, PCB_TYPE_POLY, poly->intconn));
+	lht_dom_hash_put(obj, build_flags(&poly->Flags, PCB_OBJ_POLY, poly->intconn));
 
 	if ((wrver >= 3) && (poly->Clearance > 0))
 		lht_dom_hash_put(obj, build_textf("clearance", CFMT, poly->Clearance));
+	else
+		lht_dom_hash_put(obj, dummy_node("clearance"));
 
 	geo = lht_dom_node_alloc(LHT_LIST, "geometry");
 	lht_dom_hash_put(obj, geo);
@@ -498,6 +531,8 @@ static lht_node_t *build_polygon(pcb_poly_t *poly)
 		tbl->data.table.r[row][1] = build_textf(NULL, CFMT, poly->Points[n].Y);
 	}
 
+	build_thermal_heavy(obj, (pcb_any_obj_t*)poly);
+
 	return obj;
 }
 
@@ -509,8 +544,10 @@ static lht_node_t *build_pcb_text(const char *role, pcb_text_t *text)
 	sprintf(buff, "text.%ld", text->ID);
 	obj = lht_dom_node_alloc(LHT_HASH, buff);
 
+	obj_attr_flag_warn((pcb_any_obj_t *)text);
+
 	lht_dom_hash_put(obj, build_attributes(&text->Attributes));
-	lht_dom_hash_put(obj, build_flags(&text->Flags, PCB_TYPE_TEXT, text->intconn));
+	lht_dom_hash_put(obj, build_flags(&text->Flags, PCB_OBJ_TEXT, text->intconn));
 	lht_dom_hash_put(obj, build_text("string", text->TextString));
 	lht_dom_hash_put(obj, build_textf("fid", "%ld", text->fid));
 	lht_dom_hash_put(obj, build_textf("scale", "%d", text->Scale));
@@ -524,48 +561,6 @@ static lht_node_t *build_pcb_text(const char *role, pcb_text_t *text)
 	return obj;
 }
 
-static lht_node_t *build_element(pcb_element_t *elem)
-{
-	char buff[128];
-	pcb_line_t *li;
-	pcb_arc_t *ar;
-	pcb_pin_t *pi;
-	pcb_pad_t *pa;
-	lht_node_t *obj, *lst;
-
-	sprintf(buff, "element.%ld", elem->ID);
-	obj = lht_dom_node_alloc(LHT_HASH, buff);
-
-	lht_dom_hash_put(obj, build_attributes(&elem->Attributes));
-	lht_dom_hash_put(obj, build_flags(&elem->Flags, PCB_TYPE_ELEMENT, 0));
-
-
-	/* build drawing primitives */
-	lst = lht_dom_node_alloc(LHT_LIST, "objects");
-	lht_dom_hash_put(obj, lst);
-
-	lht_dom_list_append(lst, build_pcb_text("desc", &elem->Name[PCB_ELEMNAME_IDX_DESCRIPTION]));
-	lht_dom_list_append(lst, build_pcb_text("name", &elem->Name[PCB_ELEMNAME_IDX_REFDES]));
-	lht_dom_list_append(lst, build_pcb_text("value", &elem->Name[PCB_ELEMNAME_IDX_VALUE]));
-	lht_dom_hash_put(obj, build_textf("x", CFMT, elem->MarkX));
-	lht_dom_hash_put(obj, build_textf("y", CFMT, elem->MarkY));
-
-
-	for(li = linelist_first(&elem->Line); li != NULL; li = linelist_next(li))
-		lht_dom_list_append(lst, build_line(li, -1, -elem->MarkX, -elem->MarkY, 0));
-
-	for(ar = arclist_first(&elem->Arc); ar != NULL; ar = arclist_next(ar))
-		lht_dom_list_append(lst, build_arc(ar, -elem->MarkX, -elem->MarkY));
-
-	for(pi = pinlist_first(&elem->Pin); pi != NULL; pi = pinlist_next(pi))
-		lht_dom_list_append(lst, build_pin(pi, 0, -elem->MarkX, -elem->MarkY));
-
-	for(pa = padlist_first(&elem->Pad); pa != NULL; pa = padlist_next(pa))
-		lht_dom_list_append(lst, build_pad(pa, -elem->MarkX, -elem->MarkY));
-
-	return obj;
-}
-
 static lht_node_t *build_subc_element(pcb_subc_t *subc)
 {
 	char buff[128];
@@ -584,8 +579,10 @@ static lht_node_t *build_subc_element(pcb_subc_t *subc)
 	sprintf(buff, "element.%ld", subc->ID);
 	obj = lht_dom_node_alloc(LHT_HASH, buff);
 
+	obj_attr_flag_warn((pcb_any_obj_t *)subc);
+
 	lht_dom_hash_put(obj, build_attributes(&subc->Attributes));
-	lht_dom_hash_put(obj, build_flags(&subc->Flags, PCB_TYPE_ELEMENT, 0));
+	lht_dom_hash_put(obj, build_flags(&subc->Flags, PCB_OBJ_ELEMENT, 0));
 
 	/* build drawing primitives */
 	lst = lht_dom_node_alloc(LHT_LIST, "objects");
@@ -674,8 +671,10 @@ static lht_node_t *build_subc(pcb_subc_t *sc)
 	sprintf(buff, "subc.%ld", sc->ID);
 	obj = lht_dom_node_alloc(LHT_HASH, buff);
 
+	obj_attr_flag_warn((pcb_any_obj_t *)sc);
+
 	lht_dom_hash_put(obj, build_attributes(&sc->Attributes));
-	lht_dom_hash_put(obj, build_flags(&sc->Flags, PCB_TYPE_SUBC, 0));
+	lht_dom_hash_put(obj, build_flags(&sc->Flags, PCB_OBJ_SUBC, 0));
 	lht_dom_hash_put(obj, build_data(sc->data));
 	lht_dom_hash_put(obj, build_minuid("uid", sc->uid));
 
@@ -718,6 +717,14 @@ static lht_node_t *build_pstk_protos(pcb_vtpadstack_proto_t *pp)
 		lht_dom_hash_put(nproto, build_textf("htop", "%d", proto->htop));
 		lht_dom_hash_put(nproto, build_textf("hbottom", "%d", proto->hbottom));
 		lht_dom_hash_put(nproto, build_textf("hplated", "%d", proto->hplated));
+		if (proto->name != NULL) {
+			if (wrver >= 5)
+				lht_dom_hash_put(nproto, build_text("name", proto->name));
+			else
+				pcb_io_incompat_save(NULL, NULL, "versions below lihata board v5 do not support padstack prototype names\n", "Be aware that padstack proto names are lost in save or use lihata board v5 or higher");
+		}
+		else
+			lht_dom_hash_put(nproto, dummy_node("name"));
 
 		/* save each shape */
 		lht_dom_hash_put(nproto, nshapelst = lht_dom_node_alloc(LHT_LIST, "shape"));
@@ -780,7 +787,7 @@ static lht_node_t *build_pstk(pcb_pstk_t *ps)
 	obj = lht_dom_node_alloc(LHT_HASH, buff);
 
 	lht_dom_hash_put(obj, build_attributes(&ps->Attributes));
-	lht_dom_hash_put(obj, build_flags(&ps->Flags, PCB_TYPE_PSTK, ps->intconn));
+	lht_dom_hash_put(obj, build_flags(&ps->Flags, PCB_OBJ_PSTK, ps->intconn));
 
 	lht_dom_hash_put(obj, build_textf("proto", "%ld", (long int)ps->proto));
 	lht_dom_hash_put(obj, build_textf("x", CFMT, ps->x));
@@ -822,13 +829,18 @@ static lht_node_t *build_layer_stack(pcb_board_t *pcb)
 		sprintf(tmp, "%d", n);
 		lht_dom_list_append(grps, grp = lht_dom_node_alloc(LHT_HASH, tmp));
 
+		if (wrver >= 5)
+			lht_dom_hash_put(grp, build_attributes(&g->Attributes));
+		else if (g->Attributes.Number > 0)
+			pcb_io_incompat_save(pcb->Data, (pcb_any_obj_t *)g, "Can not save layer group attributes in lihata formats below version 5.", "Either save in lihata v5 - or accept that attributes are not saved");
+
 		lht_dom_hash_put(grp, build_text("name", g->name));
 		lht_dom_hash_put(grp, layers = lht_dom_node_alloc(LHT_LIST, "layers"));
 		for(i = 0; i < g->len; i++)
 			lht_dom_list_append(layers, build_textf("", "%ld", g->lid[i]));
 
 		lht_dom_hash_put(grp, flags = lht_dom_node_alloc(LHT_HASH, "type"));
-		pcb_layer_type_map(g->type, flags, build_layer_stack_flag);
+		pcb_layer_type_map(g->ltype, flags, build_layer_stack_flag);
 	}
 
 	return lstk;
@@ -849,6 +861,8 @@ static lht_node_t *build_data_layer(pcb_data_t *data, pcb_layer_t *layer, pcb_la
 	if (!layer->is_bound) {
 		lht_dom_hash_put(obj, build_text("visible", layer->meta.real.vis ? "1" : "0"));
 		lht_dom_hash_put(obj, build_textf("group", "%ld", layer_group));
+		if (wrver >= 5)
+			lht_dom_hash_put(obj, build_text("color", layer->meta.real.color));
 	}
 	else {
 		if (wrver >= 3) {
@@ -865,7 +879,7 @@ static lht_node_t *build_data_layer(pcb_data_t *data, pcb_layer_t *layer, pcb_la
 	}
 
 	if (!layer->is_bound)
-		lht_dom_hash_put(obj, build_attributes(&layer->meta.real.Attributes));
+		lht_dom_hash_put(obj, build_attributes(&layer->Attributes));
 
 	if (wrver >= 2) {
 		lht_dom_hash_put(obj, build_textf("lid", "%ld", lid));
@@ -979,8 +993,6 @@ static lht_node_t *build_data(pcb_data_t *data)
 {
 	lht_node_t *grp, *ndt;
 	pcb_pstk_t *ps;
-	pcb_pin_t *pi;
-	pcb_element_t *el;
 	pcb_subc_t *sc;
 	gdl_iterator_t it;
 	pcb_rat_t *line;
@@ -1007,12 +1019,6 @@ static lht_node_t *build_data(pcb_data_t *data)
 		}
 	}
 
-	for(pi = pinlist_first(&data->Via); pi != NULL; pi = pinlist_next(pi))
-		lht_dom_list_append(grp, build_pin(pi, 1, 0, 0));
-
-	for(el = elementlist_first(&data->Element); el != NULL; el = elementlist_next(el))
-		lht_dom_list_append(grp, build_element(el));
-
 	for(sc = pcb_subclist_first(&data->subc); sc != NULL; sc = pcb_subclist_next(sc))
 		lht_dom_list_append_safe(grp, build_subc(sc));
 
@@ -1128,6 +1134,14 @@ static lht_node_t *build_styles(vtroutestyle_t *styles)
 		lht_dom_hash_put(sn, build_textf("diameter", CFMT, s->Diameter));
 		lht_dom_hash_put(sn, build_textf("hole", CFMT, s->Hole));
 		lht_dom_hash_put(sn, build_textf("clearance", CFMT, s->Clearance));
+		if (wrver >= 5) {
+			if (s->via_proto_set)
+				lht_dom_hash_put(sn, build_textf("via_proto", "%ld", (long int)s->via_proto));
+			else
+			lht_dom_hash_put(sn, dummy_text_node("via_proto"));
+		}
+		else
+			pcb_io_incompat_save(NULL, NULL, "lihata boards before version v5 did not support padstack prototype in route style\n", "Either save in lihata v5+ or be aware of losing this information");
 		lht_dom_hash_put(sn, build_attributes(&s->attr));
 	}
 	return stl;
@@ -1152,6 +1166,12 @@ static lht_node_t *build_netlist(pcb_lib_t *netlist, const char *name, int *none
 
 		/* create the net hash */
 		nnet = lht_dom_node_alloc(LHT_HASH, netname);
+
+		if (wrver >= 5)
+			lht_dom_hash_put(nnet, build_attributes(&menu->Attributes));
+		else if (menu->Attributes.Number > 0)
+			pcb_io_incompat_save(NULL, (pcb_any_obj_t *)menu, "Can not save netlist attributes in lihata formats below version 5.", "Either save in lihata v5 - or accept that attributes are not saved");
+
 		pl = lht_dom_node_alloc(LHT_LIST, "conn");
 		lht_dom_hash_put(nnet, pl);
 		if ((style != NULL) && (*style == '\0')) style = NULL;
@@ -1260,15 +1280,15 @@ static lht_node_t *build_netlists(pcb_board_t *pcb, pcb_lib_t *netlists, pcb_rat
 	return nls;
 }
 
-extern lht_doc_t *conf_root[CFR_max_alloc];
+extern lht_doc_t *conf_main_root[CFR_max_alloc];
 static lht_node_t *build_conf()
 {
 	const char **s, *del_paths[] = { "editor/mode", NULL };
 	lht_node_t *root, *n;
-	if ((conf_root[CFR_DESIGN] == NULL) || (conf_root[CFR_DESIGN]->root == NULL) || (conf_root[CFR_DESIGN]->root->type != LHT_LIST))
+	if ((conf_main_root[CFR_DESIGN] == NULL) || (conf_main_root[CFR_DESIGN]->root == NULL) || (conf_main_root[CFR_DESIGN]->root->type != LHT_LIST))
 		return lht_dom_node_alloc(LHT_LIST, "pcb-rnd-conf-v1");
 
-	root = conf_root[CFR_DESIGN]->root;
+	root = conf_main_root[CFR_DESIGN]->root;
 	for(n = root->data.list.first; n != NULL; n = n->next) {
 		for(s = del_paths; *s != NULL; s++) {
 			lht_node_t *sub = lht_tree_path_(n->doc, n, *s, 0, 0, NULL);
@@ -1413,6 +1433,7 @@ static int io_lihata_write_pcb(pcb_plug_io_t *ctx, FILE * FP, const char *old_fi
 			char *fe_name = pcb_concat(old_filename, ".mem.lht", NULL);
 			fe = pcb_fopen(fe_name, "w");
 			if (fe != NULL) {
+				clean_invalid(brd->root); /* remove invalid nodes placed for persistency */
 				res = lht_dom_export(brd->root, fe, "");
 				fclose(fe);
 			}
@@ -1448,6 +1469,11 @@ int io_lihata_write_pcb_v4(pcb_plug_io_t *ctx, FILE * FP, const char *old_filena
 	return io_lihata_write_pcb(ctx, FP, old_filename, new_filename, emergency, 4);
 }
 
+int io_lihata_write_pcb_v5(pcb_plug_io_t *ctx, FILE * FP, const char *old_filename, const char *new_filename, pcb_bool emergency)
+{
+	return io_lihata_write_pcb(ctx, FP, old_filename, new_filename, emergency, 5);
+}
+
 int io_lihata_write_font(pcb_plug_io_t *ctx, pcb_font_t *font, const char *Filename)
 {
 	FILE *f;
@@ -1512,7 +1538,7 @@ static int io_lihata_dump_1st_subc(pcb_plug_io_t *ctx, FILE *f, pcb_data_t *data
 
 	if (wrver == 3)
 		doc->root = lht_dom_node_alloc(LHT_LIST, "pcb-rnd-subcircuit-v3");
-	else if (wrver == 4)
+	else if (wrver >= 4)
 		doc->root = lht_dom_node_alloc(LHT_LIST, "pcb-rnd-subcircuit-v4");
 	else {
 		pcb_message(PCB_MSG_ERROR, "Invalid lihata subc version to write: %d\n", wrver);
@@ -1521,6 +1547,7 @@ static int io_lihata_dump_1st_subc(pcb_plug_io_t *ctx, FILE *f, pcb_data_t *data
 
 	lht_dom_list_append(doc->root, build_subc(sc));
 
+	clean_invalid(doc->root); /* remove invalid nodes placed for persistency */
 	res = lht_dom_export(doc->root, f, "");
 
 	lht_dom_uninit(doc);
