@@ -92,7 +92,7 @@ static int parse_xy(hkp_ctx_t *ctx, char *s, pcb_coord_t *x, pcb_coord_t *y)
 	return !(suc1 && suc2);
 }
 
-static void parse_pstk(hkp_ctx_t *ctx, pcb_element_t *elem, const char *ps, pcb_coord_t px, pcb_coord_t py, char *name)
+static void parse_pstk(hkp_ctx_t *ctx, pcb_subc_t *subc, const char *ps, pcb_coord_t px, pcb_coord_t py, char *name)
 {
 	pcb_flag_t flags = pcb_no_flags();
 	pcb_coord_t thickness, hole, ms, cl;
@@ -136,7 +136,10 @@ static void parse_pstk(hkp_ctx_t *ctx, pcb_element_t *elem, const char *ps, pcb_
 				hs++;
 				w = pcb_get_value(curr, ctx->unit, NULL, NULL);
 				h = pcb_get_value(hs, ctx->unit, NULL, NULL);
+#warning padstack TODO: rewrite
+#if 0
 				pcb_element_pad_new_rect(elem, px+w/2, py+h/2, px-w/2, py-h/2, cl, ms, name, name, flags);
+#endif
 			}
 			else if (strncmp(curr, "Round", 5) == 0) {
 				curr += 5;
@@ -159,6 +162,8 @@ static void parse_pstk(hkp_ctx_t *ctx, pcb_element_t *elem, const char *ps, pcb_
 		}
 	}
 
+#warning padstack TODO: rewrite
+#if 0
 	if (hole > 0) {
 		pcb_element_pin_new(elem, px, py, thickness, cl, ms, hole, name, name, flags);
 		sqpad_pending = 0;
@@ -166,10 +171,10 @@ static void parse_pstk(hkp_ctx_t *ctx, pcb_element_t *elem, const char *ps, pcb_
 
 	if (sqpad_pending)
 		pcb_element_pad_new_rect(elem, px, py, px+thickness, py+thickness, cl, ms, name, name, flags);
-
+#endif
 }
 
-static void parse_pin(hkp_ctx_t *ctx, pcb_element_t *elem, node_t *nd)
+static void parse_pin(hkp_ctx_t *ctx, pcb_subc_t *subc, node_t *nd)
 {
 	node_t *tmp;
 	pcb_coord_t px, py, ms, cl;
@@ -183,17 +188,17 @@ static void parse_pin(hkp_ctx_t *ctx, pcb_element_t *elem, node_t *nd)
 
 	tmp = find_nth(nd->first_child, "PADSTACK", 0);
 	if (tmp != NULL)
-		parse_pstk(ctx, elem, tmp->argv[1], px, py, nd->argv[1]);
+		parse_pstk(ctx, subc, tmp->argv[1], px, py, nd->argv[1]);
 }
 
-static void parse_silk(hkp_ctx_t *ctx, pcb_element_t *elem, node_t *nd)
+static void parse_silk(hkp_ctx_t *ctx, pcb_subc_t *subc, node_t *nd)
 {
 	node_t *tmp, *pp;
 
 	tmp = find_nth(nd->first_child, "SIDE", 0);
 	if (tmp != NULL) {
 		if (strcmp(tmp->argv[1], "MNT_SIDE") != 0) {
-			pcb_message(PCB_MSG_ERROR, "Ignoring element silk line not on mount-side\n");
+			pcb_message(PCB_MSG_ERROR, "Ignoring subcircuit silk line not on mount-side\n");
 			return;
 		}
 	}
@@ -209,7 +214,10 @@ static void parse_silk(hkp_ctx_t *ctx, pcb_element_t *elem, node_t *nd)
 		parse_xy(ctx, tmp->argv[1], &px, &py);
 		for(n = 2; n < tmp->argc; n++) {
 			parse_xy(ctx, tmp->argv[n], &x, &y);
+#warning subc TODO: rewrite this for subcircuits
+#if 0
 			pcb_element_line_new(elem, px, py, x, y, th);
+#endif
 			px = x;
 			py = y;
 		}
@@ -222,7 +230,7 @@ static void parse_package(hkp_ctx_t *ctx, node_t *nd)
 	static const char *tagnames[] = { "PACKAGE_GROUP", "MOUNT_TYPE", "NUMBER_LAYERS", "TIMESTAMP", NULL };
 	const char **t;
 #endif
-	pcb_element_t *elem;
+	pcb_subc_t *subc;
 	pcb_flag_t flags = pcb_no_flags();
 	char *desc = "", *refdes = "", *value = "";
 	node_t *n, *tt, *attr, *tmp;
@@ -245,6 +253,8 @@ static void parse_package(hkp_ctx_t *ctx, node_t *nd)
 		}
 	}
 
+#warning subc TODO: rewrite for subc
+#if 0
 	elem = pcb_element_new(ctx->pcb->Data, NULL, pcb_font(ctx->pcb, 0, 1),
 		flags, desc, refdes, value, tx, ty, dir, 100, flags, 0);
 
@@ -255,6 +265,7 @@ static void parse_package(hkp_ctx_t *ctx, node_t *nd)
 		else if (strcmp(n->argv[0], "SILKSCREEN_OUTLINE") == 0)
 			parse_silk(ctx, elem, n);
 	}
+#endif
 
 #if 0
 	/* extract tags */
@@ -410,7 +421,7 @@ int io_mentor_cell_read_pcb(pcb_plug_io_t *pctx, pcb_board_t *pcb, const char *f
 	/* we are loading the cells into a board, make a default layer stack for that */
 	pcb_layergrp_inhibit_inc();
 	pcb_layers_reset(pcb);
-	pcb_layer_group_setup_default(&pcb->LayerGroups);
+	pcb_layer_group_setup_default(pcb);
 	pcb_layer_group_setup_silks(pcb);
 	pcb_layer_auto_fixup(pcb);
 	pcb_layergrp_inhibit_dec();
