@@ -66,12 +66,20 @@
 #include "obj_poly.h"
 #include "thermal.h"
 #include "event.h"
+#include "macro.h"
 
 #include "src_plugins/lib_compat_help/layer_compat.h"
 #include "src_plugins/lib_compat_help/pstk_compat.h"
 #include "src_plugins/lib_compat_help/subc_help.h"
 #include "src_plugins/lib_compat_help/elem_rot.h"
 
+/* Note: this works because of using str_flag compat_types */
+#define PCB_OBJ_VIA PCB_OBJ_PSTK
+#define PCB_OBJ_PIN PCB_OBJ_PSTK
+#define PCB_OBJ_PAD PCB_OBJ_PSTK
+#define PCB_OBJ_ELEMENT PCB_OBJ_SUBC
+#define PCB_OBJ_ELEMENT_NAME PCB_OBJ_TEXT
+
 pcb_unit_style_t pcb_io_pcb_usty_seen;
 
 static void WritePCBInfoHeader(FILE *);
@@ -86,7 +94,7 @@ static void WriteLayerData(FILE *, pcb_cardinal_t, pcb_layer_t *);
 static char *pcb_strflg_f2s_compat(pcb_flag_t flags, int object_type, unsigned char *intconn)
 {
 	flags.f &= ~IGNORE_FLAGS;
-	return pcb_strflg_f2s(flags, object_type, intconn);
+	return pcb_strflg_f2s(flags, object_type, intconn, 1);
 }
 
 
@@ -287,10 +295,10 @@ static void WritePCBDataHeader(FILE * FP)
 	pcb_fprintf(FP, "Grid[%[0] %[0] %[0] %d]\n", PCB->Grid, PCB->GridOffsetX, PCB->GridOffsetY, conf_core.editor.draw_grid);
 	pcb_fprintf(FP, "Cursor[%[0] %[0] %s]\n", pcb_crosshair.X, pcb_crosshair.Y, c_dtostr(PCB->Zoom));
 	/* PolyArea should be output in square cmils, no suffix */
-	fprintf(FP, "PolyArea[%s]\n", c_dtostr(PCB_COORD_TO_MIL(PCB_COORD_TO_MIL(PCB->IsleArea) * 100) * 100));
+	fprintf(FP, "PolyArea[%s]\n", c_dtostr(PCB_COORD_TO_MIL(PCB_COORD_TO_MIL(conf_core.design.poly_isle_area) * 100) * 100));
 	pcb_fprintf(FP, "Thermal[%s]\n", c_dtostr(PCB->ThermScale));
-	pcb_fprintf(FP, "DRC[%[0] %[0] %[0] %[0] %[0] %[0]]\n", PCB->Bloat, PCB->Shrink,
-							PCB->minWid, PCB->minSlk, PCB->minDrill, PCB->minRing);
+	pcb_fprintf(FP, "DRC[%[0] %[0] %[0] %[0] %[0] %[0]]\n", conf_core.design.bloat, conf_core.design.shrink,
+							conf_core.design.min_wid, conf_core.design.min_slk, conf_core.design.min_drill, conf_core.design.min_ring);
 	fprintf(FP, "Flags(%s)\n", pcb_strflg_board_f2s(pcb_flags));
 	fprintf(FP, "Groups(\"%s\")\n", LayerGroupsToString(&PCB->LayerGroups));
 	fputs("Styles[\"", FP);
@@ -337,16 +345,9 @@ static void WritePCBFontData(FILE * FP)
 static void WriteViaData(FILE * FP, pcb_data_t *Data)
 {
 	gdl_iterator_t it;
-	pcb_pin_t *via;
 	pcb_pstk_t *ps;
 
 	/* write information about vias */
-	pinlist_foreach(&Data->Via, &it, via) {
-		pcb_fprintf(FP, "Via[%[0] %[0] %[0] %[0] %[0] %[0] ", via->X, via->Y,
-								via->Thickness, via->Clearance, via->Mask, via->DrillingHole);
-		pcb_print_quoted_string(FP, (char *) PCB_EMPTY(via->Name));
-		fprintf(FP, " %s]\n", F2S(via, PCB_TYPE_VIA));
-	}
 	padstacklist_foreach(&Data->padstack, &it, ps) {
 		pcb_coord_t x, y, drill_dia, pad_dia, clearance, mask;
 		pcb_pstk_compshape_t cshape;
@@ -361,7 +362,7 @@ static void WriteViaData(FILE * FP, pcb_data_t *Data)
 		pcb_fprintf(FP, "Via[%[0] %[0] %[0] %[0] %[0] %[0] ", x, y,
 			pad_dia, clearance*2, mask, drill_dia);
 		pcb_print_quoted_string(FP, (char *) PCB_EMPTY(name));
-		fprintf(FP, " %s]\n", pcb_strflg_f2s(pcb_pstk_compat_pinvia_flag(ps, cshape), PCB_TYPE_VIA, NULL));
+		fprintf(FP, " %s]\n", pcb_strflg_f2s(pcb_pstk_compat_pinvia_flag(ps, cshape), PCB_OBJ_VIA, NULL, 1));
 	}
 }
 
@@ -374,7 +375,7 @@ static void WritePCBRatData(FILE * FP)
 	ratlist_foreach(&PCB->Data->Rat, &it, line) {
 		pcb_fprintf(FP, "Rat[%[0] %[0] %d %[0] %[0] %d ",
 								line->Point1.X, line->Point1.Y, line->group1, line->Point2.X, line->Point2.Y, line->group2);
-		fprintf(FP, " %s]\n", F2S(line, PCB_TYPE_RATLINE));
+		fprintf(FP, " %s]\n", F2S(line, PCB_OBJ_RAT));
 	}
 }
 
@@ -413,74 +414,6 @@ static void WritePCBNetlistPatchData(FILE * FP)
 	}
 }
 
-int io_pcb_WriteElementData(pcb_plug_io_t *ctx, FILE * FP, pcb_data_t *Data)
-{
-	gdl_iterator_t eit;
-	pcb_line_t *line;
-	pcb_arc_t *arc;
-	pcb_element_t *element;
-
-	pcb_printf_slot[0] = ((io_pcb_ctx_t *)(ctx->plugin_data))->write_coord_fmt;
-	elementlist_foreach(&Data->Element, &eit, element) {
-		gdl_iterator_t it;
-		pcb_pin_t *pin;
-		pcb_pad_t *pad;
-
-		/* only non empty elements */
-		if (!linelist_length(&element->Line) && !pinlist_length(&element->Pin) && !arclist_length(&element->Arc) && !padlist_length(&element->Pad))
-			continue;
-		/* the coordinates and text-flags are the same for
-		 * both names of an element
-		 */
-		fprintf(FP, "\nElement[%s ", F2S(element, PCB_TYPE_ELEMENT));
-		pcb_print_quoted_string(FP, (char *) PCB_EMPTY(PCB_ELEM_NAME_DESCRIPTION(element)));
-		fputc(' ', FP);
-		pcb_print_quoted_string(FP, (char *) PCB_EMPTY(PCB_ELEM_NAME_REFDES(element)));
-		fputc(' ', FP);
-		pcb_print_quoted_string(FP, (char *) PCB_EMPTY(PCB_ELEM_NAME_VALUE(element)));
-		pcb_fprintf(FP, " %[0] %[0] %[0] %[0] %d %d %s]\n(\n",
-								element->MarkX, element->MarkY,
-								PCB_ELEM_TEXT_DESCRIPTION(element).X - element->MarkX,
-								PCB_ELEM_TEXT_DESCRIPTION(element).Y - element->MarkY,
-								PCB_ELEM_TEXT_DESCRIPTION(element).Direction,
-								PCB_ELEM_TEXT_DESCRIPTION(element).Scale, F2S(&(PCB_ELEM_TEXT_DESCRIPTION(element)), PCB_TYPE_ELEMENT_NAME));
-		WriteAttributeList(FP, &element->Attributes, "\t");
-		pinlist_foreach(&element->Pin, &it, pin) {
-			pcb_fprintf(FP, "\tPin[%[0] %[0] %[0] %[0] %[0] %[0] ",
-									pin->X - element->MarkX,
-									pin->Y - element->MarkY, pin->Thickness, pin->Clearance, pin->Mask, pin->DrillingHole);
-			pcb_print_quoted_string(FP, (char *) PCB_EMPTY(pin->Name));
-			fprintf(FP, " ");
-			pcb_print_quoted_string(FP, (char *) PCB_EMPTY(pin->Number));
-			fprintf(FP, " %s]\n", F2S(pin, PCB_TYPE_PIN));
-		}
-		pinlist_foreach(&element->Pad, &it, pad) {
-			pcb_fprintf(FP, "\tPad[%[0] %[0] %[0] %[0] %[0] %[0] %[0] ",
-									pad->Point1.X - element->MarkX,
-									pad->Point1.Y - element->MarkY,
-									pad->Point2.X - element->MarkX, pad->Point2.Y - element->MarkY, pad->Thickness, pad->Clearance, pad->Mask);
-			pcb_print_quoted_string(FP, (char *) PCB_EMPTY(pad->Name));
-			fprintf(FP, " ");
-			pcb_print_quoted_string(FP, (char *) PCB_EMPTY(pad->Number));
-			fprintf(FP, " %s]\n", F2S(pad, PCB_TYPE_PAD));
-		}
-		linelist_foreach(&element->Line, &it, line) {
-			pcb_fprintf(FP, "\tElementLine [%[0] %[0] %[0] %[0] %[0]]\n",
-									line->Point1.X - element->MarkX,
-									line->Point1.Y - element->MarkY,
-									line->Point2.X - element->MarkX, line->Point2.Y - element->MarkY, line->Thickness);
-		}
-		arclist_foreach(&element->Arc, &it, arc) {
-			pcb_fprintf(FP, "\tElementArc [%[0] %[0] %[0] %[0] %ma %ma %[0]]\n",
-									arc->X - element->MarkX,
-									arc->Y - element->MarkY, arc->Width, arc->Height, arc->StartAngle, arc->Delta, arc->Thickness);
-		}
-		fputs("\n\t)\n", FP);
-	}
-	return 0;
-}
-
-
 int io_pcb_WriteSubcData(pcb_plug_io_t *ctx, FILE *FP, pcb_data_t *Data)
 {
 	gdl_iterator_t sit, it;
@@ -490,10 +423,11 @@ int io_pcb_WriteSubcData(pcb_plug_io_t *ctx, FILE *FP, pcb_data_t *Data)
 	pcb_printf_slot[0] = ((io_pcb_ctx_t *)(ctx->plugin_data))->write_coord_fmt;
 
 	subclist_foreach(&Data->subc, &sit, sc) {
-		pcb_coord_t ox, oy, rx, ry;
+		pcb_coord_t ox, oy, rx = 0, ry = 0;
 		int rdir = 0, rscale = 100;
 		pcb_text_t *trefdes;
 		pcb_pstk_t *ps;
+		pcb_any_obj_t fobj;
 
 		pcb_subc_get_origin(sc, &ox, &oy);
 		trefdes = pcb_subc_get_refdes_text(sc);
@@ -505,17 +439,32 @@ int io_pcb_WriteSubcData(pcb_plug_io_t *ctx, FILE *FP, pcb_data_t *Data)
 			rscale = trefdes->Scale;
 		}
 		else {
-			rx = ox;
-			ry = oy;
+			const char *tmp;
+			tmp = pcb_attribute_get(&sc->Attributes, "io_pcb::hidename_x");
+			if (tmp != NULL)
+				rx = pcb_get_value(tmp, NULL, NULL, NULL) - ox;
+			tmp = pcb_attribute_get(&sc->Attributes, "io_pcb::hidename_y");
+			if (tmp != NULL)
+				ry = pcb_get_value(tmp, NULL, NULL, NULL) - oy;
+			tmp = pcb_attribute_get(&sc->Attributes, "io_pcb::hidename_direction");
+			if (tmp != NULL)
+				rdir = atoi(tmp);
+			tmp = pcb_attribute_get(&sc->Attributes, "io_pcb::hidename_scale");
+			if (tmp != NULL)
+				rscale = atoi(tmp);
 		}
 
-		fprintf(FP, "\nElement[%s ", F2S(sc, PCB_TYPE_ELEMENT));
+		memset(&fobj, 0, sizeof(fobj));
+		fobj.Flags = sc->Flags;
+		if (trefdes == NULL)
+			fobj.Flags.f |= PCB_FLAG_HIDENAME;
+		fprintf(FP, "\nElement[%s ", F2S(&fobj, PCB_OBJ_ELEMENT));
 		pcb_print_quoted_string(FP, (char *) PCB_EMPTY(pcb_attribute_get(&sc->Attributes, "footprint")));
 		fputc(' ', FP);
 		pcb_print_quoted_string(FP, (char *) PCB_EMPTY(pcb_attribute_get(&sc->Attributes, "refdes")));
 		fputc(' ', FP);
 		pcb_print_quoted_string(FP, (char *) PCB_EMPTY(pcb_attribute_get(&sc->Attributes, "value")));
-		pcb_fprintf(FP, " %[0] %[0] %[0] %[0] %d %d %s]\n(\n", ox, oy, rx, ry, rdir, rscale, trefdes != NULL ? F2S(trefdes, PCB_TYPE_ELEMENT_NAME) : "\"\"");
+		pcb_fprintf(FP, " %[0] %[0] %[0] %[0] %d %d %s]\n(\n", ox, oy, rx, ry, rdir, rscale, trefdes != NULL ? F2S(trefdes, PCB_OBJ_ELEMENT_NAME) : "\"\"");
 		WriteAttributeList(FP, &sc->Attributes, "\t");
 
 		padstacklist_foreach(&sc->data->padstack, &it, ps) {
@@ -528,16 +477,16 @@ int io_pcb_WriteSubcData(pcb_plug_io_t *ctx, FILE *FP, pcb_data_t *Data)
 				pcb_print_quoted_string(FP, (char *)PCB_EMPTY(pcb_attribute_get(&ps->Attributes, "name")));
 				fprintf(FP, " ");
 				pcb_print_quoted_string(FP, (char *) PCB_EMPTY(pcb_attribute_get(&ps->Attributes, "term")));
-				fprintf(FP, " %s]\n", pcb_strflg_f2s(pcb_pstk_compat_pinvia_flag(ps, cshape), PCB_TYPE_PIN, &ic));
+				fprintf(FP, " %s]\n", pcb_strflg_f2s(pcb_pstk_compat_pinvia_flag(ps, cshape), PCB_OBJ_PIN, &ic, 1));
 			}
 			else if (pcb_pstk_export_compat_pad(ps, &x1, &y1, &x2, &y2, &thickness, &clearance, &mask, &square, &nopaste)) {
 				unsigned long fl = (square ? PCB_FLAG_SQUARE : 0) | (nopaste ? PCB_FLAG_NOPASTE : 0);
 				pcb_fprintf(FP, "\tPad[%[0] %[0] %[0] %[0] %[0] %[0] %[0] ",
-					x1 - ox, y1 - oy, x2 - ox, y2 - oy, thickness, clearance, mask);
+					x1 - ox, y1 - oy, x2 - ox, y2 - oy, thickness, clearance*2, mask);
 					pcb_print_quoted_string(FP, (char *)PCB_EMPTY(pcb_attribute_get(&ps->Attributes, "name")));
 					fprintf(FP, " ");
 					pcb_print_quoted_string(FP, (char *) PCB_EMPTY(pcb_attribute_get(&ps->Attributes, "term")));
-					fprintf(FP, " %s]\n", pcb_strflg_f2s(pcb_flag_make(fl), PCB_TYPE_PAD, &ic));
+					fprintf(FP, " %s]\n", pcb_strflg_f2s(pcb_flag_make(fl), PCB_OBJ_PAD, &ic, 1));
 			}
 			else
 				pcb_io_incompat_save(sc->data, (pcb_any_obj_t *)ps, "Padstack can not be exported as pin or pad", "use simpler padstack; for pins, all copper layers must have the same shape and there must be no paste; for pads, use a line or a rectangle; paste and mask must match the copper shape");
@@ -617,27 +566,27 @@ static void WriteLayerData(FILE * FP, pcb_cardinal_t Number, pcb_layer_t *layer)
 		fprintf(FP, "Layer(%i ", (int) Number + 1);
 		pcb_print_quoted_string(FP, layer_name_hack(layer, PCB_EMPTY(layer->name)));
 		fputs(")\n(\n", FP);
-		WriteAttributeList(FP, &layer->meta.real.Attributes, "\t");
+		WriteAttributeList(FP, &layer->Attributes, "\t");
 
 		linelist_foreach(&layer->Line, &it, line) {
 			pcb_fprintf(FP, "\tLine[%[0] %[0] %[0] %[0] %[0] %[0] %s]\n",
 									line->Point1.X, line->Point1.Y,
-									line->Point2.X, line->Point2.Y, line->Thickness, line->Clearance, F2S(line, PCB_TYPE_LINE));
+									line->Point2.X, line->Point2.Y, line->Thickness, line->Clearance, F2S(line, PCB_OBJ_LINE));
 		}
 		arclist_foreach(&layer->Arc, &it, arc) {
 			pcb_fprintf(FP, "\tArc[%[0] %[0] %[0] %[0] %[0] %[0] %ma %ma %s]\n",
 									arc->X, arc->Y, arc->Width,
-									arc->Height, arc->Thickness, arc->Clearance, arc->StartAngle, arc->Delta, F2S(arc, PCB_TYPE_ARC));
+									arc->Height, arc->Thickness, arc->Clearance, arc->StartAngle, arc->Delta, F2S(arc, PCB_OBJ_ARC));
 		}
 		textlist_foreach(&layer->Text, &it, text) {
 			pcb_fprintf(FP, "\tText[%[0] %[0] %d %d ", text->X, text->Y, text->Direction, text->Scale);
 			pcb_print_quoted_string(FP, (char *) PCB_EMPTY(text->TextString));
-			fprintf(FP, " %s]\n", F2S(text, PCB_TYPE_TEXT));
+			fprintf(FP, " %s]\n", F2S(text, PCB_OBJ_TEXT));
 		}
 		textlist_foreach(&layer->Polygon, &it, polygon) {
 			int p, i = 0;
 			pcb_cardinal_t hole = 0;
-			fprintf(FP, "\tPolygon(%s)\n\t(", F2S(polygon, PCB_TYPE_POLY));
+			fprintf(FP, "\tPolygon(%s)\n\t(", F2S(polygon, PCB_OBJ_POLY));
 			for (p = 0; p < polygon->PointN; p++) {
 				pcb_point_t *point = &polygon->Points[p];
 
@@ -711,7 +660,7 @@ int io_pcb_WriteBuffer(pcb_plug_io_t *ctx, FILE * FP, pcb_buffer_t *buff, pcb_bo
 	pcb_printf_slot[0] = ((io_pcb_ctx_t *)(ctx->plugin_data))->write_coord_fmt;
 
 	if (elem_only) {
-		if ((elementlist_length(&buff->Data->Element) == 0) && (pcb_subclist_length(&buff->Data->subc) == 0)) {
+		if (pcb_subclist_length(&buff->Data->subc) == 0) {
 			pcb_message(PCB_MSG_ERROR, "Buffer has no subcircuits!\n");
 			return -1;
 		}
@@ -721,7 +670,6 @@ int io_pcb_WriteBuffer(pcb_plug_io_t *ctx, FILE * FP, pcb_buffer_t *buff, pcb_bo
 		WriteViaData(FP, buff->Data);
 	}
 
-	io_pcb_WriteElementData(ctx, FP, buff->Data);
 	io_pcb_WriteSubcData(ctx, FP, buff->Data);
 
 	if (!elem_only)
@@ -742,7 +690,6 @@ int io_pcb_WritePCB(pcb_plug_io_t *ctx, FILE * FP, const char *old_filename, con
 	WritePCBFontData(FP);
 	WriteAttributeList(FP, &PCB->Attributes, "");
 	WriteViaData(FP, PCB->Data);
-	io_pcb_WriteElementData(ctx, FP, PCB->Data);
 	io_pcb_WriteSubcData(ctx, FP, PCB->Data);
 	WritePCBRatData(FP);
 	WriteLayers(FP, PCB->Data);
@@ -779,7 +726,7 @@ void PostLoadElementPCB()
 		return;
 
 	pcb_board_new_postproc(yyPCB, 0);
-	pcb_layer_group_setup_default(&yyPCB->LayerGroups);
+	pcb_layer_group_setup_default(yyPCB);
 	PCB = yyPCB;
 	pcb_layer_group_setup_silks(yyPCB);
 	pcb_data_bbox(&dbb, yyPCB->Data, pcb_false);
@@ -867,7 +814,9 @@ pcb_layer_id_t static new_ly_end(pcb_board_t *pcb, const char *name)
 		return -1;
 	lid = pcb->Data->LayerN;
 	pcb->Data->Layer[lid].name = pcb_strdup(name);
-	pcb->Data->Layer[lid].parent = pcb->Data;
+	pcb->Data->Layer[lid].parent.data = pcb->Data;
+	pcb->Data->Layer[lid].parent_type = PCB_PARENT_DATA;
+	pcb->Data->Layer[lid].type = PCB_OBJ_LAYER;
 	pcb->Data->LayerN++;
 	return lid;
 }
@@ -895,7 +844,9 @@ pcb_layer_id_t static new_ly_old(pcb_board_t *pcb, const char *name)
 		if (pcb->Data->Layer[lid].meta.real.grp == 0) {
 			free((char *)pcb->Data->Layer[lid].name);
 			pcb->Data->Layer[lid].name = pcb_strdup(name);
-			pcb->Data->Layer[lid].parent = pcb->Data;
+			pcb->Data->Layer[lid].parent.data = pcb->Data;
+			pcb->Data->Layer[lid].parent_type = PCB_PARENT_DATA;
+			pcb->Data->Layer[lid].type = PCB_OBJ_LAYER;
 			return lid;
 		}
 	}
@@ -908,7 +859,7 @@ int pcb_layer_improvise(pcb_board_t *pcb, pcb_bool setup)
 	pcb_layer_id_t lid, silk = -1;
 
 	if (setup) {
-		pcb_layer_group_setup_default(&pcb->LayerGroups);
+		pcb_layer_group_setup_default(pcb);
 
 		for(lid = 0; lid < pcb->Data->LayerN; lid++) {
 			if (strcmp(pcb->Data->Layer[lid].name, "silk") == 0) {
@@ -1039,7 +990,20 @@ pcb_subc_t *io_pcb_element_new(pcb_data_t *Data, pcb_subc_t *subc,
 		pcb_attribute_put(&sc->Attributes, "value", Value);
 
 #warning subc TODO: TextFlags
-	txt = pcb_subc_add_refdes_text(sc, TextX, TextY, Direction, TextScale, yysubc_bottom);
+	if (!(Flags.f & PCB_FLAG_HIDENAME))
+		txt = pcb_subc_add_refdes_text(sc, TextX, TextY, Direction, TextScale, yysubc_bottom);
+
+	if (Flags.f & PCB_FLAG_HIDENAME) {
+		char tmp[128];
+		pcb_sprintf(tmp, "%$mm", TextX);
+		pcb_attribute_put(&sc->Attributes, "io_pcb::hidename_x", tmp);
+		pcb_sprintf(tmp, "%$mm", TextY);
+		pcb_attribute_put(&sc->Attributes, "io_pcb::hidename_y", tmp);
+		sprintf(tmp, "%d", Direction);
+		pcb_attribute_put(&sc->Attributes, "io_pcb::hidename_direction", tmp);
+		sprintf(tmp, "%d", TextScale);
+		pcb_attribute_put(&sc->Attributes, "io_pcb::hidename_scale", tmp);
+	}
 
 	return sc;
 }
@@ -1086,7 +1050,7 @@ pcb_pstk_t *io_pcb_element_pin_new(pcb_subc_t *subc, pcb_coord_t X, pcb_coord_t
 		pcb_attribute_put(&p->Attributes, "name", Name);
 
 	if (yysubc_bottom)
-		pcb_pstk_mirror(p, 0, 1);
+		pcb_pstk_mirror(p, PCB_PSTK_DONT_MIRROR_COORDS, 1, 0);
 	return p;
 }
 
@@ -1103,7 +1067,7 @@ pcb_pstk_t *io_pcb_element_pad_new(pcb_subc_t *subc, pcb_coord_t X1, pcb_coord_t
 	if (yysubc_bottom) {
 		pcb_data_t *old_hack = pcb_pstk_data_hack;
 		pcb_pstk_data_hack = subc->parent.data;
-		pcb_pstk_mirror(p, 0, 1);
+		pcb_pstk_mirror(p, PCB_PSTK_DONT_MIRROR_COORDS, 1, 1);
 		pcb_pstk_data_hack = old_hack;
 	}
 
