@@ -44,17 +44,14 @@
 #include "pcb-printf.h"
 #include "compat_misc.h"
 #include "safe_fs.h"
-#include "obj_elem.h"
 #include "obj_line.h"
 #include "obj_arc.h"
-#include "obj_pad.h"
-#include "obj_pinvia.h"
 #include "obj_pstk.h"
 #include "obj_pstk_inlines.h"
 #include "../src_plugins/lib_compat_help/subc_help.h"
 #include "../src_plugins/lib_compat_help/pstk_help.h"
 
-static void print_sqpad_coords(FILE *f, pcb_pad_t *Pad, pcb_coord_t cx, pcb_coord_t cy)
+static void print_sqpad_coords(FILE *f, pcb_any_line_t *Pad, pcb_coord_t cx, pcb_coord_t cy)
 {
 	pcb_coord_t x[4], y[4];
 
@@ -123,64 +120,6 @@ int tedax_fp_fsave(pcb_data_t *data, FILE *f)
 
 	fprintf(f, "tEDAx v1\n");
 
-	/* This block shall be removed with elements */
-	PCB_ELEMENT_LOOP(data)
-	{
-		fprintf(f, "\nbegin footprint v1 %s\n", element->Name->TextString);
-		PCB_PAD_LOOP(element)
-		{
-			char *lloc, *pnum;
-			lloc = elem_layer(element, pad);
-			safe_term_num(pnum, pad, buff);
-			print_term(pnum, pad);
-			if (PCB_FLAG_TEST(PCB_FLAG_SQUARE, pad)) { /* sqaure cap pad -> poly */
-				pcb_fprintf(f, "	polygon %s copper %s %mm 4", lloc, pnum, pad->Clearance);
-				print_sqpad_coords(f, pad,  element->MarkX, element->MarkY);
-				pcb_fprintf(f, "\n");
-			}
-			else { /* round cap pad -> line */
-				pcb_fprintf(f, "	line %s copper %s %mm %mm %mm %mm %mm %mm\n", lloc, pnum, pad->Point1.X - element->MarkX, pad->Point1.Y - element->MarkY, pad->Point2.X - element->MarkX, pad->Point2.Y - element->MarkY, pad->Thickness, pad->Clearance);
-			}
-		}
-		PCB_END_LOOP;
-
-		PCB_PIN_LOOP(element)
-		{
-			char *pnum;
-			safe_term_num(pnum, pin, buff);
-			print_term(pnum, pin);
-			pcb_fprintf(f, "	fillcircle all copper %s %mm %mm %mm %mm\n", pnum, pin->X - element->MarkX, pin->Y - element->MarkY, pin->Thickness/2, pin->Clearance);
-			pcb_fprintf(f, "	hole %s %mm %mm %mm %s\n", pnum, pin->X - element->MarkX, pin->Y - element->MarkY, pin->DrillingHole, PIN_PLATED(pin));
-		}
-		PCB_END_LOOP;
-
-		PCB_ELEMENT_PCB_LINE_LOOP(element)
-		{
-			char *lloc = /*elem_layer(element, line)*/ "primary";
-			pcb_fprintf(f, "	line %s silk - %mm %mm %mm %mm %mm %mm\n", lloc,
-				line->Point1.X - element->MarkX, line->Point1.Y - element->MarkY, line->Point2.X - element->MarkX, line->Point2.Y - element->MarkY, line->Thickness, line->Clearance);
-		}
-		PCB_END_LOOP;
-
-		PCB_ELEMENT_ARC_LOOP(element)
-		{
-			char *lloc = /*elem_layer(element, arc)*/ "primary";
-			pcb_fprintf(f, "	arc %s silk - %mm %mm %mm %f %f %mm %mm\n", lloc,
-				arc->X - element->MarkX, arc->Y - element->MarkY, (arc->Width + arc->Height)/2, arc->StartAngle, arc->Delta,
-				arc->Thickness, arc->Clearance);
-		}
-		PCB_END_LOOP;
-
-		fprintf(f, "end footprint\n");
-
-		for (e = htsp_first(&terms); e; e = htsp_next(&terms, e)) {
-			free(e->key);
-			htsp_delentry(&terms, e);
-		}
-
-	}
-	PCB_END_LOOP;
-
 	PCB_SUBC_LOOP(data)
 	{
 		pcb_coord_t ox = 0, oy = 0;
@@ -189,9 +128,9 @@ int tedax_fp_fsave(pcb_data_t *data, FILE *f)
 
 		fpname = pcb_attribute_get(&subc->Attributes, "tedax::footprint");
 		if (fpname == NULL)
-			fpname = pcb_attribute_get(&subc->Attributes, "footprint");
+			fpname = pcb_attribute_get(&subc->Attributes, "visible_footprint");
 		if (fpname == NULL)
-			fpname = pcb_attribute_get(&subc->Attributes, "name");
+			fpname = pcb_attribute_get(&subc->Attributes, "footprint");
 		if ((fpname == NULL) && (subc->refdes != NULL))
 			fpname = subc->refdes;
 		if (fpname == NULL)
@@ -267,15 +206,6 @@ int tedax_fp_fsave(pcb_data_t *data, FILE *f)
 			PCB_END_LOOP;
 		}
 
-		PCB_VIA_LOOP(subc->data)
-		{
-			if (via->term != NULL) print_terma(via->term, via);
-
-			pcb_fprintf(f, "	fillcircle all copper %s %mm %mm %mm %mm\n", TERM_NAME(via->term), via->X - ox, via->Y - oy, via->Thickness/2, via->Clearance);
-			pcb_fprintf(f, "	hole %s %mm %mm %mm %s\n", TERM_NAME(via->term), via->X - ox, via->Y - oy, via->DrillingHole, PIN_PLATED(via));
-		}
-		PCB_END_LOOP;
-
 		PCB_PADSTACK_LOOP(subc->data)
 		{
 			pcb_pstk_proto_t *proto = pcb_pstk_get_proto(padstack);
@@ -306,7 +236,7 @@ int tedax_fp_fsave(pcb_data_t *data, FILE *f)
 						break;
 					case PCB_PSSH_LINE:
 						if (shp->data.line.square) {
-							pcb_pad_t tmp;
+							pcb_any_line_t tmp;
 							tmp.Point1.X = shp->data.line.x1 + padstack->x;
 							tmp.Point1.Y = shp->data.line.y1 + padstack->y;
 							tmp.Point2.X = shp->data.line.x2 + padstack->x;
