@@ -183,20 +183,23 @@ int pplg_check_ver_io_tedax(int ver_needed) { return 0; }
 void pplg_uninit_io_tedax(void)
 {
 	pcb_hid_remove_actions_by_cookie(tedax_cookie);
+	PCB_HOOK_UNREGISTER(pcb_plug_io_t, pcb_plug_io_chain, &io_tedax);
 }
 
 #include "dolists.h"
 int pplg_init_io_tedax(void)
 {
+	PCB_API_CHK_VER;
+
 	/* register the IO hook */
 	io_tedax.plugin_data = NULL;
 	io_tedax.fmt_support_prio = io_tedax_fmt;
 	io_tedax.test_parse = io_tedax_test_parse;
 	io_tedax.parse_pcb = io_tedax_parse_pcb;
-	io_tedax.parse_element = io_tedax_parse_element;
+	io_tedax.parse_footprint = io_tedax_parse_element;
 	io_tedax.parse_font = NULL;
 	io_tedax.write_buffer = io_tedax_write_buffer;
-	io_tedax.write_element = io_tedax_write_element;
+	io_tedax.write_footprint = io_tedax_write_element;
 	io_tedax.write_pcb = NULL;
 	io_tedax.default_fmt = "tEDAx";
 	io_tedax.description = "Trivial EDA eXchange format";
