@@ -1,11 +1,9 @@
-/*!
- * \file jostle.c
+/*
+ * jostle plug-in for PCB.
  *
- * \brief jostle plug-in for PCB.
+ * Copyright (C) 2007 Ben Jackson <ben@ben.com>
  *
- * \author Copyright (C) 2007 Ben Jackson <ben@ben.com>
- *
- * \copyright Licensed under the terms of the GNU General Public
+ * Licensed under the terms of the GNU General Public
  * License, version 2 or later.
  *
  * Ported to pcb-rnd by Tibor 'Igor2' Palinkas in 2016.
@@ -83,14 +81,15 @@ static void Debugpcb_polyarea_t(pcb_polyarea_t * s, char *color)
 	pcb_pline_t *pl;
 	pcb_vnode_t *v;
 	pcb_polyarea_t *p;
-	pcb_hid_t *ddraw;
-	pcb_hid_gc_t ddgc;
 
 #ifndef DEBUG_pcb_polyarea_t
 	return;
 #endif
+
+/* TODO: rewrite this to UI layers if needed
 	ddraw = pcb_gui->request_debug_draw();
 	ddgc = ddraw->make_gc();
+*/
 
 	p = s;
 	do {
@@ -102,32 +101,30 @@ static void Debugpcb_polyarea_t(pcb_polyarea_t * s, char *color)
 				x[i] = v->point[0];
 				y[i++] = v->point[1];
 			}
-			if (1) {
+/*			if (1) {
 				pcb_gui->set_color(ddgc, color ? color : conf_core.appearance.color.connected);
 				pcb_gui->set_line_width(ddgc, 1);
 				for (i = 0; i < n - 1; i++) {
 					pcb_gui->draw_line(ddgc, x[i], y[i], x[i + 1], y[i + 1]);
-					/*  pcb_gui->fill_circle (ddgc, x[i], y[i], 30);*/
+					pcb_gui->fill_circle (ddgc, x[i], y[i], 30);
 				}
 				pcb_gui->draw_line(ddgc, x[n - 1], y[n - 1], x[0], y[0]);
-			}
+			}*/
 			free(x);
 			free(y);
 		}
 	} while ((p = p->f) != s);
-	ddraw->flush_debug_draw();
+/*	ddraw->flush_debug_draw();*/
 	pcb_event(PCB_EVENT_BUSY, NULL);
-	sleep(3);
-	ddraw->finish_debug_draw();
+/*	sleep(3);
+	ddraw->finish_debug_draw();*/
 }
 
-/*!
- * \brief Find the bounding box of a pcb_polyarea_t.
+/* Find the bounding box of a pcb_polyarea_t.
  *
  * pcb_polyarea_ts linked by ->f/b are outlines.\n
  * n->contours->next would be the start of the inner holes (irrelevant
- * for bounding box).
- */
+ * for bounding box). */
 static pcb_box_t pcb_polyarea_t_boundingBox(pcb_polyarea_t * a)
 {
 	pcb_polyarea_t *n;
@@ -155,12 +152,9 @@ static pcb_box_t pcb_polyarea_t_boundingBox(pcb_polyarea_t * a)
 	return box;
 }
 
-/*!
- * Given a polygon and a side of it (a direction north/northeast/etc),
- * find a line tangent to that side, offset by clearance, and return it
- * as a pair of vectors PQ.\n
- * Make it long so it will intersect everything in the area.
- */
+/* Given a polygon and a side of it (a direction north/northeast/etc), find a
+   line tangent to that side, offset by clearance, and return it as a pair of
+   vectors PQ. Make it long so it will intersect everything in the area. */
 static void pcb_polyarea_t_findXmostLine(pcb_polyarea_t * a, int side, pcb_vector_t p, pcb_vector_t q, int clearance)
 {
 	int extra;
@@ -258,24 +252,20 @@ static void pcb_polyarea_t_findXmostLine(pcb_polyarea_t * a, int side, pcb_vecto
 	}
 }
 
-/*!
- * Given a 'side' from the JNORTH/JSOUTH/etc enum, rotate it by n.
- */
+/* Given a 'side' from the JNORTH/JSOUTH/etc enum, rotate it by n. */
 static int rotateSide(int side, int n)
 {
 	return (side + n + 8) % 8;
 }
 
-/*!
- * Wrapper for CreateNewLineOnLayer that takes vectors and deals with Undo
- */
+/* Wrapper for CreateNewLineOnLayer that takes vectors and deals with Undo */
 static pcb_line_t *Createpcb_vector_tLineOnLayer(pcb_layer_t * layer, pcb_vector_t a, pcb_vector_t b, int thickness, int clearance, pcb_flag_t flags)
 {
 	pcb_line_t *line;
 
 	line = pcb_line_new(layer, a[0], a[1], b[0], b[1], thickness, clearance, flags);
 	if (line) {
-		pcb_undo_add_obj_to_create(PCB_TYPE_LINE, layer, line, line);
+		pcb_undo_add_obj_to_create(PCB_OBJ_LINE, layer, line, line);
 	}
 	return line;
 }
@@ -292,25 +282,21 @@ static pcb_line_t *MakeBypassLine(pcb_layer_t * layer, pcb_vector_t a, pcb_vecto
 	return line;
 }
 
-/*!
- * Given a 'brush' that's pushing things out of the way (possibly already
+/* Given a 'brush' that's pushing things out of the way (possibly already
  * cut down to just the part relevant to our line) and a line that
  * intersects it on some layer, find the 45/90 lines required to go around
  * the brush on the named side.  Create them and remove the original.
  *
  * Imagine side = north:
- * <pre>
                  /      \
             ----b##FLAT##c----
                Q          P
    lA-ORIG####a            d####ORIG-lB
              /              \
- * </pre>
  * First find the extended three lines that go around the brush.
  * Then intersect them with each other and the original to find
  * points a, b, c, d.  Finally connect the dots and remove the
- * old straight line.
- */
+ * old straight line. */
 static int MakeBypassingLines(pcb_polyarea_t * brush, pcb_layer_t * layer, pcb_line_t * line, int side, pcb_polyarea_t ** expandp)
 {
 	pcb_vector_t pA, pB, flatA, flatB, qA, qB;
@@ -350,22 +336,13 @@ struct info {
 	pcb_box_t box;
 	pcb_polyarea_t *brush;
 	pcb_layer_t *layer;
-	pcb_polyarea_t *smallest;
-	/*!< after cutting brush with line, the smallest chunk, which we
-	 * will go around on 'side'.
-	 */
+	pcb_polyarea_t *smallest; /* after cutting brush with line, the smallest chunk, which we will go around on 'side'. */
 	pcb_line_t *line;
 	int side;
-	double centroid;
-	/*!< smallest difference between slices of brush after cutting with
-	 * line, trying to find the line closest to the centroid to process
-	 * first
-	 */
+	double centroid; /* smallest difference between slices of brush after cutting with line, trying to find the line closest to the centroid to process first */
 };
 
-/*!
- * Process lines that intersect our 'brush'.
- */
+/* Process lines that intersect our 'brush'. */
 static pcb_r_dir_t jostle_callback(const pcb_box_t * targ, void *private)
 {
 	pcb_line_t *line = (pcb_line_t *) targ;
@@ -510,7 +487,7 @@ static int jostle(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
 		}
 	}
 	else {
-		value = conf_core.design.via_thickness + (PCB->Bloat + 1) * 2 + 50;
+		value = conf_core.design.via_thickness + (conf_core.design.bloat + 1) * 2 + 50;
 	}
 	x = pcb_crosshair.X;
 	y = pcb_crosshair.Y;
@@ -559,6 +536,7 @@ void pplg_uninit_jostle(void)
 #include "dolists.h"
 int pplg_init_jostle(void)
 {
+	PCB_API_CHK_VER;
 	PCB_REGISTER_ACTIONS(jostle_action_list, jostle_cookie);
 	return 0;
 }
