@@ -34,8 +34,9 @@
 
 #include "board.h"
 #include "data.h"
-#include "obj_pinvia.h"
 
+#warning padstack TODO: rewrite this for padstack, if needed
+#if 0
 static void swap_one_thermal(int lid1, int lid2, pcb_pin_t * pin)
 {
 	int was_on_l1 = !!PCB_FLAG_THERM_GET(lid1, pin);
@@ -44,6 +45,7 @@ static void swap_one_thermal(int lid1, int lid2, pcb_pin_t * pin)
 	PCB_FLAG_THERM_ASSIGN(lid2, was_on_l1, pin);
 	PCB_FLAG_THERM_ASSIGN(lid1, was_on_l2, pin);
 }
+#endif
 
 int pcb_layer_swap(pcb_board_t *pcb, pcb_layer_id_t lid1, pcb_layer_id_t lid2)
 {
@@ -59,17 +61,14 @@ int pcb_layer_swap(pcb_board_t *pcb, pcb_layer_id_t lid1, pcb_layer_id_t lid2)
 	pcb_layer_move_(&pcb->Data->Layer[lid1], &l2tmp);
 	pcb_layer_move_(&pcb->Data->Layer[lid2], &l1tmp);
 
+#warning padstack TODO: rewrite this for padstack, if needed
+#if 0
 	PCB_VIA_LOOP(pcb->Data);
 	{
 		swap_one_thermal(lid1, lid2, via);
 	}
 	PCB_END_LOOP;
-
-	PCB_PIN_ALL_LOOP(pcb->Data);
-	{
-		swap_one_thermal(lid1, lid2, pin);
-	}
-	PCB_ENDALL_LOOP;
+#endif
 
 	for(gid = 0; gid < pcb_max_group(pcb); gid++) {
 		pcb_layergrp_t *g = &pcb->LayerGroups.grp[gid];
