@@ -36,6 +36,31 @@
 
 #define sqr(o) ((double)(o)*(double)(o))
 
+/* set up x and y multiplier for an octa poly, depending on square pin style */
+void pcb_poly_square_pin_factors(int style, double *xm, double *ym)
+{
+	int i;
+	const double factor = 2.0;
+
+	/* reset multipliers */
+	for (i = 0; i < 8; i++) {
+		xm[i] = 1;
+		ym[i] = 1;
+	}
+
+	style--;
+	if (style & 1)
+		xm[0] = xm[1] = xm[6] = xm[7] = factor;
+	if (style & 2)
+		xm[2] = xm[3] = xm[4] = xm[5] = factor;
+	if (style & 4)
+		ym[4] = ym[5] = ym[6] = ym[7] = factor;
+	if (style & 8)
+		ym[0] = ym[1] = ym[2] = ym[3] = factor;
+}
+
+
+
 /* emulate old pcb-rnd "pin shape" feature */
 static void octa_shape(pcb_pstk_poly_t *dst, pcb_coord_t x0, pcb_coord_t y0, pcb_coord_t radius, int style)
 {
@@ -150,20 +175,22 @@ pcb_pstk_t *pcb_pstk_new_compat_via(pcb_data_t *data, pcb_coord_t x, pcb_coord_t
 		shape[0].layer_mask = PCB_LYT_COPPER | PCB_LYT_TOP;    shape[0].comb = 0;
 		shape[1].layer_mask = PCB_LYT_COPPER | PCB_LYT_BOTTOM; shape[1].comb = 0;
 		shape[2].layer_mask = PCB_LYT_COPPER | PCB_LYT_INTERN; shape[2].comb = 0;
-
-		if (mask > 0) {
-			if (compat_via_shape_gen(&mask_master, cshape, mask) != 0)
-				return NULL;
-		
-			memcpy(&shape[3], &mask_master, sizeof(mask_master));
-			memcpy(&shape[4], &mask_master, sizeof(mask_master));
-			shape[3].layer_mask = PCB_LYT_MASK | PCB_LYT_TOP;      shape[3].comb = PCB_LYC_SUB + PCB_LYC_AUTO;
-			shape[4].layer_mask = PCB_LYT_MASK | PCB_LYT_BOTTOM;   shape[4].comb = PCB_LYC_SUB + PCB_LYC_AUTO;
-		}
+		tshp.len = 3;
 	}
 	else
 		tshp.len = 0;
 
+	if (mask > 0) {
+		if (compat_via_shape_gen(&mask_master, cshape, mask) != 0)
+			return NULL;
+	
+		memcpy(&shape[tshp.len+0], &mask_master, sizeof(mask_master));
+		memcpy(&shape[tshp.len+1], &mask_master, sizeof(mask_master));
+		shape[tshp.len+0].layer_mask = PCB_LYT_MASK | PCB_LYT_TOP;      shape[tshp.len+0].comb = PCB_LYC_SUB + PCB_LYC_AUTO;
+		shape[tshp.len+1].layer_mask = PCB_LYT_MASK | PCB_LYT_BOTTOM;   shape[tshp.len+1].comb = PCB_LYC_SUB + PCB_LYC_AUTO;
+		tshp.len += 2;
+	}
+
 	proto.hdia = drill_dia;
 	proto.hplated = plated;
 
@@ -432,6 +459,12 @@ static void gen_pad(pcb_pstk_shape_t *dst, pcb_coord_t x1, pcb_coord_t y1, pcb_c
 		dst->shape = PCB_PSSH_POLY;
 		pad_shape(&dst->data.poly, x1, y1, x2, y2, thickness);
 	}
+	else if (x1 == x2 && y1 == y2) {
+		dst->shape = PCB_PSSH_CIRC;
+		dst->data.circ.x = x1;
+		dst->data.circ.y = y1;
+		dst->data.circ.dia = thickness;
+	}
 	else {
 		dst->shape = PCB_PSSH_LINE;
 		dst->data.line.x1 = x1;
