@@ -38,7 +38,6 @@
 #include "layer.h"
 #include "library.h"
 #include "search.h"
-#include "const.h"
 #include "find.h"
 #include "board.h"
 #include "obj_subc.h"
@@ -58,18 +57,15 @@ static char *describe_location(pcb_coord_t X, pcb_coord_t Y)
 
 	/* check if there are any pins or pads at that position */
 
-	type = pcb_search_obj_by_location(PCB_TYPEMASK_TERM | PCB_TYPE_PAD, &ptr1, &ptr2, &ptr3, X, Y, Range);
-	if (type == PCB_TYPE_NONE)
+	type = pcb_search_obj_by_location(PCB_OBJ_CLASS_TERM, &ptr1, &ptr2, &ptr3, X, Y, Range);
+	if (type == PCB_OBJ_VOID)
 		return NULL;
 
 	/* don't mess with silk objects! */
 	if ((type & PCB_SILK_TYPE) && (pcb_layer_flags_((pcb_layer_t *)ptr1) & PCB_LYT_SILK))
 		return NULL;
 
-	if (type == PCB_TYPE_PIN || type == PCB_TYPE_PAD) {
-		elename = (const char *) PCB_UNKNOWN(PCB_ELEM_NAME_REFDES((pcb_element_t *) ptr1));
-	}
-	else if (((pcb_any_obj_t *)ptr2)->term != NULL) {
+	if (((pcb_any_obj_t *)ptr2)->term != NULL) {
 		pcb_subc_t *subc = pcb_obj_parent_subc(ptr2);
 		if (subc != NULL)
 			elename = subc->refdes;
