@@ -41,9 +41,7 @@
 #include "dlg_drc.h"
 #include "hid_actions.h"
 #include "compat_nls.h"
-#include "obj_all.h"
-#include "obj_pinvia_draw.h"
-#include "obj_pad_draw.h"
+#include "obj_pstk_draw.h"
 #include "obj_rat_draw.h"
 #include "obj_line_draw.h"
 #include "obj_arc_draw.h"
@@ -57,10 +55,6 @@
 
 #include "../src_plugins/lib_gtk_config/hid_gtk_conf.h"
 
-/** \file   dlg_drc.c
-    \brief  Implementation of \ref pcb_gtk_drcwin_t DRC dialog.
- */
-
 #define VIOLATION_PIXMAP_PIXEL_SIZE   100
 #define VIOLATION_PIXMAP_PIXEL_BORDER 5
 #define VIOLATION_PIXMAP_PCB_SIZE     PCB_MIL_TO_COORD (100)
@@ -79,7 +73,6 @@ static void drc_close_cb(GtkButton *button, gpointer data)
 	gtk_widget_hide(p->drc_window);
 }
 
-/** A (*GtkCallback) function */
 static void destroy_widget(GtkWidget * widget, gpointer data)
 {
 	gtk_widget_destroy(widget);
@@ -107,6 +100,7 @@ static void unset_found_flags(int AndDraw)
 	int change = 0;
 
 #warning subc TODO: use data.h for this
+#if 0
 	PCB_VIA_LOOP(PCB->Data);
 	{
 		if (PCB_FLAG_TEST(flag, via)) {
@@ -141,6 +135,7 @@ static void unset_found_flags(int AndDraw)
 		PCB_END_LOOP;
 	}
 	PCB_END_LOOP;
+#endif
 	PCB_RAT_LOOP(PCB->Data);
 	{
 		if (PCB_FLAG_TEST(flag, line)) {
@@ -190,7 +185,6 @@ static void unset_found_flags(int AndDraw)
 	}
 }
 
-/** A (*GtkCallback) function */
 static void unselect_widget(GtkWidget * widget, gpointer data)
 {
 	gtk_widget_set_state(widget, GTK_STATE_NORMAL);
@@ -221,16 +215,16 @@ void row_clicked_cb(GtkWidget * widget, GdkEvent * event, GhidDrcViolation * vio
 			void *ptr1, *ptr2, *ptr3;
 
 			found_type = pcb_search_obj_by_id(PCB->Data, &ptr1, &ptr2, &ptr3, object_id, object_type);
-			if (found_type == PCB_TYPE_NONE) {
+			if (found_type == PCB_OBJ_VOID) {
 				pcb_message(PCB_MSG_WARNING, _("Object ID %i identified during DRC was not found. Stale DRC window?\n"), object_id);
 				continue;
 			}
 			pcb_undo_add_obj_to_flag(ptr2);
 			PCB_FLAG_SET(PCB_FLAG_FOUND, (pcb_any_obj_t *) ptr2);
 			switch (violation->object_type_list[i]) {
-			case PCB_TYPE_LINE:
-			case PCB_TYPE_ARC:
-			case PCB_TYPE_POLY:
+			case PCB_OBJ_LINE:
+			case PCB_OBJ_ARC:
+			case PCB_OBJ_POLY:
 				pcb_layervis_change_group_vis(pcb_layer_id(PCB->Data, (pcb_layer_t *) ptr1), pcb_true, pcb_true);
 			}
 			pcb_draw_obj((pcb_any_obj_t *)ptr2);
@@ -260,14 +254,8 @@ enum {
 static GObjectClass *ghid_drc_violation_parent_class = NULL;
 
 
-/*! \brief GObject finalise handler
- *
- *  \par Function Description
- *  Just before the GhidDrcViolation GObject is finalized, free our
- *  allocated data, and then chain up to the parent's finalize handler.
- *
- *  \param [in] widget  The GObject being finalized.
- */
+/* Just before the GhidDrcViolation GObject is finalized, free our
+   allocated data, and then chain up to the parent's finalize handler. */
 static void ghid_drc_violation_finalize(GObject * object)
 {
 	GhidDrcViolation *violation = GHID_DRC_VIOLATION(object);
@@ -286,18 +274,6 @@ typedef struct object_list {
 	int *type_list;
 } object_list;
 
-/*! \brief GObject property setter function
- *
- *  \par Function Description
- *  Setter function for GhidDrcViolation's GObject properties,
- *  "settings-name" and "toplevel"
- *
- *  \param [in]  object       The GObject whose properties we are setting
- *  \param [in]  property_id  The numeric id. under which the property was
- *                            registered with g_object_class_install_property()
- *  \param [in]  value        The GValue the property is being set from
- *  \param [in]  pspec        A GParamSpec describing the property being set
- */
 static void ghid_drc_violation_set_property(GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
 {
 	GhidDrcViolation *violation = GHID_DRC_VIOLATION(object);
@@ -347,19 +323,6 @@ static void ghid_drc_violation_set_property(GObject * object, guint property_id,
 	}
 }
 
-
-/*! \brief GObject property getter function
- *
- *  \par Function Description
- *  Getter function for GhidDrcViolation's GObject properties,
- *  "settings-name" and "toplevel".
- *
- *  \param [in]  object       The GObject whose properties we are getting
- *  \param [in]  property_id  The numeric id. under which the property was
- *                            registered with g_object_class_install_property()
- *  \param [out] value        The GValue in which to return the value of the property
- *  \param [in]  pspec        A GParamSpec describing the property being got
- */
 static void ghid_drc_violation_get_property(GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
 {
 	switch (property_id) {
@@ -369,15 +332,6 @@ static void ghid_drc_violation_get_property(GObject * object, guint property_id,
 
 }
 
-
-/*! \brief GType class initialiser for GhidDrcViolation
- *
- *  \par Function Description
- *  GType class initialiser for GhidDrcViolation. We override our parent
- *  virtual class methods as needed and register our GObject properties.
- *
- *  \param [in]  klass       The GhidDrcViolationClass we are initialising
- */
 static void ghid_drc_violation_class_init(GhidViolationRendererClass * klass)
 {
 	GObjectClass *gobject_class = G_OBJECT_CLASS(klass);
@@ -407,15 +361,8 @@ static void ghid_drc_violation_class_init(GhidViolationRendererClass * klass)
 																	g_param_spec_pointer("object-list", "", "", G_PARAM_WRITABLE));
 }
 
-/*! \brief Function to retrieve GhidViolationRenderer's GType identifier.
- *
- *  \par Function Description
- *  Function to retrieve GhidViolationRenderer's GType identifier.
- *  Upon first call, this registers the GhidViolationRenderer in the GType system.
- *  Subsequently it returns the saved value from its first execution.
- *
- *  \return the GType identifier associated with GhidViolationRenderer.
- */
+/* Upon first call, register the GhidViolationRenderer in the GType system
+ * and cache. Subsequently return the cached value. */
 GType ghid_drc_violation_get_type()
 {
 	static GType ghid_drc_violation_type = 0;
@@ -462,7 +409,7 @@ GhidDrcViolation *ghid_drc_violation_new(pcb_drc_violation_t * violation)
 																					 "object-list", &obj_list, NULL);
 }
 
-/** Returns the DRC text string using markup feature. Should be freed with free() */
+/* Returns the DRC text string using markup feature. Should be freed with free() */
 static char *get_drc_violation_markup(GhidDrcViolation * violation)
 {
 	char *markup;
@@ -604,6 +551,7 @@ void ghid_drc_window_append_violation(pcb_gtk_drcwin_t *drcwin, pcb_gtk_common_t
 void ghid_drc_window_reset_message(pcb_gtk_drcwin_t *drcwin)
 {
 	drcwin->num_violations = 0;
+	gtk_container_foreach(GTK_CONTAINER(drcwin->drc_vbox), destroy_widget, NULL);
 }
 
 int ghid_drc_window_throw_dialog(pcb_gtk_drcwin_t *drcwin)
