@@ -33,7 +33,6 @@
 #include "layer.h"
 #include "wt_preview.h"
 #include "stub_draw.h"
-#include "const.h"
 
 typedef struct {
 	GtkDialog *dialog;
@@ -71,7 +70,7 @@ void pcb_gtk_dlg_fontsel(pcb_gtk_common_t *com, pcb_layer_t *txtly, pcb_text_t *
 		dlg_fontsel_global_latch = 1;
 	}
 	else {
-		if ((type != PCB_TYPE_TEXT) && (type != PCB_TYPE_ELEMENT_NAME))
+		if (type != PCB_OBJ_TEXT)
 			return;
 		if (!modal) {
 			pcb_message(PCB_MSG_ERROR, "text-targeted fontsel dialogs must be modal because of the global-var API on the txt object.\n");
