@@ -37,7 +37,6 @@
 #include "compat_misc.h"
 #include "compat_nls.h"
 #include "polygon.h"
-#include "obj_all.h"
 #include "board.h"
 #include "data.h"
 #include "conf_core.h"
@@ -309,7 +308,7 @@ static void prop_preview_init(void)
 	preview_pcb.MaxWidth = preview_pcb.MaxHeight = PCB_MIL_TO_COORD(1000);
 	pcb_layer_colors_from_conf(&preview_pcb);
 	pcb_font_create_default(&preview_pcb);
-	preview_pcb.ViaOn = 1;
+	preview_pcb.pstk_on = 1;
 
 	for (n = 0; n < pcb_max_layer; n++) {
 		preview_pcb.Data->Layer[n].meta.real.vis = 1;
@@ -321,18 +320,26 @@ static void prop_preview_init(void)
 	memcpy(&preview_pcb.LayerGroups, &old_pcb->LayerGroups, sizeof(old_pcb->LayerGroups));
 	preview_pcb.Data->LayerN = 1;
 	preview_pcb.Data->Layer[0].meta.real.grp = 0;
-	preview_pcb.LayerGroups.grp[0].type = PCB_LYT_COPPER | PCB_LYT_TOP;
+	preview_pcb.LayerGroups.grp[0].ltype = PCB_LYT_COPPER | PCB_LYT_TOP;
 	preview_pcb.LayerGroups.grp[0].lid[0] = 0;
 	preview_pcb.LayerGroups.grp[0].len = 1;
+	preview_pcb.LayerGroups.grp[0].parent.any = NULL;
+	preview_pcb.LayerGroups.grp[0].parent_type = PCB_PARENT_INVALID;
+	preview_pcb.LayerGroups.grp[0].type = PCB_OBJ_LAYERGRP;
 	preview_pcb.LayerGroups.len = 1;
 	PCB_SET_PARENT(preview_pcb.Data, board, &preview_pcb);
-	preview_pcb.Data->Layer[0].parent = preview_pcb.Data;
+	preview_pcb.Data->Layer[0].parent.data = preview_pcb.Data;
+	preview_pcb.Data->Layer[0].parent_type = PCB_PARENT_DATA;
+	preview_pcb.Data->Layer[0].type = PCB_OBJ_LAYER;
 
 #warning TODO: preview_pcb is never freed
 
+#warning padstack TODO: create a padstack
+#if 0
 	pcb_via_new(preview_pcb.Data,
 							PCB_MIL_TO_COORD(1000), PCB_MIL_TO_COORD(1000),
 							PCB_MIL_TO_COORD(50), PCB_MIL_TO_COORD(40), 0, PCB_MIL_TO_COORD(20), "", pcb_no_flags());
+#endif
 
 	pcb_line_new(preview_pcb.Data->Layer + 0,
 							 PCB_MIL_TO_COORD(1000), PCB_MIL_TO_COORD(1000),
