@@ -65,12 +65,13 @@
 #include "in_mouse.h"
 #include "in_keyboard.h"
 #include "wt_layersel.h"
+#include "tool.h"
 #include "../src_plugins/lib_gtk_config/lib_gtk_config.h"
 #include "../src_plugins/lib_gtk_config/hid_gtk_conf.h"
 #include "win_place.h"
 
-/*! \brief sync the menu checkboxes with actual pcb state */
-void ghid_update_toggle_flags(pcb_gtk_topwin_t *tw)
+/* sync the menu checkboxes with actual pcb state */
+void ghid_update_toggle_flags(pcb_gtk_topwin_t *tw, const char *cookie)
 {
 	ghid_main_menu_update_toggle_state(GHID_MAIN_MENU(tw->menu.menu_bar), menu_toggle_update_cb);
 }
@@ -122,7 +123,7 @@ gboolean ghid_idle_cb(void *topwin)
 {
 	pcb_gtk_topwin_t *tw = topwin;
 	if (conf_core.editor.mode == PCB_MODE_NO)
-		pcb_crosshair_set_mode(PCB_MODE_ARROW);
+		pcb_tool_select_by_id(PCB_MODE_ARROW);
 	tw->com->mode_cursor_main(conf_core.editor.mode);
 	if (tw->mode_btn.settings_mode != conf_core.editor.mode) {
 		ghid_mode_buttons_update();
@@ -181,14 +182,12 @@ void pcb_gtk_tw_notify_filename_changed(pcb_gtk_topwin_t *tw)
 	tw->com->window_set_name_label(PCB->Name);
 }
 
-/*! \brief Install menu bar and accelerator groups */
 void ghid_install_accel_groups(GtkWindow *window, pcb_gtk_topwin_t *tw)
 {
 	gtk_window_add_accel_group(window, ghid_main_menu_get_accel_group(GHID_MAIN_MENU(tw->menu.menu_bar)));
 	gtk_window_add_accel_group(window, pcb_gtk_route_style_get_accel_group(GHID_ROUTE_STYLE(tw->route_style_selector)));
 }
 
-/*! \brief Remove menu bar and accelerator groups */
 void ghid_remove_accel_groups(GtkWindow *window, pcb_gtk_topwin_t *tw)
 {
 	gtk_window_remove_accel_group(window, ghid_main_menu_get_accel_group(GHID_MAIN_MENU(tw->menu.menu_bar)));
@@ -234,13 +233,12 @@ void pcb_gtk_tw_layer_vis_update(pcb_gtk_topwin_t *tw)
 	pcb_gtk_layersel_vis_update(&tw->layersel);
 }
 
-/*! \brief Called when user clicks OK on route style dialog */
+/* Called when user clicks OK on route style dialog */
 void pcb_gtk_tw_route_styles_edited_cb(pcb_gtk_topwin_t *tw)
 {
-	ghid_main_menu_install_route_style_selector(GHID_MAIN_MENU(tw->menu.menu_bar), GHID_ROUTE_STYLE(tw->route_style_selector));
+#warning TODO: generate a route styles changed event
 }
 
-
 /*
  * ---------------------------------------------------------------
  * Top window
@@ -565,10 +563,10 @@ void ghid_create_pcb_widgets(pcb_gtk_topwin_t *tw, GtkWidget *in_top_window)
 
 	ghid_build_pcb_top_window(tw);
 	ghid_install_accel_groups(GTK_WINDOW(tw->com->top_window), tw);
-	ghid_update_toggle_flags(tw);
+	ghid_update_toggle_flags(tw, NULL);
 
 	pcb_gtk_icons_init(GTK_WINDOW(tw->com->top_window));
-	pcb_crosshair_set_mode(PCB_MODE_ARROW);
+	pcb_tool_select_by_id(PCB_MODE_ARROW);
 	ghid_mode_buttons_update();
 }
 
