@@ -36,12 +36,12 @@
 #include <gdk/gdkkeysyms.h>
 #include "in_mouse.h"
 
-#include "const.h"
 #include "board.h"
 #include "action_helper.h"
 #include "crosshair.h"
 #include "conf_core.h"
 #include "undo.h"
+#include "tool.h"
 
 #include "in_keyboard.h"
 #include "bu_status_line.h"
