@@ -95,8 +95,8 @@ pcb_bool pcb_gtk_coords_pcb2event(const pcb_gtk_view_t *v, pcb_coord_t pcb_x, pc
 
 pcb_bool pcb_gtk_coords_event2pcb(const pcb_gtk_view_t *v, int event_x, int event_y, pcb_coord_t * pcb_x, pcb_coord_t * pcb_y)
 {
-	*pcb_x = EVENT_TO_PCB_X(v, event_x);
-	*pcb_y = EVENT_TO_PCB_Y(v, event_y);
+	*pcb_x = pcb_round(EVENT_TO_PCB_X(v, event_x));
+	*pcb_y = pcb_round(EVENT_TO_PCB_Y(v, event_y));
 
 	return pcb_true;
 }
@@ -155,6 +155,7 @@ void pcb_gtk_zoom_view_rel(pcb_gtk_view_t *v, pcb_coord_t center_x, pcb_coord_t
 	ghid_zoom_view_abs(v, center_x, center_y, v->coord_per_px * factor);
 }
 
+#warning TODO: remove this and make the side-correct version the default (rename that to this short name); check when looking from the bottom: library window, drc window
 void pcb_gtk_zoom_view_win(pcb_gtk_view_t *v, pcb_coord_t x1, pcb_coord_t y1, pcb_coord_t x2, pcb_coord_t y2)
 {
 	double xf, yf;
@@ -164,11 +165,29 @@ void pcb_gtk_zoom_view_win(pcb_gtk_view_t *v, pcb_coord_t x1, pcb_coord_t y1, pc
 
 	xf = (x2 - x1) / v->canvas_width;
 	yf = (y2 - y1) / v->canvas_height;
+	v->coord_per_px = (xf > yf ? xf : yf);
 
 	v->x0 = x1;
 	v->y0 = y1;
+
+	pcb_gtk_pan_common(v);
+}
+
+/* Side-correct version - long term this will be kept and the other is removed */
+static void pcb_gtk_zoom_view_win_side(pcb_gtk_view_t *v, pcb_coord_t x1, pcb_coord_t y1, pcb_coord_t x2, pcb_coord_t y2)
+{
+	double xf, yf;
+
+	if ((v->canvas_width < 1) || (v->canvas_height < 1))
+		return;
+
+	xf = (x2 - x1) / v->canvas_width;
+	yf = (y2 - y1) / v->canvas_height;
 	v->coord_per_px = (xf > yf ? xf : yf);
 
+	v->x0 = SIDE_X(conf_core.editor.view.flip_x ? x2 : x1);
+	v->y0 = SIDE_Y(conf_core.editor.view.flip_y ? y2 : y1);
+
 	pcb_gtk_pan_common(v);
 }
 
@@ -217,7 +236,7 @@ void pcb_gtk_pan_view_rel(pcb_gtk_view_t *v, pcb_coord_t dx, pcb_coord_t dy)
 
 /* ------------------------------------------------------------ */
 
-const char pcb_acts_zoom[] = "Zoom()\n" "Zoom(factor)";
+const char pcb_acts_zoom[] = "Zoom()\n" "Zoom(factor)\n" "Zoom(x1, y1, x2, y2)\n";
 
 const char pcb_acth_zoom[] = N_("Various zoom factor changes.");
 
@@ -251,6 +270,10 @@ about the actual resolution of most screens - resulting in an "actual
 size" board.  Similarly, a @var{factor} of 100 gives you a 10x actual
 size.
 
+@item @var{x1}, @var{y1}, @var{x2}, @var{y2}
+
+Zoom and pan to the box specified by the coords.
+
 @item ?
 
 Print the current zoom level in the message log (as an info line)
@@ -259,8 +282,6 @@ Print the current zoom level in the message log (as an info line)
 
 Note that zoom factors of zero are silently ignored.
 
-
-
 %end-doc */
 
 int pcb_gtk_zoom(pcb_gtk_view_t *vw, int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
@@ -268,14 +289,39 @@ int pcb_gtk_zoom(pcb_gtk_view_t *vw, int argc, const char **argv, pcb_coord_t x,
 	const char *vp;
 	double v;
 
-	if (argc > 1)
-		PCB_ACT_FAIL(zoom);
 
 	if (argc < 1) {
 		pcb_gtk_zoom_view_fit(vw);
 		return 0;
 	}
 
+	if (argc == 4) {
+		pcb_coord_t x1, y1, x2, y2;
+		pcb_bool succ;
+
+		x1 = pcb_get_value(argv[0], NULL, NULL, &succ);
+		if (!succ)
+			PCB_ACT_FAIL(zoom);
+
+		y1 = pcb_get_value(argv[1], NULL, NULL, &succ);
+		if (!succ)
+			PCB_ACT_FAIL(zoom);
+
+		x2 = pcb_get_value(argv[2], NULL, NULL, &succ);
+		if (!succ)
+			PCB_ACT_FAIL(zoom);
+
+		y2 = pcb_get_value(argv[3], NULL, NULL, &succ);
+		if (!succ)
+			PCB_ACT_FAIL(zoom);
+
+		pcb_gtk_zoom_view_win_side(vw, x1, y1, x2, y2);
+		return 0;
+	}
+
+	if (argc > 1)
+		PCB_ACT_FAIL(zoom);
+
 	vp = argv[0];
 	if (*vp == '?') {
 		pcb_message(PCB_MSG_INFO, "Current gtk zoom level: %f\n", vw->coord_per_px);
