@@ -26,11 +26,7 @@
  *
  */
 
-
-/*! \file <wt_coord_entry.c>
- *  \brief Implementation of pcb_gtk_coord_entry_t widget
- *  \par Description
- *  This widget is a modified spinbox for the user to enter
+/*  This widget is a modified spinbox for the user to enter
  *  pcb coords. It is assigned a default unit (for display),
  *  but this can be changed by the user by typing a new one
  *  or right-clicking on the box.
@@ -72,7 +68,7 @@ struct pcb_gtk_coord_entry_class_s {
 };
 
 /* SIGNAL HANDLERS */
-/*! \brief Callback for "Change Unit" menu click */
+/* Callback for "Change Unit" menu click */
 static void menu_item_activate_cb(GtkMenuItem * item, pcb_gtk_coord_entry_t * ce)
 {
 	const char *text = gtk_menu_item_get_label(item);
@@ -81,7 +77,7 @@ static void menu_item_activate_cb(GtkMenuItem * item, pcb_gtk_coord_entry_t * ce
 	g_signal_emit(ce, ghid_coord_entry_signals[UNIT_CHANGE_SIGNAL], 0, unit);
 }
 
-/*! \brief Callback for context menu creation */
+/* Callback for context menu creation */
 static void ghid_coord_entry_popup_cb(pcb_gtk_coord_entry_t * ce, GtkMenu * menu, gpointer data)
 {
 	int i, n;
@@ -90,7 +86,7 @@ static void ghid_coord_entry_popup_cb(pcb_gtk_coord_entry_t * ce, GtkMenu * menu
 
 	/* Build submenu */
 	n = pcb_get_n_units();
-	unit_list = get_unit_list();
+	unit_list = pcb_units;
 
 	submenu = gtk_menu_new();
 	for (i = 0; i < n; ++i) {
@@ -111,21 +107,29 @@ static void ghid_coord_entry_popup_cb(pcb_gtk_coord_entry_t * ce, GtkMenu * menu
 	gtk_widget_show(menu_item);
 }
 
-/*! \brief Callback for user output */
+/* Callback for user output */
 static gboolean ghid_coord_entry_output_cb(pcb_gtk_coord_entry_t * ce, gpointer data)
 {
 	GtkAdjustment *adj = gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(ce));
 	double value = gtk_adjustment_get_value(adj);
-	char *text;
+	const char *orig;
+	char text[128], *suffix = NULL;
+	const pcb_unit_t *orig_unit;
 
-	text = pcb_strdup_printf("%.*f %s", ce->unit->default_prec, value, ce->unit->suffix);
+	orig = gtk_entry_get_text(GTK_ENTRY(ce));
+	strtod(orig, &suffix);
+	orig_unit = get_unit_struct(suffix);
+
+	if ((suffix != NULL) && (*suffix != '\0') && (orig_unit != NULL) && (ce->unit != orig_unit))
+		pcb_gtk_coord_entry_set_unit(ce, orig_unit);
+
+	pcb_snprintf(text, sizeof(text), "%.*f %s", ce->unit->default_prec, value, ce->unit->suffix);
 	gtk_entry_set_text(GTK_ENTRY(ce), text);
-	free(text);
 
 	return TRUE;
 }
 
-/*! \brief Callback for user input */
+/* Callback for user input */
 static gboolean ghid_coord_text_changed_cb(pcb_gtk_coord_entry_t * entry, gpointer data)
 {
 	const char *text;
@@ -145,7 +149,7 @@ static gboolean ghid_coord_text_changed_cb(pcb_gtk_coord_entry_t * entry, gpoint
 	return FALSE;
 }
 
-/*! \brief Callback for change in value (input or ^v clicks) */
+/* Callback for change in value (input or ^v clicks) */
 static gboolean ghid_coord_value_changed_cb(pcb_gtk_coord_entry_t * ce, gpointer data)
 {
 	GtkAdjustment *adj = gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(ce));
@@ -159,11 +163,6 @@ static gboolean ghid_coord_value_changed_cb(pcb_gtk_coord_entry_t * ce, gpointer
 	return FALSE;
 }
 
-/*! \brief Change the unit used by a coord entry
- *
- *  \param [in] ce         The entry to be acted on
- *  \parin [in] new_unit   The new unit to be used
- */
 static void ghid_coord_entry_change_unit(pcb_gtk_coord_entry_t * ce, const pcb_unit_t * new_unit)
 {
 	double climb_rate = 0.0;
