@@ -37,6 +37,7 @@
 #include "layer_vis.h"
 #include "board.h"
 #include "data.h"
+#include "event.h"
 #include "conf_core.h"
 #include "hid_actions.h"
 #include "math_helper.h"
@@ -206,7 +207,8 @@ static void ensure_visible_current(pcb_gtk_layersel_t *ls)
 /* Toggle a virtual layer: flip the bool */
 static int vis_virt(pcb_gtk_ls_lyr_t *lsl, int toggle, int *is_on)
 {
-	pcb_bool *b = (pcb_bool *)((char *)PCB + lsl->virt_data);
+	const pcb_menu_layers_t *ml = &pcb_menu_layers[lsl->virt_idx];
+	pcb_bool *b = (pcb_bool *)((char *)PCB + ml->vis_offs);
 	if (toggle)
 		*b = !*b;
 	*is_on = *b;
@@ -215,7 +217,7 @@ static int vis_virt(pcb_gtk_ls_lyr_t *lsl, int toggle, int *is_on)
 
 static int vis_ui(pcb_gtk_ls_lyr_t *lsl, int toggle, int *is_on)
 {
-	pcb_layer_t *l = &pcb_uilayer.array[lsl->virt_data];
+	pcb_layer_t *l = &pcb_uilayer.array[lsl->vis_ui_idx];
 	if (toggle)
 		l->meta.real.vis = !l->meta.real.vis;
 	*is_on = l->meta.real.vis;
@@ -236,11 +238,19 @@ static int ev_lyr_no_select(pcb_gtk_ls_lyr_t *lsl, int do_select)
 	return -1; /* layer can not be selected ever */
 }
 
-static int ev_lyr_select_rat(pcb_gtk_ls_lyr_t *lsl, int do_select)
+static int ev_lyr_select_virt(pcb_gtk_ls_lyr_t *lsl, int do_select)
 {
+	const pcb_menu_layers_t *ml = &pcb_menu_layers[lsl->virt_idx];
+	pcb_bool *b;
+
+	if ((ml->select_name == NULL) || (ml->sel_offs == 0))
+		return -1; /* can not be selected */
+
 	if (do_select)
-		lsl->lsg->ls->no_copper_sel = PCB->RatOn = PCB->RatDraw = 1;
-	return PCB->RatDraw;
+		pcb_hid_actionl("SelectLayer", ml->select_name, NULL);
+
+	b = (pcb_bool *)((char *)PCB + ml->sel_offs);
+	return *b;
 }
 
 static gboolean group_vis_press_cb(GtkWidget *widget, GdkEvent *event, pcb_gtk_ls_grp_t *lsg)
@@ -259,6 +269,7 @@ static gboolean group_vis_press_cb(GtkWidget *widget, GdkEvent *event, pcb_gtk_l
 				layersel_lyr_vis_sync(&lsg->layer[n]);
 			group_vis_sync(lsg);
 			ls->com->invalidate_all();
+			pcb_event(PCB_EVENT_LAYERVIS_CHANGED, NULL);
 			break;
 		case 3:
 			if (lsg->grp != &lsg->ls->grp_virt) {
@@ -318,6 +329,7 @@ static gboolean layer_vis_press_cb(GtkWidget *widget, GdkEvent *event, pcb_gtk_l
 			}
 
 			ls->com->invalidate_all();
+			pcb_event(PCB_EVENT_LAYERVIS_CHANGED, NULL);
 			layer_popup(ls, event->button.button, lsl->lid, lsl->lsg->grp);
 			break;
 	}
@@ -339,6 +351,7 @@ static gboolean layer_select_press_cb(GtkWidget *widget, GdkEvent *event, pcb_gt
 				ls->com->invalidate_all();
 				pcb_gtk_layersel_vis_update(ls); /* need to do a full redraw because of side effects of special layer selections (e.g. Rats') */
 			}
+			pcb_event(PCB_EVENT_LAYERVIS_CHANGED, NULL);
 			layer_popup(ls, event->button.button, lsl->lid, lsl->lsg->grp);
 			break;
 	}
@@ -373,7 +386,7 @@ static gboolean group_open_press_cb(GtkWidget *widget, GdkEvent *event, pcb_gtk_
 	return group_any_press_cb(widget, event, lsg, 1);
 }
 
-/** Wrap w so that clicks on it are triggering a callback */
+/* Wrap w so that clicks on it are triggering a callback */
 static GtkWidget *wrap_bind_click(GtkWidget *w, GCallback cb, void *cb_data)
 {
 	GtkWidget *event_box = gtk_event_box_new();
@@ -402,7 +415,7 @@ static const char *lyr_color(pcb_layer_id_t lid)
 
 static const char * const grp_color(pcb_layergrp_t *g)
 {
-	hardwired_colors(g->type);
+	hardwired_colors(g->ltype);
 	/* normal mechanism: first layer's color or yellow */
 	if (g->len == 0) return "#ffff00";
 	return lyr_color(g->lid[0]);
@@ -555,23 +568,6 @@ static GtkWidget *build_group_real(pcb_gtk_layersel_t *ls, pcb_gtk_ls_grp_t *lsg
 }
 
 /*** Layer selector widget building function ***/
-typedef struct {
-	const char *name;
-	char * const*force_color;
-	int (*ev_vis)(pcb_gtk_ls_lyr_t *lsl, int toggle, int *is_on);
-	int (*ev_selected)(pcb_gtk_ls_lyr_t *lsl, int do_select);
-	int virt_data;
-} virt_layers_t;
-
-static const virt_layers_t virts[] = {
-	{ "Subcircuits",&conf_core.appearance.color.subc,              vis_virt, ev_lyr_no_select,  offsetof(pcb_board_t, SubcOn) },
-	{ "Subc. parts",&conf_core.appearance.color.subc,              vis_virt, ev_lyr_no_select,  offsetof(pcb_board_t, SubcPartsOn) },
-	{ "Pstk. marks",&conf_core.appearance.color.padstackmark,      vis_virt, ev_lyr_no_select,  offsetof(pcb_board_t, padstack_mark_on) },
-	{ "Holes",      &conf_core.appearance.color.pin,               vis_virt, ev_lyr_no_select,  offsetof(pcb_board_t, hole_on) },
-	{ "Far side",   &conf_core.appearance.color.invisible_objects, vis_virt, ev_lyr_no_select,  offsetof(pcb_board_t, InvisibleObjectsOn) },
-	{ "Rats",       &conf_core.appearance.color.rat,               vis_virt, ev_lyr_select_rat, offsetof(pcb_board_t, RatOn) },
-};
-
 static void layersel_populate(pcb_gtk_layersel_t *ls)
 {
 	GtkWidget *spring;
@@ -580,7 +576,7 @@ static void layersel_populate(pcb_gtk_layersel_t *ls)
 
 	for(gid = 0; gid < pcb_max_group(PCB); gid++) {
 		pcb_layergrp_t *g = &PCB->LayerGroups.grp[gid];
-		if (g->type & PCB_LYT_SUBSTRATE)
+		if (g->ltype & PCB_LYT_SUBSTRATE)
 			continue;
 		gtk_box_pack_start(GTK_BOX(ls->grp_box), build_group_real(ls, &ls->grp[gid], g), FALSE, FALSE, 0);
 	}
@@ -589,16 +585,17 @@ static void layersel_populate(pcb_gtk_layersel_t *ls)
 
 	{ /* build hardwired virtual layers */
 		pcb_gtk_ls_grp_t *lsg = &ls->lsg_virt;
+		const pcb_menu_layers_t *ml;
 
-		ls->grp_virt.len = sizeof(virts) / sizeof(virts[0]);
+		for(ml = pcb_menu_layers, ls->grp_virt.len = 0; ml->name != NULL; ml++) ls->grp_virt.len++;
 		lsg->layer = calloc(sizeof(pcb_gtk_ls_lyr_t), ls->grp_virt.len);
 		gtk_box_pack_start(GTK_BOX(ls->grp_box), build_group_start(ls, lsg, "Virtual", 0, &ls->grp_virt), FALSE, FALSE, 0);
 
 		for(n = 0; n < ls->grp_virt.len; n++) {
-			gtk_box_pack_start(GTK_BOX(lsg->layers), build_layer(lsg, &lsg->layer[n], virts[n].name, -1, (const char *const *)virts[n].force_color), FALSE, FALSE, 1);
-			lsg->layer[n].ev_selected = virts[n].ev_selected;
-			lsg->layer[n].ev_vis = virts[n].ev_vis;
-			lsg->layer[n].virt_data = virts[n].virt_data;
+			gtk_box_pack_start(GTK_BOX(lsg->layers), build_layer(lsg, &lsg->layer[n], pcb_menu_layers[n].name, -1, (const char *const *)pcb_menu_layers[n].force_color), FALSE, FALSE, 1);
+			lsg->layer[n].ev_selected = ev_lyr_select_virt;
+			lsg->layer[n].ev_vis = vis_virt;
+			lsg->layer[n].virt_idx = n;
 			lsg->layer[n].lid = ls->grp_virt.lid[n] = -1;
 		}
 
@@ -619,7 +616,7 @@ static void layersel_populate(pcb_gtk_layersel_t *ls)
 			gtk_box_pack_start(GTK_BOX(lsg->layers), build_layer(lsg, &lsg->layer[n], pcb_uilayer.array[n].name, -1, &pcb_uilayer.array[n].meta.real.color), FALSE, FALSE, 1);
 			lsg->layer[g].ev_selected = ev_lyr_no_select;
 			lsg->layer[g].ev_vis = vis_ui;
-			lsg->layer[g].virt_data = n;
+			lsg->layer[g].vis_ui_idx = n;
 			lsg->layer[g].lid = -1;
 			g++;
 		}
