@@ -46,13 +46,9 @@
 #include "draw.h"
 #include "move.h"
 #include "rotate.h"
-#include "obj_all.h"
 #include "macro.h"
 
-
-/** \file   wt_preview.c
-    \brief  Implementation of \ref pcb_gtk_preview_t widget.
- */
+static void get_ptr(pcb_gtk_preview_t *preview, pcb_coord_t *cx, pcb_coord_t *cy, gint *xp, gint *yp);
 
 static void preview_set_view(pcb_gtk_preview_t * preview)
 {
@@ -102,25 +98,16 @@ enum {
 
 static GObjectClass *ghid_preview_parent_class = NULL;
 
-/** GObject constructed
-    Initialises the preview object once it is constructed.
-    Chains up in case the parent class wants to do anything too.
-
-    \param object   The preview object
- */
+/* Initialises the preview object once it is constructed. Chains up in case
+   the parent class wants to do anything too. object is the preview object. */
 static void ghid_preview_constructed(GObject * object)
 {
-	/* chain up to the parent class */
 	if (G_OBJECT_CLASS(ghid_preview_parent_class)->constructed != NULL)
 		G_OBJECT_CLASS(ghid_preview_parent_class)->constructed(object);
 }
 
-/** GObject finalise handler
-    Just before the pcb_gtk_preview_t GObject is finalized, free our
-    allocated data, and then chain up to the parent's finalize handler.
-
-    \param widget   The GObject being finalized.
- */
+/* Just before the pcb_gtk_preview_t GObject is finalized, free our
+   allocated data, and then chain up to the parent's finalize handler. */
 static void ghid_preview_finalize(GObject * object)
 {
 	pcb_gtk_preview_t *preview = PCB_GTK_PREVIEW(object);
@@ -131,17 +118,6 @@ static void ghid_preview_finalize(GObject * object)
 	G_OBJECT_CLASS(ghid_preview_parent_class)->finalize(object);
 }
 
-/** GObject property setter function
-
-    Setter function for pcb_gtk_preview_t's GObject properties,
-    "settings-name" and "toplevel".
-
-    \param object        The GObject whose properties we are setting
-    \param property_id   The numeric id. under which the property was
-                         registered with g_object_class_install_property()
-    \param value         The GValue the property is being set from
-    \param pspec         A GParamSpec describing the property being set
- */
 static void ghid_preview_set_property(GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
 {
 	pcb_gtk_preview_t *preview = PCB_GTK_PREVIEW(object);
@@ -185,16 +161,6 @@ static void ghid_preview_set_property(GObject * object, guint property_id, const
 	}
 }
 
-/** GObject property getter function
-    Getter function for pcb_gtk_preview_t's GObject properties,
-    "settings-name" and "toplevel".
-
-    \param       object       The GObject whose properties we are getting
-    \param       property_id  The numeric id. under which the property was
-                              registered with g_object_class_install_property()
-    \param [out] value        The GValue in which to return the value of the property
-    \param       pspec        A GParamSpec describing the property being got
- */
 static void ghid_preview_get_property(GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
 {
 	switch (property_id) {
@@ -203,7 +169,7 @@ static void ghid_preview_get_property(GObject * object, guint property_id, GValu
 	}
 }
 
-/** Converter: set up a pinout expose and use the generic preview expose call */
+/* Converter: set up a pinout expose and use the generic preview expose call */
 static gboolean ghid_preview_expose(GtkWidget * widget, pcb_gtk_expose_t * ev)
 {
 	pcb_gtk_preview_t *preview = PCB_GTK_PREVIEW(widget);
@@ -230,12 +196,7 @@ static gboolean ghid_preview_expose(GtkWidget * widget, pcb_gtk_expose_t * ev)
 	return FALSE;
 }
 
-/** GType class initialiser for pcb_gtk_preview_t
-    We override our parent virtual class methods as needed and register
-    our GObject properties.
-
-    \param klass    The pcb_gtk_preview_class_t we are initialising
- */
+/* Override parent virtual class methods as needed and register GObject properties. */
 static void ghid_preview_class_init(pcb_gtk_preview_class_t * klass)
 {
 	GObjectClass *gobject_class = G_OBJECT_CLASS(klass);
@@ -297,18 +258,6 @@ static gboolean preview_configure_event_cb(GtkWidget * w, GdkEventConfigure * ev
 	return TRUE;
 }
 
-static void get_ptr(pcb_gtk_preview_t * preview, pcb_coord_t * cx, pcb_coord_t * cy, gint * xp, gint * yp)
-{
-	gdkc_window_get_pointer(GTK_WIDGET(preview), xp, yp, NULL);
-#undef SIDE_X
-#undef SIDE_Y
-#define SIDE_X(x) x
-#define SIDE_Y(y) y
-	*cx = EVENT_TO_PCB_X(&preview->view, *xp);
-	*cy = EVENT_TO_PCB_Y(&preview->view, *yp);
-#undef SIDE_X
-#undef SIDE_Y
-}
 
 static gboolean button_press(GtkWidget * w, pcb_hid_cfg_mod_t btn)
 {
@@ -325,7 +274,7 @@ static gboolean button_press(GtkWidget * w, pcb_hid_cfg_mod_t btn)
 				gtk_widget_queue_draw(w);
 		}
 		break;
-	case PCB_MB_RIGHT:
+	case PCB_MB_MIDDLE:
 		preview->view.panning = 1;
 		preview->grabx = cx;
 		preview->graby = cy;
@@ -378,7 +327,7 @@ static gboolean preview_button_release_cb(GtkWidget * w, GdkEventButton * ev, gp
 	get_ptr(preview, &cx, &cy, &wx, &wy);
 
 	switch (ghid_mouse_button(ev->button)) {
-	case PCB_MB_RIGHT:
+	case PCB_MB_MIDDLE:
 		preview->view.panning = 0;
 		if (((time(NULL) - preview->grabt) < 2) && (preview->grabmot < 4))
 			if ((preview->mouse_cb != NULL) && (preview->mouse_cb(w, PCB_HID_MOUSE_POPUP, cx, cy)))
@@ -544,12 +493,31 @@ GtkWidget *pcb_gtk_preview_board_new(pcb_gtk_common_t *com, pcb_gtk_init_drawing
 
 void pcb_gtk_preview_board_zoomto(pcb_gtk_preview_t *p, pcb_coord_t x1, pcb_coord_t y1, pcb_coord_t x2, pcb_coord_t y2, int canvas_width, int canvas_height)
 {
+	pcb_coord_t tmp;
+
 	/* just in case the size has changed meanwhile */
 	p->view.height = PCB->MaxHeight;
 	p->view.width = PCB->MaxWidth;
 	p->view.canvas_width = canvas_width;
 	p->view.canvas_height = canvas_height;
 
+	x1 = SIDE_X(x1);
+	y1 = SIDE_Y(y1);
+	x2 = SIDE_X(x2);
+	y2 = SIDE_Y(y2);
+
+	if (x1 > x2) {
+		tmp = x1;
+		x1 = x2;
+		x2 = tmp;
+	}
+
+	if (y1 > y2) {
+		tmp = y1;
+		y1 = y2;
+		y2 = tmp;
+	}
+
 	pcb_gtk_zoom_view_win(&p->view, x1, y1, x2, y2);
 }
 
@@ -568,3 +536,17 @@ void pcb_gtk_preview_get_natsize(pcb_gtk_preview_t * preview, int *width, int *h
 	*width = preview->w_pixels;
 	*height = preview->h_pixels;
 }
+
+/* Has to be at the end since it undef's SIDE_X */
+static void get_ptr(pcb_gtk_preview_t *preview, pcb_coord_t *cx, pcb_coord_t *cy, gint *xp, gint *yp)
+{
+	gdkc_window_get_pointer(GTK_WIDGET(preview), xp, yp, NULL);
+#undef SIDE_X
+#undef SIDE_Y
+#define SIDE_X(x) x
+#define SIDE_Y(y) y
+	*cx = EVENT_TO_PCB_X(&preview->view, *xp);
+	*cy = EVENT_TO_PCB_Y(&preview->view, *yp);
+#undef SIDE_X
+#undef SIDE_Y
+}
