@@ -162,6 +162,9 @@ void gtkhid_do_export(pcb_hid_attr_val_t *options)
 	ghid_create_pcb_widgets(&ghidgui->topwin, gport->top_window);
 	pcb_gtk_drcwin_init(&ghidgui->drcwin);     /* DRC window is not built, yet    */
 
+	/* assume pcb_gui is us */
+	pcb_gui->hid_cfg = ghidgui->topwin.ghid_cfg;
+
 	gport->mouse.drawing_area = ghidgui->topwin.drawing_area;
 	gport->drawing_area = ghidgui->topwin.drawing_area;
 	gport->mouse.top_window = ghidgui->common.top_window;
@@ -178,12 +181,7 @@ void gtkhid_do_export(pcb_hid_attr_val_t *options)
 
 	ghid_interface_input_signals_connect();
 
-	/* These are needed to make sure the @layerpick and @layerview menus
-	 * are properly initialized and synchronized with the current PCB.
-	 */
 	pcb_gtk_tw_layer_buttons_update(&ghidgui->topwin);
-	ghid_main_menu_install_route_style_selector
-		(GHID_MAIN_MENU(ghidgui->topwin.menu.menu_bar), GHID_ROUTE_STYLE(ghidgui->topwin.route_style_selector));
 
 	if (conf_hid_gtk.plugins.hid_gtk.listen)
 		pcb_gtk_create_listener();
@@ -192,11 +190,17 @@ void gtkhid_do_export(pcb_hid_attr_val_t *options)
 
 	pcb_event(PCB_EVENT_GUI_INIT, NULL);
 
+	/* Make sure drawing area has keyboard focus so that keys are handled
+	   while the mouse cursor is over the top window or children widgets,
+	   before first entering the drawing area */
+	gtk_widget_grab_focus(gport->drawing_area);
+
 	gtk_main();
 	pcb_hid_cfg_keys_uninit(&ghid_keymap);
 	gtkhid_end();
 
 	ghidgui->gui_is_up = 0;
+	pcb_gui->hid_cfg = NULL;
 }
 
 static void ghid_do_exit(pcb_hid_t * hid)
@@ -207,7 +211,7 @@ static void ghid_do_exit(pcb_hid_t * hid)
 	/* Create top level window for routines that will need top_window
 	   |  before ghid_create_pcb_widgets() is called.
 	 */
-void gtkhid_parse_arguments(int *argc, char ***argv)
+int gtkhid_parse_arguments(int *argc, char ***argv)
 {
 	GtkWidget *window;
 
@@ -254,7 +258,11 @@ void gtkhid_parse_arguments(int *argc, char ***argv)
 	 * We also don't want locale set if no ENABLE_NLS to keep "C" LC_NUMERIC.
 	 */
 	gtk_disable_setlocale();
-	gtk_init(argc, argv);
+
+	if (!gtk_init_check(argc, argv)) {
+		fprintf(stderr, "gtk_init_check() fail - maybe $DISPLAY not set or X/GUI not accessible?\n");
+		return 1; /* recoverable error - try another HID */
+	}
 
 
 	gport = &ghid_port;
@@ -279,6 +287,7 @@ void gtkhid_parse_arguments(int *argc, char ***argv)
 	wplc_place(WPLC_TOP, window);
 
 	gtk_widget_show_all(gport->top_window);
+	return 0;
 }
 
 static void ghid_calibrate(double xval, double yval)
@@ -286,7 +295,7 @@ static void ghid_calibrate(double xval, double yval)
 	printf(_("ghid_calibrate() -- not implemented\n"));
 }
 
-static void ghid_set_crosshair(int x, int y, int action)
+static void ghid_set_crosshair(pcb_coord_t x, pcb_coord_t y, int action)
 {
 	int offset_x, offset_y;
 
@@ -445,12 +454,23 @@ static void ghid_propedit_add_stat(void *pe, const char *propname, void *propctx
 /* Create a new menu by path */
 static int ghid_remove_menu(const char *menu_path)
 {
-	return pcb_hid_cfg_remove_menu(ghidgui->topwin.ghid_cfg, menu_path, ghid_remove_menu_widget, NULL);
+	return pcb_hid_cfg_remove_menu(ghidgui->topwin.ghid_cfg, menu_path, ghid_remove_menu_widget, ghidgui->topwin.menu.menu_bar);
 }
 
-static void ghid_create_menu(const char *menu_path, const char *action, const char *mnemonic, const char *accel, const char *tip, const char *cookie)
+static int ghid_remove_menu_node(lht_node_t *node)
 {
-	pcb_hid_cfg_create_menu(ghidgui->topwin.ghid_cfg, menu_path, action, mnemonic, accel, tip, cookie, ghid_create_menu_widget, &ghidgui->topwin.menu);
+	return pcb_hid_cfg_remove_menu_node(ghidgui->topwin.ghid_cfg, node, ghid_remove_menu_widget, ghidgui->topwin.menu.menu_bar);
+}
+
+static void ghid_create_menu(const char *menu_path, const pcb_menu_prop_t *props)
+{
+	pcb_hid_cfg_create_menu(ghidgui->topwin.ghid_cfg, menu_path, props, ghid_create_menu_widget, &ghidgui->topwin.menu);
+}
+
+static void ghid_update_menu_checkbox(const char *cookie)
+{
+	if (ghidgui->hid_active)
+		ghid_update_toggle_flags(&ghidgui->topwin, cookie);
 }
 
 static int ghid_usage(const char *topic)
@@ -550,6 +570,8 @@ void ghid_glue_hid_init(pcb_hid_t *dst)
 
 	dst->create_menu = ghid_create_menu;
 	dst->remove_menu = ghid_remove_menu;
+	dst->remove_menu_node = ghid_remove_menu_node;
+	dst->update_menu_checkbox = ghid_update_menu_checkbox;
 
 	dst->usage = ghid_usage;
 }
