@@ -23,7 +23,9 @@
  *    lead developer: email to pcb-rnd (at) igor2.repo.hu
  *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
+#include "config.h"
 
+#include <stdio.h>
 #include <stdlib.h>
 #include "plugins.h"
 
@@ -35,5 +37,6 @@ void pplg_uninit_lib_hid_gl(void)
 
 int pplg_init_lib_hid_gl(void)
 {
+	PCB_API_CHK_VER;
 	return 0;
 }
