@@ -93,44 +93,11 @@ static void list_poly_cb(void *ctx, pcb_board_t *pcb, pcb_layer_t *layer, pcb_po
 	list_obj(ctx, pcb, layer, (pcb_any_obj_t *)poly);
 }
 
-static void list_via_cb(void *ctx, pcb_board_t *pcb, pcb_pin_t *via)
-{
-	list_obj(ctx, pcb, NULL, (pcb_any_obj_t *)via);
-}
-
 static void list_pstk_cb(void *ctx, pcb_board_t *pcb, pcb_pstk_t *ps)
 {
 	list_obj(ctx, pcb, NULL, (pcb_any_obj_t *)ps);
 }
 
-static void list_epin_cb(void *ctx, pcb_board_t *pcb, pcb_element_t *element, pcb_pin_t *pin)
-{
-	pcb_netmap_t *map = ctx;
-
-	if (htpp_get(&map->o2n, pin) != NULL)
-		return;
-
-	map->curr_net = pcb_netlist_find_net4pin(map->pcb, pin);
-	if (map->curr_net == NULL)
-		map->curr_net = alloc_net(map);
-
-	pcb_lookup_conn_by_obj(map, (pcb_any_obj_t *)pin, 0, found);
-}
-
-static void list_epad_cb(void *ctx, pcb_board_t *pcb, pcb_element_t *element, pcb_pad_t *pad)
-{
-	pcb_netmap_t *map = ctx;
-
-	if (htpp_get(&map->o2n, pad) != NULL)
-		return;
-
-	map->curr_net = pcb_netlist_find_net4pad(map->pcb, pad);
-	if (map->curr_net == NULL)
-		map->curr_net = alloc_net(map);
-
-	pcb_lookup_conn_by_obj(map, (pcb_any_obj_t *)pad, 0, found);
-}
-
 int list_subc_cb(void *ctx, pcb_board_t *pcb, pcb_subc_t *subc, int enter)
 {
 	PCB_PADSTACK_LOOP(subc->data) {
@@ -149,21 +116,14 @@ int pcb_netmap_init(pcb_netmap_t *map, pcb_board_t *pcb)
 	map->dyn_nets = 0;
 	map->pcb = pcb;
 
-/* step 1: find known nets (from pins and pads) */
+/* step 1: find known nets (from terminals) */
 	pcb_loop_all(PCB, map,
 		NULL, /* layer */
 		NULL, /* line */
 		NULL, /* arc */
 		NULL, /* text */
 		NULL, /* poly */
-		NULL, /* element */
-		NULL, /* eline */
-		NULL, /* earc */
-		NULL, /* etext */
-		list_epin_cb,
-		list_epad_cb,
 		list_subc_cb, /* subc */
-		NULL, /* via */
 		NULL /* pstk */
 	);
 
@@ -174,14 +134,7 @@ int pcb_netmap_init(pcb_netmap_t *map, pcb_board_t *pcb)
 		list_arc_cb,
 		NULL, /* text */
 		list_poly_cb,
-		NULL, /* element */
-		NULL, /* eline */
-		NULL, /* earc */
-		NULL, /* etext */
-		NULL, /* pin */
-		NULL, /* pad */
 		NULL, /* subc */
-		list_via_cb,
 		list_pstk_cb
 	);
 	return 0;
@@ -224,5 +177,6 @@ void pplg_uninit_lib_netmap(void)
 
 int pplg_init_lib_netmap(void)
 {
+	PCB_API_CHK_VER;
 	return 0;
 }
