@@ -42,7 +42,7 @@
 #include "hid.h"
 
 #define VALID_TYPE(obj) (((obj)->type == PCB_OBJ_ARC)  || ((obj)->type == PCB_OBJ_LINE))
-#define CONT_TYPE (PCB_TYPE_LINE | PCB_TYPE_ARC)
+#define CONT_TYPE (PCB_OBJ_LINE | PCB_OBJ_ARC)
 
 #define NEAR(x1, x2, y1, y2) (((x1-x2)*(x1-x2) + (y1-y2)*(y1-y2)) < 4)
 
