@@ -1,11 +1,9 @@
-/*!
- * \file polycombine.c
+/*
+ * PolyCombine plug-in for PCB.
  *
- * \brief PolyCombine plug-in for PCB.
+ * Copyright (C) 2010 Peter Clifton <pcjc2@cam.ac.uk>
  *
- * \author Copyright (C) 2010 Peter Clifton <pcjc2@cam.ac.uk>
- *
- * \copyright Licensed under the terms of the GNU General Public
+ * Licensed under the terms of the GNU General Public
  * License, version 2.
  *
  * Ported to pcb-rnd by Tibor 'Igor2' Palinkas in 2016.
@@ -101,8 +99,7 @@ struct poly_tree {
 	poly_tree *next;
 };
 
-/*!
- * <pre>
+/*
  *                      ______
  *  ___________________|_  P6 |             +P1 ____ +P6
  * | P1                | |    |              |
@@ -118,7 +115,6 @@ struct poly_tree {
  * |                     |
  * |_____________________|
  *
- * </pre>
  * As we encounter each polygon, it gets a record. We need to check
  * whether it contains any of the polygons existing in our tree. If
  * it does, it will become the parent of them. (Check breadth first).
@@ -358,6 +354,7 @@ void pplg_uninit_polycombine(void)
 #include "dolists.h"
 int pplg_init_polycombine(void)
 {
+	PCB_API_CHK_VER;
 	PCB_REGISTER_ACTIONS(polycombine_action_list, polycombine_cookie);
 	return 0;
 }
