@@ -26,7 +26,6 @@
 
 #include <ctype.h>
 #include "config.h"
-#include "const.h"
 #include "data.h"
 #include "props.h"
 #include "propsel.h"
@@ -99,6 +98,47 @@ static void map_common(void *ctx, pcb_any_obj_t *obj)
 #warning TODO: flags
 }
 
+static int map_board_cb(void *ctx, pcb_board_t *pcb)
+{
+	if (!propedit_board)
+		return 0;
+
+	map_add_prop(ctx, "p/board/name",   String, pcb->Name);
+	map_add_prop(ctx, "p/board/width", pcb_coord_t, pcb->MaxWidth);
+	map_add_prop(ctx, "p/board/height", pcb_coord_t, pcb->MaxHeight);
+	map_attr(ctx, &pcb->Attributes);
+	return 0;
+}
+
+static int map_layer_cb(void *ctx, pcb_board_t *pcb, pcb_layer_t *layer, int enter)
+{
+	if (!layer->propedit)
+		return 0;
+
+	map_add_prop(ctx, "p/layer/name", String, layer->name);
+	map_add_prop(ctx, "p/layer/comb/negative", int, !!(layer->comb & PCB_LYC_SUB));
+	map_add_prop(ctx, "p/layer/comb/auto", int, !!(layer->comb & PCB_LYC_AUTO));
+#warning layer TODO: when hardwired layer colors are gone make color selection possible
+#if 0
+	if (!layer->is_bound) {
+		map_add_prop(ctx, "p/layer/color", int, layer->meta.real.color);
+		map_add_prop(ctx, "p/layer/selected_color", int, layer->meta.real.selected_color);
+	}
+#endif
+	map_attr(ctx, &layer->Attributes);
+	return 0;
+}
+
+static int map_layergrp_cb(void *ctx, pcb_board_t *pcb, pcb_layergrp_t *grp)
+{
+	if (!grp->propedit)
+		return 0;
+
+	map_add_prop(ctx, "p/layer_group/name", String, grp->name);
+	map_attr(ctx, &grp->Attributes);
+	return 0;
+}
+
 static void map_line_cb(void *ctx, pcb_board_t *pcb, pcb_layer_t *layer, pcb_line_t *line)
 {
 	map_chk_skip(ctx, line);
@@ -139,73 +179,6 @@ static void map_poly_cb(void *ctx, pcb_board_t *pcb, pcb_layer_t *layer, pcb_pol
 	map_add_prop(ctx, "p/trace/clearance", pcb_coord_t, poly->Clearance/2);
 }
 
-static void map_eline_cb(void *ctx, pcb_board_t *pcb, pcb_element_t *element, pcb_line_t *line)
-{
-	map_chk_skip(ctx, line);
-	map_line_cb(ctx, pcb, NULL, line);
-	map_attr(ctx, &line->Attributes);
-	map_common(ctx, (pcb_any_obj_t *)line);
-}
-
-static void map_earc_cb(void *ctx, pcb_board_t *pcb, pcb_element_t *element, pcb_arc_t *arc)
-{
-	map_chk_skip(ctx, arc);
-	map_arc_cb(ctx, pcb, NULL, arc);
-	map_attr(ctx, &arc->Attributes);
-	map_common(ctx, (pcb_any_obj_t *)arc);
-}
-
-static void map_etext_cb(void *ctx, pcb_board_t *pcb, pcb_element_t *element, pcb_text_t *text)
-{
-	map_chk_skip(ctx, text);
-	map_text_cb(ctx, pcb, NULL, text);
-	map_attr(ctx, &text->Attributes);
-	map_common(ctx, (pcb_any_obj_t *)text);
-}
-
-static void map_epin_cb(void *ctx, pcb_board_t *pcb, pcb_element_t *element, pcb_pin_t *pin)
-{
-	map_chk_skip(ctx, pin);
-	map_add_prop(ctx, "p/pin/thickness", pcb_coord_t, pin->Thickness);
-	map_add_prop(ctx, "p/pin/clearance", pcb_coord_t, pin->Clearance/2);
-	map_add_prop(ctx, "p/pin/mask",      pcb_coord_t, pin->Mask);
-	map_add_prop(ctx, "p/pin/hole",      pcb_coord_t, pin->DrillingHole);
-	map_attr(ctx, &pin->Attributes);
-	map_common(ctx, (pcb_any_obj_t *)pin);
-}
-
-static void map_epad_cb(void *ctx, pcb_board_t *pcb, pcb_element_t *element, pcb_pad_t *pad)
-{
-	map_chk_skip(ctx, pad);
-	map_add_prop(ctx, "p/pad/mask",      pcb_coord_t, pad->Mask);
-	map_attr(ctx, &pad->Attributes);
-	map_common(ctx, (pcb_any_obj_t *)pad);
-}
-
-static void map_subc_cb_(void *ctx, pcb_board_t *pcb, pcb_subc_t *subc)
-{
-	map_chk_skip(ctx, subc);
-	map_attr(ctx, &subc->Attributes);
-	map_common(ctx, (pcb_any_obj_t *)subc);
-}
-
-static int map_subc_cb(void *ctx, pcb_board_t *pcb, pcb_subc_t *subc, int enter)
-{
-	map_subc_cb_(ctx, pcb, subc);
-	return 0;
-}
-
-static void map_via_cb(void *ctx, pcb_board_t *pcb, pcb_pin_t *via)
-{
-	map_chk_skip(ctx, via);
-	map_add_prop(ctx, "p/via/thickness", pcb_coord_t, via->Thickness);
-	map_add_prop(ctx, "p/via/clearance", pcb_coord_t, via->Clearance/2);
-	map_add_prop(ctx, "p/via/mask",      pcb_coord_t, via->Mask);
-	map_add_prop(ctx, "p/via/hole",      pcb_coord_t, via->DrillingHole);
-	map_attr(ctx, &via->Attributes);
-	map_common(ctx, (pcb_any_obj_t *)via);
-}
-
 static void map_pstk_cb(void *ctx, pcb_board_t *pcb, pcb_pstk_t *ps)
 {
 	pcb_pstk_proto_t *proto;
@@ -227,18 +200,58 @@ static void map_pstk_cb(void *ctx, pcb_board_t *pcb, pcb_pstk_t *ps)
 	map_common(ctx, (pcb_any_obj_t *)ps);
 }
 
+static void map_subc_cb_(void *ctx, pcb_board_t *pcb, pcb_subc_t *msubc)
+{
+	PCB_ARC_ALL_LOOP(msubc->data); {
+		if (pcb_subc_part_editable(pcb, arc))
+			map_arc_cb(ctx, pcb, layer, arc);
+	} PCB_ENDALL_LOOP;
+	PCB_LINE_ALL_LOOP(msubc->data); {
+		if (pcb_subc_part_editable(pcb, line))
+			map_line_cb(ctx, pcb, layer, line);
+	} PCB_ENDALL_LOOP;
+	PCB_POLY_ALL_LOOP(msubc->data); {
+		if (pcb_subc_part_editable(pcb, polygon))
+			map_poly_cb(ctx, pcb, layer, polygon);
+	} PCB_ENDALL_LOOP;
+	PCB_TEXT_ALL_LOOP(msubc->data); {
+		if (pcb_subc_part_editable(pcb, text))
+			map_text_cb(ctx, pcb, layer, text);
+	} PCB_ENDALL_LOOP;
+	PCB_PADSTACK_LOOP(msubc->data); {
+		if (pcb_subc_part_editable(pcb, padstack))
+			map_pstk_cb(ctx, pcb, padstack);
+	} PCB_END_LOOP;
+	PCB_SUBC_LOOP(msubc->data); {
+		if (pcb_subc_part_editable(pcb, subc))
+			map_subc_cb_(ctx, pcb, subc);
+	} PCB_END_LOOP;
+	map_chk_skip(ctx, msubc);
+	map_attr(ctx, &msubc->Attributes);
+	map_common(ctx, (pcb_any_obj_t *)msubc);
+}
+
+static int map_subc_cb(void *ctx, pcb_board_t *pcb, pcb_subc_t *subc, int enter)
+{
+	map_subc_cb_(ctx, pcb, subc);
+	return 0;
+}
+
 void pcb_propsel_map_core(htsp_t *props)
 {
 	map_ctx_t ctx;
+	pcb_layergrp_id_t gid;
 
 	ctx.props = props;
 	
 	pcb_loop_all(PCB, &ctx,
-		NULL, map_line_cb, map_arc_cb, map_text_cb, map_poly_cb,
-		NULL, map_eline_cb, map_earc_cb, map_etext_cb, map_epin_cb, map_epad_cb,
+		map_layer_cb, map_line_cb, map_arc_cb, map_text_cb, map_poly_cb,
 		map_subc_cb,
-		map_via_cb, map_pstk_cb
+		map_pstk_cb
 	);
+	for(gid = 0; gid < PCB->LayerGroups.len; gid++)
+		map_layergrp_cb(&ctx, PCB, &PCB->LayerGroups.grp[gid]);
+	map_board_cb(&ctx, PCB);
 }
 
 /*******************/
@@ -269,6 +282,7 @@ static void set_attr(set_ctx_t *st, pcb_attribute_list_t *list)
 	if (!PCB_FLAG_TEST(PCB_FLAG_SELECTED, obj)) return;
 
 #define DONE { st->set_cnt++; pcb_undo_restore_serial(); return; }
+#define DONE0 { st->set_cnt++; pcb_undo_restore_serial(); return 0; }
 
 static int set_common(set_ctx_t *st, pcb_any_obj_t *obj)
 {
@@ -276,6 +290,79 @@ static int set_common(set_ctx_t *st, pcb_any_obj_t *obj)
 	return 0;
 }
 
+static int brd_resize(pcb_coord_t w, pcb_coord_t h)
+{
+	pcb_board_resize(w, h);
+	return 1;
+}
+
+static void set_board_cb(void *ctx, pcb_board_t *pcb)
+{
+	set_ctx_t *st = (set_ctx_t *)ctx;
+	const char *pn = st->name + 8;
+
+	if (!propedit_board)
+		return;
+
+	if (st->is_attr) {
+		set_attr(st, &pcb->Attributes);
+		return;
+	}
+
+	if ((strcmp(pn, "name") == 0) &&
+	    (pcb_board_change_name(pcb_strdup(st->value)))) DONE;
+
+	if (st->c_valid && (strcmp(pn, "width") == 0) &&
+	    brd_resize(st->c, PCB->MaxHeight)) DONE;
+
+	if (st->c_valid && (strcmp(pn, "height") == 0) &&
+	    brd_resize(PCB->MaxWidth,st->c)) DONE;
+
+	pcb_message(PCB_MSG_ERROR, "This property can not be changed from the property editor.\n");
+}
+
+
+static int set_layer_cb(void *ctx, pcb_board_t *pcb, pcb_layer_t *layer, int enter)
+{
+	set_ctx_t *st = (set_ctx_t *)ctx;
+	const char *pn = st->name + 8;
+
+	if (!layer->propedit)
+		return 0;
+
+	if (st->is_attr) {
+		set_attr(st, &layer->Attributes);
+		return 0;
+	}
+
+	if ((strcmp(pn, "name") == 0) &&
+	    (pcb_layer_rename_(layer, pcb_strdup(st->value)) == 0)) DONE0;
+
+	pcb_message(PCB_MSG_ERROR, "This property can not be changed from the property editor.\n");
+	return 0;
+}
+
+
+static void set_layergrp_cb(void *ctx, pcb_board_t *pcb, pcb_layergrp_t *grp)
+{
+	set_ctx_t *st = (set_ctx_t *)ctx;
+	const char *pn = st->name + 14;
+
+	if (!grp->propedit)
+		return;
+
+	if (st->is_attr) {
+		set_attr(st, &grp->Attributes);
+		return;
+	}
+
+	if ((strcmp(pn, "name") == 0) &&
+	    (pcb_layergrp_rename_(grp, pcb_strdup(st->value)) == 0)) DONE;
+
+	pcb_message(PCB_MSG_ERROR, "This property can not be changed from the property editor.\n");
+}
+
+
 static void set_line_cb(void *ctx, pcb_board_t *pcb, pcb_layer_t *layer, pcb_line_t *line)
 {
 	set_ctx_t *st = (set_ctx_t *)ctx;
@@ -291,10 +378,10 @@ static void set_line_cb(void *ctx, pcb_board_t *pcb, pcb_layer_t *layer, pcb_lin
 	if (set_common(st, (pcb_any_obj_t *)line)) return;
 
 	if (st->is_trace && st->c_valid && (strcmp(pn, "thickness") == 0) &&
-	    pcb_chg_obj_1st_size(PCB_TYPE_LINE, layer, line, NULL, st->c, st->c_absolute)) DONE;
+	    pcb_chg_obj_1st_size(PCB_OBJ_LINE, layer, line, NULL, st->c, st->c_absolute)) DONE;
 
 	if (st->is_trace && st->c_valid && (strcmp(pn, "clearance") == 0) &&
-	    pcb_chg_obj_clear_size(PCB_TYPE_LINE, layer, line, NULL, st->c*2, st->c_absolute)) DONE;
+	    pcb_chg_obj_clear_size(PCB_OBJ_LINE, layer, line, NULL, st->c*2, st->c_absolute)) DONE;
 }
 
 static void set_arc_cb(void *ctx, pcb_board_t *pcb, pcb_layer_t *layer, pcb_arc_t *arc)
@@ -312,24 +399,24 @@ static void set_arc_cb(void *ctx, pcb_board_t *pcb, pcb_layer_t *layer, pcb_arc_
 	if (set_common(st, (pcb_any_obj_t *)arc)) return;
 
 	if (st->is_trace && st->c_valid && (strcmp(pn, "thickness") == 0) &&
-	    pcb_chg_obj_1st_size(PCB_TYPE_ARC, layer, arc, NULL, st->c, st->c_absolute)) DONE;
+	    pcb_chg_obj_1st_size(PCB_OBJ_ARC, layer, arc, NULL, st->c, st->c_absolute)) DONE;
 
 	if (st->is_trace && st->c_valid && (strcmp(pn, "clearance") == 0) &&
-	    pcb_chg_obj_clear_size(PCB_TYPE_ARC, layer, arc, NULL, st->c*2, st->c_absolute)) DONE;
+	    pcb_chg_obj_clear_size(PCB_OBJ_ARC, layer, arc, NULL, st->c*2, st->c_absolute)) DONE;
 
 	pn = st->name + 6;
 
 	if (!st->is_trace && st->c_valid && (strcmp(pn, "width") == 0) &&
-	    pcb_chg_obj_radius(PCB_TYPE_ARC, layer, arc, NULL, 0, st->c, st->c_absolute)) DONE;
+	    pcb_chg_obj_radius(PCB_OBJ_ARC, layer, arc, NULL, 0, st->c, st->c_absolute)) DONE;
 
 	if (!st->is_trace && st->c_valid && (strcmp(pn, "height") == 0) &&
-	    pcb_chg_obj_radius(PCB_TYPE_ARC, layer, arc, NULL, 1, st->c, st->c_absolute)) DONE;
+	    pcb_chg_obj_radius(PCB_OBJ_ARC, layer, arc, NULL, 1, st->c, st->c_absolute)) DONE;
 
 	if (!st->is_trace && st->d_valid && (strcmp(pn, "angle/start") == 0) &&
-	    pcb_chg_obj_angle(PCB_TYPE_ARC, layer, arc, NULL, 0, st->d, st->d_absolute)) DONE;
+	    pcb_chg_obj_angle(PCB_OBJ_ARC, layer, arc, NULL, 0, st->d, st->d_absolute)) DONE;
 
 	if (!st->is_trace && st->d_valid && (strcmp(pn, "angle/delta") == 0) &&
-	    pcb_chg_obj_angle(PCB_TYPE_ARC, layer, arc, NULL, 1, st->d, st->d_absolute)) DONE;
+	    pcb_chg_obj_angle(PCB_OBJ_ARC, layer, arc, NULL, 1, st->d, st->d_absolute)) DONE;
 }
 
 static void set_text_cb_any(void *ctx, pcb_board_t *pcb, int type, void *layer_or_element, pcb_text_t *text)
@@ -373,7 +460,7 @@ static void set_text_cb_any(void *ctx, pcb_board_t *pcb, int type, void *layer_o
 
 static void set_text_cb(void *ctx, pcb_board_t *pcb, pcb_layer_t *layer, pcb_text_t *text)
 {
-	set_text_cb_any(ctx, pcb, PCB_TYPE_TEXT, layer, text);
+	set_text_cb_any(ctx, pcb, PCB_OBJ_TEXT, layer, text);
 }
 
 
@@ -387,7 +474,7 @@ static void set_poly_cb(void *ctx, pcb_board_t *pcb, pcb_layer_t *layer, pcb_pol
 	if (set_common(st, (pcb_any_obj_t *)poly)) return;
 
 	if (st->is_trace && st->c_valid && (strcmp(pn, "clearance") == 0) &&
-	    pcb_chg_obj_clear_size(PCB_TYPE_POLY, layer, poly, NULL, st->c*2, st->c_absolute)) DONE;
+	    pcb_chg_obj_clear_size(PCB_OBJ_POLY, layer, poly, NULL, st->c*2, st->c_absolute)) DONE;
 
 	if (st->is_attr) {
 		set_attr(st, &poly->Attributes);
@@ -395,132 +482,6 @@ static void set_poly_cb(void *ctx, pcb_board_t *pcb, pcb_layer_t *layer, pcb_pol
 	}
 }
 
-static void set_eline_cb(void *ctx, pcb_board_t *pcb, pcb_element_t *element, pcb_line_t *line)
-{
-	set_ctx_t *st = (set_ctx_t *)ctx;
-
-	set_chk_skip(st, line);
-
-	if (st->is_attr) {
-		set_attr(st, &line->Attributes);
-		return;
-	}
-
-	if (set_common(st, (pcb_any_obj_t *)line)) return;
-}
-
-static void set_earc_cb(void *ctx, pcb_board_t *pcb, pcb_element_t *element, pcb_arc_t *arc)
-{
-	set_ctx_t *st = (set_ctx_t *)ctx;
-
-	set_chk_skip(st, arc);
-
-	if (st->is_attr) {
-		set_attr(st, &arc->Attributes);
-		return;
-	}
-
-	if (set_common(st, (pcb_any_obj_t *)arc)) return;
-}
-
-static void set_etext_cb(void *ctx, pcb_board_t *pcb, pcb_element_t *element, pcb_text_t *text)
-{
-	set_text_cb_any(ctx, pcb, PCB_TYPE_ELEMENT_NAME, element, text);
-}
-
-static void set_epin_cb(void *ctx, pcb_board_t *pcb, pcb_element_t *element, pcb_pin_t *pin)
-{
-	set_ctx_t *st = (set_ctx_t *)ctx;
-	const char *pn = st->name + 6;
-
-	set_chk_skip(st, pin);
-
-	if (st->is_attr) {
-		set_attr(st, &pin->Attributes);
-		return;
-	}
-
-	if (set_common(st, (pcb_any_obj_t *)pin)) return;
-
-	if (st->c_valid && (strcmp(pn, "thickness") == 0) &&
-	    pcb_chg_obj_1st_size(PCB_TYPE_PIN, pin->Element, pin, NULL, st->c, st->c_absolute)) DONE;
-
-	if (st->c_valid && (strcmp(pn, "clearance") == 0) &&
-	    pcb_chg_obj_clear_size(PCB_TYPE_PIN, pin->Element, pin, NULL, st->c*2, st->c_absolute)) DONE;
-
-	if (st->c_valid && (strcmp(pn, "mask") == 0) &&
-	    pcb_chg_obj_mask_size(PCB_TYPE_PIN, pin->Element, pin, NULL, st->c, st->c_absolute)) DONE;
-
-	if (st->c_valid && (strcmp(pn, "hole") == 0) &&
-	    pcb_chg_obj_2nd_size(PCB_TYPE_PIN, pin->Element, pin, NULL, st->c, st->c_absolute, pcb_false)) DONE;
-}
-
-static void set_epad_cb(void *ctx, pcb_board_t *pcb, pcb_element_t *element, pcb_pad_t *pad)
-{
-	set_ctx_t *st = (set_ctx_t *)ctx;
-	const char *pn = st->name + 6;
-
-	set_chk_skip(st, pad);
-
-	if (st->is_attr) {
-		set_attr(st, &pad->Attributes);
-		return;
-	}
-
-	if (set_common(st, (pcb_any_obj_t *)pad)) return;
-
-	if (st->c_valid && (strcmp(pn, "mask") == 0) &&
-	    pcb_chg_obj_mask_size(PCB_TYPE_PAD, pad->Element, pad, NULL, st->c, st->c_absolute)) DONE;
-}
-
-static void set_subc_cb_(void *ctx, pcb_board_t *pcb, pcb_subc_t *subc)
-{
-	set_ctx_t *st = (set_ctx_t *)ctx;
-
-	set_chk_skip(st, subc);
-
-	if (set_common(st, (pcb_any_obj_t *)subc)) return;
-
-	if (st->is_attr) {
-		set_attr(st, &subc->Attributes);
-		return;
-	}
-}
-
-static int set_subc_cb(void *ctx, pcb_board_t *pcb, pcb_subc_t *subc, int enter)
-{
-	set_subc_cb_(ctx, pcb, subc);
-	return 0;
-}
-
-
-static void set_via_cb(void *ctx, pcb_board_t *pcb, pcb_pin_t *via)
-{
-	set_ctx_t *st = (set_ctx_t *)ctx;
-	const char *pn = st->name + 6;
-
-	set_chk_skip(st, via);
-
-	if (st->is_attr) {
-		set_attr(st, &via->Attributes);
-		return;
-	}
-
-	if (set_common(st, (pcb_any_obj_t *)via)) return;
-
-	if (st->c_valid && (strcmp(pn, "thickness") == 0) &&
-	    pcb_chg_obj_1st_size(PCB_TYPE_VIA, via, via, NULL, st->c, st->c_absolute)) DONE;
-
-	if (st->c_valid && (strcmp(pn, "clearance") == 0) &&
-	    pcb_chg_obj_clear_size(PCB_TYPE_VIA, via, via, NULL, st->c*2, st->c_absolute)) DONE;
-
-	if (st->c_valid && (strcmp(pn, "mask") == 0) &&
-	    pcb_chg_obj_mask_size(PCB_TYPE_VIA, via, via, NULL, st->c, st->c_absolute)) DONE;
-
-	if (st->c_valid && (strcmp(pn, "hole") == 0) &&
-	    pcb_chg_obj_2nd_size(PCB_TYPE_VIA, via, via, NULL, st->c, st->c_absolute, pcb_false)) DONE;
-}
-
 static void set_pstk_cb(void *ctx, pcb_board_t *pcb, pcb_pstk_t *ps)
 {
 	set_ctx_t *st = (set_ctx_t *)ctx;
@@ -542,13 +503,13 @@ static void set_pstk_cb(void *ctx, pcb_board_t *pcb, pcb_pstk_t *ps)
 	proto = pcb_pstk_get_proto(ps);
 
 	if (st->c_valid && (strcmp(pn, "clearance") == 0) &&
-	    pcb_chg_obj_clear_size(PCB_TYPE_PSTK, ps, ps, NULL, st->c, st->c_absolute)) DONE;
+	    pcb_chg_obj_clear_size(PCB_OBJ_PSTK, ps, ps, NULL, st->c*2, st->c_absolute)) DONE;
 	if (st->d_valid && (strcmp(pn, "rotation") == 0)) {
 		if (st->d_absolute) {
-			if (pcb_obj_rotate(PCB_TYPE_PSTK, ps, ps, NULL, ps->x, ps->y, st->d - ps->rot)) DONE;
+			if (pcb_obj_rotate(PCB_OBJ_PSTK, ps, ps, NULL, ps->x, ps->y, st->d - ps->rot)) DONE;
 		}
 		else {
-			if (pcb_obj_rotate(PCB_TYPE_PSTK, ps, ps, NULL, ps->x, ps->y, st->d)) DONE;
+			if (pcb_obj_rotate(PCB_OBJ_PSTK, ps, ps, NULL, ps->x, ps->y, st->d)) DONE;
 		}
 		return;
 	}
@@ -568,6 +529,51 @@ static void set_pstk_cb(void *ctx, pcb_board_t *pcb, pcb_pstk_t *ps)
 	    (pcb_pstk_proto_change_hole(proto, NULL, NULL, NULL, &i) == 0)) DONE;
 }
 
+static void set_subc_cb_(void *ctx, pcb_board_t *pcb, pcb_subc_t *ssubc)
+{
+	set_ctx_t *st = (set_ctx_t *)ctx;
+
+	PCB_ARC_ALL_LOOP(ssubc->data); {
+		if (pcb_subc_part_editable(pcb, arc))
+			set_arc_cb(ctx, pcb, layer, arc);
+	} PCB_ENDALL_LOOP;
+	PCB_LINE_ALL_LOOP(ssubc->data); {
+		if (pcb_subc_part_editable(pcb, line))
+			set_line_cb(ctx, pcb, layer, line);
+	} PCB_ENDALL_LOOP;
+	PCB_POLY_ALL_LOOP(ssubc->data); {
+		if (pcb_subc_part_editable(pcb, polygon))
+			set_poly_cb(ctx, pcb, layer, polygon);
+	} PCB_ENDALL_LOOP;
+	PCB_TEXT_ALL_LOOP(ssubc->data); {
+		if (pcb_subc_part_editable(pcb, text))
+			set_text_cb(ctx, pcb, layer, text);
+	} PCB_ENDALL_LOOP;
+	PCB_PADSTACK_LOOP(ssubc->data); {
+		if (pcb_subc_part_editable(pcb, padstack))
+			set_pstk_cb(ctx, pcb, padstack);
+	} PCB_END_LOOP;
+	PCB_SUBC_LOOP(ssubc->data); {
+		if (pcb_subc_part_editable(pcb, subc))
+			set_subc_cb_(ctx, pcb, subc);
+	} PCB_END_LOOP;
+
+	set_chk_skip(st, ssubc);
+
+	if (set_common(st, (pcb_any_obj_t *)ssubc)) return;
+
+	if (st->is_attr) {
+		set_attr(st, &ssubc->Attributes);
+		return;
+	}
+}
+
+static int set_subc_cb(void *ctx, pcb_board_t *pcb, pcb_subc_t *subc, int enter)
+{
+	set_subc_cb_(ctx, pcb, subc);
+	return 0;
+}
+
 /* use the callback if trc is true or prop matches a prefix or we are setting attributes, else NULL */
 #define MAYBE_PROP(trc, prefix, cb) \
 	(((ctx.is_attr) || (trc) || (strncmp(prop, (prefix), sizeof(prefix)-1) == 0) || (prop[0] == 'a')) ? (cb) : NULL)
@@ -580,6 +586,7 @@ int pcb_propsel_set(const char *prop, const char *value)
 	set_ctx_t ctx;
 	char *end;
 	const char *start;
+	pcb_layergrp_id_t gid;
 
 	/* sanity checks for invalid props */
 	if (prop[1] != '/')
@@ -607,21 +614,20 @@ int pcb_propsel_set(const char *prop, const char *value)
 	pcb_undo_save_serial();
 
 	pcb_loop_all(PCB, &ctx,
-		NULL,
+		MAYBE_PROP(0, "p/layer/", set_layer_cb),
 		MAYBE_PROP(ctx.is_trace, "p/line/", set_line_cb),
 		MAYBE_PROP(ctx.is_trace, "p/arc/", set_arc_cb),
 		MAYBE_PROP(0, "p/text/", set_text_cb),
 		MAYBE_PROP(ctx.is_trace, "p/poly/", set_poly_cb),
-		NULL,
-		MAYBE_ATTR(set_eline_cb),
-		MAYBE_ATTR(set_earc_cb),
-		MAYBE_PROP(0, "p/text/", set_etext_cb),
-		MAYBE_PROP(0, "p/pin/", set_epin_cb),
-		MAYBE_PROP(0, "p/pad/", set_epad_cb),
-		MAYBE_PROP(0, "p/subc/", set_subc_cb),
-		MAYBE_PROP(0, "p/via/", set_via_cb),
+		MAYBE_PROP(0, "p/", set_subc_cb),
 		MAYBE_PROP(0, "p/padstack/", set_pstk_cb)
 	);
+
+	for(gid = 0; gid < PCB->LayerGroups.len; gid++)
+		set_layergrp_cb(&ctx, PCB, &PCB->LayerGroups.grp[gid]);
+
+	set_board_cb(&ctx, PCB);
+
 	pcb_undo_inc_serial();
 	return ctx.set_cnt;
 }
@@ -667,36 +673,6 @@ static void del_poly_cb(void *ctx, pcb_board_t *pcb, pcb_layer_t *layer, pcb_pol
 	del_attr(ctx, &poly->Attributes);
 }
 
-static void del_eline_cb(void *ctx, pcb_board_t *pcb, pcb_element_t *element, pcb_line_t *line)
-{
-	map_chk_skip(ctx, line);
-	del_attr(ctx, &line->Attributes);
-}
-
-static void del_earc_cb(void *ctx, pcb_board_t *pcb, pcb_element_t *element, pcb_arc_t *arc)
-{
-	map_chk_skip(ctx, arc);
-	del_attr(ctx, &arc->Attributes);
-}
-
-static void del_etext_cb(void *ctx, pcb_board_t *pcb, pcb_element_t *element, pcb_text_t *text)
-{
-	map_chk_skip(ctx, text);
-	del_attr(ctx, &text->Attributes);
-}
-
-static void del_epin_cb(void *ctx, pcb_board_t *pcb, pcb_element_t *element, pcb_pin_t *pin)
-{
-	map_chk_skip(ctx, pin);
-	del_attr(ctx, &pin->Attributes);
-}
-
-static void del_epad_cb(void *ctx, pcb_board_t *pcb, pcb_element_t *element, pcb_pad_t *pad)
-{
-	map_chk_skip(ctx, pad);
-	del_attr(ctx, &pad->Attributes);
-}
-
 static void del_subc_cb_(void *ctx, pcb_board_t *pcb, pcb_subc_t *subc)
 {
 	map_chk_skip(ctx, subc);
@@ -709,12 +685,6 @@ static int del_subc_cb(void *ctx, pcb_board_t *pcb, pcb_subc_t *subc, int enter)
 	return 0;
 }
 
-static void del_via_cb(void *ctx, pcb_board_t *pcb, pcb_pin_t *via)
-{
-	map_chk_skip(ctx, via);
-	del_attr(ctx, &via->Attributes);
-}
-
 static void del_pstk_cb(void *ctx, pcb_board_t *pcb, pcb_pstk_t *ps)
 {
 	map_chk_skip(ctx, ps);
@@ -733,9 +703,8 @@ int pcb_propsel_del(const char *key)
 
 	pcb_loop_all(PCB, &st,
 		NULL, del_line_cb, del_arc_cb, del_text_cb, del_poly_cb,
-		NULL, del_eline_cb, del_earc_cb, del_etext_cb, del_epin_cb, del_epad_cb,
 		del_subc_cb,
-		del_via_cb, del_pstk_cb
+		del_pstk_cb
 	);
 	return st.del_cnt;
 }
