@@ -35,7 +35,7 @@ static int fnc_llen(int argc, pcb_qry_val_t *argv, pcb_qry_val_t *res)
 		return -1;
 	if (argv[0].type != PCBQ_VT_LST)
 		PCB_QRY_RET_INV(res);
-	PCB_QRY_RET_INT(res, pcb_objlist_length(&argv[0].data.lst));
+	PCB_QRY_RET_INT(res, vtp0_len(&argv[0].data.lst));
 }
 
 
