@@ -27,10 +27,12 @@
 #ifndef PCB_QUERY_H
 #define PCB_QUERY_H
 
-#include "obj_any.h"
+#include <genvector/vtp0.h>
 #include <genht/htsi.h>
 #include <genregex/regex_se.h>
 #include "fields_sphash.h"
+#include "obj_common.h"
+#include "layer.h"
 
 typedef struct pcb_qry_val_s pcb_qry_val_t;
 typedef struct pcb_query_iter_s  pcb_query_iter_t;
@@ -50,8 +52,8 @@ typedef enum pcb_qry_valtype_e {
 struct pcb_qry_val_s {
 	pcb_qry_valtype_t type;
 	union {
-		pcb_obj_t obj;
-		pcb_objlist_t lst;
+		pcb_any_obj_t *obj;
+		vtp0_t lst;
 		pcb_coord_t crd;
 		double dbl;
 		const char *str;
@@ -95,6 +97,7 @@ typedef enum {
 	PCBQ_DATA_REGEX,   /* leaf */
 	PCBQ_DATA_CONST,   /* leaf */
 	PCBQ_DATA_INVALID, /* leaf */
+	PCBQ_DATA_LYTC,    /* leaf */
 
 	PCBQ_nodetype_max
 } pcb_qry_nodetype_t;
@@ -111,6 +114,10 @@ struct pcb_qry_node_s {
 		pcb_query_iter_t *iter_ctx;
 		pcb_qry_node_t *children;   /* first child (NULL for a leaf node) */
 		pcb_qry_fnc_t fnc;
+		struct pcb_qry_lytc_s {
+			pcb_layer_type_t lyt;
+			pcb_layer_combining_t lyc;
+		} lytc;
 	} data;
 	union {                       /* field selection depends on ->type */
 		query_fields_keys_t fld;    /* field_sphash value from str */
@@ -128,14 +135,16 @@ void pcb_qry_dump_tree(const char *prefix, pcb_qry_node_t *top);
 
 void pcb_qry_set_input(const char *script);
 
-
 struct pcb_query_iter_s {
 	htsi_t names;         /* name->index hash */
 	int num_vars;
 
 	const char **vn;      /* pointers to the hash names so they can be indexed */
 	pcb_qry_val_t *lst;
-	pcb_obj_t **it;       /* iterator state for each variable - point into the correspoinding lst[] */
+
+	/* iterator state for each variable - point into the correspoinding lst[] */
+	vtp0_t **vects;
+	pcb_cardinal_t *idx;
 
 	int all_idx;          /* index of the "@" variable or -1 */
 };
