@@ -35,128 +35,105 @@
 #include "layer.h"
 #include "fields_sphash.h"
 #include "obj_pstk_inlines.h"
+#include "obj_subc_parent.h"
 
-#define APPEND(_ctx_, _type_, _obj_, _parenttype_, _parent_) \
-do { \
-	pcb_objlist_t *lst = (pcb_objlist_t *)_ctx_; \
-	pcb_obj_t *o = calloc(sizeof(pcb_obj_t), 1); \
-	o->type = _type_; \
-	o->data.any = _obj_; \
-	o->parent_type = _parenttype_; \
-	o->parent.any = _parent_; \
-	pcb_objlist_append(lst, o); \
-} while(0)
+#define APPEND(_lst_, _obj_) vtp0_append((vtp0_t *)_lst_, _obj_)
 
 static int list_layer_cb(void *ctx, pcb_board_t *pcb, pcb_layer_t *layer, int enter)
 {
 	if (enter)
-		APPEND(ctx, PCB_OBJ_LAYER, layer, PCB_PARENT_DATA, pcb->Data);
+		APPEND(ctx, layer);
 	return 0;
 }
 
 static void list_line_cb(void *ctx, pcb_board_t *pcb, pcb_layer_t *layer, pcb_line_t *line)
 {
-	APPEND(ctx, PCB_OBJ_LINE, line, PCB_PARENT_LAYER, layer);
+	APPEND(ctx, line);
 }
 
 static void list_arc_cb(void *ctx, pcb_board_t *pcb, pcb_layer_t *layer, pcb_arc_t *arc)
 {
-	APPEND(ctx, PCB_OBJ_ARC, arc, PCB_PARENT_LAYER, layer);
+	APPEND(ctx, arc);
 }
 
 static void list_text_cb(void *ctx, pcb_board_t *pcb, pcb_layer_t *layer, pcb_text_t *text)
 {
-	APPEND(ctx, PCB_OBJ_TEXT, text, PCB_PARENT_LAYER, layer);
+	APPEND(ctx, text);
 }
 
 static void list_poly_cb(void *ctx, pcb_board_t *pcb, pcb_layer_t *layer, pcb_poly_t *poly)
 {
-	APPEND(ctx, PCB_OBJ_POLY, poly, PCB_PARENT_LAYER, layer);
-}
-
-static int list_element_cb(void *ctx, pcb_board_t *pcb, pcb_element_t *element, int enter)
-{
-	if (enter)
-		APPEND(ctx, PCB_OBJ_ELEMENT, element, PCB_PARENT_DATA, pcb->Data);
-	return 0;
-}
-
-static int list_subc_cb(void *ctx, pcb_board_t *pcb, pcb_subc_t *subc, int enter)
-{
-	if (enter)
-		APPEND(ctx, PCB_OBJ_SUBC, subc, PCB_PARENT_DATA, pcb->Data);
-	return 0;
-}
-
-static void list_eline_cb(void *ctx, pcb_board_t *pcb, pcb_element_t *element, pcb_line_t *line)
-{
-	APPEND(ctx, PCB_OBJ_ELINE, line, PCB_PARENT_ELEMENT, element);
-}
-
-static void list_earc_cb(void *ctx, pcb_board_t *pcb, pcb_element_t *element, pcb_arc_t *arc)
-{
-	APPEND(ctx, PCB_OBJ_EARC, arc, PCB_PARENT_ELEMENT, element);
-}
-
-static void list_etext_cb(void *ctx, pcb_board_t *pcb, pcb_element_t *element, pcb_text_t *text)
-{
-	APPEND(ctx, PCB_OBJ_ETEXT, text, PCB_PARENT_ELEMENT, element);
-}
-
-static void list_epin_cb(void *ctx, pcb_board_t *pcb, pcb_element_t *element, pcb_pin_t *pin)
-{
-	APPEND(ctx, PCB_OBJ_PIN, pin, PCB_PARENT_ELEMENT, element);
-}
-
-static void list_epad_cb(void *ctx, pcb_board_t *pcb, pcb_element_t *element, pcb_pad_t *pad)
-{
-	APPEND(ctx, PCB_OBJ_PAD, pad, PCB_PARENT_ELEMENT, element);
-}
-
-static void list_via_cb(void *ctx, pcb_board_t *pcb, pcb_pin_t *via)
-{
-	APPEND(ctx, PCB_OBJ_VIA, via, PCB_PARENT_DATA, pcb->Data);
+	APPEND(ctx, poly);
 }
 
 static void list_pstk_cb(void *ctx, pcb_board_t *pcb, pcb_pstk_t *ps)
 {
-	APPEND(ctx, PCB_OBJ_PSTK, ps, PCB_PARENT_DATA, pcb->Data);
+	APPEND(ctx, ps);
 }
 
+static int list_subc_cb(void *ctx, pcb_board_t *pcb, pcb_subc_t *subc, int enter)
+{
+	
+	if (enter) {
+		pcb_data_t *data = subc->data;
+		APPEND(ctx, subc);
+
+		PCB_SUBC_LOOP(data);
+		{
+			list_subc_cb(ctx, pcb, subc, 1);
+		}
+		PCB_END_LOOP;
+		PCB_PADSTACK_LOOP(data);
+		{
+			list_pstk_cb(ctx, pcb, padstack);
+		}
+		PCB_END_LOOP;
+
+		PCB_ARC_ALL_LOOP(data);
+		{
+			list_arc_cb(ctx, pcb, layer, arc);
+		}
+		PCB_ENDALL_LOOP;
+		PCB_LINE_ALL_LOOP(data);
+		{
+			list_line_cb(ctx, pcb, layer, line);
+		}
+		PCB_ENDALL_LOOP;
+		PCB_TEXT_ALL_LOOP(data);
+		{
+			list_text_cb(ctx, pcb, layer, text);
+		}
+		PCB_ENDALL_LOOP;
+		PCB_POLY_ALL_LOOP(data);
+		{
+			list_poly_cb(ctx, pcb, layer, polygon);
+		}
+		PCB_ENDALL_LOOP;
+	}
+
+	return 0;
+}
+
+
 void pcb_qry_list_all(pcb_qry_val_t *lst, pcb_objtype_t mask)
 {
 	assert(lst->type == PCBQ_VT_LST);
+#warning layer TODO: make layer a real object with the common header
+#warning TODO: rather do rtree search here to avoid recursion
 	pcb_loop_all(PCB, &lst->data.lst,
-		(mask & PCB_OBJ_LAYER) ? list_layer_cb : NULL,
+		/*(mask & PCB_OBJ_LAYER) ? list_layer_cb :*/ NULL,
 		(mask & PCB_OBJ_LINE) ? list_line_cb : NULL,
 		(mask & PCB_OBJ_ARC) ? list_arc_cb : NULL,
 		(mask & PCB_OBJ_TEXT) ? list_text_cb : NULL,
 		(mask & PCB_OBJ_POLY) ? list_poly_cb : NULL,
-		(mask & PCB_OBJ_ELEMENT) ? list_element_cb : NULL,
-		(mask & PCB_OBJ_ELINE) ? list_eline_cb : NULL,
-		(mask & PCB_OBJ_EARC) ? list_earc_cb : NULL,
-		(mask & PCB_OBJ_ETEXT) ? list_etext_cb : NULL,
-		(mask & PCB_OBJ_PIN) ? list_epin_cb : NULL,
-		(mask & PCB_OBJ_PAD) ? list_epad_cb : NULL,
 		(mask & PCB_OBJ_SUBC) ? list_subc_cb : NULL,
-		(mask & PCB_OBJ_VIA) ? list_via_cb : NULL,
 		(mask & PCB_OBJ_PSTK) ? list_pstk_cb : NULL
 	);
 }
 
 void pcb_qry_list_free(pcb_qry_val_t *lst_)
 {
-	pcb_objlist_t *lst = &lst_->data.lst;
-
-	assert(lst_->type == PCBQ_VT_LST);
-
-	for(;;) {
-		pcb_obj_t *n = pcb_objlist_first(lst);
-		if (n == NULL)
-			break;
-		pcb_objlist_remove(n);
-		free(n);
-	}
+	vtp0_uninit(&lst_->data.lst);
 }
 
 int pcb_qry_list_cmp(pcb_qry_val_t *lst1, pcb_qry_val_t *lst2)
@@ -241,9 +218,68 @@ do { \
 	(s) = __res__; \
 } while(0)
 
-static int field_layer(pcb_obj_t *obj, pcb_qry_node_t *fld, pcb_qry_val_t *res)
+/* convert a layer type description and cache the result (overwriting the string in the field!) */
+#define fld2lyt_req(lyt, lyc, fld, idx) \
+do { \
+	pcb_qry_node_t *_f_; \
+	fld_nth_req(_f_, fld, idx); \
+	if (_f_->type != PCBQ_DATA_LYTC) { \
+		const char *_s2_; \
+		fld2str_req(_s2_, fld, 1); \
+		_f_->data.lytc = field_pstk_lyt(p, _s2_); \
+		_f_->type = PCBQ_DATA_LYTC; \
+	} \
+	lyt = _f_->data.lytc.lyt; \
+	lyc = _f_->data.lytc.lyc; \
+} while(0)
+
+#define COMMON_FIELDS(obj, fh1, fld, res) \
+do { \
+	if (fh1 == query_fields_a) { \
+		const char *s2; \
+		fld2str_req(s2, fld, 1); \
+		if (!PCB_OBJ_IS_CLASS(obj->type, PCB_OBJ_CLASS_OBJ)) \
+			PCB_QRY_RET_INV(res); \
+		PCB_QRY_RET_STR(res, pcb_attribute_get(&obj->Attributes, s2)); \
+	} \
+ \
+	if (fh1 == query_fields_ID) { \
+		if (!PCB_OBJ_IS_CLASS(obj->type, PCB_OBJ_CLASS_OBJ)) \
+			PCB_QRY_RET_INV(res); \
+		PCB_QRY_RET_INT(res, obj->ID); \
+	} \
+ \
+	if (fh1 == query_fields_bbox) { \
+		query_fields_keys_t fh2; \
+ \
+		if (!PCB_OBJ_IS_CLASS(obj->type, PCB_OBJ_CLASS_OBJ)) \
+			PCB_QRY_RET_INV(res); \
+ \
+		fld2hash_req(fh2, fld, 1); \
+		switch(fh2) { \
+			case query_fields_x1:     PCB_QRY_RET_INT(res, obj->BoundingBox.X1); \
+			case query_fields_y1:     PCB_QRY_RET_INT(res, obj->BoundingBox.Y1); \
+			case query_fields_x2:     PCB_QRY_RET_INT(res, obj->BoundingBox.X2); \
+			case query_fields_y2:     PCB_QRY_RET_INT(res, obj->BoundingBox.Y2); \
+			case query_fields_width:  PCB_QRY_RET_INT(res, obj->BoundingBox.X2 - obj->BoundingBox.X1); \
+			case query_fields_height: PCB_QRY_RET_INT(res, obj->BoundingBox.Y2 - obj->BoundingBox.Y1); \
+			case query_fields_area:   PCB_QRY_RET_DBL(res, (double)(obj->BoundingBox.Y2 - obj->BoundingBox.Y1) * (double)(obj->BoundingBox.X2 - obj->BoundingBox.X1)); \
+			default:; \
+		} \
+		PCB_QRY_RET_INV(res); \
+	} \
+ \
+	if (fh1 == query_fields_type) \
+		PCB_QRY_RET_INT(res, obj->type); \
+ \
+	if (fh1 == query_fields_subc) \
+	 return field_subc_obj(obj, fld->next, res); \
+ \
+} while(0)
+
+static int field_layer(pcb_any_obj_t *obj, pcb_qry_node_t *fld, pcb_qry_val_t *res)
 {
-	pcb_layer_t *l = obj->data.layer;
+	pcb_layer_t *l = (pcb_layer_t *)obj;
 	query_fields_keys_t fh1;
 
 	fld2hash_req(fh1, fld, 0);
@@ -251,7 +287,7 @@ static int field_layer(pcb_obj_t *obj, pcb_qry_node_t *fld, pcb_qry_val_t *res)
 	if (fh1 == query_fields_a) {
 		const char *s2;
 		fld2str_req(s2, fld, 1);
-		PCB_QRY_RET_STR(res, pcb_attribute_get(&l->meta.real.Attributes, s2));
+		PCB_QRY_RET_STR(res, pcb_attribute_get(&l->Attributes, s2));
 	}
 
 	if (fld->next != NULL)
@@ -270,12 +306,7 @@ static int field_layer(pcb_obj_t *obj, pcb_qry_node_t *fld, pcb_qry_val_t *res)
 
 static int field_layer_from_ptr(pcb_layer_t *l, pcb_qry_node_t *fld, pcb_qry_val_t *res)
 {
-	pcb_obj_t tmp;
-	tmp.type = PCB_OBJ_LAYER;
-	tmp.data.layer = l;
-	tmp.parent_type = PCB_PARENT_DATA;
-	tmp.parent.data = PCB->Data;
-	return field_layer(&tmp, fld, res);
+	return field_layer((pcb_any_obj_t *)l, fld, res);
 }
 
 /* process from .layer */
@@ -291,8 +322,7 @@ static int layer_of_obj(pcb_qry_node_t *fld, pcb_qry_val_t *res, pcb_layer_type_
 
 	if (s1 == NULL) {
 		res->type = PCBQ_VT_OBJ;
-		res->data.obj.type = PCB_OBJ_LAYER;
-		res->data.obj.data.layer = PCB->Data->Layer+id;
+		res->data.obj = (pcb_any_obj_t *)&(PCB->Data->Layer[id]);
 		return 0;
 	}
 
@@ -300,9 +330,9 @@ static int layer_of_obj(pcb_qry_node_t *fld, pcb_qry_val_t *res, pcb_layer_type_
 }
 
 
-static int field_line(pcb_obj_t *obj, pcb_qry_node_t *fld, pcb_qry_val_t *res)
+static int field_line(pcb_any_obj_t *obj, pcb_qry_node_t *fld, pcb_qry_val_t *res)
 {
-	pcb_line_t *l = obj->data.line;
+	pcb_line_t *l = (pcb_line_t *)obj;
 	query_fields_keys_t fh1;
 
 	fld2hash_req(fh1, fld, 0);
@@ -344,9 +374,9 @@ static int field_line(pcb_obj_t *obj, pcb_qry_node_t *fld, pcb_qry_val_t *res)
 	PCB_QRY_RET_INV(res);
 }
 
-static int field_arc(pcb_obj_t *obj, pcb_qry_node_t *fld, pcb_qry_val_t *res)
+static int field_arc(pcb_any_obj_t *obj, pcb_qry_node_t *fld, pcb_qry_val_t *res)
 {
-	pcb_arc_t *a = obj->data.arc;
+	pcb_arc_t *a = (pcb_arc_t *)obj;
 	query_fields_keys_t fh1, fh2;
 
 	fld2hash_req(fh1, fld, 0);
@@ -398,9 +428,9 @@ static int field_arc(pcb_obj_t *obj, pcb_qry_node_t *fld, pcb_qry_val_t *res)
 	PCB_QRY_RET_INV(res);
 }
 
-static int field_text(pcb_obj_t *obj, pcb_qry_node_t *fld, pcb_qry_val_t *res)
+static int field_text(pcb_any_obj_t *obj, pcb_qry_node_t *fld, pcb_qry_val_t *res)
 {
-	pcb_text_t *t = obj->data.text;
+	pcb_text_t *t = (pcb_text_t *)obj;
 	query_fields_keys_t fh1;
 
 	fld2hash_req(fh1, fld, 0);
@@ -433,9 +463,9 @@ static int field_text(pcb_obj_t *obj, pcb_qry_node_t *fld, pcb_qry_val_t *res)
 	PCB_QRY_RET_INV(res);
 }
 
-static int field_polygon(pcb_obj_t *obj, pcb_qry_node_t *fld, pcb_qry_val_t *res)
+static int field_polygon(pcb_any_obj_t *obj, pcb_qry_node_t *fld, pcb_qry_val_t *res)
 {
-	pcb_poly_t *p = obj->data.polygon;
+	pcb_poly_t *p = (pcb_poly_t *)obj;
 	query_fields_keys_t fh1;
 
 	fld2hash_req(fh1, fld, 0);
@@ -464,7 +494,7 @@ static int field_polygon(pcb_obj_t *obj, pcb_qry_node_t *fld, pcb_qry_val_t *res
 	PCB_QRY_RET_INV(res);
 }
 
-static int field_rat(pcb_obj_t *obj, pcb_qry_node_t *fld, pcb_qry_val_t *res)
+static int field_rat(pcb_any_obj_t *obj, pcb_qry_node_t *fld, pcb_qry_val_t *res)
 {
 /*	const char *s1, *s2;
 
@@ -474,40 +504,58 @@ static int field_rat(pcb_obj_t *obj, pcb_qry_node_t *fld, pcb_qry_val_t *res)
 	PCB_QRY_RET_INV(res);
 }
 
-static int field_via(pcb_obj_t *obj, pcb_qry_node_t *fld, pcb_qry_val_t *res)
+static struct pcb_qry_lytc_s field_pstk_lyt(pcb_pstk_t *ps, const char *where)
 {
-	pcb_pin_t *p = obj->data.via;
-	query_fields_keys_t fh1;
+	struct pcb_qry_lytc_s lytc;
+	const char *start, *next;
+	int got_pol = 0;
 
-	fld2hash_req(fh1, fld, 0);
+	lytc.lyt = 0;
+	lytc.lyc = 0;
 
-	if (fh1 == query_fields_a) {
-		const char *s2;
-		fld2str_req(s2, fld, 1);
-		PCB_QRY_RET_STR(res, pcb_attribute_get(&p->Attributes, s2));
+	start = next = where;
+	for(;;) {
+		int len;
+		next = strchr(start, '_');
+		if (next == 0)
+			len = strlen(start);
+		else
+			len = next - start;
+
+		if (strncmp(start, "top", len) == 0) lytc.lyt |= PCB_LYT_TOP;
+		else if (strncmp(start, "bottom", len) == 0) lytc.lyt |= PCB_LYT_BOTTOM;
+		else if (strncmp(start, "intern", len) == 0) lytc.lyt |= PCB_LYT_INTERN;
+		else if (strncmp(start, "copper", len) == 0) lytc.lyt |= PCB_LYT_COPPER;
+		else if (strncmp(start, "silk", len) == 0) lytc.lyt |= PCB_LYT_SILK;
+		else if (strncmp(start, "mask", len) == 0) lytc.lyt |= PCB_LYT_MASK;
+		else if (strncmp(start, "paste", len) == 0) lytc.lyt |= PCB_LYT_PASTE;
+		else if (strncmp(start, "outline", len) == 0) lytc.lyt |= PCB_LYT_OUTLINE;
+		else if (strncmp(start, "auto", len) == 0) { lytc.lyc |= PCB_LYC_AUTO; got_pol = 1; }
+		else if (strncmp(start, "sub", len) == 0) { lytc.lyc |= PCB_LYC_SUB; got_pol = 1; }
+		else if (strncmp(start, "neg", len) == 0) { lytc.lyc |= PCB_LYC_SUB; got_pol = 1; }
+		else if (strncmp(start, "add", len) == 0) { got_pol = 1; }
+		else if (strncmp(start, "pos", len) == 0) { got_pol = 1; }
+		else {
+			lytc.lyt = 0;
+			return lytc;
+		};
+
+		if (next == NULL)
+			break;
+		start = next+1;
 	}
 
-	if (fld->next != NULL)
-		PCB_QRY_RET_INV(res);
-
-	switch(fh1) {
-		case query_fields_x:         PCB_QRY_RET_INT(res, p->X);
-		case query_fields_y:         PCB_QRY_RET_INT(res, p->Y);
-		case query_fields_thickness: PCB_QRY_RET_INT(res, p->Thickness);
-		case query_fields_clearance: PCB_QRY_RET_INT(res, p->Clearance);
-		case query_fields_hole:      PCB_QRY_RET_INT(res, p->DrillingHole);
-		case query_fields_mask:      PCB_QRY_RET_INT(res, p->Mask);
-		case query_fields_name:      PCB_QRY_RET_STR(res, p->Name);
-		case query_fields_number:    PCB_QRY_RET_STR(res, p->Number);
-		default:;
+	/* implicit polarity (shorthand) */
+	if (!got_pol) {
+		if (lytc.lyt & PCB_LYT_MASK) lytc.lyc = PCB_LYC_SUB | PCB_LYC_AUTO;
+		if (lytc.lyt & PCB_LYT_PASTE) lytc.lyc = PCB_LYC_AUTO;
 	}
-
-	PCB_QRY_RET_INV(res);
+	return lytc;
 }
 
-static int field_pstk(pcb_obj_t *obj, pcb_qry_node_t *fld, pcb_qry_val_t *res)
+static int field_pstk(pcb_any_obj_t *obj, pcb_qry_node_t *fld, pcb_qry_val_t *res)
 {
-	pcb_pstk_t *p = obj->data.pstk;
+	pcb_pstk_t *p = (pcb_pstk_t *)obj;
 	pcb_pstk_proto_t *proto = pcb_pstk_get_proto(p);
 	query_fields_keys_t fh1;
 
@@ -519,6 +567,26 @@ static int field_pstk(pcb_obj_t *obj, pcb_qry_node_t *fld, pcb_qry_val_t *res)
 		PCB_QRY_RET_STR(res, pcb_attribute_get(&p->Attributes, s2));
 	}
 
+	if (fh1 == query_fields_shape) {
+		pcb_layer_type_t lyt;
+		pcb_layer_combining_t lyc = 0;
+		pcb_pstk_shape_t *shp;
+
+		fld2lyt_req(lyt, lyc, fld, 1);
+		if (lyt == 0)
+			PCB_QRY_RET_INV(res);
+
+		shp = pcb_pstk_shape(p, lyt, lyc);
+		if (shp != NULL) {
+			switch(shp->shape) {
+				case PCB_PSSH_POLY: PCB_QRY_RET_STR(res, "polygon");
+				case PCB_PSSH_LINE: PCB_QRY_RET_STR(res, "line");
+				case PCB_PSSH_CIRC: PCB_QRY_RET_STR(res, "circle");
+			}
+		}
+		PCB_QRY_RET_STR(res, "");
+	}
+
 	if (fld->next != NULL)
 		PCB_QRY_RET_INV(res);
 
@@ -538,70 +606,7 @@ static int field_pstk(pcb_obj_t *obj, pcb_qry_node_t *fld, pcb_qry_val_t *res)
 	PCB_QRY_RET_INV(res);
 }
 
-static int field_element(pcb_obj_t *obj, pcb_qry_node_t *fld, pcb_qry_val_t *res)
-{
-	pcb_element_t *p = obj->data.element;
-	query_fields_keys_t fh1;
-
-	fld2hash_req(fh1, fld, 0);
-	if (fh1 == query_fields_a) {
-		const char *s2;
-		fld2str_req(s2, fld, 1);
-		PCB_QRY_RET_STR(res, pcb_attribute_get(&p->Attributes, s2));
-	}
-
-	if (fh1 == query_fields_layer)
-		return layer_of_obj(fld->next, res, PCB_LYT_SILK | (PCB_FLAG_TEST(PCB_FLAG_ONSOLDER, p) ? PCB_LYT_BOTTOM : PCB_LYT_TOP));
-
-	if (fld->next != NULL)
-		PCB_QRY_RET_INV(res);
-
-	switch(fh1) {
-		case query_fields_x:            PCB_QRY_RET_INT(res, p->MarkX);
-		case query_fields_y:            PCB_QRY_RET_INT(res, p->MarkY);
-		case query_fields_refdes:       /* alias of: */
-		case query_fields_name:         PCB_QRY_RET_STR(res, p->Name[PCB_ELEMNAME_IDX_REFDES].TextString);
-		case query_fields_description:  PCB_QRY_RET_STR(res, p->Name[PCB_ELEMNAME_IDX_DESCRIPTION].TextString);
-		case query_fields_value:        PCB_QRY_RET_STR(res, p->Name[PCB_ELEMNAME_IDX_VALUE].TextString);
-
-		default:;
-	}
-	PCB_QRY_RET_INV(res);
-}
-
-static int field_element_from_ptr(pcb_element_t *e, pcb_qry_node_t *fld, pcb_qry_val_t *res)
-{
-	pcb_obj_t tmp;
-	tmp.type = PCB_OBJ_ELEMENT;
-	tmp.data.element = e;
-	tmp.parent_type = PCB_PARENT_DATA;
-	tmp.parent.data = PCB->Data;
-	return field_element(&tmp, fld, res);
-}
-
-static int field_element_obj(pcb_obj_t *obj, pcb_qry_node_t *fld, pcb_qry_val_t *res)
-{
-	const char *s1;
-
-	/* if parent is not an element (or not available) evaluate to invalid */
-	if (obj->parent_type != PCB_PARENT_ELEMENT)
-		PCB_QRY_RET_INV(res);
-
-	/* check subfield, if there's none, return the element object */
-	fld2str_opt(s1, fld, 0);
-	if (s1 == NULL) {
-		res->type = PCBQ_VT_OBJ;
-		res->data.obj.data.element = obj->parent.element;
-		res->data.obj.parent_type = PCB_PARENT_DATA;
-		res->data.obj.parent.data = PCB->Data;
-		return 0;
-	}
-
-	/* return subfields of the element */
-	return field_element_from_ptr(obj->parent.element, fld, res);
-}
-
-static int field_net(pcb_obj_t *obj, pcb_qry_node_t *fld, pcb_qry_val_t *res)
+static int field_net(pcb_any_obj_t *obj, pcb_qry_node_t *fld, pcb_qry_val_t *res)
 {
 /*	const char *s1, *s2;
 
@@ -611,60 +616,13 @@ static int field_net(pcb_obj_t *obj, pcb_qry_node_t *fld, pcb_qry_val_t *res)
 	PCB_QRY_RET_INV(res);
 }
 
-
-static int field_eline(pcb_obj_t *obj, pcb_qry_node_t *fld, pcb_qry_val_t *res)
+static int field_subc(pcb_any_obj_t *obj, pcb_qry_node_t *fld, pcb_qry_val_t *res)
 {
-	pcb_line_t *l = obj->data.line;
-	query_fields_keys_t fh1;
-
-	fld2hash_req(fh1, fld, 0);
-
-	switch(fh1) {
-		case query_fields_layer:   return layer_of_obj(fld->next, res, PCB_LYT_SILK | (PCB_FLAG_TEST(PCB_FLAG_ONSOLDER, l) ? PCB_LYT_BOTTOM : PCB_LYT_TOP));
-		case query_fields_element: return field_element_obj(obj, fld->next, res);
-		default:;
-	}
-
-	return field_line(obj, fld, res);
-}
-
-static int field_earc(pcb_obj_t *obj, pcb_qry_node_t *fld, pcb_qry_val_t *res)
-{
-	pcb_arc_t *a = obj->data.arc;
-	query_fields_keys_t fh1;
-
-	fld2hash_req(fh1, fld, 0);
-
-	switch(fh1) {
-		case query_fields_layer:   return layer_of_obj(fld->next, res, PCB_LYT_SILK | (PCB_FLAG_TEST(PCB_FLAG_ONSOLDER, a) ? PCB_LYT_BOTTOM : PCB_LYT_TOP));
-		case query_fields_element: return field_element_obj(obj, fld->next, res);
-		default:;
-	}
-
-	return field_arc(obj, fld, res);
-}
-
-static int field_etext(pcb_obj_t *obj, pcb_qry_node_t *fld, pcb_qry_val_t *res)
-{
-	pcb_text_t *t = obj->data.text;
-	query_fields_keys_t fh1;
-
-	fld2hash_req(fh1, fld, 0);
-
-	switch(fh1) {
-		case query_fields_layer:   return layer_of_obj(fld->next, res, PCB_LYT_SILK | (PCB_FLAG_TEST(PCB_FLAG_ONSOLDER, t) ? PCB_LYT_BOTTOM : PCB_LYT_TOP));
-		case query_fields_element: return field_element_obj(obj, fld->next, res);
-		default:;
-	}
-
-	return field_text(obj, fld, res);
-}
-
-static int field_subc(pcb_obj_t *obj, pcb_qry_node_t *fld, pcb_qry_val_t *res)
-{
-	pcb_subc_t *p = obj->data.subc;
+	pcb_subc_t *p = (pcb_subc_t *)obj;
 	query_fields_keys_t fh1;
 	pcb_coord_t x, y;
+	double rot;
+	int on_bottom;
 
 	fld2hash_req(fh1, fld, 0);
 	if (fh1 == query_fields_a) {
@@ -679,179 +637,80 @@ static int field_subc(pcb_obj_t *obj, pcb_qry_node_t *fld, pcb_qry_val_t *res)
 	if (fld->next != NULL)
 		PCB_QRY_RET_INV(res);
 
-
-
 	switch(fh1) {
 		case query_fields_x:            pcb_subc_get_origin(p, &x, &y); PCB_QRY_RET_INT(res, x);
 		case query_fields_y:            pcb_subc_get_origin(p, &x, &y); PCB_QRY_RET_INT(res, y);
-		case query_fields_refdes:       /* alias of: */
-		case query_fields_name:         PCB_QRY_RET_STR(res, p->refdes);
-#warning subc TODO:
-/*
-		case query_fields_description:  PCB_QRY_RET_STR(res, p->Name[PCB_ELEMNAME_IDX_DESCRIPTION].TextString);
-		case query_fields_value:        PCB_QRY_RET_STR(res, p->Name[PCB_ELEMNAME_IDX_VALUE].TextString);
-*/
+		case query_fields_rotation:     pcb_subc_get_rotation(p, &rot); PCB_QRY_RET_DBL(res, rot);
+		case query_fields_side:         pcb_subc_get_side(p, &on_bottom); PCB_QRY_RET_SIDE(res, on_bottom);
+		case query_fields_refdes:       PCB_QRY_RET_STR(res, p->refdes);
 		default:;
 	}
 	PCB_QRY_RET_INV(res);
 }
 
+static int field_subc_obj(pcb_any_obj_t *obj, pcb_qry_node_t *fld, pcb_qry_val_t *res);
+
 static int field_subc_from_ptr(pcb_subc_t *s, pcb_qry_node_t *fld, pcb_qry_val_t *res)
 {
-	pcb_obj_t tmp;
-	tmp.type = PCB_OBJ_SUBC;
-	tmp.data.subc = s;
-	tmp.parent_type = PCB_PARENT_DATA;
-	tmp.parent.data = PCB->Data;
-	return field_subc(&tmp, fld, res);
+	pcb_any_obj_t *obj = (pcb_any_obj_t *)s;
+	query_fields_keys_t fh1;
+
+	fld2hash_req(fh1, fld, 0);
+	COMMON_FIELDS(obj, fh1, fld, res);
+
+	return field_subc(obj, fld, res);
 }
 
-static int field_subc_obj(pcb_obj_t *obj, pcb_qry_node_t *fld, pcb_qry_val_t *res)
+static int field_subc_obj(pcb_any_obj_t *obj, pcb_qry_node_t *fld, pcb_qry_val_t *res)
 {
 	const char *s1;
+	pcb_subc_t *parent = pcb_obj_parent_subc(obj);
 
 	/* if parent is not a subc (or not available) evaluate to invalid */
-	if (obj->parent_type != PCB_PARENT_SUBC)
+	if (parent == NULL)
 		PCB_QRY_RET_INV(res);
 
-	/* check subfield, if there's none, return the element object */
+	/* check subfield, if there's none, return the subcircuit object */
 	fld2str_opt(s1, fld, 0);
 	if (s1 == NULL) {
 		res->type = PCBQ_VT_OBJ;
-		res->data.obj.data.subc = obj->parent.subc;
-		res->data.obj.parent_type = PCB_PARENT_DATA;
-		res->data.obj.parent.data = PCB->Data;
+		res->data.obj = (pcb_any_obj_t *)parent;
 		return 0;
 	}
 
-	/* return subfields of the element */
-	return field_subc_from_ptr(obj->parent.subc, fld, res);
+	/* return subfields of the subcircuit */
+	return field_subc_from_ptr(parent, fld, res);
 }
 
-static int field_pin(pcb_obj_t *obj, pcb_qry_node_t *fld, pcb_qry_val_t *res)
-{
-	query_fields_keys_t fh1;
-
-	fld2hash_req(fh1, fld, 0);
-
-	switch(fh1) {
-		case query_fields_element: return field_element_obj(obj, fld->next, res);
-		default:;
-	}
-
-	return field_via(obj, fld, res);
-}
-
-static int field_pad(pcb_obj_t *obj, pcb_qry_node_t *fld, pcb_qry_val_t *res)
-{
-	pcb_pad_t *p = obj->data.pad;
-	query_fields_keys_t fh1;
-
-	fld2hash_req(fh1, fld, 0);
-
-	switch(fh1) {
-		case query_fields_a:
-			{
-				const char *s2;
-				fld2str_req(s2, fld, 1);
-				PCB_QRY_RET_STR(res, pcb_attribute_get(&p->Attributes, s2));
-			}
-		case query_fields_layer:   return layer_of_obj(fld->next, res, PCB_LYT_COPPER | (PCB_FLAG_TEST(PCB_FLAG_ONSOLDER, p) ? PCB_LYT_BOTTOM : PCB_LYT_TOP));
-		case query_fields_element: return field_element_obj(obj, fld->next, res);
-		default:;
-	}
-
-
-	if (fld->next != NULL)
-		PCB_QRY_RET_INV(res);
-
-	switch(fh1) {
-		case query_fields_x1:        PCB_QRY_RET_INT(res, p->Point1.X);
-		case query_fields_y1:        PCB_QRY_RET_INT(res, p->Point1.Y);
-		case query_fields_x2:        PCB_QRY_RET_INT(res, p->Point2.X);
-		case query_fields_y2:        PCB_QRY_RET_INT(res, p->Point2.Y);
-		case query_fields_thickness: PCB_QRY_RET_INT(res, p->Thickness);
-		case query_fields_clearance: PCB_QRY_RET_INT(res, p->Clearance);
-		case query_fields_mask:      PCB_QRY_RET_INT(res, p->Mask);
-		case query_fields_name:      PCB_QRY_RET_STR(res, p->Name);
-		case query_fields_number:    PCB_QRY_RET_STR(res, p->Number);
-		default:;
-	}
-
-	PCB_QRY_RET_INV(res);
-}
-
-
 /***/
 
 int pcb_qry_obj_field(pcb_qry_val_t *objval, pcb_qry_node_t *fld, pcb_qry_val_t *res)
 {
-	pcb_obj_t *obj;
+	pcb_any_obj_t *obj;
 	query_fields_keys_t fh1;
 
 	if (objval->type != PCBQ_VT_OBJ)
 		return -1;
-	obj = &objval->data.obj;
+	obj = objval->data.obj;
 
 	fld2hash_req(fh1, fld, 0);
 
-	if (fh1 == query_fields_a) {
-		const char *s2;
-		fld2str_req(s2, fld, 1);
-		if (!PCB_OBJ_IS_CLASS(obj->type, PCB_OBJ_CLASS_OBJ))
-			PCB_QRY_RET_INV(res);
-		PCB_QRY_RET_STR(res, pcb_attribute_get(&obj->data.anyobj->Attributes, s2));
-	}
-
-	if (fh1 == query_fields_ID) {
-		if (!PCB_OBJ_IS_CLASS(obj->type, PCB_OBJ_CLASS_OBJ))
-			PCB_QRY_RET_INV(res);
-		PCB_QRY_RET_INT(res, obj->data.anyobj->ID);
-	}
-
-	if (fh1 == query_fields_bbox) {
-		query_fields_keys_t fh2;
-
-		if (!PCB_OBJ_IS_CLASS(obj->type, PCB_OBJ_CLASS_OBJ))
-			PCB_QRY_RET_INV(res);
-
-		fld2hash_req(fh2, fld, 1);
-		switch(fh2) {
-			case query_fields_x1:     PCB_QRY_RET_INT(res, obj->data.anyobj->BoundingBox.X1);
-			case query_fields_y1:     PCB_QRY_RET_INT(res, obj->data.anyobj->BoundingBox.Y1);
-			case query_fields_x2:     PCB_QRY_RET_INT(res, obj->data.anyobj->BoundingBox.X2);
-			case query_fields_y2:     PCB_QRY_RET_INT(res, obj->data.anyobj->BoundingBox.Y2);
-			case query_fields_width:  PCB_QRY_RET_INT(res, obj->data.anyobj->BoundingBox.X2 - obj->data.anyobj->BoundingBox.X1);
-			case query_fields_height: PCB_QRY_RET_INT(res, obj->data.anyobj->BoundingBox.Y2 - obj->data.anyobj->BoundingBox.Y1);
-			case query_fields_area:   PCB_QRY_RET_DBL(res, (double)(obj->data.anyobj->BoundingBox.Y2 - obj->data.anyobj->BoundingBox.Y1) * (double)(obj->data.anyobj->BoundingBox.X2 - obj->data.anyobj->BoundingBox.X1));
-			default:;
-		}
-		PCB_QRY_RET_INV(res);
-	}
-
-	if (fh1 == query_fields_type)
-		PCB_QRY_RET_INT(res, obj->type);
+	COMMON_FIELDS(obj, fh1, fld, res);
 
 	switch(obj->type) {
 /*		case PCB_OBJ_POINT:    return field_point(obj, fld, res);*/
 		case PCB_OBJ_LINE:     return field_line(obj, fld, res);
 		case PCB_OBJ_TEXT:     return field_text(obj, fld, res);
-		case PCB_OBJ_POLY:  return field_polygon(obj, fld, res);
+		case PCB_OBJ_POLY:     return field_polygon(obj, fld, res);
 		case PCB_OBJ_ARC:      return field_arc(obj, fld, res);
 		case PCB_OBJ_RAT:      return field_rat(obj, fld, res);
-		case PCB_OBJ_PAD:      return field_pad(obj, fld, res);
-		case PCB_OBJ_PIN:      return field_pin(obj, fld, res);
-		case PCB_OBJ_VIA:      return field_via(obj, fld, res);
 		case PCB_OBJ_PSTK:     return field_pstk(obj, fld, res);
-		case PCB_OBJ_ELEMENT:  return field_element(obj, fld, res);
 		case PCB_OBJ_SUBC:     return field_subc(obj, fld, res);
 
 		case PCB_OBJ_NET:      return field_net(obj, fld, res);
 		case PCB_OBJ_LAYER:    return field_layer(obj, fld, res);
-
-		case PCB_OBJ_ELINE:    return field_eline(obj, fld, res);
-		case PCB_OBJ_EARC:     return field_earc(obj, fld, res);
-		case PCB_OBJ_ETEXT:    return field_etext(obj, fld, res);
+#warning layer TODO
+		case PCB_OBJ_LAYERGRP:
 		default:;
 	}
 
