@@ -49,7 +49,7 @@ void pcb_qry_uninit(pcb_qry_exec_t *ctx)
 #warning TODO: free the iterator
 }
 
-int pcb_qry_run(pcb_qry_node_t *prg, void (*cb)(void *user_ctx, pcb_qry_val_t *res, pcb_obj_t *current), void *user_ctx)
+int pcb_qry_run(pcb_qry_node_t *prg, void (*cb)(void *user_ctx, pcb_qry_val_t *res, pcb_any_obj_t *current), void *user_ctx)
 {
 	pcb_qry_exec_t ec;
 	pcb_qry_val_t res;
@@ -61,9 +61,14 @@ int pcb_qry_run(pcb_qry_node_t *prg, void (*cb)(void *user_ctx, pcb_qry_val_t *r
 
 	do {
 		if (pcb_qry_eval(&ec, prg, &res) == 0) {
-			pcb_obj_t *current = NULL;
-			if (ec.iter->all_idx >= 0)
-				current = ec.iter->it[ec.iter->all_idx];
+			pcb_any_obj_t **tmp, *current = NULL;
+			if (ec.iter->all_idx >= 0) {
+				tmp = (pcb_any_obj_t **)vtp0_get(ec.iter->vects[ec.iter->all_idx], ec.iter->idx[ec.iter->all_idx], 0);
+				if (tmp != NULL)
+					current = *tmp;
+				else
+					current = NULL;
+			}
 			cb(user_ctx, &res, current);
 		}
 		else
@@ -152,8 +157,8 @@ int pcb_qry_is_true(pcb_qry_val_t *val)
 {
 	switch(val->type) {
 		case PCBQ_VT_VOID:     return 0;
-		case PCBQ_VT_OBJ:      return val->data.obj.type != PCB_OBJ_VOID;
-		case PCBQ_VT_LST:      return pcb_objlist_first(&val->data.lst) != NULL;
+		case PCBQ_VT_OBJ:      return val->data.obj->type != PCB_OBJ_VOID;
+		case PCBQ_VT_LST:      return vtp0_len(&val->data.lst) > 0;
 		case PCBQ_VT_COORD:    return val->data.crd;
 		case PCBQ_VT_DOUBLE:   return val->data.dbl;
 		case PCBQ_VT_STRING:   return (val->data.str != NULL) && (*val->data.str != '\0');
@@ -165,7 +170,9 @@ static void setup_iter_list(pcb_qry_exec_t *ctx, int var_id, pcb_qry_val_t *list
 {
 	ctx->iter->lst[var_id] = *listval;
 	assert(listval->type == PCBQ_VT_LST);
-	ctx->iter->it[var_id] = pcb_objlist_first(&listval->data.lst);
+
+	ctx->iter->vects[var_id] = &listval->data.lst;
+	ctx->iter->idx[var_id] = 0;
 }
 
 int pcb_qry_it_reset(pcb_qry_exec_t *ctx, pcb_qry_node_t *node)
@@ -192,18 +199,41 @@ int pcb_qry_it_next(pcb_qry_exec_t *ctx)
 {
 	int i;
 	for(i = 0; i < ctx->iter->num_vars; i++) {
-		ctx->iter->it[i] = pcb_objlist_next(ctx->iter->it[i]);
-		if (ctx->iter->it[i] != NULL)
-			return 1;
-		ctx->iter->it[i] = pcb_objlist_first(&ctx->iter->lst[i].data.lst);
+		if (ctx->iter->vects[i] != NULL) {
+			ctx->iter->idx[i]++;
+			if (ctx->iter->idx[i] < vtp0_len(ctx->iter->vects[i]))
+				return 1;
+		}
+		ctx->iter->idx[i] = 0;
 	}
 	return 0;
 }
 
+/* load s1 and s2 from o1 and o2, convert empty string to NULL */
+#define load_strings_null() \
+do { \
+	s1 = o1.data.str; \
+	s2 = o2.data.str; \
+	if ((s1 != NULL) && (*s1 == '\0')) s1 = NULL; \
+	if ((s2 != NULL) && (*s2 == '\0')) s2 = NULL; \
+} while(0)
+
+static char *pcb_qry_empty = "";
+
+/* load s1 and s2 from o1 and o2, convert NULL to empty string */
+#define load_strings_empty() \
+do { \
+	s1 = o1.data.str; \
+	s2 = o2.data.str; \
+	if (s1 == NULL) s1 = pcb_qry_empty; \
+	if (s2 == NULL) s2 = pcb_qry_empty; \
+} while(0)
 
 int pcb_qry_eval(pcb_qry_exec_t *ctx, pcb_qry_node_t *node, pcb_qry_val_t *res)
 {
 	pcb_qry_val_t o1, o2;
+	pcb_any_obj_t **tmp;
+	const char *s1, *s2;
 
 	switch(node->type) {
 		case PCBQ_EXPR:
@@ -249,16 +279,17 @@ int pcb_qry_eval(pcb_qry_exec_t *ctx, pcb_qry_node_t *node, pcb_qry_val_t *res)
 				return -1;
 			switch(o1.type) {
 				case PCBQ_VT_VOID:     PCB_QRY_RET_INV(res);
-				case PCBQ_VT_OBJ:      PCB_QRY_RET_INT(res, ((o1.data.obj.type) == (o2.data.obj.type)) && ((o1.data.obj.data.any) == (o2.data.obj.data.any)));
+				case PCBQ_VT_OBJ:      PCB_QRY_RET_INT(res, ((o1.data.obj) == (o2.data.obj)));
 				case PCBQ_VT_LST:      PCB_QRY_RET_INT(res, pcb_qry_list_cmp(&o1, &o2));
 				case PCBQ_VT_COORD:    PCB_QRY_RET_INT(res, o1.data.crd == o2.data.crd);
 				case PCBQ_VT_DOUBLE:   PCB_QRY_RET_INT(res, o1.data.dbl == o2.data.dbl);
 				case PCBQ_VT_STRING:
-					if (o1.data.str == o2.data.str)
+					load_strings_null();
+					if (s1 == s2)
 						PCB_QRY_RET_INT(res, 1);
-					if ((o1.data.str == NULL) || (o2.data.str == NULL))
+					if ((s1 == NULL) || (s2 == NULL))
 						PCB_QRY_RET_INT(res, 0);
-					PCB_QRY_RET_INT(res, strcmp(o1.data.str, o2.data.str) == 0);
+					PCB_QRY_RET_INT(res, strcmp(s1, s2) == 0);
 			}
 			return -1;
 
@@ -268,16 +299,17 @@ int pcb_qry_eval(pcb_qry_exec_t *ctx, pcb_qry_node_t *node, pcb_qry_val_t *res)
 				return -1;
 			switch(o1.type) {
 				case PCBQ_VT_VOID:     PCB_QRY_RET_INV(res);
-				case PCBQ_VT_OBJ:      PCB_QRY_RET_INT(res, ((o1.data.obj.type) != (o2.data.obj.type)) || ((o1.data.obj.data.any) != (o2.data.obj.data.any)));
+				case PCBQ_VT_OBJ:      PCB_QRY_RET_INT(res, ((o1.data.obj) != (o2.data.obj)));
 				case PCBQ_VT_LST:      PCB_QRY_RET_INT(res, !pcb_qry_list_cmp(&o1, &o2));
 				case PCBQ_VT_COORD:    PCB_QRY_RET_INT(res, o1.data.crd != o2.data.crd);
 				case PCBQ_VT_DOUBLE:   PCB_QRY_RET_INT(res, o1.data.dbl != o2.data.dbl);
 				case PCBQ_VT_STRING:
-					if (o1.data.str == o2.data.str)
+					load_strings_null();
+					if (s1 == s2)
 						PCB_QRY_RET_INT(res, 0);
-					if ((o1.data.str == NULL) || (o2.data.str == NULL))
+					if ((s1 == NULL) || (s2 == NULL))
 						PCB_QRY_RET_INT(res, 1);
-					PCB_QRY_RET_INT(res, strcmp(o1.data.str, o2.data.str) != 0);
+					PCB_QRY_RET_INT(res, strcmp(s1, s2) != 0);
 			}
 			return -1;
 
@@ -289,7 +321,9 @@ int pcb_qry_eval(pcb_qry_exec_t *ctx, pcb_qry_node_t *node, pcb_qry_val_t *res)
 				case PCBQ_VT_VOID:     PCB_QRY_RET_INV(res);
 				case PCBQ_VT_COORD:    PCB_QRY_RET_INT(res, o1.data.crd >= o2.data.crd);
 				case PCBQ_VT_DOUBLE:   PCB_QRY_RET_INT(res, o1.data.dbl >= o2.data.dbl);
-				case PCBQ_VT_STRING:   PCB_QRY_RET_INT(res, strcmp(o1.data.str, o2.data.str) >= 0);
+				case PCBQ_VT_STRING:
+					load_strings_empty();
+					PCB_QRY_RET_INT(res, strcmp(s1, s2) >= 0);
 				default:               return -1;
 			}
 			return -1;
@@ -302,7 +336,9 @@ int pcb_qry_eval(pcb_qry_exec_t *ctx, pcb_qry_node_t *node, pcb_qry_val_t *res)
 				case PCBQ_VT_VOID:     PCB_QRY_RET_INV(res);
 				case PCBQ_VT_COORD:    PCB_QRY_RET_INT(res, o1.data.crd <= o2.data.crd);
 				case PCBQ_VT_DOUBLE:   PCB_QRY_RET_INT(res, o1.data.dbl <= o2.data.dbl);
-				case PCBQ_VT_STRING:   PCB_QRY_RET_INT(res, strcmp(o1.data.str, o2.data.str) <= 0);
+				case PCBQ_VT_STRING:
+					load_strings_empty();
+					PCB_QRY_RET_INT(res, strcmp(s1, s2) <= 0);
 				default:               return -1;
 			}
 			return -1;
@@ -315,7 +351,9 @@ int pcb_qry_eval(pcb_qry_exec_t *ctx, pcb_qry_node_t *node, pcb_qry_val_t *res)
 				case PCBQ_VT_VOID:     PCB_QRY_RET_INV(res);
 				case PCBQ_VT_COORD:    PCB_QRY_RET_INT(res, o1.data.crd > o2.data.crd);
 				case PCBQ_VT_DOUBLE:   PCB_QRY_RET_INT(res, o1.data.dbl > o2.data.dbl);
-				case PCBQ_VT_STRING:   PCB_QRY_RET_INT(res, strcmp(o1.data.str, o2.data.str) > 0);
+				case PCBQ_VT_STRING:
+					load_strings_empty();
+					PCB_QRY_RET_INT(res, strcmp(s1, s2) > 0);
 				default:               return -1;
 			}
 			return -1;
@@ -327,7 +365,9 @@ int pcb_qry_eval(pcb_qry_exec_t *ctx, pcb_qry_node_t *node, pcb_qry_val_t *res)
 				case PCBQ_VT_VOID:     PCB_QRY_RET_INV(res);
 				case PCBQ_VT_COORD:    PCB_QRY_RET_INT(res, o1.data.crd < o2.data.crd);
 				case PCBQ_VT_DOUBLE:   PCB_QRY_RET_INT(res, o1.data.dbl < o2.data.dbl);
-				case PCBQ_VT_STRING:   PCB_QRY_RET_INT(res, strcmp(o1.data.str, o2.data.str) < 0);
+				case PCBQ_VT_STRING:
+					load_strings_empty();
+					PCB_QRY_RET_INT(res, strcmp(s1, s2) < 0);
 				default:               return -1;
 			}
 			return -1;
@@ -407,10 +447,14 @@ int pcb_qry_eval(pcb_qry_exec_t *ctx, pcb_qry_node_t *node, pcb_qry_val_t *res)
 
 		case PCBQ_VAR:
 			assert((node->data.crd >= 0) && (node->data.crd < ctx->iter->num_vars));
-			res->type = PCBQ_VT_OBJ;
-			if (ctx->iter->it[node->data.crd] == NULL)
+			res->type = PCBQ_VT_VOID;
+			if (ctx->iter->vects[node->data.crd] == NULL)
 				return -1;
-			res->data.obj = *ctx->iter->it[node->data.crd];
+			tmp = (pcb_any_obj_t **)vtp0_get(ctx->iter->vects[node->data.crd], ctx->iter->idx[node->data.crd], 0);
+			if ((tmp == NULL) || (*tmp == NULL))
+				return -1;
+			res->type = PCBQ_VT_OBJ;
+			res->data.obj = *tmp;
 			return 0;
 
 		case PCBQ_LISTVAR: {
@@ -467,6 +511,7 @@ int pcb_qry_eval(pcb_qry_exec_t *ctx, pcb_qry_node_t *node, pcb_qry_val_t *res)
 		case PCBQ_nodetype_max:
 		case PCBQ_FIELD:
 		case PCBQ_RNAME:
+		case PCBQ_DATA_LYTC:
 		case PCBQ_ITER_CTX:
 			return -1;
 	}
