@@ -1,11 +1,10 @@
-/*!
- * \file renumberblock.c
+/*
+ * RenumberBlock plug-in for PCB.
  *
- * \brief RenumberBlock plug-in for PCB.
+ * Copyright (C) 2006 DJ Delorie <dj@delorie.com>
+ * Copyright (C) 2018 Tibor 'Igor2' Palinkas for pcb-rnd data model
  *
- * \author Copyright (C) 2006 DJ Delorie <dj@delorie.com>
- *
- * \copyright Licensed under the terms of the GNU General Public
+ * Licensed under the terms of the GNU General Public
  * License, version 2 or later.
  *
  * Original source: http://www.delorie.com/pcb/renumberblock.c
@@ -49,16 +48,16 @@ int pcb_act_RenumberBlock(int argc, const char **argv, pcb_coord_t x, pcb_coord_
 
 	conf_set_editor(name_on_pcb, 1);
 
-	PCB_ELEMENT_LOOP(PCB->Data);
+	PCB_SUBC_LOOP(PCB->Data);
 	{
-		char *refdes_split, *cp;
-		char *old_ref, *new_ref;
+		char *new_ref;
+		const char *old_ref, *cp, *refdes_split;
 		int num;
 
-		if (!PCB_FLAG_TEST(PCB_FLAG_SELECTED, element))
+		if (!PCB_FLAG_TEST(PCB_FLAG_SELECTED, subc) || (subc->refdes == NULL))
 			continue;
 
-		old_ref = element->Name[1].TextString;
+		old_ref = subc->refdes;
 		for (refdes_split = cp = old_ref; *cp; cp++)
 			if (!isdigit(*cp))
 				refdes_split = cp + 1;
@@ -70,9 +69,9 @@ int pcb_act_RenumberBlock(int argc, const char **argv, pcb_coord_t x, pcb_coord_
 		memcpy(new_ref, old_ref, refdes_split - old_ref);
 		strcpy(new_ref + (refdes_split - old_ref), num_buf);
 
-		pcb_undo_add_obj_to_change_name(PCB_TYPE_ELEMENT, NULL, NULL, element, PCB_ELEM_NAME_REFDES(element));
+		pcb_undo_add_obj_to_change_name(PCB_OBJ_SUBC, NULL, NULL, subc, subc->refdes);
 
-		pcb_chg_obj_name(PCB_TYPE_ELEMENT, element, NULL, NULL, new_ref);
+		pcb_chg_obj_name(PCB_OBJ_SUBC, subc, subc, subc, new_ref);
 	}
 	PCB_END_LOOP;
 	pcb_undo_inc_serial();
@@ -94,13 +93,16 @@ int pcb_act_RenumberBuffer(int argc, const char **argv, pcb_coord_t x, pcb_coord
 
 	conf_set_editor(name_on_pcb, 1);
 
-	PCB_ELEMENT_LOOP(PCB_PASTEBUFFER->Data);
+	PCB_SUBC_LOOP(PCB_PASTEBUFFER->Data);
 	{
-		char *refdes_split, *cp;
-		char *old_ref, *new_ref;
+		const char *refdes_split, *cp, *old_ref;
+		char *new_ref;
 		int num;
 
-		old_ref = element->Name[1].TextString;
+		if (subc->refdes == NULL)
+			continue;
+
+		old_ref = subc->refdes;
 		for (refdes_split = cp = old_ref; *cp; cp++)
 			if (!isdigit(*cp))
 				refdes_split = cp + 1;
@@ -112,8 +114,9 @@ int pcb_act_RenumberBuffer(int argc, const char **argv, pcb_coord_t x, pcb_coord
 		memcpy(new_ref, old_ref, refdes_split - old_ref);
 		strcpy(new_ref + (refdes_split - old_ref), num_buf);
 
-		pcb_chg_obj_name(PCB_TYPE_ELEMENT, element, NULL, NULL, new_ref);
+		pcb_chg_obj_name(PCB_OBJ_SUBC, subc, subc, subc, new_ref);
 	}
 	PCB_END_LOOP;
+
 	return 0;
 }
