@@ -60,7 +60,7 @@
 #include "compat_misc.h"
 #include "compat_nls.h"
 #include "layer.h"
-#include "obj_all.h"
+#include "obj_term.h"
 #include "obj_pstk.h"
 #include "obj_pstk_inlines.h"
 #include "obj_subc_parent.h"
@@ -72,15 +72,15 @@ conf_report_t conf_report;
 #define AUSAGE(x) pcb_message(PCB_MSG_INFO, "Usage:\n%s\n", (x##_syntax))
 #define USER_UNITMASK (conf_core.editor.grid_unit->allow)
 
-static int ReportDrills(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
+static int report_drills(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
 {
-	DrillInfoTypePtr AllDrills;
+	pcb_drill_info_t *AllDrills;
 	pcb_cardinal_t n;
 	char *stringlist, *thestring;
 	int total_drills = 0;
 
-	AllDrills = GetDrillInfo(PCB->Data);
-	RoundDrillInfo(AllDrills, 100);
+	AllDrills = drill_get_info(PCB->Data);
+	drill_round_info(AllDrills, 100);
 
 	for (n = 0; n < AllDrills->DrillN; n++) {
 		total_drills += AllDrills->Drill[n].PinCount;
@@ -110,7 +110,7 @@ static int ReportDrills(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 		while (*thestring != '\0')
 			thestring++;
 	}
-	FreeDrillInfo(AllDrills);
+	drill_info_free(AllDrills);
 	/* create dialog box */
 	pcb_gui->report_dialog("Drill Report", stringlist);
 
@@ -145,7 +145,7 @@ static const char *grpname(pcb_layergrp_id_t gid)
 	return grp->name;
 }
 
-static int ReportDialog(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
+static int report_dialog(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
 {
 	void *ptr1, *ptr2, *ptr3;
 	int type = REPORT_TYPES;
@@ -154,14 +154,14 @@ static int ReportDialog(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 
 	if ((argv != NULL) && (argv[0] != NULL)) {
 		if (pcb_strncasecmp(argv[0], "Subc", 4) == 0)
-			type = PCB_TYPE_SUBC;
+			type = PCB_OBJ_SUBC;
 	}
 	type = pcb_search_screen(x, y, type, &ptr1, &ptr2, &ptr3);
-	if (type == PCB_TYPE_NONE)
-		type = pcb_search_screen(x, y, REPORT_TYPES | PCB_TYPE_LOCKED, &ptr1, &ptr2, &ptr3);
+	if (type == PCB_OBJ_VOID)
+		type = pcb_search_screen(x, y, REPORT_TYPES | PCB_OBJ_LOCKED, &ptr1, &ptr2, &ptr3);
 
 	switch (type) {
-	case PCB_TYPE_PSTK:
+	case PCB_OBJ_PSTK:
 		{
 			pcb_pstk_t *ps;
 			pcb_pstk_proto_t *proto;
@@ -178,7 +178,7 @@ static int ReportDialog(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 			gds_init(&tmp);
 
 			pcb_append_printf(&tmp, "%m+PADSTACK ID# %ld; Flags:%s\n"
-				"(X,Y) = %$mD.\n", USER_UNITMASK, ps->ID, pcb_strflg_f2s(ps->Flags, PCB_TYPE_PSTK, NULL),
+				"(X,Y) = %$mD.\n", USER_UNITMASK, ps->ID, pcb_strflg_f2s(ps->Flags, PCB_OBJ_PSTK, NULL, 0),
 				ps->x, ps->y);
 
 			if ((proto != NULL) && (proto->hdia > 0))
@@ -190,92 +190,7 @@ static int ReportDialog(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 			break;
 		}
 
-	case PCB_TYPE_VIA:
-		{
-			pcb_pin_t *via;
-#ifndef NDEBUG
-			if (pcb_gui->shift_is_pressed()) {
-				pcb_r_dump_tree(PCB->Data->via_tree, 0);
-				return 0;
-			}
-#endif
-			via = (pcb_pin_t *) ptr2;
-			if (PCB_FLAG_TEST(PCB_FLAG_HOLE, via))
-				report = pcb_strdup_printf("%m+VIA ID# %ld; Flags:%s\n"
-										"(X,Y) = %$mD.\n"
-										"It is a pure hole of diameter %$mS.\n"
-										"Name = \"%s\"."
-										"%s"
-										"%s%s%s", USER_UNITMASK, via->ID, pcb_strflg_f2s(via->Flags, PCB_TYPE_VIA, NULL),
-										via->X, via->Y, via->DrillingHole, PCB_EMPTY(via->Name), gen_locked(via), gen_term(via));
-			else
-				report = pcb_strdup_printf("%m+VIA ID# %ld;  Flags:%s\n"
-										"(X,Y) = %$mD.\n"
-										"Copper width = %$mS. Drill width = %$mS.\n"
-										"Clearance = %$mS.\n"
-										"Annulus = %$mS.\n"
-										"Solder mask hole = %$mS (gap = %$mS).\n"
-										"Name = \"%s\"."
-										"%s"
-										"%s%s%s", USER_UNITMASK, via->ID, pcb_strflg_f2s(via->Flags, PCB_TYPE_VIA, NULL),
-										via->X, via->Y,
-										via->Thickness,
-										via->DrillingHole,
-										via->Clearance / 2,
-										(via->Thickness - via->DrillingHole) / 2,
-										via->Mask,
-										(via->Mask - via->Thickness) / 2, PCB_EMPTY(via->Name), gen_locked(via), gen_term(via));
-			break;
-		}
-	case PCB_TYPE_PIN:
-		{
-			pcb_pin_t *Pin;
-			pcb_element_t *element;
-#ifndef NDEBUG
-			if (pcb_gui->shift_is_pressed()) {
-				pcb_r_dump_tree(PCB->Data->pin_tree, 0);
-				return 0;
-			}
-#endif
-			Pin = (pcb_pin_t *) ptr2;
-			element = (pcb_element_t *) ptr1;
-
-			PCB_PIN_LOOP(element);
-			{
-				if (pin == Pin)
-					break;
-			}
-			PCB_END_LOOP;
-			if (PCB_FLAG_TEST(PCB_FLAG_HOLE, Pin))
-				report = pcb_strdup_printf("%m+PIN ID# %ld; Flags:%s\n"
-										"(X,Y) = %$mD.\n"
-										"It is a mounting hole. Drill width = %$mS.\n"
-										"It is owned by element %$mS.\n"
-										"%s", USER_UNITMASK, Pin->ID, pcb_strflg_f2s(Pin->Flags, PCB_TYPE_PIN, NULL),
-										Pin->X, Pin->Y, Pin->DrillingHole,
-										PCB_EMPTY(element->Name[1].TextString), gen_locked(Pin));
-			else
-				report = pcb_strdup_printf(
-										"%m+PIN ID# %ld;  Flags:%s\n" "(X,Y) = %$mD.\n"
-										"Copper width = %$mS. Drill width = %$mS.\n"
-										"Clearance = %$mS.\n"
-										"Annulus = %$mS.\n"
-										"Solder mask hole = %$mS (gap = %$mS).\n"
-										"Name = \"%s\".\n"
-										"It is owned by element %s\n as pin number %s.\n"
-										"%s", USER_UNITMASK,
-										Pin->ID, pcb_strflg_f2s(Pin->Flags, PCB_TYPE_PIN, NULL),
-										Pin->X, Pin->Y, Pin->Thickness,
-										Pin->DrillingHole,
-										Pin->Clearance / 2,
-										(Pin->Thickness - Pin->DrillingHole) / 2,
-										Pin->Mask,
-										(Pin->Mask - Pin->Thickness) / 2,
-										PCB_EMPTY(Pin->Name),
-										PCB_EMPTY(element->Name[1].TextString), PCB_EMPTY(Pin->Number), gen_locked(Pin));
-			break;
-		}
-	case PCB_TYPE_LINE:
+	case PCB_OBJ_LINE:
 		{
 			pcb_line_t *line;
 #ifndef NDEBUG
@@ -293,16 +208,16 @@ static int ReportDialog(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 									"It is on layer %d\n"
 									"and has name \"%s\".\n"
 									"%s"
-									"%s%s%s", USER_UNITMASK,
-									line->ID, pcb_strflg_f2s(line->Flags, PCB_TYPE_LINE, NULL),
+									"%s%s", USER_UNITMASK,
+									line->ID, pcb_strflg_f2s(line->Flags, PCB_OBJ_LINE, NULL, 0),
 									line->Point1.X, line->Point1.Y, line->Point1.ID,
 									line->Point2.X, line->Point2.Y, line->Point2.ID,
 									line->Thickness, line->Clearance / 2,
 									pcb_layer_id(PCB->Data, (pcb_layer_t *) ptr1),
-									PCB_UNKNOWN(line->Number), gen_locked(line), gen_term(line));
+									gen_locked(line), gen_term(line));
 			break;
 		}
-	case PCB_TYPE_RATLINE:
+	case PCB_OBJ_RAT:
 		{
 			pcb_rat_t *line;
 #ifndef NDEBUG
@@ -317,12 +232,12 @@ static int ReportDialog(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 									"connects to layer group #%d (%s).\n"
 									"SecondPoint(X,Y) = %$mD; ID = %ld; "
 									"connects to layer group #%d (%s).\n",
-									USER_UNITMASK, line->ID, pcb_strflg_f2s(line->Flags, PCB_TYPE_LINE, NULL),
+									USER_UNITMASK, line->ID, pcb_strflg_f2s(line->Flags, PCB_OBJ_LINE, NULL, 0),
 									line->Point1.X, line->Point1.Y, line->Point1.ID, line->group1, grpname(line->group1),
 									line->Point2.X, line->Point2.Y, line->Point2.ID, line->group2, grpname(line->group2));
 			break;
 		}
-	case PCB_TYPE_ARC:
+	case PCB_OBJ_ARC:
 		{
 			pcb_arc_t *Arc;
 			pcb_box_t box;
@@ -345,7 +260,7 @@ static int ReportDialog(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 									"That makes the end points at %$mD and %$mD.\n"
 									"It is on layer %d.\n"
 									"%s"
-									"%s%s%s", USER_UNITMASK, Arc->ID, pcb_strflg_f2s(Arc->Flags, PCB_TYPE_ARC, NULL),
+									"%s%s%s", USER_UNITMASK, Arc->ID, pcb_strflg_f2s(Arc->Flags, PCB_OBJ_ARC, NULL, 0),
 									Arc->X, Arc->Y,
 									Arc->Thickness, Arc->Clearance / 2,
 									Arc->Width, Arc->StartAngle, Arc->Delta,
@@ -356,7 +271,7 @@ static int ReportDialog(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 									pcb_layer_id(PCB->Data, (pcb_layer_t *) ptr1), gen_locked(Arc), gen_term(Arc));
 			break;
 		}
-	case PCB_TYPE_POLY:
+	case PCB_OBJ_POLY:
 		{
 			pcb_poly_t *Polygon;
 			const char *aunit;
@@ -389,7 +304,7 @@ static int ReportDialog(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 									"Its unclipped area is %f square %s.\n"
 									"%s"
 									"%s%s%s", USER_UNITMASK, Polygon->ID,
-									pcb_strflg_f2s(Polygon->Flags, PCB_TYPE_POLY, NULL),
+									pcb_strflg_f2s(Polygon->Flags, PCB_OBJ_POLY, NULL, 0),
 									Polygon->BoundingBox.X1, Polygon->BoundingBox.Y1,
 									Polygon->BoundingBox.X2, Polygon->BoundingBox.Y2,
 									Polygon->PointN, Polygon->PointMax - Polygon->PointN,
@@ -399,93 +314,12 @@ static int ReportDialog(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 									gen_locked(Polygon), gen_term(Polygon));
 			break;
 		}
-	case PCB_TYPE_PAD:
-		{
-			pcb_coord_t len;
-			pcb_pad_t *Pad;
-			pcb_element_t *element;
-#ifndef NDEBUG
-			if (pcb_gui->shift_is_pressed()) {
-				pcb_r_dump_tree(PCB->Data->pad_tree, 0);
-				return 0;
-			}
-#endif
-			Pad = (pcb_pad_t *) ptr2;
-			element = (pcb_element_t *) ptr1;
-
-			PCB_PAD_LOOP(element);
-			{
-				{
-					if (pad == Pad)
-						break;
-				}
-			}
-			PCB_END_LOOP;
-			len = pcb_distance(Pad->Point1.X, Pad->Point1.Y, Pad->Point2.X, Pad->Point2.Y);
-			report = pcb_strdup_printf("%m+PAD ID# %ld;  Flags:%s\n"
-									"FirstPoint(X,Y)  = %$mD; ID = %ld.\n"
-									"SecondPoint(X,Y) = %$mD; ID = %ld.\n"
-									"Width = %$mS.  Length = %$mS.\n"
-									"Clearance = %$mS.\n"
-									"Solder mask = %$mS x %$mS (gap = %$mS).\n"
-									"Name = \"%s\".\n"
-									"It is owned by SMD element %s\n"
-									"  as pin number %s and is on the %s\n"
-									"side of the board.\n"
-									"%s", USER_UNITMASK, Pad->ID,
-									pcb_strflg_f2s(Pad->Flags, PCB_TYPE_PAD, NULL),
-									Pad->Point1.X, Pad->Point1.Y, Pad->Point1.ID,
-									Pad->Point2.X, Pad->Point2.Y, Pad->Point2.ID,
-									Pad->Thickness, len + Pad->Thickness,
-									Pad->Clearance / 2,
-									Pad->Mask, len + Pad->Mask,
-									(Pad->Mask - Pad->Thickness) / 2,
-									PCB_EMPTY(Pad->Name),
-									PCB_EMPTY(element->Name[1].TextString),
-									PCB_EMPTY(Pad->Number),
-									PCB_FLAG_TEST(PCB_FLAG_ONSOLDER,
-														Pad) ? "solder (bottom)" : "component", gen_locked(Pad));
-			break;
-		}
-	case PCB_TYPE_ELEMENT:
-		{
-			pcb_element_t *element;
-#ifndef NDEBUG
-			if (pcb_gui->shift_is_pressed()) {
-				pcb_r_dump_tree(PCB->Data->element_tree, 0);
-				return 0;
-			}
-#endif
-			element = (pcb_element_t *) ptr2;
-			report = pcb_strdup_printf("%m+ELEMENT ID# %ld;  Flags:%s\n"
-									"BoundingBox %$mD %$mD.\n"
-									"Descriptive Name \"%s\".\n"
-									"Name on board \"%s\".\n"
-									"Part number name \"%s\".\n"
-									"It is %$mS tall and is located at (X,Y) = %$mD %s.\n"
-									"Mark located at point (X,Y) = %$mD.\n"
-									"It is on the %s side of the board.\n"
-									"%s", USER_UNITMASK,
-									element->ID, pcb_strflg_f2s(element->Flags, PCB_TYPE_ELEMENT, NULL),
-									element->BoundingBox.X1, element->BoundingBox.Y1,
-									element->BoundingBox.X2, element->BoundingBox.Y2,
-									PCB_EMPTY(element->Name[0].TextString),
-									PCB_EMPTY(element->Name[1].TextString),
-									PCB_EMPTY(element->Name[2].TextString),
-									PCB_SCALE_TEXT(PCB_FONT_CAPHEIGHT, element->Name[1].Scale),
-									element->Name[1].X, element->Name[1].Y,
-									PCB_FLAG_TEST(PCB_FLAG_HIDENAME, element) ? ",\n  but it's hidden" : "",
-									element->MarkX, element->MarkY,
-									PCB_FLAG_TEST(PCB_FLAG_ONSOLDER, element) ? "solder (bottom)" : "component",
-									gen_locked(element));
-			break;
-		}
-	case PCB_TYPE_SUBC:
+	case PCB_OBJ_SUBC:
 		{
 			pcb_subc_t *subc;
 #ifndef NDEBUG
 			if (pcb_gui->shift_is_pressed()) {
-				pcb_r_dump_tree(PCB->Data->element_tree, 0);
+				pcb_r_dump_tree(PCB->Data->subc_tree, 0);
 				return 0;
 			}
 #endif
@@ -494,14 +328,14 @@ static int ReportDialog(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 									"BoundingBox %$mD %$mD.\n"
 									"Refdes \"%s\".\n"
 									"%s", USER_UNITMASK,
-									subc->ID, pcb_strflg_f2s(subc->Flags, PCB_TYPE_ELEMENT, NULL),
+									subc->ID, pcb_strflg_f2s(subc->Flags, PCB_OBJ_SUBC, NULL, 0),
 									subc->BoundingBox.X1, subc->BoundingBox.Y1,
 									subc->BoundingBox.X2, subc->BoundingBox.Y2,
 									PCB_EMPTY(subc->refdes),
 									gen_locked(subc));
 			break;
 		}
-	case PCB_TYPE_TEXT:
+	case PCB_OBJ_TEXT:
 #ifndef NDEBUG
 		if (pcb_gui->shift_is_pressed()) {
 			pcb_layer_t *layer = (pcb_layer_t *) ptr1;
@@ -509,48 +343,18 @@ static int ReportDialog(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 			return 0;
 		}
 #endif
-	case PCB_TYPE_ELEMENT_NAME:
-		{
-			char laynum[32];
-			pcb_text_t *text;
-#ifndef NDEBUG
-			if (pcb_gui->shift_is_pressed()) {
-				pcb_r_dump_tree(PCB->Data->name_tree[PCB_ELEMNAME_IDX_VISIBLE()], 0);
-				return 0;
-			}
-#endif
-			text = (pcb_text_t *) ptr2;
-
-			if (type == PCB_TYPE_TEXT)
-				sprintf(laynum, "It is on layer %ld.", pcb_layer_id(PCB->Data, (pcb_layer_t *) ptr1));
-			report = pcb_strdup_printf("%m+TEXT ID# %ld;  Flags:%s\n"
-									"Located at (X,Y) = %$mD.\n"
-									"Characters are %$mS tall.\n"
-									"Value is \"%s\".\n"
-									"Direction is %d.\n"
-									"The bounding box is %$mD %$mD.\n"
-									"%s\n"
-									"%s"
-									"%s%s%s", USER_UNITMASK, text->ID, pcb_strflg_f2s(text->Flags, PCB_TYPE_TEXT, NULL),
-									text->X, text->Y, PCB_SCALE_TEXT(PCB_FONT_CAPHEIGHT, text->Scale),
-									text->TextString, text->Direction,
-									text->BoundingBox.X1, text->BoundingBox.Y1,
-									text->BoundingBox.X2, text->BoundingBox.Y2,
-									(type == PCB_TYPE_TEXT) ? laynum : "It is an element name.", gen_locked(text), gen_term(text));
-			break;
-		}
-	case PCB_TYPE_LINE_POINT:
-	case PCB_TYPE_POLY_POINT:
+	case PCB_OBJ_LINE_POINT:
+	case PCB_OBJ_POLY_POINT:
 		{
 			pcb_point_t *point = (pcb_point_t *) ptr2;
 			report = pcb_strdup_printf("%m+POINT ID# %ld.\n"
 									"Located at (X,Y) = %$mD.\n"
 									"It belongs to a %s on layer %d.\n", USER_UNITMASK, point->ID,
 									point->X, point->Y,
-									(type == PCB_TYPE_LINE_POINT) ? "line" : "polygon", pcb_layer_id(PCB->Data, (pcb_layer_t *) ptr1));
+									(type == PCB_OBJ_LINE_POINT) ? "line" : "polygon", pcb_layer_id(PCB->Data, (pcb_layer_t *) ptr1));
 			break;
 		}
-	case PCB_TYPE_NONE:
+	case PCB_OBJ_VOID:
 		report = NULL;
 		break;
 
@@ -578,7 +382,7 @@ static int ReportDialog(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 	return 0;
 }
 
-static int ReportFoundPins(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
+static int report_found_pins(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
 {
 	gds_t list;
 	int col = 0;
@@ -586,23 +390,7 @@ static int ReportFoundPins(int argc, const char **argv, pcb_coord_t x, pcb_coord
 	pcb_subc_t *subc;
 
 	gds_init(&list);
-	gds_append_str(&list, "The following pins/pads are FOUND:\n");
-	PCB_ELEMENT_LOOP(PCB->Data);
-	{
-		PCB_PIN_LOOP(element);
-		{
-			if (PCB_FLAG_TEST(PCB_FLAG_FOUND, pin))
-				pcb_append_printf(&list, "%s-%s,%c", PCB_ELEM_NAME_REFDES(element), pin->Number, ((col++ % (conf_report.plugins.report.columns + 1)) == conf_report.plugins.report.columns) ? '\n' : ' ');
-		}
-		PCB_END_LOOP;
-		PCB_PAD_LOOP(element);
-		{
-			if (PCB_FLAG_TEST(PCB_FLAG_FOUND, pad))
-				pcb_append_printf(&list, "%s-%s,%c", PCB_ELEM_NAME_REFDES(element), pad->Number, ((col++ % (conf_report.plugins.report.columns + 1)) == conf_report.plugins.report.columns) ? '\n' : ' ');
-		}
-		PCB_END_LOOP;
-	}
-	PCB_END_LOOP;
+	gds_append_str(&list, "The following terminals are FOUND:\n");
 
 	subclist_foreach(&PCB->Data->subc, &sit, subc) {
 		pcb_any_obj_t *o;
@@ -621,7 +409,7 @@ static int ReportFoundPins(int argc, const char **argv, pcb_coord_t x, pcb_coord
  * e.g. pcb_reset_conns() has been run.
  * Does not add its own changes to the undo system
  */
-static double XYtoNetLength(pcb_coord_t x, pcb_coord_t y, int *found)
+static double xy_to_net_length(pcb_coord_t x, pcb_coord_t y, int *found)
 {
 	double length;
 
@@ -662,10 +450,9 @@ static double XYtoNetLength(pcb_coord_t x, pcb_coord_t y, int *found)
 	return length;
 }
 
-static int ReportAllNetLengths(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
+static int report_all_net_lengths(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
 {
-	int ni;
-	int found;
+	int ni, found, want_undo;
 
 	/* Reset all connection flags and save an undo-state to get back
 	 * to the state the board was in when we started this function.
@@ -675,15 +462,16 @@ static int ReportAllNetLengths(int argc, const char **argv, pcb_coord_t x, pcb_c
 	 * by resetting the connections with pcb_reset_conns() before
 	 * calling pcb_undo() at the end of the procedure.
 	 */
-	pcb_reset_conns(pcb_true);
-	pcb_undo_inc_serial();
+	want_undo = pcb_reset_conns(pcb_true);
+	if (want_undo)
+		pcb_undo_inc_serial();
 
 	for (ni = 0; ni < PCB->NetlistLib[PCB_NETLIST_EDITED].MenuN; ni++) {
 		const char *netname = PCB->NetlistLib[PCB_NETLIST_EDITED].Menu[ni].Name + 2;
 		const char *list_entry = PCB->NetlistLib[PCB_NETLIST_EDITED].Menu[ni].Entry[0].ListEntry;
 		char *ename;
 		char *pname;
-		pcb_bool got_one = 0;
+		pcb_any_obj_t *term;
 
 		ename = pcb_strdup(list_entry);
 		pname = strchr(ename, '-');
@@ -693,62 +481,75 @@ static int ReportAllNetLengths(int argc, const char **argv, pcb_coord_t x, pcb_c
 		}
 		*pname++ = 0;
 
-		PCB_ELEMENT_LOOP(PCB->Data);
-		{
-			char *es = element->Name[PCB_ELEMNAME_IDX_REFDES].TextString;
-			if (es && strcmp(es, ename) == 0) {
-				PCB_PIN_LOOP(element);
-				{
-					if (strcmp(pin->Number, pname) == 0) {
-						x = pin->X;
-						y = pin->Y;
-						got_one = 1;
-						break;
-					}
-				}
-				PCB_END_LOOP;
-				PCB_PAD_LOOP(element);
-				{
-					if (strcmp(pad->Number, pname) == 0) {
-						x = (pad->Point1.X + pad->Point2.X) / 2;
-						y = (pad->Point1.Y + pad->Point2.Y) / 2;
-						got_one = 1;
-						break;
-					}
-				}
-				PCB_END_LOOP;
-			}
-		}
-		PCB_END_LOOP;
-
-		if (got_one) {
+		term = pcb_term_find_name(PCB, PCB->Data, PCB_LYT_COPPER, ename, pname, 0, NULL, NULL);
+		free(ename);
+		if (term != NULL) {
 			char buf[50];
 			const char *units_name = argv[0];
 			pcb_coord_t length;
 
+			pcb_obj_center(term, &x, &y);
+
 			if (argc < 1)
 				units_name = conf_core.editor.grid_unit->suffix;
 
-			length = XYtoNetLength(x, y, &found);
+			length = xy_to_net_length(x, y, &found);
 
 			/* Reset connectors for the next lookup */
 			pcb_reset_conns(pcb_false);
 
 			pcb_snprintf(buf, sizeof(buf), "%$m*", units_name, length);
-			pcb_gui->log("Net %s length %s\n", netname, buf);
+			pcb_message(PCB_MSG_INFO, "Net %s length %s\n", netname, buf);
 		}
 	}
 
 	pcb_reset_conns(pcb_false);
-	pcb_undo(pcb_true);
+	if (want_undo)
+		pcb_undo(pcb_true);
 	return 0;
 }
 
-static int ReportNetLength_(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
+/* find any found object with a terminal ID and return the associated netname */
+static const char *net_name_found(pcb_board_t *pcb)
+{
+	PCB_SUBC_LOOP(pcb->Data);
+	{
+		pcb_any_obj_t *o;
+		pcb_data_it_t it;
+
+		if (subc->refdes == NULL)
+			continue;
+
+		for(o = pcb_data_first(&it, subc->data, PCB_OBJ_CLASS_REAL); o != NULL; o = pcb_data_next(&it)) {
+			char *netn;
+			int ni, nei;
+
+			if (o->term == NULL)
+				continue;
+			if (!PCB_FLAG_TEST(PCB_FLAG_FOUND, o))
+				continue;
+
+			netn = pcb_concat(subc->refdes, "-", o->term, NULL);
+
+			for (ni = 0; ni < PCB->NetlistLib[PCB_NETLIST_EDITED].MenuN; ni++) {
+				for (nei = 0; nei < PCB->NetlistLib[PCB_NETLIST_EDITED].Menu[ni].EntryN; nei++) {
+					if (strcmp(PCB->NetlistLib[PCB_NETLIST_EDITED].Menu[ni].Entry[nei].ListEntry, netn) == 0) {
+						free(netn);
+						return PCB->NetlistLib[PCB_NETLIST_EDITED].Menu[ni].Name + 2;
+					}
+				}
+			}
+			free(netn);
+		}
+	}
+	PCB_END_LOOP;
+	return NULL;
+}
+
+static int report_net_length_(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
 {
 	pcb_coord_t length = 0;
-	char *netname = 0;
-	int found = 0;
+	int found = 0, want_undo, ret;
 
 	/* Reset all connection flags and save an undo-state to get back
 	 * to the state the board was in when we started this function.
@@ -758,85 +559,37 @@ static int ReportNetLength_(int argc, const char **argv, pcb_coord_t x, pcb_coor
 	 * by resetting the connections with pcb_reset_conns() before
 	 * calling pcb_undo() at the end of the procedure.
 	 */
-	pcb_reset_conns(pcb_true);
-	pcb_undo_inc_serial();
+	want_undo = pcb_reset_conns(pcb_true);
+	if (want_undo)
+		pcb_undo_inc_serial();
 
-	length = XYtoNetLength(x, y, &found);
+	length = xy_to_net_length(x, y, &found);
 
-	if (!found) {
-		pcb_reset_conns(pcb_false);
-		pcb_undo(pcb_true);
-		pcb_gui->log("No net under cursor.\n");
-		return 1;
-	}
-
-	PCB_ELEMENT_LOOP(PCB->Data);
-	{
-		PCB_PIN_LOOP(element);
-		{
-			if (PCB_FLAG_TEST(PCB_FLAG_FOUND, pin)) {
-				int ni, nei;
-				char *ename = element->Name[PCB_ELEMNAME_IDX_REFDES].TextString;
-				char *pname = pin->Number;
-				char *n;
-
-				if (ename && pname) {
-					n = pcb_concat(ename, "-", pname, NULL);
-					for (ni = 0; ni < PCB->NetlistLib[PCB_NETLIST_EDITED].MenuN; ni++)
-						for (nei = 0; nei < PCB->NetlistLib[PCB_NETLIST_EDITED].Menu[ni].EntryN; nei++) {
-							if (strcmp(PCB->NetlistLib[PCB_NETLIST_EDITED].Menu[ni].Entry[nei].ListEntry, n) == 0) {
-								netname = PCB->NetlistLib[PCB_NETLIST_EDITED].Menu[ni].Name + 2;
-								goto got_net_name;	/* four for loops deep */
-							}
-						}
-				}
-			}
-		}
-		PCB_END_LOOP;
-		PCB_PAD_LOOP(element);
-		{
-			if (PCB_FLAG_TEST(PCB_FLAG_FOUND, pad)) {
-				int ni, nei;
-				char *ename = element->Name[PCB_ELEMNAME_IDX_REFDES].TextString;
-				char *pname = pad->Number;
-				char *n;
-
-				if (ename && pname) {
-					n = pcb_concat(ename, "-", pname, NULL);
-					for (ni = 0; ni < PCB->NetlistLib[PCB_NETLIST_EDITED].MenuN; ni++)
-						for (nei = 0; nei < PCB->NetlistLib[PCB_NETLIST_EDITED].Menu[ni].EntryN; nei++) {
-							if (strcmp(PCB->NetlistLib[PCB_NETLIST_EDITED].Menu[ni].Entry[nei].ListEntry, n) == 0) {
-								netname = PCB->NetlistLib[PCB_NETLIST_EDITED].Menu[ni].Name + 2;
-								goto got_net_name;	/* four for loops deep */
-							}
-						}
-				}
-			}
-		}
-		PCB_END_LOOP;
-	}
-	PCB_END_LOOP;
-
-	goto noelem;
-
-got_net_name:
-	pcb_reset_conns(pcb_false);
-	pcb_undo(pcb_true);
-
-noelem:;
-	{
+	if (found) {
 		char buf[50];
+		const char *netname = net_name_found(PCB);
+
 		pcb_snprintf(buf, sizeof(buf), "%$m*", conf_core.editor.grid_unit->suffix, length);
 		if (netname)
-			pcb_gui->log("Net \"%s\" length: %s\n", netname, buf);
+			pcb_message(PCB_MSG_INFO, "Net \"%s\" length: %s\n", netname, buf);
 		else
-			pcb_gui->log("Net length: %s\n", buf);
+			pcb_message(PCB_MSG_INFO, "Net length: %s\n", buf);
+
+		ret = 0;
+	}
+	else {
+		pcb_message(PCB_MSG_ERROR, "No net under cursor.\n");
+		ret = 1;
 	}
 
-	return 0;
+	pcb_reset_conns(pcb_false);
+	if (want_undo)
+		pcb_undo(pcb_true);
+
+	return ret;
 }
 
-static int ReportNetLength(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y, int split)
+static int report_net_length(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y, int split)
 {
 
 	if (split) {
@@ -848,8 +601,8 @@ static int ReportNetLength(int argc, const char **argv, pcb_coord_t x, pcb_coord
 
 		pcb_gui->get_coords("Click on a copper line", &x, &y);
 
-		type = pcb_search_screen(x, y, PCB_TYPE_LINE, &r1, &r2, &r3);
-		if (type != PCB_TYPE_LINE) {
+		type = pcb_search_screen(x, y, PCB_OBJ_LINE, &r1, &r2, &r3);
+		if (type != PCB_OBJ_LINE) {
 			pcb_message(PCB_MSG_ERROR, "can't find a line to split\n");
 			return -1;
 		}
@@ -868,11 +621,11 @@ static int ReportNetLength(int argc, const char **argv, pcb_coord_t x, pcb_coord
 		}
 #undef MINDIST2
 
-		pcb_gui->log("The two arms of the net are:\n");
+		pcb_message(PCB_MSG_INFO, "The two arms of the net are:\n");
 		pcb_r_delete_entry(ly->line_tree, (pcb_box_t *)l);
 		ox = l->Point1.X; oy = l->Point1.Y; l->Point1.X = x; l->Point1.Y = y;
 		pcb_r_insert_entry(ly->line_tree, (pcb_box_t *)l);
-		ReportNetLength_(argc, argv, x, y);
+		report_net_length_(argc, argv, x, y);
 		pcb_r_delete_entry(ly->line_tree, (pcb_box_t *)l);
 		l->Point1.X = ox; l->Point1.Y = oy;
 		pcb_r_insert_entry(ly->line_tree, (pcb_box_t *)l);
@@ -880,7 +633,7 @@ static int ReportNetLength(int argc, const char **argv, pcb_coord_t x, pcb_coord
 		pcb_r_delete_entry(ly->line_tree, (pcb_box_t *)l);
 		ox = l->Point2.X; oy = l->Point2.Y; l->Point2.X = x; l->Point2.Y = y;
 		pcb_r_insert_entry(ly->line_tree, (pcb_box_t *)l);
-		ReportNetLength_(argc, argv, x, y);
+		report_net_length_(argc, argv, x, y);
 		pcb_r_delete_entry(ly->line_tree, (pcb_box_t *)l);
 		l->Point2.X = ox; l->Point2.Y = oy;
 		pcb_r_insert_entry(ly->line_tree, (pcb_box_t *)l);
@@ -891,17 +644,16 @@ static int ReportNetLength(int argc, const char **argv, pcb_coord_t x, pcb_coord
 	}
 	else {
 /*		pcb_gui->get_coords("Click on a connection", &x, &y);*/
-		return ReportNetLength_(argc, argv, x, y);
+		return report_net_length_(argc, argv, x, y);
 	}
 }
 
 
-static int ReportNetLengthByName(const char *tofind, pcb_coord_t x, pcb_coord_t y)
+static int report_net_length_by_name(const char *tofind, pcb_coord_t x, pcb_coord_t y)
 {
 	char *netname = 0;
 	pcb_coord_t length = 0;
-	int found = 0;
-	int i;
+	int found = 0, i, want_undo;
 	pcb_lib_menu_t *net;
 	pcb_connection_t conn;
 	int net_found = 0;
@@ -943,7 +695,7 @@ static int ReportNetLengthByName(const char *tofind, pcb_coord_t x, pcb_coord_t
 
 		if (pcb_rat_seek_pad(net->Entry, &conn, pcb_false)) {
 			pcb_obj_center(conn.obj, &x, &y);
-			if ((conn.obj->type == PCB_OBJ_PIN) || (conn.obj->type == PCB_OBJ_PAD) || (conn.obj->term != NULL)) {
+			if (conn.obj->term != NULL) {
 				net_found = 1;
 				break;
 			}
@@ -951,7 +703,7 @@ static int ReportNetLengthByName(const char *tofind, pcb_coord_t x, pcb_coord_t
 	}
 
 	if (!net_found) {
-		pcb_gui->log("No net named %s\n", tofind);
+		pcb_message(PCB_MSG_ERROR, "No net named %s\n", tofind);
 		return 1;
 	}
 
@@ -966,20 +718,22 @@ static int ReportNetLengthByName(const char *tofind, pcb_coord_t x, pcb_coord_t
 	 * by resetting the connections with pcb_reset_conns() before
 	 * calling pcb_undo() when we are finished.
 	 */
-	pcb_reset_conns(pcb_true);
-	pcb_undo_inc_serial();
+	want_undo = pcb_reset_conns(pcb_true);
+	if (want_undo)
+		pcb_undo_inc_serial();
 
-	length = XYtoNetLength(x, y, &found);
+	length = xy_to_net_length(x, y, &found);
 	netname = net->Name + 2;
 
 	pcb_reset_conns(pcb_false);
-	pcb_undo(pcb_true);
+	if (want_undo)
+		pcb_undo(pcb_true);
 
 	if (!found) {
 		if (net_found)
-			pcb_gui->log("Net found, but no lines or arcs were flagged.\n");
+			pcb_message(PCB_MSG_INFO, "Net found, but no lines or arcs were flagged.\n");
 		else
-			pcb_gui->log("Net not found.\n");
+			pcb_message(PCB_MSG_ERROR, "Net not found.\n");
 
 		return 1;
 	}
@@ -988,9 +742,9 @@ static int ReportNetLengthByName(const char *tofind, pcb_coord_t x, pcb_coord_t
 		char buf[50];
 		pcb_snprintf(buf, sizeof(buf), "%$m*", conf_core.editor.grid_unit->suffix, length);
 		if (netname)
-			pcb_gui->log("Net \"%s\" length: %s\n", netname, buf);
+			pcb_message(PCB_MSG_INFO, "Net \"%s\" length: %s\n", netname, buf);
 		else
-			pcb_gui->log("Net length: %s\n", buf);
+			pcb_message(PCB_MSG_INFO, "Net length: %s\n", buf);
 	}
 
 	return 0;
@@ -1034,30 +788,30 @@ units
 
 %end-doc */
 
-static int Report(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
+static int pcb_act_report(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
 {
 	if ((argc < 1) || (argc > 2))
 		AUSAGE(report);
 	else if (pcb_strcasecmp(argv[0], "Object") == 0) {
 		pcb_gui->get_coords("Click on an object", &x, &y);
-		return ReportDialog(argc, argv, x, y);
+		return report_dialog(argc, argv, x, y);
 	}
 	else if (pcb_strncasecmp(argv[0], "Subc", 4) == 0) {
 		pcb_gui->get_coords("Click on a subcircuit", &x, &y);
-		return ReportDialog(argc, argv, x, y);
+		return report_dialog(argc, argv, x, y);
 	}
 	else if (pcb_strcasecmp(argv[0], "DrillReport") == 0)
-		return ReportDrills(argc - 1, argv + 1, x, y);
+		return report_drills(argc - 1, argv + 1, x, y);
 	else if (pcb_strcasecmp(argv[0], "FoundPins") == 0)
-		return ReportFoundPins(argc - 1, argv + 1, x, y);
+		return report_found_pins(argc - 1, argv + 1, x, y);
 	else if ((pcb_strcasecmp(argv[0], "NetLength") == 0) && (argc == 1))
-		return ReportNetLength(argc - 1, argv + 1, x, y, 0);
+		return report_net_length(argc - 1, argv + 1, x, y, 0);
 	else if ((pcb_strcasecmp(argv[0], "NetLengthTo") == 0) && (argc == 1))
-		return ReportNetLength(argc - 1, argv + 1, x, y, 1);
+		return report_net_length(argc - 1, argv + 1, x, y, 1);
 	else if (pcb_strcasecmp(argv[0], "AllNetLengths") == 0)
-		return ReportAllNetLengths(argc - 1, argv + 1, x, y);
+		return report_all_net_lengths(argc - 1, argv + 1, x, y);
 	else if ((pcb_strcasecmp(argv[0], "NetLength") == 0) && (argc == 2))
-		return ReportNetLengthByName(argv[1], x, y);
+		return report_net_length_by_name(argv[1], x, y);
 	else if (argc == 2)
 		AUSAGE(report);
 	else
@@ -1066,10 +820,10 @@ static int Report(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
 }
 
 pcb_hid_action_t report_action_list[] = {
-	{"ReportObject", "Click on an object", ReportDialog,
+	{"ReportObject", "Click on an object", report_dialog,
 	 reportdialog_help, reportdialog_syntax}
 	,
-	{"Report", 0, Report,
+	{"Report", 0, pcb_act_report,
 	 report_help, report_syntax}
 };
 
@@ -1088,6 +842,7 @@ void pplg_uninit_report(void)
 #include "dolists.h"
 int pplg_init_report(void)
 {
+	PCB_API_CHK_VER;
 	PCB_REGISTER_ACTIONS(report_action_list, report_cookie)
 #define conf_reg(field,isarray,type_name,cpath,cname,desc,flags) \
 	conf_reg_field(conf_report, field,isarray,type_name,cpath,cname,desc,flags);
