@@ -32,8 +32,8 @@
 #include "config.h"
 
 #define REPORT_TYPES \
-	(PCB_TYPE_PSTK | PCB_TYPE_VIA | PCB_TYPE_LINE | PCB_TYPE_TEXT | PCB_TYPE_POLY | PCB_TYPE_ELEMENT | \
-	 PCB_TYPE_RATLINE | PCB_TYPE_PIN | PCB_TYPE_PAD | PCB_TYPE_ELEMENT_NAME | PCB_TYPE_ARC \
-	 | PCB_TYPE_POLY_POINT | PCB_TYPE_LINE_POINT | PCB_TYPE_SUBC_PART | PCB_TYPE_SUBC)
+	(PCB_OBJ_PSTK | PCB_OBJ_LINE | PCB_OBJ_TEXT | PCB_OBJ_POLY | \
+	 PCB_OBJ_ARC | PCB_OBJ_RAT | PCB_OBJ_POLY_POINT | \
+	 PCB_OBJ_LINE_POINT | PCB_OBJ_SUBC_PART | PCB_OBJ_SUBC)
 
 #endif
