@@ -112,8 +112,6 @@ static pcb_rubberband_t *pcb_rubber_band_alloc(rubber_ctx_t *rbnd);
 static pcb_rubberband_arc_t *pcb_rubber_band_arc_alloc(rubber_ctx_t *rbnd);
 static pcb_rubberband_t *pcb_rubber_band_create(rubber_ctx_t *rbnd, pcb_layer_t *Layer, pcb_line_t *Line, int point_number,int delta_index);
 static pcb_rubberband_arc_t *pcb_rubber_band_create_arc(rubber_ctx_t *rbnd, pcb_layer_t *Layer, pcb_arc_t *Line, int end);
-static void CheckPadForRubberbandConnection(rubber_ctx_t *rbnd, pcb_pad_t *);
-static void CheckPinForRubberbandConnection(rubber_ctx_t *rbnd, pcb_pin_t *);
 static void CheckLinePointForRubberbandConnection(rubber_ctx_t *rbnd, pcb_layer_t *, pcb_line_t *, pcb_point_t *,int delta_index);
 static void CheckArcPointForRubberbandConnection(rubber_ctx_t *rbnd, pcb_layer_t *, pcb_arc_t *, int *, pcb_bool);
 static void CheckArcForRubberbandConnection(rubber_ctx_t *rbnd, pcb_layer_t *, pcb_arc_t *, pcb_bool);
@@ -346,47 +344,9 @@ static pcb_r_dir_t rubber_callback_arc(const pcb_box_t * b, void *cl)
 	return PCB_R_DIR_FOUND_CONTINUE;
 }
 
-/* ---------------------------------------------------------------------------
- * checks all visible lines which belong to the same layergroup as the
- * passed pad. If one of the endpoints of the line lays inside the pad,
- * the line is added to the 'rubberband' list
- */
-static void CheckPadForRubberbandConnection(rubber_ctx_t *rbnd, pcb_pad_t *Pad)
-{
-	pcb_coord_t half = Pad->Thickness / 2;
-	pcb_layergrp_id_t group;
-	struct rubber_info info;
-	unsigned int flg;
-
-	info.box.X1 = MIN(Pad->Point1.X, Pad->Point2.X) - half;
-	info.box.Y1 = MIN(Pad->Point1.Y, Pad->Point2.Y) - half;
-	info.box.X2 = MAX(Pad->Point1.X, Pad->Point2.X) + half;
-	info.box.Y2 = MAX(Pad->Point1.Y, Pad->Point2.Y) + half;
-	info.radius = 0;
-	info.line = NULL;
-	info.rbnd = rbnd;
-	info.delta_index = 0;
-	flg = PCB_FLAG_TEST(PCB_FLAG_ONSOLDER, Pad) ? PCB_LYT_BOTTOM : PCB_LYT_TOP;
-	if (pcb_layergrp_list(PCB, flg | PCB_LYT_COPPER, &group, 1) < 1)
-		return;
-
-	/* check all visible layers in the same group */
-	PCB_COPPER_GROUP_LOOP(PCB->Data, group);
-	{
-		/* check all visible lines of the group member */
-		info.layer = layer;
-		if (info.layer->meta.real.vis) {
-			pcb_r_search(info.layer->line_tree, &info.box, NULL, rubber_callback, &info, NULL);
-		}
-	}
-	PCB_END_LOOP;
-}
-
 struct rinfo {
 	int type;
 	pcb_layergrp_id_t group;
-	pcb_pin_t *pin;
-	pcb_pad_t *pad;
 	pcb_pstk_t *pstk;
 	pcb_point_t *point;
 	rubber_ctx_t *rbnd;
@@ -400,37 +360,13 @@ static pcb_r_dir_t rat_callback(const pcb_box_t * box, void *cl)
 	rubber_ctx_t *rbnd = i->rbnd;
 
 	switch (i->type) {
-	case PCB_TYPE_PIN:
-		if (rat->Point1.X == i->pin->X && rat->Point1.Y == i->pin->Y)
-			pcb_rubber_band_create(rbnd, NULL, (pcb_line_t *) rat, 0,i->delta_index);
-		else if (rat->Point2.X == i->pin->X && rat->Point2.Y == i->pin->Y)
-			pcb_rubber_band_create(rbnd, NULL, (pcb_line_t *) rat, 1,i->delta_index);
-		break;
-	case PCB_TYPE_PAD:
-		if (rat->Point1.X == i->pad->Point1.X && rat->Point1.Y == i->pad->Point1.Y && rat->group1 == i->group)
-			pcb_rubber_band_create(rbnd, NULL, (pcb_line_t *) rat, 0,i->delta_index);
-		else if (rat->Point2.X == i->pad->Point1.X && rat->Point2.Y == i->pad->Point1.Y && rat->group2 == i->group)
-			pcb_rubber_band_create(rbnd, NULL, (pcb_line_t *) rat, 1,i->delta_index);
-		else if (rat->Point1.X == i->pad->Point2.X && rat->Point1.Y == i->pad->Point2.Y && rat->group1 == i->group)
-			pcb_rubber_band_create(rbnd, NULL, (pcb_line_t *) rat, 0,i->delta_index);
-		else if (rat->Point2.X == i->pad->Point2.X && rat->Point2.Y == i->pad->Point2.Y && rat->group2 == i->group)
-			pcb_rubber_band_create(rbnd, NULL, (pcb_line_t *) rat, 1,i->delta_index);
-		else
-			if (rat->Point1.X == (i->pad->Point1.X + i->pad->Point2.X) / 2 &&
-					rat->Point1.Y == (i->pad->Point1.Y + i->pad->Point2.Y) / 2 && rat->group1 == i->group)
-			pcb_rubber_band_create(rbnd, NULL, (pcb_line_t *) rat, 0,i->delta_index);
-		else
-			if (rat->Point2.X == (i->pad->Point1.X + i->pad->Point2.X) / 2 &&
-					rat->Point2.Y == (i->pad->Point1.Y + i->pad->Point2.Y) / 2 && rat->group2 == i->group)
-			pcb_rubber_band_create(rbnd, NULL, (pcb_line_t *) rat, 1,i->delta_index);
-		break;
-	case PCB_TYPE_PSTK:
+	case PCB_OBJ_PSTK:
 		if (rat->Point1.X == i->pstk->x && rat->Point1.Y == i->pstk->y)
 			pcb_rubber_band_create(rbnd, NULL, (pcb_line_t *) rat, 0,i->delta_index);
 		else if (rat->Point2.X == i->pstk->x && rat->Point2.Y == i->pstk->y)
 			pcb_rubber_band_create(rbnd, NULL, (pcb_line_t *) rat, 1,i->delta_index);
 		break;
-	case PCB_TYPE_LINE_POINT:
+	case PCB_OBJ_LINE_POINT:
 		if (rat->group1 == i->group && rat->Point1.X == i->point->X && rat->Point1.Y == i->point->Y)
 			pcb_rubber_band_create(rbnd, NULL, (pcb_line_t *) rat, 0,i->delta_index);
 		else if (rat->group2 == i->group && rat->Point2.X == i->point->X && rat->Point2.Y == i->point->Y)
@@ -442,40 +378,11 @@ static pcb_r_dir_t rat_callback(const pcb_box_t * box, void *cl)
 	return PCB_R_DIR_NOT_FOUND;
 }
 
-static void CheckPadForRat(rubber_ctx_t *rbnd, pcb_pad_t *Pad)
-{
-	struct rinfo info;
-	unsigned int flg;
-	pcb_layergrp_id_t group;
-
-	flg = PCB_FLAG_TEST(PCB_FLAG_ONSOLDER, Pad) ? PCB_LYT_BOTTOM : PCB_LYT_TOP;
-	if (pcb_layergrp_list(PCB, flg | PCB_LYT_COPPER, &group, 1) < 1)
-		return;
-
-	info.group = group;
-	info.pad = Pad;
-	info.type = PCB_TYPE_PAD;
-	info.rbnd = rbnd;
-	info.delta_index = 0;
-	pcb_r_search(PCB->Data->rat_tree, &Pad->BoundingBox, NULL, rat_callback, &info, NULL);
-}
-
-static void CheckPinForRat(rubber_ctx_t *rbnd, pcb_pin_t *Pin)
-{
-	struct rinfo info;
-
-	info.type = PCB_TYPE_PIN;
-	info.pin = Pin;
-	info.rbnd = rbnd;
-	info.delta_index = 0;
-	pcb_r_search(PCB->Data->rat_tree, &Pin->BoundingBox, NULL, rat_callback, &info, NULL);
-}
-
 static void CheckPadstackForRat(rubber_ctx_t *rbnd, pcb_pstk_t *pstk)
 {
 	struct rinfo info;
 
-	info.type = PCB_TYPE_PSTK;
+	info.type = PCB_OBJ_PSTK;
 	info.pstk = pstk;
 	info.rbnd = rbnd;
 	info.delta_index = 0;
@@ -487,51 +394,13 @@ static void CheckLinePointForRat(rubber_ctx_t *rbnd, pcb_layer_t *Layer, pcb_poi
 	struct rinfo info;
 	info.group = pcb_layer_get_group_(Layer);
 	info.point = Point;
-	info.type = PCB_TYPE_LINE_POINT;
+	info.type = PCB_OBJ_LINE_POINT;
 	info.rbnd = rbnd;
 	info.delta_index = 0;
 
 	pcb_r_search(PCB->Data->rat_tree, (pcb_box_t *) Point, NULL, rat_callback, &info, NULL);
 }
 
-/* ---------------------------------------------------------------------------
- * checks all visible lines. If one of the endpoints of the line lays
- * inside the pin, the line is added to the 'rubberband' list
- *
- * Square pins are handled as if they were round. Speed
- * and readability is more important then the few %
- * of failures that are immediately recognized
- */
-static void CheckPinForRubberbandConnection(rubber_ctx_t *rbnd, pcb_pin_t *Pin)
-{
-	struct rubber_info info;
-	pcb_cardinal_t n;
-	pcb_coord_t t = Pin->Thickness / 2;
-
-	info.box.X1 = Pin->X - t;
-	info.box.X2 = Pin->X + t;
-	info.box.Y1 = Pin->Y - t;
-	info.box.Y2 = Pin->Y + t;
-	info.line = NULL;
-	info.rbnd = rbnd;
-	info.delta_index = 0;
-
-	if (PCB_FLAG_TEST(PCB_FLAG_SQUARE, Pin))
-		info.radius = 0;
-	else {
-		info.radius = t;
-		info.X = Pin->X;
-		info.Y = Pin->Y;
-	}
-
-	for (n = 0; n < pcb_max_layer; n++) {
-		if (pcb_layer_flags(PCB, n) & PCB_LYT_COPPER) {
-			info.layer = LAYER_PTR(n);
-			pcb_r_search(info.layer->line_tree, &info.box, NULL, rubber_callback, &info, NULL);
-		}
-	}
-}
-
 /* ---------------------------------------------------------------------------
  * checks all visible lines which belong to the same group as the passed line.
  * If one of the endpoints of the line is connected to an endpoint of the 
@@ -725,12 +594,13 @@ static void CheckArcForRubberbandConnection(rubber_ctx_t *rbnd, pcb_layer_t *Lay
 static void CheckEntireArcForRubberbandConnection(rubber_ctx_t *rbnd, pcb_layer_t * Layer, pcb_arc_t * Arc)
 {
 	pcb_board_t * board = pcb_data_get_top(PCB->Data);
+	pcb_layergrp_id_t group;
 
 	if(board == NULL)
 		board = PCB;
 
 	/* lookup layergroup and check all visible lines in this group */
-	pcb_layergrp_id_t group = pcb_layer_get_group_(Layer);
+	group = pcb_layer_get_group_(Layer);
 
 	if(group >= 0) {
 		pcb_cardinal_t length = board->LayerGroups.grp[group].len;
@@ -1001,11 +871,11 @@ static void pcb_rubber_band_lookup_lines(rubber_ctx_t *rbnd, int Type, void *Ptr
 	 * is connected
 	 */
 	switch (Type) {
-	case PCB_TYPE_PSTK:
+	case PCB_OBJ_PSTK:
 		CheckPadStackForRubberbandConnection(rbnd,(pcb_pstk_t *)Ptr1);
 		break;
 
-	case PCB_TYPE_SUBC:
+	case PCB_OBJ_SUBC:
 		{
 			pcb_subc_t * subc = (pcb_subc_t *)Ptr1;
 			pcb_data_it_t	it;
@@ -1029,28 +899,7 @@ static void pcb_rubber_band_lookup_lines(rubber_ctx_t *rbnd, int Type, void *Ptr
 			break;
 		}
 
-	case PCB_TYPE_ELEMENT:
-		{
-			pcb_element_t *element = (pcb_element_t *) Ptr1;
-
-			/* square pins are handled as if they are round. Speed
-			 * and readability is more important then the few %
-			 * of failures that are immediately recognized
-			 */
-			PCB_PIN_LOOP(element);
-			{
-				CheckPinForRubberbandConnection(rbnd, pin);
-			}
-			PCB_END_LOOP;
-			PCB_PAD_LOOP(element);
-			{
-				CheckPadForRubberbandConnection(rbnd, pad);
-			}
-			PCB_END_LOOP;
-			break;
-		}
-
-	case PCB_TYPE_LINE:
+	case PCB_OBJ_LINE:
 		{
 			pcb_layer_t *layer = (pcb_layer_t *) Ptr1;
 			pcb_line_t *line = (pcb_line_t *) Ptr2;
@@ -1059,25 +908,21 @@ static void pcb_rubber_band_lookup_lines(rubber_ctx_t *rbnd, int Type, void *Ptr
 			break;
 		}
 
-	case PCB_TYPE_LINE_POINT:
+	case PCB_OBJ_LINE_POINT:
 		CheckLinePointForRubberbandConnection(rbnd, (pcb_layer_t *) Ptr1, (pcb_line_t *) Ptr2, (pcb_point_t *) Ptr3,0);
 		if(conf_rbo.plugins.rubberband_orig.enable_rubberband_arcs != 0)
 			CheckLinePointForRubberbandArcConnection(rbnd, (pcb_layer_t *) Ptr1, (pcb_line_t *) Ptr2, (pcb_point_t *) Ptr3, pcb_true);
 		break;
 
-	case PCB_TYPE_ARC_POINT:
+	case PCB_OBJ_ARC_POINT:
 		CheckArcPointForRubberbandConnection(rbnd, (pcb_layer_t *) Ptr1, (pcb_arc_t *) Ptr2, (int *) Ptr3, pcb_true);
 		break;
 
-	case PCB_TYPE_ARC:
+	case PCB_OBJ_ARC:
 		CheckArcForRubberbandConnection(rbnd, (pcb_layer_t *) Ptr1, (pcb_arc_t *) Ptr2, pcb_true);
 		break;
 
-	case PCB_TYPE_VIA:
-		CheckPinForRubberbandConnection(rbnd, (pcb_pin_t *) Ptr1);
-		break;
-
-	case PCB_TYPE_POLY:
+	case PCB_OBJ_POLY:
 		CheckPolygonForRubberbandConnection(rbnd, (pcb_layer_t *) Ptr1, (pcb_poly_t *) Ptr2);
 		break;
 	}
@@ -1086,24 +931,8 @@ static void pcb_rubber_band_lookup_lines(rubber_ctx_t *rbnd, int Type, void *Ptr
 static void pcb_rubber_band_lookup_rat_lines(rubber_ctx_t *rbnd, int Type, void *Ptr1, void *Ptr2, void *Ptr3)
 {
 	switch (Type) {
-	case PCB_TYPE_ELEMENT:
-		{
-			pcb_element_t *element = (pcb_element_t *) Ptr1;
 
-			PCB_PIN_LOOP(element);
-			{
-				CheckPinForRat(rbnd, pin);
-			}
-			PCB_END_LOOP;
-			PCB_PAD_LOOP(element);
-			{
-				CheckPadForRat(rbnd, pad);
-			}
-			PCB_END_LOOP;
-			break;
-		}
-
-	case PCB_TYPE_SUBC:
+	case PCB_OBJ_SUBC:
 		{
 			pcb_subc_t *subc = (pcb_subc_t *) Ptr1;
 
@@ -1115,7 +944,7 @@ static void pcb_rubber_band_lookup_rat_lines(rubber_ctx_t *rbnd, int Type, void
 			break;
 		}
 
-	case PCB_TYPE_LINE:
+	case PCB_OBJ_LINE:
 		{
 			pcb_layer_t *layer = (pcb_layer_t *) Ptr1;
 			pcb_line_t *line = (pcb_line_t *) Ptr2;
@@ -1125,13 +954,10 @@ static void pcb_rubber_band_lookup_rat_lines(rubber_ctx_t *rbnd, int Type, void
 			break;
 		}
 
-	case PCB_TYPE_LINE_POINT:
+	case PCB_OBJ_LINE_POINT:
 		CheckLinePointForRat(rbnd, (pcb_layer_t *) Ptr1, (pcb_point_t *) Ptr3);
 		break;
 
-	case PCB_TYPE_VIA:
-		CheckPinForRat(rbnd, (pcb_pin_t *) Ptr1);
-		break;
 	}
 }
 
@@ -1212,12 +1038,12 @@ static void rbe_reset(void *user_data, int argc, pcb_event_arg_t argv[])
 	rbnd->RubberbandArcN = 0;
 }
 
-static void rbe_remove_element(void *user_data, int argc, pcb_event_arg_t argv[])
+static void rbe_remove_subc(void *user_data, int argc, pcb_event_arg_t argv[])
 {
 	rubber_ctx_t *rbnd = user_data;
 	pcb_rubberband_t *ptr;
 	void *ptr1 = argv[1].d.p, *ptr2 = argv[2].d.p, *ptr3 = argv[3].d.p;
-	int i, type = PCB_TYPE_ELEMENT;
+	int i, type = PCB_OBJ_SUBC;
 
 	rbnd->RubberbandN = 0;
 	pcb_rubber_band_lookup_rat_lines(rbnd, type, ptr1, ptr2, ptr3);
@@ -1226,7 +1052,7 @@ static void rbe_remove_element(void *user_data, int argc, pcb_event_arg_t argv[]
 		if (PCB->RatOn)
 			pcb_rat_invalidate_erase((pcb_rat_t *) ptr->Line);
 
-		pcb_undo_move_obj_to_remove(PCB_TYPE_RATLINE, ptr->Line, ptr->Line, ptr->Line);
+		pcb_undo_move_obj_to_remove(PCB_OBJ_RAT, ptr->Line, ptr->Line, ptr->Line);
 		ptr++;
 	}
 }
@@ -1278,7 +1104,7 @@ static void rbe_move(void *user_data, int argc, pcb_event_arg_t argv[])
 				ctx.move.dy = argv[argi+1].d.i;
 	
 				if(direct) {
-					pcb_undo_add_obj_to_move(PCB_TYPE_LINE_POINT, ptr->Layer, ptr->Line, &ptr->Line->Point1, ctx.move.dx, ctx.move.dy);			
+					pcb_undo_add_obj_to_move(PCB_OBJ_LINE_POINT, ptr->Layer, ptr->Line, &ptr->Line->Point1, ctx.move.dx, ctx.move.dy);			
 					pcb_lineop_move_point(&ctx, ptr->Layer, ptr->Line, &ptr->Line->Point1);
 				}
 				else
@@ -1293,7 +1119,7 @@ static void rbe_move(void *user_data, int argc, pcb_event_arg_t argv[])
 				ctx.move.dy = argv[argi+1].d.i;
 	
 				if(direct) {
-					pcb_undo_add_obj_to_move(PCB_TYPE_LINE_POINT, ptr->Layer, ptr->Line, &ptr->Line->Point2, ctx.move.dx, ctx.move.dy);			
+					pcb_undo_add_obj_to_move(PCB_OBJ_LINE_POINT, ptr->Layer, ptr->Line, &ptr->Line->Point2, ctx.move.dx, ctx.move.dy);			
 					pcb_lineop_move_point(&ctx, ptr->Layer, ptr->Line, &ptr->Line->Point2);
 				}
 				else
@@ -1355,7 +1181,7 @@ static void rbe_draw(void *user_data, int argc, pcb_event_arg_t argv[])
 				/* Draw the DRC outline if it is enabled */
 				if (conf_core.editor.show_drc) {
 					pcb_gui->set_color(pcb_crosshair.GC, conf_core.appearance.color.cross);
-					pcb_draw_wireframe_line(pcb_crosshair.GC,x[0],y[0],x[1],y[1],ptr->Line->Thickness + 2 * (PCB->Bloat + 1), 0);
+					pcb_draw_wireframe_line(pcb_crosshair.GC,x[0],y[0],x[1],y[1],ptr->Line->Thickness + 2 * (conf_core.design.bloat + 1), 0);
 				}
 			}
 			else {
@@ -1429,13 +1255,13 @@ static void rbe_rotate90(void *user_data, int argc, pcb_event_arg_t argv[])
 		*changed = 1;
 		
 		if(dindex1 >= 0) 
-			pcb_undo_add_obj_to_rotate90(PCB_TYPE_LINE_POINT, ptr->Layer, ptr->Line, &ptr->Line->Point1, cx, cy, steps);
+			pcb_undo_add_obj_to_rotate90(PCB_OBJ_LINE_POINT, ptr->Layer, ptr->Line, &ptr->Line->Point1, cx, cy, steps);
 		if(dindex2 >= 0) 
-			pcb_undo_add_obj_to_rotate90(PCB_TYPE_LINE_POINT, ptr->Layer, ptr->Line, &ptr->Line->Point2, cx, cy, steps);
+			pcb_undo_add_obj_to_rotate90(PCB_OBJ_LINE_POINT, ptr->Layer, ptr->Line, &ptr->Line->Point2, cx, cy, steps);
 
 		pcb_line_invalidate_erase(ptr->Line);
 		if (ptr->Layer) {
-			pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_LINE, ptr->Layer, ptr->Line);
+			pcb_poly_restore_to_poly(PCB->Data, PCB_OBJ_LINE, ptr->Layer, ptr->Line);
 			pcb_r_delete_entry(ptr->Layer->line_tree, (pcb_box_t *) ptr->Line);
 		}
 		else
@@ -1450,7 +1276,7 @@ static void rbe_rotate90(void *user_data, int argc, pcb_event_arg_t argv[])
 		pcb_line_bbox(ptr->Line);
 		if (ptr->Layer) {
 			pcb_r_insert_entry(ptr->Layer->line_tree, (pcb_box_t *) ptr->Line);
-			pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_LINE, ptr->Layer, ptr->Line);
+			pcb_poly_clear_from_poly(PCB->Data, PCB_OBJ_LINE, ptr->Layer, ptr->Line);
 			pcb_line_invalidate_draw(ptr->Layer, ptr->Line);
 		}
 		else {
@@ -1470,12 +1296,14 @@ static void rbe_rotate(void *user_data, int argc, pcb_event_arg_t argv[])
 
 static void rbe_rename(void *user_data, int argc, pcb_event_arg_t argv[])
 {
+#warning TODO: rewrite this for subc - also move it out from rubberband, it has nothing to do with the feature - should be some generic rat line invalidation mechanism
+#if 0
 	rubber_ctx_t *rbnd = user_data;
 	int type = argv[1].d.i;
 	void *ptr1 = argv[2].d.p, *ptr2 = argv[3].d.p, *ptr3 = argv[4].d.p;
 /*	int pinnum = argv[5].d.i;*/
 
-	if (type == PCB_TYPE_ELEMENT) {
+	if (type == PCB_OBJ_ELEMENT) {
 		pcb_rubberband_t *ptr;
 		int i;
 
@@ -1486,11 +1314,12 @@ static void rbe_rename(void *user_data, int argc, pcb_event_arg_t argv[])
 		for (i = 0; i < rbnd->RubberbandN; i++, ptr++) {
 			if (PCB->RatOn)
 				pcb_rat_invalidate_erase((pcb_rat_t *) ptr->Line);
-			pcb_undo_move_obj_to_remove(PCB_TYPE_RATLINE, ptr->Line, ptr->Line, ptr->Line);
+			pcb_undo_move_obj_to_remove(PCB_OBJ_RAT, ptr->Line, ptr->Line, ptr->Line);
 		}
 		pcb_undo_inc_serial();
 		pcb_draw();
 	}
+#endif
 }
 
 static void rbe_lookup_lines(void *user_data, int argc, pcb_event_arg_t argv[])
@@ -1517,10 +1346,10 @@ static void rbe_constrain_main_line(void *user_data, int argc, pcb_event_arg_t a
 	rubber_ctx_t *rbnd = user_data;
 	pcb_line_t *line = argv[1].d.p;
 	int * constrained = argv[2].d.p;
-	int * dx1 = argv[3].d.p;	/* in/out */
-	int * dy1 = argv[4].d.p;  /* in/out */
-	int * dx2 = argv[5].d.p;	/* out */
-	int * dy2 = argv[6].d.p;	/* out */
+	pcb_coord_t * dx1 = argv[3].d.p;	/* in/out */
+	pcb_coord_t * dy1 = argv[4].d.p;	/* in/out */
+	pcb_coord_t * dx2 = argv[5].d.p;	/* out */
+	pcb_coord_t * dy2 = argv[6].d.p;	/* out */
 	pcb_line_t *rub1, *rub2;
 	int rub1end, rub2end;
 	pcb_fline_t fmain, frub1, frub2;
@@ -1601,8 +1430,9 @@ void pplg_uninit_rubberband_orig(void)
 int pplg_init_rubberband_orig(void)
 {
 	void *ctx = &rubber_band_state;
+	PCB_API_CHK_VER;
 	pcb_event_bind(PCB_EVENT_RUBBER_RESET, rbe_reset, ctx, rubber_cookie);
-	pcb_event_bind(PCB_EVENT_RUBBER_REMOVE_ELEMENT, rbe_remove_element, ctx, rubber_cookie);
+	pcb_event_bind(PCB_EVENT_RUBBER_REMOVE_SUBC, rbe_remove_subc, ctx, rubber_cookie);
 	pcb_event_bind(PCB_EVENT_RUBBER_MOVE, rbe_move, ctx, rubber_cookie);
 	pcb_event_bind(PCB_EVENT_RUBBER_MOVE_DRAW, rbe_draw, ctx, rubber_cookie);
 	pcb_event_bind(PCB_EVENT_RUBBER_ROTATE90, rbe_rotate90, ctx, rubber_cookie);
