@@ -49,6 +49,7 @@
 #include "hid_actions.h"
 #include "compat_misc.h"
 #include "misc_util.h"
+#include "tool.h"
 
 /* ---------------------------------------------------------------------- */
 
@@ -136,7 +137,7 @@ static int CommandLoadLayout(int argc, const char **argv, pcb_coord_t x, pcb_coo
 
 static const char le_syntax[] = "le [name]";
 
-static const char le_help[] = "Loads an element into the current buffer.";
+static const char le_help[] = "Loads an element (subcircuit, footprint) into the current buffer.";
 
 /* %start-doc actions le
 
@@ -154,12 +155,12 @@ static int CommandLoadElementToBuffer(int argc, const char **argv, pcb_coord_t x
 	switch (argc) {
 	case 1:											/* filename is passed in commandline */
 		filename = argv[0];
-		if (filename && pcb_element_load_to_buffer(PCB_PASTEBUFFER, filename, NULL))
-			pcb_crosshair_set_mode(PCB_MODE_PASTE_BUFFER);
+		if (filename && pcb_buffer_load_footprint(PCB_PASTEBUFFER, filename, NULL))
+			pcb_tool_select_by_id(PCB_MODE_PASTE_BUFFER);
 		break;
 
 	default:											/* usage */
-		pcb_message(PCB_MSG_ERROR, pcb_false, "Usage: le [name]\n  loads element data to buffer\n");
+		pcb_message(PCB_MSG_ERROR, pcb_false, "Usage: le [name]\n  loads element (subcircuit, footprint) data to buffer\n");
 		return 1;
 	}
 	return 0;
@@ -191,7 +192,7 @@ static int CommandLoadLayoutToBuffer(int argc, const char **argv, pcb_coord_t x,
 	case 1:  /* filename is passed in commandline */
 		filename = argv[0];
 		if (filename && pcb_buffer_load_layout(PCB, PCB_PASTEBUFFER, filename, format))
-			pcb_crosshair_set_mode(PCB_MODE_PASTE_BUFFER);
+			pcb_tool_select_by_id(PCB_MODE_PASTE_BUFFER);
 		break;
 
 	default:  /* usage */
@@ -412,6 +413,7 @@ void pplg_uninit_shand_cmd(void)
 #include "dolists.h"
 int pplg_init_shand_cmd(void)
 {
+	PCB_API_CHK_VER;
 	PCB_REGISTER_ACTIONS(shand_cmd_action_list, shand_cmd_cookie)
 	return 0;
 }
