@@ -44,6 +44,7 @@
 #include "obj_poly.h"
 #include "obj_poly_draw.h"
 #include "rotate.h"
+#include "tool.h"
 
 const char *pcb_shape_cookie = "shape plugin";
 static pcb_layer_t pcb_shape_current_layer_;
@@ -191,11 +192,11 @@ static pcb_poly_t *any_poly_place(pcb_data_t *data, pcb_layer_t *layer, pcb_poly
 
 	if (data != PCB->Data) {
 		pcb_buffer_clear(PCB, PCB_PASTEBUFFER);
-		pcb_copy_obj_to_buffer(PCB, data, PCB->Data, PCB_TYPE_POLY, CURRENT, p, p);
+		pcb_copy_obj_to_buffer(PCB, data, PCB->Data, PCB_OBJ_POLY, CURRENT, p, p);
 		pcb_r_delete_entry(CURRENT->polygon_tree, (pcb_box_t *)p);
 		pcb_poly_free_fields(p);
 		pcb_poly_free(p);
-		pcb_crosshair_set_mode(PCB_MODE_PASTE_BUFFER);
+		pcb_tool_select_by_id(PCB_MODE_PASTE_BUFFER);
 	}
 	return p;
 }
@@ -236,14 +237,14 @@ static pcb_line_t *circle_place(pcb_data_t *data, pcb_layer_t *layer, pcb_coord_
 	l = pcb_line_new(layer, cx, cy, cx, cy, dia, conf_core.design.clearance*2, pcb_flag_make(flags));
 	if (l != NULL) {
 		if ((conf_core.editor.clear_line) && (data == PCB->Data))
-			pcb_poly_clear_from_poly(data, PCB_TYPE_LINE, layer, l);
+			pcb_poly_clear_from_poly(data, PCB_OBJ_LINE, layer, l);
 
 		if (data != PCB->Data) {
 			pcb_buffer_clear(PCB, PCB_PASTEBUFFER);
-			pcb_copy_obj_to_buffer(PCB, data, PCB->Data, PCB_TYPE_LINE, CURRENT, l, l);
+			pcb_copy_obj_to_buffer(PCB, data, PCB->Data, PCB_OBJ_LINE, CURRENT, l, l);
 			pcb_r_delete_entry(CURRENT->line_tree, (pcb_box_t *)l);
 			pcb_line_free(l);
-			pcb_crosshair_set_mode(PCB_MODE_PASTE_BUFFER);
+			pcb_tool_select_by_id(PCB_MODE_PASTE_BUFFER);
 		}
 	}
 	return l;
@@ -495,6 +496,7 @@ void pplg_uninit_shape(void)
 #include "dolists.h"
 int pplg_init_shape(void)
 {
+	PCB_API_CHK_VER;
 	PCB_REGISTER_ACTIONS(shape_action_list, pcb_shape_cookie)
 
 	pcb_event_bind(PCB_EVENT_LAYERVIS_CHANGED, shape_layer_chg, NULL, pcb_shape_cookie);
