@@ -1,4 +1,4 @@
-/*!
+/*
  * Smartdisperse plug-in for PCB.
  *
  * Copyright (C) 2007 Ben Jackson <ben@ben.com> based on
@@ -11,8 +11,8 @@
  * Ported to pcb-rnd by Tibor 'Igor2' Palinkas in 2016.
  * Upgraded to subc by Tibor 'Igor2' Palinkas in 2017.
  *
- * Improve the initial dispersion of elements by choosing an order based
- * on the netlist, rather than the arbitrary element order.  This isn't
+ * Improve the initial dispersion of subcircuits by choosing an order based
+ * on the netlist, rather than the arbitrary subcircuit order.  This isn't
  * the same as a global autoplace, it's more of a linear autoplace.  It
  * might make some useful local groupings.  For example, you should not
  * have to chase all over the board to find the resistor that goes with
@@ -39,6 +39,7 @@
 #include "hid_actions.h"
 #include "compat_nls.h"
 #include "obj_subc.h"
+#include "obj_subc_parent.h"
 
 #define GAP 10000
 static pcb_coord_t minx;
@@ -46,51 +47,16 @@ static pcb_coord_t miny;
 static pcb_coord_t maxx;
 static pcb_coord_t maxy;
 
-/* Place one element or subcircuit.
+/* the same for subcircuits
    Must initialize statics above before calling for the first time.
    This was taken almost entirely from pcb_act_DisperseElements, with cleanup */
-static void place_elem(pcb_element_t * element)
-{
-	pcb_coord_t dx, dy;
-
-	/* figure out how much to move the element */
-	dx = minx - element->BoundingBox.X1;
-	dy = miny - element->BoundingBox.Y1;
-
-	/* snap to the grid */
-	dx -= (element->MarkX + dx) % PCB->Grid;
-	dx += PCB->Grid;
-	dy -= (element->MarkY + dy) % PCB->Grid;
-	dy += PCB->Grid;
-
-	/* and add one grid size so we make sure we always space by GAP or more */
-	dx += PCB->Grid;
-
-	/* Figure out if this row has room.  If not, start a new row */
-	if (minx != GAP && GAP + element->BoundingBox.X2 + dx > PCB->MaxWidth) {
-		miny = maxy + GAP;
-		minx = GAP;
-		place_elem(element); /* recurse can't loop, now minx==GAP */
-		return;
-	}
-
-	pcb_element_move(PCB->Data, element, dx, dy);
-	pcb_undo_add_obj_to_move(PCB_TYPE_ELEMENT, NULL, NULL, element, dx, dy);
-
-	/* keep track of how tall this row is */
-	minx += element->BoundingBox.X2 - element->BoundingBox.X1 + GAP;
-	if (maxy < element->BoundingBox.Y2)
-		maxy = element->BoundingBox.Y2;
-}
-
-/* the same for subcircuits */
 static void place_subc(pcb_subc_t *sc)
 {
 	pcb_coord_t dx, dy, ox = 0, oy = 0;
 
 	pcb_subc_get_origin(sc, &ox, &oy);
 
-	/* figure out how much to move the element */
+	/* figure out how much to move the subcircuit */
 	dx = minx - sc->BoundingBox.X1;
 	dy = miny - sc->BoundingBox.Y1;
 
@@ -112,7 +78,7 @@ static void place_subc(pcb_subc_t *sc)
 	}
 
 	pcb_subc_move(sc, dx, dy, 1);
-	pcb_undo_add_obj_to_move(PCB_TYPE_SUBC, NULL, NULL, sc, dx, dy);
+	pcb_undo_add_obj_to_move(PCB_OBJ_SUBC, NULL, NULL, sc, dx, dy);
 
 	/* keep track of how tall this row is */
 	minx += sc->BoundingBox.X2 - sc->BoundingBox.X1 + GAP;
@@ -120,23 +86,13 @@ static void place_subc(pcb_subc_t *sc)
 		maxy = sc->BoundingBox.Y2;
 }
 
-static void place(pcb_any_obj_t *obj)
-{
-	switch(obj->type) {
-		case PCB_OBJ_ELEMENT: place_elem((pcb_element_t *)obj); break;
-		case PCB_OBJ_SUBC:    place_subc((pcb_subc_t *)obj); break;
-		default: break;
-	}
-}
-
-
-/* Return the X location of a connection's pad or pin within its element. */
+/* Return the X location of a connection's terminal within its subcircuit. */
 static pcb_coord_t padDX(pcb_connection_t * conn)
 {
-	pcb_element_t *element = (pcb_element_t *) conn->ptr1;
-	pcb_any_line_t *line = (pcb_any_line_t *) conn->obj;
+	pcb_subc_t *subc = (pcb_subc_t *)conn->ptr1;
+	pcb_any_obj_t *line = (pcb_any_obj_t *)conn->obj;
 
-	return line->BoundingBox.X1 - (element->BoundingBox.X1 + element->BoundingBox.X2) / 2;
+	return line->BoundingBox.X1 - (subc->BoundingBox.X1 + subc->BoundingBox.X2) / 2;
 }
 
 /* Return true if ea,eb would be the best order, else eb,ea, based on pad loc. */
@@ -152,7 +108,7 @@ static int padorder(pcb_connection_t * conna, pcb_connection_t * connb)
 	return 0;
 }
 
-#define IS_ELEMENT(CONN)	((CONN)->obj->type == PCB_OBJ_PAD || (CONN)->obj->type == PCB_OBJ_PIN)
+#define IS_IN_SUBC(CONN)  (pcb_obj_parent_subc((CONN)->obj) != NULL)
 
 #define ARG(n) (argc > (n) ? argv[n] : 0)
 
@@ -187,16 +143,9 @@ static int smartdisperse(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 		return 0;
 	}
 
-	/* remember which elements we finish with */
+	/* remember which subcircuits we finish with */
 	htpi_init(&visited, ptrhash, ptrkeyeq);
 
-	/* if we're not doing all, mark the unselected elements and subcs as "visited" */
-	PCB_ELEMENT_LOOP(PCB->Data);
-	{
-		if (!(all || PCB_FLAG_TEST(PCB_FLAG_SELECTED, element)))
-			set_visited(element);
-	}
-	PCB_END_LOOP;
 	PCB_SUBC_LOOP(PCB->Data);
 	{
 		if (!(all || PCB_FLAG_TEST(PCB_FLAG_SELECTED, subc)))
@@ -216,18 +165,18 @@ static int smartdisperse(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 	PCB_NET_LOOP(Nets);
 	{
 		pcb_connection_t *conna, *connb;
-		pcb_any_obj_t *ea, *eb;
+		pcb_subc_t *ea, *eb;
 
 		if (net->ConnectionN != 2)
 			continue;
 
 		conna = &net->Connection[0];
 		connb = &net->Connection[1];
-		if (!IS_ELEMENT(conna) || !IS_ELEMENT(conna))
+		if (!IS_IN_SUBC(conna) || !IS_IN_SUBC(conna))
 			continue;
 
-		ea = (pcb_any_obj_t *) conna->ptr1;
-		eb = (pcb_any_obj_t *) connb->ptr1;
+		ea = (pcb_subc_t *) conna->ptr1;
+		eb = (pcb_subc_t *) connb->ptr1;
 
 		/* place this pair if possible */
 		if (is_visited(ea) || is_visited(eb))
@@ -237,12 +186,12 @@ static int smartdisperse(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 
 		/* a weak attempt to get the linked pads side-by-side */
 		if (padorder(conna, connb)) {
-			place(ea);
-			place(eb);
+			place_subc(ea);
+			place_subc(eb);
 		}
 		else {
-			place(eb);
-			place(ea);
+			place_subc(eb);
+			place_subc(ea);
 		}
 	}
 	PCB_END_LOOP;
@@ -252,9 +201,9 @@ static int smartdisperse(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 	{
 		PCB_CONNECTION_LOOP(net);
 		{
-			pcb_any_obj_t *parent;
+			pcb_subc_t *parent;
 
-			if (!IS_ELEMENT(connection))
+			if (!IS_IN_SUBC(connection))
 				continue;
 
 			parent = connection->ptr1;
@@ -263,19 +212,13 @@ static int smartdisperse(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 			if (is_visited(parent))
 				continue;
 			set_visited(parent);
-			place(parent);
+			place_subc(parent);
 		}
 		PCB_END_LOOP;
 	}
 	PCB_END_LOOP;
 
 	/* Place up anything else */
-	PCB_ELEMENT_LOOP(PCB->Data);
-	{
-		if (!is_visited(element))
-			place_elem(element);
-	}
-	PCB_END_LOOP;
 	PCB_SUBC_LOOP(PCB->Data);
 	{
 		if (!is_visited(subc))
@@ -310,6 +253,7 @@ void pplg_uninit_smartdisperse(void)
 #include "dolists.h"
 int pplg_init_smartdisperse(void)
 {
+	PCB_API_CHK_VER;
 	PCB_REGISTER_ACTIONS(smartdisperse_action_list, smartdisperse_cookie);
 	return 0;
 }
