@@ -24,7 +24,6 @@
 #include "undo.h"
 #include "plugins.h"
 #include "hid_actions.h"
-#include "obj_all.h"
 #include "obj_pstk_inlines.h"
 
 #define MIN_LINE_LENGTH 700
@@ -40,13 +39,10 @@
 #define MIN_LINE_LENGTH2 ((double)MIN_LINE_LENGTH*(double)MIN_LINE_LENGTH)
 #define MAX_DISTANCE2 ((double)MAX_DISTANCE*(double)MAX_DISTANCE)
 
-static pcb_pin_t *pin;
-static pcb_pad_t *pad;
 static pcb_pstk_t *pstk;
 static int layer;
 static pcb_coord_t px, py;
 static pcb_coord_t thickness;
-static pcb_element_t *element;
 
 static int new_arcs = 0;
 
@@ -165,7 +161,7 @@ static pcb_r_dir_t check_line_callback(const pcb_box_t * box, void *cl)
 		arc = pcb_arc_new(lay, (int) ax, (int) ay, (int) radius,
 															(int) radius, (int) theta + 90 + aoffset, delta - aoffset, l->Thickness, l->Clearance, l->Flags);
 		if (arc)
-			pcb_undo_add_obj_to_create(PCB_TYPE_ARC, lay, arc, arc);
+			pcb_undo_add_obj_to_create(PCB_OBJ_ARC, lay, arc, arc);
 
 		ax = lx + dy * (x + t);
 		ay = ly - dx * (x + t);
@@ -173,7 +169,7 @@ static pcb_r_dir_t check_line_callback(const pcb_box_t * box, void *cl)
 		arc = pcb_arc_new(lay, (int) ax, (int) ay, (int) radius,
 															(int) radius, (int) theta - 90 - aoffset, -delta + aoffset, l->Thickness, l->Clearance, l->Flags);
 		if (arc)
-			pcb_undo_add_obj_to_create(PCB_TYPE_ARC, lay, arc, arc);
+			pcb_undo_add_obj_to_create(PCB_OBJ_ARC, lay, arc, arc);
 
 		radius += t * 1.9;
 		aoffset = acos((double) adist / radius) * 180.0 / M_PI;
@@ -252,117 +248,16 @@ static void check_pstk(pcb_pstk_t *ps)
 	}
 }
 
-
-static void check_pin(pcb_pin_t * _pin)
-{
-	pcb_box_t spot;
-
-	pin = _pin;
-
-	px = pin->X;
-	py = pin->Y;
-	thickness = pin->Thickness;
-
-	spot.X1 = px - 10;
-	spot.Y1 = py - 10;
-	spot.X2 = px + 10;
-	spot.Y2 = py + 10;
-
-	element = (pcb_element_t *) pin->Element;
-
-	fprintf(stderr, "Pin %s (%s) at %.6f, %.6f (element %s, %s, %s)\n", PCB_EMPTY(pin->Number), PCB_EMPTY(pin->Name),
-					/* 0.01 * pin->X, 0.01 * pin->Y, */
-					PCB_COORD_TO_MM(pin->X), PCB_COORD_TO_MM(pin->Y),
-					PCB_EMPTY(PCB_ELEM_NAME_REFDES(element)), PCB_EMPTY(PCB_ELEM_NAME_VALUE(element)), PCB_EMPTY(PCB_ELEM_NAME_DESCRIPTION(element)));
-
-	for (layer = 0; layer < pcb_max_layer; layer++) {
-		pcb_layer_t *l = &(PCB->Data->Layer[layer]);
-		if (!(pcb_layer_flags(PCB, layer) & PCB_LYT_COPPER))
-			continue;
-		pcb_r_search(l->line_tree, &spot, NULL, check_line_callback, l, NULL);
-	}
-}
-
-static void check_via(pcb_pin_t * _pin)
-{
-	pcb_box_t spot;
-
-	pin = _pin;
-
-	px = pin->X;
-	py = pin->Y;
-
-	spot.X1 = px - 10;
-	spot.Y1 = py - 10;
-	spot.X2 = px + 10;
-	spot.Y2 = py + 10;
-
-	fprintf(stderr, "Via at %.6f, %.6f\n", PCB_COORD_TO_MM(pin->X), PCB_COORD_TO_MM(pin->Y));
-
-	for (layer = 0; layer < pcb_max_layer; layer++) {
-		pcb_layer_t *l = &(PCB->Data->Layer[layer]);
-		if (!(pcb_layer_flags(PCB, layer) & PCB_LYT_COPPER))
-			continue;
-		pcb_r_search(l->line_tree, &spot, NULL, check_line_callback, l, NULL);
-	}
-}
-
-/*!
- * \brief Draw teardrops for pads.
- */
-static void check_pad(pcb_pad_t * _pad)
-{
-	pad = _pad;
-
-	px = (pad->BoundingBox.X1 + pad->BoundingBox.X2) / 2;
-	py = (pad->BoundingBox.Y1 + pad->BoundingBox.Y2) / 2;
-	thickness = pad->Thickness;
-	element = (pcb_element_t *) pad->Element;
-
-	fprintf(stderr,
-					"Pad %s (%s) at %.6f, %.6f (element %s, %s, %s) \n",
-					PCB_EMPTY(pad->Number), PCB_EMPTY(pad->Name),
-					PCB_COORD_TO_MM((pad->BoundingBox.X1 + pad->BoundingBox.X2) / 2),
-					PCB_COORD_TO_MM((pad->BoundingBox.Y1 + pad->BoundingBox.Y2) / 2),
-					PCB_EMPTY(PCB_ELEM_NAME_REFDES(element)), PCB_EMPTY(PCB_ELEM_NAME_VALUE(element)), PCB_EMPTY(PCB_ELEM_NAME_DESCRIPTION(element)));
-
-	for (layer = 0; layer < pcb_max_layer; layer++) {
-		pcb_layer_t *l = &(PCB->Data->Layer[layer]);
-		if (!(pcb_layer_flags(PCB, layer) & PCB_LYT_COPPER))
-			continue;
-		pcb_r_search(l->line_tree, &(pad->BoundingBox), NULL, check_line_callback, l, NULL);
-	}
-}
-
 static int teardrops(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
 {
-	new_arcs = 0;
 	pcb_box_t *b;
 	pcb_rtree_it_t it;
+	new_arcs = 0;
 
 	for(b = pcb_r_first(PCB->Data->padstack_tree, &it); b != NULL; b = pcb_r_next(&it))
 		check_pstk((pcb_pstk_t *)b);
 	pcb_r_end(&it);
 
-
-	PCB_VIA_LOOP(PCB->Data);
-	{
-		check_via(via);
-	}
-	PCB_END_LOOP;
-
-	PCB_PIN_ALL_LOOP(PCB->Data);
-	{
-		check_pin(pin);
-	}
-	PCB_ENDALL_LOOP;
-
-	PCB_PAD_ALL_LOOP(PCB->Data);
-	{
-		check_pad(pad);
-	}
-	PCB_ENDALL_LOOP;
-
 	pcb_gui->invalidate_all();
 
 	if (new_arcs)
@@ -389,6 +284,7 @@ void pplg_uninit_teardrops(void)
 #include "dolists.h"
 int pplg_init_teardrops(void)
 {
+	PCB_API_CHK_VER;
 	PCB_REGISTER_ACTIONS(teardrops_action_list, teardrops_cookie);
 	return 0;
 }
