@@ -4,6 +4,7 @@
  *  pcb-rnd, interactive printed circuit board design
  *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 2004, 2007 Dan McMahill
+ *  Copyright (C) 2018 Tibor 'Igor2' Palinkas
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -42,7 +43,6 @@
 #include "draw.h"
 #include "error.h"
 #include "undo.h"
-#include "stub_vendor.h"
 #include "plugins.h"
 #include "action_helper.h"
 #include "hid_flags.h"
@@ -51,9 +51,9 @@
 #include "vendor_conf.h"
 #include "compat_misc.h"
 #include "compat_nls.h"
-#include "obj_pinvia.h"
 #include "obj_pstk_inlines.h"
 #include "event.h"
+#include "macro.h"
 #include <liblihata/lihata.h>
 #include <liblihata/tree.h>
 
@@ -64,11 +64,11 @@ static void apply_vendor_map(void);
 static void process_skips(lht_node_t *);
 static pcb_bool rematch(const char *, const char *);
 static void vendor_free_all(void);
-int vendorDrillMap(int in);
+pcb_coord_t vendorDrillMap(pcb_coord_t in);
 
 
 /* list of vendor drills and a count of them */
-static int *vendor_drills = NULL;
+static pcb_coord_t *vendor_drills = NULL;
 static int n_vendor_drills = 0;
 
 static int cached_drill = -1;
@@ -96,8 +96,15 @@ static int rounding_method = ROUND_UP;
 
 #define FREE(x) if((x) != NULL) { free (x) ; (x) = NULL; }
 
+/* load a board metadata into conf_core */
+static void load_meta_coord(const char *path, pcb_coord_t crd)
+{
+	char tmp[128];
+	pcb_sprintf(tmp, "%$mm", crd);
+	conf_set(CFR_DESIGN, path, -1, tmp, POL_OVERWRITE);
+}
+
 static pcb_bool vendorIsSubcMappable(pcb_subc_t *subc);
-static pcb_bool vendorIsElementMappable(pcb_element_t *element);
 
 /* ************************************************************ */
 
@@ -263,38 +270,38 @@ int pcb_act_LoadVendorFrom(int argc, const char **argv, pcb_coord_t x, pcb_coord
 
 	sval = pcb_hid_cfg_text_value(doc, "/drc/copper_space");
 	if (sval != NULL) {
-		PCB->Bloat = floor(sf * atof(sval) + 0.5);
-		pcb_message(PCB_MSG_INFO, _("Set DRC minimum copper spacing to %ml mils\n"), PCB->Bloat);
+		load_meta_coord("design/bloat", floor(sf * atof(sval) + 0.5));
+		pcb_message(PCB_MSG_INFO, _("Set DRC minimum copper spacing to %ml mils\n"), conf_core.design.bloat);
 	}
 
 	sval = pcb_hid_cfg_text_value(doc, "/drc/copper_overlap");
 	if (sval != NULL) {
-		PCB->Shrink = floor(sf * atof(sval) + 0.5);
-		pcb_message(PCB_MSG_INFO, _("Set DRC minimum copper overlap to %ml mils\n"), PCB->Shrink);
+		load_meta_coord("design/shrink", floor(sf * atof(sval) + 0.5));
+		pcb_message(PCB_MSG_INFO, _("Set DRC minimum copper overlap to %ml mils\n"), conf_core.design.shrink);
 	}
 
 	sval = pcb_hid_cfg_text_value(doc, "/drc/copper_width");
 	if (sval != NULL) {
-		PCB->minWid = floor(sf * atof(sval) + 0.5);
-		pcb_message(PCB_MSG_INFO, _("Set DRC minimum copper spacing to %ml mils\n"), PCB->minWid);
+		load_meta_coord("design/min_wid", floor(sf * atof(sval) + 0.5));
+		pcb_message(PCB_MSG_INFO, _("Set DRC minimum copper spacing to %ml mils\n"), conf_core.design.min_wid);
 	}
 
 	sval = pcb_hid_cfg_text_value(doc, "/drc/silk_width");
 	if (sval != NULL) {
-		PCB->minSlk = floor(sf * atof(sval) + 0.5);
-		pcb_message(PCB_MSG_INFO, _("Set DRC minimum silk width to %ml mils\n"), PCB->minSlk);
+		load_meta_coord("design/min_slk", floor(sf * atof(sval) + 0.5));
+		pcb_message(PCB_MSG_INFO, _("Set DRC minimum silk width to %ml mils\n"), conf_core.design.min_slk);
 	}
 
 	sval = pcb_hid_cfg_text_value(doc, "/drc/min_drill");
 	if (sval != NULL) {
-		PCB->minDrill = floor(sf * atof(sval) + 0.5);
-		pcb_message(PCB_MSG_INFO, _("Set DRC minimum drill diameter to %ml mils\n"), PCB->minDrill);
+		load_meta_coord("design/min_drill", floor(sf * atof(sval) + 0.5));
+		pcb_message(PCB_MSG_INFO, _("Set DRC minimum drill diameter to %ml mils\n"), conf_core.design.min_drill);
 	}
 
 	sval = pcb_hid_cfg_text_value(doc, "/drc/min_ring");
 	if (sval != NULL) {
-		PCB->minRing = floor(sf * atof(sval) + 0.5);
-		pcb_message(PCB_MSG_INFO, _("Set DRC minimum annular ring to %ml mils\n"), PCB->minRing);
+		load_meta_coord("design/min_ring", floor(sf * atof(sval) + 0.5));
+		pcb_message(PCB_MSG_INFO, _("Set DRC minimum annular ring to %ml mils\n"), conf_core.design.min_ring);
 	}
 
 	pcb_message(PCB_MSG_INFO, _("Loaded %d vendor drills from %s\n"), n_vendor_drills, fname);
@@ -309,35 +316,45 @@ int pcb_act_LoadVendorFrom(int argc, const char **argv, pcb_coord_t x, pcb_coord
 	return 0;
 }
 
-static pcb_cardinal_t apply_vendor_pstk(pcb_data_t *data)
+static int apply_vendor_pstk1(pcb_pstk_t *pstk, pcb_cardinal_t *tot)
+{
+	pcb_pstk_proto_t *proto = pcb_pstk_get_proto(pstk);
+	pcb_coord_t target;
+	int res = 0;
+
+	if ((proto == NULL) || (proto->hdia == 0)) return 0;
+	(*tot)++;
+	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, pstk)) return 0;
+
+	target = vendorDrillMap(proto->hdia);
+	if (proto->hdia != target) {
+		if (pcb_chg_obj_2nd_size(PCB_OBJ_PSTK, pstk, pstk, pstk, target, pcb_true, pcb_false))
+			res = 1;
+		else {
+			pcb_message(PCB_MSG_WARNING, _
+				("Padstack at %ml, %ml not changed.  Possible reasons:\n"
+				 "\t- pad size too small\n"
+				 "\t- new size would be too large or too small\n"), pstk->x, pstk->y);
+		}
+	}
+	return res;
+}
+
+static pcb_cardinal_t apply_vendor_pstk(pcb_data_t *data, pcb_cardinal_t *tot)
 {
 	gdl_iterator_t it;
 	pcb_pstk_t *pstk;
 	pcb_cardinal_t changed = 0;
 
-	padstacklist_foreach(&data->subc, &it, pstk) {
-		pcb_pstk_proto_t *proto = pcb_pstk_get_proto(pstk);
-		pcb_coord_t target;
-
-		if ((proto == NULL) || (proto->hdia == 0) || PCB_FLAG_TEST(PCB_FLAG_LOCK, pstk)) continue;
-
-		target = vendorDrillMap(proto->hdia);
-		if (proto->hdia != target) {
-			if (pcb_chg_obj_2nd_size(PCB_TYPE_PSTK, pstk, NULL, NULL, target, pcb_true, pcb_false))
-				changed++;
-			else {
-				pcb_message(PCB_MSG_WARNING, _
-					("Via at %ml, %ml not changed.  Possible reasons:\n"
-					 "\t- pad size too small\n"
-					 "\t- new size would be too large or too small\n"), pstk->x, pstk->y);
-			}
-		}
-	}
+	padstacklist_foreach(&data->padstack, &it, pstk)
+		if (apply_vendor_pstk1(pstk, tot))
+			changed++;
 	return changed;
 }
 
 static void apply_vendor_map(void)
 {
+	int i;
 	pcb_cardinal_t changed = 0, tot = 0;
 	pcb_bool state;
 
@@ -349,104 +366,42 @@ static void apply_vendor_map(void)
 	/* If we have loaded vendor drills, then apply them to the design */
 	if (n_vendor_drills > 0) {
 
-		changed += apply_vendor_pstk(PCB->Data);
-		
-		/* first all the vias */
-		PCB_VIA_LOOP(PCB->Data);
-		{
-			tot++;
-			if (via->DrillingHole != vendorDrillMap(via->DrillingHole)) {
-				/* only change unlocked vias */
-				if (!PCB_FLAG_TEST(PCB_FLAG_LOCK, via)) {
-					if (pcb_chg_obj_2nd_size(PCB_TYPE_VIA, via, NULL, NULL, vendorDrillMap(via->DrillingHole), pcb_true, pcb_false))
-						changed++;
-					else {
-						pcb_message(PCB_MSG_WARNING, _
-										("Via at %ml, %ml not changed.  Possible reasons:\n"
-										 "\t- pad size too small\n"
-										 "\t- new size would be too large or too small\n"), via->X, via->Y);
-					}
-				}
-				else {
-					pcb_message(PCB_MSG_WARNING, _("Locked via at %ml, %ml not changed.\n"), via->X, via->Y);
-				}
-			}
-		}
-		PCB_END_LOOP;
+		/* global padsatcks (e.g. vias) */
+		changed += apply_vendor_pstk(PCB->Data, &tot);
 
 		PCB_SUBC_LOOP(PCB->Data);
 		{
 			if (vendorIsSubcMappable(subc))
-				changed += apply_vendor_pstk(subc->data);
-		}
-		PCB_END_LOOP;
-
-#warning subc TODO
-		/* and now the pins */
-		PCB_ELEMENT_LOOP(PCB->Data);
-		{
-			/*
-			 * first figure out if this element should be skipped for some
-			 * reason
-			 */
-			if (vendorIsElementMappable(element)) {
-				/* the element is ok to modify, so iterate over its pins */
-				PCB_PIN_LOOP(element);
-				{
-					tot++;
-					if (pin->DrillingHole != vendorDrillMap(pin->DrillingHole)) {
-						if (!PCB_FLAG_TEST(PCB_FLAG_LOCK, pin)) {
-							if (pcb_chg_obj_2nd_size(PCB_TYPE_PIN, element, pin, NULL, vendorDrillMap(pin->DrillingHole), pcb_true, pcb_false))
-								changed++;
-							else {
-								pcb_message(PCB_MSG_WARNING, _
-												("Pin %s (%s) at %ml, %ml (element %s, %s, %s) not changed.\n"
-												 "\tPossible reasons:\n"
-												 "\t- pad size too small\n"
-												 "\t- new size would be too large or too small\n"),
-												PCB_UNKNOWN(pin->Number), PCB_UNKNOWN(pin->Name),
-												pin->X, pin->Y,
-												PCB_UNKNOWN(PCB_ELEM_NAME_REFDES(element)), PCB_UNKNOWN(PCB_ELEM_NAME_VALUE(element)), PCB_UNKNOWN(PCB_ELEM_NAME_DESCRIPTION(element)));
-							}
-						}
-						else {
-							pcb_message(PCB_MSG_WARNING, _("Locked pin at %ml, %ml not changed.\n"), pin->X, pin->Y);
-						}
-					}
-				}
-				PCB_END_LOOP;
-			}
+				changed += apply_vendor_pstk(subc->data, &tot);
 		}
 		PCB_END_LOOP;
 
 		pcb_message(PCB_MSG_INFO, _("Updated %ld drill sizes out of %ld total\n"), (long)changed, (long)tot);
 
-#warning TODO: this should not happen; modify some local setting?
-#if 0
 		/* Update the current Via */
-		if (conf_core.design.via_drilling_hole != vendorDrillMap(Settings.ViaDrillingHole)) {
+		if (conf_core.design.via_drilling_hole != vendorDrillMap(conf_core.design.via_drilling_hole)) {
 			changed++;
-			Settings.ViaDrillingHole = vendorDrillMap(Settings.ViaDrillingHole);
-			pcb_message(PCB_MSG_INFO, _("Adjusted active via hole size to be %ml mils\n"), Settings.ViaDrillingHole);
+			conf_setf(CFR_DESIGN, "design/via_drilling_hole", -1, "%$mm", vendorDrillMap(conf_core.design.via_drilling_hole));
+			pcb_message(PCB_MSG_INFO, _("Adjusted active via hole size to be %ml mils\n"), conf_core.design.via_drilling_hole);
 		}
 
 		/* and update the vias for the various routing styles */
-		for (i = 0; i < NUM_STYLES; i++) {
-			if (PCB->RouteStyle[i].Hole != vendorDrillMap(PCB->RouteStyle[i].Hole)) {
+		for (i = 0; i < vtroutestyle_len(&PCB->RouteStyle); i++) {
+			if (PCB->RouteStyle.array[i].Hole != vendorDrillMap(PCB->RouteStyle.array[i].Hole)) {
 				changed++;
-				PCB->RouteStyle[i].Hole = vendorDrillMap(PCB->RouteStyle[i].Hole);
+				PCB->RouteStyle.array[i].Hole = vendorDrillMap(PCB->RouteStyle.array[i].Hole);
 				pcb_message(PCB_MSG_INFO, _
-								("Adjusted %s routing style via hole size to be %ml mils\n"),
-								PCB->RouteStyle[i].Name, PCB->RouteStyle[i].Hole);
-				if (PCB->RouteStyle[i].Diameter < PCB->RouteStyle[i].Hole + PCB_MIN_PINORVIACOPPER) {
-					PCB->RouteStyle[i].Diameter = PCB->RouteStyle[i].Hole + PCB_MIN_PINORVIACOPPER;
+								("Adjusted %s routing style hole size to be %ml mils\n"),
+								PCB->RouteStyle.array[i].name, PCB->RouteStyle.array[i].Hole);
+				if (PCB->RouteStyle.array[i].Diameter < PCB->RouteStyle.array[i].Hole + PCB_MIN_PINORVIACOPPER) {
+					PCB->RouteStyle.array[i].Diameter = PCB->RouteStyle.array[i].Hole + PCB_MIN_PINORVIACOPPER;
 					pcb_message(PCB_MSG_INFO, _
 									("Increased %s routing style via diameter to %ml mils\n"),
-									PCB->RouteStyle[i].Name, PCB->RouteStyle[i].Diameter);
+									PCB->RouteStyle.array[i].name, PCB->RouteStyle.array[i].Diameter);
 				}
 			}
 		}
-#endif
+
 		/*
 		 * if we've changed anything, indicate that we need to save the
 		 * file, redraw things, and make sure we can undo.
@@ -463,7 +418,7 @@ static void apply_vendor_map(void)
 }
 
 /* for a given drill size, find the closest vendor drill size */
-int vendorDrillMap(int in)
+pcb_coord_t vendorDrillMap(pcb_coord_t in)
 {
 	int i, min, max;
 
@@ -612,54 +567,6 @@ static void process_skips(lht_node_t *res)
 	}
 }
 
-static pcb_bool vendorIsElementMappable(pcb_element_t *element)
-{
-	int i;
-	int noskip;
-
-	if (!conf_vendor.plugins.vendor.enable)
-		return pcb_false;
-
-	noskip = 1;
-	for (i = 0; i < n_refdes; i++) {
-		if ((PCB_NSTRCMP(PCB_UNKNOWN(PCB_ELEM_NAME_REFDES(element)), ignore_refdes[i]) == 0)
-				|| rematch(ignore_refdes[i], PCB_UNKNOWN(PCB_ELEM_NAME_REFDES(element)))) {
-			pcb_message(PCB_MSG_INFO, _("Vendor mapping skipped because refdes = %s matches %s\n"), PCB_UNKNOWN(PCB_ELEM_NAME_REFDES(element)), ignore_refdes[i]);
-			noskip = 0;
-		}
-	}
-	if (noskip)
-		for (i = 0; i < n_value; i++) {
-			if ((PCB_NSTRCMP(PCB_UNKNOWN(PCB_ELEM_NAME_VALUE(element)), ignore_value[i]) == 0)
-					|| rematch(ignore_value[i], PCB_UNKNOWN(PCB_ELEM_NAME_VALUE(element)))) {
-				pcb_message(PCB_MSG_INFO, _("Vendor mapping skipped because value = %s matches %s\n"), PCB_UNKNOWN(PCB_ELEM_NAME_VALUE(element)), ignore_value[i]);
-				noskip = 0;
-			}
-		}
-
-	if (noskip)
-		for (i = 0; i < n_descr; i++) {
-			if ((PCB_NSTRCMP(PCB_UNKNOWN(PCB_ELEM_NAME_DESCRIPTION(element)), ignore_descr[i])
-					 == 0)
-					|| rematch(ignore_descr[i], PCB_UNKNOWN(PCB_ELEM_NAME_DESCRIPTION(element)))) {
-				pcb_message(PCB_MSG_INFO, _
-								("Vendor mapping skipped because descr = %s matches %s\n"),
-								PCB_UNKNOWN(PCB_ELEM_NAME_DESCRIPTION(element)), ignore_descr[i]);
-				noskip = 0;
-			}
-		}
-
-	if (noskip && PCB_FLAG_TEST(PCB_FLAG_LOCK, element)) {
-		pcb_message(PCB_MSG_INFO, _("Vendor mapping skipped because element %s is locked\n"), PCB_UNKNOWN(PCB_ELEM_NAME_REFDES(element)));
-		noskip = 0;
-	}
-
-	if (noskip)
-		return pcb_true;
-	else
-		return pcb_false;
-}
-
 static pcb_bool vendorIsSubcMappable(pcb_subc_t *subc)
 {
 	int i;
@@ -775,10 +682,24 @@ static void vendor_free_all(void)
 	cached_drill = -1;
 }
 
+/* Tune newly placed padstacks */
+static void vendor_new_pstk(void *user_data, int argc, pcb_event_arg_t argv[])
+{
+	pcb_pstk_t *ps;
+	pcb_cardinal_t dummy;
+
+	if ((argc < 2) || (argv[1].type != PCB_EVARG_PTR))
+		return;
+
+	ps = argv[1].d.p;
+	apply_vendor_pstk1(ps, &dummy);
+}
+
 int pplg_check_ver_vendordrill(int ver_needed) { return 0; }
 
 void pplg_uninit_vendordrill(void)
 {
+	pcb_event_unbind_allcookie(vendor_cookie);
 	pcb_hid_remove_actions_by_cookie(vendor_cookie);
 	vendor_free_all();
 	conf_unreg_fields("plugins/vendor/");
@@ -787,13 +708,12 @@ void pplg_uninit_vendordrill(void)
 #include "dolists.h"
 int pplg_init_vendordrill(void)
 {
+	PCB_API_CHK_VER;
 #define conf_reg(field,isarray,type_name,cpath,cname,desc,flags) \
 	conf_reg_field(conf_vendor, field,isarray,type_name,cpath,cname,desc,flags);
 #include "vendor_conf_fields.h"
 
-	pcb_stub_vendor_drill_map = vendorDrillMap;
-	pcb_stub_vendor_is_element_mappable = vendorIsElementMappable;
-
+	pcb_event_bind(PCB_EVENT_NEW_PSTK, vendor_new_pstk, NULL, vendor_cookie);
 	PCB_REGISTER_ACTIONS(vendor_action_list, vendor_cookie)
 	return 0;
 }
