@@ -21,10 +21,13 @@
 #include "../src/error.h"
 #include "../src/event.h"
 #include "../src/plugins.h"
+#include "../src/hid.h"
 #include "gsch2pcb_rnd_conf.h"
 
 /* glue for pcb-rnd core */
 
+pcb_hid_t *pcb_gui = NULL;
+
 void pcb_chdir_error_message(const char *DirName)
 {
 	pcb_message(PCB_MSG_WARNING, "warning: can't cd to %s\n", DirName);
@@ -70,10 +73,12 @@ void pcb_event(pcb_event_id_t ev, const char *fmt, ...)
 
 int pcb_file_loaded_set_at(const char *catname, const char *name, const char *path, const char *desc)
 {
+	return 0;
 }
 
 int pcb_file_loaded_del_at(const char *catname, const char *name)
 {
+	return 0;
 }
 
 
